# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.1
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _thostmduserapi
else:
    import _thostmduserapi

try:
    import builtins as __builtin__
except ImportError:
    print('ImportError')
    # import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


import weakref

THOST_TERT_RESTART = _thostmduserapi.THOST_TERT_RESTART
THOST_TERT_RESUME = _thostmduserapi.THOST_TERT_RESUME
THOST_TERT_QUICK = _thostmduserapi.THOST_TERT_QUICK
THOST_FTDC_EXP_Normal = _thostmduserapi.THOST_FTDC_EXP_Normal
THOST_FTDC_EXP_GenOrderByTrade = _thostmduserapi.THOST_FTDC_EXP_GenOrderByTrade
THOST_FTDC_ICT_EID = _thostmduserapi.THOST_FTDC_ICT_EID
THOST_FTDC_ICT_IDCard = _thostmduserapi.THOST_FTDC_ICT_IDCard
THOST_FTDC_ICT_OfficerIDCard = _thostmduserapi.THOST_FTDC_ICT_OfficerIDCard
THOST_FTDC_ICT_PoliceIDCard = _thostmduserapi.THOST_FTDC_ICT_PoliceIDCard
THOST_FTDC_ICT_SoldierIDCard = _thostmduserapi.THOST_FTDC_ICT_SoldierIDCard
THOST_FTDC_ICT_HouseholdRegister = _thostmduserapi.THOST_FTDC_ICT_HouseholdRegister
THOST_FTDC_ICT_Passport = _thostmduserapi.THOST_FTDC_ICT_Passport
THOST_FTDC_ICT_TaiwanCompatriotIDCard = _thostmduserapi.THOST_FTDC_ICT_TaiwanCompatriotIDCard
THOST_FTDC_ICT_HomeComingCard = _thostmduserapi.THOST_FTDC_ICT_HomeComingCard
THOST_FTDC_ICT_LicenseNo = _thostmduserapi.THOST_FTDC_ICT_LicenseNo
THOST_FTDC_ICT_TaxNo = _thostmduserapi.THOST_FTDC_ICT_TaxNo
THOST_FTDC_ICT_HMMainlandTravelPermit = _thostmduserapi.THOST_FTDC_ICT_HMMainlandTravelPermit
THOST_FTDC_ICT_TwMainlandTravelPermit = _thostmduserapi.THOST_FTDC_ICT_TwMainlandTravelPermit
THOST_FTDC_ICT_DrivingLicense = _thostmduserapi.THOST_FTDC_ICT_DrivingLicense
THOST_FTDC_ICT_SocialID = _thostmduserapi.THOST_FTDC_ICT_SocialID
THOST_FTDC_ICT_LocalID = _thostmduserapi.THOST_FTDC_ICT_LocalID
THOST_FTDC_ICT_BusinessRegistration = _thostmduserapi.THOST_FTDC_ICT_BusinessRegistration
THOST_FTDC_ICT_HKMCIDCard = _thostmduserapi.THOST_FTDC_ICT_HKMCIDCard
THOST_FTDC_ICT_AccountsPermits = _thostmduserapi.THOST_FTDC_ICT_AccountsPermits
THOST_FTDC_ICT_FrgPrmtRdCard = _thostmduserapi.THOST_FTDC_ICT_FrgPrmtRdCard
THOST_FTDC_ICT_CptMngPrdLetter = _thostmduserapi.THOST_FTDC_ICT_CptMngPrdLetter
THOST_FTDC_ICT_OtherCard = _thostmduserapi.THOST_FTDC_ICT_OtherCard
THOST_FTDC_IR_All = _thostmduserapi.THOST_FTDC_IR_All
THOST_FTDC_IR_Group = _thostmduserapi.THOST_FTDC_IR_Group
THOST_FTDC_IR_Single = _thostmduserapi.THOST_FTDC_IR_Single
THOST_FTDC_DR_All = _thostmduserapi.THOST_FTDC_DR_All
THOST_FTDC_DR_Group = _thostmduserapi.THOST_FTDC_DR_Group
THOST_FTDC_DR_Single = _thostmduserapi.THOST_FTDC_DR_Single
THOST_FTDC_DS_Asynchronous = _thostmduserapi.THOST_FTDC_DS_Asynchronous
THOST_FTDC_DS_Synchronizing = _thostmduserapi.THOST_FTDC_DS_Synchronizing
THOST_FTDC_DS_Synchronized = _thostmduserapi.THOST_FTDC_DS_Synchronized
THOST_FTDC_BDS_Synchronized = _thostmduserapi.THOST_FTDC_BDS_Synchronized
THOST_FTDC_BDS_Synchronizing = _thostmduserapi.THOST_FTDC_BDS_Synchronizing
THOST_FTDC_ECS_NoConnection = _thostmduserapi.THOST_FTDC_ECS_NoConnection
THOST_FTDC_ECS_QryInstrumentSent = _thostmduserapi.THOST_FTDC_ECS_QryInstrumentSent
THOST_FTDC_ECS_GotInformation = _thostmduserapi.THOST_FTDC_ECS_GotInformation
THOST_FTDC_TCS_NotConnected = _thostmduserapi.THOST_FTDC_TCS_NotConnected
THOST_FTDC_TCS_Connected = _thostmduserapi.THOST_FTDC_TCS_Connected
THOST_FTDC_TCS_QryInstrumentSent = _thostmduserapi.THOST_FTDC_TCS_QryInstrumentSent
THOST_FTDC_TCS_SubPrivateFlow = _thostmduserapi.THOST_FTDC_TCS_SubPrivateFlow
THOST_FTDC_FC_DataAsync = _thostmduserapi.THOST_FTDC_FC_DataAsync
THOST_FTDC_FC_ForceUserLogout = _thostmduserapi.THOST_FTDC_FC_ForceUserLogout
THOST_FTDC_FC_UserPasswordUpdate = _thostmduserapi.THOST_FTDC_FC_UserPasswordUpdate
THOST_FTDC_FC_BrokerPasswordUpdate = _thostmduserapi.THOST_FTDC_FC_BrokerPasswordUpdate
THOST_FTDC_FC_InvestorPasswordUpdate = _thostmduserapi.THOST_FTDC_FC_InvestorPasswordUpdate
THOST_FTDC_FC_OrderInsert = _thostmduserapi.THOST_FTDC_FC_OrderInsert
THOST_FTDC_FC_OrderAction = _thostmduserapi.THOST_FTDC_FC_OrderAction
THOST_FTDC_FC_SyncSystemData = _thostmduserapi.THOST_FTDC_FC_SyncSystemData
THOST_FTDC_FC_SyncBrokerData = _thostmduserapi.THOST_FTDC_FC_SyncBrokerData
THOST_FTDC_FC_BachSyncBrokerData = _thostmduserapi.THOST_FTDC_FC_BachSyncBrokerData
THOST_FTDC_FC_SuperQuery = _thostmduserapi.THOST_FTDC_FC_SuperQuery
THOST_FTDC_FC_ParkedOrderInsert = _thostmduserapi.THOST_FTDC_FC_ParkedOrderInsert
THOST_FTDC_FC_ParkedOrderAction = _thostmduserapi.THOST_FTDC_FC_ParkedOrderAction
THOST_FTDC_FC_SyncOTP = _thostmduserapi.THOST_FTDC_FC_SyncOTP
THOST_FTDC_FC_DeleteOrder = _thostmduserapi.THOST_FTDC_FC_DeleteOrder
THOST_FTDC_BFC_ForceUserLogout = _thostmduserapi.THOST_FTDC_BFC_ForceUserLogout
THOST_FTDC_BFC_UserPasswordUpdate = _thostmduserapi.THOST_FTDC_BFC_UserPasswordUpdate
THOST_FTDC_BFC_SyncBrokerData = _thostmduserapi.THOST_FTDC_BFC_SyncBrokerData
THOST_FTDC_BFC_BachSyncBrokerData = _thostmduserapi.THOST_FTDC_BFC_BachSyncBrokerData
THOST_FTDC_BFC_OrderInsert = _thostmduserapi.THOST_FTDC_BFC_OrderInsert
THOST_FTDC_BFC_OrderAction = _thostmduserapi.THOST_FTDC_BFC_OrderAction
THOST_FTDC_BFC_AllQuery = _thostmduserapi.THOST_FTDC_BFC_AllQuery
THOST_FTDC_BFC_log = _thostmduserapi.THOST_FTDC_BFC_log
THOST_FTDC_BFC_BaseQry = _thostmduserapi.THOST_FTDC_BFC_BaseQry
THOST_FTDC_BFC_TradeQry = _thostmduserapi.THOST_FTDC_BFC_TradeQry
THOST_FTDC_BFC_Trade = _thostmduserapi.THOST_FTDC_BFC_Trade
THOST_FTDC_BFC_Virement = _thostmduserapi.THOST_FTDC_BFC_Virement
THOST_FTDC_BFC_Risk = _thostmduserapi.THOST_FTDC_BFC_Risk
THOST_FTDC_BFC_Session = _thostmduserapi.THOST_FTDC_BFC_Session
THOST_FTDC_BFC_RiskNoticeCtl = _thostmduserapi.THOST_FTDC_BFC_RiskNoticeCtl
THOST_FTDC_BFC_RiskNotice = _thostmduserapi.THOST_FTDC_BFC_RiskNotice
THOST_FTDC_BFC_BrokerDeposit = _thostmduserapi.THOST_FTDC_BFC_BrokerDeposit
THOST_FTDC_BFC_QueryFund = _thostmduserapi.THOST_FTDC_BFC_QueryFund
THOST_FTDC_BFC_QueryOrder = _thostmduserapi.THOST_FTDC_BFC_QueryOrder
THOST_FTDC_BFC_QueryTrade = _thostmduserapi.THOST_FTDC_BFC_QueryTrade
THOST_FTDC_BFC_QueryPosition = _thostmduserapi.THOST_FTDC_BFC_QueryPosition
THOST_FTDC_BFC_QueryMarketData = _thostmduserapi.THOST_FTDC_BFC_QueryMarketData
THOST_FTDC_BFC_QueryUserEvent = _thostmduserapi.THOST_FTDC_BFC_QueryUserEvent
THOST_FTDC_BFC_QueryRiskNotify = _thostmduserapi.THOST_FTDC_BFC_QueryRiskNotify
THOST_FTDC_BFC_QueryFundChange = _thostmduserapi.THOST_FTDC_BFC_QueryFundChange
THOST_FTDC_BFC_QueryInvestor = _thostmduserapi.THOST_FTDC_BFC_QueryInvestor
THOST_FTDC_BFC_QueryTradingCode = _thostmduserapi.THOST_FTDC_BFC_QueryTradingCode
THOST_FTDC_BFC_ForceClose = _thostmduserapi.THOST_FTDC_BFC_ForceClose
THOST_FTDC_BFC_PressTest = _thostmduserapi.THOST_FTDC_BFC_PressTest
THOST_FTDC_BFC_RemainCalc = _thostmduserapi.THOST_FTDC_BFC_RemainCalc
THOST_FTDC_BFC_NetPositionInd = _thostmduserapi.THOST_FTDC_BFC_NetPositionInd
THOST_FTDC_BFC_RiskPredict = _thostmduserapi.THOST_FTDC_BFC_RiskPredict
THOST_FTDC_BFC_DataExport = _thostmduserapi.THOST_FTDC_BFC_DataExport
THOST_FTDC_BFC_RiskTargetSetup = _thostmduserapi.THOST_FTDC_BFC_RiskTargetSetup
THOST_FTDC_BFC_MarketDataWarn = _thostmduserapi.THOST_FTDC_BFC_MarketDataWarn
THOST_FTDC_BFC_QryBizNotice = _thostmduserapi.THOST_FTDC_BFC_QryBizNotice
THOST_FTDC_BFC_CfgBizNotice = _thostmduserapi.THOST_FTDC_BFC_CfgBizNotice
THOST_FTDC_BFC_SyncOTP = _thostmduserapi.THOST_FTDC_BFC_SyncOTP
THOST_FTDC_BFC_SendBizNotice = _thostmduserapi.THOST_FTDC_BFC_SendBizNotice
THOST_FTDC_BFC_CfgRiskLevelStd = _thostmduserapi.THOST_FTDC_BFC_CfgRiskLevelStd
THOST_FTDC_BFC_TbCommand = _thostmduserapi.THOST_FTDC_BFC_TbCommand
THOST_FTDC_BFC_DeleteOrder = _thostmduserapi.THOST_FTDC_BFC_DeleteOrder
THOST_FTDC_BFC_ParkedOrderInsert = _thostmduserapi.THOST_FTDC_BFC_ParkedOrderInsert
THOST_FTDC_BFC_ParkedOrderAction = _thostmduserapi.THOST_FTDC_BFC_ParkedOrderAction
THOST_FTDC_BFC_ExecOrderNoCheck = _thostmduserapi.THOST_FTDC_BFC_ExecOrderNoCheck
THOST_FTDC_BFC_Designate = _thostmduserapi.THOST_FTDC_BFC_Designate
THOST_FTDC_BFC_StockDisposal = _thostmduserapi.THOST_FTDC_BFC_StockDisposal
THOST_FTDC_BFC_BrokerDepositWarn = _thostmduserapi.THOST_FTDC_BFC_BrokerDepositWarn
THOST_FTDC_BFC_CoverWarn = _thostmduserapi.THOST_FTDC_BFC_CoverWarn
THOST_FTDC_BFC_PreExecOrder = _thostmduserapi.THOST_FTDC_BFC_PreExecOrder
THOST_FTDC_BFC_ExecOrderRisk = _thostmduserapi.THOST_FTDC_BFC_ExecOrderRisk
THOST_FTDC_BFC_PosiLimitWarn = _thostmduserapi.THOST_FTDC_BFC_PosiLimitWarn
THOST_FTDC_BFC_QryPosiLimit = _thostmduserapi.THOST_FTDC_BFC_QryPosiLimit
THOST_FTDC_BFC_FBSign = _thostmduserapi.THOST_FTDC_BFC_FBSign
THOST_FTDC_BFC_FBAccount = _thostmduserapi.THOST_FTDC_BFC_FBAccount
THOST_FTDC_OAS_Submitted = _thostmduserapi.THOST_FTDC_OAS_Submitted
THOST_FTDC_OAS_Accepted = _thostmduserapi.THOST_FTDC_OAS_Accepted
THOST_FTDC_OAS_Rejected = _thostmduserapi.THOST_FTDC_OAS_Rejected
THOST_FTDC_OST_AllTraded = _thostmduserapi.THOST_FTDC_OST_AllTraded
THOST_FTDC_OST_PartTradedQueueing = _thostmduserapi.THOST_FTDC_OST_PartTradedQueueing
THOST_FTDC_OST_PartTradedNotQueueing = _thostmduserapi.THOST_FTDC_OST_PartTradedNotQueueing
THOST_FTDC_OST_NoTradeQueueing = _thostmduserapi.THOST_FTDC_OST_NoTradeQueueing
THOST_FTDC_OST_NoTradeNotQueueing = _thostmduserapi.THOST_FTDC_OST_NoTradeNotQueueing
THOST_FTDC_OST_Canceled = _thostmduserapi.THOST_FTDC_OST_Canceled
THOST_FTDC_OST_Unknown = _thostmduserapi.THOST_FTDC_OST_Unknown
THOST_FTDC_OST_NotTouched = _thostmduserapi.THOST_FTDC_OST_NotTouched
THOST_FTDC_OST_Touched = _thostmduserapi.THOST_FTDC_OST_Touched
THOST_FTDC_OSS_InsertSubmitted = _thostmduserapi.THOST_FTDC_OSS_InsertSubmitted
THOST_FTDC_OSS_CancelSubmitted = _thostmduserapi.THOST_FTDC_OSS_CancelSubmitted
THOST_FTDC_OSS_ModifySubmitted = _thostmduserapi.THOST_FTDC_OSS_ModifySubmitted
THOST_FTDC_OSS_Accepted = _thostmduserapi.THOST_FTDC_OSS_Accepted
THOST_FTDC_OSS_InsertRejected = _thostmduserapi.THOST_FTDC_OSS_InsertRejected
THOST_FTDC_OSS_CancelRejected = _thostmduserapi.THOST_FTDC_OSS_CancelRejected
THOST_FTDC_OSS_ModifyRejected = _thostmduserapi.THOST_FTDC_OSS_ModifyRejected
THOST_FTDC_PSD_Today = _thostmduserapi.THOST_FTDC_PSD_Today
THOST_FTDC_PSD_History = _thostmduserapi.THOST_FTDC_PSD_History
THOST_FTDC_PDT_UseHistory = _thostmduserapi.THOST_FTDC_PDT_UseHistory
THOST_FTDC_PDT_NoUseHistory = _thostmduserapi.THOST_FTDC_PDT_NoUseHistory
THOST_FTDC_ER_Broker = _thostmduserapi.THOST_FTDC_ER_Broker
THOST_FTDC_ER_Host = _thostmduserapi.THOST_FTDC_ER_Host
THOST_FTDC_ER_Maker = _thostmduserapi.THOST_FTDC_ER_Maker
THOST_FTDC_PC_Futures = _thostmduserapi.THOST_FTDC_PC_Futures
THOST_FTDC_PC_Options = _thostmduserapi.THOST_FTDC_PC_Options
THOST_FTDC_PC_Combination = _thostmduserapi.THOST_FTDC_PC_Combination
THOST_FTDC_PC_Spot = _thostmduserapi.THOST_FTDC_PC_Spot
THOST_FTDC_PC_EFP = _thostmduserapi.THOST_FTDC_PC_EFP
THOST_FTDC_PC_SpotOption = _thostmduserapi.THOST_FTDC_PC_SpotOption
THOST_FTDC_IP_NotStart = _thostmduserapi.THOST_FTDC_IP_NotStart
THOST_FTDC_IP_Started = _thostmduserapi.THOST_FTDC_IP_Started
THOST_FTDC_IP_Pause = _thostmduserapi.THOST_FTDC_IP_Pause
THOST_FTDC_IP_Expired = _thostmduserapi.THOST_FTDC_IP_Expired
THOST_FTDC_D_Buy = _thostmduserapi.THOST_FTDC_D_Buy
THOST_FTDC_D_Sell = _thostmduserapi.THOST_FTDC_D_Sell
THOST_FTDC_PT_Net = _thostmduserapi.THOST_FTDC_PT_Net
THOST_FTDC_PT_Gross = _thostmduserapi.THOST_FTDC_PT_Gross
THOST_FTDC_PD_Net = _thostmduserapi.THOST_FTDC_PD_Net
THOST_FTDC_PD_Long = _thostmduserapi.THOST_FTDC_PD_Long
THOST_FTDC_PD_Short = _thostmduserapi.THOST_FTDC_PD_Short
THOST_FTDC_SS_NonActive = _thostmduserapi.THOST_FTDC_SS_NonActive
THOST_FTDC_SS_Startup = _thostmduserapi.THOST_FTDC_SS_Startup
THOST_FTDC_SS_Operating = _thostmduserapi.THOST_FTDC_SS_Operating
THOST_FTDC_SS_Settlement = _thostmduserapi.THOST_FTDC_SS_Settlement
THOST_FTDC_SS_SettlementFinished = _thostmduserapi.THOST_FTDC_SS_SettlementFinished
THOST_FTDC_RA_Trade = _thostmduserapi.THOST_FTDC_RA_Trade
THOST_FTDC_RA_Settlement = _thostmduserapi.THOST_FTDC_RA_Settlement
THOST_FTDC_HF_Speculation = _thostmduserapi.THOST_FTDC_HF_Speculation
THOST_FTDC_HF_Arbitrage = _thostmduserapi.THOST_FTDC_HF_Arbitrage
THOST_FTDC_HF_Hedge = _thostmduserapi.THOST_FTDC_HF_Hedge
THOST_FTDC_HF_MarketMaker = _thostmduserapi.THOST_FTDC_HF_MarketMaker
THOST_FTDC_HF_SpecHedge = _thostmduserapi.THOST_FTDC_HF_SpecHedge
THOST_FTDC_HF_HedgeSpec = _thostmduserapi.THOST_FTDC_HF_HedgeSpec
THOST_FTDC_BHF_Speculation = _thostmduserapi.THOST_FTDC_BHF_Speculation
THOST_FTDC_BHF_Arbitrage = _thostmduserapi.THOST_FTDC_BHF_Arbitrage
THOST_FTDC_BHF_Hedge = _thostmduserapi.THOST_FTDC_BHF_Hedge
THOST_FTDC_CIDT_Speculation = _thostmduserapi.THOST_FTDC_CIDT_Speculation
THOST_FTDC_CIDT_Arbitrage = _thostmduserapi.THOST_FTDC_CIDT_Arbitrage
THOST_FTDC_CIDT_Hedge = _thostmduserapi.THOST_FTDC_CIDT_Hedge
THOST_FTDC_CIDT_MarketMaker = _thostmduserapi.THOST_FTDC_CIDT_MarketMaker
THOST_FTDC_OPT_AnyPrice = _thostmduserapi.THOST_FTDC_OPT_AnyPrice
THOST_FTDC_OPT_LimitPrice = _thostmduserapi.THOST_FTDC_OPT_LimitPrice
THOST_FTDC_OPT_BestPrice = _thostmduserapi.THOST_FTDC_OPT_BestPrice
THOST_FTDC_OPT_LastPrice = _thostmduserapi.THOST_FTDC_OPT_LastPrice
THOST_FTDC_OPT_LastPricePlusOneTicks = _thostmduserapi.THOST_FTDC_OPT_LastPricePlusOneTicks
THOST_FTDC_OPT_LastPricePlusTwoTicks = _thostmduserapi.THOST_FTDC_OPT_LastPricePlusTwoTicks
THOST_FTDC_OPT_LastPricePlusThreeTicks = _thostmduserapi.THOST_FTDC_OPT_LastPricePlusThreeTicks
THOST_FTDC_OPT_AskPrice1 = _thostmduserapi.THOST_FTDC_OPT_AskPrice1
THOST_FTDC_OPT_AskPrice1PlusOneTicks = _thostmduserapi.THOST_FTDC_OPT_AskPrice1PlusOneTicks
THOST_FTDC_OPT_AskPrice1PlusTwoTicks = _thostmduserapi.THOST_FTDC_OPT_AskPrice1PlusTwoTicks
THOST_FTDC_OPT_AskPrice1PlusThreeTicks = _thostmduserapi.THOST_FTDC_OPT_AskPrice1PlusThreeTicks
THOST_FTDC_OPT_BidPrice1 = _thostmduserapi.THOST_FTDC_OPT_BidPrice1
THOST_FTDC_OPT_BidPrice1PlusOneTicks = _thostmduserapi.THOST_FTDC_OPT_BidPrice1PlusOneTicks
THOST_FTDC_OPT_BidPrice1PlusTwoTicks = _thostmduserapi.THOST_FTDC_OPT_BidPrice1PlusTwoTicks
THOST_FTDC_OPT_BidPrice1PlusThreeTicks = _thostmduserapi.THOST_FTDC_OPT_BidPrice1PlusThreeTicks
THOST_FTDC_OPT_FiveLevelPrice = _thostmduserapi.THOST_FTDC_OPT_FiveLevelPrice
THOST_FTDC_OF_Open = _thostmduserapi.THOST_FTDC_OF_Open
THOST_FTDC_OF_Close = _thostmduserapi.THOST_FTDC_OF_Close
THOST_FTDC_OF_ForceClose = _thostmduserapi.THOST_FTDC_OF_ForceClose
THOST_FTDC_OF_CloseToday = _thostmduserapi.THOST_FTDC_OF_CloseToday
THOST_FTDC_OF_CloseYesterday = _thostmduserapi.THOST_FTDC_OF_CloseYesterday
THOST_FTDC_OF_ForceOff = _thostmduserapi.THOST_FTDC_OF_ForceOff
THOST_FTDC_OF_LocalForceClose = _thostmduserapi.THOST_FTDC_OF_LocalForceClose
THOST_FTDC_FCC_NotForceClose = _thostmduserapi.THOST_FTDC_FCC_NotForceClose
THOST_FTDC_FCC_LackDeposit = _thostmduserapi.THOST_FTDC_FCC_LackDeposit
THOST_FTDC_FCC_ClientOverPositionLimit = _thostmduserapi.THOST_FTDC_FCC_ClientOverPositionLimit
THOST_FTDC_FCC_MemberOverPositionLimit = _thostmduserapi.THOST_FTDC_FCC_MemberOverPositionLimit
THOST_FTDC_FCC_NotMultiple = _thostmduserapi.THOST_FTDC_FCC_NotMultiple
THOST_FTDC_FCC_Violation = _thostmduserapi.THOST_FTDC_FCC_Violation
THOST_FTDC_FCC_Other = _thostmduserapi.THOST_FTDC_FCC_Other
THOST_FTDC_FCC_PersonDeliv = _thostmduserapi.THOST_FTDC_FCC_PersonDeliv
THOST_FTDC_ORDT_Normal = _thostmduserapi.THOST_FTDC_ORDT_Normal
THOST_FTDC_ORDT_DeriveFromQuote = _thostmduserapi.THOST_FTDC_ORDT_DeriveFromQuote
THOST_FTDC_ORDT_DeriveFromCombination = _thostmduserapi.THOST_FTDC_ORDT_DeriveFromCombination
THOST_FTDC_ORDT_Combination = _thostmduserapi.THOST_FTDC_ORDT_Combination
THOST_FTDC_ORDT_ConditionalOrder = _thostmduserapi.THOST_FTDC_ORDT_ConditionalOrder
THOST_FTDC_ORDT_Swap = _thostmduserapi.THOST_FTDC_ORDT_Swap
THOST_FTDC_ORDT_DeriveFromBlockTrade = _thostmduserapi.THOST_FTDC_ORDT_DeriveFromBlockTrade
THOST_FTDC_ORDT_DeriveFromEFPTrade = _thostmduserapi.THOST_FTDC_ORDT_DeriveFromEFPTrade
THOST_FTDC_TC_IOC = _thostmduserapi.THOST_FTDC_TC_IOC
THOST_FTDC_TC_GFS = _thostmduserapi.THOST_FTDC_TC_GFS
THOST_FTDC_TC_GFD = _thostmduserapi.THOST_FTDC_TC_GFD
THOST_FTDC_TC_GTD = _thostmduserapi.THOST_FTDC_TC_GTD
THOST_FTDC_TC_GTC = _thostmduserapi.THOST_FTDC_TC_GTC
THOST_FTDC_TC_GFA = _thostmduserapi.THOST_FTDC_TC_GFA
THOST_FTDC_VC_AV = _thostmduserapi.THOST_FTDC_VC_AV
THOST_FTDC_VC_MV = _thostmduserapi.THOST_FTDC_VC_MV
THOST_FTDC_VC_CV = _thostmduserapi.THOST_FTDC_VC_CV
THOST_FTDC_CC_Immediately = _thostmduserapi.THOST_FTDC_CC_Immediately
THOST_FTDC_CC_Touch = _thostmduserapi.THOST_FTDC_CC_Touch
THOST_FTDC_CC_TouchProfit = _thostmduserapi.THOST_FTDC_CC_TouchProfit
THOST_FTDC_CC_ParkedOrder = _thostmduserapi.THOST_FTDC_CC_ParkedOrder
THOST_FTDC_CC_LastPriceGreaterThanStopPrice = _thostmduserapi.THOST_FTDC_CC_LastPriceGreaterThanStopPrice
THOST_FTDC_CC_LastPriceGreaterEqualStopPrice = _thostmduserapi.THOST_FTDC_CC_LastPriceGreaterEqualStopPrice
THOST_FTDC_CC_LastPriceLesserThanStopPrice = _thostmduserapi.THOST_FTDC_CC_LastPriceLesserThanStopPrice
THOST_FTDC_CC_LastPriceLesserEqualStopPrice = _thostmduserapi.THOST_FTDC_CC_LastPriceLesserEqualStopPrice
THOST_FTDC_CC_AskPriceGreaterThanStopPrice = _thostmduserapi.THOST_FTDC_CC_AskPriceGreaterThanStopPrice
THOST_FTDC_CC_AskPriceGreaterEqualStopPrice = _thostmduserapi.THOST_FTDC_CC_AskPriceGreaterEqualStopPrice
THOST_FTDC_CC_AskPriceLesserThanStopPrice = _thostmduserapi.THOST_FTDC_CC_AskPriceLesserThanStopPrice
THOST_FTDC_CC_AskPriceLesserEqualStopPrice = _thostmduserapi.THOST_FTDC_CC_AskPriceLesserEqualStopPrice
THOST_FTDC_CC_BidPriceGreaterThanStopPrice = _thostmduserapi.THOST_FTDC_CC_BidPriceGreaterThanStopPrice
THOST_FTDC_CC_BidPriceGreaterEqualStopPrice = _thostmduserapi.THOST_FTDC_CC_BidPriceGreaterEqualStopPrice
THOST_FTDC_CC_BidPriceLesserThanStopPrice = _thostmduserapi.THOST_FTDC_CC_BidPriceLesserThanStopPrice
THOST_FTDC_CC_BidPriceLesserEqualStopPrice = _thostmduserapi.THOST_FTDC_CC_BidPriceLesserEqualStopPrice
THOST_FTDC_AF_Delete = _thostmduserapi.THOST_FTDC_AF_Delete
THOST_FTDC_AF_Modify = _thostmduserapi.THOST_FTDC_AF_Modify
THOST_FTDC_TR_Allow = _thostmduserapi.THOST_FTDC_TR_Allow
THOST_FTDC_TR_CloseOnly = _thostmduserapi.THOST_FTDC_TR_CloseOnly
THOST_FTDC_TR_Forbidden = _thostmduserapi.THOST_FTDC_TR_Forbidden
THOST_FTDC_OSRC_Participant = _thostmduserapi.THOST_FTDC_OSRC_Participant
THOST_FTDC_OSRC_Administrator = _thostmduserapi.THOST_FTDC_OSRC_Administrator
THOST_FTDC_TRDT_SplitCombination = _thostmduserapi.THOST_FTDC_TRDT_SplitCombination
THOST_FTDC_TRDT_Common = _thostmduserapi.THOST_FTDC_TRDT_Common
THOST_FTDC_TRDT_OptionsExecution = _thostmduserapi.THOST_FTDC_TRDT_OptionsExecution
THOST_FTDC_TRDT_OTC = _thostmduserapi.THOST_FTDC_TRDT_OTC
THOST_FTDC_TRDT_EFPDerived = _thostmduserapi.THOST_FTDC_TRDT_EFPDerived
THOST_FTDC_TRDT_CombinationDerived = _thostmduserapi.THOST_FTDC_TRDT_CombinationDerived
THOST_FTDC_TRDT_BlockTrade = _thostmduserapi.THOST_FTDC_TRDT_BlockTrade
THOST_FTDC_PSRC_LastPrice = _thostmduserapi.THOST_FTDC_PSRC_LastPrice
THOST_FTDC_PSRC_Buy = _thostmduserapi.THOST_FTDC_PSRC_Buy
THOST_FTDC_PSRC_Sell = _thostmduserapi.THOST_FTDC_PSRC_Sell
THOST_FTDC_PSRC_OTC = _thostmduserapi.THOST_FTDC_PSRC_OTC
THOST_FTDC_IS_BeforeTrading = _thostmduserapi.THOST_FTDC_IS_BeforeTrading
THOST_FTDC_IS_NoTrading = _thostmduserapi.THOST_FTDC_IS_NoTrading
THOST_FTDC_IS_Continous = _thostmduserapi.THOST_FTDC_IS_Continous
THOST_FTDC_IS_AuctionOrdering = _thostmduserapi.THOST_FTDC_IS_AuctionOrdering
THOST_FTDC_IS_AuctionBalance = _thostmduserapi.THOST_FTDC_IS_AuctionBalance
THOST_FTDC_IS_AuctionMatch = _thostmduserapi.THOST_FTDC_IS_AuctionMatch
THOST_FTDC_IS_Closed = _thostmduserapi.THOST_FTDC_IS_Closed
THOST_FTDC_IER_Automatic = _thostmduserapi.THOST_FTDC_IER_Automatic
THOST_FTDC_IER_Manual = _thostmduserapi.THOST_FTDC_IER_Manual
THOST_FTDC_IER_Fuse = _thostmduserapi.THOST_FTDC_IER_Fuse
THOST_FTDC_BS_NoUpload = _thostmduserapi.THOST_FTDC_BS_NoUpload
THOST_FTDC_BS_Uploaded = _thostmduserapi.THOST_FTDC_BS_Uploaded
THOST_FTDC_BS_Failed = _thostmduserapi.THOST_FTDC_BS_Failed
THOST_FTDC_RS_All = _thostmduserapi.THOST_FTDC_RS_All
THOST_FTDC_RS_ByProduct = _thostmduserapi.THOST_FTDC_RS_ByProduct
THOST_FTDC_RP_ByVolume = _thostmduserapi.THOST_FTDC_RP_ByVolume
THOST_FTDC_RP_ByFeeOnHand = _thostmduserapi.THOST_FTDC_RP_ByFeeOnHand
THOST_FTDC_RL_Level1 = _thostmduserapi.THOST_FTDC_RL_Level1
THOST_FTDC_RL_Level2 = _thostmduserapi.THOST_FTDC_RL_Level2
THOST_FTDC_RL_Level3 = _thostmduserapi.THOST_FTDC_RL_Level3
THOST_FTDC_RL_Level4 = _thostmduserapi.THOST_FTDC_RL_Level4
THOST_FTDC_RL_Level5 = _thostmduserapi.THOST_FTDC_RL_Level5
THOST_FTDC_RL_Level6 = _thostmduserapi.THOST_FTDC_RL_Level6
THOST_FTDC_RL_Level7 = _thostmduserapi.THOST_FTDC_RL_Level7
THOST_FTDC_RL_Level8 = _thostmduserapi.THOST_FTDC_RL_Level8
THOST_FTDC_RL_Level9 = _thostmduserapi.THOST_FTDC_RL_Level9
THOST_FTDC_RSD_ByPeriod = _thostmduserapi.THOST_FTDC_RSD_ByPeriod
THOST_FTDC_RSD_ByStandard = _thostmduserapi.THOST_FTDC_RSD_ByStandard
THOST_FTDC_MT_Out = _thostmduserapi.THOST_FTDC_MT_Out
THOST_FTDC_MT_In = _thostmduserapi.THOST_FTDC_MT_In
THOST_FTDC_ISPI_MortgageRatio = _thostmduserapi.THOST_FTDC_ISPI_MortgageRatio
THOST_FTDC_ISPI_MarginWay = _thostmduserapi.THOST_FTDC_ISPI_MarginWay
THOST_FTDC_ISPI_BillDeposit = _thostmduserapi.THOST_FTDC_ISPI_BillDeposit
THOST_FTDC_ESPI_MortgageRatio = _thostmduserapi.THOST_FTDC_ESPI_MortgageRatio
THOST_FTDC_ESPI_OtherFundItem = _thostmduserapi.THOST_FTDC_ESPI_OtherFundItem
THOST_FTDC_ESPI_OtherFundImport = _thostmduserapi.THOST_FTDC_ESPI_OtherFundImport
THOST_FTDC_ESPI_CFFEXMinPrepa = _thostmduserapi.THOST_FTDC_ESPI_CFFEXMinPrepa
THOST_FTDC_ESPI_CZCESettlementType = _thostmduserapi.THOST_FTDC_ESPI_CZCESettlementType
THOST_FTDC_ESPI_ExchDelivFeeMode = _thostmduserapi.THOST_FTDC_ESPI_ExchDelivFeeMode
THOST_FTDC_ESPI_DelivFeeMode = _thostmduserapi.THOST_FTDC_ESPI_DelivFeeMode
THOST_FTDC_ESPI_CZCEComMarginType = _thostmduserapi.THOST_FTDC_ESPI_CZCEComMarginType
THOST_FTDC_ESPI_DceComMarginType = _thostmduserapi.THOST_FTDC_ESPI_DceComMarginType
THOST_FTDC_ESPI_OptOutDisCountRate = _thostmduserapi.THOST_FTDC_ESPI_OptOutDisCountRate
THOST_FTDC_ESPI_OptMiniGuarantee = _thostmduserapi.THOST_FTDC_ESPI_OptMiniGuarantee
THOST_FTDC_SPI_InvestorIDMinLength = _thostmduserapi.THOST_FTDC_SPI_InvestorIDMinLength
THOST_FTDC_SPI_AccountIDMinLength = _thostmduserapi.THOST_FTDC_SPI_AccountIDMinLength
THOST_FTDC_SPI_UserRightLogon = _thostmduserapi.THOST_FTDC_SPI_UserRightLogon
THOST_FTDC_SPI_SettlementBillTrade = _thostmduserapi.THOST_FTDC_SPI_SettlementBillTrade
THOST_FTDC_SPI_TradingCode = _thostmduserapi.THOST_FTDC_SPI_TradingCode
THOST_FTDC_SPI_CheckFund = _thostmduserapi.THOST_FTDC_SPI_CheckFund
THOST_FTDC_SPI_CommModelRight = _thostmduserapi.THOST_FTDC_SPI_CommModelRight
THOST_FTDC_SPI_MarginModelRight = _thostmduserapi.THOST_FTDC_SPI_MarginModelRight
THOST_FTDC_SPI_IsStandardActive = _thostmduserapi.THOST_FTDC_SPI_IsStandardActive
THOST_FTDC_SPI_UploadSettlementFile = _thostmduserapi.THOST_FTDC_SPI_UploadSettlementFile
THOST_FTDC_SPI_DownloadCSRCFile = _thostmduserapi.THOST_FTDC_SPI_DownloadCSRCFile
THOST_FTDC_SPI_SettlementBillFile = _thostmduserapi.THOST_FTDC_SPI_SettlementBillFile
THOST_FTDC_SPI_CSRCOthersFile = _thostmduserapi.THOST_FTDC_SPI_CSRCOthersFile
THOST_FTDC_SPI_InvestorPhoto = _thostmduserapi.THOST_FTDC_SPI_InvestorPhoto
THOST_FTDC_SPI_CSRCData = _thostmduserapi.THOST_FTDC_SPI_CSRCData
THOST_FTDC_SPI_InvestorPwdModel = _thostmduserapi.THOST_FTDC_SPI_InvestorPwdModel
THOST_FTDC_SPI_CFFEXInvestorSettleFile = _thostmduserapi.THOST_FTDC_SPI_CFFEXInvestorSettleFile
THOST_FTDC_SPI_InvestorIDType = _thostmduserapi.THOST_FTDC_SPI_InvestorIDType
THOST_FTDC_SPI_FreezeMaxReMain = _thostmduserapi.THOST_FTDC_SPI_FreezeMaxReMain
THOST_FTDC_SPI_IsSync = _thostmduserapi.THOST_FTDC_SPI_IsSync
THOST_FTDC_SPI_RelieveOpenLimit = _thostmduserapi.THOST_FTDC_SPI_RelieveOpenLimit
THOST_FTDC_SPI_IsStandardFreeze = _thostmduserapi.THOST_FTDC_SPI_IsStandardFreeze
THOST_FTDC_SPI_CZCENormalProductHedge = _thostmduserapi.THOST_FTDC_SPI_CZCENormalProductHedge
THOST_FTDC_TPID_EncryptionStandard = _thostmduserapi.THOST_FTDC_TPID_EncryptionStandard
THOST_FTDC_TPID_RiskMode = _thostmduserapi.THOST_FTDC_TPID_RiskMode
THOST_FTDC_TPID_RiskModeGlobal = _thostmduserapi.THOST_FTDC_TPID_RiskModeGlobal
THOST_FTDC_TPID_modeEncode = _thostmduserapi.THOST_FTDC_TPID_modeEncode
THOST_FTDC_TPID_tickMode = _thostmduserapi.THOST_FTDC_TPID_tickMode
THOST_FTDC_TPID_SingleUserSessionMaxNum = _thostmduserapi.THOST_FTDC_TPID_SingleUserSessionMaxNum
THOST_FTDC_TPID_LoginFailMaxNum = _thostmduserapi.THOST_FTDC_TPID_LoginFailMaxNum
THOST_FTDC_TPID_IsAuthForce = _thostmduserapi.THOST_FTDC_TPID_IsAuthForce
THOST_FTDC_TPID_IsPosiFreeze = _thostmduserapi.THOST_FTDC_TPID_IsPosiFreeze
THOST_FTDC_TPID_IsPosiLimit = _thostmduserapi.THOST_FTDC_TPID_IsPosiLimit
THOST_FTDC_TPID_ForQuoteTimeInterval = _thostmduserapi.THOST_FTDC_TPID_ForQuoteTimeInterval
THOST_FTDC_TPID_IsFuturePosiLimit = _thostmduserapi.THOST_FTDC_TPID_IsFuturePosiLimit
THOST_FTDC_TPID_IsFutureOrderFreq = _thostmduserapi.THOST_FTDC_TPID_IsFutureOrderFreq
THOST_FTDC_TPID_IsExecOrderProfit = _thostmduserapi.THOST_FTDC_TPID_IsExecOrderProfit
THOST_FTDC_TPID_IsCheckBankAcc = _thostmduserapi.THOST_FTDC_TPID_IsCheckBankAcc
THOST_FTDC_TPID_PasswordDeadLine = _thostmduserapi.THOST_FTDC_TPID_PasswordDeadLine
THOST_FTDC_TPID_IsStrongPassword = _thostmduserapi.THOST_FTDC_TPID_IsStrongPassword
THOST_FTDC_TPID_BalanceMorgage = _thostmduserapi.THOST_FTDC_TPID_BalanceMorgage
THOST_FTDC_TPID_MinPwdLen = _thostmduserapi.THOST_FTDC_TPID_MinPwdLen
THOST_FTDC_TPID_LoginFailMaxNumForIP = _thostmduserapi.THOST_FTDC_TPID_LoginFailMaxNumForIP
THOST_FTDC_TPID_PasswordPeriod = _thostmduserapi.THOST_FTDC_TPID_PasswordPeriod
THOST_FTDC_FI_SettlementFund = _thostmduserapi.THOST_FTDC_FI_SettlementFund
THOST_FTDC_FI_Trade = _thostmduserapi.THOST_FTDC_FI_Trade
THOST_FTDC_FI_InvestorPosition = _thostmduserapi.THOST_FTDC_FI_InvestorPosition
THOST_FTDC_FI_SubEntryFund = _thostmduserapi.THOST_FTDC_FI_SubEntryFund
THOST_FTDC_FI_CZCECombinationPos = _thostmduserapi.THOST_FTDC_FI_CZCECombinationPos
THOST_FTDC_FI_CSRCData = _thostmduserapi.THOST_FTDC_FI_CSRCData
THOST_FTDC_FI_CZCEClose = _thostmduserapi.THOST_FTDC_FI_CZCEClose
THOST_FTDC_FI_CZCENoClose = _thostmduserapi.THOST_FTDC_FI_CZCENoClose
THOST_FTDC_FI_PositionDtl = _thostmduserapi.THOST_FTDC_FI_PositionDtl
THOST_FTDC_FI_OptionStrike = _thostmduserapi.THOST_FTDC_FI_OptionStrike
THOST_FTDC_FI_SettlementPriceComparison = _thostmduserapi.THOST_FTDC_FI_SettlementPriceComparison
THOST_FTDC_FI_NonTradePosChange = _thostmduserapi.THOST_FTDC_FI_NonTradePosChange
THOST_FTDC_FUT_Settlement = _thostmduserapi.THOST_FTDC_FUT_Settlement
THOST_FTDC_FUT_Check = _thostmduserapi.THOST_FTDC_FUT_Check
THOST_FTDC_FFT_Txt = _thostmduserapi.THOST_FTDC_FFT_Txt
THOST_FTDC_FFT_Zip = _thostmduserapi.THOST_FTDC_FFT_Zip
THOST_FTDC_FFT_DBF = _thostmduserapi.THOST_FTDC_FFT_DBF
THOST_FTDC_FUS_SucceedUpload = _thostmduserapi.THOST_FTDC_FUS_SucceedUpload
THOST_FTDC_FUS_FailedUpload = _thostmduserapi.THOST_FTDC_FUS_FailedUpload
THOST_FTDC_FUS_SucceedLoad = _thostmduserapi.THOST_FTDC_FUS_SucceedLoad
THOST_FTDC_FUS_PartSucceedLoad = _thostmduserapi.THOST_FTDC_FUS_PartSucceedLoad
THOST_FTDC_FUS_FailedLoad = _thostmduserapi.THOST_FTDC_FUS_FailedLoad
THOST_FTDC_TD_Out = _thostmduserapi.THOST_FTDC_TD_Out
THOST_FTDC_TD_In = _thostmduserapi.THOST_FTDC_TD_In
THOST_FTDC_SC_NoSpecialRule = _thostmduserapi.THOST_FTDC_SC_NoSpecialRule
THOST_FTDC_SC_NoSpringFestival = _thostmduserapi.THOST_FTDC_SC_NoSpringFestival
THOST_FTDC_IPT_LastSettlement = _thostmduserapi.THOST_FTDC_IPT_LastSettlement
THOST_FTDC_IPT_LaseClose = _thostmduserapi.THOST_FTDC_IPT_LaseClose
THOST_FTDC_PLP_Active = _thostmduserapi.THOST_FTDC_PLP_Active
THOST_FTDC_PLP_NonActive = _thostmduserapi.THOST_FTDC_PLP_NonActive
THOST_FTDC_PLP_Canceled = _thostmduserapi.THOST_FTDC_PLP_Canceled
THOST_FTDC_DM_CashDeliv = _thostmduserapi.THOST_FTDC_DM_CashDeliv
THOST_FTDC_DM_CommodityDeliv = _thostmduserapi.THOST_FTDC_DM_CommodityDeliv
THOST_FTDC_FIOT_FundIO = _thostmduserapi.THOST_FTDC_FIOT_FundIO
THOST_FTDC_FIOT_Transfer = _thostmduserapi.THOST_FTDC_FIOT_Transfer
THOST_FTDC_FIOT_SwapCurrency = _thostmduserapi.THOST_FTDC_FIOT_SwapCurrency
THOST_FTDC_FT_Deposite = _thostmduserapi.THOST_FTDC_FT_Deposite
THOST_FTDC_FT_ItemFund = _thostmduserapi.THOST_FTDC_FT_ItemFund
THOST_FTDC_FT_Company = _thostmduserapi.THOST_FTDC_FT_Company
THOST_FTDC_FT_InnerTransfer = _thostmduserapi.THOST_FTDC_FT_InnerTransfer
THOST_FTDC_FD_In = _thostmduserapi.THOST_FTDC_FD_In
THOST_FTDC_FD_Out = _thostmduserapi.THOST_FTDC_FD_Out
THOST_FTDC_FS_Record = _thostmduserapi.THOST_FTDC_FS_Record
THOST_FTDC_FS_Check = _thostmduserapi.THOST_FTDC_FS_Check
THOST_FTDC_FS_Charge = _thostmduserapi.THOST_FTDC_FS_Charge
THOST_FTDC_PS_None = _thostmduserapi.THOST_FTDC_PS_None
THOST_FTDC_PS_Publishing = _thostmduserapi.THOST_FTDC_PS_Publishing
THOST_FTDC_PS_Published = _thostmduserapi.THOST_FTDC_PS_Published
THOST_FTDC_ES_NonActive = _thostmduserapi.THOST_FTDC_ES_NonActive
THOST_FTDC_ES_Startup = _thostmduserapi.THOST_FTDC_ES_Startup
THOST_FTDC_ES_Initialize = _thostmduserapi.THOST_FTDC_ES_Initialize
THOST_FTDC_ES_Initialized = _thostmduserapi.THOST_FTDC_ES_Initialized
THOST_FTDC_ES_Close = _thostmduserapi.THOST_FTDC_ES_Close
THOST_FTDC_ES_Closed = _thostmduserapi.THOST_FTDC_ES_Closed
THOST_FTDC_ES_Settlement = _thostmduserapi.THOST_FTDC_ES_Settlement
THOST_FTDC_STS_Initialize = _thostmduserapi.THOST_FTDC_STS_Initialize
THOST_FTDC_STS_Settlementing = _thostmduserapi.THOST_FTDC_STS_Settlementing
THOST_FTDC_STS_Settlemented = _thostmduserapi.THOST_FTDC_STS_Settlemented
THOST_FTDC_STS_Finished = _thostmduserapi.THOST_FTDC_STS_Finished
THOST_FTDC_CT_Person = _thostmduserapi.THOST_FTDC_CT_Person
THOST_FTDC_CT_Company = _thostmduserapi.THOST_FTDC_CT_Company
THOST_FTDC_CT_Fund = _thostmduserapi.THOST_FTDC_CT_Fund
THOST_FTDC_CT_SpecialOrgan = _thostmduserapi.THOST_FTDC_CT_SpecialOrgan
THOST_FTDC_CT_Asset = _thostmduserapi.THOST_FTDC_CT_Asset
THOST_FTDC_BT_Trade = _thostmduserapi.THOST_FTDC_BT_Trade
THOST_FTDC_BT_TradeSettle = _thostmduserapi.THOST_FTDC_BT_TradeSettle
THOST_FTDC_FAS_Low = _thostmduserapi.THOST_FTDC_FAS_Low
THOST_FTDC_FAS_Normal = _thostmduserapi.THOST_FTDC_FAS_Normal
THOST_FTDC_FAS_Focus = _thostmduserapi.THOST_FTDC_FAS_Focus
THOST_FTDC_FAS_Risk = _thostmduserapi.THOST_FTDC_FAS_Risk
THOST_FTDC_FAS_ByTrade = _thostmduserapi.THOST_FTDC_FAS_ByTrade
THOST_FTDC_FAS_ByDeliv = _thostmduserapi.THOST_FTDC_FAS_ByDeliv
THOST_FTDC_FAS_None = _thostmduserapi.THOST_FTDC_FAS_None
THOST_FTDC_FAS_FixFee = _thostmduserapi.THOST_FTDC_FAS_FixFee
THOST_FTDC_PWDT_Trade = _thostmduserapi.THOST_FTDC_PWDT_Trade
THOST_FTDC_PWDT_Account = _thostmduserapi.THOST_FTDC_PWDT_Account
THOST_FTDC_AG_All = _thostmduserapi.THOST_FTDC_AG_All
THOST_FTDC_AG_OnlyLost = _thostmduserapi.THOST_FTDC_AG_OnlyLost
THOST_FTDC_AG_OnlyGain = _thostmduserapi.THOST_FTDC_AG_OnlyGain
THOST_FTDC_AG_None = _thostmduserapi.THOST_FTDC_AG_None
THOST_FTDC_ICP_Include = _thostmduserapi.THOST_FTDC_ICP_Include
THOST_FTDC_ICP_NotInclude = _thostmduserapi.THOST_FTDC_ICP_NotInclude
THOST_FTDC_AWT_Enable = _thostmduserapi.THOST_FTDC_AWT_Enable
THOST_FTDC_AWT_Disable = _thostmduserapi.THOST_FTDC_AWT_Disable
THOST_FTDC_AWT_NoHoldEnable = _thostmduserapi.THOST_FTDC_AWT_NoHoldEnable
THOST_FTDC_FPWD_UnCheck = _thostmduserapi.THOST_FTDC_FPWD_UnCheck
THOST_FTDC_FPWD_Check = _thostmduserapi.THOST_FTDC_FPWD_Check
THOST_FTDC_TT_BankToFuture = _thostmduserapi.THOST_FTDC_TT_BankToFuture
THOST_FTDC_TT_FutureToBank = _thostmduserapi.THOST_FTDC_TT_FutureToBank
THOST_FTDC_TVF_Invalid = _thostmduserapi.THOST_FTDC_TVF_Invalid
THOST_FTDC_TVF_Valid = _thostmduserapi.THOST_FTDC_TVF_Valid
THOST_FTDC_TVF_Reverse = _thostmduserapi.THOST_FTDC_TVF_Reverse
THOST_FTDC_RN_CD = _thostmduserapi.THOST_FTDC_RN_CD
THOST_FTDC_RN_ZT = _thostmduserapi.THOST_FTDC_RN_ZT
THOST_FTDC_RN_QT = _thostmduserapi.THOST_FTDC_RN_QT
THOST_FTDC_SEX_None = _thostmduserapi.THOST_FTDC_SEX_None
THOST_FTDC_SEX_Man = _thostmduserapi.THOST_FTDC_SEX_Man
THOST_FTDC_SEX_Woman = _thostmduserapi.THOST_FTDC_SEX_Woman
THOST_FTDC_UT_Investor = _thostmduserapi.THOST_FTDC_UT_Investor
THOST_FTDC_UT_Operator = _thostmduserapi.THOST_FTDC_UT_Operator
THOST_FTDC_UT_SuperUser = _thostmduserapi.THOST_FTDC_UT_SuperUser
THOST_FTDC_RATETYPE_MarginRate = _thostmduserapi.THOST_FTDC_RATETYPE_MarginRate
THOST_FTDC_NOTETYPE_TradeSettleBill = _thostmduserapi.THOST_FTDC_NOTETYPE_TradeSettleBill
THOST_FTDC_NOTETYPE_TradeSettleMonth = _thostmduserapi.THOST_FTDC_NOTETYPE_TradeSettleMonth
THOST_FTDC_NOTETYPE_CallMarginNotes = _thostmduserapi.THOST_FTDC_NOTETYPE_CallMarginNotes
THOST_FTDC_NOTETYPE_ForceCloseNotes = _thostmduserapi.THOST_FTDC_NOTETYPE_ForceCloseNotes
THOST_FTDC_NOTETYPE_TradeNotes = _thostmduserapi.THOST_FTDC_NOTETYPE_TradeNotes
THOST_FTDC_NOTETYPE_DelivNotes = _thostmduserapi.THOST_FTDC_NOTETYPE_DelivNotes
THOST_FTDC_SBS_Day = _thostmduserapi.THOST_FTDC_SBS_Day
THOST_FTDC_SBS_Volume = _thostmduserapi.THOST_FTDC_SBS_Volume
THOST_FTDC_ST_Day = _thostmduserapi.THOST_FTDC_ST_Day
THOST_FTDC_ST_Month = _thostmduserapi.THOST_FTDC_ST_Month
THOST_FTDC_URT_Logon = _thostmduserapi.THOST_FTDC_URT_Logon
THOST_FTDC_URT_Transfer = _thostmduserapi.THOST_FTDC_URT_Transfer
THOST_FTDC_URT_EMail = _thostmduserapi.THOST_FTDC_URT_EMail
THOST_FTDC_URT_Fax = _thostmduserapi.THOST_FTDC_URT_Fax
THOST_FTDC_URT_ConditionOrder = _thostmduserapi.THOST_FTDC_URT_ConditionOrder
THOST_FTDC_MPT_PreSettlementPrice = _thostmduserapi.THOST_FTDC_MPT_PreSettlementPrice
THOST_FTDC_MPT_SettlementPrice = _thostmduserapi.THOST_FTDC_MPT_SettlementPrice
THOST_FTDC_MPT_AveragePrice = _thostmduserapi.THOST_FTDC_MPT_AveragePrice
THOST_FTDC_MPT_OpenPrice = _thostmduserapi.THOST_FTDC_MPT_OpenPrice
THOST_FTDC_BGS_None = _thostmduserapi.THOST_FTDC_BGS_None
THOST_FTDC_BGS_NoGenerated = _thostmduserapi.THOST_FTDC_BGS_NoGenerated
THOST_FTDC_BGS_Generated = _thostmduserapi.THOST_FTDC_BGS_Generated
THOST_FTDC_AT_HandlePositionAlgo = _thostmduserapi.THOST_FTDC_AT_HandlePositionAlgo
THOST_FTDC_AT_FindMarginRateAlgo = _thostmduserapi.THOST_FTDC_AT_FindMarginRateAlgo
THOST_FTDC_HPA_Base = _thostmduserapi.THOST_FTDC_HPA_Base
THOST_FTDC_HPA_DCE = _thostmduserapi.THOST_FTDC_HPA_DCE
THOST_FTDC_HPA_CZCE = _thostmduserapi.THOST_FTDC_HPA_CZCE
THOST_FTDC_FMRA_Base = _thostmduserapi.THOST_FTDC_FMRA_Base
THOST_FTDC_FMRA_DCE = _thostmduserapi.THOST_FTDC_FMRA_DCE
THOST_FTDC_FMRA_CZCE = _thostmduserapi.THOST_FTDC_FMRA_CZCE
THOST_FTDC_HTAA_Base = _thostmduserapi.THOST_FTDC_HTAA_Base
THOST_FTDC_HTAA_DCE = _thostmduserapi.THOST_FTDC_HTAA_DCE
THOST_FTDC_HTAA_CZCE = _thostmduserapi.THOST_FTDC_HTAA_CZCE
THOST_FTDC_PST_Order = _thostmduserapi.THOST_FTDC_PST_Order
THOST_FTDC_PST_Open = _thostmduserapi.THOST_FTDC_PST_Open
THOST_FTDC_PST_Fund = _thostmduserapi.THOST_FTDC_PST_Fund
THOST_FTDC_PST_Settlement = _thostmduserapi.THOST_FTDC_PST_Settlement
THOST_FTDC_PST_Company = _thostmduserapi.THOST_FTDC_PST_Company
THOST_FTDC_PST_Corporation = _thostmduserapi.THOST_FTDC_PST_Corporation
THOST_FTDC_PST_LinkMan = _thostmduserapi.THOST_FTDC_PST_LinkMan
THOST_FTDC_PST_Ledger = _thostmduserapi.THOST_FTDC_PST_Ledger
THOST_FTDC_PST_Trustee = _thostmduserapi.THOST_FTDC_PST_Trustee
THOST_FTDC_PST_TrusteeCorporation = _thostmduserapi.THOST_FTDC_PST_TrusteeCorporation
THOST_FTDC_PST_TrusteeOpen = _thostmduserapi.THOST_FTDC_PST_TrusteeOpen
THOST_FTDC_PST_TrusteeContact = _thostmduserapi.THOST_FTDC_PST_TrusteeContact
THOST_FTDC_PST_ForeignerRefer = _thostmduserapi.THOST_FTDC_PST_ForeignerRefer
THOST_FTDC_PST_CorporationRefer = _thostmduserapi.THOST_FTDC_PST_CorporationRefer
THOST_FTDC_QIR_All = _thostmduserapi.THOST_FTDC_QIR_All
THOST_FTDC_QIR_Group = _thostmduserapi.THOST_FTDC_QIR_Group
THOST_FTDC_QIR_Single = _thostmduserapi.THOST_FTDC_QIR_Single
THOST_FTDC_IRS_Normal = _thostmduserapi.THOST_FTDC_IRS_Normal
THOST_FTDC_IRS_Warn = _thostmduserapi.THOST_FTDC_IRS_Warn
THOST_FTDC_IRS_Call = _thostmduserapi.THOST_FTDC_IRS_Call
THOST_FTDC_IRS_Force = _thostmduserapi.THOST_FTDC_IRS_Force
THOST_FTDC_IRS_Exception = _thostmduserapi.THOST_FTDC_IRS_Exception
THOST_FTDC_UET_Login = _thostmduserapi.THOST_FTDC_UET_Login
THOST_FTDC_UET_Logout = _thostmduserapi.THOST_FTDC_UET_Logout
THOST_FTDC_UET_Trading = _thostmduserapi.THOST_FTDC_UET_Trading
THOST_FTDC_UET_TradingError = _thostmduserapi.THOST_FTDC_UET_TradingError
THOST_FTDC_UET_UpdatePassword = _thostmduserapi.THOST_FTDC_UET_UpdatePassword
THOST_FTDC_UET_Authenticate = _thostmduserapi.THOST_FTDC_UET_Authenticate
THOST_FTDC_UET_Other = _thostmduserapi.THOST_FTDC_UET_Other
THOST_FTDC_ICS_Close = _thostmduserapi.THOST_FTDC_ICS_Close
THOST_FTDC_ICS_CloseToday = _thostmduserapi.THOST_FTDC_ICS_CloseToday
THOST_FTDC_SM_Non = _thostmduserapi.THOST_FTDC_SM_Non
THOST_FTDC_SM_Instrument = _thostmduserapi.THOST_FTDC_SM_Instrument
THOST_FTDC_SM_Product = _thostmduserapi.THOST_FTDC_SM_Product
THOST_FTDC_SM_Investor = _thostmduserapi.THOST_FTDC_SM_Investor
THOST_FTDC_PAOS_NotSend = _thostmduserapi.THOST_FTDC_PAOS_NotSend
THOST_FTDC_PAOS_Send = _thostmduserapi.THOST_FTDC_PAOS_Send
THOST_FTDC_PAOS_Deleted = _thostmduserapi.THOST_FTDC_PAOS_Deleted
THOST_FTDC_VDS_Dealing = _thostmduserapi.THOST_FTDC_VDS_Dealing
THOST_FTDC_VDS_DeaclSucceed = _thostmduserapi.THOST_FTDC_VDS_DeaclSucceed
THOST_FTDC_ORGS_Standard = _thostmduserapi.THOST_FTDC_ORGS_Standard
THOST_FTDC_ORGS_ESunny = _thostmduserapi.THOST_FTDC_ORGS_ESunny
THOST_FTDC_ORGS_KingStarV6 = _thostmduserapi.THOST_FTDC_ORGS_KingStarV6
THOST_FTDC_VTS_NaturalDeal = _thostmduserapi.THOST_FTDC_VTS_NaturalDeal
THOST_FTDC_VTS_SucceedEnd = _thostmduserapi.THOST_FTDC_VTS_SucceedEnd
THOST_FTDC_VTS_FailedEND = _thostmduserapi.THOST_FTDC_VTS_FailedEND
THOST_FTDC_VTS_Exception = _thostmduserapi.THOST_FTDC_VTS_Exception
THOST_FTDC_VTS_ManualDeal = _thostmduserapi.THOST_FTDC_VTS_ManualDeal
THOST_FTDC_VTS_MesException = _thostmduserapi.THOST_FTDC_VTS_MesException
THOST_FTDC_VTS_SysException = _thostmduserapi.THOST_FTDC_VTS_SysException
THOST_FTDC_VBAT_BankBook = _thostmduserapi.THOST_FTDC_VBAT_BankBook
THOST_FTDC_VBAT_BankCard = _thostmduserapi.THOST_FTDC_VBAT_BankCard
THOST_FTDC_VBAT_CreditCard = _thostmduserapi.THOST_FTDC_VBAT_CreditCard
THOST_FTDC_VMS_Natural = _thostmduserapi.THOST_FTDC_VMS_Natural
THOST_FTDC_VMS_Canceled = _thostmduserapi.THOST_FTDC_VMS_Canceled
THOST_FTDC_VAA_NoAvailAbility = _thostmduserapi.THOST_FTDC_VAA_NoAvailAbility
THOST_FTDC_VAA_AvailAbility = _thostmduserapi.THOST_FTDC_VAA_AvailAbility
THOST_FTDC_VAA_Repeal = _thostmduserapi.THOST_FTDC_VAA_Repeal
THOST_FTDC_GEN_Program = _thostmduserapi.THOST_FTDC_GEN_Program
THOST_FTDC_GEN_HandWork = _thostmduserapi.THOST_FTDC_GEN_HandWork
THOST_FTDC_CFMMCKK_REQUEST = _thostmduserapi.THOST_FTDC_CFMMCKK_REQUEST
THOST_FTDC_CFMMCKK_AUTO = _thostmduserapi.THOST_FTDC_CFMMCKK_AUTO
THOST_FTDC_CFMMCKK_MANUAL = _thostmduserapi.THOST_FTDC_CFMMCKK_MANUAL
THOST_FTDC_CFT_IDCard = _thostmduserapi.THOST_FTDC_CFT_IDCard
THOST_FTDC_CFT_Passport = _thostmduserapi.THOST_FTDC_CFT_Passport
THOST_FTDC_CFT_OfficerIDCard = _thostmduserapi.THOST_FTDC_CFT_OfficerIDCard
THOST_FTDC_CFT_SoldierIDCard = _thostmduserapi.THOST_FTDC_CFT_SoldierIDCard
THOST_FTDC_CFT_HomeComingCard = _thostmduserapi.THOST_FTDC_CFT_HomeComingCard
THOST_FTDC_CFT_HouseholdRegister = _thostmduserapi.THOST_FTDC_CFT_HouseholdRegister
THOST_FTDC_CFT_LicenseNo = _thostmduserapi.THOST_FTDC_CFT_LicenseNo
THOST_FTDC_CFT_InstitutionCodeCard = _thostmduserapi.THOST_FTDC_CFT_InstitutionCodeCard
THOST_FTDC_CFT_TempLicenseNo = _thostmduserapi.THOST_FTDC_CFT_TempLicenseNo
THOST_FTDC_CFT_NoEnterpriseLicenseNo = _thostmduserapi.THOST_FTDC_CFT_NoEnterpriseLicenseNo
THOST_FTDC_CFT_OtherCard = _thostmduserapi.THOST_FTDC_CFT_OtherCard
THOST_FTDC_CFT_SuperDepAgree = _thostmduserapi.THOST_FTDC_CFT_SuperDepAgree
THOST_FTDC_FBC_Others = _thostmduserapi.THOST_FTDC_FBC_Others
THOST_FTDC_FBC_TransferDetails = _thostmduserapi.THOST_FTDC_FBC_TransferDetails
THOST_FTDC_FBC_CustAccStatus = _thostmduserapi.THOST_FTDC_FBC_CustAccStatus
THOST_FTDC_FBC_AccountTradeDetails = _thostmduserapi.THOST_FTDC_FBC_AccountTradeDetails
THOST_FTDC_FBC_FutureAccountChangeInfoDetails = _thostmduserapi.THOST_FTDC_FBC_FutureAccountChangeInfoDetails
THOST_FTDC_FBC_CustMoneyDetail = _thostmduserapi.THOST_FTDC_FBC_CustMoneyDetail
THOST_FTDC_FBC_CustCancelAccountInfo = _thostmduserapi.THOST_FTDC_FBC_CustCancelAccountInfo
THOST_FTDC_FBC_CustMoneyResult = _thostmduserapi.THOST_FTDC_FBC_CustMoneyResult
THOST_FTDC_FBC_OthersExceptionResult = _thostmduserapi.THOST_FTDC_FBC_OthersExceptionResult
THOST_FTDC_FBC_CustInterestNetMoneyDetails = _thostmduserapi.THOST_FTDC_FBC_CustInterestNetMoneyDetails
THOST_FTDC_FBC_CustMoneySendAndReceiveDetails = _thostmduserapi.THOST_FTDC_FBC_CustMoneySendAndReceiveDetails
THOST_FTDC_FBC_CorporationMoneyTotal = _thostmduserapi.THOST_FTDC_FBC_CorporationMoneyTotal
THOST_FTDC_FBC_MainbodyMoneyTotal = _thostmduserapi.THOST_FTDC_FBC_MainbodyMoneyTotal
THOST_FTDC_FBC_MainPartMonitorData = _thostmduserapi.THOST_FTDC_FBC_MainPartMonitorData
THOST_FTDC_FBC_PreparationMoney = _thostmduserapi.THOST_FTDC_FBC_PreparationMoney
THOST_FTDC_FBC_BankMoneyMonitorData = _thostmduserapi.THOST_FTDC_FBC_BankMoneyMonitorData
THOST_FTDC_CEC_Exchange = _thostmduserapi.THOST_FTDC_CEC_Exchange
THOST_FTDC_CEC_Cash = _thostmduserapi.THOST_FTDC_CEC_Cash
THOST_FTDC_YNI_Yes = _thostmduserapi.THOST_FTDC_YNI_Yes
THOST_FTDC_YNI_No = _thostmduserapi.THOST_FTDC_YNI_No
THOST_FTDC_BLT_CurrentMoney = _thostmduserapi.THOST_FTDC_BLT_CurrentMoney
THOST_FTDC_BLT_UsableMoney = _thostmduserapi.THOST_FTDC_BLT_UsableMoney
THOST_FTDC_BLT_FetchableMoney = _thostmduserapi.THOST_FTDC_BLT_FetchableMoney
THOST_FTDC_BLT_FreezeMoney = _thostmduserapi.THOST_FTDC_BLT_FreezeMoney
THOST_FTDC_GD_Unknown = _thostmduserapi.THOST_FTDC_GD_Unknown
THOST_FTDC_GD_Male = _thostmduserapi.THOST_FTDC_GD_Male
THOST_FTDC_GD_Female = _thostmduserapi.THOST_FTDC_GD_Female
THOST_FTDC_FPF_BEN = _thostmduserapi.THOST_FTDC_FPF_BEN
THOST_FTDC_FPF_OUR = _thostmduserapi.THOST_FTDC_FPF_OUR
THOST_FTDC_FPF_SHA = _thostmduserapi.THOST_FTDC_FPF_SHA
THOST_FTDC_PWKT_ExchangeKey = _thostmduserapi.THOST_FTDC_PWKT_ExchangeKey
THOST_FTDC_PWKT_PassWordKey = _thostmduserapi.THOST_FTDC_PWKT_PassWordKey
THOST_FTDC_PWKT_MACKey = _thostmduserapi.THOST_FTDC_PWKT_MACKey
THOST_FTDC_PWKT_MessageKey = _thostmduserapi.THOST_FTDC_PWKT_MessageKey
THOST_FTDC_PWT_Query = _thostmduserapi.THOST_FTDC_PWT_Query
THOST_FTDC_PWT_Fetch = _thostmduserapi.THOST_FTDC_PWT_Fetch
THOST_FTDC_PWT_Transfer = _thostmduserapi.THOST_FTDC_PWT_Transfer
THOST_FTDC_PWT_Trade = _thostmduserapi.THOST_FTDC_PWT_Trade
THOST_FTDC_EM_NoEncry = _thostmduserapi.THOST_FTDC_EM_NoEncry
THOST_FTDC_EM_DES = _thostmduserapi.THOST_FTDC_EM_DES
THOST_FTDC_EM_3DES = _thostmduserapi.THOST_FTDC_EM_3DES
THOST_FTDC_BRF_BankNotNeedRepeal = _thostmduserapi.THOST_FTDC_BRF_BankNotNeedRepeal
THOST_FTDC_BRF_BankWaitingRepeal = _thostmduserapi.THOST_FTDC_BRF_BankWaitingRepeal
THOST_FTDC_BRF_BankBeenRepealed = _thostmduserapi.THOST_FTDC_BRF_BankBeenRepealed
THOST_FTDC_BRORF_BrokerNotNeedRepeal = _thostmduserapi.THOST_FTDC_BRORF_BrokerNotNeedRepeal
THOST_FTDC_BRORF_BrokerWaitingRepeal = _thostmduserapi.THOST_FTDC_BRORF_BrokerWaitingRepeal
THOST_FTDC_BRORF_BrokerBeenRepealed = _thostmduserapi.THOST_FTDC_BRORF_BrokerBeenRepealed
THOST_FTDC_TS_Bank = _thostmduserapi.THOST_FTDC_TS_Bank
THOST_FTDC_TS_Future = _thostmduserapi.THOST_FTDC_TS_Future
THOST_FTDC_TS_Store = _thostmduserapi.THOST_FTDC_TS_Store
THOST_FTDC_LF_Yes = _thostmduserapi.THOST_FTDC_LF_Yes
THOST_FTDC_LF_No = _thostmduserapi.THOST_FTDC_LF_No
THOST_FTDC_BAS_Normal = _thostmduserapi.THOST_FTDC_BAS_Normal
THOST_FTDC_BAS_Freeze = _thostmduserapi.THOST_FTDC_BAS_Freeze
THOST_FTDC_BAS_ReportLoss = _thostmduserapi.THOST_FTDC_BAS_ReportLoss
THOST_FTDC_MAS_Normal = _thostmduserapi.THOST_FTDC_MAS_Normal
THOST_FTDC_MAS_Cancel = _thostmduserapi.THOST_FTDC_MAS_Cancel
THOST_FTDC_MSS_Point = _thostmduserapi.THOST_FTDC_MSS_Point
THOST_FTDC_MSS_PrePoint = _thostmduserapi.THOST_FTDC_MSS_PrePoint
THOST_FTDC_MSS_CancelPoint = _thostmduserapi.THOST_FTDC_MSS_CancelPoint
THOST_FTDC_SYT_FutureBankTransfer = _thostmduserapi.THOST_FTDC_SYT_FutureBankTransfer
THOST_FTDC_SYT_StockBankTransfer = _thostmduserapi.THOST_FTDC_SYT_StockBankTransfer
THOST_FTDC_SYT_TheThirdPartStore = _thostmduserapi.THOST_FTDC_SYT_TheThirdPartStore
THOST_FTDC_TEF_NormalProcessing = _thostmduserapi.THOST_FTDC_TEF_NormalProcessing
THOST_FTDC_TEF_Success = _thostmduserapi.THOST_FTDC_TEF_Success
THOST_FTDC_TEF_Failed = _thostmduserapi.THOST_FTDC_TEF_Failed
THOST_FTDC_TEF_Abnormal = _thostmduserapi.THOST_FTDC_TEF_Abnormal
THOST_FTDC_TEF_ManualProcessedForException = _thostmduserapi.THOST_FTDC_TEF_ManualProcessedForException
THOST_FTDC_TEF_CommuFailedNeedManualProcess = _thostmduserapi.THOST_FTDC_TEF_CommuFailedNeedManualProcess
THOST_FTDC_TEF_SysErrorNeedManualProcess = _thostmduserapi.THOST_FTDC_TEF_SysErrorNeedManualProcess
THOST_FTDC_PSS_NotProcess = _thostmduserapi.THOST_FTDC_PSS_NotProcess
THOST_FTDC_PSS_StartProcess = _thostmduserapi.THOST_FTDC_PSS_StartProcess
THOST_FTDC_PSS_Finished = _thostmduserapi.THOST_FTDC_PSS_Finished
THOST_FTDC_CUSTT_Person = _thostmduserapi.THOST_FTDC_CUSTT_Person
THOST_FTDC_CUSTT_Institution = _thostmduserapi.THOST_FTDC_CUSTT_Institution
THOST_FTDC_FBTTD_FromBankToFuture = _thostmduserapi.THOST_FTDC_FBTTD_FromBankToFuture
THOST_FTDC_FBTTD_FromFutureToBank = _thostmduserapi.THOST_FTDC_FBTTD_FromFutureToBank
THOST_FTDC_OOD_Open = _thostmduserapi.THOST_FTDC_OOD_Open
THOST_FTDC_OOD_Destroy = _thostmduserapi.THOST_FTDC_OOD_Destroy
THOST_FTDC_AVAF_Invalid = _thostmduserapi.THOST_FTDC_AVAF_Invalid
THOST_FTDC_AVAF_Valid = _thostmduserapi.THOST_FTDC_AVAF_Valid
THOST_FTDC_AVAF_Repeal = _thostmduserapi.THOST_FTDC_AVAF_Repeal
THOST_FTDC_OT_Bank = _thostmduserapi.THOST_FTDC_OT_Bank
THOST_FTDC_OT_Future = _thostmduserapi.THOST_FTDC_OT_Future
THOST_FTDC_OT_PlateForm = _thostmduserapi.THOST_FTDC_OT_PlateForm
THOST_FTDC_OL_HeadQuarters = _thostmduserapi.THOST_FTDC_OL_HeadQuarters
THOST_FTDC_OL_Branch = _thostmduserapi.THOST_FTDC_OL_Branch
THOST_FTDC_PID_FutureProtocal = _thostmduserapi.THOST_FTDC_PID_FutureProtocal
THOST_FTDC_PID_ICBCProtocal = _thostmduserapi.THOST_FTDC_PID_ICBCProtocal
THOST_FTDC_PID_ABCProtocal = _thostmduserapi.THOST_FTDC_PID_ABCProtocal
THOST_FTDC_PID_CBCProtocal = _thostmduserapi.THOST_FTDC_PID_CBCProtocal
THOST_FTDC_PID_CCBProtocal = _thostmduserapi.THOST_FTDC_PID_CCBProtocal
THOST_FTDC_PID_BOCOMProtocal = _thostmduserapi.THOST_FTDC_PID_BOCOMProtocal
THOST_FTDC_PID_FBTPlateFormProtocal = _thostmduserapi.THOST_FTDC_PID_FBTPlateFormProtocal
THOST_FTDC_CM_ShortConnect = _thostmduserapi.THOST_FTDC_CM_ShortConnect
THOST_FTDC_CM_LongConnect = _thostmduserapi.THOST_FTDC_CM_LongConnect
THOST_FTDC_SRM_ASync = _thostmduserapi.THOST_FTDC_SRM_ASync
THOST_FTDC_SRM_Sync = _thostmduserapi.THOST_FTDC_SRM_Sync
THOST_FTDC_BAT_BankBook = _thostmduserapi.THOST_FTDC_BAT_BankBook
THOST_FTDC_BAT_SavingCard = _thostmduserapi.THOST_FTDC_BAT_SavingCard
THOST_FTDC_BAT_CreditCard = _thostmduserapi.THOST_FTDC_BAT_CreditCard
THOST_FTDC_FAT_BankBook = _thostmduserapi.THOST_FTDC_FAT_BankBook
THOST_FTDC_FAT_SavingCard = _thostmduserapi.THOST_FTDC_FAT_SavingCard
THOST_FTDC_FAT_CreditCard = _thostmduserapi.THOST_FTDC_FAT_CreditCard
THOST_FTDC_OS_Ready = _thostmduserapi.THOST_FTDC_OS_Ready
THOST_FTDC_OS_CheckIn = _thostmduserapi.THOST_FTDC_OS_CheckIn
THOST_FTDC_OS_CheckOut = _thostmduserapi.THOST_FTDC_OS_CheckOut
THOST_FTDC_OS_CheckFileArrived = _thostmduserapi.THOST_FTDC_OS_CheckFileArrived
THOST_FTDC_OS_CheckDetail = _thostmduserapi.THOST_FTDC_OS_CheckDetail
THOST_FTDC_OS_DayEndClean = _thostmduserapi.THOST_FTDC_OS_DayEndClean
THOST_FTDC_OS_Invalid = _thostmduserapi.THOST_FTDC_OS_Invalid
THOST_FTDC_CCBFM_ByAmount = _thostmduserapi.THOST_FTDC_CCBFM_ByAmount
THOST_FTDC_CCBFM_ByMonth = _thostmduserapi.THOST_FTDC_CCBFM_ByMonth
THOST_FTDC_CAPIT_Client = _thostmduserapi.THOST_FTDC_CAPIT_Client
THOST_FTDC_CAPIT_Server = _thostmduserapi.THOST_FTDC_CAPIT_Server
THOST_FTDC_CAPIT_UserApi = _thostmduserapi.THOST_FTDC_CAPIT_UserApi
THOST_FTDC_LS_Connected = _thostmduserapi.THOST_FTDC_LS_Connected
THOST_FTDC_LS_Disconnected = _thostmduserapi.THOST_FTDC_LS_Disconnected
THOST_FTDC_BPWDF_NoCheck = _thostmduserapi.THOST_FTDC_BPWDF_NoCheck
THOST_FTDC_BPWDF_BlankCheck = _thostmduserapi.THOST_FTDC_BPWDF_BlankCheck
THOST_FTDC_BPWDF_EncryptCheck = _thostmduserapi.THOST_FTDC_BPWDF_EncryptCheck
THOST_FTDC_SAT_AccountID = _thostmduserapi.THOST_FTDC_SAT_AccountID
THOST_FTDC_SAT_CardID = _thostmduserapi.THOST_FTDC_SAT_CardID
THOST_FTDC_SAT_SHStockholderID = _thostmduserapi.THOST_FTDC_SAT_SHStockholderID
THOST_FTDC_SAT_SZStockholderID = _thostmduserapi.THOST_FTDC_SAT_SZStockholderID
THOST_FTDC_TRFS_Normal = _thostmduserapi.THOST_FTDC_TRFS_Normal
THOST_FTDC_TRFS_Repealed = _thostmduserapi.THOST_FTDC_TRFS_Repealed
THOST_FTDC_SPTYPE_Broker = _thostmduserapi.THOST_FTDC_SPTYPE_Broker
THOST_FTDC_SPTYPE_Bank = _thostmduserapi.THOST_FTDC_SPTYPE_Bank
THOST_FTDC_REQRSP_Request = _thostmduserapi.THOST_FTDC_REQRSP_Request
THOST_FTDC_REQRSP_Response = _thostmduserapi.THOST_FTDC_REQRSP_Response
THOST_FTDC_FBTUET_SignIn = _thostmduserapi.THOST_FTDC_FBTUET_SignIn
THOST_FTDC_FBTUET_FromBankToFuture = _thostmduserapi.THOST_FTDC_FBTUET_FromBankToFuture
THOST_FTDC_FBTUET_FromFutureToBank = _thostmduserapi.THOST_FTDC_FBTUET_FromFutureToBank
THOST_FTDC_FBTUET_OpenAccount = _thostmduserapi.THOST_FTDC_FBTUET_OpenAccount
THOST_FTDC_FBTUET_CancelAccount = _thostmduserapi.THOST_FTDC_FBTUET_CancelAccount
THOST_FTDC_FBTUET_ChangeAccount = _thostmduserapi.THOST_FTDC_FBTUET_ChangeAccount
THOST_FTDC_FBTUET_RepealFromBankToFuture = _thostmduserapi.THOST_FTDC_FBTUET_RepealFromBankToFuture
THOST_FTDC_FBTUET_RepealFromFutureToBank = _thostmduserapi.THOST_FTDC_FBTUET_RepealFromFutureToBank
THOST_FTDC_FBTUET_QueryBankAccount = _thostmduserapi.THOST_FTDC_FBTUET_QueryBankAccount
THOST_FTDC_FBTUET_QueryFutureAccount = _thostmduserapi.THOST_FTDC_FBTUET_QueryFutureAccount
THOST_FTDC_FBTUET_SignOut = _thostmduserapi.THOST_FTDC_FBTUET_SignOut
THOST_FTDC_FBTUET_SyncKey = _thostmduserapi.THOST_FTDC_FBTUET_SyncKey
THOST_FTDC_FBTUET_ReserveOpenAccount = _thostmduserapi.THOST_FTDC_FBTUET_ReserveOpenAccount
THOST_FTDC_FBTUET_CancelReserveOpenAccount = _thostmduserapi.THOST_FTDC_FBTUET_CancelReserveOpenAccount
THOST_FTDC_FBTUET_ReserveOpenAccountConfirm = _thostmduserapi.THOST_FTDC_FBTUET_ReserveOpenAccountConfirm
THOST_FTDC_FBTUET_Other = _thostmduserapi.THOST_FTDC_FBTUET_Other
THOST_FTDC_DBOP_Insert = _thostmduserapi.THOST_FTDC_DBOP_Insert
THOST_FTDC_DBOP_Update = _thostmduserapi.THOST_FTDC_DBOP_Update
THOST_FTDC_DBOP_Delete = _thostmduserapi.THOST_FTDC_DBOP_Delete
THOST_FTDC_SYNF_Yes = _thostmduserapi.THOST_FTDC_SYNF_Yes
THOST_FTDC_SYNF_No = _thostmduserapi.THOST_FTDC_SYNF_No
THOST_FTDC_SYNT_OneOffSync = _thostmduserapi.THOST_FTDC_SYNT_OneOffSync
THOST_FTDC_SYNT_TimerSync = _thostmduserapi.THOST_FTDC_SYNT_TimerSync
THOST_FTDC_SYNT_TimerFullSync = _thostmduserapi.THOST_FTDC_SYNT_TimerFullSync
THOST_FTDC_FBEDIR_Settlement = _thostmduserapi.THOST_FTDC_FBEDIR_Settlement
THOST_FTDC_FBEDIR_Sale = _thostmduserapi.THOST_FTDC_FBEDIR_Sale
THOST_FTDC_FBERES_Success = _thostmduserapi.THOST_FTDC_FBERES_Success
THOST_FTDC_FBERES_InsufficientBalance = _thostmduserapi.THOST_FTDC_FBERES_InsufficientBalance
THOST_FTDC_FBERES_UnknownTrading = _thostmduserapi.THOST_FTDC_FBERES_UnknownTrading
THOST_FTDC_FBERES_Fail = _thostmduserapi.THOST_FTDC_FBERES_Fail
THOST_FTDC_FBEES_Normal = _thostmduserapi.THOST_FTDC_FBEES_Normal
THOST_FTDC_FBEES_ReExchange = _thostmduserapi.THOST_FTDC_FBEES_ReExchange
THOST_FTDC_FBEFG_DataPackage = _thostmduserapi.THOST_FTDC_FBEFG_DataPackage
THOST_FTDC_FBEFG_File = _thostmduserapi.THOST_FTDC_FBEFG_File
THOST_FTDC_FBEAT_NotTrade = _thostmduserapi.THOST_FTDC_FBEAT_NotTrade
THOST_FTDC_FBEAT_Trade = _thostmduserapi.THOST_FTDC_FBEAT_Trade
THOST_FTDC_FBEUET_SignIn = _thostmduserapi.THOST_FTDC_FBEUET_SignIn
THOST_FTDC_FBEUET_Exchange = _thostmduserapi.THOST_FTDC_FBEUET_Exchange
THOST_FTDC_FBEUET_ReExchange = _thostmduserapi.THOST_FTDC_FBEUET_ReExchange
THOST_FTDC_FBEUET_QueryBankAccount = _thostmduserapi.THOST_FTDC_FBEUET_QueryBankAccount
THOST_FTDC_FBEUET_QueryExchDetial = _thostmduserapi.THOST_FTDC_FBEUET_QueryExchDetial
THOST_FTDC_FBEUET_QueryExchSummary = _thostmduserapi.THOST_FTDC_FBEUET_QueryExchSummary
THOST_FTDC_FBEUET_QueryExchRate = _thostmduserapi.THOST_FTDC_FBEUET_QueryExchRate
THOST_FTDC_FBEUET_CheckBankAccount = _thostmduserapi.THOST_FTDC_FBEUET_CheckBankAccount
THOST_FTDC_FBEUET_SignOut = _thostmduserapi.THOST_FTDC_FBEUET_SignOut
THOST_FTDC_FBEUET_Other = _thostmduserapi.THOST_FTDC_FBEUET_Other
THOST_FTDC_FBERF_UnProcessed = _thostmduserapi.THOST_FTDC_FBERF_UnProcessed
THOST_FTDC_FBERF_WaitSend = _thostmduserapi.THOST_FTDC_FBERF_WaitSend
THOST_FTDC_FBERF_SendSuccess = _thostmduserapi.THOST_FTDC_FBERF_SendSuccess
THOST_FTDC_FBERF_SendFailed = _thostmduserapi.THOST_FTDC_FBERF_SendFailed
THOST_FTDC_FBERF_WaitReSend = _thostmduserapi.THOST_FTDC_FBERF_WaitReSend
THOST_FTDC_NC_NOERROR = _thostmduserapi.THOST_FTDC_NC_NOERROR
THOST_FTDC_NC_Warn = _thostmduserapi.THOST_FTDC_NC_Warn
THOST_FTDC_NC_Call = _thostmduserapi.THOST_FTDC_NC_Call
THOST_FTDC_NC_Force = _thostmduserapi.THOST_FTDC_NC_Force
THOST_FTDC_NC_CHUANCANG = _thostmduserapi.THOST_FTDC_NC_CHUANCANG
THOST_FTDC_NC_Exception = _thostmduserapi.THOST_FTDC_NC_Exception
THOST_FTDC_FCT_Manual = _thostmduserapi.THOST_FTDC_FCT_Manual
THOST_FTDC_FCT_Single = _thostmduserapi.THOST_FTDC_FCT_Single
THOST_FTDC_FCT_Group = _thostmduserapi.THOST_FTDC_FCT_Group
THOST_FTDC_RNM_System = _thostmduserapi.THOST_FTDC_RNM_System
THOST_FTDC_RNM_SMS = _thostmduserapi.THOST_FTDC_RNM_SMS
THOST_FTDC_RNM_EMail = _thostmduserapi.THOST_FTDC_RNM_EMail
THOST_FTDC_RNM_Manual = _thostmduserapi.THOST_FTDC_RNM_Manual
THOST_FTDC_RNS_NotGen = _thostmduserapi.THOST_FTDC_RNS_NotGen
THOST_FTDC_RNS_Generated = _thostmduserapi.THOST_FTDC_RNS_Generated
THOST_FTDC_RNS_SendError = _thostmduserapi.THOST_FTDC_RNS_SendError
THOST_FTDC_RNS_SendOk = _thostmduserapi.THOST_FTDC_RNS_SendOk
THOST_FTDC_RNS_Received = _thostmduserapi.THOST_FTDC_RNS_Received
THOST_FTDC_RNS_Confirmed = _thostmduserapi.THOST_FTDC_RNS_Confirmed
THOST_FTDC_RUE_ExportData = _thostmduserapi.THOST_FTDC_RUE_ExportData
THOST_FTDC_COST_LastPriceAsc = _thostmduserapi.THOST_FTDC_COST_LastPriceAsc
THOST_FTDC_COST_LastPriceDesc = _thostmduserapi.THOST_FTDC_COST_LastPriceDesc
THOST_FTDC_COST_AskPriceAsc = _thostmduserapi.THOST_FTDC_COST_AskPriceAsc
THOST_FTDC_COST_AskPriceDesc = _thostmduserapi.THOST_FTDC_COST_AskPriceDesc
THOST_FTDC_COST_BidPriceAsc = _thostmduserapi.THOST_FTDC_COST_BidPriceAsc
THOST_FTDC_COST_BidPriceDesc = _thostmduserapi.THOST_FTDC_COST_BidPriceDesc
THOST_FTDC_UOAST_NoSend = _thostmduserapi.THOST_FTDC_UOAST_NoSend
THOST_FTDC_UOAST_Sended = _thostmduserapi.THOST_FTDC_UOAST_Sended
THOST_FTDC_UOAST_Generated = _thostmduserapi.THOST_FTDC_UOAST_Generated
THOST_FTDC_UOAST_SendFail = _thostmduserapi.THOST_FTDC_UOAST_SendFail
THOST_FTDC_UOAST_Success = _thostmduserapi.THOST_FTDC_UOAST_Success
THOST_FTDC_UOAST_Fail = _thostmduserapi.THOST_FTDC_UOAST_Fail
THOST_FTDC_UOAST_Cancel = _thostmduserapi.THOST_FTDC_UOAST_Cancel
THOST_FTDC_UOACS_NoApply = _thostmduserapi.THOST_FTDC_UOACS_NoApply
THOST_FTDC_UOACS_Submited = _thostmduserapi.THOST_FTDC_UOACS_Submited
THOST_FTDC_UOACS_Sended = _thostmduserapi.THOST_FTDC_UOACS_Sended
THOST_FTDC_UOACS_Success = _thostmduserapi.THOST_FTDC_UOACS_Success
THOST_FTDC_UOACS_Refuse = _thostmduserapi.THOST_FTDC_UOACS_Refuse
THOST_FTDC_UOACS_Cancel = _thostmduserapi.THOST_FTDC_UOACS_Cancel
THOST_FTDC_QT_Radio = _thostmduserapi.THOST_FTDC_QT_Radio
THOST_FTDC_QT_Option = _thostmduserapi.THOST_FTDC_QT_Option
THOST_FTDC_QT_Blank = _thostmduserapi.THOST_FTDC_QT_Blank
THOST_FTDC_BT_Request = _thostmduserapi.THOST_FTDC_BT_Request
THOST_FTDC_BT_Response = _thostmduserapi.THOST_FTDC_BT_Response
THOST_FTDC_BT_Notice = _thostmduserapi.THOST_FTDC_BT_Notice
THOST_FTDC_CRC_Success = _thostmduserapi.THOST_FTDC_CRC_Success
THOST_FTDC_CRC_Working = _thostmduserapi.THOST_FTDC_CRC_Working
THOST_FTDC_CRC_InfoFail = _thostmduserapi.THOST_FTDC_CRC_InfoFail
THOST_FTDC_CRC_IDCardFail = _thostmduserapi.THOST_FTDC_CRC_IDCardFail
THOST_FTDC_CRC_OtherFail = _thostmduserapi.THOST_FTDC_CRC_OtherFail
THOST_FTDC_CfMMCCT_All = _thostmduserapi.THOST_FTDC_CfMMCCT_All
THOST_FTDC_CfMMCCT_Person = _thostmduserapi.THOST_FTDC_CfMMCCT_Person
THOST_FTDC_CfMMCCT_Company = _thostmduserapi.THOST_FTDC_CfMMCCT_Company
THOST_FTDC_CfMMCCT_Other = _thostmduserapi.THOST_FTDC_CfMMCCT_Other
THOST_FTDC_CfMMCCT_SpecialOrgan = _thostmduserapi.THOST_FTDC_CfMMCCT_SpecialOrgan
THOST_FTDC_CfMMCCT_Asset = _thostmduserapi.THOST_FTDC_CfMMCCT_Asset
THOST_FTDC_EIDT_SHFE = _thostmduserapi.THOST_FTDC_EIDT_SHFE
THOST_FTDC_EIDT_CZCE = _thostmduserapi.THOST_FTDC_EIDT_CZCE
THOST_FTDC_EIDT_DCE = _thostmduserapi.THOST_FTDC_EIDT_DCE
THOST_FTDC_EIDT_CFFEX = _thostmduserapi.THOST_FTDC_EIDT_CFFEX
THOST_FTDC_EIDT_INE = _thostmduserapi.THOST_FTDC_EIDT_INE
THOST_FTDC_ECIDT_Hedge = _thostmduserapi.THOST_FTDC_ECIDT_Hedge
THOST_FTDC_ECIDT_Arbitrage = _thostmduserapi.THOST_FTDC_ECIDT_Arbitrage
THOST_FTDC_ECIDT_Speculation = _thostmduserapi.THOST_FTDC_ECIDT_Speculation
THOST_FTDC_UF_NoUpdate = _thostmduserapi.THOST_FTDC_UF_NoUpdate
THOST_FTDC_UF_Success = _thostmduserapi.THOST_FTDC_UF_Success
THOST_FTDC_UF_Fail = _thostmduserapi.THOST_FTDC_UF_Fail
THOST_FTDC_UF_TCSuccess = _thostmduserapi.THOST_FTDC_UF_TCSuccess
THOST_FTDC_UF_TCFail = _thostmduserapi.THOST_FTDC_UF_TCFail
THOST_FTDC_UF_Cancel = _thostmduserapi.THOST_FTDC_UF_Cancel
THOST_FTDC_AOID_OpenInvestor = _thostmduserapi.THOST_FTDC_AOID_OpenInvestor
THOST_FTDC_AOID_ModifyIDCard = _thostmduserapi.THOST_FTDC_AOID_ModifyIDCard
THOST_FTDC_AOID_ModifyNoIDCard = _thostmduserapi.THOST_FTDC_AOID_ModifyNoIDCard
THOST_FTDC_AOID_ApplyTradingCode = _thostmduserapi.THOST_FTDC_AOID_ApplyTradingCode
THOST_FTDC_AOID_CancelTradingCode = _thostmduserapi.THOST_FTDC_AOID_CancelTradingCode
THOST_FTDC_AOID_CancelInvestor = _thostmduserapi.THOST_FTDC_AOID_CancelInvestor
THOST_FTDC_AOID_FreezeAccount = _thostmduserapi.THOST_FTDC_AOID_FreezeAccount
THOST_FTDC_AOID_ActiveFreezeAccount = _thostmduserapi.THOST_FTDC_AOID_ActiveFreezeAccount
THOST_FTDC_ASID_NoComplete = _thostmduserapi.THOST_FTDC_ASID_NoComplete
THOST_FTDC_ASID_Submited = _thostmduserapi.THOST_FTDC_ASID_Submited
THOST_FTDC_ASID_Checked = _thostmduserapi.THOST_FTDC_ASID_Checked
THOST_FTDC_ASID_Refused = _thostmduserapi.THOST_FTDC_ASID_Refused
THOST_FTDC_ASID_Deleted = _thostmduserapi.THOST_FTDC_ASID_Deleted
THOST_FTDC_UOASM_ByAPI = _thostmduserapi.THOST_FTDC_UOASM_ByAPI
THOST_FTDC_UOASM_ByFile = _thostmduserapi.THOST_FTDC_UOASM_ByFile
THOST_FTDC_EvM_ADD = _thostmduserapi.THOST_FTDC_EvM_ADD
THOST_FTDC_EvM_UPDATE = _thostmduserapi.THOST_FTDC_EvM_UPDATE
THOST_FTDC_EvM_DELETE = _thostmduserapi.THOST_FTDC_EvM_DELETE
THOST_FTDC_EvM_CHECK = _thostmduserapi.THOST_FTDC_EvM_CHECK
THOST_FTDC_EvM_COPY = _thostmduserapi.THOST_FTDC_EvM_COPY
THOST_FTDC_EvM_CANCEL = _thostmduserapi.THOST_FTDC_EvM_CANCEL
THOST_FTDC_EvM_Reverse = _thostmduserapi.THOST_FTDC_EvM_Reverse
THOST_FTDC_UOAA_ASR = _thostmduserapi.THOST_FTDC_UOAA_ASR
THOST_FTDC_UOAA_ASNR = _thostmduserapi.THOST_FTDC_UOAA_ASNR
THOST_FTDC_UOAA_NSAR = _thostmduserapi.THOST_FTDC_UOAA_NSAR
THOST_FTDC_UOAA_NSR = _thostmduserapi.THOST_FTDC_UOAA_NSR
THOST_FTDC_EvM_InvestorGroupFlow = _thostmduserapi.THOST_FTDC_EvM_InvestorGroupFlow
THOST_FTDC_EvM_InvestorRate = _thostmduserapi.THOST_FTDC_EvM_InvestorRate
THOST_FTDC_EvM_InvestorCommRateModel = _thostmduserapi.THOST_FTDC_EvM_InvestorCommRateModel
THOST_FTDC_CL_Zero = _thostmduserapi.THOST_FTDC_CL_Zero
THOST_FTDC_CL_One = _thostmduserapi.THOST_FTDC_CL_One
THOST_FTDC_CL_Two = _thostmduserapi.THOST_FTDC_CL_Two
THOST_FTDC_CHS_Init = _thostmduserapi.THOST_FTDC_CHS_Init
THOST_FTDC_CHS_Checking = _thostmduserapi.THOST_FTDC_CHS_Checking
THOST_FTDC_CHS_Checked = _thostmduserapi.THOST_FTDC_CHS_Checked
THOST_FTDC_CHS_Refuse = _thostmduserapi.THOST_FTDC_CHS_Refuse
THOST_FTDC_CHS_Cancel = _thostmduserapi.THOST_FTDC_CHS_Cancel
THOST_FTDC_CHU_Unused = _thostmduserapi.THOST_FTDC_CHU_Unused
THOST_FTDC_CHU_Used = _thostmduserapi.THOST_FTDC_CHU_Used
THOST_FTDC_CHU_Fail = _thostmduserapi.THOST_FTDC_CHU_Fail
THOST_FTDC_BAO_ByAccProperty = _thostmduserapi.THOST_FTDC_BAO_ByAccProperty
THOST_FTDC_BAO_ByFBTransfer = _thostmduserapi.THOST_FTDC_BAO_ByFBTransfer
THOST_FTDC_MBTS_ByInstrument = _thostmduserapi.THOST_FTDC_MBTS_ByInstrument
THOST_FTDC_MBTS_ByDayInsPrc = _thostmduserapi.THOST_FTDC_MBTS_ByDayInsPrc
THOST_FTDC_MBTS_ByDayIns = _thostmduserapi.THOST_FTDC_MBTS_ByDayIns
THOST_FTDC_OTP_NONE = _thostmduserapi.THOST_FTDC_OTP_NONE
THOST_FTDC_OTP_TOTP = _thostmduserapi.THOST_FTDC_OTP_TOTP
THOST_FTDC_OTPS_Unused = _thostmduserapi.THOST_FTDC_OTPS_Unused
THOST_FTDC_OTPS_Used = _thostmduserapi.THOST_FTDC_OTPS_Used
THOST_FTDC_OTPS_Disuse = _thostmduserapi.THOST_FTDC_OTPS_Disuse
THOST_FTDC_BUT_Investor = _thostmduserapi.THOST_FTDC_BUT_Investor
THOST_FTDC_BUT_BrokerUser = _thostmduserapi.THOST_FTDC_BUT_BrokerUser
THOST_FTDC_FUTT_Commodity = _thostmduserapi.THOST_FTDC_FUTT_Commodity
THOST_FTDC_FUTT_Financial = _thostmduserapi.THOST_FTDC_FUTT_Financial
THOST_FTDC_FET_Restriction = _thostmduserapi.THOST_FTDC_FET_Restriction
THOST_FTDC_FET_TodayRestriction = _thostmduserapi.THOST_FTDC_FET_TodayRestriction
THOST_FTDC_FET_Transfer = _thostmduserapi.THOST_FTDC_FET_Transfer
THOST_FTDC_FET_Credit = _thostmduserapi.THOST_FTDC_FET_Credit
THOST_FTDC_FET_InvestorWithdrawAlm = _thostmduserapi.THOST_FTDC_FET_InvestorWithdrawAlm
THOST_FTDC_FET_BankRestriction = _thostmduserapi.THOST_FTDC_FET_BankRestriction
THOST_FTDC_FET_Accountregister = _thostmduserapi.THOST_FTDC_FET_Accountregister
THOST_FTDC_FET_ExchangeFundIO = _thostmduserapi.THOST_FTDC_FET_ExchangeFundIO
THOST_FTDC_FET_InvestorFundIO = _thostmduserapi.THOST_FTDC_FET_InvestorFundIO
THOST_FTDC_AST_FBTransfer = _thostmduserapi.THOST_FTDC_AST_FBTransfer
THOST_FTDC_AST_ManualEntry = _thostmduserapi.THOST_FTDC_AST_ManualEntry
THOST_FTDC_CST_UnifyAccount = _thostmduserapi.THOST_FTDC_CST_UnifyAccount
THOST_FTDC_CST_ManualEntry = _thostmduserapi.THOST_FTDC_CST_ManualEntry
THOST_FTDC_UR_All = _thostmduserapi.THOST_FTDC_UR_All
THOST_FTDC_UR_Single = _thostmduserapi.THOST_FTDC_UR_Single
THOST_FTDC_BG_Investor = _thostmduserapi.THOST_FTDC_BG_Investor
THOST_FTDC_BG_Group = _thostmduserapi.THOST_FTDC_BG_Group
THOST_FTDC_TSSM_Instrument = _thostmduserapi.THOST_FTDC_TSSM_Instrument
THOST_FTDC_TSSM_Product = _thostmduserapi.THOST_FTDC_TSSM_Product
THOST_FTDC_TSSM_Exchange = _thostmduserapi.THOST_FTDC_TSSM_Exchange
THOST_FTDC_ESM_Relative = _thostmduserapi.THOST_FTDC_ESM_Relative
THOST_FTDC_ESM_Typical = _thostmduserapi.THOST_FTDC_ESM_Typical
THOST_FTDC_RIR_All = _thostmduserapi.THOST_FTDC_RIR_All
THOST_FTDC_RIR_Model = _thostmduserapi.THOST_FTDC_RIR_Model
THOST_FTDC_RIR_Single = _thostmduserapi.THOST_FTDC_RIR_Single
THOST_FTDC_SDS_Initialize = _thostmduserapi.THOST_FTDC_SDS_Initialize
THOST_FTDC_SDS_Settlementing = _thostmduserapi.THOST_FTDC_SDS_Settlementing
THOST_FTDC_SDS_Settlemented = _thostmduserapi.THOST_FTDC_SDS_Settlemented
THOST_FTDC_TSRC_NORMAL = _thostmduserapi.THOST_FTDC_TSRC_NORMAL
THOST_FTDC_TSRC_QUERY = _thostmduserapi.THOST_FTDC_TSRC_QUERY
THOST_FTDC_FSM_Product = _thostmduserapi.THOST_FTDC_FSM_Product
THOST_FTDC_FSM_Exchange = _thostmduserapi.THOST_FTDC_FSM_Exchange
THOST_FTDC_FSM_All = _thostmduserapi.THOST_FTDC_FSM_All
THOST_FTDC_BIR_Property = _thostmduserapi.THOST_FTDC_BIR_Property
THOST_FTDC_BIR_All = _thostmduserapi.THOST_FTDC_BIR_All
THOST_FTDC_PIR_All = _thostmduserapi.THOST_FTDC_PIR_All
THOST_FTDC_PIR_Property = _thostmduserapi.THOST_FTDC_PIR_Property
THOST_FTDC_PIR_Single = _thostmduserapi.THOST_FTDC_PIR_Single
THOST_FTDC_FIS_NoCreate = _thostmduserapi.THOST_FTDC_FIS_NoCreate
THOST_FTDC_FIS_Created = _thostmduserapi.THOST_FTDC_FIS_Created
THOST_FTDC_FIS_Failed = _thostmduserapi.THOST_FTDC_FIS_Failed
THOST_FTDC_FGS_FileTransmit = _thostmduserapi.THOST_FTDC_FGS_FileTransmit
THOST_FTDC_FGS_FileGen = _thostmduserapi.THOST_FTDC_FGS_FileGen
THOST_FTDC_SoM_Add = _thostmduserapi.THOST_FTDC_SoM_Add
THOST_FTDC_SoM_Update = _thostmduserapi.THOST_FTDC_SoM_Update
THOST_FTDC_SoM_Delete = _thostmduserapi.THOST_FTDC_SoM_Delete
THOST_FTDC_SoM_Copy = _thostmduserapi.THOST_FTDC_SoM_Copy
THOST_FTDC_SoM_AcTive = _thostmduserapi.THOST_FTDC_SoM_AcTive
THOST_FTDC_SoM_CanCel = _thostmduserapi.THOST_FTDC_SoM_CanCel
THOST_FTDC_SoM_ReSet = _thostmduserapi.THOST_FTDC_SoM_ReSet
THOST_FTDC_SoT_UpdatePassword = _thostmduserapi.THOST_FTDC_SoT_UpdatePassword
THOST_FTDC_SoT_UserDepartment = _thostmduserapi.THOST_FTDC_SoT_UserDepartment
THOST_FTDC_SoT_RoleManager = _thostmduserapi.THOST_FTDC_SoT_RoleManager
THOST_FTDC_SoT_RoleFunction = _thostmduserapi.THOST_FTDC_SoT_RoleFunction
THOST_FTDC_SoT_BaseParam = _thostmduserapi.THOST_FTDC_SoT_BaseParam
THOST_FTDC_SoT_SetUserID = _thostmduserapi.THOST_FTDC_SoT_SetUserID
THOST_FTDC_SoT_SetUserRole = _thostmduserapi.THOST_FTDC_SoT_SetUserRole
THOST_FTDC_SoT_UserIpRestriction = _thostmduserapi.THOST_FTDC_SoT_UserIpRestriction
THOST_FTDC_SoT_DepartmentManager = _thostmduserapi.THOST_FTDC_SoT_DepartmentManager
THOST_FTDC_SoT_DepartmentCopy = _thostmduserapi.THOST_FTDC_SoT_DepartmentCopy
THOST_FTDC_SoT_Tradingcode = _thostmduserapi.THOST_FTDC_SoT_Tradingcode
THOST_FTDC_SoT_InvestorStatus = _thostmduserapi.THOST_FTDC_SoT_InvestorStatus
THOST_FTDC_SoT_InvestorAuthority = _thostmduserapi.THOST_FTDC_SoT_InvestorAuthority
THOST_FTDC_SoT_PropertySet = _thostmduserapi.THOST_FTDC_SoT_PropertySet
THOST_FTDC_SoT_ReSetInvestorPasswd = _thostmduserapi.THOST_FTDC_SoT_ReSetInvestorPasswd
THOST_FTDC_SoT_InvestorPersonalityInfo = _thostmduserapi.THOST_FTDC_SoT_InvestorPersonalityInfo
THOST_FTDC_CSRCQ_Current = _thostmduserapi.THOST_FTDC_CSRCQ_Current
THOST_FTDC_CSRCQ_History = _thostmduserapi.THOST_FTDC_CSRCQ_History
THOST_FTDC_FRS_Normal = _thostmduserapi.THOST_FTDC_FRS_Normal
THOST_FTDC_FRS_Freeze = _thostmduserapi.THOST_FTDC_FRS_Freeze
THOST_FTDC_STST_Standard = _thostmduserapi.THOST_FTDC_STST_Standard
THOST_FTDC_STST_NonStandard = _thostmduserapi.THOST_FTDC_STST_NonStandard
THOST_FTDC_RPT_Freeze = _thostmduserapi.THOST_FTDC_RPT_Freeze
THOST_FTDC_RPT_FreezeActive = _thostmduserapi.THOST_FTDC_RPT_FreezeActive
THOST_FTDC_RPT_OpenLimit = _thostmduserapi.THOST_FTDC_RPT_OpenLimit
THOST_FTDC_RPT_RelieveOpenLimit = _thostmduserapi.THOST_FTDC_RPT_RelieveOpenLimit
THOST_FTDC_AMLDS_Normal = _thostmduserapi.THOST_FTDC_AMLDS_Normal
THOST_FTDC_AMLDS_Deleted = _thostmduserapi.THOST_FTDC_AMLDS_Deleted
THOST_FTDC_AMLCHS_Init = _thostmduserapi.THOST_FTDC_AMLCHS_Init
THOST_FTDC_AMLCHS_Checking = _thostmduserapi.THOST_FTDC_AMLCHS_Checking
THOST_FTDC_AMLCHS_Checked = _thostmduserapi.THOST_FTDC_AMLCHS_Checked
THOST_FTDC_AMLCHS_RefuseReport = _thostmduserapi.THOST_FTDC_AMLCHS_RefuseReport
THOST_FTDC_AMLDT_DrawDay = _thostmduserapi.THOST_FTDC_AMLDT_DrawDay
THOST_FTDC_AMLDT_TouchDay = _thostmduserapi.THOST_FTDC_AMLDT_TouchDay
THOST_FTDC_AMLCL_CheckLevel0 = _thostmduserapi.THOST_FTDC_AMLCL_CheckLevel0
THOST_FTDC_AMLCL_CheckLevel1 = _thostmduserapi.THOST_FTDC_AMLCL_CheckLevel1
THOST_FTDC_AMLCL_CheckLevel2 = _thostmduserapi.THOST_FTDC_AMLCL_CheckLevel2
THOST_FTDC_AMLCL_CheckLevel3 = _thostmduserapi.THOST_FTDC_AMLCL_CheckLevel3
THOST_FTDC_EFT_CSV = _thostmduserapi.THOST_FTDC_EFT_CSV
THOST_FTDC_EFT_EXCEL = _thostmduserapi.THOST_FTDC_EFT_EXCEL
THOST_FTDC_EFT_DBF = _thostmduserapi.THOST_FTDC_EFT_DBF
THOST_FTDC_SMT_Before = _thostmduserapi.THOST_FTDC_SMT_Before
THOST_FTDC_SMT_Settlement = _thostmduserapi.THOST_FTDC_SMT_Settlement
THOST_FTDC_SMT_After = _thostmduserapi.THOST_FTDC_SMT_After
THOST_FTDC_SMT_Settlemented = _thostmduserapi.THOST_FTDC_SMT_Settlemented
THOST_FTDC_SML_Must = _thostmduserapi.THOST_FTDC_SML_Must
THOST_FTDC_SML_Alarm = _thostmduserapi.THOST_FTDC_SML_Alarm
THOST_FTDC_SML_Prompt = _thostmduserapi.THOST_FTDC_SML_Prompt
THOST_FTDC_SML_Ignore = _thostmduserapi.THOST_FTDC_SML_Ignore
THOST_FTDC_SMG_Exhcange = _thostmduserapi.THOST_FTDC_SMG_Exhcange
THOST_FTDC_SMG_ASP = _thostmduserapi.THOST_FTDC_SMG_ASP
THOST_FTDC_SMG_CSRC = _thostmduserapi.THOST_FTDC_SMG_CSRC
THOST_FTDC_LUT_Repeatable = _thostmduserapi.THOST_FTDC_LUT_Repeatable
THOST_FTDC_LUT_Unrepeatable = _thostmduserapi.THOST_FTDC_LUT_Unrepeatable
THOST_FTDC_DAR_Settle = _thostmduserapi.THOST_FTDC_DAR_Settle
THOST_FTDC_DAR_Exchange = _thostmduserapi.THOST_FTDC_DAR_Exchange
THOST_FTDC_DAR_CSRC = _thostmduserapi.THOST_FTDC_DAR_CSRC
THOST_FTDC_MGT_ExchMarginRate = _thostmduserapi.THOST_FTDC_MGT_ExchMarginRate
THOST_FTDC_MGT_InstrMarginRate = _thostmduserapi.THOST_FTDC_MGT_InstrMarginRate
THOST_FTDC_MGT_InstrMarginRateTrade = _thostmduserapi.THOST_FTDC_MGT_InstrMarginRateTrade
THOST_FTDC_ACT_Intraday = _thostmduserapi.THOST_FTDC_ACT_Intraday
THOST_FTDC_ACT_Long = _thostmduserapi.THOST_FTDC_ACT_Long
THOST_FTDC_MRT_Exchange = _thostmduserapi.THOST_FTDC_MRT_Exchange
THOST_FTDC_MRT_Investor = _thostmduserapi.THOST_FTDC_MRT_Investor
THOST_FTDC_MRT_InvestorTrade = _thostmduserapi.THOST_FTDC_MRT_InvestorTrade
THOST_FTDC_BUS_UnBak = _thostmduserapi.THOST_FTDC_BUS_UnBak
THOST_FTDC_BUS_BakUp = _thostmduserapi.THOST_FTDC_BUS_BakUp
THOST_FTDC_BUS_BakUped = _thostmduserapi.THOST_FTDC_BUS_BakUped
THOST_FTDC_BUS_BakFail = _thostmduserapi.THOST_FTDC_BUS_BakFail
THOST_FTDC_SIS_UnInitialize = _thostmduserapi.THOST_FTDC_SIS_UnInitialize
THOST_FTDC_SIS_Initialize = _thostmduserapi.THOST_FTDC_SIS_Initialize
THOST_FTDC_SIS_Initialized = _thostmduserapi.THOST_FTDC_SIS_Initialized
THOST_FTDC_SRS_NoCreate = _thostmduserapi.THOST_FTDC_SRS_NoCreate
THOST_FTDC_SRS_Create = _thostmduserapi.THOST_FTDC_SRS_Create
THOST_FTDC_SRS_Created = _thostmduserapi.THOST_FTDC_SRS_Created
THOST_FTDC_SRS_CreateFail = _thostmduserapi.THOST_FTDC_SRS_CreateFail
THOST_FTDC_SSS_UnSaveData = _thostmduserapi.THOST_FTDC_SSS_UnSaveData
THOST_FTDC_SSS_SaveDatad = _thostmduserapi.THOST_FTDC_SSS_SaveDatad
THOST_FTDC_SAS_UnArchived = _thostmduserapi.THOST_FTDC_SAS_UnArchived
THOST_FTDC_SAS_Archiving = _thostmduserapi.THOST_FTDC_SAS_Archiving
THOST_FTDC_SAS_Archived = _thostmduserapi.THOST_FTDC_SAS_Archived
THOST_FTDC_SAS_ArchiveFail = _thostmduserapi.THOST_FTDC_SAS_ArchiveFail
THOST_FTDC_CTPT_Unkown = _thostmduserapi.THOST_FTDC_CTPT_Unkown
THOST_FTDC_CTPT_MainCenter = _thostmduserapi.THOST_FTDC_CTPT_MainCenter
THOST_FTDC_CTPT_BackUp = _thostmduserapi.THOST_FTDC_CTPT_BackUp
THOST_FTDC_CDT_Normal = _thostmduserapi.THOST_FTDC_CDT_Normal
THOST_FTDC_CDT_SpecFirst = _thostmduserapi.THOST_FTDC_CDT_SpecFirst
THOST_FTDC_MFUR_None = _thostmduserapi.THOST_FTDC_MFUR_None
THOST_FTDC_MFUR_Margin = _thostmduserapi.THOST_FTDC_MFUR_Margin
THOST_FTDC_MFUR_All = _thostmduserapi.THOST_FTDC_MFUR_All
THOST_FTDC_MFUR_CNY3 = _thostmduserapi.THOST_FTDC_MFUR_CNY3
THOST_FTDC_SPT_CzceHedge = _thostmduserapi.THOST_FTDC_SPT_CzceHedge
THOST_FTDC_SPT_IneForeignCurrency = _thostmduserapi.THOST_FTDC_SPT_IneForeignCurrency
THOST_FTDC_SPT_DceOpenClose = _thostmduserapi.THOST_FTDC_SPT_DceOpenClose
THOST_FTDC_FMT_Mortgage = _thostmduserapi.THOST_FTDC_FMT_Mortgage
THOST_FTDC_FMT_Redemption = _thostmduserapi.THOST_FTDC_FMT_Redemption
THOST_FTDC_ASPI_BaseMargin = _thostmduserapi.THOST_FTDC_ASPI_BaseMargin
THOST_FTDC_ASPI_LowestInterest = _thostmduserapi.THOST_FTDC_ASPI_LowestInterest
THOST_FTDC_FMD_In = _thostmduserapi.THOST_FTDC_FMD_In
THOST_FTDC_FMD_Out = _thostmduserapi.THOST_FTDC_FMD_Out
THOST_FTDC_BT_Profit = _thostmduserapi.THOST_FTDC_BT_Profit
THOST_FTDC_BT_Loss = _thostmduserapi.THOST_FTDC_BT_Loss
THOST_FTDC_BT_Other = _thostmduserapi.THOST_FTDC_BT_Other
THOST_FTDC_SST_Manual = _thostmduserapi.THOST_FTDC_SST_Manual
THOST_FTDC_SST_Automatic = _thostmduserapi.THOST_FTDC_SST_Automatic
THOST_FTDC_CED_Settlement = _thostmduserapi.THOST_FTDC_CED_Settlement
THOST_FTDC_CED_Sale = _thostmduserapi.THOST_FTDC_CED_Sale
THOST_FTDC_CSS_Entry = _thostmduserapi.THOST_FTDC_CSS_Entry
THOST_FTDC_CSS_Approve = _thostmduserapi.THOST_FTDC_CSS_Approve
THOST_FTDC_CSS_Refuse = _thostmduserapi.THOST_FTDC_CSS_Refuse
THOST_FTDC_CSS_Revoke = _thostmduserapi.THOST_FTDC_CSS_Revoke
THOST_FTDC_CSS_Send = _thostmduserapi.THOST_FTDC_CSS_Send
THOST_FTDC_CSS_Success = _thostmduserapi.THOST_FTDC_CSS_Success
THOST_FTDC_CSS_Failure = _thostmduserapi.THOST_FTDC_CSS_Failure
THOST_FTDC_REQF_NoSend = _thostmduserapi.THOST_FTDC_REQF_NoSend
THOST_FTDC_REQF_SendSuccess = _thostmduserapi.THOST_FTDC_REQF_SendSuccess
THOST_FTDC_REQF_SendFailed = _thostmduserapi.THOST_FTDC_REQF_SendFailed
THOST_FTDC_REQF_WaitReSend = _thostmduserapi.THOST_FTDC_REQF_WaitReSend
THOST_FTDC_RESF_Success = _thostmduserapi.THOST_FTDC_RESF_Success
THOST_FTDC_RESF_InsuffiCient = _thostmduserapi.THOST_FTDC_RESF_InsuffiCient
THOST_FTDC_RESF_UnKnown = _thostmduserapi.THOST_FTDC_RESF_UnKnown
THOST_FTDC_EXS_Before = _thostmduserapi.THOST_FTDC_EXS_Before
THOST_FTDC_EXS_After = _thostmduserapi.THOST_FTDC_EXS_After
THOST_FTDC_CR_Domestic = _thostmduserapi.THOST_FTDC_CR_Domestic
THOST_FTDC_CR_GMT = _thostmduserapi.THOST_FTDC_CR_GMT
THOST_FTDC_CR_Foreign = _thostmduserapi.THOST_FTDC_CR_Foreign
THOST_FTDC_HB_No = _thostmduserapi.THOST_FTDC_HB_No
THOST_FTDC_HB_Yes = _thostmduserapi.THOST_FTDC_HB_Yes
THOST_FTDC_SM_Normal = _thostmduserapi.THOST_FTDC_SM_Normal
THOST_FTDC_SM_Emerge = _thostmduserapi.THOST_FTDC_SM_Emerge
THOST_FTDC_SM_Restore = _thostmduserapi.THOST_FTDC_SM_Restore
THOST_FTDC_TPT_Full = _thostmduserapi.THOST_FTDC_TPT_Full
THOST_FTDC_TPT_Increment = _thostmduserapi.THOST_FTDC_TPT_Increment
THOST_FTDC_TPT_BackUp = _thostmduserapi.THOST_FTDC_TPT_BackUp
THOST_FTDC_LM_Trade = _thostmduserapi.THOST_FTDC_LM_Trade
THOST_FTDC_LM_Transfer = _thostmduserapi.THOST_FTDC_LM_Transfer
THOST_FTDC_CPT_Instrument = _thostmduserapi.THOST_FTDC_CPT_Instrument
THOST_FTDC_CPT_Margin = _thostmduserapi.THOST_FTDC_CPT_Margin
THOST_FTDC_HT_Yes = _thostmduserapi.THOST_FTDC_HT_Yes
THOST_FTDC_HT_No = _thostmduserapi.THOST_FTDC_HT_No
THOST_FTDC_AMT_Bank = _thostmduserapi.THOST_FTDC_AMT_Bank
THOST_FTDC_AMT_Securities = _thostmduserapi.THOST_FTDC_AMT_Securities
THOST_FTDC_AMT_Fund = _thostmduserapi.THOST_FTDC_AMT_Fund
THOST_FTDC_AMT_Insurance = _thostmduserapi.THOST_FTDC_AMT_Insurance
THOST_FTDC_AMT_Trust = _thostmduserapi.THOST_FTDC_AMT_Trust
THOST_FTDC_AMT_Other = _thostmduserapi.THOST_FTDC_AMT_Other
THOST_FTDC_CFIOT_FundIO = _thostmduserapi.THOST_FTDC_CFIOT_FundIO
THOST_FTDC_CFIOT_SwapCurrency = _thostmduserapi.THOST_FTDC_CFIOT_SwapCurrency
THOST_FTDC_CAT_Futures = _thostmduserapi.THOST_FTDC_CAT_Futures
THOST_FTDC_CAT_AssetmgrFuture = _thostmduserapi.THOST_FTDC_CAT_AssetmgrFuture
THOST_FTDC_CAT_AssetmgrTrustee = _thostmduserapi.THOST_FTDC_CAT_AssetmgrTrustee
THOST_FTDC_CAT_AssetmgrTransfer = _thostmduserapi.THOST_FTDC_CAT_AssetmgrTransfer
THOST_FTDC_LT_Chinese = _thostmduserapi.THOST_FTDC_LT_Chinese
THOST_FTDC_LT_English = _thostmduserapi.THOST_FTDC_LT_English
THOST_FTDC_AMCT_Person = _thostmduserapi.THOST_FTDC_AMCT_Person
THOST_FTDC_AMCT_Organ = _thostmduserapi.THOST_FTDC_AMCT_Organ
THOST_FTDC_AMCT_SpecialOrgan = _thostmduserapi.THOST_FTDC_AMCT_SpecialOrgan
THOST_FTDC_ASST_Futures = _thostmduserapi.THOST_FTDC_ASST_Futures
THOST_FTDC_ASST_SpecialOrgan = _thostmduserapi.THOST_FTDC_ASST_SpecialOrgan
THOST_FTDC_CIT_HasExch = _thostmduserapi.THOST_FTDC_CIT_HasExch
THOST_FTDC_CIT_HasATP = _thostmduserapi.THOST_FTDC_CIT_HasATP
THOST_FTDC_CIT_HasDiff = _thostmduserapi.THOST_FTDC_CIT_HasDiff
THOST_FTDC_DT_HandDeliv = _thostmduserapi.THOST_FTDC_DT_HandDeliv
THOST_FTDC_DT_PersonDeliv = _thostmduserapi.THOST_FTDC_DT_PersonDeliv
THOST_FTDC_MMSA_NO = _thostmduserapi.THOST_FTDC_MMSA_NO
THOST_FTDC_MMSA_YES = _thostmduserapi.THOST_FTDC_MMSA_YES
THOST_FTDC_CACT_Person = _thostmduserapi.THOST_FTDC_CACT_Person
THOST_FTDC_CACT_Company = _thostmduserapi.THOST_FTDC_CACT_Company
THOST_FTDC_CACT_Other = _thostmduserapi.THOST_FTDC_CACT_Other
THOST_FTDC_UOAAT_Futures = _thostmduserapi.THOST_FTDC_UOAAT_Futures
THOST_FTDC_UOAAT_SpecialOrgan = _thostmduserapi.THOST_FTDC_UOAAT_SpecialOrgan
THOST_FTDC_DEN_Buy = _thostmduserapi.THOST_FTDC_DEN_Buy
THOST_FTDC_DEN_Sell = _thostmduserapi.THOST_FTDC_DEN_Sell
THOST_FTDC_OFEN_Open = _thostmduserapi.THOST_FTDC_OFEN_Open
THOST_FTDC_OFEN_Close = _thostmduserapi.THOST_FTDC_OFEN_Close
THOST_FTDC_OFEN_ForceClose = _thostmduserapi.THOST_FTDC_OFEN_ForceClose
THOST_FTDC_OFEN_CloseToday = _thostmduserapi.THOST_FTDC_OFEN_CloseToday
THOST_FTDC_OFEN_CloseYesterday = _thostmduserapi.THOST_FTDC_OFEN_CloseYesterday
THOST_FTDC_OFEN_ForceOff = _thostmduserapi.THOST_FTDC_OFEN_ForceOff
THOST_FTDC_OFEN_LocalForceClose = _thostmduserapi.THOST_FTDC_OFEN_LocalForceClose
THOST_FTDC_HFEN_Speculation = _thostmduserapi.THOST_FTDC_HFEN_Speculation
THOST_FTDC_HFEN_Arbitrage = _thostmduserapi.THOST_FTDC_HFEN_Arbitrage
THOST_FTDC_HFEN_Hedge = _thostmduserapi.THOST_FTDC_HFEN_Hedge
THOST_FTDC_FIOTEN_FundIO = _thostmduserapi.THOST_FTDC_FIOTEN_FundIO
THOST_FTDC_FIOTEN_Transfer = _thostmduserapi.THOST_FTDC_FIOTEN_Transfer
THOST_FTDC_FIOTEN_SwapCurrency = _thostmduserapi.THOST_FTDC_FIOTEN_SwapCurrency
THOST_FTDC_FTEN_Deposite = _thostmduserapi.THOST_FTDC_FTEN_Deposite
THOST_FTDC_FTEN_ItemFund = _thostmduserapi.THOST_FTDC_FTEN_ItemFund
THOST_FTDC_FTEN_Company = _thostmduserapi.THOST_FTDC_FTEN_Company
THOST_FTDC_FTEN_InnerTransfer = _thostmduserapi.THOST_FTDC_FTEN_InnerTransfer
THOST_FTDC_FDEN_In = _thostmduserapi.THOST_FTDC_FDEN_In
THOST_FTDC_FDEN_Out = _thostmduserapi.THOST_FTDC_FDEN_Out
THOST_FTDC_FMDEN_In = _thostmduserapi.THOST_FTDC_FMDEN_In
THOST_FTDC_FMDEN_Out = _thostmduserapi.THOST_FTDC_FMDEN_Out
THOST_FTDC_CP_CallOptions = _thostmduserapi.THOST_FTDC_CP_CallOptions
THOST_FTDC_CP_PutOptions = _thostmduserapi.THOST_FTDC_CP_PutOptions
THOST_FTDC_STM_Continental = _thostmduserapi.THOST_FTDC_STM_Continental
THOST_FTDC_STM_American = _thostmduserapi.THOST_FTDC_STM_American
THOST_FTDC_STM_Bermuda = _thostmduserapi.THOST_FTDC_STM_Bermuda
THOST_FTDC_STT_Hedge = _thostmduserapi.THOST_FTDC_STT_Hedge
THOST_FTDC_STT_Match = _thostmduserapi.THOST_FTDC_STT_Match
THOST_FTDC_APPT_NotStrikeNum = _thostmduserapi.THOST_FTDC_APPT_NotStrikeNum
THOST_FTDC_GUDS_Gen = _thostmduserapi.THOST_FTDC_GUDS_Gen
THOST_FTDC_GUDS_Hand = _thostmduserapi.THOST_FTDC_GUDS_Hand
THOST_FTDC_OER_NoExec = _thostmduserapi.THOST_FTDC_OER_NoExec
THOST_FTDC_OER_Canceled = _thostmduserapi.THOST_FTDC_OER_Canceled
THOST_FTDC_OER_OK = _thostmduserapi.THOST_FTDC_OER_OK
THOST_FTDC_OER_NoPosition = _thostmduserapi.THOST_FTDC_OER_NoPosition
THOST_FTDC_OER_NoDeposit = _thostmduserapi.THOST_FTDC_OER_NoDeposit
THOST_FTDC_OER_NoParticipant = _thostmduserapi.THOST_FTDC_OER_NoParticipant
THOST_FTDC_OER_NoClient = _thostmduserapi.THOST_FTDC_OER_NoClient
THOST_FTDC_OER_NoInstrument = _thostmduserapi.THOST_FTDC_OER_NoInstrument
THOST_FTDC_OER_NoRight = _thostmduserapi.THOST_FTDC_OER_NoRight
THOST_FTDC_OER_InvalidVolume = _thostmduserapi.THOST_FTDC_OER_InvalidVolume
THOST_FTDC_OER_NoEnoughHistoryTrade = _thostmduserapi.THOST_FTDC_OER_NoEnoughHistoryTrade
THOST_FTDC_OER_Unknown = _thostmduserapi.THOST_FTDC_OER_Unknown
THOST_FTDC_COMBT_Future = _thostmduserapi.THOST_FTDC_COMBT_Future
THOST_FTDC_COMBT_BUL = _thostmduserapi.THOST_FTDC_COMBT_BUL
THOST_FTDC_COMBT_BER = _thostmduserapi.THOST_FTDC_COMBT_BER
THOST_FTDC_COMBT_STD = _thostmduserapi.THOST_FTDC_COMBT_STD
THOST_FTDC_COMBT_STG = _thostmduserapi.THOST_FTDC_COMBT_STG
THOST_FTDC_COMBT_PRT = _thostmduserapi.THOST_FTDC_COMBT_PRT
THOST_FTDC_COMBT_CLD = _thostmduserapi.THOST_FTDC_COMBT_CLD
THOST_FTDC_DCECOMBT_SPL = _thostmduserapi.THOST_FTDC_DCECOMBT_SPL
THOST_FTDC_DCECOMBT_OPL = _thostmduserapi.THOST_FTDC_DCECOMBT_OPL
THOST_FTDC_DCECOMBT_SP = _thostmduserapi.THOST_FTDC_DCECOMBT_SP
THOST_FTDC_DCECOMBT_SPC = _thostmduserapi.THOST_FTDC_DCECOMBT_SPC
THOST_FTDC_DCECOMBT_BLS = _thostmduserapi.THOST_FTDC_DCECOMBT_BLS
THOST_FTDC_DCECOMBT_BES = _thostmduserapi.THOST_FTDC_DCECOMBT_BES
THOST_FTDC_DCECOMBT_CAS = _thostmduserapi.THOST_FTDC_DCECOMBT_CAS
THOST_FTDC_DCECOMBT_STD = _thostmduserapi.THOST_FTDC_DCECOMBT_STD
THOST_FTDC_DCECOMBT_STG = _thostmduserapi.THOST_FTDC_DCECOMBT_STG
THOST_FTDC_DCECOMBT_BFO = _thostmduserapi.THOST_FTDC_DCECOMBT_BFO
THOST_FTDC_DCECOMBT_SFO = _thostmduserapi.THOST_FTDC_DCECOMBT_SFO
THOST_FTDC_ORPT_PreSettlementPrice = _thostmduserapi.THOST_FTDC_ORPT_PreSettlementPrice
THOST_FTDC_ORPT_OpenPrice = _thostmduserapi.THOST_FTDC_ORPT_OpenPrice
THOST_FTDC_ORPT_MaxPreSettlementPrice = _thostmduserapi.THOST_FTDC_ORPT_MaxPreSettlementPrice
THOST_FTDC_BLAG_Default = _thostmduserapi.THOST_FTDC_BLAG_Default
THOST_FTDC_BLAG_IncludeOptValLost = _thostmduserapi.THOST_FTDC_BLAG_IncludeOptValLost
THOST_FTDC_ACTP_Exec = _thostmduserapi.THOST_FTDC_ACTP_Exec
THOST_FTDC_ACTP_Abandon = _thostmduserapi.THOST_FTDC_ACTP_Abandon
THOST_FTDC_FQST_Submitted = _thostmduserapi.THOST_FTDC_FQST_Submitted
THOST_FTDC_FQST_Accepted = _thostmduserapi.THOST_FTDC_FQST_Accepted
THOST_FTDC_FQST_Rejected = _thostmduserapi.THOST_FTDC_FQST_Rejected
THOST_FTDC_VM_Absolute = _thostmduserapi.THOST_FTDC_VM_Absolute
THOST_FTDC_VM_Ratio = _thostmduserapi.THOST_FTDC_VM_Ratio
THOST_FTDC_EOPF_Reserve = _thostmduserapi.THOST_FTDC_EOPF_Reserve
THOST_FTDC_EOPF_UnReserve = _thostmduserapi.THOST_FTDC_EOPF_UnReserve
THOST_FTDC_EOCF_AutoClose = _thostmduserapi.THOST_FTDC_EOCF_AutoClose
THOST_FTDC_EOCF_NotToClose = _thostmduserapi.THOST_FTDC_EOCF_NotToClose
THOST_FTDC_PTE_Futures = _thostmduserapi.THOST_FTDC_PTE_Futures
THOST_FTDC_PTE_Options = _thostmduserapi.THOST_FTDC_PTE_Options
THOST_FTDC_CUFN_CUFN_O = _thostmduserapi.THOST_FTDC_CUFN_CUFN_O
THOST_FTDC_CUFN_CUFN_T = _thostmduserapi.THOST_FTDC_CUFN_CUFN_T
THOST_FTDC_CUFN_CUFN_P = _thostmduserapi.THOST_FTDC_CUFN_CUFN_P
THOST_FTDC_CUFN_CUFN_N = _thostmduserapi.THOST_FTDC_CUFN_CUFN_N
THOST_FTDC_CUFN_CUFN_L = _thostmduserapi.THOST_FTDC_CUFN_CUFN_L
THOST_FTDC_CUFN_CUFN_F = _thostmduserapi.THOST_FTDC_CUFN_CUFN_F
THOST_FTDC_CUFN_CUFN_C = _thostmduserapi.THOST_FTDC_CUFN_CUFN_C
THOST_FTDC_CUFN_CUFN_M = _thostmduserapi.THOST_FTDC_CUFN_CUFN_M
THOST_FTDC_DUFN_DUFN_O = _thostmduserapi.THOST_FTDC_DUFN_DUFN_O
THOST_FTDC_DUFN_DUFN_T = _thostmduserapi.THOST_FTDC_DUFN_DUFN_T
THOST_FTDC_DUFN_DUFN_P = _thostmduserapi.THOST_FTDC_DUFN_DUFN_P
THOST_FTDC_DUFN_DUFN_F = _thostmduserapi.THOST_FTDC_DUFN_DUFN_F
THOST_FTDC_DUFN_DUFN_C = _thostmduserapi.THOST_FTDC_DUFN_DUFN_C
THOST_FTDC_DUFN_DUFN_D = _thostmduserapi.THOST_FTDC_DUFN_DUFN_D
THOST_FTDC_DUFN_DUFN_M = _thostmduserapi.THOST_FTDC_DUFN_DUFN_M
THOST_FTDC_DUFN_DUFN_S = _thostmduserapi.THOST_FTDC_DUFN_DUFN_S
THOST_FTDC_SUFN_SUFN_O = _thostmduserapi.THOST_FTDC_SUFN_SUFN_O
THOST_FTDC_SUFN_SUFN_T = _thostmduserapi.THOST_FTDC_SUFN_SUFN_T
THOST_FTDC_SUFN_SUFN_P = _thostmduserapi.THOST_FTDC_SUFN_SUFN_P
THOST_FTDC_SUFN_SUFN_F = _thostmduserapi.THOST_FTDC_SUFN_SUFN_F
THOST_FTDC_CFUFN_SUFN_T = _thostmduserapi.THOST_FTDC_CFUFN_SUFN_T
THOST_FTDC_CFUFN_SUFN_P = _thostmduserapi.THOST_FTDC_CFUFN_SUFN_P
THOST_FTDC_CFUFN_SUFN_F = _thostmduserapi.THOST_FTDC_CFUFN_SUFN_F
THOST_FTDC_CFUFN_SUFN_S = _thostmduserapi.THOST_FTDC_CFUFN_SUFN_S
THOST_FTDC_CMDR_Comb = _thostmduserapi.THOST_FTDC_CMDR_Comb
THOST_FTDC_CMDR_UnComb = _thostmduserapi.THOST_FTDC_CMDR_UnComb
THOST_FTDC_STOV_RealValue = _thostmduserapi.THOST_FTDC_STOV_RealValue
THOST_FTDC_STOV_ProfitValue = _thostmduserapi.THOST_FTDC_STOV_ProfitValue
THOST_FTDC_STOV_RealRatio = _thostmduserapi.THOST_FTDC_STOV_RealRatio
THOST_FTDC_STOV_ProfitRatio = _thostmduserapi.THOST_FTDC_STOV_ProfitRatio
THOST_FTDC_ROAST_Processing = _thostmduserapi.THOST_FTDC_ROAST_Processing
THOST_FTDC_ROAST_Cancelled = _thostmduserapi.THOST_FTDC_ROAST_Cancelled
THOST_FTDC_ROAST_Opened = _thostmduserapi.THOST_FTDC_ROAST_Opened
THOST_FTDC_ROAST_Invalid = _thostmduserapi.THOST_FTDC_ROAST_Invalid
THOST_FTDC_WPSR_Lib = _thostmduserapi.THOST_FTDC_WPSR_Lib
THOST_FTDC_WPSR_Manual = _thostmduserapi.THOST_FTDC_WPSR_Manual
THOST_FTDC_OSCF_CloseSelfOptionPosition = _thostmduserapi.THOST_FTDC_OSCF_CloseSelfOptionPosition
THOST_FTDC_OSCF_ReserveOptionPosition = _thostmduserapi.THOST_FTDC_OSCF_ReserveOptionPosition
THOST_FTDC_OSCF_SellCloseSelfFuturePosition = _thostmduserapi.THOST_FTDC_OSCF_SellCloseSelfFuturePosition
THOST_FTDC_OSCF_ReserveFuturePosition = _thostmduserapi.THOST_FTDC_OSCF_ReserveFuturePosition
THOST_FTDC_BZTP_Future = _thostmduserapi.THOST_FTDC_BZTP_Future
THOST_FTDC_BZTP_Stock = _thostmduserapi.THOST_FTDC_BZTP_Stock
THOST_FTDC_APP_TYPE_Investor = _thostmduserapi.THOST_FTDC_APP_TYPE_Investor
THOST_FTDC_APP_TYPE_InvestorRelay = _thostmduserapi.THOST_FTDC_APP_TYPE_InvestorRelay
THOST_FTDC_APP_TYPE_OperatorRelay = _thostmduserapi.THOST_FTDC_APP_TYPE_OperatorRelay
THOST_FTDC_APP_TYPE_UnKnown = _thostmduserapi.THOST_FTDC_APP_TYPE_UnKnown
THOST_FTDC_RV_Right = _thostmduserapi.THOST_FTDC_RV_Right
THOST_FTDC_RV_Refuse = _thostmduserapi.THOST_FTDC_RV_Refuse
THOST_FTDC_OTC_TRDT_Block = _thostmduserapi.THOST_FTDC_OTC_TRDT_Block
THOST_FTDC_OTC_TRDT_EFP = _thostmduserapi.THOST_FTDC_OTC_TRDT_EFP
THOST_FTDC_OTC_MT_DV01 = _thostmduserapi.THOST_FTDC_OTC_MT_DV01
THOST_FTDC_OTC_MT_ParValue = _thostmduserapi.THOST_FTDC_OTC_MT_ParValue
class CThostFtdcDisseminationField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    SequenceSeries = property(_thostmduserapi.CThostFtdcDisseminationField_SequenceSeries_get, _thostmduserapi.CThostFtdcDisseminationField_SequenceSeries_set)
    SequenceNo = property(_thostmduserapi.CThostFtdcDisseminationField_SequenceNo_get, _thostmduserapi.CThostFtdcDisseminationField_SequenceNo_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcDisseminationField_swiginit(self, _thostmduserapi.new_CThostFtdcDisseminationField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcDisseminationField

# Register CThostFtdcDisseminationField in _thostmduserapi:
_thostmduserapi.CThostFtdcDisseminationField_swigregister(CThostFtdcDisseminationField)

class CThostFtdcReqUserLoginField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradingDay = property(_thostmduserapi.CThostFtdcReqUserLoginField_TradingDay_get, _thostmduserapi.CThostFtdcReqUserLoginField_TradingDay_set)
    BrokerID = property(_thostmduserapi.CThostFtdcReqUserLoginField_BrokerID_get, _thostmduserapi.CThostFtdcReqUserLoginField_BrokerID_set)
    UserID = property(_thostmduserapi.CThostFtdcReqUserLoginField_UserID_get, _thostmduserapi.CThostFtdcReqUserLoginField_UserID_set)
    Password = property(_thostmduserapi.CThostFtdcReqUserLoginField_Password_get, _thostmduserapi.CThostFtdcReqUserLoginField_Password_set)
    UserProductInfo = property(_thostmduserapi.CThostFtdcReqUserLoginField_UserProductInfo_get, _thostmduserapi.CThostFtdcReqUserLoginField_UserProductInfo_set)
    InterfaceProductInfo = property(_thostmduserapi.CThostFtdcReqUserLoginField_InterfaceProductInfo_get, _thostmduserapi.CThostFtdcReqUserLoginField_InterfaceProductInfo_set)
    ProtocolInfo = property(_thostmduserapi.CThostFtdcReqUserLoginField_ProtocolInfo_get, _thostmduserapi.CThostFtdcReqUserLoginField_ProtocolInfo_set)
    MacAddress = property(_thostmduserapi.CThostFtdcReqUserLoginField_MacAddress_get, _thostmduserapi.CThostFtdcReqUserLoginField_MacAddress_set)
    OneTimePassword = property(_thostmduserapi.CThostFtdcReqUserLoginField_OneTimePassword_get, _thostmduserapi.CThostFtdcReqUserLoginField_OneTimePassword_set)
    ClientIPAddress = property(_thostmduserapi.CThostFtdcReqUserLoginField_ClientIPAddress_get, _thostmduserapi.CThostFtdcReqUserLoginField_ClientIPAddress_set)
    LoginRemark = property(_thostmduserapi.CThostFtdcReqUserLoginField_LoginRemark_get, _thostmduserapi.CThostFtdcReqUserLoginField_LoginRemark_set)
    ClientIPPort = property(_thostmduserapi.CThostFtdcReqUserLoginField_ClientIPPort_get, _thostmduserapi.CThostFtdcReqUserLoginField_ClientIPPort_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcReqUserLoginField_swiginit(self, _thostmduserapi.new_CThostFtdcReqUserLoginField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcReqUserLoginField

# Register CThostFtdcReqUserLoginField in _thostmduserapi:
_thostmduserapi.CThostFtdcReqUserLoginField_swigregister(CThostFtdcReqUserLoginField)

class CThostFtdcRspUserLoginField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradingDay = property(_thostmduserapi.CThostFtdcRspUserLoginField_TradingDay_get, _thostmduserapi.CThostFtdcRspUserLoginField_TradingDay_set)
    LoginTime = property(_thostmduserapi.CThostFtdcRspUserLoginField_LoginTime_get, _thostmduserapi.CThostFtdcRspUserLoginField_LoginTime_set)
    BrokerID = property(_thostmduserapi.CThostFtdcRspUserLoginField_BrokerID_get, _thostmduserapi.CThostFtdcRspUserLoginField_BrokerID_set)
    UserID = property(_thostmduserapi.CThostFtdcRspUserLoginField_UserID_get, _thostmduserapi.CThostFtdcRspUserLoginField_UserID_set)
    SystemName = property(_thostmduserapi.CThostFtdcRspUserLoginField_SystemName_get, _thostmduserapi.CThostFtdcRspUserLoginField_SystemName_set)
    FrontID = property(_thostmduserapi.CThostFtdcRspUserLoginField_FrontID_get, _thostmduserapi.CThostFtdcRspUserLoginField_FrontID_set)
    SessionID = property(_thostmduserapi.CThostFtdcRspUserLoginField_SessionID_get, _thostmduserapi.CThostFtdcRspUserLoginField_SessionID_set)
    MaxOrderRef = property(_thostmduserapi.CThostFtdcRspUserLoginField_MaxOrderRef_get, _thostmduserapi.CThostFtdcRspUserLoginField_MaxOrderRef_set)
    SHFETime = property(_thostmduserapi.CThostFtdcRspUserLoginField_SHFETime_get, _thostmduserapi.CThostFtdcRspUserLoginField_SHFETime_set)
    DCETime = property(_thostmduserapi.CThostFtdcRspUserLoginField_DCETime_get, _thostmduserapi.CThostFtdcRspUserLoginField_DCETime_set)
    CZCETime = property(_thostmduserapi.CThostFtdcRspUserLoginField_CZCETime_get, _thostmduserapi.CThostFtdcRspUserLoginField_CZCETime_set)
    FFEXTime = property(_thostmduserapi.CThostFtdcRspUserLoginField_FFEXTime_get, _thostmduserapi.CThostFtdcRspUserLoginField_FFEXTime_set)
    INETime = property(_thostmduserapi.CThostFtdcRspUserLoginField_INETime_get, _thostmduserapi.CThostFtdcRspUserLoginField_INETime_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcRspUserLoginField_swiginit(self, _thostmduserapi.new_CThostFtdcRspUserLoginField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcRspUserLoginField

# Register CThostFtdcRspUserLoginField in _thostmduserapi:
_thostmduserapi.CThostFtdcRspUserLoginField_swigregister(CThostFtdcRspUserLoginField)

class CThostFtdcUserLogoutField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcUserLogoutField_BrokerID_get, _thostmduserapi.CThostFtdcUserLogoutField_BrokerID_set)
    UserID = property(_thostmduserapi.CThostFtdcUserLogoutField_UserID_get, _thostmduserapi.CThostFtdcUserLogoutField_UserID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcUserLogoutField_swiginit(self, _thostmduserapi.new_CThostFtdcUserLogoutField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcUserLogoutField

# Register CThostFtdcUserLogoutField in _thostmduserapi:
_thostmduserapi.CThostFtdcUserLogoutField_swigregister(CThostFtdcUserLogoutField)

class CThostFtdcForceUserLogoutField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcForceUserLogoutField_BrokerID_get, _thostmduserapi.CThostFtdcForceUserLogoutField_BrokerID_set)
    UserID = property(_thostmduserapi.CThostFtdcForceUserLogoutField_UserID_get, _thostmduserapi.CThostFtdcForceUserLogoutField_UserID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcForceUserLogoutField_swiginit(self, _thostmduserapi.new_CThostFtdcForceUserLogoutField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcForceUserLogoutField

# Register CThostFtdcForceUserLogoutField in _thostmduserapi:
_thostmduserapi.CThostFtdcForceUserLogoutField_swigregister(CThostFtdcForceUserLogoutField)

class CThostFtdcReqAuthenticateField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcReqAuthenticateField_BrokerID_get, _thostmduserapi.CThostFtdcReqAuthenticateField_BrokerID_set)
    UserID = property(_thostmduserapi.CThostFtdcReqAuthenticateField_UserID_get, _thostmduserapi.CThostFtdcReqAuthenticateField_UserID_set)
    UserProductInfo = property(_thostmduserapi.CThostFtdcReqAuthenticateField_UserProductInfo_get, _thostmduserapi.CThostFtdcReqAuthenticateField_UserProductInfo_set)
    AuthCode = property(_thostmduserapi.CThostFtdcReqAuthenticateField_AuthCode_get, _thostmduserapi.CThostFtdcReqAuthenticateField_AuthCode_set)
    AppID = property(_thostmduserapi.CThostFtdcReqAuthenticateField_AppID_get, _thostmduserapi.CThostFtdcReqAuthenticateField_AppID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcReqAuthenticateField_swiginit(self, _thostmduserapi.new_CThostFtdcReqAuthenticateField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcReqAuthenticateField

# Register CThostFtdcReqAuthenticateField in _thostmduserapi:
_thostmduserapi.CThostFtdcReqAuthenticateField_swigregister(CThostFtdcReqAuthenticateField)

class CThostFtdcRspAuthenticateField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcRspAuthenticateField_BrokerID_get, _thostmduserapi.CThostFtdcRspAuthenticateField_BrokerID_set)
    UserID = property(_thostmduserapi.CThostFtdcRspAuthenticateField_UserID_get, _thostmduserapi.CThostFtdcRspAuthenticateField_UserID_set)
    UserProductInfo = property(_thostmduserapi.CThostFtdcRspAuthenticateField_UserProductInfo_get, _thostmduserapi.CThostFtdcRspAuthenticateField_UserProductInfo_set)
    AppID = property(_thostmduserapi.CThostFtdcRspAuthenticateField_AppID_get, _thostmduserapi.CThostFtdcRspAuthenticateField_AppID_set)
    AppType = property(_thostmduserapi.CThostFtdcRspAuthenticateField_AppType_get, _thostmduserapi.CThostFtdcRspAuthenticateField_AppType_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcRspAuthenticateField_swiginit(self, _thostmduserapi.new_CThostFtdcRspAuthenticateField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcRspAuthenticateField

# Register CThostFtdcRspAuthenticateField in _thostmduserapi:
_thostmduserapi.CThostFtdcRspAuthenticateField_swigregister(CThostFtdcRspAuthenticateField)

class CThostFtdcAuthenticationInfoField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcAuthenticationInfoField_BrokerID_get, _thostmduserapi.CThostFtdcAuthenticationInfoField_BrokerID_set)
    UserID = property(_thostmduserapi.CThostFtdcAuthenticationInfoField_UserID_get, _thostmduserapi.CThostFtdcAuthenticationInfoField_UserID_set)
    UserProductInfo = property(_thostmduserapi.CThostFtdcAuthenticationInfoField_UserProductInfo_get, _thostmduserapi.CThostFtdcAuthenticationInfoField_UserProductInfo_set)
    AuthInfo = property(_thostmduserapi.CThostFtdcAuthenticationInfoField_AuthInfo_get, _thostmduserapi.CThostFtdcAuthenticationInfoField_AuthInfo_set)
    IsResult = property(_thostmduserapi.CThostFtdcAuthenticationInfoField_IsResult_get, _thostmduserapi.CThostFtdcAuthenticationInfoField_IsResult_set)
    AppID = property(_thostmduserapi.CThostFtdcAuthenticationInfoField_AppID_get, _thostmduserapi.CThostFtdcAuthenticationInfoField_AppID_set)
    AppType = property(_thostmduserapi.CThostFtdcAuthenticationInfoField_AppType_get, _thostmduserapi.CThostFtdcAuthenticationInfoField_AppType_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcAuthenticationInfoField_swiginit(self, _thostmduserapi.new_CThostFtdcAuthenticationInfoField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcAuthenticationInfoField

# Register CThostFtdcAuthenticationInfoField in _thostmduserapi:
_thostmduserapi.CThostFtdcAuthenticationInfoField_swigregister(CThostFtdcAuthenticationInfoField)

class CThostFtdcRspUserLogin2Field(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradingDay = property(_thostmduserapi.CThostFtdcRspUserLogin2Field_TradingDay_get, _thostmduserapi.CThostFtdcRspUserLogin2Field_TradingDay_set)
    LoginTime = property(_thostmduserapi.CThostFtdcRspUserLogin2Field_LoginTime_get, _thostmduserapi.CThostFtdcRspUserLogin2Field_LoginTime_set)
    BrokerID = property(_thostmduserapi.CThostFtdcRspUserLogin2Field_BrokerID_get, _thostmduserapi.CThostFtdcRspUserLogin2Field_BrokerID_set)
    UserID = property(_thostmduserapi.CThostFtdcRspUserLogin2Field_UserID_get, _thostmduserapi.CThostFtdcRspUserLogin2Field_UserID_set)
    SystemName = property(_thostmduserapi.CThostFtdcRspUserLogin2Field_SystemName_get, _thostmduserapi.CThostFtdcRspUserLogin2Field_SystemName_set)
    FrontID = property(_thostmduserapi.CThostFtdcRspUserLogin2Field_FrontID_get, _thostmduserapi.CThostFtdcRspUserLogin2Field_FrontID_set)
    SessionID = property(_thostmduserapi.CThostFtdcRspUserLogin2Field_SessionID_get, _thostmduserapi.CThostFtdcRspUserLogin2Field_SessionID_set)
    MaxOrderRef = property(_thostmduserapi.CThostFtdcRspUserLogin2Field_MaxOrderRef_get, _thostmduserapi.CThostFtdcRspUserLogin2Field_MaxOrderRef_set)
    SHFETime = property(_thostmduserapi.CThostFtdcRspUserLogin2Field_SHFETime_get, _thostmduserapi.CThostFtdcRspUserLogin2Field_SHFETime_set)
    DCETime = property(_thostmduserapi.CThostFtdcRspUserLogin2Field_DCETime_get, _thostmduserapi.CThostFtdcRspUserLogin2Field_DCETime_set)
    CZCETime = property(_thostmduserapi.CThostFtdcRspUserLogin2Field_CZCETime_get, _thostmduserapi.CThostFtdcRspUserLogin2Field_CZCETime_set)
    FFEXTime = property(_thostmduserapi.CThostFtdcRspUserLogin2Field_FFEXTime_get, _thostmduserapi.CThostFtdcRspUserLogin2Field_FFEXTime_set)
    INETime = property(_thostmduserapi.CThostFtdcRspUserLogin2Field_INETime_get, _thostmduserapi.CThostFtdcRspUserLogin2Field_INETime_set)
    RandomString = property(_thostmduserapi.CThostFtdcRspUserLogin2Field_RandomString_get, _thostmduserapi.CThostFtdcRspUserLogin2Field_RandomString_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcRspUserLogin2Field_swiginit(self, _thostmduserapi.new_CThostFtdcRspUserLogin2Field())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcRspUserLogin2Field

# Register CThostFtdcRspUserLogin2Field in _thostmduserapi:
_thostmduserapi.CThostFtdcRspUserLogin2Field_swigregister(CThostFtdcRspUserLogin2Field)

class CThostFtdcTransferHeaderField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    Version = property(_thostmduserapi.CThostFtdcTransferHeaderField_Version_get, _thostmduserapi.CThostFtdcTransferHeaderField_Version_set)
    TradeCode = property(_thostmduserapi.CThostFtdcTransferHeaderField_TradeCode_get, _thostmduserapi.CThostFtdcTransferHeaderField_TradeCode_set)
    TradeDate = property(_thostmduserapi.CThostFtdcTransferHeaderField_TradeDate_get, _thostmduserapi.CThostFtdcTransferHeaderField_TradeDate_set)
    TradeTime = property(_thostmduserapi.CThostFtdcTransferHeaderField_TradeTime_get, _thostmduserapi.CThostFtdcTransferHeaderField_TradeTime_set)
    TradeSerial = property(_thostmduserapi.CThostFtdcTransferHeaderField_TradeSerial_get, _thostmduserapi.CThostFtdcTransferHeaderField_TradeSerial_set)
    FutureID = property(_thostmduserapi.CThostFtdcTransferHeaderField_FutureID_get, _thostmduserapi.CThostFtdcTransferHeaderField_FutureID_set)
    BankID = property(_thostmduserapi.CThostFtdcTransferHeaderField_BankID_get, _thostmduserapi.CThostFtdcTransferHeaderField_BankID_set)
    BankBrchID = property(_thostmduserapi.CThostFtdcTransferHeaderField_BankBrchID_get, _thostmduserapi.CThostFtdcTransferHeaderField_BankBrchID_set)
    OperNo = property(_thostmduserapi.CThostFtdcTransferHeaderField_OperNo_get, _thostmduserapi.CThostFtdcTransferHeaderField_OperNo_set)
    DeviceID = property(_thostmduserapi.CThostFtdcTransferHeaderField_DeviceID_get, _thostmduserapi.CThostFtdcTransferHeaderField_DeviceID_set)
    RecordNum = property(_thostmduserapi.CThostFtdcTransferHeaderField_RecordNum_get, _thostmduserapi.CThostFtdcTransferHeaderField_RecordNum_set)
    SessionID = property(_thostmduserapi.CThostFtdcTransferHeaderField_SessionID_get, _thostmduserapi.CThostFtdcTransferHeaderField_SessionID_set)
    RequestID = property(_thostmduserapi.CThostFtdcTransferHeaderField_RequestID_get, _thostmduserapi.CThostFtdcTransferHeaderField_RequestID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcTransferHeaderField_swiginit(self, _thostmduserapi.new_CThostFtdcTransferHeaderField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcTransferHeaderField

# Register CThostFtdcTransferHeaderField in _thostmduserapi:
_thostmduserapi.CThostFtdcTransferHeaderField_swigregister(CThostFtdcTransferHeaderField)

class CThostFtdcTransferBankToFutureReqField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    FutureAccount = property(_thostmduserapi.CThostFtdcTransferBankToFutureReqField_FutureAccount_get, _thostmduserapi.CThostFtdcTransferBankToFutureReqField_FutureAccount_set)
    FuturePwdFlag = property(_thostmduserapi.CThostFtdcTransferBankToFutureReqField_FuturePwdFlag_get, _thostmduserapi.CThostFtdcTransferBankToFutureReqField_FuturePwdFlag_set)
    FutureAccPwd = property(_thostmduserapi.CThostFtdcTransferBankToFutureReqField_FutureAccPwd_get, _thostmduserapi.CThostFtdcTransferBankToFutureReqField_FutureAccPwd_set)
    TradeAmt = property(_thostmduserapi.CThostFtdcTransferBankToFutureReqField_TradeAmt_get, _thostmduserapi.CThostFtdcTransferBankToFutureReqField_TradeAmt_set)
    CustFee = property(_thostmduserapi.CThostFtdcTransferBankToFutureReqField_CustFee_get, _thostmduserapi.CThostFtdcTransferBankToFutureReqField_CustFee_set)
    CurrencyCode = property(_thostmduserapi.CThostFtdcTransferBankToFutureReqField_CurrencyCode_get, _thostmduserapi.CThostFtdcTransferBankToFutureReqField_CurrencyCode_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcTransferBankToFutureReqField_swiginit(self, _thostmduserapi.new_CThostFtdcTransferBankToFutureReqField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcTransferBankToFutureReqField

# Register CThostFtdcTransferBankToFutureReqField in _thostmduserapi:
_thostmduserapi.CThostFtdcTransferBankToFutureReqField_swigregister(CThostFtdcTransferBankToFutureReqField)

class CThostFtdcTransferBankToFutureRspField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    RetCode = property(_thostmduserapi.CThostFtdcTransferBankToFutureRspField_RetCode_get, _thostmduserapi.CThostFtdcTransferBankToFutureRspField_RetCode_set)
    RetInfo = property(_thostmduserapi.CThostFtdcTransferBankToFutureRspField_RetInfo_get, _thostmduserapi.CThostFtdcTransferBankToFutureRspField_RetInfo_set)
    FutureAccount = property(_thostmduserapi.CThostFtdcTransferBankToFutureRspField_FutureAccount_get, _thostmduserapi.CThostFtdcTransferBankToFutureRspField_FutureAccount_set)
    TradeAmt = property(_thostmduserapi.CThostFtdcTransferBankToFutureRspField_TradeAmt_get, _thostmduserapi.CThostFtdcTransferBankToFutureRspField_TradeAmt_set)
    CustFee = property(_thostmduserapi.CThostFtdcTransferBankToFutureRspField_CustFee_get, _thostmduserapi.CThostFtdcTransferBankToFutureRspField_CustFee_set)
    CurrencyCode = property(_thostmduserapi.CThostFtdcTransferBankToFutureRspField_CurrencyCode_get, _thostmduserapi.CThostFtdcTransferBankToFutureRspField_CurrencyCode_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcTransferBankToFutureRspField_swiginit(self, _thostmduserapi.new_CThostFtdcTransferBankToFutureRspField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcTransferBankToFutureRspField

# Register CThostFtdcTransferBankToFutureRspField in _thostmduserapi:
_thostmduserapi.CThostFtdcTransferBankToFutureRspField_swigregister(CThostFtdcTransferBankToFutureRspField)

class CThostFtdcTransferFutureToBankReqField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    FutureAccount = property(_thostmduserapi.CThostFtdcTransferFutureToBankReqField_FutureAccount_get, _thostmduserapi.CThostFtdcTransferFutureToBankReqField_FutureAccount_set)
    FuturePwdFlag = property(_thostmduserapi.CThostFtdcTransferFutureToBankReqField_FuturePwdFlag_get, _thostmduserapi.CThostFtdcTransferFutureToBankReqField_FuturePwdFlag_set)
    FutureAccPwd = property(_thostmduserapi.CThostFtdcTransferFutureToBankReqField_FutureAccPwd_get, _thostmduserapi.CThostFtdcTransferFutureToBankReqField_FutureAccPwd_set)
    TradeAmt = property(_thostmduserapi.CThostFtdcTransferFutureToBankReqField_TradeAmt_get, _thostmduserapi.CThostFtdcTransferFutureToBankReqField_TradeAmt_set)
    CustFee = property(_thostmduserapi.CThostFtdcTransferFutureToBankReqField_CustFee_get, _thostmduserapi.CThostFtdcTransferFutureToBankReqField_CustFee_set)
    CurrencyCode = property(_thostmduserapi.CThostFtdcTransferFutureToBankReqField_CurrencyCode_get, _thostmduserapi.CThostFtdcTransferFutureToBankReqField_CurrencyCode_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcTransferFutureToBankReqField_swiginit(self, _thostmduserapi.new_CThostFtdcTransferFutureToBankReqField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcTransferFutureToBankReqField

# Register CThostFtdcTransferFutureToBankReqField in _thostmduserapi:
_thostmduserapi.CThostFtdcTransferFutureToBankReqField_swigregister(CThostFtdcTransferFutureToBankReqField)

class CThostFtdcTransferFutureToBankRspField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    RetCode = property(_thostmduserapi.CThostFtdcTransferFutureToBankRspField_RetCode_get, _thostmduserapi.CThostFtdcTransferFutureToBankRspField_RetCode_set)
    RetInfo = property(_thostmduserapi.CThostFtdcTransferFutureToBankRspField_RetInfo_get, _thostmduserapi.CThostFtdcTransferFutureToBankRspField_RetInfo_set)
    FutureAccount = property(_thostmduserapi.CThostFtdcTransferFutureToBankRspField_FutureAccount_get, _thostmduserapi.CThostFtdcTransferFutureToBankRspField_FutureAccount_set)
    TradeAmt = property(_thostmduserapi.CThostFtdcTransferFutureToBankRspField_TradeAmt_get, _thostmduserapi.CThostFtdcTransferFutureToBankRspField_TradeAmt_set)
    CustFee = property(_thostmduserapi.CThostFtdcTransferFutureToBankRspField_CustFee_get, _thostmduserapi.CThostFtdcTransferFutureToBankRspField_CustFee_set)
    CurrencyCode = property(_thostmduserapi.CThostFtdcTransferFutureToBankRspField_CurrencyCode_get, _thostmduserapi.CThostFtdcTransferFutureToBankRspField_CurrencyCode_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcTransferFutureToBankRspField_swiginit(self, _thostmduserapi.new_CThostFtdcTransferFutureToBankRspField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcTransferFutureToBankRspField

# Register CThostFtdcTransferFutureToBankRspField in _thostmduserapi:
_thostmduserapi.CThostFtdcTransferFutureToBankRspField_swigregister(CThostFtdcTransferFutureToBankRspField)

class CThostFtdcTransferQryBankReqField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    FutureAccount = property(_thostmduserapi.CThostFtdcTransferQryBankReqField_FutureAccount_get, _thostmduserapi.CThostFtdcTransferQryBankReqField_FutureAccount_set)
    FuturePwdFlag = property(_thostmduserapi.CThostFtdcTransferQryBankReqField_FuturePwdFlag_get, _thostmduserapi.CThostFtdcTransferQryBankReqField_FuturePwdFlag_set)
    FutureAccPwd = property(_thostmduserapi.CThostFtdcTransferQryBankReqField_FutureAccPwd_get, _thostmduserapi.CThostFtdcTransferQryBankReqField_FutureAccPwd_set)
    CurrencyCode = property(_thostmduserapi.CThostFtdcTransferQryBankReqField_CurrencyCode_get, _thostmduserapi.CThostFtdcTransferQryBankReqField_CurrencyCode_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcTransferQryBankReqField_swiginit(self, _thostmduserapi.new_CThostFtdcTransferQryBankReqField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcTransferQryBankReqField

# Register CThostFtdcTransferQryBankReqField in _thostmduserapi:
_thostmduserapi.CThostFtdcTransferQryBankReqField_swigregister(CThostFtdcTransferQryBankReqField)

class CThostFtdcTransferQryBankRspField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    RetCode = property(_thostmduserapi.CThostFtdcTransferQryBankRspField_RetCode_get, _thostmduserapi.CThostFtdcTransferQryBankRspField_RetCode_set)
    RetInfo = property(_thostmduserapi.CThostFtdcTransferQryBankRspField_RetInfo_get, _thostmduserapi.CThostFtdcTransferQryBankRspField_RetInfo_set)
    FutureAccount = property(_thostmduserapi.CThostFtdcTransferQryBankRspField_FutureAccount_get, _thostmduserapi.CThostFtdcTransferQryBankRspField_FutureAccount_set)
    TradeAmt = property(_thostmduserapi.CThostFtdcTransferQryBankRspField_TradeAmt_get, _thostmduserapi.CThostFtdcTransferQryBankRspField_TradeAmt_set)
    UseAmt = property(_thostmduserapi.CThostFtdcTransferQryBankRspField_UseAmt_get, _thostmduserapi.CThostFtdcTransferQryBankRspField_UseAmt_set)
    FetchAmt = property(_thostmduserapi.CThostFtdcTransferQryBankRspField_FetchAmt_get, _thostmduserapi.CThostFtdcTransferQryBankRspField_FetchAmt_set)
    CurrencyCode = property(_thostmduserapi.CThostFtdcTransferQryBankRspField_CurrencyCode_get, _thostmduserapi.CThostFtdcTransferQryBankRspField_CurrencyCode_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcTransferQryBankRspField_swiginit(self, _thostmduserapi.new_CThostFtdcTransferQryBankRspField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcTransferQryBankRspField

# Register CThostFtdcTransferQryBankRspField in _thostmduserapi:
_thostmduserapi.CThostFtdcTransferQryBankRspField_swigregister(CThostFtdcTransferQryBankRspField)

class CThostFtdcTransferQryDetailReqField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    FutureAccount = property(_thostmduserapi.CThostFtdcTransferQryDetailReqField_FutureAccount_get, _thostmduserapi.CThostFtdcTransferQryDetailReqField_FutureAccount_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcTransferQryDetailReqField_swiginit(self, _thostmduserapi.new_CThostFtdcTransferQryDetailReqField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcTransferQryDetailReqField

# Register CThostFtdcTransferQryDetailReqField in _thostmduserapi:
_thostmduserapi.CThostFtdcTransferQryDetailReqField_swigregister(CThostFtdcTransferQryDetailReqField)

class CThostFtdcTransferQryDetailRspField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradeDate = property(_thostmduserapi.CThostFtdcTransferQryDetailRspField_TradeDate_get, _thostmduserapi.CThostFtdcTransferQryDetailRspField_TradeDate_set)
    TradeTime = property(_thostmduserapi.CThostFtdcTransferQryDetailRspField_TradeTime_get, _thostmduserapi.CThostFtdcTransferQryDetailRspField_TradeTime_set)
    TradeCode = property(_thostmduserapi.CThostFtdcTransferQryDetailRspField_TradeCode_get, _thostmduserapi.CThostFtdcTransferQryDetailRspField_TradeCode_set)
    FutureSerial = property(_thostmduserapi.CThostFtdcTransferQryDetailRspField_FutureSerial_get, _thostmduserapi.CThostFtdcTransferQryDetailRspField_FutureSerial_set)
    FutureID = property(_thostmduserapi.CThostFtdcTransferQryDetailRspField_FutureID_get, _thostmduserapi.CThostFtdcTransferQryDetailRspField_FutureID_set)
    FutureAccount = property(_thostmduserapi.CThostFtdcTransferQryDetailRspField_FutureAccount_get, _thostmduserapi.CThostFtdcTransferQryDetailRspField_FutureAccount_set)
    BankSerial = property(_thostmduserapi.CThostFtdcTransferQryDetailRspField_BankSerial_get, _thostmduserapi.CThostFtdcTransferQryDetailRspField_BankSerial_set)
    BankID = property(_thostmduserapi.CThostFtdcTransferQryDetailRspField_BankID_get, _thostmduserapi.CThostFtdcTransferQryDetailRspField_BankID_set)
    BankBrchID = property(_thostmduserapi.CThostFtdcTransferQryDetailRspField_BankBrchID_get, _thostmduserapi.CThostFtdcTransferQryDetailRspField_BankBrchID_set)
    BankAccount = property(_thostmduserapi.CThostFtdcTransferQryDetailRspField_BankAccount_get, _thostmduserapi.CThostFtdcTransferQryDetailRspField_BankAccount_set)
    CertCode = property(_thostmduserapi.CThostFtdcTransferQryDetailRspField_CertCode_get, _thostmduserapi.CThostFtdcTransferQryDetailRspField_CertCode_set)
    CurrencyCode = property(_thostmduserapi.CThostFtdcTransferQryDetailRspField_CurrencyCode_get, _thostmduserapi.CThostFtdcTransferQryDetailRspField_CurrencyCode_set)
    TxAmount = property(_thostmduserapi.CThostFtdcTransferQryDetailRspField_TxAmount_get, _thostmduserapi.CThostFtdcTransferQryDetailRspField_TxAmount_set)
    Flag = property(_thostmduserapi.CThostFtdcTransferQryDetailRspField_Flag_get, _thostmduserapi.CThostFtdcTransferQryDetailRspField_Flag_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcTransferQryDetailRspField_swiginit(self, _thostmduserapi.new_CThostFtdcTransferQryDetailRspField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcTransferQryDetailRspField

# Register CThostFtdcTransferQryDetailRspField in _thostmduserapi:
_thostmduserapi.CThostFtdcTransferQryDetailRspField_swigregister(CThostFtdcTransferQryDetailRspField)

class CThostFtdcRspInfoField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ErrorID = property(_thostmduserapi.CThostFtdcRspInfoField_ErrorID_get, _thostmduserapi.CThostFtdcRspInfoField_ErrorID_set)
    ErrorMsg = property(_thostmduserapi.CThostFtdcRspInfoField_ErrorMsg_get, _thostmduserapi.CThostFtdcRspInfoField_ErrorMsg_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcRspInfoField_swiginit(self, _thostmduserapi.new_CThostFtdcRspInfoField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcRspInfoField

# Register CThostFtdcRspInfoField in _thostmduserapi:
_thostmduserapi.CThostFtdcRspInfoField_swigregister(CThostFtdcRspInfoField)

class CThostFtdcExchangeField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ExchangeID = property(_thostmduserapi.CThostFtdcExchangeField_ExchangeID_get, _thostmduserapi.CThostFtdcExchangeField_ExchangeID_set)
    ExchangeName = property(_thostmduserapi.CThostFtdcExchangeField_ExchangeName_get, _thostmduserapi.CThostFtdcExchangeField_ExchangeName_set)
    ExchangeProperty = property(_thostmduserapi.CThostFtdcExchangeField_ExchangeProperty_get, _thostmduserapi.CThostFtdcExchangeField_ExchangeProperty_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcExchangeField_swiginit(self, _thostmduserapi.new_CThostFtdcExchangeField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcExchangeField

# Register CThostFtdcExchangeField in _thostmduserapi:
_thostmduserapi.CThostFtdcExchangeField_swigregister(CThostFtdcExchangeField)

class CThostFtdcProductField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ProductID = property(_thostmduserapi.CThostFtdcProductField_ProductID_get, _thostmduserapi.CThostFtdcProductField_ProductID_set)
    ProductName = property(_thostmduserapi.CThostFtdcProductField_ProductName_get, _thostmduserapi.CThostFtdcProductField_ProductName_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcProductField_ExchangeID_get, _thostmduserapi.CThostFtdcProductField_ExchangeID_set)
    ProductClass = property(_thostmduserapi.CThostFtdcProductField_ProductClass_get, _thostmduserapi.CThostFtdcProductField_ProductClass_set)
    VolumeMultiple = property(_thostmduserapi.CThostFtdcProductField_VolumeMultiple_get, _thostmduserapi.CThostFtdcProductField_VolumeMultiple_set)
    PriceTick = property(_thostmduserapi.CThostFtdcProductField_PriceTick_get, _thostmduserapi.CThostFtdcProductField_PriceTick_set)
    MaxMarketOrderVolume = property(_thostmduserapi.CThostFtdcProductField_MaxMarketOrderVolume_get, _thostmduserapi.CThostFtdcProductField_MaxMarketOrderVolume_set)
    MinMarketOrderVolume = property(_thostmduserapi.CThostFtdcProductField_MinMarketOrderVolume_get, _thostmduserapi.CThostFtdcProductField_MinMarketOrderVolume_set)
    MaxLimitOrderVolume = property(_thostmduserapi.CThostFtdcProductField_MaxLimitOrderVolume_get, _thostmduserapi.CThostFtdcProductField_MaxLimitOrderVolume_set)
    MinLimitOrderVolume = property(_thostmduserapi.CThostFtdcProductField_MinLimitOrderVolume_get, _thostmduserapi.CThostFtdcProductField_MinLimitOrderVolume_set)
    PositionType = property(_thostmduserapi.CThostFtdcProductField_PositionType_get, _thostmduserapi.CThostFtdcProductField_PositionType_set)
    PositionDateType = property(_thostmduserapi.CThostFtdcProductField_PositionDateType_get, _thostmduserapi.CThostFtdcProductField_PositionDateType_set)
    CloseDealType = property(_thostmduserapi.CThostFtdcProductField_CloseDealType_get, _thostmduserapi.CThostFtdcProductField_CloseDealType_set)
    TradeCurrencyID = property(_thostmduserapi.CThostFtdcProductField_TradeCurrencyID_get, _thostmduserapi.CThostFtdcProductField_TradeCurrencyID_set)
    MortgageFundUseRange = property(_thostmduserapi.CThostFtdcProductField_MortgageFundUseRange_get, _thostmduserapi.CThostFtdcProductField_MortgageFundUseRange_set)
    ExchangeProductID = property(_thostmduserapi.CThostFtdcProductField_ExchangeProductID_get, _thostmduserapi.CThostFtdcProductField_ExchangeProductID_set)
    UnderlyingMultiple = property(_thostmduserapi.CThostFtdcProductField_UnderlyingMultiple_get, _thostmduserapi.CThostFtdcProductField_UnderlyingMultiple_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcProductField_swiginit(self, _thostmduserapi.new_CThostFtdcProductField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcProductField

# Register CThostFtdcProductField in _thostmduserapi:
_thostmduserapi.CThostFtdcProductField_swigregister(CThostFtdcProductField)

class CThostFtdcInstrumentField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    InstrumentID = property(_thostmduserapi.CThostFtdcInstrumentField_InstrumentID_get, _thostmduserapi.CThostFtdcInstrumentField_InstrumentID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcInstrumentField_ExchangeID_get, _thostmduserapi.CThostFtdcInstrumentField_ExchangeID_set)
    InstrumentName = property(_thostmduserapi.CThostFtdcInstrumentField_InstrumentName_get, _thostmduserapi.CThostFtdcInstrumentField_InstrumentName_set)
    ExchangeInstID = property(_thostmduserapi.CThostFtdcInstrumentField_ExchangeInstID_get, _thostmduserapi.CThostFtdcInstrumentField_ExchangeInstID_set)
    ProductID = property(_thostmduserapi.CThostFtdcInstrumentField_ProductID_get, _thostmduserapi.CThostFtdcInstrumentField_ProductID_set)
    ProductClass = property(_thostmduserapi.CThostFtdcInstrumentField_ProductClass_get, _thostmduserapi.CThostFtdcInstrumentField_ProductClass_set)
    DeliveryYear = property(_thostmduserapi.CThostFtdcInstrumentField_DeliveryYear_get, _thostmduserapi.CThostFtdcInstrumentField_DeliveryYear_set)
    DeliveryMonth = property(_thostmduserapi.CThostFtdcInstrumentField_DeliveryMonth_get, _thostmduserapi.CThostFtdcInstrumentField_DeliveryMonth_set)
    MaxMarketOrderVolume = property(_thostmduserapi.CThostFtdcInstrumentField_MaxMarketOrderVolume_get, _thostmduserapi.CThostFtdcInstrumentField_MaxMarketOrderVolume_set)
    MinMarketOrderVolume = property(_thostmduserapi.CThostFtdcInstrumentField_MinMarketOrderVolume_get, _thostmduserapi.CThostFtdcInstrumentField_MinMarketOrderVolume_set)
    MaxLimitOrderVolume = property(_thostmduserapi.CThostFtdcInstrumentField_MaxLimitOrderVolume_get, _thostmduserapi.CThostFtdcInstrumentField_MaxLimitOrderVolume_set)
    MinLimitOrderVolume = property(_thostmduserapi.CThostFtdcInstrumentField_MinLimitOrderVolume_get, _thostmduserapi.CThostFtdcInstrumentField_MinLimitOrderVolume_set)
    VolumeMultiple = property(_thostmduserapi.CThostFtdcInstrumentField_VolumeMultiple_get, _thostmduserapi.CThostFtdcInstrumentField_VolumeMultiple_set)
    PriceTick = property(_thostmduserapi.CThostFtdcInstrumentField_PriceTick_get, _thostmduserapi.CThostFtdcInstrumentField_PriceTick_set)
    CreateDate = property(_thostmduserapi.CThostFtdcInstrumentField_CreateDate_get, _thostmduserapi.CThostFtdcInstrumentField_CreateDate_set)
    OpenDate = property(_thostmduserapi.CThostFtdcInstrumentField_OpenDate_get, _thostmduserapi.CThostFtdcInstrumentField_OpenDate_set)
    ExpireDate = property(_thostmduserapi.CThostFtdcInstrumentField_ExpireDate_get, _thostmduserapi.CThostFtdcInstrumentField_ExpireDate_set)
    StartDelivDate = property(_thostmduserapi.CThostFtdcInstrumentField_StartDelivDate_get, _thostmduserapi.CThostFtdcInstrumentField_StartDelivDate_set)
    EndDelivDate = property(_thostmduserapi.CThostFtdcInstrumentField_EndDelivDate_get, _thostmduserapi.CThostFtdcInstrumentField_EndDelivDate_set)
    InstLifePhase = property(_thostmduserapi.CThostFtdcInstrumentField_InstLifePhase_get, _thostmduserapi.CThostFtdcInstrumentField_InstLifePhase_set)
    IsTrading = property(_thostmduserapi.CThostFtdcInstrumentField_IsTrading_get, _thostmduserapi.CThostFtdcInstrumentField_IsTrading_set)
    PositionType = property(_thostmduserapi.CThostFtdcInstrumentField_PositionType_get, _thostmduserapi.CThostFtdcInstrumentField_PositionType_set)
    PositionDateType = property(_thostmduserapi.CThostFtdcInstrumentField_PositionDateType_get, _thostmduserapi.CThostFtdcInstrumentField_PositionDateType_set)
    LongMarginRatio = property(_thostmduserapi.CThostFtdcInstrumentField_LongMarginRatio_get, _thostmduserapi.CThostFtdcInstrumentField_LongMarginRatio_set)
    ShortMarginRatio = property(_thostmduserapi.CThostFtdcInstrumentField_ShortMarginRatio_get, _thostmduserapi.CThostFtdcInstrumentField_ShortMarginRatio_set)
    MaxMarginSideAlgorithm = property(_thostmduserapi.CThostFtdcInstrumentField_MaxMarginSideAlgorithm_get, _thostmduserapi.CThostFtdcInstrumentField_MaxMarginSideAlgorithm_set)
    UnderlyingInstrID = property(_thostmduserapi.CThostFtdcInstrumentField_UnderlyingInstrID_get, _thostmduserapi.CThostFtdcInstrumentField_UnderlyingInstrID_set)
    StrikePrice = property(_thostmduserapi.CThostFtdcInstrumentField_StrikePrice_get, _thostmduserapi.CThostFtdcInstrumentField_StrikePrice_set)
    OptionsType = property(_thostmduserapi.CThostFtdcInstrumentField_OptionsType_get, _thostmduserapi.CThostFtdcInstrumentField_OptionsType_set)
    UnderlyingMultiple = property(_thostmduserapi.CThostFtdcInstrumentField_UnderlyingMultiple_get, _thostmduserapi.CThostFtdcInstrumentField_UnderlyingMultiple_set)
    CombinationType = property(_thostmduserapi.CThostFtdcInstrumentField_CombinationType_get, _thostmduserapi.CThostFtdcInstrumentField_CombinationType_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcInstrumentField_swiginit(self, _thostmduserapi.new_CThostFtdcInstrumentField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcInstrumentField

# Register CThostFtdcInstrumentField in _thostmduserapi:
_thostmduserapi.CThostFtdcInstrumentField_swigregister(CThostFtdcInstrumentField)

class CThostFtdcBrokerField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcBrokerField_BrokerID_get, _thostmduserapi.CThostFtdcBrokerField_BrokerID_set)
    BrokerAbbr = property(_thostmduserapi.CThostFtdcBrokerField_BrokerAbbr_get, _thostmduserapi.CThostFtdcBrokerField_BrokerAbbr_set)
    BrokerName = property(_thostmduserapi.CThostFtdcBrokerField_BrokerName_get, _thostmduserapi.CThostFtdcBrokerField_BrokerName_set)
    IsActive = property(_thostmduserapi.CThostFtdcBrokerField_IsActive_get, _thostmduserapi.CThostFtdcBrokerField_IsActive_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcBrokerField_swiginit(self, _thostmduserapi.new_CThostFtdcBrokerField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcBrokerField

# Register CThostFtdcBrokerField in _thostmduserapi:
_thostmduserapi.CThostFtdcBrokerField_swigregister(CThostFtdcBrokerField)

class CThostFtdcTraderField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ExchangeID = property(_thostmduserapi.CThostFtdcTraderField_ExchangeID_get, _thostmduserapi.CThostFtdcTraderField_ExchangeID_set)
    TraderID = property(_thostmduserapi.CThostFtdcTraderField_TraderID_get, _thostmduserapi.CThostFtdcTraderField_TraderID_set)
    ParticipantID = property(_thostmduserapi.CThostFtdcTraderField_ParticipantID_get, _thostmduserapi.CThostFtdcTraderField_ParticipantID_set)
    Password = property(_thostmduserapi.CThostFtdcTraderField_Password_get, _thostmduserapi.CThostFtdcTraderField_Password_set)
    InstallCount = property(_thostmduserapi.CThostFtdcTraderField_InstallCount_get, _thostmduserapi.CThostFtdcTraderField_InstallCount_set)
    BrokerID = property(_thostmduserapi.CThostFtdcTraderField_BrokerID_get, _thostmduserapi.CThostFtdcTraderField_BrokerID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcTraderField_swiginit(self, _thostmduserapi.new_CThostFtdcTraderField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcTraderField

# Register CThostFtdcTraderField in _thostmduserapi:
_thostmduserapi.CThostFtdcTraderField_swigregister(CThostFtdcTraderField)

class CThostFtdcInvestorField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    InvestorID = property(_thostmduserapi.CThostFtdcInvestorField_InvestorID_get, _thostmduserapi.CThostFtdcInvestorField_InvestorID_set)
    BrokerID = property(_thostmduserapi.CThostFtdcInvestorField_BrokerID_get, _thostmduserapi.CThostFtdcInvestorField_BrokerID_set)
    InvestorGroupID = property(_thostmduserapi.CThostFtdcInvestorField_InvestorGroupID_get, _thostmduserapi.CThostFtdcInvestorField_InvestorGroupID_set)
    InvestorName = property(_thostmduserapi.CThostFtdcInvestorField_InvestorName_get, _thostmduserapi.CThostFtdcInvestorField_InvestorName_set)
    IdentifiedCardType = property(_thostmduserapi.CThostFtdcInvestorField_IdentifiedCardType_get, _thostmduserapi.CThostFtdcInvestorField_IdentifiedCardType_set)
    IdentifiedCardNo = property(_thostmduserapi.CThostFtdcInvestorField_IdentifiedCardNo_get, _thostmduserapi.CThostFtdcInvestorField_IdentifiedCardNo_set)
    IsActive = property(_thostmduserapi.CThostFtdcInvestorField_IsActive_get, _thostmduserapi.CThostFtdcInvestorField_IsActive_set)
    Telephone = property(_thostmduserapi.CThostFtdcInvestorField_Telephone_get, _thostmduserapi.CThostFtdcInvestorField_Telephone_set)
    Address = property(_thostmduserapi.CThostFtdcInvestorField_Address_get, _thostmduserapi.CThostFtdcInvestorField_Address_set)
    OpenDate = property(_thostmduserapi.CThostFtdcInvestorField_OpenDate_get, _thostmduserapi.CThostFtdcInvestorField_OpenDate_set)
    Mobile = property(_thostmduserapi.CThostFtdcInvestorField_Mobile_get, _thostmduserapi.CThostFtdcInvestorField_Mobile_set)
    CommModelID = property(_thostmduserapi.CThostFtdcInvestorField_CommModelID_get, _thostmduserapi.CThostFtdcInvestorField_CommModelID_set)
    MarginModelID = property(_thostmduserapi.CThostFtdcInvestorField_MarginModelID_get, _thostmduserapi.CThostFtdcInvestorField_MarginModelID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcInvestorField_swiginit(self, _thostmduserapi.new_CThostFtdcInvestorField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcInvestorField

# Register CThostFtdcInvestorField in _thostmduserapi:
_thostmduserapi.CThostFtdcInvestorField_swigregister(CThostFtdcInvestorField)

class CThostFtdcTradingCodeField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    InvestorID = property(_thostmduserapi.CThostFtdcTradingCodeField_InvestorID_get, _thostmduserapi.CThostFtdcTradingCodeField_InvestorID_set)
    BrokerID = property(_thostmduserapi.CThostFtdcTradingCodeField_BrokerID_get, _thostmduserapi.CThostFtdcTradingCodeField_BrokerID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcTradingCodeField_ExchangeID_get, _thostmduserapi.CThostFtdcTradingCodeField_ExchangeID_set)
    ClientID = property(_thostmduserapi.CThostFtdcTradingCodeField_ClientID_get, _thostmduserapi.CThostFtdcTradingCodeField_ClientID_set)
    IsActive = property(_thostmduserapi.CThostFtdcTradingCodeField_IsActive_get, _thostmduserapi.CThostFtdcTradingCodeField_IsActive_set)
    ClientIDType = property(_thostmduserapi.CThostFtdcTradingCodeField_ClientIDType_get, _thostmduserapi.CThostFtdcTradingCodeField_ClientIDType_set)
    BranchID = property(_thostmduserapi.CThostFtdcTradingCodeField_BranchID_get, _thostmduserapi.CThostFtdcTradingCodeField_BranchID_set)
    BizType = property(_thostmduserapi.CThostFtdcTradingCodeField_BizType_get, _thostmduserapi.CThostFtdcTradingCodeField_BizType_set)
    InvestUnitID = property(_thostmduserapi.CThostFtdcTradingCodeField_InvestUnitID_get, _thostmduserapi.CThostFtdcTradingCodeField_InvestUnitID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcTradingCodeField_swiginit(self, _thostmduserapi.new_CThostFtdcTradingCodeField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcTradingCodeField

# Register CThostFtdcTradingCodeField in _thostmduserapi:
_thostmduserapi.CThostFtdcTradingCodeField_swigregister(CThostFtdcTradingCodeField)

class CThostFtdcPartBrokerField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcPartBrokerField_BrokerID_get, _thostmduserapi.CThostFtdcPartBrokerField_BrokerID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcPartBrokerField_ExchangeID_get, _thostmduserapi.CThostFtdcPartBrokerField_ExchangeID_set)
    ParticipantID = property(_thostmduserapi.CThostFtdcPartBrokerField_ParticipantID_get, _thostmduserapi.CThostFtdcPartBrokerField_ParticipantID_set)
    IsActive = property(_thostmduserapi.CThostFtdcPartBrokerField_IsActive_get, _thostmduserapi.CThostFtdcPartBrokerField_IsActive_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcPartBrokerField_swiginit(self, _thostmduserapi.new_CThostFtdcPartBrokerField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcPartBrokerField

# Register CThostFtdcPartBrokerField in _thostmduserapi:
_thostmduserapi.CThostFtdcPartBrokerField_swigregister(CThostFtdcPartBrokerField)

class CThostFtdcSuperUserField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    UserID = property(_thostmduserapi.CThostFtdcSuperUserField_UserID_get, _thostmduserapi.CThostFtdcSuperUserField_UserID_set)
    UserName = property(_thostmduserapi.CThostFtdcSuperUserField_UserName_get, _thostmduserapi.CThostFtdcSuperUserField_UserName_set)
    Password = property(_thostmduserapi.CThostFtdcSuperUserField_Password_get, _thostmduserapi.CThostFtdcSuperUserField_Password_set)
    IsActive = property(_thostmduserapi.CThostFtdcSuperUserField_IsActive_get, _thostmduserapi.CThostFtdcSuperUserField_IsActive_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcSuperUserField_swiginit(self, _thostmduserapi.new_CThostFtdcSuperUserField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcSuperUserField

# Register CThostFtdcSuperUserField in _thostmduserapi:
_thostmduserapi.CThostFtdcSuperUserField_swigregister(CThostFtdcSuperUserField)

class CThostFtdcSuperUserFunctionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    UserID = property(_thostmduserapi.CThostFtdcSuperUserFunctionField_UserID_get, _thostmduserapi.CThostFtdcSuperUserFunctionField_UserID_set)
    FunctionCode = property(_thostmduserapi.CThostFtdcSuperUserFunctionField_FunctionCode_get, _thostmduserapi.CThostFtdcSuperUserFunctionField_FunctionCode_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcSuperUserFunctionField_swiginit(self, _thostmduserapi.new_CThostFtdcSuperUserFunctionField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcSuperUserFunctionField

# Register CThostFtdcSuperUserFunctionField in _thostmduserapi:
_thostmduserapi.CThostFtdcSuperUserFunctionField_swigregister(CThostFtdcSuperUserFunctionField)

class CThostFtdcInvestorGroupField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcInvestorGroupField_BrokerID_get, _thostmduserapi.CThostFtdcInvestorGroupField_BrokerID_set)
    InvestorGroupID = property(_thostmduserapi.CThostFtdcInvestorGroupField_InvestorGroupID_get, _thostmduserapi.CThostFtdcInvestorGroupField_InvestorGroupID_set)
    InvestorGroupName = property(_thostmduserapi.CThostFtdcInvestorGroupField_InvestorGroupName_get, _thostmduserapi.CThostFtdcInvestorGroupField_InvestorGroupName_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcInvestorGroupField_swiginit(self, _thostmduserapi.new_CThostFtdcInvestorGroupField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcInvestorGroupField

# Register CThostFtdcInvestorGroupField in _thostmduserapi:
_thostmduserapi.CThostFtdcInvestorGroupField_swigregister(CThostFtdcInvestorGroupField)

class CThostFtdcTradingAccountField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcTradingAccountField_BrokerID_get, _thostmduserapi.CThostFtdcTradingAccountField_BrokerID_set)
    AccountID = property(_thostmduserapi.CThostFtdcTradingAccountField_AccountID_get, _thostmduserapi.CThostFtdcTradingAccountField_AccountID_set)
    PreMortgage = property(_thostmduserapi.CThostFtdcTradingAccountField_PreMortgage_get, _thostmduserapi.CThostFtdcTradingAccountField_PreMortgage_set)
    PreCredit = property(_thostmduserapi.CThostFtdcTradingAccountField_PreCredit_get, _thostmduserapi.CThostFtdcTradingAccountField_PreCredit_set)
    PreDeposit = property(_thostmduserapi.CThostFtdcTradingAccountField_PreDeposit_get, _thostmduserapi.CThostFtdcTradingAccountField_PreDeposit_set)
    PreBalance = property(_thostmduserapi.CThostFtdcTradingAccountField_PreBalance_get, _thostmduserapi.CThostFtdcTradingAccountField_PreBalance_set)
    PreMargin = property(_thostmduserapi.CThostFtdcTradingAccountField_PreMargin_get, _thostmduserapi.CThostFtdcTradingAccountField_PreMargin_set)
    InterestBase = property(_thostmduserapi.CThostFtdcTradingAccountField_InterestBase_get, _thostmduserapi.CThostFtdcTradingAccountField_InterestBase_set)
    Interest = property(_thostmduserapi.CThostFtdcTradingAccountField_Interest_get, _thostmduserapi.CThostFtdcTradingAccountField_Interest_set)
    Deposit = property(_thostmduserapi.CThostFtdcTradingAccountField_Deposit_get, _thostmduserapi.CThostFtdcTradingAccountField_Deposit_set)
    Withdraw = property(_thostmduserapi.CThostFtdcTradingAccountField_Withdraw_get, _thostmduserapi.CThostFtdcTradingAccountField_Withdraw_set)
    FrozenMargin = property(_thostmduserapi.CThostFtdcTradingAccountField_FrozenMargin_get, _thostmduserapi.CThostFtdcTradingAccountField_FrozenMargin_set)
    FrozenCash = property(_thostmduserapi.CThostFtdcTradingAccountField_FrozenCash_get, _thostmduserapi.CThostFtdcTradingAccountField_FrozenCash_set)
    FrozenCommission = property(_thostmduserapi.CThostFtdcTradingAccountField_FrozenCommission_get, _thostmduserapi.CThostFtdcTradingAccountField_FrozenCommission_set)
    CurrMargin = property(_thostmduserapi.CThostFtdcTradingAccountField_CurrMargin_get, _thostmduserapi.CThostFtdcTradingAccountField_CurrMargin_set)
    CashIn = property(_thostmduserapi.CThostFtdcTradingAccountField_CashIn_get, _thostmduserapi.CThostFtdcTradingAccountField_CashIn_set)
    Commission = property(_thostmduserapi.CThostFtdcTradingAccountField_Commission_get, _thostmduserapi.CThostFtdcTradingAccountField_Commission_set)
    CloseProfit = property(_thostmduserapi.CThostFtdcTradingAccountField_CloseProfit_get, _thostmduserapi.CThostFtdcTradingAccountField_CloseProfit_set)
    PositionProfit = property(_thostmduserapi.CThostFtdcTradingAccountField_PositionProfit_get, _thostmduserapi.CThostFtdcTradingAccountField_PositionProfit_set)
    Balance = property(_thostmduserapi.CThostFtdcTradingAccountField_Balance_get, _thostmduserapi.CThostFtdcTradingAccountField_Balance_set)
    Available = property(_thostmduserapi.CThostFtdcTradingAccountField_Available_get, _thostmduserapi.CThostFtdcTradingAccountField_Available_set)
    WithdrawQuota = property(_thostmduserapi.CThostFtdcTradingAccountField_WithdrawQuota_get, _thostmduserapi.CThostFtdcTradingAccountField_WithdrawQuota_set)
    Reserve = property(_thostmduserapi.CThostFtdcTradingAccountField_Reserve_get, _thostmduserapi.CThostFtdcTradingAccountField_Reserve_set)
    TradingDay = property(_thostmduserapi.CThostFtdcTradingAccountField_TradingDay_get, _thostmduserapi.CThostFtdcTradingAccountField_TradingDay_set)
    SettlementID = property(_thostmduserapi.CThostFtdcTradingAccountField_SettlementID_get, _thostmduserapi.CThostFtdcTradingAccountField_SettlementID_set)
    Credit = property(_thostmduserapi.CThostFtdcTradingAccountField_Credit_get, _thostmduserapi.CThostFtdcTradingAccountField_Credit_set)
    Mortgage = property(_thostmduserapi.CThostFtdcTradingAccountField_Mortgage_get, _thostmduserapi.CThostFtdcTradingAccountField_Mortgage_set)
    ExchangeMargin = property(_thostmduserapi.CThostFtdcTradingAccountField_ExchangeMargin_get, _thostmduserapi.CThostFtdcTradingAccountField_ExchangeMargin_set)
    DeliveryMargin = property(_thostmduserapi.CThostFtdcTradingAccountField_DeliveryMargin_get, _thostmduserapi.CThostFtdcTradingAccountField_DeliveryMargin_set)
    ExchangeDeliveryMargin = property(_thostmduserapi.CThostFtdcTradingAccountField_ExchangeDeliveryMargin_get, _thostmduserapi.CThostFtdcTradingAccountField_ExchangeDeliveryMargin_set)
    ReserveBalance = property(_thostmduserapi.CThostFtdcTradingAccountField_ReserveBalance_get, _thostmduserapi.CThostFtdcTradingAccountField_ReserveBalance_set)
    CurrencyID = property(_thostmduserapi.CThostFtdcTradingAccountField_CurrencyID_get, _thostmduserapi.CThostFtdcTradingAccountField_CurrencyID_set)
    PreFundMortgageIn = property(_thostmduserapi.CThostFtdcTradingAccountField_PreFundMortgageIn_get, _thostmduserapi.CThostFtdcTradingAccountField_PreFundMortgageIn_set)
    PreFundMortgageOut = property(_thostmduserapi.CThostFtdcTradingAccountField_PreFundMortgageOut_get, _thostmduserapi.CThostFtdcTradingAccountField_PreFundMortgageOut_set)
    FundMortgageIn = property(_thostmduserapi.CThostFtdcTradingAccountField_FundMortgageIn_get, _thostmduserapi.CThostFtdcTradingAccountField_FundMortgageIn_set)
    FundMortgageOut = property(_thostmduserapi.CThostFtdcTradingAccountField_FundMortgageOut_get, _thostmduserapi.CThostFtdcTradingAccountField_FundMortgageOut_set)
    FundMortgageAvailable = property(_thostmduserapi.CThostFtdcTradingAccountField_FundMortgageAvailable_get, _thostmduserapi.CThostFtdcTradingAccountField_FundMortgageAvailable_set)
    MortgageableFund = property(_thostmduserapi.CThostFtdcTradingAccountField_MortgageableFund_get, _thostmduserapi.CThostFtdcTradingAccountField_MortgageableFund_set)
    SpecProductMargin = property(_thostmduserapi.CThostFtdcTradingAccountField_SpecProductMargin_get, _thostmduserapi.CThostFtdcTradingAccountField_SpecProductMargin_set)
    SpecProductFrozenMargin = property(_thostmduserapi.CThostFtdcTradingAccountField_SpecProductFrozenMargin_get, _thostmduserapi.CThostFtdcTradingAccountField_SpecProductFrozenMargin_set)
    SpecProductCommission = property(_thostmduserapi.CThostFtdcTradingAccountField_SpecProductCommission_get, _thostmduserapi.CThostFtdcTradingAccountField_SpecProductCommission_set)
    SpecProductFrozenCommission = property(_thostmduserapi.CThostFtdcTradingAccountField_SpecProductFrozenCommission_get, _thostmduserapi.CThostFtdcTradingAccountField_SpecProductFrozenCommission_set)
    SpecProductPositionProfit = property(_thostmduserapi.CThostFtdcTradingAccountField_SpecProductPositionProfit_get, _thostmduserapi.CThostFtdcTradingAccountField_SpecProductPositionProfit_set)
    SpecProductCloseProfit = property(_thostmduserapi.CThostFtdcTradingAccountField_SpecProductCloseProfit_get, _thostmduserapi.CThostFtdcTradingAccountField_SpecProductCloseProfit_set)
    SpecProductPositionProfitByAlg = property(_thostmduserapi.CThostFtdcTradingAccountField_SpecProductPositionProfitByAlg_get, _thostmduserapi.CThostFtdcTradingAccountField_SpecProductPositionProfitByAlg_set)
    SpecProductExchangeMargin = property(_thostmduserapi.CThostFtdcTradingAccountField_SpecProductExchangeMargin_get, _thostmduserapi.CThostFtdcTradingAccountField_SpecProductExchangeMargin_set)
    BizType = property(_thostmduserapi.CThostFtdcTradingAccountField_BizType_get, _thostmduserapi.CThostFtdcTradingAccountField_BizType_set)
    FrozenSwap = property(_thostmduserapi.CThostFtdcTradingAccountField_FrozenSwap_get, _thostmduserapi.CThostFtdcTradingAccountField_FrozenSwap_set)
    RemainSwap = property(_thostmduserapi.CThostFtdcTradingAccountField_RemainSwap_get, _thostmduserapi.CThostFtdcTradingAccountField_RemainSwap_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcTradingAccountField_swiginit(self, _thostmduserapi.new_CThostFtdcTradingAccountField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcTradingAccountField

# Register CThostFtdcTradingAccountField in _thostmduserapi:
_thostmduserapi.CThostFtdcTradingAccountField_swigregister(CThostFtdcTradingAccountField)

class CThostFtdcInvestorPositionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    InstrumentID = property(_thostmduserapi.CThostFtdcInvestorPositionField_InstrumentID_get, _thostmduserapi.CThostFtdcInvestorPositionField_InstrumentID_set)
    BrokerID = property(_thostmduserapi.CThostFtdcInvestorPositionField_BrokerID_get, _thostmduserapi.CThostFtdcInvestorPositionField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcInvestorPositionField_InvestorID_get, _thostmduserapi.CThostFtdcInvestorPositionField_InvestorID_set)
    PosiDirection = property(_thostmduserapi.CThostFtdcInvestorPositionField_PosiDirection_get, _thostmduserapi.CThostFtdcInvestorPositionField_PosiDirection_set)
    HedgeFlag = property(_thostmduserapi.CThostFtdcInvestorPositionField_HedgeFlag_get, _thostmduserapi.CThostFtdcInvestorPositionField_HedgeFlag_set)
    PositionDate = property(_thostmduserapi.CThostFtdcInvestorPositionField_PositionDate_get, _thostmduserapi.CThostFtdcInvestorPositionField_PositionDate_set)
    YdPosition = property(_thostmduserapi.CThostFtdcInvestorPositionField_YdPosition_get, _thostmduserapi.CThostFtdcInvestorPositionField_YdPosition_set)
    Position = property(_thostmduserapi.CThostFtdcInvestorPositionField_Position_get, _thostmduserapi.CThostFtdcInvestorPositionField_Position_set)
    LongFrozen = property(_thostmduserapi.CThostFtdcInvestorPositionField_LongFrozen_get, _thostmduserapi.CThostFtdcInvestorPositionField_LongFrozen_set)
    ShortFrozen = property(_thostmduserapi.CThostFtdcInvestorPositionField_ShortFrozen_get, _thostmduserapi.CThostFtdcInvestorPositionField_ShortFrozen_set)
    LongFrozenAmount = property(_thostmduserapi.CThostFtdcInvestorPositionField_LongFrozenAmount_get, _thostmduserapi.CThostFtdcInvestorPositionField_LongFrozenAmount_set)
    ShortFrozenAmount = property(_thostmduserapi.CThostFtdcInvestorPositionField_ShortFrozenAmount_get, _thostmduserapi.CThostFtdcInvestorPositionField_ShortFrozenAmount_set)
    OpenVolume = property(_thostmduserapi.CThostFtdcInvestorPositionField_OpenVolume_get, _thostmduserapi.CThostFtdcInvestorPositionField_OpenVolume_set)
    CloseVolume = property(_thostmduserapi.CThostFtdcInvestorPositionField_CloseVolume_get, _thostmduserapi.CThostFtdcInvestorPositionField_CloseVolume_set)
    OpenAmount = property(_thostmduserapi.CThostFtdcInvestorPositionField_OpenAmount_get, _thostmduserapi.CThostFtdcInvestorPositionField_OpenAmount_set)
    CloseAmount = property(_thostmduserapi.CThostFtdcInvestorPositionField_CloseAmount_get, _thostmduserapi.CThostFtdcInvestorPositionField_CloseAmount_set)
    PositionCost = property(_thostmduserapi.CThostFtdcInvestorPositionField_PositionCost_get, _thostmduserapi.CThostFtdcInvestorPositionField_PositionCost_set)
    PreMargin = property(_thostmduserapi.CThostFtdcInvestorPositionField_PreMargin_get, _thostmduserapi.CThostFtdcInvestorPositionField_PreMargin_set)
    UseMargin = property(_thostmduserapi.CThostFtdcInvestorPositionField_UseMargin_get, _thostmduserapi.CThostFtdcInvestorPositionField_UseMargin_set)
    FrozenMargin = property(_thostmduserapi.CThostFtdcInvestorPositionField_FrozenMargin_get, _thostmduserapi.CThostFtdcInvestorPositionField_FrozenMargin_set)
    FrozenCash = property(_thostmduserapi.CThostFtdcInvestorPositionField_FrozenCash_get, _thostmduserapi.CThostFtdcInvestorPositionField_FrozenCash_set)
    FrozenCommission = property(_thostmduserapi.CThostFtdcInvestorPositionField_FrozenCommission_get, _thostmduserapi.CThostFtdcInvestorPositionField_FrozenCommission_set)
    CashIn = property(_thostmduserapi.CThostFtdcInvestorPositionField_CashIn_get, _thostmduserapi.CThostFtdcInvestorPositionField_CashIn_set)
    Commission = property(_thostmduserapi.CThostFtdcInvestorPositionField_Commission_get, _thostmduserapi.CThostFtdcInvestorPositionField_Commission_set)
    CloseProfit = property(_thostmduserapi.CThostFtdcInvestorPositionField_CloseProfit_get, _thostmduserapi.CThostFtdcInvestorPositionField_CloseProfit_set)
    PositionProfit = property(_thostmduserapi.CThostFtdcInvestorPositionField_PositionProfit_get, _thostmduserapi.CThostFtdcInvestorPositionField_PositionProfit_set)
    PreSettlementPrice = property(_thostmduserapi.CThostFtdcInvestorPositionField_PreSettlementPrice_get, _thostmduserapi.CThostFtdcInvestorPositionField_PreSettlementPrice_set)
    SettlementPrice = property(_thostmduserapi.CThostFtdcInvestorPositionField_SettlementPrice_get, _thostmduserapi.CThostFtdcInvestorPositionField_SettlementPrice_set)
    TradingDay = property(_thostmduserapi.CThostFtdcInvestorPositionField_TradingDay_get, _thostmduserapi.CThostFtdcInvestorPositionField_TradingDay_set)
    SettlementID = property(_thostmduserapi.CThostFtdcInvestorPositionField_SettlementID_get, _thostmduserapi.CThostFtdcInvestorPositionField_SettlementID_set)
    OpenCost = property(_thostmduserapi.CThostFtdcInvestorPositionField_OpenCost_get, _thostmduserapi.CThostFtdcInvestorPositionField_OpenCost_set)
    ExchangeMargin = property(_thostmduserapi.CThostFtdcInvestorPositionField_ExchangeMargin_get, _thostmduserapi.CThostFtdcInvestorPositionField_ExchangeMargin_set)
    CombPosition = property(_thostmduserapi.CThostFtdcInvestorPositionField_CombPosition_get, _thostmduserapi.CThostFtdcInvestorPositionField_CombPosition_set)
    CombLongFrozen = property(_thostmduserapi.CThostFtdcInvestorPositionField_CombLongFrozen_get, _thostmduserapi.CThostFtdcInvestorPositionField_CombLongFrozen_set)
    CombShortFrozen = property(_thostmduserapi.CThostFtdcInvestorPositionField_CombShortFrozen_get, _thostmduserapi.CThostFtdcInvestorPositionField_CombShortFrozen_set)
    CloseProfitByDate = property(_thostmduserapi.CThostFtdcInvestorPositionField_CloseProfitByDate_get, _thostmduserapi.CThostFtdcInvestorPositionField_CloseProfitByDate_set)
    CloseProfitByTrade = property(_thostmduserapi.CThostFtdcInvestorPositionField_CloseProfitByTrade_get, _thostmduserapi.CThostFtdcInvestorPositionField_CloseProfitByTrade_set)
    TodayPosition = property(_thostmduserapi.CThostFtdcInvestorPositionField_TodayPosition_get, _thostmduserapi.CThostFtdcInvestorPositionField_TodayPosition_set)
    MarginRateByMoney = property(_thostmduserapi.CThostFtdcInvestorPositionField_MarginRateByMoney_get, _thostmduserapi.CThostFtdcInvestorPositionField_MarginRateByMoney_set)
    MarginRateByVolume = property(_thostmduserapi.CThostFtdcInvestorPositionField_MarginRateByVolume_get, _thostmduserapi.CThostFtdcInvestorPositionField_MarginRateByVolume_set)
    StrikeFrozen = property(_thostmduserapi.CThostFtdcInvestorPositionField_StrikeFrozen_get, _thostmduserapi.CThostFtdcInvestorPositionField_StrikeFrozen_set)
    StrikeFrozenAmount = property(_thostmduserapi.CThostFtdcInvestorPositionField_StrikeFrozenAmount_get, _thostmduserapi.CThostFtdcInvestorPositionField_StrikeFrozenAmount_set)
    AbandonFrozen = property(_thostmduserapi.CThostFtdcInvestorPositionField_AbandonFrozen_get, _thostmduserapi.CThostFtdcInvestorPositionField_AbandonFrozen_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcInvestorPositionField_ExchangeID_get, _thostmduserapi.CThostFtdcInvestorPositionField_ExchangeID_set)
    YdStrikeFrozen = property(_thostmduserapi.CThostFtdcInvestorPositionField_YdStrikeFrozen_get, _thostmduserapi.CThostFtdcInvestorPositionField_YdStrikeFrozen_set)
    InvestUnitID = property(_thostmduserapi.CThostFtdcInvestorPositionField_InvestUnitID_get, _thostmduserapi.CThostFtdcInvestorPositionField_InvestUnitID_set)
    PositionCostOffset = property(_thostmduserapi.CThostFtdcInvestorPositionField_PositionCostOffset_get, _thostmduserapi.CThostFtdcInvestorPositionField_PositionCostOffset_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcInvestorPositionField_swiginit(self, _thostmduserapi.new_CThostFtdcInvestorPositionField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcInvestorPositionField

# Register CThostFtdcInvestorPositionField in _thostmduserapi:
_thostmduserapi.CThostFtdcInvestorPositionField_swigregister(CThostFtdcInvestorPositionField)

class CThostFtdcInstrumentMarginRateField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    InstrumentID = property(_thostmduserapi.CThostFtdcInstrumentMarginRateField_InstrumentID_get, _thostmduserapi.CThostFtdcInstrumentMarginRateField_InstrumentID_set)
    InvestorRange = property(_thostmduserapi.CThostFtdcInstrumentMarginRateField_InvestorRange_get, _thostmduserapi.CThostFtdcInstrumentMarginRateField_InvestorRange_set)
    BrokerID = property(_thostmduserapi.CThostFtdcInstrumentMarginRateField_BrokerID_get, _thostmduserapi.CThostFtdcInstrumentMarginRateField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcInstrumentMarginRateField_InvestorID_get, _thostmduserapi.CThostFtdcInstrumentMarginRateField_InvestorID_set)
    HedgeFlag = property(_thostmduserapi.CThostFtdcInstrumentMarginRateField_HedgeFlag_get, _thostmduserapi.CThostFtdcInstrumentMarginRateField_HedgeFlag_set)
    LongMarginRatioByMoney = property(_thostmduserapi.CThostFtdcInstrumentMarginRateField_LongMarginRatioByMoney_get, _thostmduserapi.CThostFtdcInstrumentMarginRateField_LongMarginRatioByMoney_set)
    LongMarginRatioByVolume = property(_thostmduserapi.CThostFtdcInstrumentMarginRateField_LongMarginRatioByVolume_get, _thostmduserapi.CThostFtdcInstrumentMarginRateField_LongMarginRatioByVolume_set)
    ShortMarginRatioByMoney = property(_thostmduserapi.CThostFtdcInstrumentMarginRateField_ShortMarginRatioByMoney_get, _thostmduserapi.CThostFtdcInstrumentMarginRateField_ShortMarginRatioByMoney_set)
    ShortMarginRatioByVolume = property(_thostmduserapi.CThostFtdcInstrumentMarginRateField_ShortMarginRatioByVolume_get, _thostmduserapi.CThostFtdcInstrumentMarginRateField_ShortMarginRatioByVolume_set)
    IsRelative = property(_thostmduserapi.CThostFtdcInstrumentMarginRateField_IsRelative_get, _thostmduserapi.CThostFtdcInstrumentMarginRateField_IsRelative_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcInstrumentMarginRateField_ExchangeID_get, _thostmduserapi.CThostFtdcInstrumentMarginRateField_ExchangeID_set)
    InvestUnitID = property(_thostmduserapi.CThostFtdcInstrumentMarginRateField_InvestUnitID_get, _thostmduserapi.CThostFtdcInstrumentMarginRateField_InvestUnitID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcInstrumentMarginRateField_swiginit(self, _thostmduserapi.new_CThostFtdcInstrumentMarginRateField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcInstrumentMarginRateField

# Register CThostFtdcInstrumentMarginRateField in _thostmduserapi:
_thostmduserapi.CThostFtdcInstrumentMarginRateField_swigregister(CThostFtdcInstrumentMarginRateField)

class CThostFtdcInstrumentCommissionRateField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    InstrumentID = property(_thostmduserapi.CThostFtdcInstrumentCommissionRateField_InstrumentID_get, _thostmduserapi.CThostFtdcInstrumentCommissionRateField_InstrumentID_set)
    InvestorRange = property(_thostmduserapi.CThostFtdcInstrumentCommissionRateField_InvestorRange_get, _thostmduserapi.CThostFtdcInstrumentCommissionRateField_InvestorRange_set)
    BrokerID = property(_thostmduserapi.CThostFtdcInstrumentCommissionRateField_BrokerID_get, _thostmduserapi.CThostFtdcInstrumentCommissionRateField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcInstrumentCommissionRateField_InvestorID_get, _thostmduserapi.CThostFtdcInstrumentCommissionRateField_InvestorID_set)
    OpenRatioByMoney = property(_thostmduserapi.CThostFtdcInstrumentCommissionRateField_OpenRatioByMoney_get, _thostmduserapi.CThostFtdcInstrumentCommissionRateField_OpenRatioByMoney_set)
    OpenRatioByVolume = property(_thostmduserapi.CThostFtdcInstrumentCommissionRateField_OpenRatioByVolume_get, _thostmduserapi.CThostFtdcInstrumentCommissionRateField_OpenRatioByVolume_set)
    CloseRatioByMoney = property(_thostmduserapi.CThostFtdcInstrumentCommissionRateField_CloseRatioByMoney_get, _thostmduserapi.CThostFtdcInstrumentCommissionRateField_CloseRatioByMoney_set)
    CloseRatioByVolume = property(_thostmduserapi.CThostFtdcInstrumentCommissionRateField_CloseRatioByVolume_get, _thostmduserapi.CThostFtdcInstrumentCommissionRateField_CloseRatioByVolume_set)
    CloseTodayRatioByMoney = property(_thostmduserapi.CThostFtdcInstrumentCommissionRateField_CloseTodayRatioByMoney_get, _thostmduserapi.CThostFtdcInstrumentCommissionRateField_CloseTodayRatioByMoney_set)
    CloseTodayRatioByVolume = property(_thostmduserapi.CThostFtdcInstrumentCommissionRateField_CloseTodayRatioByVolume_get, _thostmduserapi.CThostFtdcInstrumentCommissionRateField_CloseTodayRatioByVolume_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcInstrumentCommissionRateField_ExchangeID_get, _thostmduserapi.CThostFtdcInstrumentCommissionRateField_ExchangeID_set)
    BizType = property(_thostmduserapi.CThostFtdcInstrumentCommissionRateField_BizType_get, _thostmduserapi.CThostFtdcInstrumentCommissionRateField_BizType_set)
    InvestUnitID = property(_thostmduserapi.CThostFtdcInstrumentCommissionRateField_InvestUnitID_get, _thostmduserapi.CThostFtdcInstrumentCommissionRateField_InvestUnitID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcInstrumentCommissionRateField_swiginit(self, _thostmduserapi.new_CThostFtdcInstrumentCommissionRateField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcInstrumentCommissionRateField

# Register CThostFtdcInstrumentCommissionRateField in _thostmduserapi:
_thostmduserapi.CThostFtdcInstrumentCommissionRateField_swigregister(CThostFtdcInstrumentCommissionRateField)

class CThostFtdcDepthMarketDataField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradingDay = property(_thostmduserapi.CThostFtdcDepthMarketDataField_TradingDay_get, _thostmduserapi.CThostFtdcDepthMarketDataField_TradingDay_set)
    InstrumentID = property(_thostmduserapi.CThostFtdcDepthMarketDataField_InstrumentID_get, _thostmduserapi.CThostFtdcDepthMarketDataField_InstrumentID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcDepthMarketDataField_ExchangeID_get, _thostmduserapi.CThostFtdcDepthMarketDataField_ExchangeID_set)
    ExchangeInstID = property(_thostmduserapi.CThostFtdcDepthMarketDataField_ExchangeInstID_get, _thostmduserapi.CThostFtdcDepthMarketDataField_ExchangeInstID_set)
    LastPrice = property(_thostmduserapi.CThostFtdcDepthMarketDataField_LastPrice_get, _thostmduserapi.CThostFtdcDepthMarketDataField_LastPrice_set)
    PreSettlementPrice = property(_thostmduserapi.CThostFtdcDepthMarketDataField_PreSettlementPrice_get, _thostmduserapi.CThostFtdcDepthMarketDataField_PreSettlementPrice_set)
    PreClosePrice = property(_thostmduserapi.CThostFtdcDepthMarketDataField_PreClosePrice_get, _thostmduserapi.CThostFtdcDepthMarketDataField_PreClosePrice_set)
    PreOpenInterest = property(_thostmduserapi.CThostFtdcDepthMarketDataField_PreOpenInterest_get, _thostmduserapi.CThostFtdcDepthMarketDataField_PreOpenInterest_set)
    OpenPrice = property(_thostmduserapi.CThostFtdcDepthMarketDataField_OpenPrice_get, _thostmduserapi.CThostFtdcDepthMarketDataField_OpenPrice_set)
    HighestPrice = property(_thostmduserapi.CThostFtdcDepthMarketDataField_HighestPrice_get, _thostmduserapi.CThostFtdcDepthMarketDataField_HighestPrice_set)
    LowestPrice = property(_thostmduserapi.CThostFtdcDepthMarketDataField_LowestPrice_get, _thostmduserapi.CThostFtdcDepthMarketDataField_LowestPrice_set)
    Volume = property(_thostmduserapi.CThostFtdcDepthMarketDataField_Volume_get, _thostmduserapi.CThostFtdcDepthMarketDataField_Volume_set)
    Turnover = property(_thostmduserapi.CThostFtdcDepthMarketDataField_Turnover_get, _thostmduserapi.CThostFtdcDepthMarketDataField_Turnover_set)
    OpenInterest = property(_thostmduserapi.CThostFtdcDepthMarketDataField_OpenInterest_get, _thostmduserapi.CThostFtdcDepthMarketDataField_OpenInterest_set)
    ClosePrice = property(_thostmduserapi.CThostFtdcDepthMarketDataField_ClosePrice_get, _thostmduserapi.CThostFtdcDepthMarketDataField_ClosePrice_set)
    SettlementPrice = property(_thostmduserapi.CThostFtdcDepthMarketDataField_SettlementPrice_get, _thostmduserapi.CThostFtdcDepthMarketDataField_SettlementPrice_set)
    UpperLimitPrice = property(_thostmduserapi.CThostFtdcDepthMarketDataField_UpperLimitPrice_get, _thostmduserapi.CThostFtdcDepthMarketDataField_UpperLimitPrice_set)
    LowerLimitPrice = property(_thostmduserapi.CThostFtdcDepthMarketDataField_LowerLimitPrice_get, _thostmduserapi.CThostFtdcDepthMarketDataField_LowerLimitPrice_set)
    PreDelta = property(_thostmduserapi.CThostFtdcDepthMarketDataField_PreDelta_get, _thostmduserapi.CThostFtdcDepthMarketDataField_PreDelta_set)
    CurrDelta = property(_thostmduserapi.CThostFtdcDepthMarketDataField_CurrDelta_get, _thostmduserapi.CThostFtdcDepthMarketDataField_CurrDelta_set)
    UpdateTime = property(_thostmduserapi.CThostFtdcDepthMarketDataField_UpdateTime_get, _thostmduserapi.CThostFtdcDepthMarketDataField_UpdateTime_set)
    UpdateMillisec = property(_thostmduserapi.CThostFtdcDepthMarketDataField_UpdateMillisec_get, _thostmduserapi.CThostFtdcDepthMarketDataField_UpdateMillisec_set)
    BidPrice1 = property(_thostmduserapi.CThostFtdcDepthMarketDataField_BidPrice1_get, _thostmduserapi.CThostFtdcDepthMarketDataField_BidPrice1_set)
    BidVolume1 = property(_thostmduserapi.CThostFtdcDepthMarketDataField_BidVolume1_get, _thostmduserapi.CThostFtdcDepthMarketDataField_BidVolume1_set)
    AskPrice1 = property(_thostmduserapi.CThostFtdcDepthMarketDataField_AskPrice1_get, _thostmduserapi.CThostFtdcDepthMarketDataField_AskPrice1_set)
    AskVolume1 = property(_thostmduserapi.CThostFtdcDepthMarketDataField_AskVolume1_get, _thostmduserapi.CThostFtdcDepthMarketDataField_AskVolume1_set)
    BidPrice2 = property(_thostmduserapi.CThostFtdcDepthMarketDataField_BidPrice2_get, _thostmduserapi.CThostFtdcDepthMarketDataField_BidPrice2_set)
    BidVolume2 = property(_thostmduserapi.CThostFtdcDepthMarketDataField_BidVolume2_get, _thostmduserapi.CThostFtdcDepthMarketDataField_BidVolume2_set)
    AskPrice2 = property(_thostmduserapi.CThostFtdcDepthMarketDataField_AskPrice2_get, _thostmduserapi.CThostFtdcDepthMarketDataField_AskPrice2_set)
    AskVolume2 = property(_thostmduserapi.CThostFtdcDepthMarketDataField_AskVolume2_get, _thostmduserapi.CThostFtdcDepthMarketDataField_AskVolume2_set)
    BidPrice3 = property(_thostmduserapi.CThostFtdcDepthMarketDataField_BidPrice3_get, _thostmduserapi.CThostFtdcDepthMarketDataField_BidPrice3_set)
    BidVolume3 = property(_thostmduserapi.CThostFtdcDepthMarketDataField_BidVolume3_get, _thostmduserapi.CThostFtdcDepthMarketDataField_BidVolume3_set)
    AskPrice3 = property(_thostmduserapi.CThostFtdcDepthMarketDataField_AskPrice3_get, _thostmduserapi.CThostFtdcDepthMarketDataField_AskPrice3_set)
    AskVolume3 = property(_thostmduserapi.CThostFtdcDepthMarketDataField_AskVolume3_get, _thostmduserapi.CThostFtdcDepthMarketDataField_AskVolume3_set)
    BidPrice4 = property(_thostmduserapi.CThostFtdcDepthMarketDataField_BidPrice4_get, _thostmduserapi.CThostFtdcDepthMarketDataField_BidPrice4_set)
    BidVolume4 = property(_thostmduserapi.CThostFtdcDepthMarketDataField_BidVolume4_get, _thostmduserapi.CThostFtdcDepthMarketDataField_BidVolume4_set)
    AskPrice4 = property(_thostmduserapi.CThostFtdcDepthMarketDataField_AskPrice4_get, _thostmduserapi.CThostFtdcDepthMarketDataField_AskPrice4_set)
    AskVolume4 = property(_thostmduserapi.CThostFtdcDepthMarketDataField_AskVolume4_get, _thostmduserapi.CThostFtdcDepthMarketDataField_AskVolume4_set)
    BidPrice5 = property(_thostmduserapi.CThostFtdcDepthMarketDataField_BidPrice5_get, _thostmduserapi.CThostFtdcDepthMarketDataField_BidPrice5_set)
    BidVolume5 = property(_thostmduserapi.CThostFtdcDepthMarketDataField_BidVolume5_get, _thostmduserapi.CThostFtdcDepthMarketDataField_BidVolume5_set)
    AskPrice5 = property(_thostmduserapi.CThostFtdcDepthMarketDataField_AskPrice5_get, _thostmduserapi.CThostFtdcDepthMarketDataField_AskPrice5_set)
    AskVolume5 = property(_thostmduserapi.CThostFtdcDepthMarketDataField_AskVolume5_get, _thostmduserapi.CThostFtdcDepthMarketDataField_AskVolume5_set)
    AveragePrice = property(_thostmduserapi.CThostFtdcDepthMarketDataField_AveragePrice_get, _thostmduserapi.CThostFtdcDepthMarketDataField_AveragePrice_set)
    ActionDay = property(_thostmduserapi.CThostFtdcDepthMarketDataField_ActionDay_get, _thostmduserapi.CThostFtdcDepthMarketDataField_ActionDay_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcDepthMarketDataField_swiginit(self, _thostmduserapi.new_CThostFtdcDepthMarketDataField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcDepthMarketDataField

# Register CThostFtdcDepthMarketDataField in _thostmduserapi:
_thostmduserapi.CThostFtdcDepthMarketDataField_swigregister(CThostFtdcDepthMarketDataField)

class CThostFtdcInstrumentTradingRightField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    InstrumentID = property(_thostmduserapi.CThostFtdcInstrumentTradingRightField_InstrumentID_get, _thostmduserapi.CThostFtdcInstrumentTradingRightField_InstrumentID_set)
    InvestorRange = property(_thostmduserapi.CThostFtdcInstrumentTradingRightField_InvestorRange_get, _thostmduserapi.CThostFtdcInstrumentTradingRightField_InvestorRange_set)
    BrokerID = property(_thostmduserapi.CThostFtdcInstrumentTradingRightField_BrokerID_get, _thostmduserapi.CThostFtdcInstrumentTradingRightField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcInstrumentTradingRightField_InvestorID_get, _thostmduserapi.CThostFtdcInstrumentTradingRightField_InvestorID_set)
    TradingRight = property(_thostmduserapi.CThostFtdcInstrumentTradingRightField_TradingRight_get, _thostmduserapi.CThostFtdcInstrumentTradingRightField_TradingRight_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcInstrumentTradingRightField_swiginit(self, _thostmduserapi.new_CThostFtdcInstrumentTradingRightField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcInstrumentTradingRightField

# Register CThostFtdcInstrumentTradingRightField in _thostmduserapi:
_thostmduserapi.CThostFtdcInstrumentTradingRightField_swigregister(CThostFtdcInstrumentTradingRightField)

class CThostFtdcBrokerUserField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcBrokerUserField_BrokerID_get, _thostmduserapi.CThostFtdcBrokerUserField_BrokerID_set)
    UserID = property(_thostmduserapi.CThostFtdcBrokerUserField_UserID_get, _thostmduserapi.CThostFtdcBrokerUserField_UserID_set)
    UserName = property(_thostmduserapi.CThostFtdcBrokerUserField_UserName_get, _thostmduserapi.CThostFtdcBrokerUserField_UserName_set)
    UserType = property(_thostmduserapi.CThostFtdcBrokerUserField_UserType_get, _thostmduserapi.CThostFtdcBrokerUserField_UserType_set)
    IsActive = property(_thostmduserapi.CThostFtdcBrokerUserField_IsActive_get, _thostmduserapi.CThostFtdcBrokerUserField_IsActive_set)
    IsUsingOTP = property(_thostmduserapi.CThostFtdcBrokerUserField_IsUsingOTP_get, _thostmduserapi.CThostFtdcBrokerUserField_IsUsingOTP_set)
    IsAuthForce = property(_thostmduserapi.CThostFtdcBrokerUserField_IsAuthForce_get, _thostmduserapi.CThostFtdcBrokerUserField_IsAuthForce_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcBrokerUserField_swiginit(self, _thostmduserapi.new_CThostFtdcBrokerUserField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcBrokerUserField

# Register CThostFtdcBrokerUserField in _thostmduserapi:
_thostmduserapi.CThostFtdcBrokerUserField_swigregister(CThostFtdcBrokerUserField)

class CThostFtdcBrokerUserPasswordField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcBrokerUserPasswordField_BrokerID_get, _thostmduserapi.CThostFtdcBrokerUserPasswordField_BrokerID_set)
    UserID = property(_thostmduserapi.CThostFtdcBrokerUserPasswordField_UserID_get, _thostmduserapi.CThostFtdcBrokerUserPasswordField_UserID_set)
    Password = property(_thostmduserapi.CThostFtdcBrokerUserPasswordField_Password_get, _thostmduserapi.CThostFtdcBrokerUserPasswordField_Password_set)
    LastUpdateTime = property(_thostmduserapi.CThostFtdcBrokerUserPasswordField_LastUpdateTime_get, _thostmduserapi.CThostFtdcBrokerUserPasswordField_LastUpdateTime_set)
    LastLoginTime = property(_thostmduserapi.CThostFtdcBrokerUserPasswordField_LastLoginTime_get, _thostmduserapi.CThostFtdcBrokerUserPasswordField_LastLoginTime_set)
    ExpireDate = property(_thostmduserapi.CThostFtdcBrokerUserPasswordField_ExpireDate_get, _thostmduserapi.CThostFtdcBrokerUserPasswordField_ExpireDate_set)
    WeakExpireDate = property(_thostmduserapi.CThostFtdcBrokerUserPasswordField_WeakExpireDate_get, _thostmduserapi.CThostFtdcBrokerUserPasswordField_WeakExpireDate_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcBrokerUserPasswordField_swiginit(self, _thostmduserapi.new_CThostFtdcBrokerUserPasswordField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcBrokerUserPasswordField

# Register CThostFtdcBrokerUserPasswordField in _thostmduserapi:
_thostmduserapi.CThostFtdcBrokerUserPasswordField_swigregister(CThostFtdcBrokerUserPasswordField)

class CThostFtdcBrokerUserFunctionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcBrokerUserFunctionField_BrokerID_get, _thostmduserapi.CThostFtdcBrokerUserFunctionField_BrokerID_set)
    UserID = property(_thostmduserapi.CThostFtdcBrokerUserFunctionField_UserID_get, _thostmduserapi.CThostFtdcBrokerUserFunctionField_UserID_set)
    BrokerFunctionCode = property(_thostmduserapi.CThostFtdcBrokerUserFunctionField_BrokerFunctionCode_get, _thostmduserapi.CThostFtdcBrokerUserFunctionField_BrokerFunctionCode_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcBrokerUserFunctionField_swiginit(self, _thostmduserapi.new_CThostFtdcBrokerUserFunctionField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcBrokerUserFunctionField

# Register CThostFtdcBrokerUserFunctionField in _thostmduserapi:
_thostmduserapi.CThostFtdcBrokerUserFunctionField_swigregister(CThostFtdcBrokerUserFunctionField)

class CThostFtdcTraderOfferField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ExchangeID = property(_thostmduserapi.CThostFtdcTraderOfferField_ExchangeID_get, _thostmduserapi.CThostFtdcTraderOfferField_ExchangeID_set)
    TraderID = property(_thostmduserapi.CThostFtdcTraderOfferField_TraderID_get, _thostmduserapi.CThostFtdcTraderOfferField_TraderID_set)
    ParticipantID = property(_thostmduserapi.CThostFtdcTraderOfferField_ParticipantID_get, _thostmduserapi.CThostFtdcTraderOfferField_ParticipantID_set)
    Password = property(_thostmduserapi.CThostFtdcTraderOfferField_Password_get, _thostmduserapi.CThostFtdcTraderOfferField_Password_set)
    InstallID = property(_thostmduserapi.CThostFtdcTraderOfferField_InstallID_get, _thostmduserapi.CThostFtdcTraderOfferField_InstallID_set)
    OrderLocalID = property(_thostmduserapi.CThostFtdcTraderOfferField_OrderLocalID_get, _thostmduserapi.CThostFtdcTraderOfferField_OrderLocalID_set)
    TraderConnectStatus = property(_thostmduserapi.CThostFtdcTraderOfferField_TraderConnectStatus_get, _thostmduserapi.CThostFtdcTraderOfferField_TraderConnectStatus_set)
    ConnectRequestDate = property(_thostmduserapi.CThostFtdcTraderOfferField_ConnectRequestDate_get, _thostmduserapi.CThostFtdcTraderOfferField_ConnectRequestDate_set)
    ConnectRequestTime = property(_thostmduserapi.CThostFtdcTraderOfferField_ConnectRequestTime_get, _thostmduserapi.CThostFtdcTraderOfferField_ConnectRequestTime_set)
    LastReportDate = property(_thostmduserapi.CThostFtdcTraderOfferField_LastReportDate_get, _thostmduserapi.CThostFtdcTraderOfferField_LastReportDate_set)
    LastReportTime = property(_thostmduserapi.CThostFtdcTraderOfferField_LastReportTime_get, _thostmduserapi.CThostFtdcTraderOfferField_LastReportTime_set)
    ConnectDate = property(_thostmduserapi.CThostFtdcTraderOfferField_ConnectDate_get, _thostmduserapi.CThostFtdcTraderOfferField_ConnectDate_set)
    ConnectTime = property(_thostmduserapi.CThostFtdcTraderOfferField_ConnectTime_get, _thostmduserapi.CThostFtdcTraderOfferField_ConnectTime_set)
    StartDate = property(_thostmduserapi.CThostFtdcTraderOfferField_StartDate_get, _thostmduserapi.CThostFtdcTraderOfferField_StartDate_set)
    StartTime = property(_thostmduserapi.CThostFtdcTraderOfferField_StartTime_get, _thostmduserapi.CThostFtdcTraderOfferField_StartTime_set)
    TradingDay = property(_thostmduserapi.CThostFtdcTraderOfferField_TradingDay_get, _thostmduserapi.CThostFtdcTraderOfferField_TradingDay_set)
    BrokerID = property(_thostmduserapi.CThostFtdcTraderOfferField_BrokerID_get, _thostmduserapi.CThostFtdcTraderOfferField_BrokerID_set)
    MaxTradeID = property(_thostmduserapi.CThostFtdcTraderOfferField_MaxTradeID_get, _thostmduserapi.CThostFtdcTraderOfferField_MaxTradeID_set)
    MaxOrderMessageReference = property(_thostmduserapi.CThostFtdcTraderOfferField_MaxOrderMessageReference_get, _thostmduserapi.CThostFtdcTraderOfferField_MaxOrderMessageReference_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcTraderOfferField_swiginit(self, _thostmduserapi.new_CThostFtdcTraderOfferField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcTraderOfferField

# Register CThostFtdcTraderOfferField in _thostmduserapi:
_thostmduserapi.CThostFtdcTraderOfferField_swigregister(CThostFtdcTraderOfferField)

class CThostFtdcSettlementInfoField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradingDay = property(_thostmduserapi.CThostFtdcSettlementInfoField_TradingDay_get, _thostmduserapi.CThostFtdcSettlementInfoField_TradingDay_set)
    SettlementID = property(_thostmduserapi.CThostFtdcSettlementInfoField_SettlementID_get, _thostmduserapi.CThostFtdcSettlementInfoField_SettlementID_set)
    BrokerID = property(_thostmduserapi.CThostFtdcSettlementInfoField_BrokerID_get, _thostmduserapi.CThostFtdcSettlementInfoField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcSettlementInfoField_InvestorID_get, _thostmduserapi.CThostFtdcSettlementInfoField_InvestorID_set)
    SequenceNo = property(_thostmduserapi.CThostFtdcSettlementInfoField_SequenceNo_get, _thostmduserapi.CThostFtdcSettlementInfoField_SequenceNo_set)
    Content = property(_thostmduserapi.CThostFtdcSettlementInfoField_Content_get, _thostmduserapi.CThostFtdcSettlementInfoField_Content_set)
    AccountID = property(_thostmduserapi.CThostFtdcSettlementInfoField_AccountID_get, _thostmduserapi.CThostFtdcSettlementInfoField_AccountID_set)
    CurrencyID = property(_thostmduserapi.CThostFtdcSettlementInfoField_CurrencyID_get, _thostmduserapi.CThostFtdcSettlementInfoField_CurrencyID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcSettlementInfoField_swiginit(self, _thostmduserapi.new_CThostFtdcSettlementInfoField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcSettlementInfoField

# Register CThostFtdcSettlementInfoField in _thostmduserapi:
_thostmduserapi.CThostFtdcSettlementInfoField_swigregister(CThostFtdcSettlementInfoField)

class CThostFtdcInstrumentMarginRateAdjustField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    InstrumentID = property(_thostmduserapi.CThostFtdcInstrumentMarginRateAdjustField_InstrumentID_get, _thostmduserapi.CThostFtdcInstrumentMarginRateAdjustField_InstrumentID_set)
    InvestorRange = property(_thostmduserapi.CThostFtdcInstrumentMarginRateAdjustField_InvestorRange_get, _thostmduserapi.CThostFtdcInstrumentMarginRateAdjustField_InvestorRange_set)
    BrokerID = property(_thostmduserapi.CThostFtdcInstrumentMarginRateAdjustField_BrokerID_get, _thostmduserapi.CThostFtdcInstrumentMarginRateAdjustField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcInstrumentMarginRateAdjustField_InvestorID_get, _thostmduserapi.CThostFtdcInstrumentMarginRateAdjustField_InvestorID_set)
    HedgeFlag = property(_thostmduserapi.CThostFtdcInstrumentMarginRateAdjustField_HedgeFlag_get, _thostmduserapi.CThostFtdcInstrumentMarginRateAdjustField_HedgeFlag_set)
    LongMarginRatioByMoney = property(_thostmduserapi.CThostFtdcInstrumentMarginRateAdjustField_LongMarginRatioByMoney_get, _thostmduserapi.CThostFtdcInstrumentMarginRateAdjustField_LongMarginRatioByMoney_set)
    LongMarginRatioByVolume = property(_thostmduserapi.CThostFtdcInstrumentMarginRateAdjustField_LongMarginRatioByVolume_get, _thostmduserapi.CThostFtdcInstrumentMarginRateAdjustField_LongMarginRatioByVolume_set)
    ShortMarginRatioByMoney = property(_thostmduserapi.CThostFtdcInstrumentMarginRateAdjustField_ShortMarginRatioByMoney_get, _thostmduserapi.CThostFtdcInstrumentMarginRateAdjustField_ShortMarginRatioByMoney_set)
    ShortMarginRatioByVolume = property(_thostmduserapi.CThostFtdcInstrumentMarginRateAdjustField_ShortMarginRatioByVolume_get, _thostmduserapi.CThostFtdcInstrumentMarginRateAdjustField_ShortMarginRatioByVolume_set)
    IsRelative = property(_thostmduserapi.CThostFtdcInstrumentMarginRateAdjustField_IsRelative_get, _thostmduserapi.CThostFtdcInstrumentMarginRateAdjustField_IsRelative_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcInstrumentMarginRateAdjustField_swiginit(self, _thostmduserapi.new_CThostFtdcInstrumentMarginRateAdjustField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcInstrumentMarginRateAdjustField

# Register CThostFtdcInstrumentMarginRateAdjustField in _thostmduserapi:
_thostmduserapi.CThostFtdcInstrumentMarginRateAdjustField_swigregister(CThostFtdcInstrumentMarginRateAdjustField)

class CThostFtdcExchangeMarginRateField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcExchangeMarginRateField_BrokerID_get, _thostmduserapi.CThostFtdcExchangeMarginRateField_BrokerID_set)
    InstrumentID = property(_thostmduserapi.CThostFtdcExchangeMarginRateField_InstrumentID_get, _thostmduserapi.CThostFtdcExchangeMarginRateField_InstrumentID_set)
    HedgeFlag = property(_thostmduserapi.CThostFtdcExchangeMarginRateField_HedgeFlag_get, _thostmduserapi.CThostFtdcExchangeMarginRateField_HedgeFlag_set)
    LongMarginRatioByMoney = property(_thostmduserapi.CThostFtdcExchangeMarginRateField_LongMarginRatioByMoney_get, _thostmduserapi.CThostFtdcExchangeMarginRateField_LongMarginRatioByMoney_set)
    LongMarginRatioByVolume = property(_thostmduserapi.CThostFtdcExchangeMarginRateField_LongMarginRatioByVolume_get, _thostmduserapi.CThostFtdcExchangeMarginRateField_LongMarginRatioByVolume_set)
    ShortMarginRatioByMoney = property(_thostmduserapi.CThostFtdcExchangeMarginRateField_ShortMarginRatioByMoney_get, _thostmduserapi.CThostFtdcExchangeMarginRateField_ShortMarginRatioByMoney_set)
    ShortMarginRatioByVolume = property(_thostmduserapi.CThostFtdcExchangeMarginRateField_ShortMarginRatioByVolume_get, _thostmduserapi.CThostFtdcExchangeMarginRateField_ShortMarginRatioByVolume_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcExchangeMarginRateField_ExchangeID_get, _thostmduserapi.CThostFtdcExchangeMarginRateField_ExchangeID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcExchangeMarginRateField_swiginit(self, _thostmduserapi.new_CThostFtdcExchangeMarginRateField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcExchangeMarginRateField

# Register CThostFtdcExchangeMarginRateField in _thostmduserapi:
_thostmduserapi.CThostFtdcExchangeMarginRateField_swigregister(CThostFtdcExchangeMarginRateField)

class CThostFtdcExchangeMarginRateAdjustField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcExchangeMarginRateAdjustField_BrokerID_get, _thostmduserapi.CThostFtdcExchangeMarginRateAdjustField_BrokerID_set)
    InstrumentID = property(_thostmduserapi.CThostFtdcExchangeMarginRateAdjustField_InstrumentID_get, _thostmduserapi.CThostFtdcExchangeMarginRateAdjustField_InstrumentID_set)
    HedgeFlag = property(_thostmduserapi.CThostFtdcExchangeMarginRateAdjustField_HedgeFlag_get, _thostmduserapi.CThostFtdcExchangeMarginRateAdjustField_HedgeFlag_set)
    LongMarginRatioByMoney = property(_thostmduserapi.CThostFtdcExchangeMarginRateAdjustField_LongMarginRatioByMoney_get, _thostmduserapi.CThostFtdcExchangeMarginRateAdjustField_LongMarginRatioByMoney_set)
    LongMarginRatioByVolume = property(_thostmduserapi.CThostFtdcExchangeMarginRateAdjustField_LongMarginRatioByVolume_get, _thostmduserapi.CThostFtdcExchangeMarginRateAdjustField_LongMarginRatioByVolume_set)
    ShortMarginRatioByMoney = property(_thostmduserapi.CThostFtdcExchangeMarginRateAdjustField_ShortMarginRatioByMoney_get, _thostmduserapi.CThostFtdcExchangeMarginRateAdjustField_ShortMarginRatioByMoney_set)
    ShortMarginRatioByVolume = property(_thostmduserapi.CThostFtdcExchangeMarginRateAdjustField_ShortMarginRatioByVolume_get, _thostmduserapi.CThostFtdcExchangeMarginRateAdjustField_ShortMarginRatioByVolume_set)
    ExchLongMarginRatioByMoney = property(_thostmduserapi.CThostFtdcExchangeMarginRateAdjustField_ExchLongMarginRatioByMoney_get, _thostmduserapi.CThostFtdcExchangeMarginRateAdjustField_ExchLongMarginRatioByMoney_set)
    ExchLongMarginRatioByVolume = property(_thostmduserapi.CThostFtdcExchangeMarginRateAdjustField_ExchLongMarginRatioByVolume_get, _thostmduserapi.CThostFtdcExchangeMarginRateAdjustField_ExchLongMarginRatioByVolume_set)
    ExchShortMarginRatioByMoney = property(_thostmduserapi.CThostFtdcExchangeMarginRateAdjustField_ExchShortMarginRatioByMoney_get, _thostmduserapi.CThostFtdcExchangeMarginRateAdjustField_ExchShortMarginRatioByMoney_set)
    ExchShortMarginRatioByVolume = property(_thostmduserapi.CThostFtdcExchangeMarginRateAdjustField_ExchShortMarginRatioByVolume_get, _thostmduserapi.CThostFtdcExchangeMarginRateAdjustField_ExchShortMarginRatioByVolume_set)
    NoLongMarginRatioByMoney = property(_thostmduserapi.CThostFtdcExchangeMarginRateAdjustField_NoLongMarginRatioByMoney_get, _thostmduserapi.CThostFtdcExchangeMarginRateAdjustField_NoLongMarginRatioByMoney_set)
    NoLongMarginRatioByVolume = property(_thostmduserapi.CThostFtdcExchangeMarginRateAdjustField_NoLongMarginRatioByVolume_get, _thostmduserapi.CThostFtdcExchangeMarginRateAdjustField_NoLongMarginRatioByVolume_set)
    NoShortMarginRatioByMoney = property(_thostmduserapi.CThostFtdcExchangeMarginRateAdjustField_NoShortMarginRatioByMoney_get, _thostmduserapi.CThostFtdcExchangeMarginRateAdjustField_NoShortMarginRatioByMoney_set)
    NoShortMarginRatioByVolume = property(_thostmduserapi.CThostFtdcExchangeMarginRateAdjustField_NoShortMarginRatioByVolume_get, _thostmduserapi.CThostFtdcExchangeMarginRateAdjustField_NoShortMarginRatioByVolume_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcExchangeMarginRateAdjustField_swiginit(self, _thostmduserapi.new_CThostFtdcExchangeMarginRateAdjustField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcExchangeMarginRateAdjustField

# Register CThostFtdcExchangeMarginRateAdjustField in _thostmduserapi:
_thostmduserapi.CThostFtdcExchangeMarginRateAdjustField_swigregister(CThostFtdcExchangeMarginRateAdjustField)

class CThostFtdcExchangeRateField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcExchangeRateField_BrokerID_get, _thostmduserapi.CThostFtdcExchangeRateField_BrokerID_set)
    FromCurrencyID = property(_thostmduserapi.CThostFtdcExchangeRateField_FromCurrencyID_get, _thostmduserapi.CThostFtdcExchangeRateField_FromCurrencyID_set)
    FromCurrencyUnit = property(_thostmduserapi.CThostFtdcExchangeRateField_FromCurrencyUnit_get, _thostmduserapi.CThostFtdcExchangeRateField_FromCurrencyUnit_set)
    ToCurrencyID = property(_thostmduserapi.CThostFtdcExchangeRateField_ToCurrencyID_get, _thostmduserapi.CThostFtdcExchangeRateField_ToCurrencyID_set)
    ExchangeRate = property(_thostmduserapi.CThostFtdcExchangeRateField_ExchangeRate_get, _thostmduserapi.CThostFtdcExchangeRateField_ExchangeRate_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcExchangeRateField_swiginit(self, _thostmduserapi.new_CThostFtdcExchangeRateField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcExchangeRateField

# Register CThostFtdcExchangeRateField in _thostmduserapi:
_thostmduserapi.CThostFtdcExchangeRateField_swigregister(CThostFtdcExchangeRateField)

class CThostFtdcSettlementRefField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradingDay = property(_thostmduserapi.CThostFtdcSettlementRefField_TradingDay_get, _thostmduserapi.CThostFtdcSettlementRefField_TradingDay_set)
    SettlementID = property(_thostmduserapi.CThostFtdcSettlementRefField_SettlementID_get, _thostmduserapi.CThostFtdcSettlementRefField_SettlementID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcSettlementRefField_swiginit(self, _thostmduserapi.new_CThostFtdcSettlementRefField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcSettlementRefField

# Register CThostFtdcSettlementRefField in _thostmduserapi:
_thostmduserapi.CThostFtdcSettlementRefField_swigregister(CThostFtdcSettlementRefField)

class CThostFtdcCurrentTimeField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    CurrDate = property(_thostmduserapi.CThostFtdcCurrentTimeField_CurrDate_get, _thostmduserapi.CThostFtdcCurrentTimeField_CurrDate_set)
    CurrTime = property(_thostmduserapi.CThostFtdcCurrentTimeField_CurrTime_get, _thostmduserapi.CThostFtdcCurrentTimeField_CurrTime_set)
    CurrMillisec = property(_thostmduserapi.CThostFtdcCurrentTimeField_CurrMillisec_get, _thostmduserapi.CThostFtdcCurrentTimeField_CurrMillisec_set)
    ActionDay = property(_thostmduserapi.CThostFtdcCurrentTimeField_ActionDay_get, _thostmduserapi.CThostFtdcCurrentTimeField_ActionDay_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcCurrentTimeField_swiginit(self, _thostmduserapi.new_CThostFtdcCurrentTimeField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcCurrentTimeField

# Register CThostFtdcCurrentTimeField in _thostmduserapi:
_thostmduserapi.CThostFtdcCurrentTimeField_swigregister(CThostFtdcCurrentTimeField)

class CThostFtdcCommPhaseField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradingDay = property(_thostmduserapi.CThostFtdcCommPhaseField_TradingDay_get, _thostmduserapi.CThostFtdcCommPhaseField_TradingDay_set)
    CommPhaseNo = property(_thostmduserapi.CThostFtdcCommPhaseField_CommPhaseNo_get, _thostmduserapi.CThostFtdcCommPhaseField_CommPhaseNo_set)
    SystemID = property(_thostmduserapi.CThostFtdcCommPhaseField_SystemID_get, _thostmduserapi.CThostFtdcCommPhaseField_SystemID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcCommPhaseField_swiginit(self, _thostmduserapi.new_CThostFtdcCommPhaseField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcCommPhaseField

# Register CThostFtdcCommPhaseField in _thostmduserapi:
_thostmduserapi.CThostFtdcCommPhaseField_swigregister(CThostFtdcCommPhaseField)

class CThostFtdcLoginInfoField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    FrontID = property(_thostmduserapi.CThostFtdcLoginInfoField_FrontID_get, _thostmduserapi.CThostFtdcLoginInfoField_FrontID_set)
    SessionID = property(_thostmduserapi.CThostFtdcLoginInfoField_SessionID_get, _thostmduserapi.CThostFtdcLoginInfoField_SessionID_set)
    BrokerID = property(_thostmduserapi.CThostFtdcLoginInfoField_BrokerID_get, _thostmduserapi.CThostFtdcLoginInfoField_BrokerID_set)
    UserID = property(_thostmduserapi.CThostFtdcLoginInfoField_UserID_get, _thostmduserapi.CThostFtdcLoginInfoField_UserID_set)
    LoginDate = property(_thostmduserapi.CThostFtdcLoginInfoField_LoginDate_get, _thostmduserapi.CThostFtdcLoginInfoField_LoginDate_set)
    LoginTime = property(_thostmduserapi.CThostFtdcLoginInfoField_LoginTime_get, _thostmduserapi.CThostFtdcLoginInfoField_LoginTime_set)
    IPAddress = property(_thostmduserapi.CThostFtdcLoginInfoField_IPAddress_get, _thostmduserapi.CThostFtdcLoginInfoField_IPAddress_set)
    UserProductInfo = property(_thostmduserapi.CThostFtdcLoginInfoField_UserProductInfo_get, _thostmduserapi.CThostFtdcLoginInfoField_UserProductInfo_set)
    InterfaceProductInfo = property(_thostmduserapi.CThostFtdcLoginInfoField_InterfaceProductInfo_get, _thostmduserapi.CThostFtdcLoginInfoField_InterfaceProductInfo_set)
    ProtocolInfo = property(_thostmduserapi.CThostFtdcLoginInfoField_ProtocolInfo_get, _thostmduserapi.CThostFtdcLoginInfoField_ProtocolInfo_set)
    SystemName = property(_thostmduserapi.CThostFtdcLoginInfoField_SystemName_get, _thostmduserapi.CThostFtdcLoginInfoField_SystemName_set)
    PasswordDeprecated = property(_thostmduserapi.CThostFtdcLoginInfoField_PasswordDeprecated_get, _thostmduserapi.CThostFtdcLoginInfoField_PasswordDeprecated_set)
    MaxOrderRef = property(_thostmduserapi.CThostFtdcLoginInfoField_MaxOrderRef_get, _thostmduserapi.CThostFtdcLoginInfoField_MaxOrderRef_set)
    SHFETime = property(_thostmduserapi.CThostFtdcLoginInfoField_SHFETime_get, _thostmduserapi.CThostFtdcLoginInfoField_SHFETime_set)
    DCETime = property(_thostmduserapi.CThostFtdcLoginInfoField_DCETime_get, _thostmduserapi.CThostFtdcLoginInfoField_DCETime_set)
    CZCETime = property(_thostmduserapi.CThostFtdcLoginInfoField_CZCETime_get, _thostmduserapi.CThostFtdcLoginInfoField_CZCETime_set)
    FFEXTime = property(_thostmduserapi.CThostFtdcLoginInfoField_FFEXTime_get, _thostmduserapi.CThostFtdcLoginInfoField_FFEXTime_set)
    MacAddress = property(_thostmduserapi.CThostFtdcLoginInfoField_MacAddress_get, _thostmduserapi.CThostFtdcLoginInfoField_MacAddress_set)
    OneTimePassword = property(_thostmduserapi.CThostFtdcLoginInfoField_OneTimePassword_get, _thostmduserapi.CThostFtdcLoginInfoField_OneTimePassword_set)
    INETime = property(_thostmduserapi.CThostFtdcLoginInfoField_INETime_get, _thostmduserapi.CThostFtdcLoginInfoField_INETime_set)
    IsQryControl = property(_thostmduserapi.CThostFtdcLoginInfoField_IsQryControl_get, _thostmduserapi.CThostFtdcLoginInfoField_IsQryControl_set)
    LoginRemark = property(_thostmduserapi.CThostFtdcLoginInfoField_LoginRemark_get, _thostmduserapi.CThostFtdcLoginInfoField_LoginRemark_set)
    Password = property(_thostmduserapi.CThostFtdcLoginInfoField_Password_get, _thostmduserapi.CThostFtdcLoginInfoField_Password_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcLoginInfoField_swiginit(self, _thostmduserapi.new_CThostFtdcLoginInfoField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcLoginInfoField

# Register CThostFtdcLoginInfoField in _thostmduserapi:
_thostmduserapi.CThostFtdcLoginInfoField_swigregister(CThostFtdcLoginInfoField)

class CThostFtdcLogoutAllField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    FrontID = property(_thostmduserapi.CThostFtdcLogoutAllField_FrontID_get, _thostmduserapi.CThostFtdcLogoutAllField_FrontID_set)
    SessionID = property(_thostmduserapi.CThostFtdcLogoutAllField_SessionID_get, _thostmduserapi.CThostFtdcLogoutAllField_SessionID_set)
    SystemName = property(_thostmduserapi.CThostFtdcLogoutAllField_SystemName_get, _thostmduserapi.CThostFtdcLogoutAllField_SystemName_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcLogoutAllField_swiginit(self, _thostmduserapi.new_CThostFtdcLogoutAllField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcLogoutAllField

# Register CThostFtdcLogoutAllField in _thostmduserapi:
_thostmduserapi.CThostFtdcLogoutAllField_swigregister(CThostFtdcLogoutAllField)

class CThostFtdcFrontStatusField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    FrontID = property(_thostmduserapi.CThostFtdcFrontStatusField_FrontID_get, _thostmduserapi.CThostFtdcFrontStatusField_FrontID_set)
    LastReportDate = property(_thostmduserapi.CThostFtdcFrontStatusField_LastReportDate_get, _thostmduserapi.CThostFtdcFrontStatusField_LastReportDate_set)
    LastReportTime = property(_thostmduserapi.CThostFtdcFrontStatusField_LastReportTime_get, _thostmduserapi.CThostFtdcFrontStatusField_LastReportTime_set)
    IsActive = property(_thostmduserapi.CThostFtdcFrontStatusField_IsActive_get, _thostmduserapi.CThostFtdcFrontStatusField_IsActive_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcFrontStatusField_swiginit(self, _thostmduserapi.new_CThostFtdcFrontStatusField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcFrontStatusField

# Register CThostFtdcFrontStatusField in _thostmduserapi:
_thostmduserapi.CThostFtdcFrontStatusField_swigregister(CThostFtdcFrontStatusField)

class CThostFtdcUserPasswordUpdateField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcUserPasswordUpdateField_BrokerID_get, _thostmduserapi.CThostFtdcUserPasswordUpdateField_BrokerID_set)
    UserID = property(_thostmduserapi.CThostFtdcUserPasswordUpdateField_UserID_get, _thostmduserapi.CThostFtdcUserPasswordUpdateField_UserID_set)
    OldPassword = property(_thostmduserapi.CThostFtdcUserPasswordUpdateField_OldPassword_get, _thostmduserapi.CThostFtdcUserPasswordUpdateField_OldPassword_set)
    NewPassword = property(_thostmduserapi.CThostFtdcUserPasswordUpdateField_NewPassword_get, _thostmduserapi.CThostFtdcUserPasswordUpdateField_NewPassword_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcUserPasswordUpdateField_swiginit(self, _thostmduserapi.new_CThostFtdcUserPasswordUpdateField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcUserPasswordUpdateField

# Register CThostFtdcUserPasswordUpdateField in _thostmduserapi:
_thostmduserapi.CThostFtdcUserPasswordUpdateField_swigregister(CThostFtdcUserPasswordUpdateField)

class CThostFtdcInputOrderField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcInputOrderField_BrokerID_get, _thostmduserapi.CThostFtdcInputOrderField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcInputOrderField_InvestorID_get, _thostmduserapi.CThostFtdcInputOrderField_InvestorID_set)
    InstrumentID = property(_thostmduserapi.CThostFtdcInputOrderField_InstrumentID_get, _thostmduserapi.CThostFtdcInputOrderField_InstrumentID_set)
    OrderRef = property(_thostmduserapi.CThostFtdcInputOrderField_OrderRef_get, _thostmduserapi.CThostFtdcInputOrderField_OrderRef_set)
    UserID = property(_thostmduserapi.CThostFtdcInputOrderField_UserID_get, _thostmduserapi.CThostFtdcInputOrderField_UserID_set)
    OrderPriceType = property(_thostmduserapi.CThostFtdcInputOrderField_OrderPriceType_get, _thostmduserapi.CThostFtdcInputOrderField_OrderPriceType_set)
    Direction = property(_thostmduserapi.CThostFtdcInputOrderField_Direction_get, _thostmduserapi.CThostFtdcInputOrderField_Direction_set)
    CombOffsetFlag = property(_thostmduserapi.CThostFtdcInputOrderField_CombOffsetFlag_get, _thostmduserapi.CThostFtdcInputOrderField_CombOffsetFlag_set)
    CombHedgeFlag = property(_thostmduserapi.CThostFtdcInputOrderField_CombHedgeFlag_get, _thostmduserapi.CThostFtdcInputOrderField_CombHedgeFlag_set)
    LimitPrice = property(_thostmduserapi.CThostFtdcInputOrderField_LimitPrice_get, _thostmduserapi.CThostFtdcInputOrderField_LimitPrice_set)
    VolumeTotalOriginal = property(_thostmduserapi.CThostFtdcInputOrderField_VolumeTotalOriginal_get, _thostmduserapi.CThostFtdcInputOrderField_VolumeTotalOriginal_set)
    TimeCondition = property(_thostmduserapi.CThostFtdcInputOrderField_TimeCondition_get, _thostmduserapi.CThostFtdcInputOrderField_TimeCondition_set)
    GTDDate = property(_thostmduserapi.CThostFtdcInputOrderField_GTDDate_get, _thostmduserapi.CThostFtdcInputOrderField_GTDDate_set)
    VolumeCondition = property(_thostmduserapi.CThostFtdcInputOrderField_VolumeCondition_get, _thostmduserapi.CThostFtdcInputOrderField_VolumeCondition_set)
    MinVolume = property(_thostmduserapi.CThostFtdcInputOrderField_MinVolume_get, _thostmduserapi.CThostFtdcInputOrderField_MinVolume_set)
    ContingentCondition = property(_thostmduserapi.CThostFtdcInputOrderField_ContingentCondition_get, _thostmduserapi.CThostFtdcInputOrderField_ContingentCondition_set)
    StopPrice = property(_thostmduserapi.CThostFtdcInputOrderField_StopPrice_get, _thostmduserapi.CThostFtdcInputOrderField_StopPrice_set)
    ForceCloseReason = property(_thostmduserapi.CThostFtdcInputOrderField_ForceCloseReason_get, _thostmduserapi.CThostFtdcInputOrderField_ForceCloseReason_set)
    IsAutoSuspend = property(_thostmduserapi.CThostFtdcInputOrderField_IsAutoSuspend_get, _thostmduserapi.CThostFtdcInputOrderField_IsAutoSuspend_set)
    BusinessUnit = property(_thostmduserapi.CThostFtdcInputOrderField_BusinessUnit_get, _thostmduserapi.CThostFtdcInputOrderField_BusinessUnit_set)
    RequestID = property(_thostmduserapi.CThostFtdcInputOrderField_RequestID_get, _thostmduserapi.CThostFtdcInputOrderField_RequestID_set)
    UserForceClose = property(_thostmduserapi.CThostFtdcInputOrderField_UserForceClose_get, _thostmduserapi.CThostFtdcInputOrderField_UserForceClose_set)
    IsSwapOrder = property(_thostmduserapi.CThostFtdcInputOrderField_IsSwapOrder_get, _thostmduserapi.CThostFtdcInputOrderField_IsSwapOrder_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcInputOrderField_ExchangeID_get, _thostmduserapi.CThostFtdcInputOrderField_ExchangeID_set)
    InvestUnitID = property(_thostmduserapi.CThostFtdcInputOrderField_InvestUnitID_get, _thostmduserapi.CThostFtdcInputOrderField_InvestUnitID_set)
    AccountID = property(_thostmduserapi.CThostFtdcInputOrderField_AccountID_get, _thostmduserapi.CThostFtdcInputOrderField_AccountID_set)
    CurrencyID = property(_thostmduserapi.CThostFtdcInputOrderField_CurrencyID_get, _thostmduserapi.CThostFtdcInputOrderField_CurrencyID_set)
    ClientID = property(_thostmduserapi.CThostFtdcInputOrderField_ClientID_get, _thostmduserapi.CThostFtdcInputOrderField_ClientID_set)
    IPAddress = property(_thostmduserapi.CThostFtdcInputOrderField_IPAddress_get, _thostmduserapi.CThostFtdcInputOrderField_IPAddress_set)
    MacAddress = property(_thostmduserapi.CThostFtdcInputOrderField_MacAddress_get, _thostmduserapi.CThostFtdcInputOrderField_MacAddress_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcInputOrderField_swiginit(self, _thostmduserapi.new_CThostFtdcInputOrderField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcInputOrderField

# Register CThostFtdcInputOrderField in _thostmduserapi:
_thostmduserapi.CThostFtdcInputOrderField_swigregister(CThostFtdcInputOrderField)

class CThostFtdcOrderField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcOrderField_BrokerID_get, _thostmduserapi.CThostFtdcOrderField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcOrderField_InvestorID_get, _thostmduserapi.CThostFtdcOrderField_InvestorID_set)
    InstrumentID = property(_thostmduserapi.CThostFtdcOrderField_InstrumentID_get, _thostmduserapi.CThostFtdcOrderField_InstrumentID_set)
    OrderRef = property(_thostmduserapi.CThostFtdcOrderField_OrderRef_get, _thostmduserapi.CThostFtdcOrderField_OrderRef_set)
    UserID = property(_thostmduserapi.CThostFtdcOrderField_UserID_get, _thostmduserapi.CThostFtdcOrderField_UserID_set)
    OrderPriceType = property(_thostmduserapi.CThostFtdcOrderField_OrderPriceType_get, _thostmduserapi.CThostFtdcOrderField_OrderPriceType_set)
    Direction = property(_thostmduserapi.CThostFtdcOrderField_Direction_get, _thostmduserapi.CThostFtdcOrderField_Direction_set)
    CombOffsetFlag = property(_thostmduserapi.CThostFtdcOrderField_CombOffsetFlag_get, _thostmduserapi.CThostFtdcOrderField_CombOffsetFlag_set)
    CombHedgeFlag = property(_thostmduserapi.CThostFtdcOrderField_CombHedgeFlag_get, _thostmduserapi.CThostFtdcOrderField_CombHedgeFlag_set)
    LimitPrice = property(_thostmduserapi.CThostFtdcOrderField_LimitPrice_get, _thostmduserapi.CThostFtdcOrderField_LimitPrice_set)
    VolumeTotalOriginal = property(_thostmduserapi.CThostFtdcOrderField_VolumeTotalOriginal_get, _thostmduserapi.CThostFtdcOrderField_VolumeTotalOriginal_set)
    TimeCondition = property(_thostmduserapi.CThostFtdcOrderField_TimeCondition_get, _thostmduserapi.CThostFtdcOrderField_TimeCondition_set)
    GTDDate = property(_thostmduserapi.CThostFtdcOrderField_GTDDate_get, _thostmduserapi.CThostFtdcOrderField_GTDDate_set)
    VolumeCondition = property(_thostmduserapi.CThostFtdcOrderField_VolumeCondition_get, _thostmduserapi.CThostFtdcOrderField_VolumeCondition_set)
    MinVolume = property(_thostmduserapi.CThostFtdcOrderField_MinVolume_get, _thostmduserapi.CThostFtdcOrderField_MinVolume_set)
    ContingentCondition = property(_thostmduserapi.CThostFtdcOrderField_ContingentCondition_get, _thostmduserapi.CThostFtdcOrderField_ContingentCondition_set)
    StopPrice = property(_thostmduserapi.CThostFtdcOrderField_StopPrice_get, _thostmduserapi.CThostFtdcOrderField_StopPrice_set)
    ForceCloseReason = property(_thostmduserapi.CThostFtdcOrderField_ForceCloseReason_get, _thostmduserapi.CThostFtdcOrderField_ForceCloseReason_set)
    IsAutoSuspend = property(_thostmduserapi.CThostFtdcOrderField_IsAutoSuspend_get, _thostmduserapi.CThostFtdcOrderField_IsAutoSuspend_set)
    BusinessUnit = property(_thostmduserapi.CThostFtdcOrderField_BusinessUnit_get, _thostmduserapi.CThostFtdcOrderField_BusinessUnit_set)
    RequestID = property(_thostmduserapi.CThostFtdcOrderField_RequestID_get, _thostmduserapi.CThostFtdcOrderField_RequestID_set)
    OrderLocalID = property(_thostmduserapi.CThostFtdcOrderField_OrderLocalID_get, _thostmduserapi.CThostFtdcOrderField_OrderLocalID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcOrderField_ExchangeID_get, _thostmduserapi.CThostFtdcOrderField_ExchangeID_set)
    ParticipantID = property(_thostmduserapi.CThostFtdcOrderField_ParticipantID_get, _thostmduserapi.CThostFtdcOrderField_ParticipantID_set)
    ClientID = property(_thostmduserapi.CThostFtdcOrderField_ClientID_get, _thostmduserapi.CThostFtdcOrderField_ClientID_set)
    ExchangeInstID = property(_thostmduserapi.CThostFtdcOrderField_ExchangeInstID_get, _thostmduserapi.CThostFtdcOrderField_ExchangeInstID_set)
    TraderID = property(_thostmduserapi.CThostFtdcOrderField_TraderID_get, _thostmduserapi.CThostFtdcOrderField_TraderID_set)
    InstallID = property(_thostmduserapi.CThostFtdcOrderField_InstallID_get, _thostmduserapi.CThostFtdcOrderField_InstallID_set)
    OrderSubmitStatus = property(_thostmduserapi.CThostFtdcOrderField_OrderSubmitStatus_get, _thostmduserapi.CThostFtdcOrderField_OrderSubmitStatus_set)
    NotifySequence = property(_thostmduserapi.CThostFtdcOrderField_NotifySequence_get, _thostmduserapi.CThostFtdcOrderField_NotifySequence_set)
    TradingDay = property(_thostmduserapi.CThostFtdcOrderField_TradingDay_get, _thostmduserapi.CThostFtdcOrderField_TradingDay_set)
    SettlementID = property(_thostmduserapi.CThostFtdcOrderField_SettlementID_get, _thostmduserapi.CThostFtdcOrderField_SettlementID_set)
    OrderSysID = property(_thostmduserapi.CThostFtdcOrderField_OrderSysID_get, _thostmduserapi.CThostFtdcOrderField_OrderSysID_set)
    OrderSource = property(_thostmduserapi.CThostFtdcOrderField_OrderSource_get, _thostmduserapi.CThostFtdcOrderField_OrderSource_set)
    OrderStatus = property(_thostmduserapi.CThostFtdcOrderField_OrderStatus_get, _thostmduserapi.CThostFtdcOrderField_OrderStatus_set)
    OrderType = property(_thostmduserapi.CThostFtdcOrderField_OrderType_get, _thostmduserapi.CThostFtdcOrderField_OrderType_set)
    VolumeTraded = property(_thostmduserapi.CThostFtdcOrderField_VolumeTraded_get, _thostmduserapi.CThostFtdcOrderField_VolumeTraded_set)
    VolumeTotal = property(_thostmduserapi.CThostFtdcOrderField_VolumeTotal_get, _thostmduserapi.CThostFtdcOrderField_VolumeTotal_set)
    InsertDate = property(_thostmduserapi.CThostFtdcOrderField_InsertDate_get, _thostmduserapi.CThostFtdcOrderField_InsertDate_set)
    InsertTime = property(_thostmduserapi.CThostFtdcOrderField_InsertTime_get, _thostmduserapi.CThostFtdcOrderField_InsertTime_set)
    ActiveTime = property(_thostmduserapi.CThostFtdcOrderField_ActiveTime_get, _thostmduserapi.CThostFtdcOrderField_ActiveTime_set)
    SuspendTime = property(_thostmduserapi.CThostFtdcOrderField_SuspendTime_get, _thostmduserapi.CThostFtdcOrderField_SuspendTime_set)
    UpdateTime = property(_thostmduserapi.CThostFtdcOrderField_UpdateTime_get, _thostmduserapi.CThostFtdcOrderField_UpdateTime_set)
    CancelTime = property(_thostmduserapi.CThostFtdcOrderField_CancelTime_get, _thostmduserapi.CThostFtdcOrderField_CancelTime_set)
    ActiveTraderID = property(_thostmduserapi.CThostFtdcOrderField_ActiveTraderID_get, _thostmduserapi.CThostFtdcOrderField_ActiveTraderID_set)
    ClearingPartID = property(_thostmduserapi.CThostFtdcOrderField_ClearingPartID_get, _thostmduserapi.CThostFtdcOrderField_ClearingPartID_set)
    SequenceNo = property(_thostmduserapi.CThostFtdcOrderField_SequenceNo_get, _thostmduserapi.CThostFtdcOrderField_SequenceNo_set)
    FrontID = property(_thostmduserapi.CThostFtdcOrderField_FrontID_get, _thostmduserapi.CThostFtdcOrderField_FrontID_set)
    SessionID = property(_thostmduserapi.CThostFtdcOrderField_SessionID_get, _thostmduserapi.CThostFtdcOrderField_SessionID_set)
    UserProductInfo = property(_thostmduserapi.CThostFtdcOrderField_UserProductInfo_get, _thostmduserapi.CThostFtdcOrderField_UserProductInfo_set)
    StatusMsg = property(_thostmduserapi.CThostFtdcOrderField_StatusMsg_get, _thostmduserapi.CThostFtdcOrderField_StatusMsg_set)
    UserForceClose = property(_thostmduserapi.CThostFtdcOrderField_UserForceClose_get, _thostmduserapi.CThostFtdcOrderField_UserForceClose_set)
    ActiveUserID = property(_thostmduserapi.CThostFtdcOrderField_ActiveUserID_get, _thostmduserapi.CThostFtdcOrderField_ActiveUserID_set)
    BrokerOrderSeq = property(_thostmduserapi.CThostFtdcOrderField_BrokerOrderSeq_get, _thostmduserapi.CThostFtdcOrderField_BrokerOrderSeq_set)
    RelativeOrderSysID = property(_thostmduserapi.CThostFtdcOrderField_RelativeOrderSysID_get, _thostmduserapi.CThostFtdcOrderField_RelativeOrderSysID_set)
    ZCETotalTradedVolume = property(_thostmduserapi.CThostFtdcOrderField_ZCETotalTradedVolume_get, _thostmduserapi.CThostFtdcOrderField_ZCETotalTradedVolume_set)
    IsSwapOrder = property(_thostmduserapi.CThostFtdcOrderField_IsSwapOrder_get, _thostmduserapi.CThostFtdcOrderField_IsSwapOrder_set)
    BranchID = property(_thostmduserapi.CThostFtdcOrderField_BranchID_get, _thostmduserapi.CThostFtdcOrderField_BranchID_set)
    InvestUnitID = property(_thostmduserapi.CThostFtdcOrderField_InvestUnitID_get, _thostmduserapi.CThostFtdcOrderField_InvestUnitID_set)
    AccountID = property(_thostmduserapi.CThostFtdcOrderField_AccountID_get, _thostmduserapi.CThostFtdcOrderField_AccountID_set)
    CurrencyID = property(_thostmduserapi.CThostFtdcOrderField_CurrencyID_get, _thostmduserapi.CThostFtdcOrderField_CurrencyID_set)
    IPAddress = property(_thostmduserapi.CThostFtdcOrderField_IPAddress_get, _thostmduserapi.CThostFtdcOrderField_IPAddress_set)
    MacAddress = property(_thostmduserapi.CThostFtdcOrderField_MacAddress_get, _thostmduserapi.CThostFtdcOrderField_MacAddress_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcOrderField_swiginit(self, _thostmduserapi.new_CThostFtdcOrderField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcOrderField

# Register CThostFtdcOrderField in _thostmduserapi:
_thostmduserapi.CThostFtdcOrderField_swigregister(CThostFtdcOrderField)

class CThostFtdcExchangeOrderField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    OrderPriceType = property(_thostmduserapi.CThostFtdcExchangeOrderField_OrderPriceType_get, _thostmduserapi.CThostFtdcExchangeOrderField_OrderPriceType_set)
    Direction = property(_thostmduserapi.CThostFtdcExchangeOrderField_Direction_get, _thostmduserapi.CThostFtdcExchangeOrderField_Direction_set)
    CombOffsetFlag = property(_thostmduserapi.CThostFtdcExchangeOrderField_CombOffsetFlag_get, _thostmduserapi.CThostFtdcExchangeOrderField_CombOffsetFlag_set)
    CombHedgeFlag = property(_thostmduserapi.CThostFtdcExchangeOrderField_CombHedgeFlag_get, _thostmduserapi.CThostFtdcExchangeOrderField_CombHedgeFlag_set)
    LimitPrice = property(_thostmduserapi.CThostFtdcExchangeOrderField_LimitPrice_get, _thostmduserapi.CThostFtdcExchangeOrderField_LimitPrice_set)
    VolumeTotalOriginal = property(_thostmduserapi.CThostFtdcExchangeOrderField_VolumeTotalOriginal_get, _thostmduserapi.CThostFtdcExchangeOrderField_VolumeTotalOriginal_set)
    TimeCondition = property(_thostmduserapi.CThostFtdcExchangeOrderField_TimeCondition_get, _thostmduserapi.CThostFtdcExchangeOrderField_TimeCondition_set)
    GTDDate = property(_thostmduserapi.CThostFtdcExchangeOrderField_GTDDate_get, _thostmduserapi.CThostFtdcExchangeOrderField_GTDDate_set)
    VolumeCondition = property(_thostmduserapi.CThostFtdcExchangeOrderField_VolumeCondition_get, _thostmduserapi.CThostFtdcExchangeOrderField_VolumeCondition_set)
    MinVolume = property(_thostmduserapi.CThostFtdcExchangeOrderField_MinVolume_get, _thostmduserapi.CThostFtdcExchangeOrderField_MinVolume_set)
    ContingentCondition = property(_thostmduserapi.CThostFtdcExchangeOrderField_ContingentCondition_get, _thostmduserapi.CThostFtdcExchangeOrderField_ContingentCondition_set)
    StopPrice = property(_thostmduserapi.CThostFtdcExchangeOrderField_StopPrice_get, _thostmduserapi.CThostFtdcExchangeOrderField_StopPrice_set)
    ForceCloseReason = property(_thostmduserapi.CThostFtdcExchangeOrderField_ForceCloseReason_get, _thostmduserapi.CThostFtdcExchangeOrderField_ForceCloseReason_set)
    IsAutoSuspend = property(_thostmduserapi.CThostFtdcExchangeOrderField_IsAutoSuspend_get, _thostmduserapi.CThostFtdcExchangeOrderField_IsAutoSuspend_set)
    BusinessUnit = property(_thostmduserapi.CThostFtdcExchangeOrderField_BusinessUnit_get, _thostmduserapi.CThostFtdcExchangeOrderField_BusinessUnit_set)
    RequestID = property(_thostmduserapi.CThostFtdcExchangeOrderField_RequestID_get, _thostmduserapi.CThostFtdcExchangeOrderField_RequestID_set)
    OrderLocalID = property(_thostmduserapi.CThostFtdcExchangeOrderField_OrderLocalID_get, _thostmduserapi.CThostFtdcExchangeOrderField_OrderLocalID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcExchangeOrderField_ExchangeID_get, _thostmduserapi.CThostFtdcExchangeOrderField_ExchangeID_set)
    ParticipantID = property(_thostmduserapi.CThostFtdcExchangeOrderField_ParticipantID_get, _thostmduserapi.CThostFtdcExchangeOrderField_ParticipantID_set)
    ClientID = property(_thostmduserapi.CThostFtdcExchangeOrderField_ClientID_get, _thostmduserapi.CThostFtdcExchangeOrderField_ClientID_set)
    ExchangeInstID = property(_thostmduserapi.CThostFtdcExchangeOrderField_ExchangeInstID_get, _thostmduserapi.CThostFtdcExchangeOrderField_ExchangeInstID_set)
    TraderID = property(_thostmduserapi.CThostFtdcExchangeOrderField_TraderID_get, _thostmduserapi.CThostFtdcExchangeOrderField_TraderID_set)
    InstallID = property(_thostmduserapi.CThostFtdcExchangeOrderField_InstallID_get, _thostmduserapi.CThostFtdcExchangeOrderField_InstallID_set)
    OrderSubmitStatus = property(_thostmduserapi.CThostFtdcExchangeOrderField_OrderSubmitStatus_get, _thostmduserapi.CThostFtdcExchangeOrderField_OrderSubmitStatus_set)
    NotifySequence = property(_thostmduserapi.CThostFtdcExchangeOrderField_NotifySequence_get, _thostmduserapi.CThostFtdcExchangeOrderField_NotifySequence_set)
    TradingDay = property(_thostmduserapi.CThostFtdcExchangeOrderField_TradingDay_get, _thostmduserapi.CThostFtdcExchangeOrderField_TradingDay_set)
    SettlementID = property(_thostmduserapi.CThostFtdcExchangeOrderField_SettlementID_get, _thostmduserapi.CThostFtdcExchangeOrderField_SettlementID_set)
    OrderSysID = property(_thostmduserapi.CThostFtdcExchangeOrderField_OrderSysID_get, _thostmduserapi.CThostFtdcExchangeOrderField_OrderSysID_set)
    OrderSource = property(_thostmduserapi.CThostFtdcExchangeOrderField_OrderSource_get, _thostmduserapi.CThostFtdcExchangeOrderField_OrderSource_set)
    OrderStatus = property(_thostmduserapi.CThostFtdcExchangeOrderField_OrderStatus_get, _thostmduserapi.CThostFtdcExchangeOrderField_OrderStatus_set)
    OrderType = property(_thostmduserapi.CThostFtdcExchangeOrderField_OrderType_get, _thostmduserapi.CThostFtdcExchangeOrderField_OrderType_set)
    VolumeTraded = property(_thostmduserapi.CThostFtdcExchangeOrderField_VolumeTraded_get, _thostmduserapi.CThostFtdcExchangeOrderField_VolumeTraded_set)
    VolumeTotal = property(_thostmduserapi.CThostFtdcExchangeOrderField_VolumeTotal_get, _thostmduserapi.CThostFtdcExchangeOrderField_VolumeTotal_set)
    InsertDate = property(_thostmduserapi.CThostFtdcExchangeOrderField_InsertDate_get, _thostmduserapi.CThostFtdcExchangeOrderField_InsertDate_set)
    InsertTime = property(_thostmduserapi.CThostFtdcExchangeOrderField_InsertTime_get, _thostmduserapi.CThostFtdcExchangeOrderField_InsertTime_set)
    ActiveTime = property(_thostmduserapi.CThostFtdcExchangeOrderField_ActiveTime_get, _thostmduserapi.CThostFtdcExchangeOrderField_ActiveTime_set)
    SuspendTime = property(_thostmduserapi.CThostFtdcExchangeOrderField_SuspendTime_get, _thostmduserapi.CThostFtdcExchangeOrderField_SuspendTime_set)
    UpdateTime = property(_thostmduserapi.CThostFtdcExchangeOrderField_UpdateTime_get, _thostmduserapi.CThostFtdcExchangeOrderField_UpdateTime_set)
    CancelTime = property(_thostmduserapi.CThostFtdcExchangeOrderField_CancelTime_get, _thostmduserapi.CThostFtdcExchangeOrderField_CancelTime_set)
    ActiveTraderID = property(_thostmduserapi.CThostFtdcExchangeOrderField_ActiveTraderID_get, _thostmduserapi.CThostFtdcExchangeOrderField_ActiveTraderID_set)
    ClearingPartID = property(_thostmduserapi.CThostFtdcExchangeOrderField_ClearingPartID_get, _thostmduserapi.CThostFtdcExchangeOrderField_ClearingPartID_set)
    SequenceNo = property(_thostmduserapi.CThostFtdcExchangeOrderField_SequenceNo_get, _thostmduserapi.CThostFtdcExchangeOrderField_SequenceNo_set)
    BranchID = property(_thostmduserapi.CThostFtdcExchangeOrderField_BranchID_get, _thostmduserapi.CThostFtdcExchangeOrderField_BranchID_set)
    IPAddress = property(_thostmduserapi.CThostFtdcExchangeOrderField_IPAddress_get, _thostmduserapi.CThostFtdcExchangeOrderField_IPAddress_set)
    MacAddress = property(_thostmduserapi.CThostFtdcExchangeOrderField_MacAddress_get, _thostmduserapi.CThostFtdcExchangeOrderField_MacAddress_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcExchangeOrderField_swiginit(self, _thostmduserapi.new_CThostFtdcExchangeOrderField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcExchangeOrderField

# Register CThostFtdcExchangeOrderField in _thostmduserapi:
_thostmduserapi.CThostFtdcExchangeOrderField_swigregister(CThostFtdcExchangeOrderField)

class CThostFtdcExchangeOrderInsertErrorField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ExchangeID = property(_thostmduserapi.CThostFtdcExchangeOrderInsertErrorField_ExchangeID_get, _thostmduserapi.CThostFtdcExchangeOrderInsertErrorField_ExchangeID_set)
    ParticipantID = property(_thostmduserapi.CThostFtdcExchangeOrderInsertErrorField_ParticipantID_get, _thostmduserapi.CThostFtdcExchangeOrderInsertErrorField_ParticipantID_set)
    TraderID = property(_thostmduserapi.CThostFtdcExchangeOrderInsertErrorField_TraderID_get, _thostmduserapi.CThostFtdcExchangeOrderInsertErrorField_TraderID_set)
    InstallID = property(_thostmduserapi.CThostFtdcExchangeOrderInsertErrorField_InstallID_get, _thostmduserapi.CThostFtdcExchangeOrderInsertErrorField_InstallID_set)
    OrderLocalID = property(_thostmduserapi.CThostFtdcExchangeOrderInsertErrorField_OrderLocalID_get, _thostmduserapi.CThostFtdcExchangeOrderInsertErrorField_OrderLocalID_set)
    ErrorID = property(_thostmduserapi.CThostFtdcExchangeOrderInsertErrorField_ErrorID_get, _thostmduserapi.CThostFtdcExchangeOrderInsertErrorField_ErrorID_set)
    ErrorMsg = property(_thostmduserapi.CThostFtdcExchangeOrderInsertErrorField_ErrorMsg_get, _thostmduserapi.CThostFtdcExchangeOrderInsertErrorField_ErrorMsg_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcExchangeOrderInsertErrorField_swiginit(self, _thostmduserapi.new_CThostFtdcExchangeOrderInsertErrorField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcExchangeOrderInsertErrorField

# Register CThostFtdcExchangeOrderInsertErrorField in _thostmduserapi:
_thostmduserapi.CThostFtdcExchangeOrderInsertErrorField_swigregister(CThostFtdcExchangeOrderInsertErrorField)

class CThostFtdcInputOrderActionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcInputOrderActionField_BrokerID_get, _thostmduserapi.CThostFtdcInputOrderActionField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcInputOrderActionField_InvestorID_get, _thostmduserapi.CThostFtdcInputOrderActionField_InvestorID_set)
    OrderActionRef = property(_thostmduserapi.CThostFtdcInputOrderActionField_OrderActionRef_get, _thostmduserapi.CThostFtdcInputOrderActionField_OrderActionRef_set)
    OrderRef = property(_thostmduserapi.CThostFtdcInputOrderActionField_OrderRef_get, _thostmduserapi.CThostFtdcInputOrderActionField_OrderRef_set)
    RequestID = property(_thostmduserapi.CThostFtdcInputOrderActionField_RequestID_get, _thostmduserapi.CThostFtdcInputOrderActionField_RequestID_set)
    FrontID = property(_thostmduserapi.CThostFtdcInputOrderActionField_FrontID_get, _thostmduserapi.CThostFtdcInputOrderActionField_FrontID_set)
    SessionID = property(_thostmduserapi.CThostFtdcInputOrderActionField_SessionID_get, _thostmduserapi.CThostFtdcInputOrderActionField_SessionID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcInputOrderActionField_ExchangeID_get, _thostmduserapi.CThostFtdcInputOrderActionField_ExchangeID_set)
    OrderSysID = property(_thostmduserapi.CThostFtdcInputOrderActionField_OrderSysID_get, _thostmduserapi.CThostFtdcInputOrderActionField_OrderSysID_set)
    ActionFlag = property(_thostmduserapi.CThostFtdcInputOrderActionField_ActionFlag_get, _thostmduserapi.CThostFtdcInputOrderActionField_ActionFlag_set)
    LimitPrice = property(_thostmduserapi.CThostFtdcInputOrderActionField_LimitPrice_get, _thostmduserapi.CThostFtdcInputOrderActionField_LimitPrice_set)
    VolumeChange = property(_thostmduserapi.CThostFtdcInputOrderActionField_VolumeChange_get, _thostmduserapi.CThostFtdcInputOrderActionField_VolumeChange_set)
    UserID = property(_thostmduserapi.CThostFtdcInputOrderActionField_UserID_get, _thostmduserapi.CThostFtdcInputOrderActionField_UserID_set)
    InstrumentID = property(_thostmduserapi.CThostFtdcInputOrderActionField_InstrumentID_get, _thostmduserapi.CThostFtdcInputOrderActionField_InstrumentID_set)
    InvestUnitID = property(_thostmduserapi.CThostFtdcInputOrderActionField_InvestUnitID_get, _thostmduserapi.CThostFtdcInputOrderActionField_InvestUnitID_set)
    IPAddress = property(_thostmduserapi.CThostFtdcInputOrderActionField_IPAddress_get, _thostmduserapi.CThostFtdcInputOrderActionField_IPAddress_set)
    MacAddress = property(_thostmduserapi.CThostFtdcInputOrderActionField_MacAddress_get, _thostmduserapi.CThostFtdcInputOrderActionField_MacAddress_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcInputOrderActionField_swiginit(self, _thostmduserapi.new_CThostFtdcInputOrderActionField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcInputOrderActionField

# Register CThostFtdcInputOrderActionField in _thostmduserapi:
_thostmduserapi.CThostFtdcInputOrderActionField_swigregister(CThostFtdcInputOrderActionField)

class CThostFtdcOrderActionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcOrderActionField_BrokerID_get, _thostmduserapi.CThostFtdcOrderActionField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcOrderActionField_InvestorID_get, _thostmduserapi.CThostFtdcOrderActionField_InvestorID_set)
    OrderActionRef = property(_thostmduserapi.CThostFtdcOrderActionField_OrderActionRef_get, _thostmduserapi.CThostFtdcOrderActionField_OrderActionRef_set)
    OrderRef = property(_thostmduserapi.CThostFtdcOrderActionField_OrderRef_get, _thostmduserapi.CThostFtdcOrderActionField_OrderRef_set)
    RequestID = property(_thostmduserapi.CThostFtdcOrderActionField_RequestID_get, _thostmduserapi.CThostFtdcOrderActionField_RequestID_set)
    FrontID = property(_thostmduserapi.CThostFtdcOrderActionField_FrontID_get, _thostmduserapi.CThostFtdcOrderActionField_FrontID_set)
    SessionID = property(_thostmduserapi.CThostFtdcOrderActionField_SessionID_get, _thostmduserapi.CThostFtdcOrderActionField_SessionID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcOrderActionField_ExchangeID_get, _thostmduserapi.CThostFtdcOrderActionField_ExchangeID_set)
    OrderSysID = property(_thostmduserapi.CThostFtdcOrderActionField_OrderSysID_get, _thostmduserapi.CThostFtdcOrderActionField_OrderSysID_set)
    ActionFlag = property(_thostmduserapi.CThostFtdcOrderActionField_ActionFlag_get, _thostmduserapi.CThostFtdcOrderActionField_ActionFlag_set)
    LimitPrice = property(_thostmduserapi.CThostFtdcOrderActionField_LimitPrice_get, _thostmduserapi.CThostFtdcOrderActionField_LimitPrice_set)
    VolumeChange = property(_thostmduserapi.CThostFtdcOrderActionField_VolumeChange_get, _thostmduserapi.CThostFtdcOrderActionField_VolumeChange_set)
    ActionDate = property(_thostmduserapi.CThostFtdcOrderActionField_ActionDate_get, _thostmduserapi.CThostFtdcOrderActionField_ActionDate_set)
    ActionTime = property(_thostmduserapi.CThostFtdcOrderActionField_ActionTime_get, _thostmduserapi.CThostFtdcOrderActionField_ActionTime_set)
    TraderID = property(_thostmduserapi.CThostFtdcOrderActionField_TraderID_get, _thostmduserapi.CThostFtdcOrderActionField_TraderID_set)
    InstallID = property(_thostmduserapi.CThostFtdcOrderActionField_InstallID_get, _thostmduserapi.CThostFtdcOrderActionField_InstallID_set)
    OrderLocalID = property(_thostmduserapi.CThostFtdcOrderActionField_OrderLocalID_get, _thostmduserapi.CThostFtdcOrderActionField_OrderLocalID_set)
    ActionLocalID = property(_thostmduserapi.CThostFtdcOrderActionField_ActionLocalID_get, _thostmduserapi.CThostFtdcOrderActionField_ActionLocalID_set)
    ParticipantID = property(_thostmduserapi.CThostFtdcOrderActionField_ParticipantID_get, _thostmduserapi.CThostFtdcOrderActionField_ParticipantID_set)
    ClientID = property(_thostmduserapi.CThostFtdcOrderActionField_ClientID_get, _thostmduserapi.CThostFtdcOrderActionField_ClientID_set)
    BusinessUnit = property(_thostmduserapi.CThostFtdcOrderActionField_BusinessUnit_get, _thostmduserapi.CThostFtdcOrderActionField_BusinessUnit_set)
    OrderActionStatus = property(_thostmduserapi.CThostFtdcOrderActionField_OrderActionStatus_get, _thostmduserapi.CThostFtdcOrderActionField_OrderActionStatus_set)
    UserID = property(_thostmduserapi.CThostFtdcOrderActionField_UserID_get, _thostmduserapi.CThostFtdcOrderActionField_UserID_set)
    StatusMsg = property(_thostmduserapi.CThostFtdcOrderActionField_StatusMsg_get, _thostmduserapi.CThostFtdcOrderActionField_StatusMsg_set)
    InstrumentID = property(_thostmduserapi.CThostFtdcOrderActionField_InstrumentID_get, _thostmduserapi.CThostFtdcOrderActionField_InstrumentID_set)
    BranchID = property(_thostmduserapi.CThostFtdcOrderActionField_BranchID_get, _thostmduserapi.CThostFtdcOrderActionField_BranchID_set)
    InvestUnitID = property(_thostmduserapi.CThostFtdcOrderActionField_InvestUnitID_get, _thostmduserapi.CThostFtdcOrderActionField_InvestUnitID_set)
    IPAddress = property(_thostmduserapi.CThostFtdcOrderActionField_IPAddress_get, _thostmduserapi.CThostFtdcOrderActionField_IPAddress_set)
    MacAddress = property(_thostmduserapi.CThostFtdcOrderActionField_MacAddress_get, _thostmduserapi.CThostFtdcOrderActionField_MacAddress_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcOrderActionField_swiginit(self, _thostmduserapi.new_CThostFtdcOrderActionField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcOrderActionField

# Register CThostFtdcOrderActionField in _thostmduserapi:
_thostmduserapi.CThostFtdcOrderActionField_swigregister(CThostFtdcOrderActionField)

class CThostFtdcExchangeOrderActionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ExchangeID = property(_thostmduserapi.CThostFtdcExchangeOrderActionField_ExchangeID_get, _thostmduserapi.CThostFtdcExchangeOrderActionField_ExchangeID_set)
    OrderSysID = property(_thostmduserapi.CThostFtdcExchangeOrderActionField_OrderSysID_get, _thostmduserapi.CThostFtdcExchangeOrderActionField_OrderSysID_set)
    ActionFlag = property(_thostmduserapi.CThostFtdcExchangeOrderActionField_ActionFlag_get, _thostmduserapi.CThostFtdcExchangeOrderActionField_ActionFlag_set)
    LimitPrice = property(_thostmduserapi.CThostFtdcExchangeOrderActionField_LimitPrice_get, _thostmduserapi.CThostFtdcExchangeOrderActionField_LimitPrice_set)
    VolumeChange = property(_thostmduserapi.CThostFtdcExchangeOrderActionField_VolumeChange_get, _thostmduserapi.CThostFtdcExchangeOrderActionField_VolumeChange_set)
    ActionDate = property(_thostmduserapi.CThostFtdcExchangeOrderActionField_ActionDate_get, _thostmduserapi.CThostFtdcExchangeOrderActionField_ActionDate_set)
    ActionTime = property(_thostmduserapi.CThostFtdcExchangeOrderActionField_ActionTime_get, _thostmduserapi.CThostFtdcExchangeOrderActionField_ActionTime_set)
    TraderID = property(_thostmduserapi.CThostFtdcExchangeOrderActionField_TraderID_get, _thostmduserapi.CThostFtdcExchangeOrderActionField_TraderID_set)
    InstallID = property(_thostmduserapi.CThostFtdcExchangeOrderActionField_InstallID_get, _thostmduserapi.CThostFtdcExchangeOrderActionField_InstallID_set)
    OrderLocalID = property(_thostmduserapi.CThostFtdcExchangeOrderActionField_OrderLocalID_get, _thostmduserapi.CThostFtdcExchangeOrderActionField_OrderLocalID_set)
    ActionLocalID = property(_thostmduserapi.CThostFtdcExchangeOrderActionField_ActionLocalID_get, _thostmduserapi.CThostFtdcExchangeOrderActionField_ActionLocalID_set)
    ParticipantID = property(_thostmduserapi.CThostFtdcExchangeOrderActionField_ParticipantID_get, _thostmduserapi.CThostFtdcExchangeOrderActionField_ParticipantID_set)
    ClientID = property(_thostmduserapi.CThostFtdcExchangeOrderActionField_ClientID_get, _thostmduserapi.CThostFtdcExchangeOrderActionField_ClientID_set)
    BusinessUnit = property(_thostmduserapi.CThostFtdcExchangeOrderActionField_BusinessUnit_get, _thostmduserapi.CThostFtdcExchangeOrderActionField_BusinessUnit_set)
    OrderActionStatus = property(_thostmduserapi.CThostFtdcExchangeOrderActionField_OrderActionStatus_get, _thostmduserapi.CThostFtdcExchangeOrderActionField_OrderActionStatus_set)
    UserID = property(_thostmduserapi.CThostFtdcExchangeOrderActionField_UserID_get, _thostmduserapi.CThostFtdcExchangeOrderActionField_UserID_set)
    BranchID = property(_thostmduserapi.CThostFtdcExchangeOrderActionField_BranchID_get, _thostmduserapi.CThostFtdcExchangeOrderActionField_BranchID_set)
    IPAddress = property(_thostmduserapi.CThostFtdcExchangeOrderActionField_IPAddress_get, _thostmduserapi.CThostFtdcExchangeOrderActionField_IPAddress_set)
    MacAddress = property(_thostmduserapi.CThostFtdcExchangeOrderActionField_MacAddress_get, _thostmduserapi.CThostFtdcExchangeOrderActionField_MacAddress_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcExchangeOrderActionField_swiginit(self, _thostmduserapi.new_CThostFtdcExchangeOrderActionField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcExchangeOrderActionField

# Register CThostFtdcExchangeOrderActionField in _thostmduserapi:
_thostmduserapi.CThostFtdcExchangeOrderActionField_swigregister(CThostFtdcExchangeOrderActionField)

class CThostFtdcExchangeOrderActionErrorField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ExchangeID = property(_thostmduserapi.CThostFtdcExchangeOrderActionErrorField_ExchangeID_get, _thostmduserapi.CThostFtdcExchangeOrderActionErrorField_ExchangeID_set)
    OrderSysID = property(_thostmduserapi.CThostFtdcExchangeOrderActionErrorField_OrderSysID_get, _thostmduserapi.CThostFtdcExchangeOrderActionErrorField_OrderSysID_set)
    TraderID = property(_thostmduserapi.CThostFtdcExchangeOrderActionErrorField_TraderID_get, _thostmduserapi.CThostFtdcExchangeOrderActionErrorField_TraderID_set)
    InstallID = property(_thostmduserapi.CThostFtdcExchangeOrderActionErrorField_InstallID_get, _thostmduserapi.CThostFtdcExchangeOrderActionErrorField_InstallID_set)
    OrderLocalID = property(_thostmduserapi.CThostFtdcExchangeOrderActionErrorField_OrderLocalID_get, _thostmduserapi.CThostFtdcExchangeOrderActionErrorField_OrderLocalID_set)
    ActionLocalID = property(_thostmduserapi.CThostFtdcExchangeOrderActionErrorField_ActionLocalID_get, _thostmduserapi.CThostFtdcExchangeOrderActionErrorField_ActionLocalID_set)
    ErrorID = property(_thostmduserapi.CThostFtdcExchangeOrderActionErrorField_ErrorID_get, _thostmduserapi.CThostFtdcExchangeOrderActionErrorField_ErrorID_set)
    ErrorMsg = property(_thostmduserapi.CThostFtdcExchangeOrderActionErrorField_ErrorMsg_get, _thostmduserapi.CThostFtdcExchangeOrderActionErrorField_ErrorMsg_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcExchangeOrderActionErrorField_swiginit(self, _thostmduserapi.new_CThostFtdcExchangeOrderActionErrorField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcExchangeOrderActionErrorField

# Register CThostFtdcExchangeOrderActionErrorField in _thostmduserapi:
_thostmduserapi.CThostFtdcExchangeOrderActionErrorField_swigregister(CThostFtdcExchangeOrderActionErrorField)

class CThostFtdcExchangeTradeField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ExchangeID = property(_thostmduserapi.CThostFtdcExchangeTradeField_ExchangeID_get, _thostmduserapi.CThostFtdcExchangeTradeField_ExchangeID_set)
    TradeID = property(_thostmduserapi.CThostFtdcExchangeTradeField_TradeID_get, _thostmduserapi.CThostFtdcExchangeTradeField_TradeID_set)
    Direction = property(_thostmduserapi.CThostFtdcExchangeTradeField_Direction_get, _thostmduserapi.CThostFtdcExchangeTradeField_Direction_set)
    OrderSysID = property(_thostmduserapi.CThostFtdcExchangeTradeField_OrderSysID_get, _thostmduserapi.CThostFtdcExchangeTradeField_OrderSysID_set)
    ParticipantID = property(_thostmduserapi.CThostFtdcExchangeTradeField_ParticipantID_get, _thostmduserapi.CThostFtdcExchangeTradeField_ParticipantID_set)
    ClientID = property(_thostmduserapi.CThostFtdcExchangeTradeField_ClientID_get, _thostmduserapi.CThostFtdcExchangeTradeField_ClientID_set)
    TradingRole = property(_thostmduserapi.CThostFtdcExchangeTradeField_TradingRole_get, _thostmduserapi.CThostFtdcExchangeTradeField_TradingRole_set)
    ExchangeInstID = property(_thostmduserapi.CThostFtdcExchangeTradeField_ExchangeInstID_get, _thostmduserapi.CThostFtdcExchangeTradeField_ExchangeInstID_set)
    OffsetFlag = property(_thostmduserapi.CThostFtdcExchangeTradeField_OffsetFlag_get, _thostmduserapi.CThostFtdcExchangeTradeField_OffsetFlag_set)
    HedgeFlag = property(_thostmduserapi.CThostFtdcExchangeTradeField_HedgeFlag_get, _thostmduserapi.CThostFtdcExchangeTradeField_HedgeFlag_set)
    Price = property(_thostmduserapi.CThostFtdcExchangeTradeField_Price_get, _thostmduserapi.CThostFtdcExchangeTradeField_Price_set)
    Volume = property(_thostmduserapi.CThostFtdcExchangeTradeField_Volume_get, _thostmduserapi.CThostFtdcExchangeTradeField_Volume_set)
    TradeDate = property(_thostmduserapi.CThostFtdcExchangeTradeField_TradeDate_get, _thostmduserapi.CThostFtdcExchangeTradeField_TradeDate_set)
    TradeTime = property(_thostmduserapi.CThostFtdcExchangeTradeField_TradeTime_get, _thostmduserapi.CThostFtdcExchangeTradeField_TradeTime_set)
    TradeType = property(_thostmduserapi.CThostFtdcExchangeTradeField_TradeType_get, _thostmduserapi.CThostFtdcExchangeTradeField_TradeType_set)
    PriceSource = property(_thostmduserapi.CThostFtdcExchangeTradeField_PriceSource_get, _thostmduserapi.CThostFtdcExchangeTradeField_PriceSource_set)
    TraderID = property(_thostmduserapi.CThostFtdcExchangeTradeField_TraderID_get, _thostmduserapi.CThostFtdcExchangeTradeField_TraderID_set)
    OrderLocalID = property(_thostmduserapi.CThostFtdcExchangeTradeField_OrderLocalID_get, _thostmduserapi.CThostFtdcExchangeTradeField_OrderLocalID_set)
    ClearingPartID = property(_thostmduserapi.CThostFtdcExchangeTradeField_ClearingPartID_get, _thostmduserapi.CThostFtdcExchangeTradeField_ClearingPartID_set)
    BusinessUnit = property(_thostmduserapi.CThostFtdcExchangeTradeField_BusinessUnit_get, _thostmduserapi.CThostFtdcExchangeTradeField_BusinessUnit_set)
    SequenceNo = property(_thostmduserapi.CThostFtdcExchangeTradeField_SequenceNo_get, _thostmduserapi.CThostFtdcExchangeTradeField_SequenceNo_set)
    TradeSource = property(_thostmduserapi.CThostFtdcExchangeTradeField_TradeSource_get, _thostmduserapi.CThostFtdcExchangeTradeField_TradeSource_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcExchangeTradeField_swiginit(self, _thostmduserapi.new_CThostFtdcExchangeTradeField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcExchangeTradeField

# Register CThostFtdcExchangeTradeField in _thostmduserapi:
_thostmduserapi.CThostFtdcExchangeTradeField_swigregister(CThostFtdcExchangeTradeField)

class CThostFtdcTradeField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcTradeField_BrokerID_get, _thostmduserapi.CThostFtdcTradeField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcTradeField_InvestorID_get, _thostmduserapi.CThostFtdcTradeField_InvestorID_set)
    InstrumentID = property(_thostmduserapi.CThostFtdcTradeField_InstrumentID_get, _thostmduserapi.CThostFtdcTradeField_InstrumentID_set)
    OrderRef = property(_thostmduserapi.CThostFtdcTradeField_OrderRef_get, _thostmduserapi.CThostFtdcTradeField_OrderRef_set)
    UserID = property(_thostmduserapi.CThostFtdcTradeField_UserID_get, _thostmduserapi.CThostFtdcTradeField_UserID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcTradeField_ExchangeID_get, _thostmduserapi.CThostFtdcTradeField_ExchangeID_set)
    TradeID = property(_thostmduserapi.CThostFtdcTradeField_TradeID_get, _thostmduserapi.CThostFtdcTradeField_TradeID_set)
    Direction = property(_thostmduserapi.CThostFtdcTradeField_Direction_get, _thostmduserapi.CThostFtdcTradeField_Direction_set)
    OrderSysID = property(_thostmduserapi.CThostFtdcTradeField_OrderSysID_get, _thostmduserapi.CThostFtdcTradeField_OrderSysID_set)
    ParticipantID = property(_thostmduserapi.CThostFtdcTradeField_ParticipantID_get, _thostmduserapi.CThostFtdcTradeField_ParticipantID_set)
    ClientID = property(_thostmduserapi.CThostFtdcTradeField_ClientID_get, _thostmduserapi.CThostFtdcTradeField_ClientID_set)
    TradingRole = property(_thostmduserapi.CThostFtdcTradeField_TradingRole_get, _thostmduserapi.CThostFtdcTradeField_TradingRole_set)
    ExchangeInstID = property(_thostmduserapi.CThostFtdcTradeField_ExchangeInstID_get, _thostmduserapi.CThostFtdcTradeField_ExchangeInstID_set)
    OffsetFlag = property(_thostmduserapi.CThostFtdcTradeField_OffsetFlag_get, _thostmduserapi.CThostFtdcTradeField_OffsetFlag_set)
    HedgeFlag = property(_thostmduserapi.CThostFtdcTradeField_HedgeFlag_get, _thostmduserapi.CThostFtdcTradeField_HedgeFlag_set)
    Price = property(_thostmduserapi.CThostFtdcTradeField_Price_get, _thostmduserapi.CThostFtdcTradeField_Price_set)
    Volume = property(_thostmduserapi.CThostFtdcTradeField_Volume_get, _thostmduserapi.CThostFtdcTradeField_Volume_set)
    TradeDate = property(_thostmduserapi.CThostFtdcTradeField_TradeDate_get, _thostmduserapi.CThostFtdcTradeField_TradeDate_set)
    TradeTime = property(_thostmduserapi.CThostFtdcTradeField_TradeTime_get, _thostmduserapi.CThostFtdcTradeField_TradeTime_set)
    TradeType = property(_thostmduserapi.CThostFtdcTradeField_TradeType_get, _thostmduserapi.CThostFtdcTradeField_TradeType_set)
    PriceSource = property(_thostmduserapi.CThostFtdcTradeField_PriceSource_get, _thostmduserapi.CThostFtdcTradeField_PriceSource_set)
    TraderID = property(_thostmduserapi.CThostFtdcTradeField_TraderID_get, _thostmduserapi.CThostFtdcTradeField_TraderID_set)
    OrderLocalID = property(_thostmduserapi.CThostFtdcTradeField_OrderLocalID_get, _thostmduserapi.CThostFtdcTradeField_OrderLocalID_set)
    ClearingPartID = property(_thostmduserapi.CThostFtdcTradeField_ClearingPartID_get, _thostmduserapi.CThostFtdcTradeField_ClearingPartID_set)
    BusinessUnit = property(_thostmduserapi.CThostFtdcTradeField_BusinessUnit_get, _thostmduserapi.CThostFtdcTradeField_BusinessUnit_set)
    SequenceNo = property(_thostmduserapi.CThostFtdcTradeField_SequenceNo_get, _thostmduserapi.CThostFtdcTradeField_SequenceNo_set)
    TradingDay = property(_thostmduserapi.CThostFtdcTradeField_TradingDay_get, _thostmduserapi.CThostFtdcTradeField_TradingDay_set)
    SettlementID = property(_thostmduserapi.CThostFtdcTradeField_SettlementID_get, _thostmduserapi.CThostFtdcTradeField_SettlementID_set)
    BrokerOrderSeq = property(_thostmduserapi.CThostFtdcTradeField_BrokerOrderSeq_get, _thostmduserapi.CThostFtdcTradeField_BrokerOrderSeq_set)
    TradeSource = property(_thostmduserapi.CThostFtdcTradeField_TradeSource_get, _thostmduserapi.CThostFtdcTradeField_TradeSource_set)
    InvestUnitID = property(_thostmduserapi.CThostFtdcTradeField_InvestUnitID_get, _thostmduserapi.CThostFtdcTradeField_InvestUnitID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcTradeField_swiginit(self, _thostmduserapi.new_CThostFtdcTradeField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcTradeField

# Register CThostFtdcTradeField in _thostmduserapi:
_thostmduserapi.CThostFtdcTradeField_swigregister(CThostFtdcTradeField)

class CThostFtdcUserSessionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    FrontID = property(_thostmduserapi.CThostFtdcUserSessionField_FrontID_get, _thostmduserapi.CThostFtdcUserSessionField_FrontID_set)
    SessionID = property(_thostmduserapi.CThostFtdcUserSessionField_SessionID_get, _thostmduserapi.CThostFtdcUserSessionField_SessionID_set)
    BrokerID = property(_thostmduserapi.CThostFtdcUserSessionField_BrokerID_get, _thostmduserapi.CThostFtdcUserSessionField_BrokerID_set)
    UserID = property(_thostmduserapi.CThostFtdcUserSessionField_UserID_get, _thostmduserapi.CThostFtdcUserSessionField_UserID_set)
    LoginDate = property(_thostmduserapi.CThostFtdcUserSessionField_LoginDate_get, _thostmduserapi.CThostFtdcUserSessionField_LoginDate_set)
    LoginTime = property(_thostmduserapi.CThostFtdcUserSessionField_LoginTime_get, _thostmduserapi.CThostFtdcUserSessionField_LoginTime_set)
    IPAddress = property(_thostmduserapi.CThostFtdcUserSessionField_IPAddress_get, _thostmduserapi.CThostFtdcUserSessionField_IPAddress_set)
    UserProductInfo = property(_thostmduserapi.CThostFtdcUserSessionField_UserProductInfo_get, _thostmduserapi.CThostFtdcUserSessionField_UserProductInfo_set)
    InterfaceProductInfo = property(_thostmduserapi.CThostFtdcUserSessionField_InterfaceProductInfo_get, _thostmduserapi.CThostFtdcUserSessionField_InterfaceProductInfo_set)
    ProtocolInfo = property(_thostmduserapi.CThostFtdcUserSessionField_ProtocolInfo_get, _thostmduserapi.CThostFtdcUserSessionField_ProtocolInfo_set)
    MacAddress = property(_thostmduserapi.CThostFtdcUserSessionField_MacAddress_get, _thostmduserapi.CThostFtdcUserSessionField_MacAddress_set)
    LoginRemark = property(_thostmduserapi.CThostFtdcUserSessionField_LoginRemark_get, _thostmduserapi.CThostFtdcUserSessionField_LoginRemark_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcUserSessionField_swiginit(self, _thostmduserapi.new_CThostFtdcUserSessionField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcUserSessionField

# Register CThostFtdcUserSessionField in _thostmduserapi:
_thostmduserapi.CThostFtdcUserSessionField_swigregister(CThostFtdcUserSessionField)

class CThostFtdcQueryMaxOrderVolumeField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQueryMaxOrderVolumeField_BrokerID_get, _thostmduserapi.CThostFtdcQueryMaxOrderVolumeField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcQueryMaxOrderVolumeField_InvestorID_get, _thostmduserapi.CThostFtdcQueryMaxOrderVolumeField_InvestorID_set)
    InstrumentID = property(_thostmduserapi.CThostFtdcQueryMaxOrderVolumeField_InstrumentID_get, _thostmduserapi.CThostFtdcQueryMaxOrderVolumeField_InstrumentID_set)
    Direction = property(_thostmduserapi.CThostFtdcQueryMaxOrderVolumeField_Direction_get, _thostmduserapi.CThostFtdcQueryMaxOrderVolumeField_Direction_set)
    OffsetFlag = property(_thostmduserapi.CThostFtdcQueryMaxOrderVolumeField_OffsetFlag_get, _thostmduserapi.CThostFtdcQueryMaxOrderVolumeField_OffsetFlag_set)
    HedgeFlag = property(_thostmduserapi.CThostFtdcQueryMaxOrderVolumeField_HedgeFlag_get, _thostmduserapi.CThostFtdcQueryMaxOrderVolumeField_HedgeFlag_set)
    MaxVolume = property(_thostmduserapi.CThostFtdcQueryMaxOrderVolumeField_MaxVolume_get, _thostmduserapi.CThostFtdcQueryMaxOrderVolumeField_MaxVolume_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcQueryMaxOrderVolumeField_ExchangeID_get, _thostmduserapi.CThostFtdcQueryMaxOrderVolumeField_ExchangeID_set)
    InvestUnitID = property(_thostmduserapi.CThostFtdcQueryMaxOrderVolumeField_InvestUnitID_get, _thostmduserapi.CThostFtdcQueryMaxOrderVolumeField_InvestUnitID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQueryMaxOrderVolumeField_swiginit(self, _thostmduserapi.new_CThostFtdcQueryMaxOrderVolumeField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQueryMaxOrderVolumeField

# Register CThostFtdcQueryMaxOrderVolumeField in _thostmduserapi:
_thostmduserapi.CThostFtdcQueryMaxOrderVolumeField_swigregister(CThostFtdcQueryMaxOrderVolumeField)

class CThostFtdcSettlementInfoConfirmField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcSettlementInfoConfirmField_BrokerID_get, _thostmduserapi.CThostFtdcSettlementInfoConfirmField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcSettlementInfoConfirmField_InvestorID_get, _thostmduserapi.CThostFtdcSettlementInfoConfirmField_InvestorID_set)
    ConfirmDate = property(_thostmduserapi.CThostFtdcSettlementInfoConfirmField_ConfirmDate_get, _thostmduserapi.CThostFtdcSettlementInfoConfirmField_ConfirmDate_set)
    ConfirmTime = property(_thostmduserapi.CThostFtdcSettlementInfoConfirmField_ConfirmTime_get, _thostmduserapi.CThostFtdcSettlementInfoConfirmField_ConfirmTime_set)
    SettlementID = property(_thostmduserapi.CThostFtdcSettlementInfoConfirmField_SettlementID_get, _thostmduserapi.CThostFtdcSettlementInfoConfirmField_SettlementID_set)
    AccountID = property(_thostmduserapi.CThostFtdcSettlementInfoConfirmField_AccountID_get, _thostmduserapi.CThostFtdcSettlementInfoConfirmField_AccountID_set)
    CurrencyID = property(_thostmduserapi.CThostFtdcSettlementInfoConfirmField_CurrencyID_get, _thostmduserapi.CThostFtdcSettlementInfoConfirmField_CurrencyID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcSettlementInfoConfirmField_swiginit(self, _thostmduserapi.new_CThostFtdcSettlementInfoConfirmField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcSettlementInfoConfirmField

# Register CThostFtdcSettlementInfoConfirmField in _thostmduserapi:
_thostmduserapi.CThostFtdcSettlementInfoConfirmField_swigregister(CThostFtdcSettlementInfoConfirmField)

class CThostFtdcSyncDepositField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    DepositSeqNo = property(_thostmduserapi.CThostFtdcSyncDepositField_DepositSeqNo_get, _thostmduserapi.CThostFtdcSyncDepositField_DepositSeqNo_set)
    BrokerID = property(_thostmduserapi.CThostFtdcSyncDepositField_BrokerID_get, _thostmduserapi.CThostFtdcSyncDepositField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcSyncDepositField_InvestorID_get, _thostmduserapi.CThostFtdcSyncDepositField_InvestorID_set)
    Deposit = property(_thostmduserapi.CThostFtdcSyncDepositField_Deposit_get, _thostmduserapi.CThostFtdcSyncDepositField_Deposit_set)
    IsForce = property(_thostmduserapi.CThostFtdcSyncDepositField_IsForce_get, _thostmduserapi.CThostFtdcSyncDepositField_IsForce_set)
    CurrencyID = property(_thostmduserapi.CThostFtdcSyncDepositField_CurrencyID_get, _thostmduserapi.CThostFtdcSyncDepositField_CurrencyID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcSyncDepositField_swiginit(self, _thostmduserapi.new_CThostFtdcSyncDepositField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcSyncDepositField

# Register CThostFtdcSyncDepositField in _thostmduserapi:
_thostmduserapi.CThostFtdcSyncDepositField_swigregister(CThostFtdcSyncDepositField)

class CThostFtdcSyncFundMortgageField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    MortgageSeqNo = property(_thostmduserapi.CThostFtdcSyncFundMortgageField_MortgageSeqNo_get, _thostmduserapi.CThostFtdcSyncFundMortgageField_MortgageSeqNo_set)
    BrokerID = property(_thostmduserapi.CThostFtdcSyncFundMortgageField_BrokerID_get, _thostmduserapi.CThostFtdcSyncFundMortgageField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcSyncFundMortgageField_InvestorID_get, _thostmduserapi.CThostFtdcSyncFundMortgageField_InvestorID_set)
    FromCurrencyID = property(_thostmduserapi.CThostFtdcSyncFundMortgageField_FromCurrencyID_get, _thostmduserapi.CThostFtdcSyncFundMortgageField_FromCurrencyID_set)
    MortgageAmount = property(_thostmduserapi.CThostFtdcSyncFundMortgageField_MortgageAmount_get, _thostmduserapi.CThostFtdcSyncFundMortgageField_MortgageAmount_set)
    ToCurrencyID = property(_thostmduserapi.CThostFtdcSyncFundMortgageField_ToCurrencyID_get, _thostmduserapi.CThostFtdcSyncFundMortgageField_ToCurrencyID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcSyncFundMortgageField_swiginit(self, _thostmduserapi.new_CThostFtdcSyncFundMortgageField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcSyncFundMortgageField

# Register CThostFtdcSyncFundMortgageField in _thostmduserapi:
_thostmduserapi.CThostFtdcSyncFundMortgageField_swigregister(CThostFtdcSyncFundMortgageField)

class CThostFtdcBrokerSyncField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcBrokerSyncField_BrokerID_get, _thostmduserapi.CThostFtdcBrokerSyncField_BrokerID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcBrokerSyncField_swiginit(self, _thostmduserapi.new_CThostFtdcBrokerSyncField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcBrokerSyncField

# Register CThostFtdcBrokerSyncField in _thostmduserapi:
_thostmduserapi.CThostFtdcBrokerSyncField_swigregister(CThostFtdcBrokerSyncField)

class CThostFtdcSyncingInvestorField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    InvestorID = property(_thostmduserapi.CThostFtdcSyncingInvestorField_InvestorID_get, _thostmduserapi.CThostFtdcSyncingInvestorField_InvestorID_set)
    BrokerID = property(_thostmduserapi.CThostFtdcSyncingInvestorField_BrokerID_get, _thostmduserapi.CThostFtdcSyncingInvestorField_BrokerID_set)
    InvestorGroupID = property(_thostmduserapi.CThostFtdcSyncingInvestorField_InvestorGroupID_get, _thostmduserapi.CThostFtdcSyncingInvestorField_InvestorGroupID_set)
    InvestorName = property(_thostmduserapi.CThostFtdcSyncingInvestorField_InvestorName_get, _thostmduserapi.CThostFtdcSyncingInvestorField_InvestorName_set)
    IdentifiedCardType = property(_thostmduserapi.CThostFtdcSyncingInvestorField_IdentifiedCardType_get, _thostmduserapi.CThostFtdcSyncingInvestorField_IdentifiedCardType_set)
    IdentifiedCardNo = property(_thostmduserapi.CThostFtdcSyncingInvestorField_IdentifiedCardNo_get, _thostmduserapi.CThostFtdcSyncingInvestorField_IdentifiedCardNo_set)
    IsActive = property(_thostmduserapi.CThostFtdcSyncingInvestorField_IsActive_get, _thostmduserapi.CThostFtdcSyncingInvestorField_IsActive_set)
    Telephone = property(_thostmduserapi.CThostFtdcSyncingInvestorField_Telephone_get, _thostmduserapi.CThostFtdcSyncingInvestorField_Telephone_set)
    Address = property(_thostmduserapi.CThostFtdcSyncingInvestorField_Address_get, _thostmduserapi.CThostFtdcSyncingInvestorField_Address_set)
    OpenDate = property(_thostmduserapi.CThostFtdcSyncingInvestorField_OpenDate_get, _thostmduserapi.CThostFtdcSyncingInvestorField_OpenDate_set)
    Mobile = property(_thostmduserapi.CThostFtdcSyncingInvestorField_Mobile_get, _thostmduserapi.CThostFtdcSyncingInvestorField_Mobile_set)
    CommModelID = property(_thostmduserapi.CThostFtdcSyncingInvestorField_CommModelID_get, _thostmduserapi.CThostFtdcSyncingInvestorField_CommModelID_set)
    MarginModelID = property(_thostmduserapi.CThostFtdcSyncingInvestorField_MarginModelID_get, _thostmduserapi.CThostFtdcSyncingInvestorField_MarginModelID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcSyncingInvestorField_swiginit(self, _thostmduserapi.new_CThostFtdcSyncingInvestorField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcSyncingInvestorField

# Register CThostFtdcSyncingInvestorField in _thostmduserapi:
_thostmduserapi.CThostFtdcSyncingInvestorField_swigregister(CThostFtdcSyncingInvestorField)

class CThostFtdcSyncingTradingCodeField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    InvestorID = property(_thostmduserapi.CThostFtdcSyncingTradingCodeField_InvestorID_get, _thostmduserapi.CThostFtdcSyncingTradingCodeField_InvestorID_set)
    BrokerID = property(_thostmduserapi.CThostFtdcSyncingTradingCodeField_BrokerID_get, _thostmduserapi.CThostFtdcSyncingTradingCodeField_BrokerID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcSyncingTradingCodeField_ExchangeID_get, _thostmduserapi.CThostFtdcSyncingTradingCodeField_ExchangeID_set)
    ClientID = property(_thostmduserapi.CThostFtdcSyncingTradingCodeField_ClientID_get, _thostmduserapi.CThostFtdcSyncingTradingCodeField_ClientID_set)
    IsActive = property(_thostmduserapi.CThostFtdcSyncingTradingCodeField_IsActive_get, _thostmduserapi.CThostFtdcSyncingTradingCodeField_IsActive_set)
    ClientIDType = property(_thostmduserapi.CThostFtdcSyncingTradingCodeField_ClientIDType_get, _thostmduserapi.CThostFtdcSyncingTradingCodeField_ClientIDType_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcSyncingTradingCodeField_swiginit(self, _thostmduserapi.new_CThostFtdcSyncingTradingCodeField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcSyncingTradingCodeField

# Register CThostFtdcSyncingTradingCodeField in _thostmduserapi:
_thostmduserapi.CThostFtdcSyncingTradingCodeField_swigregister(CThostFtdcSyncingTradingCodeField)

class CThostFtdcSyncingInvestorGroupField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcSyncingInvestorGroupField_BrokerID_get, _thostmduserapi.CThostFtdcSyncingInvestorGroupField_BrokerID_set)
    InvestorGroupID = property(_thostmduserapi.CThostFtdcSyncingInvestorGroupField_InvestorGroupID_get, _thostmduserapi.CThostFtdcSyncingInvestorGroupField_InvestorGroupID_set)
    InvestorGroupName = property(_thostmduserapi.CThostFtdcSyncingInvestorGroupField_InvestorGroupName_get, _thostmduserapi.CThostFtdcSyncingInvestorGroupField_InvestorGroupName_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcSyncingInvestorGroupField_swiginit(self, _thostmduserapi.new_CThostFtdcSyncingInvestorGroupField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcSyncingInvestorGroupField

# Register CThostFtdcSyncingInvestorGroupField in _thostmduserapi:
_thostmduserapi.CThostFtdcSyncingInvestorGroupField_swigregister(CThostFtdcSyncingInvestorGroupField)

class CThostFtdcSyncingTradingAccountField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcSyncingTradingAccountField_BrokerID_get, _thostmduserapi.CThostFtdcSyncingTradingAccountField_BrokerID_set)
    AccountID = property(_thostmduserapi.CThostFtdcSyncingTradingAccountField_AccountID_get, _thostmduserapi.CThostFtdcSyncingTradingAccountField_AccountID_set)
    PreMortgage = property(_thostmduserapi.CThostFtdcSyncingTradingAccountField_PreMortgage_get, _thostmduserapi.CThostFtdcSyncingTradingAccountField_PreMortgage_set)
    PreCredit = property(_thostmduserapi.CThostFtdcSyncingTradingAccountField_PreCredit_get, _thostmduserapi.CThostFtdcSyncingTradingAccountField_PreCredit_set)
    PreDeposit = property(_thostmduserapi.CThostFtdcSyncingTradingAccountField_PreDeposit_get, _thostmduserapi.CThostFtdcSyncingTradingAccountField_PreDeposit_set)
    PreBalance = property(_thostmduserapi.CThostFtdcSyncingTradingAccountField_PreBalance_get, _thostmduserapi.CThostFtdcSyncingTradingAccountField_PreBalance_set)
    PreMargin = property(_thostmduserapi.CThostFtdcSyncingTradingAccountField_PreMargin_get, _thostmduserapi.CThostFtdcSyncingTradingAccountField_PreMargin_set)
    InterestBase = property(_thostmduserapi.CThostFtdcSyncingTradingAccountField_InterestBase_get, _thostmduserapi.CThostFtdcSyncingTradingAccountField_InterestBase_set)
    Interest = property(_thostmduserapi.CThostFtdcSyncingTradingAccountField_Interest_get, _thostmduserapi.CThostFtdcSyncingTradingAccountField_Interest_set)
    Deposit = property(_thostmduserapi.CThostFtdcSyncingTradingAccountField_Deposit_get, _thostmduserapi.CThostFtdcSyncingTradingAccountField_Deposit_set)
    Withdraw = property(_thostmduserapi.CThostFtdcSyncingTradingAccountField_Withdraw_get, _thostmduserapi.CThostFtdcSyncingTradingAccountField_Withdraw_set)
    FrozenMargin = property(_thostmduserapi.CThostFtdcSyncingTradingAccountField_FrozenMargin_get, _thostmduserapi.CThostFtdcSyncingTradingAccountField_FrozenMargin_set)
    FrozenCash = property(_thostmduserapi.CThostFtdcSyncingTradingAccountField_FrozenCash_get, _thostmduserapi.CThostFtdcSyncingTradingAccountField_FrozenCash_set)
    FrozenCommission = property(_thostmduserapi.CThostFtdcSyncingTradingAccountField_FrozenCommission_get, _thostmduserapi.CThostFtdcSyncingTradingAccountField_FrozenCommission_set)
    CurrMargin = property(_thostmduserapi.CThostFtdcSyncingTradingAccountField_CurrMargin_get, _thostmduserapi.CThostFtdcSyncingTradingAccountField_CurrMargin_set)
    CashIn = property(_thostmduserapi.CThostFtdcSyncingTradingAccountField_CashIn_get, _thostmduserapi.CThostFtdcSyncingTradingAccountField_CashIn_set)
    Commission = property(_thostmduserapi.CThostFtdcSyncingTradingAccountField_Commission_get, _thostmduserapi.CThostFtdcSyncingTradingAccountField_Commission_set)
    CloseProfit = property(_thostmduserapi.CThostFtdcSyncingTradingAccountField_CloseProfit_get, _thostmduserapi.CThostFtdcSyncingTradingAccountField_CloseProfit_set)
    PositionProfit = property(_thostmduserapi.CThostFtdcSyncingTradingAccountField_PositionProfit_get, _thostmduserapi.CThostFtdcSyncingTradingAccountField_PositionProfit_set)
    Balance = property(_thostmduserapi.CThostFtdcSyncingTradingAccountField_Balance_get, _thostmduserapi.CThostFtdcSyncingTradingAccountField_Balance_set)
    Available = property(_thostmduserapi.CThostFtdcSyncingTradingAccountField_Available_get, _thostmduserapi.CThostFtdcSyncingTradingAccountField_Available_set)
    WithdrawQuota = property(_thostmduserapi.CThostFtdcSyncingTradingAccountField_WithdrawQuota_get, _thostmduserapi.CThostFtdcSyncingTradingAccountField_WithdrawQuota_set)
    Reserve = property(_thostmduserapi.CThostFtdcSyncingTradingAccountField_Reserve_get, _thostmduserapi.CThostFtdcSyncingTradingAccountField_Reserve_set)
    TradingDay = property(_thostmduserapi.CThostFtdcSyncingTradingAccountField_TradingDay_get, _thostmduserapi.CThostFtdcSyncingTradingAccountField_TradingDay_set)
    SettlementID = property(_thostmduserapi.CThostFtdcSyncingTradingAccountField_SettlementID_get, _thostmduserapi.CThostFtdcSyncingTradingAccountField_SettlementID_set)
    Credit = property(_thostmduserapi.CThostFtdcSyncingTradingAccountField_Credit_get, _thostmduserapi.CThostFtdcSyncingTradingAccountField_Credit_set)
    Mortgage = property(_thostmduserapi.CThostFtdcSyncingTradingAccountField_Mortgage_get, _thostmduserapi.CThostFtdcSyncingTradingAccountField_Mortgage_set)
    ExchangeMargin = property(_thostmduserapi.CThostFtdcSyncingTradingAccountField_ExchangeMargin_get, _thostmduserapi.CThostFtdcSyncingTradingAccountField_ExchangeMargin_set)
    DeliveryMargin = property(_thostmduserapi.CThostFtdcSyncingTradingAccountField_DeliveryMargin_get, _thostmduserapi.CThostFtdcSyncingTradingAccountField_DeliveryMargin_set)
    ExchangeDeliveryMargin = property(_thostmduserapi.CThostFtdcSyncingTradingAccountField_ExchangeDeliveryMargin_get, _thostmduserapi.CThostFtdcSyncingTradingAccountField_ExchangeDeliveryMargin_set)
    ReserveBalance = property(_thostmduserapi.CThostFtdcSyncingTradingAccountField_ReserveBalance_get, _thostmduserapi.CThostFtdcSyncingTradingAccountField_ReserveBalance_set)
    CurrencyID = property(_thostmduserapi.CThostFtdcSyncingTradingAccountField_CurrencyID_get, _thostmduserapi.CThostFtdcSyncingTradingAccountField_CurrencyID_set)
    PreFundMortgageIn = property(_thostmduserapi.CThostFtdcSyncingTradingAccountField_PreFundMortgageIn_get, _thostmduserapi.CThostFtdcSyncingTradingAccountField_PreFundMortgageIn_set)
    PreFundMortgageOut = property(_thostmduserapi.CThostFtdcSyncingTradingAccountField_PreFundMortgageOut_get, _thostmduserapi.CThostFtdcSyncingTradingAccountField_PreFundMortgageOut_set)
    FundMortgageIn = property(_thostmduserapi.CThostFtdcSyncingTradingAccountField_FundMortgageIn_get, _thostmduserapi.CThostFtdcSyncingTradingAccountField_FundMortgageIn_set)
    FundMortgageOut = property(_thostmduserapi.CThostFtdcSyncingTradingAccountField_FundMortgageOut_get, _thostmduserapi.CThostFtdcSyncingTradingAccountField_FundMortgageOut_set)
    FundMortgageAvailable = property(_thostmduserapi.CThostFtdcSyncingTradingAccountField_FundMortgageAvailable_get, _thostmduserapi.CThostFtdcSyncingTradingAccountField_FundMortgageAvailable_set)
    MortgageableFund = property(_thostmduserapi.CThostFtdcSyncingTradingAccountField_MortgageableFund_get, _thostmduserapi.CThostFtdcSyncingTradingAccountField_MortgageableFund_set)
    SpecProductMargin = property(_thostmduserapi.CThostFtdcSyncingTradingAccountField_SpecProductMargin_get, _thostmduserapi.CThostFtdcSyncingTradingAccountField_SpecProductMargin_set)
    SpecProductFrozenMargin = property(_thostmduserapi.CThostFtdcSyncingTradingAccountField_SpecProductFrozenMargin_get, _thostmduserapi.CThostFtdcSyncingTradingAccountField_SpecProductFrozenMargin_set)
    SpecProductCommission = property(_thostmduserapi.CThostFtdcSyncingTradingAccountField_SpecProductCommission_get, _thostmduserapi.CThostFtdcSyncingTradingAccountField_SpecProductCommission_set)
    SpecProductFrozenCommission = property(_thostmduserapi.CThostFtdcSyncingTradingAccountField_SpecProductFrozenCommission_get, _thostmduserapi.CThostFtdcSyncingTradingAccountField_SpecProductFrozenCommission_set)
    SpecProductPositionProfit = property(_thostmduserapi.CThostFtdcSyncingTradingAccountField_SpecProductPositionProfit_get, _thostmduserapi.CThostFtdcSyncingTradingAccountField_SpecProductPositionProfit_set)
    SpecProductCloseProfit = property(_thostmduserapi.CThostFtdcSyncingTradingAccountField_SpecProductCloseProfit_get, _thostmduserapi.CThostFtdcSyncingTradingAccountField_SpecProductCloseProfit_set)
    SpecProductPositionProfitByAlg = property(_thostmduserapi.CThostFtdcSyncingTradingAccountField_SpecProductPositionProfitByAlg_get, _thostmduserapi.CThostFtdcSyncingTradingAccountField_SpecProductPositionProfitByAlg_set)
    SpecProductExchangeMargin = property(_thostmduserapi.CThostFtdcSyncingTradingAccountField_SpecProductExchangeMargin_get, _thostmduserapi.CThostFtdcSyncingTradingAccountField_SpecProductExchangeMargin_set)
    FrozenSwap = property(_thostmduserapi.CThostFtdcSyncingTradingAccountField_FrozenSwap_get, _thostmduserapi.CThostFtdcSyncingTradingAccountField_FrozenSwap_set)
    RemainSwap = property(_thostmduserapi.CThostFtdcSyncingTradingAccountField_RemainSwap_get, _thostmduserapi.CThostFtdcSyncingTradingAccountField_RemainSwap_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcSyncingTradingAccountField_swiginit(self, _thostmduserapi.new_CThostFtdcSyncingTradingAccountField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcSyncingTradingAccountField

# Register CThostFtdcSyncingTradingAccountField in _thostmduserapi:
_thostmduserapi.CThostFtdcSyncingTradingAccountField_swigregister(CThostFtdcSyncingTradingAccountField)

class CThostFtdcSyncingInvestorPositionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    InstrumentID = property(_thostmduserapi.CThostFtdcSyncingInvestorPositionField_InstrumentID_get, _thostmduserapi.CThostFtdcSyncingInvestorPositionField_InstrumentID_set)
    BrokerID = property(_thostmduserapi.CThostFtdcSyncingInvestorPositionField_BrokerID_get, _thostmduserapi.CThostFtdcSyncingInvestorPositionField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcSyncingInvestorPositionField_InvestorID_get, _thostmduserapi.CThostFtdcSyncingInvestorPositionField_InvestorID_set)
    PosiDirection = property(_thostmduserapi.CThostFtdcSyncingInvestorPositionField_PosiDirection_get, _thostmduserapi.CThostFtdcSyncingInvestorPositionField_PosiDirection_set)
    HedgeFlag = property(_thostmduserapi.CThostFtdcSyncingInvestorPositionField_HedgeFlag_get, _thostmduserapi.CThostFtdcSyncingInvestorPositionField_HedgeFlag_set)
    PositionDate = property(_thostmduserapi.CThostFtdcSyncingInvestorPositionField_PositionDate_get, _thostmduserapi.CThostFtdcSyncingInvestorPositionField_PositionDate_set)
    YdPosition = property(_thostmduserapi.CThostFtdcSyncingInvestorPositionField_YdPosition_get, _thostmduserapi.CThostFtdcSyncingInvestorPositionField_YdPosition_set)
    Position = property(_thostmduserapi.CThostFtdcSyncingInvestorPositionField_Position_get, _thostmduserapi.CThostFtdcSyncingInvestorPositionField_Position_set)
    LongFrozen = property(_thostmduserapi.CThostFtdcSyncingInvestorPositionField_LongFrozen_get, _thostmduserapi.CThostFtdcSyncingInvestorPositionField_LongFrozen_set)
    ShortFrozen = property(_thostmduserapi.CThostFtdcSyncingInvestorPositionField_ShortFrozen_get, _thostmduserapi.CThostFtdcSyncingInvestorPositionField_ShortFrozen_set)
    LongFrozenAmount = property(_thostmduserapi.CThostFtdcSyncingInvestorPositionField_LongFrozenAmount_get, _thostmduserapi.CThostFtdcSyncingInvestorPositionField_LongFrozenAmount_set)
    ShortFrozenAmount = property(_thostmduserapi.CThostFtdcSyncingInvestorPositionField_ShortFrozenAmount_get, _thostmduserapi.CThostFtdcSyncingInvestorPositionField_ShortFrozenAmount_set)
    OpenVolume = property(_thostmduserapi.CThostFtdcSyncingInvestorPositionField_OpenVolume_get, _thostmduserapi.CThostFtdcSyncingInvestorPositionField_OpenVolume_set)
    CloseVolume = property(_thostmduserapi.CThostFtdcSyncingInvestorPositionField_CloseVolume_get, _thostmduserapi.CThostFtdcSyncingInvestorPositionField_CloseVolume_set)
    OpenAmount = property(_thostmduserapi.CThostFtdcSyncingInvestorPositionField_OpenAmount_get, _thostmduserapi.CThostFtdcSyncingInvestorPositionField_OpenAmount_set)
    CloseAmount = property(_thostmduserapi.CThostFtdcSyncingInvestorPositionField_CloseAmount_get, _thostmduserapi.CThostFtdcSyncingInvestorPositionField_CloseAmount_set)
    PositionCost = property(_thostmduserapi.CThostFtdcSyncingInvestorPositionField_PositionCost_get, _thostmduserapi.CThostFtdcSyncingInvestorPositionField_PositionCost_set)
    PreMargin = property(_thostmduserapi.CThostFtdcSyncingInvestorPositionField_PreMargin_get, _thostmduserapi.CThostFtdcSyncingInvestorPositionField_PreMargin_set)
    UseMargin = property(_thostmduserapi.CThostFtdcSyncingInvestorPositionField_UseMargin_get, _thostmduserapi.CThostFtdcSyncingInvestorPositionField_UseMargin_set)
    FrozenMargin = property(_thostmduserapi.CThostFtdcSyncingInvestorPositionField_FrozenMargin_get, _thostmduserapi.CThostFtdcSyncingInvestorPositionField_FrozenMargin_set)
    FrozenCash = property(_thostmduserapi.CThostFtdcSyncingInvestorPositionField_FrozenCash_get, _thostmduserapi.CThostFtdcSyncingInvestorPositionField_FrozenCash_set)
    FrozenCommission = property(_thostmduserapi.CThostFtdcSyncingInvestorPositionField_FrozenCommission_get, _thostmduserapi.CThostFtdcSyncingInvestorPositionField_FrozenCommission_set)
    CashIn = property(_thostmduserapi.CThostFtdcSyncingInvestorPositionField_CashIn_get, _thostmduserapi.CThostFtdcSyncingInvestorPositionField_CashIn_set)
    Commission = property(_thostmduserapi.CThostFtdcSyncingInvestorPositionField_Commission_get, _thostmduserapi.CThostFtdcSyncingInvestorPositionField_Commission_set)
    CloseProfit = property(_thostmduserapi.CThostFtdcSyncingInvestorPositionField_CloseProfit_get, _thostmduserapi.CThostFtdcSyncingInvestorPositionField_CloseProfit_set)
    PositionProfit = property(_thostmduserapi.CThostFtdcSyncingInvestorPositionField_PositionProfit_get, _thostmduserapi.CThostFtdcSyncingInvestorPositionField_PositionProfit_set)
    PreSettlementPrice = property(_thostmduserapi.CThostFtdcSyncingInvestorPositionField_PreSettlementPrice_get, _thostmduserapi.CThostFtdcSyncingInvestorPositionField_PreSettlementPrice_set)
    SettlementPrice = property(_thostmduserapi.CThostFtdcSyncingInvestorPositionField_SettlementPrice_get, _thostmduserapi.CThostFtdcSyncingInvestorPositionField_SettlementPrice_set)
    TradingDay = property(_thostmduserapi.CThostFtdcSyncingInvestorPositionField_TradingDay_get, _thostmduserapi.CThostFtdcSyncingInvestorPositionField_TradingDay_set)
    SettlementID = property(_thostmduserapi.CThostFtdcSyncingInvestorPositionField_SettlementID_get, _thostmduserapi.CThostFtdcSyncingInvestorPositionField_SettlementID_set)
    OpenCost = property(_thostmduserapi.CThostFtdcSyncingInvestorPositionField_OpenCost_get, _thostmduserapi.CThostFtdcSyncingInvestorPositionField_OpenCost_set)
    ExchangeMargin = property(_thostmduserapi.CThostFtdcSyncingInvestorPositionField_ExchangeMargin_get, _thostmduserapi.CThostFtdcSyncingInvestorPositionField_ExchangeMargin_set)
    CombPosition = property(_thostmduserapi.CThostFtdcSyncingInvestorPositionField_CombPosition_get, _thostmduserapi.CThostFtdcSyncingInvestorPositionField_CombPosition_set)
    CombLongFrozen = property(_thostmduserapi.CThostFtdcSyncingInvestorPositionField_CombLongFrozen_get, _thostmduserapi.CThostFtdcSyncingInvestorPositionField_CombLongFrozen_set)
    CombShortFrozen = property(_thostmduserapi.CThostFtdcSyncingInvestorPositionField_CombShortFrozen_get, _thostmduserapi.CThostFtdcSyncingInvestorPositionField_CombShortFrozen_set)
    CloseProfitByDate = property(_thostmduserapi.CThostFtdcSyncingInvestorPositionField_CloseProfitByDate_get, _thostmduserapi.CThostFtdcSyncingInvestorPositionField_CloseProfitByDate_set)
    CloseProfitByTrade = property(_thostmduserapi.CThostFtdcSyncingInvestorPositionField_CloseProfitByTrade_get, _thostmduserapi.CThostFtdcSyncingInvestorPositionField_CloseProfitByTrade_set)
    TodayPosition = property(_thostmduserapi.CThostFtdcSyncingInvestorPositionField_TodayPosition_get, _thostmduserapi.CThostFtdcSyncingInvestorPositionField_TodayPosition_set)
    MarginRateByMoney = property(_thostmduserapi.CThostFtdcSyncingInvestorPositionField_MarginRateByMoney_get, _thostmduserapi.CThostFtdcSyncingInvestorPositionField_MarginRateByMoney_set)
    MarginRateByVolume = property(_thostmduserapi.CThostFtdcSyncingInvestorPositionField_MarginRateByVolume_get, _thostmduserapi.CThostFtdcSyncingInvestorPositionField_MarginRateByVolume_set)
    StrikeFrozen = property(_thostmduserapi.CThostFtdcSyncingInvestorPositionField_StrikeFrozen_get, _thostmduserapi.CThostFtdcSyncingInvestorPositionField_StrikeFrozen_set)
    StrikeFrozenAmount = property(_thostmduserapi.CThostFtdcSyncingInvestorPositionField_StrikeFrozenAmount_get, _thostmduserapi.CThostFtdcSyncingInvestorPositionField_StrikeFrozenAmount_set)
    AbandonFrozen = property(_thostmduserapi.CThostFtdcSyncingInvestorPositionField_AbandonFrozen_get, _thostmduserapi.CThostFtdcSyncingInvestorPositionField_AbandonFrozen_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcSyncingInvestorPositionField_ExchangeID_get, _thostmduserapi.CThostFtdcSyncingInvestorPositionField_ExchangeID_set)
    YdStrikeFrozen = property(_thostmduserapi.CThostFtdcSyncingInvestorPositionField_YdStrikeFrozen_get, _thostmduserapi.CThostFtdcSyncingInvestorPositionField_YdStrikeFrozen_set)
    InvestUnitID = property(_thostmduserapi.CThostFtdcSyncingInvestorPositionField_InvestUnitID_get, _thostmduserapi.CThostFtdcSyncingInvestorPositionField_InvestUnitID_set)
    PositionCostOffset = property(_thostmduserapi.CThostFtdcSyncingInvestorPositionField_PositionCostOffset_get, _thostmduserapi.CThostFtdcSyncingInvestorPositionField_PositionCostOffset_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcSyncingInvestorPositionField_swiginit(self, _thostmduserapi.new_CThostFtdcSyncingInvestorPositionField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcSyncingInvestorPositionField

# Register CThostFtdcSyncingInvestorPositionField in _thostmduserapi:
_thostmduserapi.CThostFtdcSyncingInvestorPositionField_swigregister(CThostFtdcSyncingInvestorPositionField)

class CThostFtdcSyncingInstrumentMarginRateField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    InstrumentID = property(_thostmduserapi.CThostFtdcSyncingInstrumentMarginRateField_InstrumentID_get, _thostmduserapi.CThostFtdcSyncingInstrumentMarginRateField_InstrumentID_set)
    InvestorRange = property(_thostmduserapi.CThostFtdcSyncingInstrumentMarginRateField_InvestorRange_get, _thostmduserapi.CThostFtdcSyncingInstrumentMarginRateField_InvestorRange_set)
    BrokerID = property(_thostmduserapi.CThostFtdcSyncingInstrumentMarginRateField_BrokerID_get, _thostmduserapi.CThostFtdcSyncingInstrumentMarginRateField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcSyncingInstrumentMarginRateField_InvestorID_get, _thostmduserapi.CThostFtdcSyncingInstrumentMarginRateField_InvestorID_set)
    HedgeFlag = property(_thostmduserapi.CThostFtdcSyncingInstrumentMarginRateField_HedgeFlag_get, _thostmduserapi.CThostFtdcSyncingInstrumentMarginRateField_HedgeFlag_set)
    LongMarginRatioByMoney = property(_thostmduserapi.CThostFtdcSyncingInstrumentMarginRateField_LongMarginRatioByMoney_get, _thostmduserapi.CThostFtdcSyncingInstrumentMarginRateField_LongMarginRatioByMoney_set)
    LongMarginRatioByVolume = property(_thostmduserapi.CThostFtdcSyncingInstrumentMarginRateField_LongMarginRatioByVolume_get, _thostmduserapi.CThostFtdcSyncingInstrumentMarginRateField_LongMarginRatioByVolume_set)
    ShortMarginRatioByMoney = property(_thostmduserapi.CThostFtdcSyncingInstrumentMarginRateField_ShortMarginRatioByMoney_get, _thostmduserapi.CThostFtdcSyncingInstrumentMarginRateField_ShortMarginRatioByMoney_set)
    ShortMarginRatioByVolume = property(_thostmduserapi.CThostFtdcSyncingInstrumentMarginRateField_ShortMarginRatioByVolume_get, _thostmduserapi.CThostFtdcSyncingInstrumentMarginRateField_ShortMarginRatioByVolume_set)
    IsRelative = property(_thostmduserapi.CThostFtdcSyncingInstrumentMarginRateField_IsRelative_get, _thostmduserapi.CThostFtdcSyncingInstrumentMarginRateField_IsRelative_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcSyncingInstrumentMarginRateField_swiginit(self, _thostmduserapi.new_CThostFtdcSyncingInstrumentMarginRateField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcSyncingInstrumentMarginRateField

# Register CThostFtdcSyncingInstrumentMarginRateField in _thostmduserapi:
_thostmduserapi.CThostFtdcSyncingInstrumentMarginRateField_swigregister(CThostFtdcSyncingInstrumentMarginRateField)

class CThostFtdcSyncingInstrumentCommissionRateField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    InstrumentID = property(_thostmduserapi.CThostFtdcSyncingInstrumentCommissionRateField_InstrumentID_get, _thostmduserapi.CThostFtdcSyncingInstrumentCommissionRateField_InstrumentID_set)
    InvestorRange = property(_thostmduserapi.CThostFtdcSyncingInstrumentCommissionRateField_InvestorRange_get, _thostmduserapi.CThostFtdcSyncingInstrumentCommissionRateField_InvestorRange_set)
    BrokerID = property(_thostmduserapi.CThostFtdcSyncingInstrumentCommissionRateField_BrokerID_get, _thostmduserapi.CThostFtdcSyncingInstrumentCommissionRateField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcSyncingInstrumentCommissionRateField_InvestorID_get, _thostmduserapi.CThostFtdcSyncingInstrumentCommissionRateField_InvestorID_set)
    OpenRatioByMoney = property(_thostmduserapi.CThostFtdcSyncingInstrumentCommissionRateField_OpenRatioByMoney_get, _thostmduserapi.CThostFtdcSyncingInstrumentCommissionRateField_OpenRatioByMoney_set)
    OpenRatioByVolume = property(_thostmduserapi.CThostFtdcSyncingInstrumentCommissionRateField_OpenRatioByVolume_get, _thostmduserapi.CThostFtdcSyncingInstrumentCommissionRateField_OpenRatioByVolume_set)
    CloseRatioByMoney = property(_thostmduserapi.CThostFtdcSyncingInstrumentCommissionRateField_CloseRatioByMoney_get, _thostmduserapi.CThostFtdcSyncingInstrumentCommissionRateField_CloseRatioByMoney_set)
    CloseRatioByVolume = property(_thostmduserapi.CThostFtdcSyncingInstrumentCommissionRateField_CloseRatioByVolume_get, _thostmduserapi.CThostFtdcSyncingInstrumentCommissionRateField_CloseRatioByVolume_set)
    CloseTodayRatioByMoney = property(_thostmduserapi.CThostFtdcSyncingInstrumentCommissionRateField_CloseTodayRatioByMoney_get, _thostmduserapi.CThostFtdcSyncingInstrumentCommissionRateField_CloseTodayRatioByMoney_set)
    CloseTodayRatioByVolume = property(_thostmduserapi.CThostFtdcSyncingInstrumentCommissionRateField_CloseTodayRatioByVolume_get, _thostmduserapi.CThostFtdcSyncingInstrumentCommissionRateField_CloseTodayRatioByVolume_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcSyncingInstrumentCommissionRateField_swiginit(self, _thostmduserapi.new_CThostFtdcSyncingInstrumentCommissionRateField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcSyncingInstrumentCommissionRateField

# Register CThostFtdcSyncingInstrumentCommissionRateField in _thostmduserapi:
_thostmduserapi.CThostFtdcSyncingInstrumentCommissionRateField_swigregister(CThostFtdcSyncingInstrumentCommissionRateField)

class CThostFtdcSyncingInstrumentTradingRightField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    InstrumentID = property(_thostmduserapi.CThostFtdcSyncingInstrumentTradingRightField_InstrumentID_get, _thostmduserapi.CThostFtdcSyncingInstrumentTradingRightField_InstrumentID_set)
    InvestorRange = property(_thostmduserapi.CThostFtdcSyncingInstrumentTradingRightField_InvestorRange_get, _thostmduserapi.CThostFtdcSyncingInstrumentTradingRightField_InvestorRange_set)
    BrokerID = property(_thostmduserapi.CThostFtdcSyncingInstrumentTradingRightField_BrokerID_get, _thostmduserapi.CThostFtdcSyncingInstrumentTradingRightField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcSyncingInstrumentTradingRightField_InvestorID_get, _thostmduserapi.CThostFtdcSyncingInstrumentTradingRightField_InvestorID_set)
    TradingRight = property(_thostmduserapi.CThostFtdcSyncingInstrumentTradingRightField_TradingRight_get, _thostmduserapi.CThostFtdcSyncingInstrumentTradingRightField_TradingRight_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcSyncingInstrumentTradingRightField_swiginit(self, _thostmduserapi.new_CThostFtdcSyncingInstrumentTradingRightField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcSyncingInstrumentTradingRightField

# Register CThostFtdcSyncingInstrumentTradingRightField in _thostmduserapi:
_thostmduserapi.CThostFtdcSyncingInstrumentTradingRightField_swigregister(CThostFtdcSyncingInstrumentTradingRightField)

class CThostFtdcQryOrderField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQryOrderField_BrokerID_get, _thostmduserapi.CThostFtdcQryOrderField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcQryOrderField_InvestorID_get, _thostmduserapi.CThostFtdcQryOrderField_InvestorID_set)
    InstrumentID = property(_thostmduserapi.CThostFtdcQryOrderField_InstrumentID_get, _thostmduserapi.CThostFtdcQryOrderField_InstrumentID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcQryOrderField_ExchangeID_get, _thostmduserapi.CThostFtdcQryOrderField_ExchangeID_set)
    OrderSysID = property(_thostmduserapi.CThostFtdcQryOrderField_OrderSysID_get, _thostmduserapi.CThostFtdcQryOrderField_OrderSysID_set)
    InsertTimeStart = property(_thostmduserapi.CThostFtdcQryOrderField_InsertTimeStart_get, _thostmduserapi.CThostFtdcQryOrderField_InsertTimeStart_set)
    InsertTimeEnd = property(_thostmduserapi.CThostFtdcQryOrderField_InsertTimeEnd_get, _thostmduserapi.CThostFtdcQryOrderField_InsertTimeEnd_set)
    InvestUnitID = property(_thostmduserapi.CThostFtdcQryOrderField_InvestUnitID_get, _thostmduserapi.CThostFtdcQryOrderField_InvestUnitID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryOrderField_swiginit(self, _thostmduserapi.new_CThostFtdcQryOrderField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryOrderField

# Register CThostFtdcQryOrderField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryOrderField_swigregister(CThostFtdcQryOrderField)

class CThostFtdcQryTradeField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQryTradeField_BrokerID_get, _thostmduserapi.CThostFtdcQryTradeField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcQryTradeField_InvestorID_get, _thostmduserapi.CThostFtdcQryTradeField_InvestorID_set)
    InstrumentID = property(_thostmduserapi.CThostFtdcQryTradeField_InstrumentID_get, _thostmduserapi.CThostFtdcQryTradeField_InstrumentID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcQryTradeField_ExchangeID_get, _thostmduserapi.CThostFtdcQryTradeField_ExchangeID_set)
    TradeID = property(_thostmduserapi.CThostFtdcQryTradeField_TradeID_get, _thostmduserapi.CThostFtdcQryTradeField_TradeID_set)
    TradeTimeStart = property(_thostmduserapi.CThostFtdcQryTradeField_TradeTimeStart_get, _thostmduserapi.CThostFtdcQryTradeField_TradeTimeStart_set)
    TradeTimeEnd = property(_thostmduserapi.CThostFtdcQryTradeField_TradeTimeEnd_get, _thostmduserapi.CThostFtdcQryTradeField_TradeTimeEnd_set)
    InvestUnitID = property(_thostmduserapi.CThostFtdcQryTradeField_InvestUnitID_get, _thostmduserapi.CThostFtdcQryTradeField_InvestUnitID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryTradeField_swiginit(self, _thostmduserapi.new_CThostFtdcQryTradeField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryTradeField

# Register CThostFtdcQryTradeField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryTradeField_swigregister(CThostFtdcQryTradeField)

class CThostFtdcQryInvestorPositionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQryInvestorPositionField_BrokerID_get, _thostmduserapi.CThostFtdcQryInvestorPositionField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcQryInvestorPositionField_InvestorID_get, _thostmduserapi.CThostFtdcQryInvestorPositionField_InvestorID_set)
    InstrumentID = property(_thostmduserapi.CThostFtdcQryInvestorPositionField_InstrumentID_get, _thostmduserapi.CThostFtdcQryInvestorPositionField_InstrumentID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcQryInvestorPositionField_ExchangeID_get, _thostmduserapi.CThostFtdcQryInvestorPositionField_ExchangeID_set)
    InvestUnitID = property(_thostmduserapi.CThostFtdcQryInvestorPositionField_InvestUnitID_get, _thostmduserapi.CThostFtdcQryInvestorPositionField_InvestUnitID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryInvestorPositionField_swiginit(self, _thostmduserapi.new_CThostFtdcQryInvestorPositionField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryInvestorPositionField

# Register CThostFtdcQryInvestorPositionField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryInvestorPositionField_swigregister(CThostFtdcQryInvestorPositionField)

class CThostFtdcQryTradingAccountField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQryTradingAccountField_BrokerID_get, _thostmduserapi.CThostFtdcQryTradingAccountField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcQryTradingAccountField_InvestorID_get, _thostmduserapi.CThostFtdcQryTradingAccountField_InvestorID_set)
    CurrencyID = property(_thostmduserapi.CThostFtdcQryTradingAccountField_CurrencyID_get, _thostmduserapi.CThostFtdcQryTradingAccountField_CurrencyID_set)
    BizType = property(_thostmduserapi.CThostFtdcQryTradingAccountField_BizType_get, _thostmduserapi.CThostFtdcQryTradingAccountField_BizType_set)
    AccountID = property(_thostmduserapi.CThostFtdcQryTradingAccountField_AccountID_get, _thostmduserapi.CThostFtdcQryTradingAccountField_AccountID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryTradingAccountField_swiginit(self, _thostmduserapi.new_CThostFtdcQryTradingAccountField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryTradingAccountField

# Register CThostFtdcQryTradingAccountField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryTradingAccountField_swigregister(CThostFtdcQryTradingAccountField)

class CThostFtdcQryInvestorField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQryInvestorField_BrokerID_get, _thostmduserapi.CThostFtdcQryInvestorField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcQryInvestorField_InvestorID_get, _thostmduserapi.CThostFtdcQryInvestorField_InvestorID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryInvestorField_swiginit(self, _thostmduserapi.new_CThostFtdcQryInvestorField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryInvestorField

# Register CThostFtdcQryInvestorField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryInvestorField_swigregister(CThostFtdcQryInvestorField)

class CThostFtdcQryTradingCodeField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQryTradingCodeField_BrokerID_get, _thostmduserapi.CThostFtdcQryTradingCodeField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcQryTradingCodeField_InvestorID_get, _thostmduserapi.CThostFtdcQryTradingCodeField_InvestorID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcQryTradingCodeField_ExchangeID_get, _thostmduserapi.CThostFtdcQryTradingCodeField_ExchangeID_set)
    ClientID = property(_thostmduserapi.CThostFtdcQryTradingCodeField_ClientID_get, _thostmduserapi.CThostFtdcQryTradingCodeField_ClientID_set)
    ClientIDType = property(_thostmduserapi.CThostFtdcQryTradingCodeField_ClientIDType_get, _thostmduserapi.CThostFtdcQryTradingCodeField_ClientIDType_set)
    InvestUnitID = property(_thostmduserapi.CThostFtdcQryTradingCodeField_InvestUnitID_get, _thostmduserapi.CThostFtdcQryTradingCodeField_InvestUnitID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryTradingCodeField_swiginit(self, _thostmduserapi.new_CThostFtdcQryTradingCodeField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryTradingCodeField

# Register CThostFtdcQryTradingCodeField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryTradingCodeField_swigregister(CThostFtdcQryTradingCodeField)

class CThostFtdcQryInvestorGroupField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQryInvestorGroupField_BrokerID_get, _thostmduserapi.CThostFtdcQryInvestorGroupField_BrokerID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryInvestorGroupField_swiginit(self, _thostmduserapi.new_CThostFtdcQryInvestorGroupField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryInvestorGroupField

# Register CThostFtdcQryInvestorGroupField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryInvestorGroupField_swigregister(CThostFtdcQryInvestorGroupField)

class CThostFtdcQryInstrumentMarginRateField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQryInstrumentMarginRateField_BrokerID_get, _thostmduserapi.CThostFtdcQryInstrumentMarginRateField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcQryInstrumentMarginRateField_InvestorID_get, _thostmduserapi.CThostFtdcQryInstrumentMarginRateField_InvestorID_set)
    InstrumentID = property(_thostmduserapi.CThostFtdcQryInstrumentMarginRateField_InstrumentID_get, _thostmduserapi.CThostFtdcQryInstrumentMarginRateField_InstrumentID_set)
    HedgeFlag = property(_thostmduserapi.CThostFtdcQryInstrumentMarginRateField_HedgeFlag_get, _thostmduserapi.CThostFtdcQryInstrumentMarginRateField_HedgeFlag_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcQryInstrumentMarginRateField_ExchangeID_get, _thostmduserapi.CThostFtdcQryInstrumentMarginRateField_ExchangeID_set)
    InvestUnitID = property(_thostmduserapi.CThostFtdcQryInstrumentMarginRateField_InvestUnitID_get, _thostmduserapi.CThostFtdcQryInstrumentMarginRateField_InvestUnitID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryInstrumentMarginRateField_swiginit(self, _thostmduserapi.new_CThostFtdcQryInstrumentMarginRateField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryInstrumentMarginRateField

# Register CThostFtdcQryInstrumentMarginRateField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryInstrumentMarginRateField_swigregister(CThostFtdcQryInstrumentMarginRateField)

class CThostFtdcQryInstrumentCommissionRateField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQryInstrumentCommissionRateField_BrokerID_get, _thostmduserapi.CThostFtdcQryInstrumentCommissionRateField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcQryInstrumentCommissionRateField_InvestorID_get, _thostmduserapi.CThostFtdcQryInstrumentCommissionRateField_InvestorID_set)
    InstrumentID = property(_thostmduserapi.CThostFtdcQryInstrumentCommissionRateField_InstrumentID_get, _thostmduserapi.CThostFtdcQryInstrumentCommissionRateField_InstrumentID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcQryInstrumentCommissionRateField_ExchangeID_get, _thostmduserapi.CThostFtdcQryInstrumentCommissionRateField_ExchangeID_set)
    InvestUnitID = property(_thostmduserapi.CThostFtdcQryInstrumentCommissionRateField_InvestUnitID_get, _thostmduserapi.CThostFtdcQryInstrumentCommissionRateField_InvestUnitID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryInstrumentCommissionRateField_swiginit(self, _thostmduserapi.new_CThostFtdcQryInstrumentCommissionRateField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryInstrumentCommissionRateField

# Register CThostFtdcQryInstrumentCommissionRateField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryInstrumentCommissionRateField_swigregister(CThostFtdcQryInstrumentCommissionRateField)

class CThostFtdcQryInstrumentTradingRightField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQryInstrumentTradingRightField_BrokerID_get, _thostmduserapi.CThostFtdcQryInstrumentTradingRightField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcQryInstrumentTradingRightField_InvestorID_get, _thostmduserapi.CThostFtdcQryInstrumentTradingRightField_InvestorID_set)
    InstrumentID = property(_thostmduserapi.CThostFtdcQryInstrumentTradingRightField_InstrumentID_get, _thostmduserapi.CThostFtdcQryInstrumentTradingRightField_InstrumentID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryInstrumentTradingRightField_swiginit(self, _thostmduserapi.new_CThostFtdcQryInstrumentTradingRightField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryInstrumentTradingRightField

# Register CThostFtdcQryInstrumentTradingRightField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryInstrumentTradingRightField_swigregister(CThostFtdcQryInstrumentTradingRightField)

class CThostFtdcQryBrokerField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQryBrokerField_BrokerID_get, _thostmduserapi.CThostFtdcQryBrokerField_BrokerID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryBrokerField_swiginit(self, _thostmduserapi.new_CThostFtdcQryBrokerField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryBrokerField

# Register CThostFtdcQryBrokerField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryBrokerField_swigregister(CThostFtdcQryBrokerField)

class CThostFtdcQryTraderField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ExchangeID = property(_thostmduserapi.CThostFtdcQryTraderField_ExchangeID_get, _thostmduserapi.CThostFtdcQryTraderField_ExchangeID_set)
    ParticipantID = property(_thostmduserapi.CThostFtdcQryTraderField_ParticipantID_get, _thostmduserapi.CThostFtdcQryTraderField_ParticipantID_set)
    TraderID = property(_thostmduserapi.CThostFtdcQryTraderField_TraderID_get, _thostmduserapi.CThostFtdcQryTraderField_TraderID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryTraderField_swiginit(self, _thostmduserapi.new_CThostFtdcQryTraderField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryTraderField

# Register CThostFtdcQryTraderField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryTraderField_swigregister(CThostFtdcQryTraderField)

class CThostFtdcQrySuperUserFunctionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    UserID = property(_thostmduserapi.CThostFtdcQrySuperUserFunctionField_UserID_get, _thostmduserapi.CThostFtdcQrySuperUserFunctionField_UserID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQrySuperUserFunctionField_swiginit(self, _thostmduserapi.new_CThostFtdcQrySuperUserFunctionField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQrySuperUserFunctionField

# Register CThostFtdcQrySuperUserFunctionField in _thostmduserapi:
_thostmduserapi.CThostFtdcQrySuperUserFunctionField_swigregister(CThostFtdcQrySuperUserFunctionField)

class CThostFtdcQryUserSessionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    FrontID = property(_thostmduserapi.CThostFtdcQryUserSessionField_FrontID_get, _thostmduserapi.CThostFtdcQryUserSessionField_FrontID_set)
    SessionID = property(_thostmduserapi.CThostFtdcQryUserSessionField_SessionID_get, _thostmduserapi.CThostFtdcQryUserSessionField_SessionID_set)
    BrokerID = property(_thostmduserapi.CThostFtdcQryUserSessionField_BrokerID_get, _thostmduserapi.CThostFtdcQryUserSessionField_BrokerID_set)
    UserID = property(_thostmduserapi.CThostFtdcQryUserSessionField_UserID_get, _thostmduserapi.CThostFtdcQryUserSessionField_UserID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryUserSessionField_swiginit(self, _thostmduserapi.new_CThostFtdcQryUserSessionField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryUserSessionField

# Register CThostFtdcQryUserSessionField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryUserSessionField_swigregister(CThostFtdcQryUserSessionField)

class CThostFtdcQryPartBrokerField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ExchangeID = property(_thostmduserapi.CThostFtdcQryPartBrokerField_ExchangeID_get, _thostmduserapi.CThostFtdcQryPartBrokerField_ExchangeID_set)
    BrokerID = property(_thostmduserapi.CThostFtdcQryPartBrokerField_BrokerID_get, _thostmduserapi.CThostFtdcQryPartBrokerField_BrokerID_set)
    ParticipantID = property(_thostmduserapi.CThostFtdcQryPartBrokerField_ParticipantID_get, _thostmduserapi.CThostFtdcQryPartBrokerField_ParticipantID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryPartBrokerField_swiginit(self, _thostmduserapi.new_CThostFtdcQryPartBrokerField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryPartBrokerField

# Register CThostFtdcQryPartBrokerField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryPartBrokerField_swigregister(CThostFtdcQryPartBrokerField)

class CThostFtdcQryFrontStatusField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    FrontID = property(_thostmduserapi.CThostFtdcQryFrontStatusField_FrontID_get, _thostmduserapi.CThostFtdcQryFrontStatusField_FrontID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryFrontStatusField_swiginit(self, _thostmduserapi.new_CThostFtdcQryFrontStatusField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryFrontStatusField

# Register CThostFtdcQryFrontStatusField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryFrontStatusField_swigregister(CThostFtdcQryFrontStatusField)

class CThostFtdcQryExchangeOrderField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ParticipantID = property(_thostmduserapi.CThostFtdcQryExchangeOrderField_ParticipantID_get, _thostmduserapi.CThostFtdcQryExchangeOrderField_ParticipantID_set)
    ClientID = property(_thostmduserapi.CThostFtdcQryExchangeOrderField_ClientID_get, _thostmduserapi.CThostFtdcQryExchangeOrderField_ClientID_set)
    ExchangeInstID = property(_thostmduserapi.CThostFtdcQryExchangeOrderField_ExchangeInstID_get, _thostmduserapi.CThostFtdcQryExchangeOrderField_ExchangeInstID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcQryExchangeOrderField_ExchangeID_get, _thostmduserapi.CThostFtdcQryExchangeOrderField_ExchangeID_set)
    TraderID = property(_thostmduserapi.CThostFtdcQryExchangeOrderField_TraderID_get, _thostmduserapi.CThostFtdcQryExchangeOrderField_TraderID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryExchangeOrderField_swiginit(self, _thostmduserapi.new_CThostFtdcQryExchangeOrderField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryExchangeOrderField

# Register CThostFtdcQryExchangeOrderField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryExchangeOrderField_swigregister(CThostFtdcQryExchangeOrderField)

class CThostFtdcQryOrderActionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQryOrderActionField_BrokerID_get, _thostmduserapi.CThostFtdcQryOrderActionField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcQryOrderActionField_InvestorID_get, _thostmduserapi.CThostFtdcQryOrderActionField_InvestorID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcQryOrderActionField_ExchangeID_get, _thostmduserapi.CThostFtdcQryOrderActionField_ExchangeID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryOrderActionField_swiginit(self, _thostmduserapi.new_CThostFtdcQryOrderActionField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryOrderActionField

# Register CThostFtdcQryOrderActionField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryOrderActionField_swigregister(CThostFtdcQryOrderActionField)

class CThostFtdcQryExchangeOrderActionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ParticipantID = property(_thostmduserapi.CThostFtdcQryExchangeOrderActionField_ParticipantID_get, _thostmduserapi.CThostFtdcQryExchangeOrderActionField_ParticipantID_set)
    ClientID = property(_thostmduserapi.CThostFtdcQryExchangeOrderActionField_ClientID_get, _thostmduserapi.CThostFtdcQryExchangeOrderActionField_ClientID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcQryExchangeOrderActionField_ExchangeID_get, _thostmduserapi.CThostFtdcQryExchangeOrderActionField_ExchangeID_set)
    TraderID = property(_thostmduserapi.CThostFtdcQryExchangeOrderActionField_TraderID_get, _thostmduserapi.CThostFtdcQryExchangeOrderActionField_TraderID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryExchangeOrderActionField_swiginit(self, _thostmduserapi.new_CThostFtdcQryExchangeOrderActionField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryExchangeOrderActionField

# Register CThostFtdcQryExchangeOrderActionField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryExchangeOrderActionField_swigregister(CThostFtdcQryExchangeOrderActionField)

class CThostFtdcQrySuperUserField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    UserID = property(_thostmduserapi.CThostFtdcQrySuperUserField_UserID_get, _thostmduserapi.CThostFtdcQrySuperUserField_UserID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQrySuperUserField_swiginit(self, _thostmduserapi.new_CThostFtdcQrySuperUserField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQrySuperUserField

# Register CThostFtdcQrySuperUserField in _thostmduserapi:
_thostmduserapi.CThostFtdcQrySuperUserField_swigregister(CThostFtdcQrySuperUserField)

class CThostFtdcQryExchangeField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ExchangeID = property(_thostmduserapi.CThostFtdcQryExchangeField_ExchangeID_get, _thostmduserapi.CThostFtdcQryExchangeField_ExchangeID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryExchangeField_swiginit(self, _thostmduserapi.new_CThostFtdcQryExchangeField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryExchangeField

# Register CThostFtdcQryExchangeField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryExchangeField_swigregister(CThostFtdcQryExchangeField)

class CThostFtdcQryProductField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ProductID = property(_thostmduserapi.CThostFtdcQryProductField_ProductID_get, _thostmduserapi.CThostFtdcQryProductField_ProductID_set)
    ProductClass = property(_thostmduserapi.CThostFtdcQryProductField_ProductClass_get, _thostmduserapi.CThostFtdcQryProductField_ProductClass_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcQryProductField_ExchangeID_get, _thostmduserapi.CThostFtdcQryProductField_ExchangeID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryProductField_swiginit(self, _thostmduserapi.new_CThostFtdcQryProductField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryProductField

# Register CThostFtdcQryProductField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryProductField_swigregister(CThostFtdcQryProductField)

class CThostFtdcQryInstrumentField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    InstrumentID = property(_thostmduserapi.CThostFtdcQryInstrumentField_InstrumentID_get, _thostmduserapi.CThostFtdcQryInstrumentField_InstrumentID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcQryInstrumentField_ExchangeID_get, _thostmduserapi.CThostFtdcQryInstrumentField_ExchangeID_set)
    ExchangeInstID = property(_thostmduserapi.CThostFtdcQryInstrumentField_ExchangeInstID_get, _thostmduserapi.CThostFtdcQryInstrumentField_ExchangeInstID_set)
    ProductID = property(_thostmduserapi.CThostFtdcQryInstrumentField_ProductID_get, _thostmduserapi.CThostFtdcQryInstrumentField_ProductID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryInstrumentField_swiginit(self, _thostmduserapi.new_CThostFtdcQryInstrumentField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryInstrumentField

# Register CThostFtdcQryInstrumentField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryInstrumentField_swigregister(CThostFtdcQryInstrumentField)

class CThostFtdcQryDepthMarketDataField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    InstrumentID = property(_thostmduserapi.CThostFtdcQryDepthMarketDataField_InstrumentID_get, _thostmduserapi.CThostFtdcQryDepthMarketDataField_InstrumentID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcQryDepthMarketDataField_ExchangeID_get, _thostmduserapi.CThostFtdcQryDepthMarketDataField_ExchangeID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryDepthMarketDataField_swiginit(self, _thostmduserapi.new_CThostFtdcQryDepthMarketDataField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryDepthMarketDataField

# Register CThostFtdcQryDepthMarketDataField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryDepthMarketDataField_swigregister(CThostFtdcQryDepthMarketDataField)

class CThostFtdcQryBrokerUserField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQryBrokerUserField_BrokerID_get, _thostmduserapi.CThostFtdcQryBrokerUserField_BrokerID_set)
    UserID = property(_thostmduserapi.CThostFtdcQryBrokerUserField_UserID_get, _thostmduserapi.CThostFtdcQryBrokerUserField_UserID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryBrokerUserField_swiginit(self, _thostmduserapi.new_CThostFtdcQryBrokerUserField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryBrokerUserField

# Register CThostFtdcQryBrokerUserField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryBrokerUserField_swigregister(CThostFtdcQryBrokerUserField)

class CThostFtdcQryBrokerUserFunctionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQryBrokerUserFunctionField_BrokerID_get, _thostmduserapi.CThostFtdcQryBrokerUserFunctionField_BrokerID_set)
    UserID = property(_thostmduserapi.CThostFtdcQryBrokerUserFunctionField_UserID_get, _thostmduserapi.CThostFtdcQryBrokerUserFunctionField_UserID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryBrokerUserFunctionField_swiginit(self, _thostmduserapi.new_CThostFtdcQryBrokerUserFunctionField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryBrokerUserFunctionField

# Register CThostFtdcQryBrokerUserFunctionField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryBrokerUserFunctionField_swigregister(CThostFtdcQryBrokerUserFunctionField)

class CThostFtdcQryTraderOfferField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ExchangeID = property(_thostmduserapi.CThostFtdcQryTraderOfferField_ExchangeID_get, _thostmduserapi.CThostFtdcQryTraderOfferField_ExchangeID_set)
    ParticipantID = property(_thostmduserapi.CThostFtdcQryTraderOfferField_ParticipantID_get, _thostmduserapi.CThostFtdcQryTraderOfferField_ParticipantID_set)
    TraderID = property(_thostmduserapi.CThostFtdcQryTraderOfferField_TraderID_get, _thostmduserapi.CThostFtdcQryTraderOfferField_TraderID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryTraderOfferField_swiginit(self, _thostmduserapi.new_CThostFtdcQryTraderOfferField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryTraderOfferField

# Register CThostFtdcQryTraderOfferField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryTraderOfferField_swigregister(CThostFtdcQryTraderOfferField)

class CThostFtdcQrySyncDepositField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQrySyncDepositField_BrokerID_get, _thostmduserapi.CThostFtdcQrySyncDepositField_BrokerID_set)
    DepositSeqNo = property(_thostmduserapi.CThostFtdcQrySyncDepositField_DepositSeqNo_get, _thostmduserapi.CThostFtdcQrySyncDepositField_DepositSeqNo_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQrySyncDepositField_swiginit(self, _thostmduserapi.new_CThostFtdcQrySyncDepositField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQrySyncDepositField

# Register CThostFtdcQrySyncDepositField in _thostmduserapi:
_thostmduserapi.CThostFtdcQrySyncDepositField_swigregister(CThostFtdcQrySyncDepositField)

class CThostFtdcQrySettlementInfoField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQrySettlementInfoField_BrokerID_get, _thostmduserapi.CThostFtdcQrySettlementInfoField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcQrySettlementInfoField_InvestorID_get, _thostmduserapi.CThostFtdcQrySettlementInfoField_InvestorID_set)
    TradingDay = property(_thostmduserapi.CThostFtdcQrySettlementInfoField_TradingDay_get, _thostmduserapi.CThostFtdcQrySettlementInfoField_TradingDay_set)
    AccountID = property(_thostmduserapi.CThostFtdcQrySettlementInfoField_AccountID_get, _thostmduserapi.CThostFtdcQrySettlementInfoField_AccountID_set)
    CurrencyID = property(_thostmduserapi.CThostFtdcQrySettlementInfoField_CurrencyID_get, _thostmduserapi.CThostFtdcQrySettlementInfoField_CurrencyID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQrySettlementInfoField_swiginit(self, _thostmduserapi.new_CThostFtdcQrySettlementInfoField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQrySettlementInfoField

# Register CThostFtdcQrySettlementInfoField in _thostmduserapi:
_thostmduserapi.CThostFtdcQrySettlementInfoField_swigregister(CThostFtdcQrySettlementInfoField)

class CThostFtdcQryExchangeMarginRateField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQryExchangeMarginRateField_BrokerID_get, _thostmduserapi.CThostFtdcQryExchangeMarginRateField_BrokerID_set)
    InstrumentID = property(_thostmduserapi.CThostFtdcQryExchangeMarginRateField_InstrumentID_get, _thostmduserapi.CThostFtdcQryExchangeMarginRateField_InstrumentID_set)
    HedgeFlag = property(_thostmduserapi.CThostFtdcQryExchangeMarginRateField_HedgeFlag_get, _thostmduserapi.CThostFtdcQryExchangeMarginRateField_HedgeFlag_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcQryExchangeMarginRateField_ExchangeID_get, _thostmduserapi.CThostFtdcQryExchangeMarginRateField_ExchangeID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryExchangeMarginRateField_swiginit(self, _thostmduserapi.new_CThostFtdcQryExchangeMarginRateField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryExchangeMarginRateField

# Register CThostFtdcQryExchangeMarginRateField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryExchangeMarginRateField_swigregister(CThostFtdcQryExchangeMarginRateField)

class CThostFtdcQryExchangeMarginRateAdjustField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQryExchangeMarginRateAdjustField_BrokerID_get, _thostmduserapi.CThostFtdcQryExchangeMarginRateAdjustField_BrokerID_set)
    InstrumentID = property(_thostmduserapi.CThostFtdcQryExchangeMarginRateAdjustField_InstrumentID_get, _thostmduserapi.CThostFtdcQryExchangeMarginRateAdjustField_InstrumentID_set)
    HedgeFlag = property(_thostmduserapi.CThostFtdcQryExchangeMarginRateAdjustField_HedgeFlag_get, _thostmduserapi.CThostFtdcQryExchangeMarginRateAdjustField_HedgeFlag_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryExchangeMarginRateAdjustField_swiginit(self, _thostmduserapi.new_CThostFtdcQryExchangeMarginRateAdjustField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryExchangeMarginRateAdjustField

# Register CThostFtdcQryExchangeMarginRateAdjustField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryExchangeMarginRateAdjustField_swigregister(CThostFtdcQryExchangeMarginRateAdjustField)

class CThostFtdcQryExchangeRateField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQryExchangeRateField_BrokerID_get, _thostmduserapi.CThostFtdcQryExchangeRateField_BrokerID_set)
    FromCurrencyID = property(_thostmduserapi.CThostFtdcQryExchangeRateField_FromCurrencyID_get, _thostmduserapi.CThostFtdcQryExchangeRateField_FromCurrencyID_set)
    ToCurrencyID = property(_thostmduserapi.CThostFtdcQryExchangeRateField_ToCurrencyID_get, _thostmduserapi.CThostFtdcQryExchangeRateField_ToCurrencyID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryExchangeRateField_swiginit(self, _thostmduserapi.new_CThostFtdcQryExchangeRateField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryExchangeRateField

# Register CThostFtdcQryExchangeRateField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryExchangeRateField_swigregister(CThostFtdcQryExchangeRateField)

class CThostFtdcQrySyncFundMortgageField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQrySyncFundMortgageField_BrokerID_get, _thostmduserapi.CThostFtdcQrySyncFundMortgageField_BrokerID_set)
    MortgageSeqNo = property(_thostmduserapi.CThostFtdcQrySyncFundMortgageField_MortgageSeqNo_get, _thostmduserapi.CThostFtdcQrySyncFundMortgageField_MortgageSeqNo_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQrySyncFundMortgageField_swiginit(self, _thostmduserapi.new_CThostFtdcQrySyncFundMortgageField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQrySyncFundMortgageField

# Register CThostFtdcQrySyncFundMortgageField in _thostmduserapi:
_thostmduserapi.CThostFtdcQrySyncFundMortgageField_swigregister(CThostFtdcQrySyncFundMortgageField)

class CThostFtdcQryHisOrderField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQryHisOrderField_BrokerID_get, _thostmduserapi.CThostFtdcQryHisOrderField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcQryHisOrderField_InvestorID_get, _thostmduserapi.CThostFtdcQryHisOrderField_InvestorID_set)
    InstrumentID = property(_thostmduserapi.CThostFtdcQryHisOrderField_InstrumentID_get, _thostmduserapi.CThostFtdcQryHisOrderField_InstrumentID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcQryHisOrderField_ExchangeID_get, _thostmduserapi.CThostFtdcQryHisOrderField_ExchangeID_set)
    OrderSysID = property(_thostmduserapi.CThostFtdcQryHisOrderField_OrderSysID_get, _thostmduserapi.CThostFtdcQryHisOrderField_OrderSysID_set)
    InsertTimeStart = property(_thostmduserapi.CThostFtdcQryHisOrderField_InsertTimeStart_get, _thostmduserapi.CThostFtdcQryHisOrderField_InsertTimeStart_set)
    InsertTimeEnd = property(_thostmduserapi.CThostFtdcQryHisOrderField_InsertTimeEnd_get, _thostmduserapi.CThostFtdcQryHisOrderField_InsertTimeEnd_set)
    TradingDay = property(_thostmduserapi.CThostFtdcQryHisOrderField_TradingDay_get, _thostmduserapi.CThostFtdcQryHisOrderField_TradingDay_set)
    SettlementID = property(_thostmduserapi.CThostFtdcQryHisOrderField_SettlementID_get, _thostmduserapi.CThostFtdcQryHisOrderField_SettlementID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryHisOrderField_swiginit(self, _thostmduserapi.new_CThostFtdcQryHisOrderField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryHisOrderField

# Register CThostFtdcQryHisOrderField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryHisOrderField_swigregister(CThostFtdcQryHisOrderField)

class CThostFtdcOptionInstrMiniMarginField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    InstrumentID = property(_thostmduserapi.CThostFtdcOptionInstrMiniMarginField_InstrumentID_get, _thostmduserapi.CThostFtdcOptionInstrMiniMarginField_InstrumentID_set)
    InvestorRange = property(_thostmduserapi.CThostFtdcOptionInstrMiniMarginField_InvestorRange_get, _thostmduserapi.CThostFtdcOptionInstrMiniMarginField_InvestorRange_set)
    BrokerID = property(_thostmduserapi.CThostFtdcOptionInstrMiniMarginField_BrokerID_get, _thostmduserapi.CThostFtdcOptionInstrMiniMarginField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcOptionInstrMiniMarginField_InvestorID_get, _thostmduserapi.CThostFtdcOptionInstrMiniMarginField_InvestorID_set)
    MinMargin = property(_thostmduserapi.CThostFtdcOptionInstrMiniMarginField_MinMargin_get, _thostmduserapi.CThostFtdcOptionInstrMiniMarginField_MinMargin_set)
    ValueMethod = property(_thostmduserapi.CThostFtdcOptionInstrMiniMarginField_ValueMethod_get, _thostmduserapi.CThostFtdcOptionInstrMiniMarginField_ValueMethod_set)
    IsRelative = property(_thostmduserapi.CThostFtdcOptionInstrMiniMarginField_IsRelative_get, _thostmduserapi.CThostFtdcOptionInstrMiniMarginField_IsRelative_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcOptionInstrMiniMarginField_swiginit(self, _thostmduserapi.new_CThostFtdcOptionInstrMiniMarginField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcOptionInstrMiniMarginField

# Register CThostFtdcOptionInstrMiniMarginField in _thostmduserapi:
_thostmduserapi.CThostFtdcOptionInstrMiniMarginField_swigregister(CThostFtdcOptionInstrMiniMarginField)

class CThostFtdcOptionInstrMarginAdjustField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    InstrumentID = property(_thostmduserapi.CThostFtdcOptionInstrMarginAdjustField_InstrumentID_get, _thostmduserapi.CThostFtdcOptionInstrMarginAdjustField_InstrumentID_set)
    InvestorRange = property(_thostmduserapi.CThostFtdcOptionInstrMarginAdjustField_InvestorRange_get, _thostmduserapi.CThostFtdcOptionInstrMarginAdjustField_InvestorRange_set)
    BrokerID = property(_thostmduserapi.CThostFtdcOptionInstrMarginAdjustField_BrokerID_get, _thostmduserapi.CThostFtdcOptionInstrMarginAdjustField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcOptionInstrMarginAdjustField_InvestorID_get, _thostmduserapi.CThostFtdcOptionInstrMarginAdjustField_InvestorID_set)
    SShortMarginRatioByMoney = property(_thostmduserapi.CThostFtdcOptionInstrMarginAdjustField_SShortMarginRatioByMoney_get, _thostmduserapi.CThostFtdcOptionInstrMarginAdjustField_SShortMarginRatioByMoney_set)
    SShortMarginRatioByVolume = property(_thostmduserapi.CThostFtdcOptionInstrMarginAdjustField_SShortMarginRatioByVolume_get, _thostmduserapi.CThostFtdcOptionInstrMarginAdjustField_SShortMarginRatioByVolume_set)
    HShortMarginRatioByMoney = property(_thostmduserapi.CThostFtdcOptionInstrMarginAdjustField_HShortMarginRatioByMoney_get, _thostmduserapi.CThostFtdcOptionInstrMarginAdjustField_HShortMarginRatioByMoney_set)
    HShortMarginRatioByVolume = property(_thostmduserapi.CThostFtdcOptionInstrMarginAdjustField_HShortMarginRatioByVolume_get, _thostmduserapi.CThostFtdcOptionInstrMarginAdjustField_HShortMarginRatioByVolume_set)
    AShortMarginRatioByMoney = property(_thostmduserapi.CThostFtdcOptionInstrMarginAdjustField_AShortMarginRatioByMoney_get, _thostmduserapi.CThostFtdcOptionInstrMarginAdjustField_AShortMarginRatioByMoney_set)
    AShortMarginRatioByVolume = property(_thostmduserapi.CThostFtdcOptionInstrMarginAdjustField_AShortMarginRatioByVolume_get, _thostmduserapi.CThostFtdcOptionInstrMarginAdjustField_AShortMarginRatioByVolume_set)
    IsRelative = property(_thostmduserapi.CThostFtdcOptionInstrMarginAdjustField_IsRelative_get, _thostmduserapi.CThostFtdcOptionInstrMarginAdjustField_IsRelative_set)
    MShortMarginRatioByMoney = property(_thostmduserapi.CThostFtdcOptionInstrMarginAdjustField_MShortMarginRatioByMoney_get, _thostmduserapi.CThostFtdcOptionInstrMarginAdjustField_MShortMarginRatioByMoney_set)
    MShortMarginRatioByVolume = property(_thostmduserapi.CThostFtdcOptionInstrMarginAdjustField_MShortMarginRatioByVolume_get, _thostmduserapi.CThostFtdcOptionInstrMarginAdjustField_MShortMarginRatioByVolume_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcOptionInstrMarginAdjustField_swiginit(self, _thostmduserapi.new_CThostFtdcOptionInstrMarginAdjustField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcOptionInstrMarginAdjustField

# Register CThostFtdcOptionInstrMarginAdjustField in _thostmduserapi:
_thostmduserapi.CThostFtdcOptionInstrMarginAdjustField_swigregister(CThostFtdcOptionInstrMarginAdjustField)

class CThostFtdcOptionInstrCommRateField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    InstrumentID = property(_thostmduserapi.CThostFtdcOptionInstrCommRateField_InstrumentID_get, _thostmduserapi.CThostFtdcOptionInstrCommRateField_InstrumentID_set)
    InvestorRange = property(_thostmduserapi.CThostFtdcOptionInstrCommRateField_InvestorRange_get, _thostmduserapi.CThostFtdcOptionInstrCommRateField_InvestorRange_set)
    BrokerID = property(_thostmduserapi.CThostFtdcOptionInstrCommRateField_BrokerID_get, _thostmduserapi.CThostFtdcOptionInstrCommRateField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcOptionInstrCommRateField_InvestorID_get, _thostmduserapi.CThostFtdcOptionInstrCommRateField_InvestorID_set)
    OpenRatioByMoney = property(_thostmduserapi.CThostFtdcOptionInstrCommRateField_OpenRatioByMoney_get, _thostmduserapi.CThostFtdcOptionInstrCommRateField_OpenRatioByMoney_set)
    OpenRatioByVolume = property(_thostmduserapi.CThostFtdcOptionInstrCommRateField_OpenRatioByVolume_get, _thostmduserapi.CThostFtdcOptionInstrCommRateField_OpenRatioByVolume_set)
    CloseRatioByMoney = property(_thostmduserapi.CThostFtdcOptionInstrCommRateField_CloseRatioByMoney_get, _thostmduserapi.CThostFtdcOptionInstrCommRateField_CloseRatioByMoney_set)
    CloseRatioByVolume = property(_thostmduserapi.CThostFtdcOptionInstrCommRateField_CloseRatioByVolume_get, _thostmduserapi.CThostFtdcOptionInstrCommRateField_CloseRatioByVolume_set)
    CloseTodayRatioByMoney = property(_thostmduserapi.CThostFtdcOptionInstrCommRateField_CloseTodayRatioByMoney_get, _thostmduserapi.CThostFtdcOptionInstrCommRateField_CloseTodayRatioByMoney_set)
    CloseTodayRatioByVolume = property(_thostmduserapi.CThostFtdcOptionInstrCommRateField_CloseTodayRatioByVolume_get, _thostmduserapi.CThostFtdcOptionInstrCommRateField_CloseTodayRatioByVolume_set)
    StrikeRatioByMoney = property(_thostmduserapi.CThostFtdcOptionInstrCommRateField_StrikeRatioByMoney_get, _thostmduserapi.CThostFtdcOptionInstrCommRateField_StrikeRatioByMoney_set)
    StrikeRatioByVolume = property(_thostmduserapi.CThostFtdcOptionInstrCommRateField_StrikeRatioByVolume_get, _thostmduserapi.CThostFtdcOptionInstrCommRateField_StrikeRatioByVolume_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcOptionInstrCommRateField_ExchangeID_get, _thostmduserapi.CThostFtdcOptionInstrCommRateField_ExchangeID_set)
    InvestUnitID = property(_thostmduserapi.CThostFtdcOptionInstrCommRateField_InvestUnitID_get, _thostmduserapi.CThostFtdcOptionInstrCommRateField_InvestUnitID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcOptionInstrCommRateField_swiginit(self, _thostmduserapi.new_CThostFtdcOptionInstrCommRateField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcOptionInstrCommRateField

# Register CThostFtdcOptionInstrCommRateField in _thostmduserapi:
_thostmduserapi.CThostFtdcOptionInstrCommRateField_swigregister(CThostFtdcOptionInstrCommRateField)

class CThostFtdcOptionInstrTradeCostField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcOptionInstrTradeCostField_BrokerID_get, _thostmduserapi.CThostFtdcOptionInstrTradeCostField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcOptionInstrTradeCostField_InvestorID_get, _thostmduserapi.CThostFtdcOptionInstrTradeCostField_InvestorID_set)
    InstrumentID = property(_thostmduserapi.CThostFtdcOptionInstrTradeCostField_InstrumentID_get, _thostmduserapi.CThostFtdcOptionInstrTradeCostField_InstrumentID_set)
    HedgeFlag = property(_thostmduserapi.CThostFtdcOptionInstrTradeCostField_HedgeFlag_get, _thostmduserapi.CThostFtdcOptionInstrTradeCostField_HedgeFlag_set)
    FixedMargin = property(_thostmduserapi.CThostFtdcOptionInstrTradeCostField_FixedMargin_get, _thostmduserapi.CThostFtdcOptionInstrTradeCostField_FixedMargin_set)
    MiniMargin = property(_thostmduserapi.CThostFtdcOptionInstrTradeCostField_MiniMargin_get, _thostmduserapi.CThostFtdcOptionInstrTradeCostField_MiniMargin_set)
    Royalty = property(_thostmduserapi.CThostFtdcOptionInstrTradeCostField_Royalty_get, _thostmduserapi.CThostFtdcOptionInstrTradeCostField_Royalty_set)
    ExchFixedMargin = property(_thostmduserapi.CThostFtdcOptionInstrTradeCostField_ExchFixedMargin_get, _thostmduserapi.CThostFtdcOptionInstrTradeCostField_ExchFixedMargin_set)
    ExchMiniMargin = property(_thostmduserapi.CThostFtdcOptionInstrTradeCostField_ExchMiniMargin_get, _thostmduserapi.CThostFtdcOptionInstrTradeCostField_ExchMiniMargin_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcOptionInstrTradeCostField_ExchangeID_get, _thostmduserapi.CThostFtdcOptionInstrTradeCostField_ExchangeID_set)
    InvestUnitID = property(_thostmduserapi.CThostFtdcOptionInstrTradeCostField_InvestUnitID_get, _thostmduserapi.CThostFtdcOptionInstrTradeCostField_InvestUnitID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcOptionInstrTradeCostField_swiginit(self, _thostmduserapi.new_CThostFtdcOptionInstrTradeCostField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcOptionInstrTradeCostField

# Register CThostFtdcOptionInstrTradeCostField in _thostmduserapi:
_thostmduserapi.CThostFtdcOptionInstrTradeCostField_swigregister(CThostFtdcOptionInstrTradeCostField)

class CThostFtdcQryOptionInstrTradeCostField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQryOptionInstrTradeCostField_BrokerID_get, _thostmduserapi.CThostFtdcQryOptionInstrTradeCostField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcQryOptionInstrTradeCostField_InvestorID_get, _thostmduserapi.CThostFtdcQryOptionInstrTradeCostField_InvestorID_set)
    InstrumentID = property(_thostmduserapi.CThostFtdcQryOptionInstrTradeCostField_InstrumentID_get, _thostmduserapi.CThostFtdcQryOptionInstrTradeCostField_InstrumentID_set)
    HedgeFlag = property(_thostmduserapi.CThostFtdcQryOptionInstrTradeCostField_HedgeFlag_get, _thostmduserapi.CThostFtdcQryOptionInstrTradeCostField_HedgeFlag_set)
    InputPrice = property(_thostmduserapi.CThostFtdcQryOptionInstrTradeCostField_InputPrice_get, _thostmduserapi.CThostFtdcQryOptionInstrTradeCostField_InputPrice_set)
    UnderlyingPrice = property(_thostmduserapi.CThostFtdcQryOptionInstrTradeCostField_UnderlyingPrice_get, _thostmduserapi.CThostFtdcQryOptionInstrTradeCostField_UnderlyingPrice_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcQryOptionInstrTradeCostField_ExchangeID_get, _thostmduserapi.CThostFtdcQryOptionInstrTradeCostField_ExchangeID_set)
    InvestUnitID = property(_thostmduserapi.CThostFtdcQryOptionInstrTradeCostField_InvestUnitID_get, _thostmduserapi.CThostFtdcQryOptionInstrTradeCostField_InvestUnitID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryOptionInstrTradeCostField_swiginit(self, _thostmduserapi.new_CThostFtdcQryOptionInstrTradeCostField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryOptionInstrTradeCostField

# Register CThostFtdcQryOptionInstrTradeCostField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryOptionInstrTradeCostField_swigregister(CThostFtdcQryOptionInstrTradeCostField)

class CThostFtdcQryOptionInstrCommRateField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQryOptionInstrCommRateField_BrokerID_get, _thostmduserapi.CThostFtdcQryOptionInstrCommRateField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcQryOptionInstrCommRateField_InvestorID_get, _thostmduserapi.CThostFtdcQryOptionInstrCommRateField_InvestorID_set)
    InstrumentID = property(_thostmduserapi.CThostFtdcQryOptionInstrCommRateField_InstrumentID_get, _thostmduserapi.CThostFtdcQryOptionInstrCommRateField_InstrumentID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcQryOptionInstrCommRateField_ExchangeID_get, _thostmduserapi.CThostFtdcQryOptionInstrCommRateField_ExchangeID_set)
    InvestUnitID = property(_thostmduserapi.CThostFtdcQryOptionInstrCommRateField_InvestUnitID_get, _thostmduserapi.CThostFtdcQryOptionInstrCommRateField_InvestUnitID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryOptionInstrCommRateField_swiginit(self, _thostmduserapi.new_CThostFtdcQryOptionInstrCommRateField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryOptionInstrCommRateField

# Register CThostFtdcQryOptionInstrCommRateField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryOptionInstrCommRateField_swigregister(CThostFtdcQryOptionInstrCommRateField)

class CThostFtdcIndexPriceField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcIndexPriceField_BrokerID_get, _thostmduserapi.CThostFtdcIndexPriceField_BrokerID_set)
    InstrumentID = property(_thostmduserapi.CThostFtdcIndexPriceField_InstrumentID_get, _thostmduserapi.CThostFtdcIndexPriceField_InstrumentID_set)
    ClosePrice = property(_thostmduserapi.CThostFtdcIndexPriceField_ClosePrice_get, _thostmduserapi.CThostFtdcIndexPriceField_ClosePrice_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcIndexPriceField_swiginit(self, _thostmduserapi.new_CThostFtdcIndexPriceField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcIndexPriceField

# Register CThostFtdcIndexPriceField in _thostmduserapi:
_thostmduserapi.CThostFtdcIndexPriceField_swigregister(CThostFtdcIndexPriceField)

class CThostFtdcInputExecOrderField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcInputExecOrderField_BrokerID_get, _thostmduserapi.CThostFtdcInputExecOrderField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcInputExecOrderField_InvestorID_get, _thostmduserapi.CThostFtdcInputExecOrderField_InvestorID_set)
    InstrumentID = property(_thostmduserapi.CThostFtdcInputExecOrderField_InstrumentID_get, _thostmduserapi.CThostFtdcInputExecOrderField_InstrumentID_set)
    ExecOrderRef = property(_thostmduserapi.CThostFtdcInputExecOrderField_ExecOrderRef_get, _thostmduserapi.CThostFtdcInputExecOrderField_ExecOrderRef_set)
    UserID = property(_thostmduserapi.CThostFtdcInputExecOrderField_UserID_get, _thostmduserapi.CThostFtdcInputExecOrderField_UserID_set)
    Volume = property(_thostmduserapi.CThostFtdcInputExecOrderField_Volume_get, _thostmduserapi.CThostFtdcInputExecOrderField_Volume_set)
    RequestID = property(_thostmduserapi.CThostFtdcInputExecOrderField_RequestID_get, _thostmduserapi.CThostFtdcInputExecOrderField_RequestID_set)
    BusinessUnit = property(_thostmduserapi.CThostFtdcInputExecOrderField_BusinessUnit_get, _thostmduserapi.CThostFtdcInputExecOrderField_BusinessUnit_set)
    OffsetFlag = property(_thostmduserapi.CThostFtdcInputExecOrderField_OffsetFlag_get, _thostmduserapi.CThostFtdcInputExecOrderField_OffsetFlag_set)
    HedgeFlag = property(_thostmduserapi.CThostFtdcInputExecOrderField_HedgeFlag_get, _thostmduserapi.CThostFtdcInputExecOrderField_HedgeFlag_set)
    ActionType = property(_thostmduserapi.CThostFtdcInputExecOrderField_ActionType_get, _thostmduserapi.CThostFtdcInputExecOrderField_ActionType_set)
    PosiDirection = property(_thostmduserapi.CThostFtdcInputExecOrderField_PosiDirection_get, _thostmduserapi.CThostFtdcInputExecOrderField_PosiDirection_set)
    ReservePositionFlag = property(_thostmduserapi.CThostFtdcInputExecOrderField_ReservePositionFlag_get, _thostmduserapi.CThostFtdcInputExecOrderField_ReservePositionFlag_set)
    CloseFlag = property(_thostmduserapi.CThostFtdcInputExecOrderField_CloseFlag_get, _thostmduserapi.CThostFtdcInputExecOrderField_CloseFlag_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcInputExecOrderField_ExchangeID_get, _thostmduserapi.CThostFtdcInputExecOrderField_ExchangeID_set)
    InvestUnitID = property(_thostmduserapi.CThostFtdcInputExecOrderField_InvestUnitID_get, _thostmduserapi.CThostFtdcInputExecOrderField_InvestUnitID_set)
    AccountID = property(_thostmduserapi.CThostFtdcInputExecOrderField_AccountID_get, _thostmduserapi.CThostFtdcInputExecOrderField_AccountID_set)
    CurrencyID = property(_thostmduserapi.CThostFtdcInputExecOrderField_CurrencyID_get, _thostmduserapi.CThostFtdcInputExecOrderField_CurrencyID_set)
    ClientID = property(_thostmduserapi.CThostFtdcInputExecOrderField_ClientID_get, _thostmduserapi.CThostFtdcInputExecOrderField_ClientID_set)
    IPAddress = property(_thostmduserapi.CThostFtdcInputExecOrderField_IPAddress_get, _thostmduserapi.CThostFtdcInputExecOrderField_IPAddress_set)
    MacAddress = property(_thostmduserapi.CThostFtdcInputExecOrderField_MacAddress_get, _thostmduserapi.CThostFtdcInputExecOrderField_MacAddress_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcInputExecOrderField_swiginit(self, _thostmduserapi.new_CThostFtdcInputExecOrderField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcInputExecOrderField

# Register CThostFtdcInputExecOrderField in _thostmduserapi:
_thostmduserapi.CThostFtdcInputExecOrderField_swigregister(CThostFtdcInputExecOrderField)

class CThostFtdcInputExecOrderActionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcInputExecOrderActionField_BrokerID_get, _thostmduserapi.CThostFtdcInputExecOrderActionField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcInputExecOrderActionField_InvestorID_get, _thostmduserapi.CThostFtdcInputExecOrderActionField_InvestorID_set)
    ExecOrderActionRef = property(_thostmduserapi.CThostFtdcInputExecOrderActionField_ExecOrderActionRef_get, _thostmduserapi.CThostFtdcInputExecOrderActionField_ExecOrderActionRef_set)
    ExecOrderRef = property(_thostmduserapi.CThostFtdcInputExecOrderActionField_ExecOrderRef_get, _thostmduserapi.CThostFtdcInputExecOrderActionField_ExecOrderRef_set)
    RequestID = property(_thostmduserapi.CThostFtdcInputExecOrderActionField_RequestID_get, _thostmduserapi.CThostFtdcInputExecOrderActionField_RequestID_set)
    FrontID = property(_thostmduserapi.CThostFtdcInputExecOrderActionField_FrontID_get, _thostmduserapi.CThostFtdcInputExecOrderActionField_FrontID_set)
    SessionID = property(_thostmduserapi.CThostFtdcInputExecOrderActionField_SessionID_get, _thostmduserapi.CThostFtdcInputExecOrderActionField_SessionID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcInputExecOrderActionField_ExchangeID_get, _thostmduserapi.CThostFtdcInputExecOrderActionField_ExchangeID_set)
    ExecOrderSysID = property(_thostmduserapi.CThostFtdcInputExecOrderActionField_ExecOrderSysID_get, _thostmduserapi.CThostFtdcInputExecOrderActionField_ExecOrderSysID_set)
    ActionFlag = property(_thostmduserapi.CThostFtdcInputExecOrderActionField_ActionFlag_get, _thostmduserapi.CThostFtdcInputExecOrderActionField_ActionFlag_set)
    UserID = property(_thostmduserapi.CThostFtdcInputExecOrderActionField_UserID_get, _thostmduserapi.CThostFtdcInputExecOrderActionField_UserID_set)
    InstrumentID = property(_thostmduserapi.CThostFtdcInputExecOrderActionField_InstrumentID_get, _thostmduserapi.CThostFtdcInputExecOrderActionField_InstrumentID_set)
    InvestUnitID = property(_thostmduserapi.CThostFtdcInputExecOrderActionField_InvestUnitID_get, _thostmduserapi.CThostFtdcInputExecOrderActionField_InvestUnitID_set)
    IPAddress = property(_thostmduserapi.CThostFtdcInputExecOrderActionField_IPAddress_get, _thostmduserapi.CThostFtdcInputExecOrderActionField_IPAddress_set)
    MacAddress = property(_thostmduserapi.CThostFtdcInputExecOrderActionField_MacAddress_get, _thostmduserapi.CThostFtdcInputExecOrderActionField_MacAddress_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcInputExecOrderActionField_swiginit(self, _thostmduserapi.new_CThostFtdcInputExecOrderActionField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcInputExecOrderActionField

# Register CThostFtdcInputExecOrderActionField in _thostmduserapi:
_thostmduserapi.CThostFtdcInputExecOrderActionField_swigregister(CThostFtdcInputExecOrderActionField)

class CThostFtdcExecOrderField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcExecOrderField_BrokerID_get, _thostmduserapi.CThostFtdcExecOrderField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcExecOrderField_InvestorID_get, _thostmduserapi.CThostFtdcExecOrderField_InvestorID_set)
    InstrumentID = property(_thostmduserapi.CThostFtdcExecOrderField_InstrumentID_get, _thostmduserapi.CThostFtdcExecOrderField_InstrumentID_set)
    ExecOrderRef = property(_thostmduserapi.CThostFtdcExecOrderField_ExecOrderRef_get, _thostmduserapi.CThostFtdcExecOrderField_ExecOrderRef_set)
    UserID = property(_thostmduserapi.CThostFtdcExecOrderField_UserID_get, _thostmduserapi.CThostFtdcExecOrderField_UserID_set)
    Volume = property(_thostmduserapi.CThostFtdcExecOrderField_Volume_get, _thostmduserapi.CThostFtdcExecOrderField_Volume_set)
    RequestID = property(_thostmduserapi.CThostFtdcExecOrderField_RequestID_get, _thostmduserapi.CThostFtdcExecOrderField_RequestID_set)
    BusinessUnit = property(_thostmduserapi.CThostFtdcExecOrderField_BusinessUnit_get, _thostmduserapi.CThostFtdcExecOrderField_BusinessUnit_set)
    OffsetFlag = property(_thostmduserapi.CThostFtdcExecOrderField_OffsetFlag_get, _thostmduserapi.CThostFtdcExecOrderField_OffsetFlag_set)
    HedgeFlag = property(_thostmduserapi.CThostFtdcExecOrderField_HedgeFlag_get, _thostmduserapi.CThostFtdcExecOrderField_HedgeFlag_set)
    ActionType = property(_thostmduserapi.CThostFtdcExecOrderField_ActionType_get, _thostmduserapi.CThostFtdcExecOrderField_ActionType_set)
    PosiDirection = property(_thostmduserapi.CThostFtdcExecOrderField_PosiDirection_get, _thostmduserapi.CThostFtdcExecOrderField_PosiDirection_set)
    ReservePositionFlag = property(_thostmduserapi.CThostFtdcExecOrderField_ReservePositionFlag_get, _thostmduserapi.CThostFtdcExecOrderField_ReservePositionFlag_set)
    CloseFlag = property(_thostmduserapi.CThostFtdcExecOrderField_CloseFlag_get, _thostmduserapi.CThostFtdcExecOrderField_CloseFlag_set)
    ExecOrderLocalID = property(_thostmduserapi.CThostFtdcExecOrderField_ExecOrderLocalID_get, _thostmduserapi.CThostFtdcExecOrderField_ExecOrderLocalID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcExecOrderField_ExchangeID_get, _thostmduserapi.CThostFtdcExecOrderField_ExchangeID_set)
    ParticipantID = property(_thostmduserapi.CThostFtdcExecOrderField_ParticipantID_get, _thostmduserapi.CThostFtdcExecOrderField_ParticipantID_set)
    ClientID = property(_thostmduserapi.CThostFtdcExecOrderField_ClientID_get, _thostmduserapi.CThostFtdcExecOrderField_ClientID_set)
    ExchangeInstID = property(_thostmduserapi.CThostFtdcExecOrderField_ExchangeInstID_get, _thostmduserapi.CThostFtdcExecOrderField_ExchangeInstID_set)
    TraderID = property(_thostmduserapi.CThostFtdcExecOrderField_TraderID_get, _thostmduserapi.CThostFtdcExecOrderField_TraderID_set)
    InstallID = property(_thostmduserapi.CThostFtdcExecOrderField_InstallID_get, _thostmduserapi.CThostFtdcExecOrderField_InstallID_set)
    OrderSubmitStatus = property(_thostmduserapi.CThostFtdcExecOrderField_OrderSubmitStatus_get, _thostmduserapi.CThostFtdcExecOrderField_OrderSubmitStatus_set)
    NotifySequence = property(_thostmduserapi.CThostFtdcExecOrderField_NotifySequence_get, _thostmduserapi.CThostFtdcExecOrderField_NotifySequence_set)
    TradingDay = property(_thostmduserapi.CThostFtdcExecOrderField_TradingDay_get, _thostmduserapi.CThostFtdcExecOrderField_TradingDay_set)
    SettlementID = property(_thostmduserapi.CThostFtdcExecOrderField_SettlementID_get, _thostmduserapi.CThostFtdcExecOrderField_SettlementID_set)
    ExecOrderSysID = property(_thostmduserapi.CThostFtdcExecOrderField_ExecOrderSysID_get, _thostmduserapi.CThostFtdcExecOrderField_ExecOrderSysID_set)
    InsertDate = property(_thostmduserapi.CThostFtdcExecOrderField_InsertDate_get, _thostmduserapi.CThostFtdcExecOrderField_InsertDate_set)
    InsertTime = property(_thostmduserapi.CThostFtdcExecOrderField_InsertTime_get, _thostmduserapi.CThostFtdcExecOrderField_InsertTime_set)
    CancelTime = property(_thostmduserapi.CThostFtdcExecOrderField_CancelTime_get, _thostmduserapi.CThostFtdcExecOrderField_CancelTime_set)
    ExecResult = property(_thostmduserapi.CThostFtdcExecOrderField_ExecResult_get, _thostmduserapi.CThostFtdcExecOrderField_ExecResult_set)
    ClearingPartID = property(_thostmduserapi.CThostFtdcExecOrderField_ClearingPartID_get, _thostmduserapi.CThostFtdcExecOrderField_ClearingPartID_set)
    SequenceNo = property(_thostmduserapi.CThostFtdcExecOrderField_SequenceNo_get, _thostmduserapi.CThostFtdcExecOrderField_SequenceNo_set)
    FrontID = property(_thostmduserapi.CThostFtdcExecOrderField_FrontID_get, _thostmduserapi.CThostFtdcExecOrderField_FrontID_set)
    SessionID = property(_thostmduserapi.CThostFtdcExecOrderField_SessionID_get, _thostmduserapi.CThostFtdcExecOrderField_SessionID_set)
    UserProductInfo = property(_thostmduserapi.CThostFtdcExecOrderField_UserProductInfo_get, _thostmduserapi.CThostFtdcExecOrderField_UserProductInfo_set)
    StatusMsg = property(_thostmduserapi.CThostFtdcExecOrderField_StatusMsg_get, _thostmduserapi.CThostFtdcExecOrderField_StatusMsg_set)
    ActiveUserID = property(_thostmduserapi.CThostFtdcExecOrderField_ActiveUserID_get, _thostmduserapi.CThostFtdcExecOrderField_ActiveUserID_set)
    BrokerExecOrderSeq = property(_thostmduserapi.CThostFtdcExecOrderField_BrokerExecOrderSeq_get, _thostmduserapi.CThostFtdcExecOrderField_BrokerExecOrderSeq_set)
    BranchID = property(_thostmduserapi.CThostFtdcExecOrderField_BranchID_get, _thostmduserapi.CThostFtdcExecOrderField_BranchID_set)
    InvestUnitID = property(_thostmduserapi.CThostFtdcExecOrderField_InvestUnitID_get, _thostmduserapi.CThostFtdcExecOrderField_InvestUnitID_set)
    AccountID = property(_thostmduserapi.CThostFtdcExecOrderField_AccountID_get, _thostmduserapi.CThostFtdcExecOrderField_AccountID_set)
    CurrencyID = property(_thostmduserapi.CThostFtdcExecOrderField_CurrencyID_get, _thostmduserapi.CThostFtdcExecOrderField_CurrencyID_set)
    IPAddress = property(_thostmduserapi.CThostFtdcExecOrderField_IPAddress_get, _thostmduserapi.CThostFtdcExecOrderField_IPAddress_set)
    MacAddress = property(_thostmduserapi.CThostFtdcExecOrderField_MacAddress_get, _thostmduserapi.CThostFtdcExecOrderField_MacAddress_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcExecOrderField_swiginit(self, _thostmduserapi.new_CThostFtdcExecOrderField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcExecOrderField

# Register CThostFtdcExecOrderField in _thostmduserapi:
_thostmduserapi.CThostFtdcExecOrderField_swigregister(CThostFtdcExecOrderField)

class CThostFtdcExecOrderActionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcExecOrderActionField_BrokerID_get, _thostmduserapi.CThostFtdcExecOrderActionField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcExecOrderActionField_InvestorID_get, _thostmduserapi.CThostFtdcExecOrderActionField_InvestorID_set)
    ExecOrderActionRef = property(_thostmduserapi.CThostFtdcExecOrderActionField_ExecOrderActionRef_get, _thostmduserapi.CThostFtdcExecOrderActionField_ExecOrderActionRef_set)
    ExecOrderRef = property(_thostmduserapi.CThostFtdcExecOrderActionField_ExecOrderRef_get, _thostmduserapi.CThostFtdcExecOrderActionField_ExecOrderRef_set)
    RequestID = property(_thostmduserapi.CThostFtdcExecOrderActionField_RequestID_get, _thostmduserapi.CThostFtdcExecOrderActionField_RequestID_set)
    FrontID = property(_thostmduserapi.CThostFtdcExecOrderActionField_FrontID_get, _thostmduserapi.CThostFtdcExecOrderActionField_FrontID_set)
    SessionID = property(_thostmduserapi.CThostFtdcExecOrderActionField_SessionID_get, _thostmduserapi.CThostFtdcExecOrderActionField_SessionID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcExecOrderActionField_ExchangeID_get, _thostmduserapi.CThostFtdcExecOrderActionField_ExchangeID_set)
    ExecOrderSysID = property(_thostmduserapi.CThostFtdcExecOrderActionField_ExecOrderSysID_get, _thostmduserapi.CThostFtdcExecOrderActionField_ExecOrderSysID_set)
    ActionFlag = property(_thostmduserapi.CThostFtdcExecOrderActionField_ActionFlag_get, _thostmduserapi.CThostFtdcExecOrderActionField_ActionFlag_set)
    ActionDate = property(_thostmduserapi.CThostFtdcExecOrderActionField_ActionDate_get, _thostmduserapi.CThostFtdcExecOrderActionField_ActionDate_set)
    ActionTime = property(_thostmduserapi.CThostFtdcExecOrderActionField_ActionTime_get, _thostmduserapi.CThostFtdcExecOrderActionField_ActionTime_set)
    TraderID = property(_thostmduserapi.CThostFtdcExecOrderActionField_TraderID_get, _thostmduserapi.CThostFtdcExecOrderActionField_TraderID_set)
    InstallID = property(_thostmduserapi.CThostFtdcExecOrderActionField_InstallID_get, _thostmduserapi.CThostFtdcExecOrderActionField_InstallID_set)
    ExecOrderLocalID = property(_thostmduserapi.CThostFtdcExecOrderActionField_ExecOrderLocalID_get, _thostmduserapi.CThostFtdcExecOrderActionField_ExecOrderLocalID_set)
    ActionLocalID = property(_thostmduserapi.CThostFtdcExecOrderActionField_ActionLocalID_get, _thostmduserapi.CThostFtdcExecOrderActionField_ActionLocalID_set)
    ParticipantID = property(_thostmduserapi.CThostFtdcExecOrderActionField_ParticipantID_get, _thostmduserapi.CThostFtdcExecOrderActionField_ParticipantID_set)
    ClientID = property(_thostmduserapi.CThostFtdcExecOrderActionField_ClientID_get, _thostmduserapi.CThostFtdcExecOrderActionField_ClientID_set)
    BusinessUnit = property(_thostmduserapi.CThostFtdcExecOrderActionField_BusinessUnit_get, _thostmduserapi.CThostFtdcExecOrderActionField_BusinessUnit_set)
    OrderActionStatus = property(_thostmduserapi.CThostFtdcExecOrderActionField_OrderActionStatus_get, _thostmduserapi.CThostFtdcExecOrderActionField_OrderActionStatus_set)
    UserID = property(_thostmduserapi.CThostFtdcExecOrderActionField_UserID_get, _thostmduserapi.CThostFtdcExecOrderActionField_UserID_set)
    ActionType = property(_thostmduserapi.CThostFtdcExecOrderActionField_ActionType_get, _thostmduserapi.CThostFtdcExecOrderActionField_ActionType_set)
    StatusMsg = property(_thostmduserapi.CThostFtdcExecOrderActionField_StatusMsg_get, _thostmduserapi.CThostFtdcExecOrderActionField_StatusMsg_set)
    InstrumentID = property(_thostmduserapi.CThostFtdcExecOrderActionField_InstrumentID_get, _thostmduserapi.CThostFtdcExecOrderActionField_InstrumentID_set)
    BranchID = property(_thostmduserapi.CThostFtdcExecOrderActionField_BranchID_get, _thostmduserapi.CThostFtdcExecOrderActionField_BranchID_set)
    InvestUnitID = property(_thostmduserapi.CThostFtdcExecOrderActionField_InvestUnitID_get, _thostmduserapi.CThostFtdcExecOrderActionField_InvestUnitID_set)
    IPAddress = property(_thostmduserapi.CThostFtdcExecOrderActionField_IPAddress_get, _thostmduserapi.CThostFtdcExecOrderActionField_IPAddress_set)
    MacAddress = property(_thostmduserapi.CThostFtdcExecOrderActionField_MacAddress_get, _thostmduserapi.CThostFtdcExecOrderActionField_MacAddress_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcExecOrderActionField_swiginit(self, _thostmduserapi.new_CThostFtdcExecOrderActionField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcExecOrderActionField

# Register CThostFtdcExecOrderActionField in _thostmduserapi:
_thostmduserapi.CThostFtdcExecOrderActionField_swigregister(CThostFtdcExecOrderActionField)

class CThostFtdcQryExecOrderField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQryExecOrderField_BrokerID_get, _thostmduserapi.CThostFtdcQryExecOrderField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcQryExecOrderField_InvestorID_get, _thostmduserapi.CThostFtdcQryExecOrderField_InvestorID_set)
    InstrumentID = property(_thostmduserapi.CThostFtdcQryExecOrderField_InstrumentID_get, _thostmduserapi.CThostFtdcQryExecOrderField_InstrumentID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcQryExecOrderField_ExchangeID_get, _thostmduserapi.CThostFtdcQryExecOrderField_ExchangeID_set)
    ExecOrderSysID = property(_thostmduserapi.CThostFtdcQryExecOrderField_ExecOrderSysID_get, _thostmduserapi.CThostFtdcQryExecOrderField_ExecOrderSysID_set)
    InsertTimeStart = property(_thostmduserapi.CThostFtdcQryExecOrderField_InsertTimeStart_get, _thostmduserapi.CThostFtdcQryExecOrderField_InsertTimeStart_set)
    InsertTimeEnd = property(_thostmduserapi.CThostFtdcQryExecOrderField_InsertTimeEnd_get, _thostmduserapi.CThostFtdcQryExecOrderField_InsertTimeEnd_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryExecOrderField_swiginit(self, _thostmduserapi.new_CThostFtdcQryExecOrderField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryExecOrderField

# Register CThostFtdcQryExecOrderField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryExecOrderField_swigregister(CThostFtdcQryExecOrderField)

class CThostFtdcExchangeExecOrderField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    Volume = property(_thostmduserapi.CThostFtdcExchangeExecOrderField_Volume_get, _thostmduserapi.CThostFtdcExchangeExecOrderField_Volume_set)
    RequestID = property(_thostmduserapi.CThostFtdcExchangeExecOrderField_RequestID_get, _thostmduserapi.CThostFtdcExchangeExecOrderField_RequestID_set)
    BusinessUnit = property(_thostmduserapi.CThostFtdcExchangeExecOrderField_BusinessUnit_get, _thostmduserapi.CThostFtdcExchangeExecOrderField_BusinessUnit_set)
    OffsetFlag = property(_thostmduserapi.CThostFtdcExchangeExecOrderField_OffsetFlag_get, _thostmduserapi.CThostFtdcExchangeExecOrderField_OffsetFlag_set)
    HedgeFlag = property(_thostmduserapi.CThostFtdcExchangeExecOrderField_HedgeFlag_get, _thostmduserapi.CThostFtdcExchangeExecOrderField_HedgeFlag_set)
    ActionType = property(_thostmduserapi.CThostFtdcExchangeExecOrderField_ActionType_get, _thostmduserapi.CThostFtdcExchangeExecOrderField_ActionType_set)
    PosiDirection = property(_thostmduserapi.CThostFtdcExchangeExecOrderField_PosiDirection_get, _thostmduserapi.CThostFtdcExchangeExecOrderField_PosiDirection_set)
    ReservePositionFlag = property(_thostmduserapi.CThostFtdcExchangeExecOrderField_ReservePositionFlag_get, _thostmduserapi.CThostFtdcExchangeExecOrderField_ReservePositionFlag_set)
    CloseFlag = property(_thostmduserapi.CThostFtdcExchangeExecOrderField_CloseFlag_get, _thostmduserapi.CThostFtdcExchangeExecOrderField_CloseFlag_set)
    ExecOrderLocalID = property(_thostmduserapi.CThostFtdcExchangeExecOrderField_ExecOrderLocalID_get, _thostmduserapi.CThostFtdcExchangeExecOrderField_ExecOrderLocalID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcExchangeExecOrderField_ExchangeID_get, _thostmduserapi.CThostFtdcExchangeExecOrderField_ExchangeID_set)
    ParticipantID = property(_thostmduserapi.CThostFtdcExchangeExecOrderField_ParticipantID_get, _thostmduserapi.CThostFtdcExchangeExecOrderField_ParticipantID_set)
    ClientID = property(_thostmduserapi.CThostFtdcExchangeExecOrderField_ClientID_get, _thostmduserapi.CThostFtdcExchangeExecOrderField_ClientID_set)
    ExchangeInstID = property(_thostmduserapi.CThostFtdcExchangeExecOrderField_ExchangeInstID_get, _thostmduserapi.CThostFtdcExchangeExecOrderField_ExchangeInstID_set)
    TraderID = property(_thostmduserapi.CThostFtdcExchangeExecOrderField_TraderID_get, _thostmduserapi.CThostFtdcExchangeExecOrderField_TraderID_set)
    InstallID = property(_thostmduserapi.CThostFtdcExchangeExecOrderField_InstallID_get, _thostmduserapi.CThostFtdcExchangeExecOrderField_InstallID_set)
    OrderSubmitStatus = property(_thostmduserapi.CThostFtdcExchangeExecOrderField_OrderSubmitStatus_get, _thostmduserapi.CThostFtdcExchangeExecOrderField_OrderSubmitStatus_set)
    NotifySequence = property(_thostmduserapi.CThostFtdcExchangeExecOrderField_NotifySequence_get, _thostmduserapi.CThostFtdcExchangeExecOrderField_NotifySequence_set)
    TradingDay = property(_thostmduserapi.CThostFtdcExchangeExecOrderField_TradingDay_get, _thostmduserapi.CThostFtdcExchangeExecOrderField_TradingDay_set)
    SettlementID = property(_thostmduserapi.CThostFtdcExchangeExecOrderField_SettlementID_get, _thostmduserapi.CThostFtdcExchangeExecOrderField_SettlementID_set)
    ExecOrderSysID = property(_thostmduserapi.CThostFtdcExchangeExecOrderField_ExecOrderSysID_get, _thostmduserapi.CThostFtdcExchangeExecOrderField_ExecOrderSysID_set)
    InsertDate = property(_thostmduserapi.CThostFtdcExchangeExecOrderField_InsertDate_get, _thostmduserapi.CThostFtdcExchangeExecOrderField_InsertDate_set)
    InsertTime = property(_thostmduserapi.CThostFtdcExchangeExecOrderField_InsertTime_get, _thostmduserapi.CThostFtdcExchangeExecOrderField_InsertTime_set)
    CancelTime = property(_thostmduserapi.CThostFtdcExchangeExecOrderField_CancelTime_get, _thostmduserapi.CThostFtdcExchangeExecOrderField_CancelTime_set)
    ExecResult = property(_thostmduserapi.CThostFtdcExchangeExecOrderField_ExecResult_get, _thostmduserapi.CThostFtdcExchangeExecOrderField_ExecResult_set)
    ClearingPartID = property(_thostmduserapi.CThostFtdcExchangeExecOrderField_ClearingPartID_get, _thostmduserapi.CThostFtdcExchangeExecOrderField_ClearingPartID_set)
    SequenceNo = property(_thostmduserapi.CThostFtdcExchangeExecOrderField_SequenceNo_get, _thostmduserapi.CThostFtdcExchangeExecOrderField_SequenceNo_set)
    BranchID = property(_thostmduserapi.CThostFtdcExchangeExecOrderField_BranchID_get, _thostmduserapi.CThostFtdcExchangeExecOrderField_BranchID_set)
    IPAddress = property(_thostmduserapi.CThostFtdcExchangeExecOrderField_IPAddress_get, _thostmduserapi.CThostFtdcExchangeExecOrderField_IPAddress_set)
    MacAddress = property(_thostmduserapi.CThostFtdcExchangeExecOrderField_MacAddress_get, _thostmduserapi.CThostFtdcExchangeExecOrderField_MacAddress_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcExchangeExecOrderField_swiginit(self, _thostmduserapi.new_CThostFtdcExchangeExecOrderField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcExchangeExecOrderField

# Register CThostFtdcExchangeExecOrderField in _thostmduserapi:
_thostmduserapi.CThostFtdcExchangeExecOrderField_swigregister(CThostFtdcExchangeExecOrderField)

class CThostFtdcQryExchangeExecOrderField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ParticipantID = property(_thostmduserapi.CThostFtdcQryExchangeExecOrderField_ParticipantID_get, _thostmduserapi.CThostFtdcQryExchangeExecOrderField_ParticipantID_set)
    ClientID = property(_thostmduserapi.CThostFtdcQryExchangeExecOrderField_ClientID_get, _thostmduserapi.CThostFtdcQryExchangeExecOrderField_ClientID_set)
    ExchangeInstID = property(_thostmduserapi.CThostFtdcQryExchangeExecOrderField_ExchangeInstID_get, _thostmduserapi.CThostFtdcQryExchangeExecOrderField_ExchangeInstID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcQryExchangeExecOrderField_ExchangeID_get, _thostmduserapi.CThostFtdcQryExchangeExecOrderField_ExchangeID_set)
    TraderID = property(_thostmduserapi.CThostFtdcQryExchangeExecOrderField_TraderID_get, _thostmduserapi.CThostFtdcQryExchangeExecOrderField_TraderID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryExchangeExecOrderField_swiginit(self, _thostmduserapi.new_CThostFtdcQryExchangeExecOrderField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryExchangeExecOrderField

# Register CThostFtdcQryExchangeExecOrderField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryExchangeExecOrderField_swigregister(CThostFtdcQryExchangeExecOrderField)

class CThostFtdcQryExecOrderActionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQryExecOrderActionField_BrokerID_get, _thostmduserapi.CThostFtdcQryExecOrderActionField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcQryExecOrderActionField_InvestorID_get, _thostmduserapi.CThostFtdcQryExecOrderActionField_InvestorID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcQryExecOrderActionField_ExchangeID_get, _thostmduserapi.CThostFtdcQryExecOrderActionField_ExchangeID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryExecOrderActionField_swiginit(self, _thostmduserapi.new_CThostFtdcQryExecOrderActionField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryExecOrderActionField

# Register CThostFtdcQryExecOrderActionField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryExecOrderActionField_swigregister(CThostFtdcQryExecOrderActionField)

class CThostFtdcExchangeExecOrderActionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ExchangeID = property(_thostmduserapi.CThostFtdcExchangeExecOrderActionField_ExchangeID_get, _thostmduserapi.CThostFtdcExchangeExecOrderActionField_ExchangeID_set)
    ExecOrderSysID = property(_thostmduserapi.CThostFtdcExchangeExecOrderActionField_ExecOrderSysID_get, _thostmduserapi.CThostFtdcExchangeExecOrderActionField_ExecOrderSysID_set)
    ActionFlag = property(_thostmduserapi.CThostFtdcExchangeExecOrderActionField_ActionFlag_get, _thostmduserapi.CThostFtdcExchangeExecOrderActionField_ActionFlag_set)
    ActionDate = property(_thostmduserapi.CThostFtdcExchangeExecOrderActionField_ActionDate_get, _thostmduserapi.CThostFtdcExchangeExecOrderActionField_ActionDate_set)
    ActionTime = property(_thostmduserapi.CThostFtdcExchangeExecOrderActionField_ActionTime_get, _thostmduserapi.CThostFtdcExchangeExecOrderActionField_ActionTime_set)
    TraderID = property(_thostmduserapi.CThostFtdcExchangeExecOrderActionField_TraderID_get, _thostmduserapi.CThostFtdcExchangeExecOrderActionField_TraderID_set)
    InstallID = property(_thostmduserapi.CThostFtdcExchangeExecOrderActionField_InstallID_get, _thostmduserapi.CThostFtdcExchangeExecOrderActionField_InstallID_set)
    ExecOrderLocalID = property(_thostmduserapi.CThostFtdcExchangeExecOrderActionField_ExecOrderLocalID_get, _thostmduserapi.CThostFtdcExchangeExecOrderActionField_ExecOrderLocalID_set)
    ActionLocalID = property(_thostmduserapi.CThostFtdcExchangeExecOrderActionField_ActionLocalID_get, _thostmduserapi.CThostFtdcExchangeExecOrderActionField_ActionLocalID_set)
    ParticipantID = property(_thostmduserapi.CThostFtdcExchangeExecOrderActionField_ParticipantID_get, _thostmduserapi.CThostFtdcExchangeExecOrderActionField_ParticipantID_set)
    ClientID = property(_thostmduserapi.CThostFtdcExchangeExecOrderActionField_ClientID_get, _thostmduserapi.CThostFtdcExchangeExecOrderActionField_ClientID_set)
    BusinessUnit = property(_thostmduserapi.CThostFtdcExchangeExecOrderActionField_BusinessUnit_get, _thostmduserapi.CThostFtdcExchangeExecOrderActionField_BusinessUnit_set)
    OrderActionStatus = property(_thostmduserapi.CThostFtdcExchangeExecOrderActionField_OrderActionStatus_get, _thostmduserapi.CThostFtdcExchangeExecOrderActionField_OrderActionStatus_set)
    UserID = property(_thostmduserapi.CThostFtdcExchangeExecOrderActionField_UserID_get, _thostmduserapi.CThostFtdcExchangeExecOrderActionField_UserID_set)
    ActionType = property(_thostmduserapi.CThostFtdcExchangeExecOrderActionField_ActionType_get, _thostmduserapi.CThostFtdcExchangeExecOrderActionField_ActionType_set)
    BranchID = property(_thostmduserapi.CThostFtdcExchangeExecOrderActionField_BranchID_get, _thostmduserapi.CThostFtdcExchangeExecOrderActionField_BranchID_set)
    IPAddress = property(_thostmduserapi.CThostFtdcExchangeExecOrderActionField_IPAddress_get, _thostmduserapi.CThostFtdcExchangeExecOrderActionField_IPAddress_set)
    MacAddress = property(_thostmduserapi.CThostFtdcExchangeExecOrderActionField_MacAddress_get, _thostmduserapi.CThostFtdcExchangeExecOrderActionField_MacAddress_set)
    ExchangeInstID = property(_thostmduserapi.CThostFtdcExchangeExecOrderActionField_ExchangeInstID_get, _thostmduserapi.CThostFtdcExchangeExecOrderActionField_ExchangeInstID_set)
    Volume = property(_thostmduserapi.CThostFtdcExchangeExecOrderActionField_Volume_get, _thostmduserapi.CThostFtdcExchangeExecOrderActionField_Volume_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcExchangeExecOrderActionField_swiginit(self, _thostmduserapi.new_CThostFtdcExchangeExecOrderActionField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcExchangeExecOrderActionField

# Register CThostFtdcExchangeExecOrderActionField in _thostmduserapi:
_thostmduserapi.CThostFtdcExchangeExecOrderActionField_swigregister(CThostFtdcExchangeExecOrderActionField)

class CThostFtdcQryExchangeExecOrderActionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ParticipantID = property(_thostmduserapi.CThostFtdcQryExchangeExecOrderActionField_ParticipantID_get, _thostmduserapi.CThostFtdcQryExchangeExecOrderActionField_ParticipantID_set)
    ClientID = property(_thostmduserapi.CThostFtdcQryExchangeExecOrderActionField_ClientID_get, _thostmduserapi.CThostFtdcQryExchangeExecOrderActionField_ClientID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcQryExchangeExecOrderActionField_ExchangeID_get, _thostmduserapi.CThostFtdcQryExchangeExecOrderActionField_ExchangeID_set)
    TraderID = property(_thostmduserapi.CThostFtdcQryExchangeExecOrderActionField_TraderID_get, _thostmduserapi.CThostFtdcQryExchangeExecOrderActionField_TraderID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryExchangeExecOrderActionField_swiginit(self, _thostmduserapi.new_CThostFtdcQryExchangeExecOrderActionField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryExchangeExecOrderActionField

# Register CThostFtdcQryExchangeExecOrderActionField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryExchangeExecOrderActionField_swigregister(CThostFtdcQryExchangeExecOrderActionField)

class CThostFtdcErrExecOrderField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcErrExecOrderField_BrokerID_get, _thostmduserapi.CThostFtdcErrExecOrderField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcErrExecOrderField_InvestorID_get, _thostmduserapi.CThostFtdcErrExecOrderField_InvestorID_set)
    InstrumentID = property(_thostmduserapi.CThostFtdcErrExecOrderField_InstrumentID_get, _thostmduserapi.CThostFtdcErrExecOrderField_InstrumentID_set)
    ExecOrderRef = property(_thostmduserapi.CThostFtdcErrExecOrderField_ExecOrderRef_get, _thostmduserapi.CThostFtdcErrExecOrderField_ExecOrderRef_set)
    UserID = property(_thostmduserapi.CThostFtdcErrExecOrderField_UserID_get, _thostmduserapi.CThostFtdcErrExecOrderField_UserID_set)
    Volume = property(_thostmduserapi.CThostFtdcErrExecOrderField_Volume_get, _thostmduserapi.CThostFtdcErrExecOrderField_Volume_set)
    RequestID = property(_thostmduserapi.CThostFtdcErrExecOrderField_RequestID_get, _thostmduserapi.CThostFtdcErrExecOrderField_RequestID_set)
    BusinessUnit = property(_thostmduserapi.CThostFtdcErrExecOrderField_BusinessUnit_get, _thostmduserapi.CThostFtdcErrExecOrderField_BusinessUnit_set)
    OffsetFlag = property(_thostmduserapi.CThostFtdcErrExecOrderField_OffsetFlag_get, _thostmduserapi.CThostFtdcErrExecOrderField_OffsetFlag_set)
    HedgeFlag = property(_thostmduserapi.CThostFtdcErrExecOrderField_HedgeFlag_get, _thostmduserapi.CThostFtdcErrExecOrderField_HedgeFlag_set)
    ActionType = property(_thostmduserapi.CThostFtdcErrExecOrderField_ActionType_get, _thostmduserapi.CThostFtdcErrExecOrderField_ActionType_set)
    PosiDirection = property(_thostmduserapi.CThostFtdcErrExecOrderField_PosiDirection_get, _thostmduserapi.CThostFtdcErrExecOrderField_PosiDirection_set)
    ReservePositionFlag = property(_thostmduserapi.CThostFtdcErrExecOrderField_ReservePositionFlag_get, _thostmduserapi.CThostFtdcErrExecOrderField_ReservePositionFlag_set)
    CloseFlag = property(_thostmduserapi.CThostFtdcErrExecOrderField_CloseFlag_get, _thostmduserapi.CThostFtdcErrExecOrderField_CloseFlag_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcErrExecOrderField_ExchangeID_get, _thostmduserapi.CThostFtdcErrExecOrderField_ExchangeID_set)
    InvestUnitID = property(_thostmduserapi.CThostFtdcErrExecOrderField_InvestUnitID_get, _thostmduserapi.CThostFtdcErrExecOrderField_InvestUnitID_set)
    AccountID = property(_thostmduserapi.CThostFtdcErrExecOrderField_AccountID_get, _thostmduserapi.CThostFtdcErrExecOrderField_AccountID_set)
    CurrencyID = property(_thostmduserapi.CThostFtdcErrExecOrderField_CurrencyID_get, _thostmduserapi.CThostFtdcErrExecOrderField_CurrencyID_set)
    ClientID = property(_thostmduserapi.CThostFtdcErrExecOrderField_ClientID_get, _thostmduserapi.CThostFtdcErrExecOrderField_ClientID_set)
    IPAddress = property(_thostmduserapi.CThostFtdcErrExecOrderField_IPAddress_get, _thostmduserapi.CThostFtdcErrExecOrderField_IPAddress_set)
    MacAddress = property(_thostmduserapi.CThostFtdcErrExecOrderField_MacAddress_get, _thostmduserapi.CThostFtdcErrExecOrderField_MacAddress_set)
    ErrorID = property(_thostmduserapi.CThostFtdcErrExecOrderField_ErrorID_get, _thostmduserapi.CThostFtdcErrExecOrderField_ErrorID_set)
    ErrorMsg = property(_thostmduserapi.CThostFtdcErrExecOrderField_ErrorMsg_get, _thostmduserapi.CThostFtdcErrExecOrderField_ErrorMsg_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcErrExecOrderField_swiginit(self, _thostmduserapi.new_CThostFtdcErrExecOrderField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcErrExecOrderField

# Register CThostFtdcErrExecOrderField in _thostmduserapi:
_thostmduserapi.CThostFtdcErrExecOrderField_swigregister(CThostFtdcErrExecOrderField)

class CThostFtdcQryErrExecOrderField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQryErrExecOrderField_BrokerID_get, _thostmduserapi.CThostFtdcQryErrExecOrderField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcQryErrExecOrderField_InvestorID_get, _thostmduserapi.CThostFtdcQryErrExecOrderField_InvestorID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryErrExecOrderField_swiginit(self, _thostmduserapi.new_CThostFtdcQryErrExecOrderField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryErrExecOrderField

# Register CThostFtdcQryErrExecOrderField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryErrExecOrderField_swigregister(CThostFtdcQryErrExecOrderField)

class CThostFtdcErrExecOrderActionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcErrExecOrderActionField_BrokerID_get, _thostmduserapi.CThostFtdcErrExecOrderActionField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcErrExecOrderActionField_InvestorID_get, _thostmduserapi.CThostFtdcErrExecOrderActionField_InvestorID_set)
    ExecOrderActionRef = property(_thostmduserapi.CThostFtdcErrExecOrderActionField_ExecOrderActionRef_get, _thostmduserapi.CThostFtdcErrExecOrderActionField_ExecOrderActionRef_set)
    ExecOrderRef = property(_thostmduserapi.CThostFtdcErrExecOrderActionField_ExecOrderRef_get, _thostmduserapi.CThostFtdcErrExecOrderActionField_ExecOrderRef_set)
    RequestID = property(_thostmduserapi.CThostFtdcErrExecOrderActionField_RequestID_get, _thostmduserapi.CThostFtdcErrExecOrderActionField_RequestID_set)
    FrontID = property(_thostmduserapi.CThostFtdcErrExecOrderActionField_FrontID_get, _thostmduserapi.CThostFtdcErrExecOrderActionField_FrontID_set)
    SessionID = property(_thostmduserapi.CThostFtdcErrExecOrderActionField_SessionID_get, _thostmduserapi.CThostFtdcErrExecOrderActionField_SessionID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcErrExecOrderActionField_ExchangeID_get, _thostmduserapi.CThostFtdcErrExecOrderActionField_ExchangeID_set)
    ExecOrderSysID = property(_thostmduserapi.CThostFtdcErrExecOrderActionField_ExecOrderSysID_get, _thostmduserapi.CThostFtdcErrExecOrderActionField_ExecOrderSysID_set)
    ActionFlag = property(_thostmduserapi.CThostFtdcErrExecOrderActionField_ActionFlag_get, _thostmduserapi.CThostFtdcErrExecOrderActionField_ActionFlag_set)
    UserID = property(_thostmduserapi.CThostFtdcErrExecOrderActionField_UserID_get, _thostmduserapi.CThostFtdcErrExecOrderActionField_UserID_set)
    InstrumentID = property(_thostmduserapi.CThostFtdcErrExecOrderActionField_InstrumentID_get, _thostmduserapi.CThostFtdcErrExecOrderActionField_InstrumentID_set)
    InvestUnitID = property(_thostmduserapi.CThostFtdcErrExecOrderActionField_InvestUnitID_get, _thostmduserapi.CThostFtdcErrExecOrderActionField_InvestUnitID_set)
    IPAddress = property(_thostmduserapi.CThostFtdcErrExecOrderActionField_IPAddress_get, _thostmduserapi.CThostFtdcErrExecOrderActionField_IPAddress_set)
    MacAddress = property(_thostmduserapi.CThostFtdcErrExecOrderActionField_MacAddress_get, _thostmduserapi.CThostFtdcErrExecOrderActionField_MacAddress_set)
    ErrorID = property(_thostmduserapi.CThostFtdcErrExecOrderActionField_ErrorID_get, _thostmduserapi.CThostFtdcErrExecOrderActionField_ErrorID_set)
    ErrorMsg = property(_thostmduserapi.CThostFtdcErrExecOrderActionField_ErrorMsg_get, _thostmduserapi.CThostFtdcErrExecOrderActionField_ErrorMsg_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcErrExecOrderActionField_swiginit(self, _thostmduserapi.new_CThostFtdcErrExecOrderActionField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcErrExecOrderActionField

# Register CThostFtdcErrExecOrderActionField in _thostmduserapi:
_thostmduserapi.CThostFtdcErrExecOrderActionField_swigregister(CThostFtdcErrExecOrderActionField)

class CThostFtdcQryErrExecOrderActionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQryErrExecOrderActionField_BrokerID_get, _thostmduserapi.CThostFtdcQryErrExecOrderActionField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcQryErrExecOrderActionField_InvestorID_get, _thostmduserapi.CThostFtdcQryErrExecOrderActionField_InvestorID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryErrExecOrderActionField_swiginit(self, _thostmduserapi.new_CThostFtdcQryErrExecOrderActionField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryErrExecOrderActionField

# Register CThostFtdcQryErrExecOrderActionField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryErrExecOrderActionField_swigregister(CThostFtdcQryErrExecOrderActionField)

class CThostFtdcOptionInstrTradingRightField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    InstrumentID = property(_thostmduserapi.CThostFtdcOptionInstrTradingRightField_InstrumentID_get, _thostmduserapi.CThostFtdcOptionInstrTradingRightField_InstrumentID_set)
    InvestorRange = property(_thostmduserapi.CThostFtdcOptionInstrTradingRightField_InvestorRange_get, _thostmduserapi.CThostFtdcOptionInstrTradingRightField_InvestorRange_set)
    BrokerID = property(_thostmduserapi.CThostFtdcOptionInstrTradingRightField_BrokerID_get, _thostmduserapi.CThostFtdcOptionInstrTradingRightField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcOptionInstrTradingRightField_InvestorID_get, _thostmduserapi.CThostFtdcOptionInstrTradingRightField_InvestorID_set)
    Direction = property(_thostmduserapi.CThostFtdcOptionInstrTradingRightField_Direction_get, _thostmduserapi.CThostFtdcOptionInstrTradingRightField_Direction_set)
    TradingRight = property(_thostmduserapi.CThostFtdcOptionInstrTradingRightField_TradingRight_get, _thostmduserapi.CThostFtdcOptionInstrTradingRightField_TradingRight_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcOptionInstrTradingRightField_swiginit(self, _thostmduserapi.new_CThostFtdcOptionInstrTradingRightField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcOptionInstrTradingRightField

# Register CThostFtdcOptionInstrTradingRightField in _thostmduserapi:
_thostmduserapi.CThostFtdcOptionInstrTradingRightField_swigregister(CThostFtdcOptionInstrTradingRightField)

class CThostFtdcQryOptionInstrTradingRightField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQryOptionInstrTradingRightField_BrokerID_get, _thostmduserapi.CThostFtdcQryOptionInstrTradingRightField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcQryOptionInstrTradingRightField_InvestorID_get, _thostmduserapi.CThostFtdcQryOptionInstrTradingRightField_InvestorID_set)
    InstrumentID = property(_thostmduserapi.CThostFtdcQryOptionInstrTradingRightField_InstrumentID_get, _thostmduserapi.CThostFtdcQryOptionInstrTradingRightField_InstrumentID_set)
    Direction = property(_thostmduserapi.CThostFtdcQryOptionInstrTradingRightField_Direction_get, _thostmduserapi.CThostFtdcQryOptionInstrTradingRightField_Direction_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryOptionInstrTradingRightField_swiginit(self, _thostmduserapi.new_CThostFtdcQryOptionInstrTradingRightField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryOptionInstrTradingRightField

# Register CThostFtdcQryOptionInstrTradingRightField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryOptionInstrTradingRightField_swigregister(CThostFtdcQryOptionInstrTradingRightField)

class CThostFtdcInputForQuoteField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcInputForQuoteField_BrokerID_get, _thostmduserapi.CThostFtdcInputForQuoteField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcInputForQuoteField_InvestorID_get, _thostmduserapi.CThostFtdcInputForQuoteField_InvestorID_set)
    InstrumentID = property(_thostmduserapi.CThostFtdcInputForQuoteField_InstrumentID_get, _thostmduserapi.CThostFtdcInputForQuoteField_InstrumentID_set)
    ForQuoteRef = property(_thostmduserapi.CThostFtdcInputForQuoteField_ForQuoteRef_get, _thostmduserapi.CThostFtdcInputForQuoteField_ForQuoteRef_set)
    UserID = property(_thostmduserapi.CThostFtdcInputForQuoteField_UserID_get, _thostmduserapi.CThostFtdcInputForQuoteField_UserID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcInputForQuoteField_ExchangeID_get, _thostmduserapi.CThostFtdcInputForQuoteField_ExchangeID_set)
    InvestUnitID = property(_thostmduserapi.CThostFtdcInputForQuoteField_InvestUnitID_get, _thostmduserapi.CThostFtdcInputForQuoteField_InvestUnitID_set)
    IPAddress = property(_thostmduserapi.CThostFtdcInputForQuoteField_IPAddress_get, _thostmduserapi.CThostFtdcInputForQuoteField_IPAddress_set)
    MacAddress = property(_thostmduserapi.CThostFtdcInputForQuoteField_MacAddress_get, _thostmduserapi.CThostFtdcInputForQuoteField_MacAddress_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcInputForQuoteField_swiginit(self, _thostmduserapi.new_CThostFtdcInputForQuoteField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcInputForQuoteField

# Register CThostFtdcInputForQuoteField in _thostmduserapi:
_thostmduserapi.CThostFtdcInputForQuoteField_swigregister(CThostFtdcInputForQuoteField)

class CThostFtdcForQuoteField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcForQuoteField_BrokerID_get, _thostmduserapi.CThostFtdcForQuoteField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcForQuoteField_InvestorID_get, _thostmduserapi.CThostFtdcForQuoteField_InvestorID_set)
    InstrumentID = property(_thostmduserapi.CThostFtdcForQuoteField_InstrumentID_get, _thostmduserapi.CThostFtdcForQuoteField_InstrumentID_set)
    ForQuoteRef = property(_thostmduserapi.CThostFtdcForQuoteField_ForQuoteRef_get, _thostmduserapi.CThostFtdcForQuoteField_ForQuoteRef_set)
    UserID = property(_thostmduserapi.CThostFtdcForQuoteField_UserID_get, _thostmduserapi.CThostFtdcForQuoteField_UserID_set)
    ForQuoteLocalID = property(_thostmduserapi.CThostFtdcForQuoteField_ForQuoteLocalID_get, _thostmduserapi.CThostFtdcForQuoteField_ForQuoteLocalID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcForQuoteField_ExchangeID_get, _thostmduserapi.CThostFtdcForQuoteField_ExchangeID_set)
    ParticipantID = property(_thostmduserapi.CThostFtdcForQuoteField_ParticipantID_get, _thostmduserapi.CThostFtdcForQuoteField_ParticipantID_set)
    ClientID = property(_thostmduserapi.CThostFtdcForQuoteField_ClientID_get, _thostmduserapi.CThostFtdcForQuoteField_ClientID_set)
    ExchangeInstID = property(_thostmduserapi.CThostFtdcForQuoteField_ExchangeInstID_get, _thostmduserapi.CThostFtdcForQuoteField_ExchangeInstID_set)
    TraderID = property(_thostmduserapi.CThostFtdcForQuoteField_TraderID_get, _thostmduserapi.CThostFtdcForQuoteField_TraderID_set)
    InstallID = property(_thostmduserapi.CThostFtdcForQuoteField_InstallID_get, _thostmduserapi.CThostFtdcForQuoteField_InstallID_set)
    InsertDate = property(_thostmduserapi.CThostFtdcForQuoteField_InsertDate_get, _thostmduserapi.CThostFtdcForQuoteField_InsertDate_set)
    InsertTime = property(_thostmduserapi.CThostFtdcForQuoteField_InsertTime_get, _thostmduserapi.CThostFtdcForQuoteField_InsertTime_set)
    ForQuoteStatus = property(_thostmduserapi.CThostFtdcForQuoteField_ForQuoteStatus_get, _thostmduserapi.CThostFtdcForQuoteField_ForQuoteStatus_set)
    FrontID = property(_thostmduserapi.CThostFtdcForQuoteField_FrontID_get, _thostmduserapi.CThostFtdcForQuoteField_FrontID_set)
    SessionID = property(_thostmduserapi.CThostFtdcForQuoteField_SessionID_get, _thostmduserapi.CThostFtdcForQuoteField_SessionID_set)
    StatusMsg = property(_thostmduserapi.CThostFtdcForQuoteField_StatusMsg_get, _thostmduserapi.CThostFtdcForQuoteField_StatusMsg_set)
    ActiveUserID = property(_thostmduserapi.CThostFtdcForQuoteField_ActiveUserID_get, _thostmduserapi.CThostFtdcForQuoteField_ActiveUserID_set)
    BrokerForQutoSeq = property(_thostmduserapi.CThostFtdcForQuoteField_BrokerForQutoSeq_get, _thostmduserapi.CThostFtdcForQuoteField_BrokerForQutoSeq_set)
    InvestUnitID = property(_thostmduserapi.CThostFtdcForQuoteField_InvestUnitID_get, _thostmduserapi.CThostFtdcForQuoteField_InvestUnitID_set)
    IPAddress = property(_thostmduserapi.CThostFtdcForQuoteField_IPAddress_get, _thostmduserapi.CThostFtdcForQuoteField_IPAddress_set)
    MacAddress = property(_thostmduserapi.CThostFtdcForQuoteField_MacAddress_get, _thostmduserapi.CThostFtdcForQuoteField_MacAddress_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcForQuoteField_swiginit(self, _thostmduserapi.new_CThostFtdcForQuoteField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcForQuoteField

# Register CThostFtdcForQuoteField in _thostmduserapi:
_thostmduserapi.CThostFtdcForQuoteField_swigregister(CThostFtdcForQuoteField)

class CThostFtdcQryForQuoteField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQryForQuoteField_BrokerID_get, _thostmduserapi.CThostFtdcQryForQuoteField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcQryForQuoteField_InvestorID_get, _thostmduserapi.CThostFtdcQryForQuoteField_InvestorID_set)
    InstrumentID = property(_thostmduserapi.CThostFtdcQryForQuoteField_InstrumentID_get, _thostmduserapi.CThostFtdcQryForQuoteField_InstrumentID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcQryForQuoteField_ExchangeID_get, _thostmduserapi.CThostFtdcQryForQuoteField_ExchangeID_set)
    InsertTimeStart = property(_thostmduserapi.CThostFtdcQryForQuoteField_InsertTimeStart_get, _thostmduserapi.CThostFtdcQryForQuoteField_InsertTimeStart_set)
    InsertTimeEnd = property(_thostmduserapi.CThostFtdcQryForQuoteField_InsertTimeEnd_get, _thostmduserapi.CThostFtdcQryForQuoteField_InsertTimeEnd_set)
    InvestUnitID = property(_thostmduserapi.CThostFtdcQryForQuoteField_InvestUnitID_get, _thostmduserapi.CThostFtdcQryForQuoteField_InvestUnitID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryForQuoteField_swiginit(self, _thostmduserapi.new_CThostFtdcQryForQuoteField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryForQuoteField

# Register CThostFtdcQryForQuoteField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryForQuoteField_swigregister(CThostFtdcQryForQuoteField)

class CThostFtdcExchangeForQuoteField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ForQuoteLocalID = property(_thostmduserapi.CThostFtdcExchangeForQuoteField_ForQuoteLocalID_get, _thostmduserapi.CThostFtdcExchangeForQuoteField_ForQuoteLocalID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcExchangeForQuoteField_ExchangeID_get, _thostmduserapi.CThostFtdcExchangeForQuoteField_ExchangeID_set)
    ParticipantID = property(_thostmduserapi.CThostFtdcExchangeForQuoteField_ParticipantID_get, _thostmduserapi.CThostFtdcExchangeForQuoteField_ParticipantID_set)
    ClientID = property(_thostmduserapi.CThostFtdcExchangeForQuoteField_ClientID_get, _thostmduserapi.CThostFtdcExchangeForQuoteField_ClientID_set)
    ExchangeInstID = property(_thostmduserapi.CThostFtdcExchangeForQuoteField_ExchangeInstID_get, _thostmduserapi.CThostFtdcExchangeForQuoteField_ExchangeInstID_set)
    TraderID = property(_thostmduserapi.CThostFtdcExchangeForQuoteField_TraderID_get, _thostmduserapi.CThostFtdcExchangeForQuoteField_TraderID_set)
    InstallID = property(_thostmduserapi.CThostFtdcExchangeForQuoteField_InstallID_get, _thostmduserapi.CThostFtdcExchangeForQuoteField_InstallID_set)
    InsertDate = property(_thostmduserapi.CThostFtdcExchangeForQuoteField_InsertDate_get, _thostmduserapi.CThostFtdcExchangeForQuoteField_InsertDate_set)
    InsertTime = property(_thostmduserapi.CThostFtdcExchangeForQuoteField_InsertTime_get, _thostmduserapi.CThostFtdcExchangeForQuoteField_InsertTime_set)
    ForQuoteStatus = property(_thostmduserapi.CThostFtdcExchangeForQuoteField_ForQuoteStatus_get, _thostmduserapi.CThostFtdcExchangeForQuoteField_ForQuoteStatus_set)
    IPAddress = property(_thostmduserapi.CThostFtdcExchangeForQuoteField_IPAddress_get, _thostmduserapi.CThostFtdcExchangeForQuoteField_IPAddress_set)
    MacAddress = property(_thostmduserapi.CThostFtdcExchangeForQuoteField_MacAddress_get, _thostmduserapi.CThostFtdcExchangeForQuoteField_MacAddress_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcExchangeForQuoteField_swiginit(self, _thostmduserapi.new_CThostFtdcExchangeForQuoteField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcExchangeForQuoteField

# Register CThostFtdcExchangeForQuoteField in _thostmduserapi:
_thostmduserapi.CThostFtdcExchangeForQuoteField_swigregister(CThostFtdcExchangeForQuoteField)

class CThostFtdcQryExchangeForQuoteField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ParticipantID = property(_thostmduserapi.CThostFtdcQryExchangeForQuoteField_ParticipantID_get, _thostmduserapi.CThostFtdcQryExchangeForQuoteField_ParticipantID_set)
    ClientID = property(_thostmduserapi.CThostFtdcQryExchangeForQuoteField_ClientID_get, _thostmduserapi.CThostFtdcQryExchangeForQuoteField_ClientID_set)
    ExchangeInstID = property(_thostmduserapi.CThostFtdcQryExchangeForQuoteField_ExchangeInstID_get, _thostmduserapi.CThostFtdcQryExchangeForQuoteField_ExchangeInstID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcQryExchangeForQuoteField_ExchangeID_get, _thostmduserapi.CThostFtdcQryExchangeForQuoteField_ExchangeID_set)
    TraderID = property(_thostmduserapi.CThostFtdcQryExchangeForQuoteField_TraderID_get, _thostmduserapi.CThostFtdcQryExchangeForQuoteField_TraderID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryExchangeForQuoteField_swiginit(self, _thostmduserapi.new_CThostFtdcQryExchangeForQuoteField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryExchangeForQuoteField

# Register CThostFtdcQryExchangeForQuoteField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryExchangeForQuoteField_swigregister(CThostFtdcQryExchangeForQuoteField)

class CThostFtdcInputQuoteField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcInputQuoteField_BrokerID_get, _thostmduserapi.CThostFtdcInputQuoteField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcInputQuoteField_InvestorID_get, _thostmduserapi.CThostFtdcInputQuoteField_InvestorID_set)
    InstrumentID = property(_thostmduserapi.CThostFtdcInputQuoteField_InstrumentID_get, _thostmduserapi.CThostFtdcInputQuoteField_InstrumentID_set)
    QuoteRef = property(_thostmduserapi.CThostFtdcInputQuoteField_QuoteRef_get, _thostmduserapi.CThostFtdcInputQuoteField_QuoteRef_set)
    UserID = property(_thostmduserapi.CThostFtdcInputQuoteField_UserID_get, _thostmduserapi.CThostFtdcInputQuoteField_UserID_set)
    AskPrice = property(_thostmduserapi.CThostFtdcInputQuoteField_AskPrice_get, _thostmduserapi.CThostFtdcInputQuoteField_AskPrice_set)
    BidPrice = property(_thostmduserapi.CThostFtdcInputQuoteField_BidPrice_get, _thostmduserapi.CThostFtdcInputQuoteField_BidPrice_set)
    AskVolume = property(_thostmduserapi.CThostFtdcInputQuoteField_AskVolume_get, _thostmduserapi.CThostFtdcInputQuoteField_AskVolume_set)
    BidVolume = property(_thostmduserapi.CThostFtdcInputQuoteField_BidVolume_get, _thostmduserapi.CThostFtdcInputQuoteField_BidVolume_set)
    RequestID = property(_thostmduserapi.CThostFtdcInputQuoteField_RequestID_get, _thostmduserapi.CThostFtdcInputQuoteField_RequestID_set)
    BusinessUnit = property(_thostmduserapi.CThostFtdcInputQuoteField_BusinessUnit_get, _thostmduserapi.CThostFtdcInputQuoteField_BusinessUnit_set)
    AskOffsetFlag = property(_thostmduserapi.CThostFtdcInputQuoteField_AskOffsetFlag_get, _thostmduserapi.CThostFtdcInputQuoteField_AskOffsetFlag_set)
    BidOffsetFlag = property(_thostmduserapi.CThostFtdcInputQuoteField_BidOffsetFlag_get, _thostmduserapi.CThostFtdcInputQuoteField_BidOffsetFlag_set)
    AskHedgeFlag = property(_thostmduserapi.CThostFtdcInputQuoteField_AskHedgeFlag_get, _thostmduserapi.CThostFtdcInputQuoteField_AskHedgeFlag_set)
    BidHedgeFlag = property(_thostmduserapi.CThostFtdcInputQuoteField_BidHedgeFlag_get, _thostmduserapi.CThostFtdcInputQuoteField_BidHedgeFlag_set)
    AskOrderRef = property(_thostmduserapi.CThostFtdcInputQuoteField_AskOrderRef_get, _thostmduserapi.CThostFtdcInputQuoteField_AskOrderRef_set)
    BidOrderRef = property(_thostmduserapi.CThostFtdcInputQuoteField_BidOrderRef_get, _thostmduserapi.CThostFtdcInputQuoteField_BidOrderRef_set)
    ForQuoteSysID = property(_thostmduserapi.CThostFtdcInputQuoteField_ForQuoteSysID_get, _thostmduserapi.CThostFtdcInputQuoteField_ForQuoteSysID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcInputQuoteField_ExchangeID_get, _thostmduserapi.CThostFtdcInputQuoteField_ExchangeID_set)
    InvestUnitID = property(_thostmduserapi.CThostFtdcInputQuoteField_InvestUnitID_get, _thostmduserapi.CThostFtdcInputQuoteField_InvestUnitID_set)
    ClientID = property(_thostmduserapi.CThostFtdcInputQuoteField_ClientID_get, _thostmduserapi.CThostFtdcInputQuoteField_ClientID_set)
    IPAddress = property(_thostmduserapi.CThostFtdcInputQuoteField_IPAddress_get, _thostmduserapi.CThostFtdcInputQuoteField_IPAddress_set)
    MacAddress = property(_thostmduserapi.CThostFtdcInputQuoteField_MacAddress_get, _thostmduserapi.CThostFtdcInputQuoteField_MacAddress_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcInputQuoteField_swiginit(self, _thostmduserapi.new_CThostFtdcInputQuoteField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcInputQuoteField

# Register CThostFtdcInputQuoteField in _thostmduserapi:
_thostmduserapi.CThostFtdcInputQuoteField_swigregister(CThostFtdcInputQuoteField)

class CThostFtdcInputQuoteActionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcInputQuoteActionField_BrokerID_get, _thostmduserapi.CThostFtdcInputQuoteActionField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcInputQuoteActionField_InvestorID_get, _thostmduserapi.CThostFtdcInputQuoteActionField_InvestorID_set)
    QuoteActionRef = property(_thostmduserapi.CThostFtdcInputQuoteActionField_QuoteActionRef_get, _thostmduserapi.CThostFtdcInputQuoteActionField_QuoteActionRef_set)
    QuoteRef = property(_thostmduserapi.CThostFtdcInputQuoteActionField_QuoteRef_get, _thostmduserapi.CThostFtdcInputQuoteActionField_QuoteRef_set)
    RequestID = property(_thostmduserapi.CThostFtdcInputQuoteActionField_RequestID_get, _thostmduserapi.CThostFtdcInputQuoteActionField_RequestID_set)
    FrontID = property(_thostmduserapi.CThostFtdcInputQuoteActionField_FrontID_get, _thostmduserapi.CThostFtdcInputQuoteActionField_FrontID_set)
    SessionID = property(_thostmduserapi.CThostFtdcInputQuoteActionField_SessionID_get, _thostmduserapi.CThostFtdcInputQuoteActionField_SessionID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcInputQuoteActionField_ExchangeID_get, _thostmduserapi.CThostFtdcInputQuoteActionField_ExchangeID_set)
    QuoteSysID = property(_thostmduserapi.CThostFtdcInputQuoteActionField_QuoteSysID_get, _thostmduserapi.CThostFtdcInputQuoteActionField_QuoteSysID_set)
    ActionFlag = property(_thostmduserapi.CThostFtdcInputQuoteActionField_ActionFlag_get, _thostmduserapi.CThostFtdcInputQuoteActionField_ActionFlag_set)
    UserID = property(_thostmduserapi.CThostFtdcInputQuoteActionField_UserID_get, _thostmduserapi.CThostFtdcInputQuoteActionField_UserID_set)
    InstrumentID = property(_thostmduserapi.CThostFtdcInputQuoteActionField_InstrumentID_get, _thostmduserapi.CThostFtdcInputQuoteActionField_InstrumentID_set)
    InvestUnitID = property(_thostmduserapi.CThostFtdcInputQuoteActionField_InvestUnitID_get, _thostmduserapi.CThostFtdcInputQuoteActionField_InvestUnitID_set)
    ClientID = property(_thostmduserapi.CThostFtdcInputQuoteActionField_ClientID_get, _thostmduserapi.CThostFtdcInputQuoteActionField_ClientID_set)
    IPAddress = property(_thostmduserapi.CThostFtdcInputQuoteActionField_IPAddress_get, _thostmduserapi.CThostFtdcInputQuoteActionField_IPAddress_set)
    MacAddress = property(_thostmduserapi.CThostFtdcInputQuoteActionField_MacAddress_get, _thostmduserapi.CThostFtdcInputQuoteActionField_MacAddress_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcInputQuoteActionField_swiginit(self, _thostmduserapi.new_CThostFtdcInputQuoteActionField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcInputQuoteActionField

# Register CThostFtdcInputQuoteActionField in _thostmduserapi:
_thostmduserapi.CThostFtdcInputQuoteActionField_swigregister(CThostFtdcInputQuoteActionField)

class CThostFtdcQuoteField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQuoteField_BrokerID_get, _thostmduserapi.CThostFtdcQuoteField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcQuoteField_InvestorID_get, _thostmduserapi.CThostFtdcQuoteField_InvestorID_set)
    InstrumentID = property(_thostmduserapi.CThostFtdcQuoteField_InstrumentID_get, _thostmduserapi.CThostFtdcQuoteField_InstrumentID_set)
    QuoteRef = property(_thostmduserapi.CThostFtdcQuoteField_QuoteRef_get, _thostmduserapi.CThostFtdcQuoteField_QuoteRef_set)
    UserID = property(_thostmduserapi.CThostFtdcQuoteField_UserID_get, _thostmduserapi.CThostFtdcQuoteField_UserID_set)
    AskPrice = property(_thostmduserapi.CThostFtdcQuoteField_AskPrice_get, _thostmduserapi.CThostFtdcQuoteField_AskPrice_set)
    BidPrice = property(_thostmduserapi.CThostFtdcQuoteField_BidPrice_get, _thostmduserapi.CThostFtdcQuoteField_BidPrice_set)
    AskVolume = property(_thostmduserapi.CThostFtdcQuoteField_AskVolume_get, _thostmduserapi.CThostFtdcQuoteField_AskVolume_set)
    BidVolume = property(_thostmduserapi.CThostFtdcQuoteField_BidVolume_get, _thostmduserapi.CThostFtdcQuoteField_BidVolume_set)
    RequestID = property(_thostmduserapi.CThostFtdcQuoteField_RequestID_get, _thostmduserapi.CThostFtdcQuoteField_RequestID_set)
    BusinessUnit = property(_thostmduserapi.CThostFtdcQuoteField_BusinessUnit_get, _thostmduserapi.CThostFtdcQuoteField_BusinessUnit_set)
    AskOffsetFlag = property(_thostmduserapi.CThostFtdcQuoteField_AskOffsetFlag_get, _thostmduserapi.CThostFtdcQuoteField_AskOffsetFlag_set)
    BidOffsetFlag = property(_thostmduserapi.CThostFtdcQuoteField_BidOffsetFlag_get, _thostmduserapi.CThostFtdcQuoteField_BidOffsetFlag_set)
    AskHedgeFlag = property(_thostmduserapi.CThostFtdcQuoteField_AskHedgeFlag_get, _thostmduserapi.CThostFtdcQuoteField_AskHedgeFlag_set)
    BidHedgeFlag = property(_thostmduserapi.CThostFtdcQuoteField_BidHedgeFlag_get, _thostmduserapi.CThostFtdcQuoteField_BidHedgeFlag_set)
    QuoteLocalID = property(_thostmduserapi.CThostFtdcQuoteField_QuoteLocalID_get, _thostmduserapi.CThostFtdcQuoteField_QuoteLocalID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcQuoteField_ExchangeID_get, _thostmduserapi.CThostFtdcQuoteField_ExchangeID_set)
    ParticipantID = property(_thostmduserapi.CThostFtdcQuoteField_ParticipantID_get, _thostmduserapi.CThostFtdcQuoteField_ParticipantID_set)
    ClientID = property(_thostmduserapi.CThostFtdcQuoteField_ClientID_get, _thostmduserapi.CThostFtdcQuoteField_ClientID_set)
    ExchangeInstID = property(_thostmduserapi.CThostFtdcQuoteField_ExchangeInstID_get, _thostmduserapi.CThostFtdcQuoteField_ExchangeInstID_set)
    TraderID = property(_thostmduserapi.CThostFtdcQuoteField_TraderID_get, _thostmduserapi.CThostFtdcQuoteField_TraderID_set)
    InstallID = property(_thostmduserapi.CThostFtdcQuoteField_InstallID_get, _thostmduserapi.CThostFtdcQuoteField_InstallID_set)
    NotifySequence = property(_thostmduserapi.CThostFtdcQuoteField_NotifySequence_get, _thostmduserapi.CThostFtdcQuoteField_NotifySequence_set)
    OrderSubmitStatus = property(_thostmduserapi.CThostFtdcQuoteField_OrderSubmitStatus_get, _thostmduserapi.CThostFtdcQuoteField_OrderSubmitStatus_set)
    TradingDay = property(_thostmduserapi.CThostFtdcQuoteField_TradingDay_get, _thostmduserapi.CThostFtdcQuoteField_TradingDay_set)
    SettlementID = property(_thostmduserapi.CThostFtdcQuoteField_SettlementID_get, _thostmduserapi.CThostFtdcQuoteField_SettlementID_set)
    QuoteSysID = property(_thostmduserapi.CThostFtdcQuoteField_QuoteSysID_get, _thostmduserapi.CThostFtdcQuoteField_QuoteSysID_set)
    InsertDate = property(_thostmduserapi.CThostFtdcQuoteField_InsertDate_get, _thostmduserapi.CThostFtdcQuoteField_InsertDate_set)
    InsertTime = property(_thostmduserapi.CThostFtdcQuoteField_InsertTime_get, _thostmduserapi.CThostFtdcQuoteField_InsertTime_set)
    CancelTime = property(_thostmduserapi.CThostFtdcQuoteField_CancelTime_get, _thostmduserapi.CThostFtdcQuoteField_CancelTime_set)
    QuoteStatus = property(_thostmduserapi.CThostFtdcQuoteField_QuoteStatus_get, _thostmduserapi.CThostFtdcQuoteField_QuoteStatus_set)
    ClearingPartID = property(_thostmduserapi.CThostFtdcQuoteField_ClearingPartID_get, _thostmduserapi.CThostFtdcQuoteField_ClearingPartID_set)
    SequenceNo = property(_thostmduserapi.CThostFtdcQuoteField_SequenceNo_get, _thostmduserapi.CThostFtdcQuoteField_SequenceNo_set)
    AskOrderSysID = property(_thostmduserapi.CThostFtdcQuoteField_AskOrderSysID_get, _thostmduserapi.CThostFtdcQuoteField_AskOrderSysID_set)
    BidOrderSysID = property(_thostmduserapi.CThostFtdcQuoteField_BidOrderSysID_get, _thostmduserapi.CThostFtdcQuoteField_BidOrderSysID_set)
    FrontID = property(_thostmduserapi.CThostFtdcQuoteField_FrontID_get, _thostmduserapi.CThostFtdcQuoteField_FrontID_set)
    SessionID = property(_thostmduserapi.CThostFtdcQuoteField_SessionID_get, _thostmduserapi.CThostFtdcQuoteField_SessionID_set)
    UserProductInfo = property(_thostmduserapi.CThostFtdcQuoteField_UserProductInfo_get, _thostmduserapi.CThostFtdcQuoteField_UserProductInfo_set)
    StatusMsg = property(_thostmduserapi.CThostFtdcQuoteField_StatusMsg_get, _thostmduserapi.CThostFtdcQuoteField_StatusMsg_set)
    ActiveUserID = property(_thostmduserapi.CThostFtdcQuoteField_ActiveUserID_get, _thostmduserapi.CThostFtdcQuoteField_ActiveUserID_set)
    BrokerQuoteSeq = property(_thostmduserapi.CThostFtdcQuoteField_BrokerQuoteSeq_get, _thostmduserapi.CThostFtdcQuoteField_BrokerQuoteSeq_set)
    AskOrderRef = property(_thostmduserapi.CThostFtdcQuoteField_AskOrderRef_get, _thostmduserapi.CThostFtdcQuoteField_AskOrderRef_set)
    BidOrderRef = property(_thostmduserapi.CThostFtdcQuoteField_BidOrderRef_get, _thostmduserapi.CThostFtdcQuoteField_BidOrderRef_set)
    ForQuoteSysID = property(_thostmduserapi.CThostFtdcQuoteField_ForQuoteSysID_get, _thostmduserapi.CThostFtdcQuoteField_ForQuoteSysID_set)
    BranchID = property(_thostmduserapi.CThostFtdcQuoteField_BranchID_get, _thostmduserapi.CThostFtdcQuoteField_BranchID_set)
    InvestUnitID = property(_thostmduserapi.CThostFtdcQuoteField_InvestUnitID_get, _thostmduserapi.CThostFtdcQuoteField_InvestUnitID_set)
    AccountID = property(_thostmduserapi.CThostFtdcQuoteField_AccountID_get, _thostmduserapi.CThostFtdcQuoteField_AccountID_set)
    CurrencyID = property(_thostmduserapi.CThostFtdcQuoteField_CurrencyID_get, _thostmduserapi.CThostFtdcQuoteField_CurrencyID_set)
    IPAddress = property(_thostmduserapi.CThostFtdcQuoteField_IPAddress_get, _thostmduserapi.CThostFtdcQuoteField_IPAddress_set)
    MacAddress = property(_thostmduserapi.CThostFtdcQuoteField_MacAddress_get, _thostmduserapi.CThostFtdcQuoteField_MacAddress_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQuoteField_swiginit(self, _thostmduserapi.new_CThostFtdcQuoteField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQuoteField

# Register CThostFtdcQuoteField in _thostmduserapi:
_thostmduserapi.CThostFtdcQuoteField_swigregister(CThostFtdcQuoteField)

class CThostFtdcQuoteActionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQuoteActionField_BrokerID_get, _thostmduserapi.CThostFtdcQuoteActionField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcQuoteActionField_InvestorID_get, _thostmduserapi.CThostFtdcQuoteActionField_InvestorID_set)
    QuoteActionRef = property(_thostmduserapi.CThostFtdcQuoteActionField_QuoteActionRef_get, _thostmduserapi.CThostFtdcQuoteActionField_QuoteActionRef_set)
    QuoteRef = property(_thostmduserapi.CThostFtdcQuoteActionField_QuoteRef_get, _thostmduserapi.CThostFtdcQuoteActionField_QuoteRef_set)
    RequestID = property(_thostmduserapi.CThostFtdcQuoteActionField_RequestID_get, _thostmduserapi.CThostFtdcQuoteActionField_RequestID_set)
    FrontID = property(_thostmduserapi.CThostFtdcQuoteActionField_FrontID_get, _thostmduserapi.CThostFtdcQuoteActionField_FrontID_set)
    SessionID = property(_thostmduserapi.CThostFtdcQuoteActionField_SessionID_get, _thostmduserapi.CThostFtdcQuoteActionField_SessionID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcQuoteActionField_ExchangeID_get, _thostmduserapi.CThostFtdcQuoteActionField_ExchangeID_set)
    QuoteSysID = property(_thostmduserapi.CThostFtdcQuoteActionField_QuoteSysID_get, _thostmduserapi.CThostFtdcQuoteActionField_QuoteSysID_set)
    ActionFlag = property(_thostmduserapi.CThostFtdcQuoteActionField_ActionFlag_get, _thostmduserapi.CThostFtdcQuoteActionField_ActionFlag_set)
    ActionDate = property(_thostmduserapi.CThostFtdcQuoteActionField_ActionDate_get, _thostmduserapi.CThostFtdcQuoteActionField_ActionDate_set)
    ActionTime = property(_thostmduserapi.CThostFtdcQuoteActionField_ActionTime_get, _thostmduserapi.CThostFtdcQuoteActionField_ActionTime_set)
    TraderID = property(_thostmduserapi.CThostFtdcQuoteActionField_TraderID_get, _thostmduserapi.CThostFtdcQuoteActionField_TraderID_set)
    InstallID = property(_thostmduserapi.CThostFtdcQuoteActionField_InstallID_get, _thostmduserapi.CThostFtdcQuoteActionField_InstallID_set)
    QuoteLocalID = property(_thostmduserapi.CThostFtdcQuoteActionField_QuoteLocalID_get, _thostmduserapi.CThostFtdcQuoteActionField_QuoteLocalID_set)
    ActionLocalID = property(_thostmduserapi.CThostFtdcQuoteActionField_ActionLocalID_get, _thostmduserapi.CThostFtdcQuoteActionField_ActionLocalID_set)
    ParticipantID = property(_thostmduserapi.CThostFtdcQuoteActionField_ParticipantID_get, _thostmduserapi.CThostFtdcQuoteActionField_ParticipantID_set)
    ClientID = property(_thostmduserapi.CThostFtdcQuoteActionField_ClientID_get, _thostmduserapi.CThostFtdcQuoteActionField_ClientID_set)
    BusinessUnit = property(_thostmduserapi.CThostFtdcQuoteActionField_BusinessUnit_get, _thostmduserapi.CThostFtdcQuoteActionField_BusinessUnit_set)
    OrderActionStatus = property(_thostmduserapi.CThostFtdcQuoteActionField_OrderActionStatus_get, _thostmduserapi.CThostFtdcQuoteActionField_OrderActionStatus_set)
    UserID = property(_thostmduserapi.CThostFtdcQuoteActionField_UserID_get, _thostmduserapi.CThostFtdcQuoteActionField_UserID_set)
    StatusMsg = property(_thostmduserapi.CThostFtdcQuoteActionField_StatusMsg_get, _thostmduserapi.CThostFtdcQuoteActionField_StatusMsg_set)
    InstrumentID = property(_thostmduserapi.CThostFtdcQuoteActionField_InstrumentID_get, _thostmduserapi.CThostFtdcQuoteActionField_InstrumentID_set)
    BranchID = property(_thostmduserapi.CThostFtdcQuoteActionField_BranchID_get, _thostmduserapi.CThostFtdcQuoteActionField_BranchID_set)
    InvestUnitID = property(_thostmduserapi.CThostFtdcQuoteActionField_InvestUnitID_get, _thostmduserapi.CThostFtdcQuoteActionField_InvestUnitID_set)
    IPAddress = property(_thostmduserapi.CThostFtdcQuoteActionField_IPAddress_get, _thostmduserapi.CThostFtdcQuoteActionField_IPAddress_set)
    MacAddress = property(_thostmduserapi.CThostFtdcQuoteActionField_MacAddress_get, _thostmduserapi.CThostFtdcQuoteActionField_MacAddress_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQuoteActionField_swiginit(self, _thostmduserapi.new_CThostFtdcQuoteActionField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQuoteActionField

# Register CThostFtdcQuoteActionField in _thostmduserapi:
_thostmduserapi.CThostFtdcQuoteActionField_swigregister(CThostFtdcQuoteActionField)

class CThostFtdcQryQuoteField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQryQuoteField_BrokerID_get, _thostmduserapi.CThostFtdcQryQuoteField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcQryQuoteField_InvestorID_get, _thostmduserapi.CThostFtdcQryQuoteField_InvestorID_set)
    InstrumentID = property(_thostmduserapi.CThostFtdcQryQuoteField_InstrumentID_get, _thostmduserapi.CThostFtdcQryQuoteField_InstrumentID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcQryQuoteField_ExchangeID_get, _thostmduserapi.CThostFtdcQryQuoteField_ExchangeID_set)
    QuoteSysID = property(_thostmduserapi.CThostFtdcQryQuoteField_QuoteSysID_get, _thostmduserapi.CThostFtdcQryQuoteField_QuoteSysID_set)
    InsertTimeStart = property(_thostmduserapi.CThostFtdcQryQuoteField_InsertTimeStart_get, _thostmduserapi.CThostFtdcQryQuoteField_InsertTimeStart_set)
    InsertTimeEnd = property(_thostmduserapi.CThostFtdcQryQuoteField_InsertTimeEnd_get, _thostmduserapi.CThostFtdcQryQuoteField_InsertTimeEnd_set)
    InvestUnitID = property(_thostmduserapi.CThostFtdcQryQuoteField_InvestUnitID_get, _thostmduserapi.CThostFtdcQryQuoteField_InvestUnitID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryQuoteField_swiginit(self, _thostmduserapi.new_CThostFtdcQryQuoteField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryQuoteField

# Register CThostFtdcQryQuoteField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryQuoteField_swigregister(CThostFtdcQryQuoteField)

class CThostFtdcExchangeQuoteField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    AskPrice = property(_thostmduserapi.CThostFtdcExchangeQuoteField_AskPrice_get, _thostmduserapi.CThostFtdcExchangeQuoteField_AskPrice_set)
    BidPrice = property(_thostmduserapi.CThostFtdcExchangeQuoteField_BidPrice_get, _thostmduserapi.CThostFtdcExchangeQuoteField_BidPrice_set)
    AskVolume = property(_thostmduserapi.CThostFtdcExchangeQuoteField_AskVolume_get, _thostmduserapi.CThostFtdcExchangeQuoteField_AskVolume_set)
    BidVolume = property(_thostmduserapi.CThostFtdcExchangeQuoteField_BidVolume_get, _thostmduserapi.CThostFtdcExchangeQuoteField_BidVolume_set)
    RequestID = property(_thostmduserapi.CThostFtdcExchangeQuoteField_RequestID_get, _thostmduserapi.CThostFtdcExchangeQuoteField_RequestID_set)
    BusinessUnit = property(_thostmduserapi.CThostFtdcExchangeQuoteField_BusinessUnit_get, _thostmduserapi.CThostFtdcExchangeQuoteField_BusinessUnit_set)
    AskOffsetFlag = property(_thostmduserapi.CThostFtdcExchangeQuoteField_AskOffsetFlag_get, _thostmduserapi.CThostFtdcExchangeQuoteField_AskOffsetFlag_set)
    BidOffsetFlag = property(_thostmduserapi.CThostFtdcExchangeQuoteField_BidOffsetFlag_get, _thostmduserapi.CThostFtdcExchangeQuoteField_BidOffsetFlag_set)
    AskHedgeFlag = property(_thostmduserapi.CThostFtdcExchangeQuoteField_AskHedgeFlag_get, _thostmduserapi.CThostFtdcExchangeQuoteField_AskHedgeFlag_set)
    BidHedgeFlag = property(_thostmduserapi.CThostFtdcExchangeQuoteField_BidHedgeFlag_get, _thostmduserapi.CThostFtdcExchangeQuoteField_BidHedgeFlag_set)
    QuoteLocalID = property(_thostmduserapi.CThostFtdcExchangeQuoteField_QuoteLocalID_get, _thostmduserapi.CThostFtdcExchangeQuoteField_QuoteLocalID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcExchangeQuoteField_ExchangeID_get, _thostmduserapi.CThostFtdcExchangeQuoteField_ExchangeID_set)
    ParticipantID = property(_thostmduserapi.CThostFtdcExchangeQuoteField_ParticipantID_get, _thostmduserapi.CThostFtdcExchangeQuoteField_ParticipantID_set)
    ClientID = property(_thostmduserapi.CThostFtdcExchangeQuoteField_ClientID_get, _thostmduserapi.CThostFtdcExchangeQuoteField_ClientID_set)
    ExchangeInstID = property(_thostmduserapi.CThostFtdcExchangeQuoteField_ExchangeInstID_get, _thostmduserapi.CThostFtdcExchangeQuoteField_ExchangeInstID_set)
    TraderID = property(_thostmduserapi.CThostFtdcExchangeQuoteField_TraderID_get, _thostmduserapi.CThostFtdcExchangeQuoteField_TraderID_set)
    InstallID = property(_thostmduserapi.CThostFtdcExchangeQuoteField_InstallID_get, _thostmduserapi.CThostFtdcExchangeQuoteField_InstallID_set)
    NotifySequence = property(_thostmduserapi.CThostFtdcExchangeQuoteField_NotifySequence_get, _thostmduserapi.CThostFtdcExchangeQuoteField_NotifySequence_set)
    OrderSubmitStatus = property(_thostmduserapi.CThostFtdcExchangeQuoteField_OrderSubmitStatus_get, _thostmduserapi.CThostFtdcExchangeQuoteField_OrderSubmitStatus_set)
    TradingDay = property(_thostmduserapi.CThostFtdcExchangeQuoteField_TradingDay_get, _thostmduserapi.CThostFtdcExchangeQuoteField_TradingDay_set)
    SettlementID = property(_thostmduserapi.CThostFtdcExchangeQuoteField_SettlementID_get, _thostmduserapi.CThostFtdcExchangeQuoteField_SettlementID_set)
    QuoteSysID = property(_thostmduserapi.CThostFtdcExchangeQuoteField_QuoteSysID_get, _thostmduserapi.CThostFtdcExchangeQuoteField_QuoteSysID_set)
    InsertDate = property(_thostmduserapi.CThostFtdcExchangeQuoteField_InsertDate_get, _thostmduserapi.CThostFtdcExchangeQuoteField_InsertDate_set)
    InsertTime = property(_thostmduserapi.CThostFtdcExchangeQuoteField_InsertTime_get, _thostmduserapi.CThostFtdcExchangeQuoteField_InsertTime_set)
    CancelTime = property(_thostmduserapi.CThostFtdcExchangeQuoteField_CancelTime_get, _thostmduserapi.CThostFtdcExchangeQuoteField_CancelTime_set)
    QuoteStatus = property(_thostmduserapi.CThostFtdcExchangeQuoteField_QuoteStatus_get, _thostmduserapi.CThostFtdcExchangeQuoteField_QuoteStatus_set)
    ClearingPartID = property(_thostmduserapi.CThostFtdcExchangeQuoteField_ClearingPartID_get, _thostmduserapi.CThostFtdcExchangeQuoteField_ClearingPartID_set)
    SequenceNo = property(_thostmduserapi.CThostFtdcExchangeQuoteField_SequenceNo_get, _thostmduserapi.CThostFtdcExchangeQuoteField_SequenceNo_set)
    AskOrderSysID = property(_thostmduserapi.CThostFtdcExchangeQuoteField_AskOrderSysID_get, _thostmduserapi.CThostFtdcExchangeQuoteField_AskOrderSysID_set)
    BidOrderSysID = property(_thostmduserapi.CThostFtdcExchangeQuoteField_BidOrderSysID_get, _thostmduserapi.CThostFtdcExchangeQuoteField_BidOrderSysID_set)
    ForQuoteSysID = property(_thostmduserapi.CThostFtdcExchangeQuoteField_ForQuoteSysID_get, _thostmduserapi.CThostFtdcExchangeQuoteField_ForQuoteSysID_set)
    BranchID = property(_thostmduserapi.CThostFtdcExchangeQuoteField_BranchID_get, _thostmduserapi.CThostFtdcExchangeQuoteField_BranchID_set)
    IPAddress = property(_thostmduserapi.CThostFtdcExchangeQuoteField_IPAddress_get, _thostmduserapi.CThostFtdcExchangeQuoteField_IPAddress_set)
    MacAddress = property(_thostmduserapi.CThostFtdcExchangeQuoteField_MacAddress_get, _thostmduserapi.CThostFtdcExchangeQuoteField_MacAddress_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcExchangeQuoteField_swiginit(self, _thostmduserapi.new_CThostFtdcExchangeQuoteField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcExchangeQuoteField

# Register CThostFtdcExchangeQuoteField in _thostmduserapi:
_thostmduserapi.CThostFtdcExchangeQuoteField_swigregister(CThostFtdcExchangeQuoteField)

class CThostFtdcQryExchangeQuoteField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ParticipantID = property(_thostmduserapi.CThostFtdcQryExchangeQuoteField_ParticipantID_get, _thostmduserapi.CThostFtdcQryExchangeQuoteField_ParticipantID_set)
    ClientID = property(_thostmduserapi.CThostFtdcQryExchangeQuoteField_ClientID_get, _thostmduserapi.CThostFtdcQryExchangeQuoteField_ClientID_set)
    ExchangeInstID = property(_thostmduserapi.CThostFtdcQryExchangeQuoteField_ExchangeInstID_get, _thostmduserapi.CThostFtdcQryExchangeQuoteField_ExchangeInstID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcQryExchangeQuoteField_ExchangeID_get, _thostmduserapi.CThostFtdcQryExchangeQuoteField_ExchangeID_set)
    TraderID = property(_thostmduserapi.CThostFtdcQryExchangeQuoteField_TraderID_get, _thostmduserapi.CThostFtdcQryExchangeQuoteField_TraderID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryExchangeQuoteField_swiginit(self, _thostmduserapi.new_CThostFtdcQryExchangeQuoteField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryExchangeQuoteField

# Register CThostFtdcQryExchangeQuoteField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryExchangeQuoteField_swigregister(CThostFtdcQryExchangeQuoteField)

class CThostFtdcQryQuoteActionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQryQuoteActionField_BrokerID_get, _thostmduserapi.CThostFtdcQryQuoteActionField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcQryQuoteActionField_InvestorID_get, _thostmduserapi.CThostFtdcQryQuoteActionField_InvestorID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcQryQuoteActionField_ExchangeID_get, _thostmduserapi.CThostFtdcQryQuoteActionField_ExchangeID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryQuoteActionField_swiginit(self, _thostmduserapi.new_CThostFtdcQryQuoteActionField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryQuoteActionField

# Register CThostFtdcQryQuoteActionField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryQuoteActionField_swigregister(CThostFtdcQryQuoteActionField)

class CThostFtdcExchangeQuoteActionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ExchangeID = property(_thostmduserapi.CThostFtdcExchangeQuoteActionField_ExchangeID_get, _thostmduserapi.CThostFtdcExchangeQuoteActionField_ExchangeID_set)
    QuoteSysID = property(_thostmduserapi.CThostFtdcExchangeQuoteActionField_QuoteSysID_get, _thostmduserapi.CThostFtdcExchangeQuoteActionField_QuoteSysID_set)
    ActionFlag = property(_thostmduserapi.CThostFtdcExchangeQuoteActionField_ActionFlag_get, _thostmduserapi.CThostFtdcExchangeQuoteActionField_ActionFlag_set)
    ActionDate = property(_thostmduserapi.CThostFtdcExchangeQuoteActionField_ActionDate_get, _thostmduserapi.CThostFtdcExchangeQuoteActionField_ActionDate_set)
    ActionTime = property(_thostmduserapi.CThostFtdcExchangeQuoteActionField_ActionTime_get, _thostmduserapi.CThostFtdcExchangeQuoteActionField_ActionTime_set)
    TraderID = property(_thostmduserapi.CThostFtdcExchangeQuoteActionField_TraderID_get, _thostmduserapi.CThostFtdcExchangeQuoteActionField_TraderID_set)
    InstallID = property(_thostmduserapi.CThostFtdcExchangeQuoteActionField_InstallID_get, _thostmduserapi.CThostFtdcExchangeQuoteActionField_InstallID_set)
    QuoteLocalID = property(_thostmduserapi.CThostFtdcExchangeQuoteActionField_QuoteLocalID_get, _thostmduserapi.CThostFtdcExchangeQuoteActionField_QuoteLocalID_set)
    ActionLocalID = property(_thostmduserapi.CThostFtdcExchangeQuoteActionField_ActionLocalID_get, _thostmduserapi.CThostFtdcExchangeQuoteActionField_ActionLocalID_set)
    ParticipantID = property(_thostmduserapi.CThostFtdcExchangeQuoteActionField_ParticipantID_get, _thostmduserapi.CThostFtdcExchangeQuoteActionField_ParticipantID_set)
    ClientID = property(_thostmduserapi.CThostFtdcExchangeQuoteActionField_ClientID_get, _thostmduserapi.CThostFtdcExchangeQuoteActionField_ClientID_set)
    BusinessUnit = property(_thostmduserapi.CThostFtdcExchangeQuoteActionField_BusinessUnit_get, _thostmduserapi.CThostFtdcExchangeQuoteActionField_BusinessUnit_set)
    OrderActionStatus = property(_thostmduserapi.CThostFtdcExchangeQuoteActionField_OrderActionStatus_get, _thostmduserapi.CThostFtdcExchangeQuoteActionField_OrderActionStatus_set)
    UserID = property(_thostmduserapi.CThostFtdcExchangeQuoteActionField_UserID_get, _thostmduserapi.CThostFtdcExchangeQuoteActionField_UserID_set)
    IPAddress = property(_thostmduserapi.CThostFtdcExchangeQuoteActionField_IPAddress_get, _thostmduserapi.CThostFtdcExchangeQuoteActionField_IPAddress_set)
    MacAddress = property(_thostmduserapi.CThostFtdcExchangeQuoteActionField_MacAddress_get, _thostmduserapi.CThostFtdcExchangeQuoteActionField_MacAddress_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcExchangeQuoteActionField_swiginit(self, _thostmduserapi.new_CThostFtdcExchangeQuoteActionField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcExchangeQuoteActionField

# Register CThostFtdcExchangeQuoteActionField in _thostmduserapi:
_thostmduserapi.CThostFtdcExchangeQuoteActionField_swigregister(CThostFtdcExchangeQuoteActionField)

class CThostFtdcQryExchangeQuoteActionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ParticipantID = property(_thostmduserapi.CThostFtdcQryExchangeQuoteActionField_ParticipantID_get, _thostmduserapi.CThostFtdcQryExchangeQuoteActionField_ParticipantID_set)
    ClientID = property(_thostmduserapi.CThostFtdcQryExchangeQuoteActionField_ClientID_get, _thostmduserapi.CThostFtdcQryExchangeQuoteActionField_ClientID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcQryExchangeQuoteActionField_ExchangeID_get, _thostmduserapi.CThostFtdcQryExchangeQuoteActionField_ExchangeID_set)
    TraderID = property(_thostmduserapi.CThostFtdcQryExchangeQuoteActionField_TraderID_get, _thostmduserapi.CThostFtdcQryExchangeQuoteActionField_TraderID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryExchangeQuoteActionField_swiginit(self, _thostmduserapi.new_CThostFtdcQryExchangeQuoteActionField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryExchangeQuoteActionField

# Register CThostFtdcQryExchangeQuoteActionField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryExchangeQuoteActionField_swigregister(CThostFtdcQryExchangeQuoteActionField)

class CThostFtdcOptionInstrDeltaField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    InstrumentID = property(_thostmduserapi.CThostFtdcOptionInstrDeltaField_InstrumentID_get, _thostmduserapi.CThostFtdcOptionInstrDeltaField_InstrumentID_set)
    InvestorRange = property(_thostmduserapi.CThostFtdcOptionInstrDeltaField_InvestorRange_get, _thostmduserapi.CThostFtdcOptionInstrDeltaField_InvestorRange_set)
    BrokerID = property(_thostmduserapi.CThostFtdcOptionInstrDeltaField_BrokerID_get, _thostmduserapi.CThostFtdcOptionInstrDeltaField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcOptionInstrDeltaField_InvestorID_get, _thostmduserapi.CThostFtdcOptionInstrDeltaField_InvestorID_set)
    Delta = property(_thostmduserapi.CThostFtdcOptionInstrDeltaField_Delta_get, _thostmduserapi.CThostFtdcOptionInstrDeltaField_Delta_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcOptionInstrDeltaField_swiginit(self, _thostmduserapi.new_CThostFtdcOptionInstrDeltaField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcOptionInstrDeltaField

# Register CThostFtdcOptionInstrDeltaField in _thostmduserapi:
_thostmduserapi.CThostFtdcOptionInstrDeltaField_swigregister(CThostFtdcOptionInstrDeltaField)

class CThostFtdcForQuoteRspField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradingDay = property(_thostmduserapi.CThostFtdcForQuoteRspField_TradingDay_get, _thostmduserapi.CThostFtdcForQuoteRspField_TradingDay_set)
    InstrumentID = property(_thostmduserapi.CThostFtdcForQuoteRspField_InstrumentID_get, _thostmduserapi.CThostFtdcForQuoteRspField_InstrumentID_set)
    ForQuoteSysID = property(_thostmduserapi.CThostFtdcForQuoteRspField_ForQuoteSysID_get, _thostmduserapi.CThostFtdcForQuoteRspField_ForQuoteSysID_set)
    ForQuoteTime = property(_thostmduserapi.CThostFtdcForQuoteRspField_ForQuoteTime_get, _thostmduserapi.CThostFtdcForQuoteRspField_ForQuoteTime_set)
    ActionDay = property(_thostmduserapi.CThostFtdcForQuoteRspField_ActionDay_get, _thostmduserapi.CThostFtdcForQuoteRspField_ActionDay_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcForQuoteRspField_ExchangeID_get, _thostmduserapi.CThostFtdcForQuoteRspField_ExchangeID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcForQuoteRspField_swiginit(self, _thostmduserapi.new_CThostFtdcForQuoteRspField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcForQuoteRspField

# Register CThostFtdcForQuoteRspField in _thostmduserapi:
_thostmduserapi.CThostFtdcForQuoteRspField_swigregister(CThostFtdcForQuoteRspField)

class CThostFtdcStrikeOffsetField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    InstrumentID = property(_thostmduserapi.CThostFtdcStrikeOffsetField_InstrumentID_get, _thostmduserapi.CThostFtdcStrikeOffsetField_InstrumentID_set)
    InvestorRange = property(_thostmduserapi.CThostFtdcStrikeOffsetField_InvestorRange_get, _thostmduserapi.CThostFtdcStrikeOffsetField_InvestorRange_set)
    BrokerID = property(_thostmduserapi.CThostFtdcStrikeOffsetField_BrokerID_get, _thostmduserapi.CThostFtdcStrikeOffsetField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcStrikeOffsetField_InvestorID_get, _thostmduserapi.CThostFtdcStrikeOffsetField_InvestorID_set)
    Offset = property(_thostmduserapi.CThostFtdcStrikeOffsetField_Offset_get, _thostmduserapi.CThostFtdcStrikeOffsetField_Offset_set)
    OffsetType = property(_thostmduserapi.CThostFtdcStrikeOffsetField_OffsetType_get, _thostmduserapi.CThostFtdcStrikeOffsetField_OffsetType_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcStrikeOffsetField_swiginit(self, _thostmduserapi.new_CThostFtdcStrikeOffsetField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcStrikeOffsetField

# Register CThostFtdcStrikeOffsetField in _thostmduserapi:
_thostmduserapi.CThostFtdcStrikeOffsetField_swigregister(CThostFtdcStrikeOffsetField)

class CThostFtdcQryStrikeOffsetField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQryStrikeOffsetField_BrokerID_get, _thostmduserapi.CThostFtdcQryStrikeOffsetField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcQryStrikeOffsetField_InvestorID_get, _thostmduserapi.CThostFtdcQryStrikeOffsetField_InvestorID_set)
    InstrumentID = property(_thostmduserapi.CThostFtdcQryStrikeOffsetField_InstrumentID_get, _thostmduserapi.CThostFtdcQryStrikeOffsetField_InstrumentID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryStrikeOffsetField_swiginit(self, _thostmduserapi.new_CThostFtdcQryStrikeOffsetField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryStrikeOffsetField

# Register CThostFtdcQryStrikeOffsetField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryStrikeOffsetField_swigregister(CThostFtdcQryStrikeOffsetField)

class CThostFtdcInputBatchOrderActionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcInputBatchOrderActionField_BrokerID_get, _thostmduserapi.CThostFtdcInputBatchOrderActionField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcInputBatchOrderActionField_InvestorID_get, _thostmduserapi.CThostFtdcInputBatchOrderActionField_InvestorID_set)
    OrderActionRef = property(_thostmduserapi.CThostFtdcInputBatchOrderActionField_OrderActionRef_get, _thostmduserapi.CThostFtdcInputBatchOrderActionField_OrderActionRef_set)
    RequestID = property(_thostmduserapi.CThostFtdcInputBatchOrderActionField_RequestID_get, _thostmduserapi.CThostFtdcInputBatchOrderActionField_RequestID_set)
    FrontID = property(_thostmduserapi.CThostFtdcInputBatchOrderActionField_FrontID_get, _thostmduserapi.CThostFtdcInputBatchOrderActionField_FrontID_set)
    SessionID = property(_thostmduserapi.CThostFtdcInputBatchOrderActionField_SessionID_get, _thostmduserapi.CThostFtdcInputBatchOrderActionField_SessionID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcInputBatchOrderActionField_ExchangeID_get, _thostmduserapi.CThostFtdcInputBatchOrderActionField_ExchangeID_set)
    UserID = property(_thostmduserapi.CThostFtdcInputBatchOrderActionField_UserID_get, _thostmduserapi.CThostFtdcInputBatchOrderActionField_UserID_set)
    InvestUnitID = property(_thostmduserapi.CThostFtdcInputBatchOrderActionField_InvestUnitID_get, _thostmduserapi.CThostFtdcInputBatchOrderActionField_InvestUnitID_set)
    IPAddress = property(_thostmduserapi.CThostFtdcInputBatchOrderActionField_IPAddress_get, _thostmduserapi.CThostFtdcInputBatchOrderActionField_IPAddress_set)
    MacAddress = property(_thostmduserapi.CThostFtdcInputBatchOrderActionField_MacAddress_get, _thostmduserapi.CThostFtdcInputBatchOrderActionField_MacAddress_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcInputBatchOrderActionField_swiginit(self, _thostmduserapi.new_CThostFtdcInputBatchOrderActionField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcInputBatchOrderActionField

# Register CThostFtdcInputBatchOrderActionField in _thostmduserapi:
_thostmduserapi.CThostFtdcInputBatchOrderActionField_swigregister(CThostFtdcInputBatchOrderActionField)

class CThostFtdcBatchOrderActionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcBatchOrderActionField_BrokerID_get, _thostmduserapi.CThostFtdcBatchOrderActionField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcBatchOrderActionField_InvestorID_get, _thostmduserapi.CThostFtdcBatchOrderActionField_InvestorID_set)
    OrderActionRef = property(_thostmduserapi.CThostFtdcBatchOrderActionField_OrderActionRef_get, _thostmduserapi.CThostFtdcBatchOrderActionField_OrderActionRef_set)
    RequestID = property(_thostmduserapi.CThostFtdcBatchOrderActionField_RequestID_get, _thostmduserapi.CThostFtdcBatchOrderActionField_RequestID_set)
    FrontID = property(_thostmduserapi.CThostFtdcBatchOrderActionField_FrontID_get, _thostmduserapi.CThostFtdcBatchOrderActionField_FrontID_set)
    SessionID = property(_thostmduserapi.CThostFtdcBatchOrderActionField_SessionID_get, _thostmduserapi.CThostFtdcBatchOrderActionField_SessionID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcBatchOrderActionField_ExchangeID_get, _thostmduserapi.CThostFtdcBatchOrderActionField_ExchangeID_set)
    ActionDate = property(_thostmduserapi.CThostFtdcBatchOrderActionField_ActionDate_get, _thostmduserapi.CThostFtdcBatchOrderActionField_ActionDate_set)
    ActionTime = property(_thostmduserapi.CThostFtdcBatchOrderActionField_ActionTime_get, _thostmduserapi.CThostFtdcBatchOrderActionField_ActionTime_set)
    TraderID = property(_thostmduserapi.CThostFtdcBatchOrderActionField_TraderID_get, _thostmduserapi.CThostFtdcBatchOrderActionField_TraderID_set)
    InstallID = property(_thostmduserapi.CThostFtdcBatchOrderActionField_InstallID_get, _thostmduserapi.CThostFtdcBatchOrderActionField_InstallID_set)
    ActionLocalID = property(_thostmduserapi.CThostFtdcBatchOrderActionField_ActionLocalID_get, _thostmduserapi.CThostFtdcBatchOrderActionField_ActionLocalID_set)
    ParticipantID = property(_thostmduserapi.CThostFtdcBatchOrderActionField_ParticipantID_get, _thostmduserapi.CThostFtdcBatchOrderActionField_ParticipantID_set)
    ClientID = property(_thostmduserapi.CThostFtdcBatchOrderActionField_ClientID_get, _thostmduserapi.CThostFtdcBatchOrderActionField_ClientID_set)
    BusinessUnit = property(_thostmduserapi.CThostFtdcBatchOrderActionField_BusinessUnit_get, _thostmduserapi.CThostFtdcBatchOrderActionField_BusinessUnit_set)
    OrderActionStatus = property(_thostmduserapi.CThostFtdcBatchOrderActionField_OrderActionStatus_get, _thostmduserapi.CThostFtdcBatchOrderActionField_OrderActionStatus_set)
    UserID = property(_thostmduserapi.CThostFtdcBatchOrderActionField_UserID_get, _thostmduserapi.CThostFtdcBatchOrderActionField_UserID_set)
    StatusMsg = property(_thostmduserapi.CThostFtdcBatchOrderActionField_StatusMsg_get, _thostmduserapi.CThostFtdcBatchOrderActionField_StatusMsg_set)
    InvestUnitID = property(_thostmduserapi.CThostFtdcBatchOrderActionField_InvestUnitID_get, _thostmduserapi.CThostFtdcBatchOrderActionField_InvestUnitID_set)
    IPAddress = property(_thostmduserapi.CThostFtdcBatchOrderActionField_IPAddress_get, _thostmduserapi.CThostFtdcBatchOrderActionField_IPAddress_set)
    MacAddress = property(_thostmduserapi.CThostFtdcBatchOrderActionField_MacAddress_get, _thostmduserapi.CThostFtdcBatchOrderActionField_MacAddress_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcBatchOrderActionField_swiginit(self, _thostmduserapi.new_CThostFtdcBatchOrderActionField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcBatchOrderActionField

# Register CThostFtdcBatchOrderActionField in _thostmduserapi:
_thostmduserapi.CThostFtdcBatchOrderActionField_swigregister(CThostFtdcBatchOrderActionField)

class CThostFtdcExchangeBatchOrderActionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ExchangeID = property(_thostmduserapi.CThostFtdcExchangeBatchOrderActionField_ExchangeID_get, _thostmduserapi.CThostFtdcExchangeBatchOrderActionField_ExchangeID_set)
    ActionDate = property(_thostmduserapi.CThostFtdcExchangeBatchOrderActionField_ActionDate_get, _thostmduserapi.CThostFtdcExchangeBatchOrderActionField_ActionDate_set)
    ActionTime = property(_thostmduserapi.CThostFtdcExchangeBatchOrderActionField_ActionTime_get, _thostmduserapi.CThostFtdcExchangeBatchOrderActionField_ActionTime_set)
    TraderID = property(_thostmduserapi.CThostFtdcExchangeBatchOrderActionField_TraderID_get, _thostmduserapi.CThostFtdcExchangeBatchOrderActionField_TraderID_set)
    InstallID = property(_thostmduserapi.CThostFtdcExchangeBatchOrderActionField_InstallID_get, _thostmduserapi.CThostFtdcExchangeBatchOrderActionField_InstallID_set)
    ActionLocalID = property(_thostmduserapi.CThostFtdcExchangeBatchOrderActionField_ActionLocalID_get, _thostmduserapi.CThostFtdcExchangeBatchOrderActionField_ActionLocalID_set)
    ParticipantID = property(_thostmduserapi.CThostFtdcExchangeBatchOrderActionField_ParticipantID_get, _thostmduserapi.CThostFtdcExchangeBatchOrderActionField_ParticipantID_set)
    ClientID = property(_thostmduserapi.CThostFtdcExchangeBatchOrderActionField_ClientID_get, _thostmduserapi.CThostFtdcExchangeBatchOrderActionField_ClientID_set)
    BusinessUnit = property(_thostmduserapi.CThostFtdcExchangeBatchOrderActionField_BusinessUnit_get, _thostmduserapi.CThostFtdcExchangeBatchOrderActionField_BusinessUnit_set)
    OrderActionStatus = property(_thostmduserapi.CThostFtdcExchangeBatchOrderActionField_OrderActionStatus_get, _thostmduserapi.CThostFtdcExchangeBatchOrderActionField_OrderActionStatus_set)
    UserID = property(_thostmduserapi.CThostFtdcExchangeBatchOrderActionField_UserID_get, _thostmduserapi.CThostFtdcExchangeBatchOrderActionField_UserID_set)
    IPAddress = property(_thostmduserapi.CThostFtdcExchangeBatchOrderActionField_IPAddress_get, _thostmduserapi.CThostFtdcExchangeBatchOrderActionField_IPAddress_set)
    MacAddress = property(_thostmduserapi.CThostFtdcExchangeBatchOrderActionField_MacAddress_get, _thostmduserapi.CThostFtdcExchangeBatchOrderActionField_MacAddress_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcExchangeBatchOrderActionField_swiginit(self, _thostmduserapi.new_CThostFtdcExchangeBatchOrderActionField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcExchangeBatchOrderActionField

# Register CThostFtdcExchangeBatchOrderActionField in _thostmduserapi:
_thostmduserapi.CThostFtdcExchangeBatchOrderActionField_swigregister(CThostFtdcExchangeBatchOrderActionField)

class CThostFtdcQryBatchOrderActionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQryBatchOrderActionField_BrokerID_get, _thostmduserapi.CThostFtdcQryBatchOrderActionField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcQryBatchOrderActionField_InvestorID_get, _thostmduserapi.CThostFtdcQryBatchOrderActionField_InvestorID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcQryBatchOrderActionField_ExchangeID_get, _thostmduserapi.CThostFtdcQryBatchOrderActionField_ExchangeID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryBatchOrderActionField_swiginit(self, _thostmduserapi.new_CThostFtdcQryBatchOrderActionField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryBatchOrderActionField

# Register CThostFtdcQryBatchOrderActionField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryBatchOrderActionField_swigregister(CThostFtdcQryBatchOrderActionField)

class CThostFtdcCombInstrumentGuardField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcCombInstrumentGuardField_BrokerID_get, _thostmduserapi.CThostFtdcCombInstrumentGuardField_BrokerID_set)
    InstrumentID = property(_thostmduserapi.CThostFtdcCombInstrumentGuardField_InstrumentID_get, _thostmduserapi.CThostFtdcCombInstrumentGuardField_InstrumentID_set)
    GuarantRatio = property(_thostmduserapi.CThostFtdcCombInstrumentGuardField_GuarantRatio_get, _thostmduserapi.CThostFtdcCombInstrumentGuardField_GuarantRatio_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcCombInstrumentGuardField_ExchangeID_get, _thostmduserapi.CThostFtdcCombInstrumentGuardField_ExchangeID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcCombInstrumentGuardField_swiginit(self, _thostmduserapi.new_CThostFtdcCombInstrumentGuardField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcCombInstrumentGuardField

# Register CThostFtdcCombInstrumentGuardField in _thostmduserapi:
_thostmduserapi.CThostFtdcCombInstrumentGuardField_swigregister(CThostFtdcCombInstrumentGuardField)

class CThostFtdcQryCombInstrumentGuardField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQryCombInstrumentGuardField_BrokerID_get, _thostmduserapi.CThostFtdcQryCombInstrumentGuardField_BrokerID_set)
    InstrumentID = property(_thostmduserapi.CThostFtdcQryCombInstrumentGuardField_InstrumentID_get, _thostmduserapi.CThostFtdcQryCombInstrumentGuardField_InstrumentID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcQryCombInstrumentGuardField_ExchangeID_get, _thostmduserapi.CThostFtdcQryCombInstrumentGuardField_ExchangeID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryCombInstrumentGuardField_swiginit(self, _thostmduserapi.new_CThostFtdcQryCombInstrumentGuardField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryCombInstrumentGuardField

# Register CThostFtdcQryCombInstrumentGuardField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryCombInstrumentGuardField_swigregister(CThostFtdcQryCombInstrumentGuardField)

class CThostFtdcInputCombActionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcInputCombActionField_BrokerID_get, _thostmduserapi.CThostFtdcInputCombActionField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcInputCombActionField_InvestorID_get, _thostmduserapi.CThostFtdcInputCombActionField_InvestorID_set)
    InstrumentID = property(_thostmduserapi.CThostFtdcInputCombActionField_InstrumentID_get, _thostmduserapi.CThostFtdcInputCombActionField_InstrumentID_set)
    CombActionRef = property(_thostmduserapi.CThostFtdcInputCombActionField_CombActionRef_get, _thostmduserapi.CThostFtdcInputCombActionField_CombActionRef_set)
    UserID = property(_thostmduserapi.CThostFtdcInputCombActionField_UserID_get, _thostmduserapi.CThostFtdcInputCombActionField_UserID_set)
    Direction = property(_thostmduserapi.CThostFtdcInputCombActionField_Direction_get, _thostmduserapi.CThostFtdcInputCombActionField_Direction_set)
    Volume = property(_thostmduserapi.CThostFtdcInputCombActionField_Volume_get, _thostmduserapi.CThostFtdcInputCombActionField_Volume_set)
    CombDirection = property(_thostmduserapi.CThostFtdcInputCombActionField_CombDirection_get, _thostmduserapi.CThostFtdcInputCombActionField_CombDirection_set)
    HedgeFlag = property(_thostmduserapi.CThostFtdcInputCombActionField_HedgeFlag_get, _thostmduserapi.CThostFtdcInputCombActionField_HedgeFlag_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcInputCombActionField_ExchangeID_get, _thostmduserapi.CThostFtdcInputCombActionField_ExchangeID_set)
    IPAddress = property(_thostmduserapi.CThostFtdcInputCombActionField_IPAddress_get, _thostmduserapi.CThostFtdcInputCombActionField_IPAddress_set)
    MacAddress = property(_thostmduserapi.CThostFtdcInputCombActionField_MacAddress_get, _thostmduserapi.CThostFtdcInputCombActionField_MacAddress_set)
    InvestUnitID = property(_thostmduserapi.CThostFtdcInputCombActionField_InvestUnitID_get, _thostmduserapi.CThostFtdcInputCombActionField_InvestUnitID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcInputCombActionField_swiginit(self, _thostmduserapi.new_CThostFtdcInputCombActionField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcInputCombActionField

# Register CThostFtdcInputCombActionField in _thostmduserapi:
_thostmduserapi.CThostFtdcInputCombActionField_swigregister(CThostFtdcInputCombActionField)

class CThostFtdcCombActionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcCombActionField_BrokerID_get, _thostmduserapi.CThostFtdcCombActionField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcCombActionField_InvestorID_get, _thostmduserapi.CThostFtdcCombActionField_InvestorID_set)
    InstrumentID = property(_thostmduserapi.CThostFtdcCombActionField_InstrumentID_get, _thostmduserapi.CThostFtdcCombActionField_InstrumentID_set)
    CombActionRef = property(_thostmduserapi.CThostFtdcCombActionField_CombActionRef_get, _thostmduserapi.CThostFtdcCombActionField_CombActionRef_set)
    UserID = property(_thostmduserapi.CThostFtdcCombActionField_UserID_get, _thostmduserapi.CThostFtdcCombActionField_UserID_set)
    Direction = property(_thostmduserapi.CThostFtdcCombActionField_Direction_get, _thostmduserapi.CThostFtdcCombActionField_Direction_set)
    Volume = property(_thostmduserapi.CThostFtdcCombActionField_Volume_get, _thostmduserapi.CThostFtdcCombActionField_Volume_set)
    CombDirection = property(_thostmduserapi.CThostFtdcCombActionField_CombDirection_get, _thostmduserapi.CThostFtdcCombActionField_CombDirection_set)
    HedgeFlag = property(_thostmduserapi.CThostFtdcCombActionField_HedgeFlag_get, _thostmduserapi.CThostFtdcCombActionField_HedgeFlag_set)
    ActionLocalID = property(_thostmduserapi.CThostFtdcCombActionField_ActionLocalID_get, _thostmduserapi.CThostFtdcCombActionField_ActionLocalID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcCombActionField_ExchangeID_get, _thostmduserapi.CThostFtdcCombActionField_ExchangeID_set)
    ParticipantID = property(_thostmduserapi.CThostFtdcCombActionField_ParticipantID_get, _thostmduserapi.CThostFtdcCombActionField_ParticipantID_set)
    ClientID = property(_thostmduserapi.CThostFtdcCombActionField_ClientID_get, _thostmduserapi.CThostFtdcCombActionField_ClientID_set)
    ExchangeInstID = property(_thostmduserapi.CThostFtdcCombActionField_ExchangeInstID_get, _thostmduserapi.CThostFtdcCombActionField_ExchangeInstID_set)
    TraderID = property(_thostmduserapi.CThostFtdcCombActionField_TraderID_get, _thostmduserapi.CThostFtdcCombActionField_TraderID_set)
    InstallID = property(_thostmduserapi.CThostFtdcCombActionField_InstallID_get, _thostmduserapi.CThostFtdcCombActionField_InstallID_set)
    ActionStatus = property(_thostmduserapi.CThostFtdcCombActionField_ActionStatus_get, _thostmduserapi.CThostFtdcCombActionField_ActionStatus_set)
    NotifySequence = property(_thostmduserapi.CThostFtdcCombActionField_NotifySequence_get, _thostmduserapi.CThostFtdcCombActionField_NotifySequence_set)
    TradingDay = property(_thostmduserapi.CThostFtdcCombActionField_TradingDay_get, _thostmduserapi.CThostFtdcCombActionField_TradingDay_set)
    SettlementID = property(_thostmduserapi.CThostFtdcCombActionField_SettlementID_get, _thostmduserapi.CThostFtdcCombActionField_SettlementID_set)
    SequenceNo = property(_thostmduserapi.CThostFtdcCombActionField_SequenceNo_get, _thostmduserapi.CThostFtdcCombActionField_SequenceNo_set)
    FrontID = property(_thostmduserapi.CThostFtdcCombActionField_FrontID_get, _thostmduserapi.CThostFtdcCombActionField_FrontID_set)
    SessionID = property(_thostmduserapi.CThostFtdcCombActionField_SessionID_get, _thostmduserapi.CThostFtdcCombActionField_SessionID_set)
    UserProductInfo = property(_thostmduserapi.CThostFtdcCombActionField_UserProductInfo_get, _thostmduserapi.CThostFtdcCombActionField_UserProductInfo_set)
    StatusMsg = property(_thostmduserapi.CThostFtdcCombActionField_StatusMsg_get, _thostmduserapi.CThostFtdcCombActionField_StatusMsg_set)
    IPAddress = property(_thostmduserapi.CThostFtdcCombActionField_IPAddress_get, _thostmduserapi.CThostFtdcCombActionField_IPAddress_set)
    MacAddress = property(_thostmduserapi.CThostFtdcCombActionField_MacAddress_get, _thostmduserapi.CThostFtdcCombActionField_MacAddress_set)
    ComTradeID = property(_thostmduserapi.CThostFtdcCombActionField_ComTradeID_get, _thostmduserapi.CThostFtdcCombActionField_ComTradeID_set)
    BranchID = property(_thostmduserapi.CThostFtdcCombActionField_BranchID_get, _thostmduserapi.CThostFtdcCombActionField_BranchID_set)
    InvestUnitID = property(_thostmduserapi.CThostFtdcCombActionField_InvestUnitID_get, _thostmduserapi.CThostFtdcCombActionField_InvestUnitID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcCombActionField_swiginit(self, _thostmduserapi.new_CThostFtdcCombActionField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcCombActionField

# Register CThostFtdcCombActionField in _thostmduserapi:
_thostmduserapi.CThostFtdcCombActionField_swigregister(CThostFtdcCombActionField)

class CThostFtdcQryCombActionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQryCombActionField_BrokerID_get, _thostmduserapi.CThostFtdcQryCombActionField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcQryCombActionField_InvestorID_get, _thostmduserapi.CThostFtdcQryCombActionField_InvestorID_set)
    InstrumentID = property(_thostmduserapi.CThostFtdcQryCombActionField_InstrumentID_get, _thostmduserapi.CThostFtdcQryCombActionField_InstrumentID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcQryCombActionField_ExchangeID_get, _thostmduserapi.CThostFtdcQryCombActionField_ExchangeID_set)
    InvestUnitID = property(_thostmduserapi.CThostFtdcQryCombActionField_InvestUnitID_get, _thostmduserapi.CThostFtdcQryCombActionField_InvestUnitID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryCombActionField_swiginit(self, _thostmduserapi.new_CThostFtdcQryCombActionField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryCombActionField

# Register CThostFtdcQryCombActionField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryCombActionField_swigregister(CThostFtdcQryCombActionField)

class CThostFtdcExchangeCombActionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    Direction = property(_thostmduserapi.CThostFtdcExchangeCombActionField_Direction_get, _thostmduserapi.CThostFtdcExchangeCombActionField_Direction_set)
    Volume = property(_thostmduserapi.CThostFtdcExchangeCombActionField_Volume_get, _thostmduserapi.CThostFtdcExchangeCombActionField_Volume_set)
    CombDirection = property(_thostmduserapi.CThostFtdcExchangeCombActionField_CombDirection_get, _thostmduserapi.CThostFtdcExchangeCombActionField_CombDirection_set)
    HedgeFlag = property(_thostmduserapi.CThostFtdcExchangeCombActionField_HedgeFlag_get, _thostmduserapi.CThostFtdcExchangeCombActionField_HedgeFlag_set)
    ActionLocalID = property(_thostmduserapi.CThostFtdcExchangeCombActionField_ActionLocalID_get, _thostmduserapi.CThostFtdcExchangeCombActionField_ActionLocalID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcExchangeCombActionField_ExchangeID_get, _thostmduserapi.CThostFtdcExchangeCombActionField_ExchangeID_set)
    ParticipantID = property(_thostmduserapi.CThostFtdcExchangeCombActionField_ParticipantID_get, _thostmduserapi.CThostFtdcExchangeCombActionField_ParticipantID_set)
    ClientID = property(_thostmduserapi.CThostFtdcExchangeCombActionField_ClientID_get, _thostmduserapi.CThostFtdcExchangeCombActionField_ClientID_set)
    ExchangeInstID = property(_thostmduserapi.CThostFtdcExchangeCombActionField_ExchangeInstID_get, _thostmduserapi.CThostFtdcExchangeCombActionField_ExchangeInstID_set)
    TraderID = property(_thostmduserapi.CThostFtdcExchangeCombActionField_TraderID_get, _thostmduserapi.CThostFtdcExchangeCombActionField_TraderID_set)
    InstallID = property(_thostmduserapi.CThostFtdcExchangeCombActionField_InstallID_get, _thostmduserapi.CThostFtdcExchangeCombActionField_InstallID_set)
    ActionStatus = property(_thostmduserapi.CThostFtdcExchangeCombActionField_ActionStatus_get, _thostmduserapi.CThostFtdcExchangeCombActionField_ActionStatus_set)
    NotifySequence = property(_thostmduserapi.CThostFtdcExchangeCombActionField_NotifySequence_get, _thostmduserapi.CThostFtdcExchangeCombActionField_NotifySequence_set)
    TradingDay = property(_thostmduserapi.CThostFtdcExchangeCombActionField_TradingDay_get, _thostmduserapi.CThostFtdcExchangeCombActionField_TradingDay_set)
    SettlementID = property(_thostmduserapi.CThostFtdcExchangeCombActionField_SettlementID_get, _thostmduserapi.CThostFtdcExchangeCombActionField_SettlementID_set)
    SequenceNo = property(_thostmduserapi.CThostFtdcExchangeCombActionField_SequenceNo_get, _thostmduserapi.CThostFtdcExchangeCombActionField_SequenceNo_set)
    IPAddress = property(_thostmduserapi.CThostFtdcExchangeCombActionField_IPAddress_get, _thostmduserapi.CThostFtdcExchangeCombActionField_IPAddress_set)
    MacAddress = property(_thostmduserapi.CThostFtdcExchangeCombActionField_MacAddress_get, _thostmduserapi.CThostFtdcExchangeCombActionField_MacAddress_set)
    ComTradeID = property(_thostmduserapi.CThostFtdcExchangeCombActionField_ComTradeID_get, _thostmduserapi.CThostFtdcExchangeCombActionField_ComTradeID_set)
    BranchID = property(_thostmduserapi.CThostFtdcExchangeCombActionField_BranchID_get, _thostmduserapi.CThostFtdcExchangeCombActionField_BranchID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcExchangeCombActionField_swiginit(self, _thostmduserapi.new_CThostFtdcExchangeCombActionField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcExchangeCombActionField

# Register CThostFtdcExchangeCombActionField in _thostmduserapi:
_thostmduserapi.CThostFtdcExchangeCombActionField_swigregister(CThostFtdcExchangeCombActionField)

class CThostFtdcQryExchangeCombActionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ParticipantID = property(_thostmduserapi.CThostFtdcQryExchangeCombActionField_ParticipantID_get, _thostmduserapi.CThostFtdcQryExchangeCombActionField_ParticipantID_set)
    ClientID = property(_thostmduserapi.CThostFtdcQryExchangeCombActionField_ClientID_get, _thostmduserapi.CThostFtdcQryExchangeCombActionField_ClientID_set)
    ExchangeInstID = property(_thostmduserapi.CThostFtdcQryExchangeCombActionField_ExchangeInstID_get, _thostmduserapi.CThostFtdcQryExchangeCombActionField_ExchangeInstID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcQryExchangeCombActionField_ExchangeID_get, _thostmduserapi.CThostFtdcQryExchangeCombActionField_ExchangeID_set)
    TraderID = property(_thostmduserapi.CThostFtdcQryExchangeCombActionField_TraderID_get, _thostmduserapi.CThostFtdcQryExchangeCombActionField_TraderID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryExchangeCombActionField_swiginit(self, _thostmduserapi.new_CThostFtdcQryExchangeCombActionField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryExchangeCombActionField

# Register CThostFtdcQryExchangeCombActionField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryExchangeCombActionField_swigregister(CThostFtdcQryExchangeCombActionField)

class CThostFtdcProductExchRateField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ProductID = property(_thostmduserapi.CThostFtdcProductExchRateField_ProductID_get, _thostmduserapi.CThostFtdcProductExchRateField_ProductID_set)
    QuoteCurrencyID = property(_thostmduserapi.CThostFtdcProductExchRateField_QuoteCurrencyID_get, _thostmduserapi.CThostFtdcProductExchRateField_QuoteCurrencyID_set)
    ExchangeRate = property(_thostmduserapi.CThostFtdcProductExchRateField_ExchangeRate_get, _thostmduserapi.CThostFtdcProductExchRateField_ExchangeRate_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcProductExchRateField_ExchangeID_get, _thostmduserapi.CThostFtdcProductExchRateField_ExchangeID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcProductExchRateField_swiginit(self, _thostmduserapi.new_CThostFtdcProductExchRateField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcProductExchRateField

# Register CThostFtdcProductExchRateField in _thostmduserapi:
_thostmduserapi.CThostFtdcProductExchRateField_swigregister(CThostFtdcProductExchRateField)

class CThostFtdcQryProductExchRateField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ProductID = property(_thostmduserapi.CThostFtdcQryProductExchRateField_ProductID_get, _thostmduserapi.CThostFtdcQryProductExchRateField_ProductID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcQryProductExchRateField_ExchangeID_get, _thostmduserapi.CThostFtdcQryProductExchRateField_ExchangeID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryProductExchRateField_swiginit(self, _thostmduserapi.new_CThostFtdcQryProductExchRateField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryProductExchRateField

# Register CThostFtdcQryProductExchRateField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryProductExchRateField_swigregister(CThostFtdcQryProductExchRateField)

class CThostFtdcQryForQuoteParamField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQryForQuoteParamField_BrokerID_get, _thostmduserapi.CThostFtdcQryForQuoteParamField_BrokerID_set)
    InstrumentID = property(_thostmduserapi.CThostFtdcQryForQuoteParamField_InstrumentID_get, _thostmduserapi.CThostFtdcQryForQuoteParamField_InstrumentID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcQryForQuoteParamField_ExchangeID_get, _thostmduserapi.CThostFtdcQryForQuoteParamField_ExchangeID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryForQuoteParamField_swiginit(self, _thostmduserapi.new_CThostFtdcQryForQuoteParamField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryForQuoteParamField

# Register CThostFtdcQryForQuoteParamField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryForQuoteParamField_swigregister(CThostFtdcQryForQuoteParamField)

class CThostFtdcForQuoteParamField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcForQuoteParamField_BrokerID_get, _thostmduserapi.CThostFtdcForQuoteParamField_BrokerID_set)
    InstrumentID = property(_thostmduserapi.CThostFtdcForQuoteParamField_InstrumentID_get, _thostmduserapi.CThostFtdcForQuoteParamField_InstrumentID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcForQuoteParamField_ExchangeID_get, _thostmduserapi.CThostFtdcForQuoteParamField_ExchangeID_set)
    LastPrice = property(_thostmduserapi.CThostFtdcForQuoteParamField_LastPrice_get, _thostmduserapi.CThostFtdcForQuoteParamField_LastPrice_set)
    PriceInterval = property(_thostmduserapi.CThostFtdcForQuoteParamField_PriceInterval_get, _thostmduserapi.CThostFtdcForQuoteParamField_PriceInterval_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcForQuoteParamField_swiginit(self, _thostmduserapi.new_CThostFtdcForQuoteParamField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcForQuoteParamField

# Register CThostFtdcForQuoteParamField in _thostmduserapi:
_thostmduserapi.CThostFtdcForQuoteParamField_swigregister(CThostFtdcForQuoteParamField)

class CThostFtdcMMOptionInstrCommRateField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    InstrumentID = property(_thostmduserapi.CThostFtdcMMOptionInstrCommRateField_InstrumentID_get, _thostmduserapi.CThostFtdcMMOptionInstrCommRateField_InstrumentID_set)
    InvestorRange = property(_thostmduserapi.CThostFtdcMMOptionInstrCommRateField_InvestorRange_get, _thostmduserapi.CThostFtdcMMOptionInstrCommRateField_InvestorRange_set)
    BrokerID = property(_thostmduserapi.CThostFtdcMMOptionInstrCommRateField_BrokerID_get, _thostmduserapi.CThostFtdcMMOptionInstrCommRateField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcMMOptionInstrCommRateField_InvestorID_get, _thostmduserapi.CThostFtdcMMOptionInstrCommRateField_InvestorID_set)
    OpenRatioByMoney = property(_thostmduserapi.CThostFtdcMMOptionInstrCommRateField_OpenRatioByMoney_get, _thostmduserapi.CThostFtdcMMOptionInstrCommRateField_OpenRatioByMoney_set)
    OpenRatioByVolume = property(_thostmduserapi.CThostFtdcMMOptionInstrCommRateField_OpenRatioByVolume_get, _thostmduserapi.CThostFtdcMMOptionInstrCommRateField_OpenRatioByVolume_set)
    CloseRatioByMoney = property(_thostmduserapi.CThostFtdcMMOptionInstrCommRateField_CloseRatioByMoney_get, _thostmduserapi.CThostFtdcMMOptionInstrCommRateField_CloseRatioByMoney_set)
    CloseRatioByVolume = property(_thostmduserapi.CThostFtdcMMOptionInstrCommRateField_CloseRatioByVolume_get, _thostmduserapi.CThostFtdcMMOptionInstrCommRateField_CloseRatioByVolume_set)
    CloseTodayRatioByMoney = property(_thostmduserapi.CThostFtdcMMOptionInstrCommRateField_CloseTodayRatioByMoney_get, _thostmduserapi.CThostFtdcMMOptionInstrCommRateField_CloseTodayRatioByMoney_set)
    CloseTodayRatioByVolume = property(_thostmduserapi.CThostFtdcMMOptionInstrCommRateField_CloseTodayRatioByVolume_get, _thostmduserapi.CThostFtdcMMOptionInstrCommRateField_CloseTodayRatioByVolume_set)
    StrikeRatioByMoney = property(_thostmduserapi.CThostFtdcMMOptionInstrCommRateField_StrikeRatioByMoney_get, _thostmduserapi.CThostFtdcMMOptionInstrCommRateField_StrikeRatioByMoney_set)
    StrikeRatioByVolume = property(_thostmduserapi.CThostFtdcMMOptionInstrCommRateField_StrikeRatioByVolume_get, _thostmduserapi.CThostFtdcMMOptionInstrCommRateField_StrikeRatioByVolume_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcMMOptionInstrCommRateField_swiginit(self, _thostmduserapi.new_CThostFtdcMMOptionInstrCommRateField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcMMOptionInstrCommRateField

# Register CThostFtdcMMOptionInstrCommRateField in _thostmduserapi:
_thostmduserapi.CThostFtdcMMOptionInstrCommRateField_swigregister(CThostFtdcMMOptionInstrCommRateField)

class CThostFtdcQryMMOptionInstrCommRateField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQryMMOptionInstrCommRateField_BrokerID_get, _thostmduserapi.CThostFtdcQryMMOptionInstrCommRateField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcQryMMOptionInstrCommRateField_InvestorID_get, _thostmduserapi.CThostFtdcQryMMOptionInstrCommRateField_InvestorID_set)
    InstrumentID = property(_thostmduserapi.CThostFtdcQryMMOptionInstrCommRateField_InstrumentID_get, _thostmduserapi.CThostFtdcQryMMOptionInstrCommRateField_InstrumentID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryMMOptionInstrCommRateField_swiginit(self, _thostmduserapi.new_CThostFtdcQryMMOptionInstrCommRateField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryMMOptionInstrCommRateField

# Register CThostFtdcQryMMOptionInstrCommRateField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryMMOptionInstrCommRateField_swigregister(CThostFtdcQryMMOptionInstrCommRateField)

class CThostFtdcMMInstrumentCommissionRateField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    InstrumentID = property(_thostmduserapi.CThostFtdcMMInstrumentCommissionRateField_InstrumentID_get, _thostmduserapi.CThostFtdcMMInstrumentCommissionRateField_InstrumentID_set)
    InvestorRange = property(_thostmduserapi.CThostFtdcMMInstrumentCommissionRateField_InvestorRange_get, _thostmduserapi.CThostFtdcMMInstrumentCommissionRateField_InvestorRange_set)
    BrokerID = property(_thostmduserapi.CThostFtdcMMInstrumentCommissionRateField_BrokerID_get, _thostmduserapi.CThostFtdcMMInstrumentCommissionRateField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcMMInstrumentCommissionRateField_InvestorID_get, _thostmduserapi.CThostFtdcMMInstrumentCommissionRateField_InvestorID_set)
    OpenRatioByMoney = property(_thostmduserapi.CThostFtdcMMInstrumentCommissionRateField_OpenRatioByMoney_get, _thostmduserapi.CThostFtdcMMInstrumentCommissionRateField_OpenRatioByMoney_set)
    OpenRatioByVolume = property(_thostmduserapi.CThostFtdcMMInstrumentCommissionRateField_OpenRatioByVolume_get, _thostmduserapi.CThostFtdcMMInstrumentCommissionRateField_OpenRatioByVolume_set)
    CloseRatioByMoney = property(_thostmduserapi.CThostFtdcMMInstrumentCommissionRateField_CloseRatioByMoney_get, _thostmduserapi.CThostFtdcMMInstrumentCommissionRateField_CloseRatioByMoney_set)
    CloseRatioByVolume = property(_thostmduserapi.CThostFtdcMMInstrumentCommissionRateField_CloseRatioByVolume_get, _thostmduserapi.CThostFtdcMMInstrumentCommissionRateField_CloseRatioByVolume_set)
    CloseTodayRatioByMoney = property(_thostmduserapi.CThostFtdcMMInstrumentCommissionRateField_CloseTodayRatioByMoney_get, _thostmduserapi.CThostFtdcMMInstrumentCommissionRateField_CloseTodayRatioByMoney_set)
    CloseTodayRatioByVolume = property(_thostmduserapi.CThostFtdcMMInstrumentCommissionRateField_CloseTodayRatioByVolume_get, _thostmduserapi.CThostFtdcMMInstrumentCommissionRateField_CloseTodayRatioByVolume_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcMMInstrumentCommissionRateField_swiginit(self, _thostmduserapi.new_CThostFtdcMMInstrumentCommissionRateField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcMMInstrumentCommissionRateField

# Register CThostFtdcMMInstrumentCommissionRateField in _thostmduserapi:
_thostmduserapi.CThostFtdcMMInstrumentCommissionRateField_swigregister(CThostFtdcMMInstrumentCommissionRateField)

class CThostFtdcQryMMInstrumentCommissionRateField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQryMMInstrumentCommissionRateField_BrokerID_get, _thostmduserapi.CThostFtdcQryMMInstrumentCommissionRateField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcQryMMInstrumentCommissionRateField_InvestorID_get, _thostmduserapi.CThostFtdcQryMMInstrumentCommissionRateField_InvestorID_set)
    InstrumentID = property(_thostmduserapi.CThostFtdcQryMMInstrumentCommissionRateField_InstrumentID_get, _thostmduserapi.CThostFtdcQryMMInstrumentCommissionRateField_InstrumentID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryMMInstrumentCommissionRateField_swiginit(self, _thostmduserapi.new_CThostFtdcQryMMInstrumentCommissionRateField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryMMInstrumentCommissionRateField

# Register CThostFtdcQryMMInstrumentCommissionRateField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryMMInstrumentCommissionRateField_swigregister(CThostFtdcQryMMInstrumentCommissionRateField)

class CThostFtdcInstrumentOrderCommRateField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    InstrumentID = property(_thostmduserapi.CThostFtdcInstrumentOrderCommRateField_InstrumentID_get, _thostmduserapi.CThostFtdcInstrumentOrderCommRateField_InstrumentID_set)
    InvestorRange = property(_thostmduserapi.CThostFtdcInstrumentOrderCommRateField_InvestorRange_get, _thostmduserapi.CThostFtdcInstrumentOrderCommRateField_InvestorRange_set)
    BrokerID = property(_thostmduserapi.CThostFtdcInstrumentOrderCommRateField_BrokerID_get, _thostmduserapi.CThostFtdcInstrumentOrderCommRateField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcInstrumentOrderCommRateField_InvestorID_get, _thostmduserapi.CThostFtdcInstrumentOrderCommRateField_InvestorID_set)
    HedgeFlag = property(_thostmduserapi.CThostFtdcInstrumentOrderCommRateField_HedgeFlag_get, _thostmduserapi.CThostFtdcInstrumentOrderCommRateField_HedgeFlag_set)
    OrderCommByVolume = property(_thostmduserapi.CThostFtdcInstrumentOrderCommRateField_OrderCommByVolume_get, _thostmduserapi.CThostFtdcInstrumentOrderCommRateField_OrderCommByVolume_set)
    OrderActionCommByVolume = property(_thostmduserapi.CThostFtdcInstrumentOrderCommRateField_OrderActionCommByVolume_get, _thostmduserapi.CThostFtdcInstrumentOrderCommRateField_OrderActionCommByVolume_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcInstrumentOrderCommRateField_ExchangeID_get, _thostmduserapi.CThostFtdcInstrumentOrderCommRateField_ExchangeID_set)
    InvestUnitID = property(_thostmduserapi.CThostFtdcInstrumentOrderCommRateField_InvestUnitID_get, _thostmduserapi.CThostFtdcInstrumentOrderCommRateField_InvestUnitID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcInstrumentOrderCommRateField_swiginit(self, _thostmduserapi.new_CThostFtdcInstrumentOrderCommRateField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcInstrumentOrderCommRateField

# Register CThostFtdcInstrumentOrderCommRateField in _thostmduserapi:
_thostmduserapi.CThostFtdcInstrumentOrderCommRateField_swigregister(CThostFtdcInstrumentOrderCommRateField)

class CThostFtdcQryInstrumentOrderCommRateField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQryInstrumentOrderCommRateField_BrokerID_get, _thostmduserapi.CThostFtdcQryInstrumentOrderCommRateField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcQryInstrumentOrderCommRateField_InvestorID_get, _thostmduserapi.CThostFtdcQryInstrumentOrderCommRateField_InvestorID_set)
    InstrumentID = property(_thostmduserapi.CThostFtdcQryInstrumentOrderCommRateField_InstrumentID_get, _thostmduserapi.CThostFtdcQryInstrumentOrderCommRateField_InstrumentID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryInstrumentOrderCommRateField_swiginit(self, _thostmduserapi.new_CThostFtdcQryInstrumentOrderCommRateField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryInstrumentOrderCommRateField

# Register CThostFtdcQryInstrumentOrderCommRateField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryInstrumentOrderCommRateField_swigregister(CThostFtdcQryInstrumentOrderCommRateField)

class CThostFtdcTradeParamField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcTradeParamField_BrokerID_get, _thostmduserapi.CThostFtdcTradeParamField_BrokerID_set)
    TradeParamID = property(_thostmduserapi.CThostFtdcTradeParamField_TradeParamID_get, _thostmduserapi.CThostFtdcTradeParamField_TradeParamID_set)
    TradeParamValue = property(_thostmduserapi.CThostFtdcTradeParamField_TradeParamValue_get, _thostmduserapi.CThostFtdcTradeParamField_TradeParamValue_set)
    Memo = property(_thostmduserapi.CThostFtdcTradeParamField_Memo_get, _thostmduserapi.CThostFtdcTradeParamField_Memo_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcTradeParamField_swiginit(self, _thostmduserapi.new_CThostFtdcTradeParamField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcTradeParamField

# Register CThostFtdcTradeParamField in _thostmduserapi:
_thostmduserapi.CThostFtdcTradeParamField_swigregister(CThostFtdcTradeParamField)

class CThostFtdcInstrumentMarginRateULField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    InstrumentID = property(_thostmduserapi.CThostFtdcInstrumentMarginRateULField_InstrumentID_get, _thostmduserapi.CThostFtdcInstrumentMarginRateULField_InstrumentID_set)
    InvestorRange = property(_thostmduserapi.CThostFtdcInstrumentMarginRateULField_InvestorRange_get, _thostmduserapi.CThostFtdcInstrumentMarginRateULField_InvestorRange_set)
    BrokerID = property(_thostmduserapi.CThostFtdcInstrumentMarginRateULField_BrokerID_get, _thostmduserapi.CThostFtdcInstrumentMarginRateULField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcInstrumentMarginRateULField_InvestorID_get, _thostmduserapi.CThostFtdcInstrumentMarginRateULField_InvestorID_set)
    HedgeFlag = property(_thostmduserapi.CThostFtdcInstrumentMarginRateULField_HedgeFlag_get, _thostmduserapi.CThostFtdcInstrumentMarginRateULField_HedgeFlag_set)
    LongMarginRatioByMoney = property(_thostmduserapi.CThostFtdcInstrumentMarginRateULField_LongMarginRatioByMoney_get, _thostmduserapi.CThostFtdcInstrumentMarginRateULField_LongMarginRatioByMoney_set)
    LongMarginRatioByVolume = property(_thostmduserapi.CThostFtdcInstrumentMarginRateULField_LongMarginRatioByVolume_get, _thostmduserapi.CThostFtdcInstrumentMarginRateULField_LongMarginRatioByVolume_set)
    ShortMarginRatioByMoney = property(_thostmduserapi.CThostFtdcInstrumentMarginRateULField_ShortMarginRatioByMoney_get, _thostmduserapi.CThostFtdcInstrumentMarginRateULField_ShortMarginRatioByMoney_set)
    ShortMarginRatioByVolume = property(_thostmduserapi.CThostFtdcInstrumentMarginRateULField_ShortMarginRatioByVolume_get, _thostmduserapi.CThostFtdcInstrumentMarginRateULField_ShortMarginRatioByVolume_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcInstrumentMarginRateULField_swiginit(self, _thostmduserapi.new_CThostFtdcInstrumentMarginRateULField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcInstrumentMarginRateULField

# Register CThostFtdcInstrumentMarginRateULField in _thostmduserapi:
_thostmduserapi.CThostFtdcInstrumentMarginRateULField_swigregister(CThostFtdcInstrumentMarginRateULField)

class CThostFtdcFutureLimitPosiParamField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    InvestorRange = property(_thostmduserapi.CThostFtdcFutureLimitPosiParamField_InvestorRange_get, _thostmduserapi.CThostFtdcFutureLimitPosiParamField_InvestorRange_set)
    BrokerID = property(_thostmduserapi.CThostFtdcFutureLimitPosiParamField_BrokerID_get, _thostmduserapi.CThostFtdcFutureLimitPosiParamField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcFutureLimitPosiParamField_InvestorID_get, _thostmduserapi.CThostFtdcFutureLimitPosiParamField_InvestorID_set)
    ProductID = property(_thostmduserapi.CThostFtdcFutureLimitPosiParamField_ProductID_get, _thostmduserapi.CThostFtdcFutureLimitPosiParamField_ProductID_set)
    SpecOpenVolume = property(_thostmduserapi.CThostFtdcFutureLimitPosiParamField_SpecOpenVolume_get, _thostmduserapi.CThostFtdcFutureLimitPosiParamField_SpecOpenVolume_set)
    ArbiOpenVolume = property(_thostmduserapi.CThostFtdcFutureLimitPosiParamField_ArbiOpenVolume_get, _thostmduserapi.CThostFtdcFutureLimitPosiParamField_ArbiOpenVolume_set)
    OpenVolume = property(_thostmduserapi.CThostFtdcFutureLimitPosiParamField_OpenVolume_get, _thostmduserapi.CThostFtdcFutureLimitPosiParamField_OpenVolume_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcFutureLimitPosiParamField_swiginit(self, _thostmduserapi.new_CThostFtdcFutureLimitPosiParamField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcFutureLimitPosiParamField

# Register CThostFtdcFutureLimitPosiParamField in _thostmduserapi:
_thostmduserapi.CThostFtdcFutureLimitPosiParamField_swigregister(CThostFtdcFutureLimitPosiParamField)

class CThostFtdcLoginForbiddenIPField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    IPAddress = property(_thostmduserapi.CThostFtdcLoginForbiddenIPField_IPAddress_get, _thostmduserapi.CThostFtdcLoginForbiddenIPField_IPAddress_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcLoginForbiddenIPField_swiginit(self, _thostmduserapi.new_CThostFtdcLoginForbiddenIPField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcLoginForbiddenIPField

# Register CThostFtdcLoginForbiddenIPField in _thostmduserapi:
_thostmduserapi.CThostFtdcLoginForbiddenIPField_swigregister(CThostFtdcLoginForbiddenIPField)

class CThostFtdcIPListField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    IPAddress = property(_thostmduserapi.CThostFtdcIPListField_IPAddress_get, _thostmduserapi.CThostFtdcIPListField_IPAddress_set)
    IsWhite = property(_thostmduserapi.CThostFtdcIPListField_IsWhite_get, _thostmduserapi.CThostFtdcIPListField_IsWhite_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcIPListField_swiginit(self, _thostmduserapi.new_CThostFtdcIPListField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcIPListField

# Register CThostFtdcIPListField in _thostmduserapi:
_thostmduserapi.CThostFtdcIPListField_swigregister(CThostFtdcIPListField)

class CThostFtdcInputOptionSelfCloseField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcInputOptionSelfCloseField_BrokerID_get, _thostmduserapi.CThostFtdcInputOptionSelfCloseField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcInputOptionSelfCloseField_InvestorID_get, _thostmduserapi.CThostFtdcInputOptionSelfCloseField_InvestorID_set)
    InstrumentID = property(_thostmduserapi.CThostFtdcInputOptionSelfCloseField_InstrumentID_get, _thostmduserapi.CThostFtdcInputOptionSelfCloseField_InstrumentID_set)
    OptionSelfCloseRef = property(_thostmduserapi.CThostFtdcInputOptionSelfCloseField_OptionSelfCloseRef_get, _thostmduserapi.CThostFtdcInputOptionSelfCloseField_OptionSelfCloseRef_set)
    UserID = property(_thostmduserapi.CThostFtdcInputOptionSelfCloseField_UserID_get, _thostmduserapi.CThostFtdcInputOptionSelfCloseField_UserID_set)
    Volume = property(_thostmduserapi.CThostFtdcInputOptionSelfCloseField_Volume_get, _thostmduserapi.CThostFtdcInputOptionSelfCloseField_Volume_set)
    RequestID = property(_thostmduserapi.CThostFtdcInputOptionSelfCloseField_RequestID_get, _thostmduserapi.CThostFtdcInputOptionSelfCloseField_RequestID_set)
    BusinessUnit = property(_thostmduserapi.CThostFtdcInputOptionSelfCloseField_BusinessUnit_get, _thostmduserapi.CThostFtdcInputOptionSelfCloseField_BusinessUnit_set)
    HedgeFlag = property(_thostmduserapi.CThostFtdcInputOptionSelfCloseField_HedgeFlag_get, _thostmduserapi.CThostFtdcInputOptionSelfCloseField_HedgeFlag_set)
    OptSelfCloseFlag = property(_thostmduserapi.CThostFtdcInputOptionSelfCloseField_OptSelfCloseFlag_get, _thostmduserapi.CThostFtdcInputOptionSelfCloseField_OptSelfCloseFlag_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcInputOptionSelfCloseField_ExchangeID_get, _thostmduserapi.CThostFtdcInputOptionSelfCloseField_ExchangeID_set)
    InvestUnitID = property(_thostmduserapi.CThostFtdcInputOptionSelfCloseField_InvestUnitID_get, _thostmduserapi.CThostFtdcInputOptionSelfCloseField_InvestUnitID_set)
    AccountID = property(_thostmduserapi.CThostFtdcInputOptionSelfCloseField_AccountID_get, _thostmduserapi.CThostFtdcInputOptionSelfCloseField_AccountID_set)
    CurrencyID = property(_thostmduserapi.CThostFtdcInputOptionSelfCloseField_CurrencyID_get, _thostmduserapi.CThostFtdcInputOptionSelfCloseField_CurrencyID_set)
    ClientID = property(_thostmduserapi.CThostFtdcInputOptionSelfCloseField_ClientID_get, _thostmduserapi.CThostFtdcInputOptionSelfCloseField_ClientID_set)
    IPAddress = property(_thostmduserapi.CThostFtdcInputOptionSelfCloseField_IPAddress_get, _thostmduserapi.CThostFtdcInputOptionSelfCloseField_IPAddress_set)
    MacAddress = property(_thostmduserapi.CThostFtdcInputOptionSelfCloseField_MacAddress_get, _thostmduserapi.CThostFtdcInputOptionSelfCloseField_MacAddress_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcInputOptionSelfCloseField_swiginit(self, _thostmduserapi.new_CThostFtdcInputOptionSelfCloseField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcInputOptionSelfCloseField

# Register CThostFtdcInputOptionSelfCloseField in _thostmduserapi:
_thostmduserapi.CThostFtdcInputOptionSelfCloseField_swigregister(CThostFtdcInputOptionSelfCloseField)

class CThostFtdcInputOptionSelfCloseActionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcInputOptionSelfCloseActionField_BrokerID_get, _thostmduserapi.CThostFtdcInputOptionSelfCloseActionField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcInputOptionSelfCloseActionField_InvestorID_get, _thostmduserapi.CThostFtdcInputOptionSelfCloseActionField_InvestorID_set)
    OptionSelfCloseActionRef = property(_thostmduserapi.CThostFtdcInputOptionSelfCloseActionField_OptionSelfCloseActionRef_get, _thostmduserapi.CThostFtdcInputOptionSelfCloseActionField_OptionSelfCloseActionRef_set)
    OptionSelfCloseRef = property(_thostmduserapi.CThostFtdcInputOptionSelfCloseActionField_OptionSelfCloseRef_get, _thostmduserapi.CThostFtdcInputOptionSelfCloseActionField_OptionSelfCloseRef_set)
    RequestID = property(_thostmduserapi.CThostFtdcInputOptionSelfCloseActionField_RequestID_get, _thostmduserapi.CThostFtdcInputOptionSelfCloseActionField_RequestID_set)
    FrontID = property(_thostmduserapi.CThostFtdcInputOptionSelfCloseActionField_FrontID_get, _thostmduserapi.CThostFtdcInputOptionSelfCloseActionField_FrontID_set)
    SessionID = property(_thostmduserapi.CThostFtdcInputOptionSelfCloseActionField_SessionID_get, _thostmduserapi.CThostFtdcInputOptionSelfCloseActionField_SessionID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcInputOptionSelfCloseActionField_ExchangeID_get, _thostmduserapi.CThostFtdcInputOptionSelfCloseActionField_ExchangeID_set)
    OptionSelfCloseSysID = property(_thostmduserapi.CThostFtdcInputOptionSelfCloseActionField_OptionSelfCloseSysID_get, _thostmduserapi.CThostFtdcInputOptionSelfCloseActionField_OptionSelfCloseSysID_set)
    ActionFlag = property(_thostmduserapi.CThostFtdcInputOptionSelfCloseActionField_ActionFlag_get, _thostmduserapi.CThostFtdcInputOptionSelfCloseActionField_ActionFlag_set)
    UserID = property(_thostmduserapi.CThostFtdcInputOptionSelfCloseActionField_UserID_get, _thostmduserapi.CThostFtdcInputOptionSelfCloseActionField_UserID_set)
    InstrumentID = property(_thostmduserapi.CThostFtdcInputOptionSelfCloseActionField_InstrumentID_get, _thostmduserapi.CThostFtdcInputOptionSelfCloseActionField_InstrumentID_set)
    InvestUnitID = property(_thostmduserapi.CThostFtdcInputOptionSelfCloseActionField_InvestUnitID_get, _thostmduserapi.CThostFtdcInputOptionSelfCloseActionField_InvestUnitID_set)
    IPAddress = property(_thostmduserapi.CThostFtdcInputOptionSelfCloseActionField_IPAddress_get, _thostmduserapi.CThostFtdcInputOptionSelfCloseActionField_IPAddress_set)
    MacAddress = property(_thostmduserapi.CThostFtdcInputOptionSelfCloseActionField_MacAddress_get, _thostmduserapi.CThostFtdcInputOptionSelfCloseActionField_MacAddress_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcInputOptionSelfCloseActionField_swiginit(self, _thostmduserapi.new_CThostFtdcInputOptionSelfCloseActionField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcInputOptionSelfCloseActionField

# Register CThostFtdcInputOptionSelfCloseActionField in _thostmduserapi:
_thostmduserapi.CThostFtdcInputOptionSelfCloseActionField_swigregister(CThostFtdcInputOptionSelfCloseActionField)

class CThostFtdcOptionSelfCloseField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcOptionSelfCloseField_BrokerID_get, _thostmduserapi.CThostFtdcOptionSelfCloseField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcOptionSelfCloseField_InvestorID_get, _thostmduserapi.CThostFtdcOptionSelfCloseField_InvestorID_set)
    InstrumentID = property(_thostmduserapi.CThostFtdcOptionSelfCloseField_InstrumentID_get, _thostmduserapi.CThostFtdcOptionSelfCloseField_InstrumentID_set)
    OptionSelfCloseRef = property(_thostmduserapi.CThostFtdcOptionSelfCloseField_OptionSelfCloseRef_get, _thostmduserapi.CThostFtdcOptionSelfCloseField_OptionSelfCloseRef_set)
    UserID = property(_thostmduserapi.CThostFtdcOptionSelfCloseField_UserID_get, _thostmduserapi.CThostFtdcOptionSelfCloseField_UserID_set)
    Volume = property(_thostmduserapi.CThostFtdcOptionSelfCloseField_Volume_get, _thostmduserapi.CThostFtdcOptionSelfCloseField_Volume_set)
    RequestID = property(_thostmduserapi.CThostFtdcOptionSelfCloseField_RequestID_get, _thostmduserapi.CThostFtdcOptionSelfCloseField_RequestID_set)
    BusinessUnit = property(_thostmduserapi.CThostFtdcOptionSelfCloseField_BusinessUnit_get, _thostmduserapi.CThostFtdcOptionSelfCloseField_BusinessUnit_set)
    HedgeFlag = property(_thostmduserapi.CThostFtdcOptionSelfCloseField_HedgeFlag_get, _thostmduserapi.CThostFtdcOptionSelfCloseField_HedgeFlag_set)
    OptSelfCloseFlag = property(_thostmduserapi.CThostFtdcOptionSelfCloseField_OptSelfCloseFlag_get, _thostmduserapi.CThostFtdcOptionSelfCloseField_OptSelfCloseFlag_set)
    OptionSelfCloseLocalID = property(_thostmduserapi.CThostFtdcOptionSelfCloseField_OptionSelfCloseLocalID_get, _thostmduserapi.CThostFtdcOptionSelfCloseField_OptionSelfCloseLocalID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcOptionSelfCloseField_ExchangeID_get, _thostmduserapi.CThostFtdcOptionSelfCloseField_ExchangeID_set)
    ParticipantID = property(_thostmduserapi.CThostFtdcOptionSelfCloseField_ParticipantID_get, _thostmduserapi.CThostFtdcOptionSelfCloseField_ParticipantID_set)
    ClientID = property(_thostmduserapi.CThostFtdcOptionSelfCloseField_ClientID_get, _thostmduserapi.CThostFtdcOptionSelfCloseField_ClientID_set)
    ExchangeInstID = property(_thostmduserapi.CThostFtdcOptionSelfCloseField_ExchangeInstID_get, _thostmduserapi.CThostFtdcOptionSelfCloseField_ExchangeInstID_set)
    TraderID = property(_thostmduserapi.CThostFtdcOptionSelfCloseField_TraderID_get, _thostmduserapi.CThostFtdcOptionSelfCloseField_TraderID_set)
    InstallID = property(_thostmduserapi.CThostFtdcOptionSelfCloseField_InstallID_get, _thostmduserapi.CThostFtdcOptionSelfCloseField_InstallID_set)
    OrderSubmitStatus = property(_thostmduserapi.CThostFtdcOptionSelfCloseField_OrderSubmitStatus_get, _thostmduserapi.CThostFtdcOptionSelfCloseField_OrderSubmitStatus_set)
    NotifySequence = property(_thostmduserapi.CThostFtdcOptionSelfCloseField_NotifySequence_get, _thostmduserapi.CThostFtdcOptionSelfCloseField_NotifySequence_set)
    TradingDay = property(_thostmduserapi.CThostFtdcOptionSelfCloseField_TradingDay_get, _thostmduserapi.CThostFtdcOptionSelfCloseField_TradingDay_set)
    SettlementID = property(_thostmduserapi.CThostFtdcOptionSelfCloseField_SettlementID_get, _thostmduserapi.CThostFtdcOptionSelfCloseField_SettlementID_set)
    OptionSelfCloseSysID = property(_thostmduserapi.CThostFtdcOptionSelfCloseField_OptionSelfCloseSysID_get, _thostmduserapi.CThostFtdcOptionSelfCloseField_OptionSelfCloseSysID_set)
    InsertDate = property(_thostmduserapi.CThostFtdcOptionSelfCloseField_InsertDate_get, _thostmduserapi.CThostFtdcOptionSelfCloseField_InsertDate_set)
    InsertTime = property(_thostmduserapi.CThostFtdcOptionSelfCloseField_InsertTime_get, _thostmduserapi.CThostFtdcOptionSelfCloseField_InsertTime_set)
    CancelTime = property(_thostmduserapi.CThostFtdcOptionSelfCloseField_CancelTime_get, _thostmduserapi.CThostFtdcOptionSelfCloseField_CancelTime_set)
    ExecResult = property(_thostmduserapi.CThostFtdcOptionSelfCloseField_ExecResult_get, _thostmduserapi.CThostFtdcOptionSelfCloseField_ExecResult_set)
    ClearingPartID = property(_thostmduserapi.CThostFtdcOptionSelfCloseField_ClearingPartID_get, _thostmduserapi.CThostFtdcOptionSelfCloseField_ClearingPartID_set)
    SequenceNo = property(_thostmduserapi.CThostFtdcOptionSelfCloseField_SequenceNo_get, _thostmduserapi.CThostFtdcOptionSelfCloseField_SequenceNo_set)
    FrontID = property(_thostmduserapi.CThostFtdcOptionSelfCloseField_FrontID_get, _thostmduserapi.CThostFtdcOptionSelfCloseField_FrontID_set)
    SessionID = property(_thostmduserapi.CThostFtdcOptionSelfCloseField_SessionID_get, _thostmduserapi.CThostFtdcOptionSelfCloseField_SessionID_set)
    UserProductInfo = property(_thostmduserapi.CThostFtdcOptionSelfCloseField_UserProductInfo_get, _thostmduserapi.CThostFtdcOptionSelfCloseField_UserProductInfo_set)
    StatusMsg = property(_thostmduserapi.CThostFtdcOptionSelfCloseField_StatusMsg_get, _thostmduserapi.CThostFtdcOptionSelfCloseField_StatusMsg_set)
    ActiveUserID = property(_thostmduserapi.CThostFtdcOptionSelfCloseField_ActiveUserID_get, _thostmduserapi.CThostFtdcOptionSelfCloseField_ActiveUserID_set)
    BrokerOptionSelfCloseSeq = property(_thostmduserapi.CThostFtdcOptionSelfCloseField_BrokerOptionSelfCloseSeq_get, _thostmduserapi.CThostFtdcOptionSelfCloseField_BrokerOptionSelfCloseSeq_set)
    BranchID = property(_thostmduserapi.CThostFtdcOptionSelfCloseField_BranchID_get, _thostmduserapi.CThostFtdcOptionSelfCloseField_BranchID_set)
    InvestUnitID = property(_thostmduserapi.CThostFtdcOptionSelfCloseField_InvestUnitID_get, _thostmduserapi.CThostFtdcOptionSelfCloseField_InvestUnitID_set)
    AccountID = property(_thostmduserapi.CThostFtdcOptionSelfCloseField_AccountID_get, _thostmduserapi.CThostFtdcOptionSelfCloseField_AccountID_set)
    CurrencyID = property(_thostmduserapi.CThostFtdcOptionSelfCloseField_CurrencyID_get, _thostmduserapi.CThostFtdcOptionSelfCloseField_CurrencyID_set)
    IPAddress = property(_thostmduserapi.CThostFtdcOptionSelfCloseField_IPAddress_get, _thostmduserapi.CThostFtdcOptionSelfCloseField_IPAddress_set)
    MacAddress = property(_thostmduserapi.CThostFtdcOptionSelfCloseField_MacAddress_get, _thostmduserapi.CThostFtdcOptionSelfCloseField_MacAddress_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcOptionSelfCloseField_swiginit(self, _thostmduserapi.new_CThostFtdcOptionSelfCloseField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcOptionSelfCloseField

# Register CThostFtdcOptionSelfCloseField in _thostmduserapi:
_thostmduserapi.CThostFtdcOptionSelfCloseField_swigregister(CThostFtdcOptionSelfCloseField)

class CThostFtdcOptionSelfCloseActionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcOptionSelfCloseActionField_BrokerID_get, _thostmduserapi.CThostFtdcOptionSelfCloseActionField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcOptionSelfCloseActionField_InvestorID_get, _thostmduserapi.CThostFtdcOptionSelfCloseActionField_InvestorID_set)
    OptionSelfCloseActionRef = property(_thostmduserapi.CThostFtdcOptionSelfCloseActionField_OptionSelfCloseActionRef_get, _thostmduserapi.CThostFtdcOptionSelfCloseActionField_OptionSelfCloseActionRef_set)
    OptionSelfCloseRef = property(_thostmduserapi.CThostFtdcOptionSelfCloseActionField_OptionSelfCloseRef_get, _thostmduserapi.CThostFtdcOptionSelfCloseActionField_OptionSelfCloseRef_set)
    RequestID = property(_thostmduserapi.CThostFtdcOptionSelfCloseActionField_RequestID_get, _thostmduserapi.CThostFtdcOptionSelfCloseActionField_RequestID_set)
    FrontID = property(_thostmduserapi.CThostFtdcOptionSelfCloseActionField_FrontID_get, _thostmduserapi.CThostFtdcOptionSelfCloseActionField_FrontID_set)
    SessionID = property(_thostmduserapi.CThostFtdcOptionSelfCloseActionField_SessionID_get, _thostmduserapi.CThostFtdcOptionSelfCloseActionField_SessionID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcOptionSelfCloseActionField_ExchangeID_get, _thostmduserapi.CThostFtdcOptionSelfCloseActionField_ExchangeID_set)
    OptionSelfCloseSysID = property(_thostmduserapi.CThostFtdcOptionSelfCloseActionField_OptionSelfCloseSysID_get, _thostmduserapi.CThostFtdcOptionSelfCloseActionField_OptionSelfCloseSysID_set)
    ActionFlag = property(_thostmduserapi.CThostFtdcOptionSelfCloseActionField_ActionFlag_get, _thostmduserapi.CThostFtdcOptionSelfCloseActionField_ActionFlag_set)
    ActionDate = property(_thostmduserapi.CThostFtdcOptionSelfCloseActionField_ActionDate_get, _thostmduserapi.CThostFtdcOptionSelfCloseActionField_ActionDate_set)
    ActionTime = property(_thostmduserapi.CThostFtdcOptionSelfCloseActionField_ActionTime_get, _thostmduserapi.CThostFtdcOptionSelfCloseActionField_ActionTime_set)
    TraderID = property(_thostmduserapi.CThostFtdcOptionSelfCloseActionField_TraderID_get, _thostmduserapi.CThostFtdcOptionSelfCloseActionField_TraderID_set)
    InstallID = property(_thostmduserapi.CThostFtdcOptionSelfCloseActionField_InstallID_get, _thostmduserapi.CThostFtdcOptionSelfCloseActionField_InstallID_set)
    OptionSelfCloseLocalID = property(_thostmduserapi.CThostFtdcOptionSelfCloseActionField_OptionSelfCloseLocalID_get, _thostmduserapi.CThostFtdcOptionSelfCloseActionField_OptionSelfCloseLocalID_set)
    ActionLocalID = property(_thostmduserapi.CThostFtdcOptionSelfCloseActionField_ActionLocalID_get, _thostmduserapi.CThostFtdcOptionSelfCloseActionField_ActionLocalID_set)
    ParticipantID = property(_thostmduserapi.CThostFtdcOptionSelfCloseActionField_ParticipantID_get, _thostmduserapi.CThostFtdcOptionSelfCloseActionField_ParticipantID_set)
    ClientID = property(_thostmduserapi.CThostFtdcOptionSelfCloseActionField_ClientID_get, _thostmduserapi.CThostFtdcOptionSelfCloseActionField_ClientID_set)
    BusinessUnit = property(_thostmduserapi.CThostFtdcOptionSelfCloseActionField_BusinessUnit_get, _thostmduserapi.CThostFtdcOptionSelfCloseActionField_BusinessUnit_set)
    OrderActionStatus = property(_thostmduserapi.CThostFtdcOptionSelfCloseActionField_OrderActionStatus_get, _thostmduserapi.CThostFtdcOptionSelfCloseActionField_OrderActionStatus_set)
    UserID = property(_thostmduserapi.CThostFtdcOptionSelfCloseActionField_UserID_get, _thostmduserapi.CThostFtdcOptionSelfCloseActionField_UserID_set)
    StatusMsg = property(_thostmduserapi.CThostFtdcOptionSelfCloseActionField_StatusMsg_get, _thostmduserapi.CThostFtdcOptionSelfCloseActionField_StatusMsg_set)
    InstrumentID = property(_thostmduserapi.CThostFtdcOptionSelfCloseActionField_InstrumentID_get, _thostmduserapi.CThostFtdcOptionSelfCloseActionField_InstrumentID_set)
    BranchID = property(_thostmduserapi.CThostFtdcOptionSelfCloseActionField_BranchID_get, _thostmduserapi.CThostFtdcOptionSelfCloseActionField_BranchID_set)
    InvestUnitID = property(_thostmduserapi.CThostFtdcOptionSelfCloseActionField_InvestUnitID_get, _thostmduserapi.CThostFtdcOptionSelfCloseActionField_InvestUnitID_set)
    IPAddress = property(_thostmduserapi.CThostFtdcOptionSelfCloseActionField_IPAddress_get, _thostmduserapi.CThostFtdcOptionSelfCloseActionField_IPAddress_set)
    MacAddress = property(_thostmduserapi.CThostFtdcOptionSelfCloseActionField_MacAddress_get, _thostmduserapi.CThostFtdcOptionSelfCloseActionField_MacAddress_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcOptionSelfCloseActionField_swiginit(self, _thostmduserapi.new_CThostFtdcOptionSelfCloseActionField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcOptionSelfCloseActionField

# Register CThostFtdcOptionSelfCloseActionField in _thostmduserapi:
_thostmduserapi.CThostFtdcOptionSelfCloseActionField_swigregister(CThostFtdcOptionSelfCloseActionField)

class CThostFtdcQryOptionSelfCloseField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQryOptionSelfCloseField_BrokerID_get, _thostmduserapi.CThostFtdcQryOptionSelfCloseField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcQryOptionSelfCloseField_InvestorID_get, _thostmduserapi.CThostFtdcQryOptionSelfCloseField_InvestorID_set)
    InstrumentID = property(_thostmduserapi.CThostFtdcQryOptionSelfCloseField_InstrumentID_get, _thostmduserapi.CThostFtdcQryOptionSelfCloseField_InstrumentID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcQryOptionSelfCloseField_ExchangeID_get, _thostmduserapi.CThostFtdcQryOptionSelfCloseField_ExchangeID_set)
    OptionSelfCloseSysID = property(_thostmduserapi.CThostFtdcQryOptionSelfCloseField_OptionSelfCloseSysID_get, _thostmduserapi.CThostFtdcQryOptionSelfCloseField_OptionSelfCloseSysID_set)
    InsertTimeStart = property(_thostmduserapi.CThostFtdcQryOptionSelfCloseField_InsertTimeStart_get, _thostmduserapi.CThostFtdcQryOptionSelfCloseField_InsertTimeStart_set)
    InsertTimeEnd = property(_thostmduserapi.CThostFtdcQryOptionSelfCloseField_InsertTimeEnd_get, _thostmduserapi.CThostFtdcQryOptionSelfCloseField_InsertTimeEnd_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryOptionSelfCloseField_swiginit(self, _thostmduserapi.new_CThostFtdcQryOptionSelfCloseField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryOptionSelfCloseField

# Register CThostFtdcQryOptionSelfCloseField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryOptionSelfCloseField_swigregister(CThostFtdcQryOptionSelfCloseField)

class CThostFtdcExchangeOptionSelfCloseField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    Volume = property(_thostmduserapi.CThostFtdcExchangeOptionSelfCloseField_Volume_get, _thostmduserapi.CThostFtdcExchangeOptionSelfCloseField_Volume_set)
    RequestID = property(_thostmduserapi.CThostFtdcExchangeOptionSelfCloseField_RequestID_get, _thostmduserapi.CThostFtdcExchangeOptionSelfCloseField_RequestID_set)
    BusinessUnit = property(_thostmduserapi.CThostFtdcExchangeOptionSelfCloseField_BusinessUnit_get, _thostmduserapi.CThostFtdcExchangeOptionSelfCloseField_BusinessUnit_set)
    HedgeFlag = property(_thostmduserapi.CThostFtdcExchangeOptionSelfCloseField_HedgeFlag_get, _thostmduserapi.CThostFtdcExchangeOptionSelfCloseField_HedgeFlag_set)
    OptSelfCloseFlag = property(_thostmduserapi.CThostFtdcExchangeOptionSelfCloseField_OptSelfCloseFlag_get, _thostmduserapi.CThostFtdcExchangeOptionSelfCloseField_OptSelfCloseFlag_set)
    OptionSelfCloseLocalID = property(_thostmduserapi.CThostFtdcExchangeOptionSelfCloseField_OptionSelfCloseLocalID_get, _thostmduserapi.CThostFtdcExchangeOptionSelfCloseField_OptionSelfCloseLocalID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcExchangeOptionSelfCloseField_ExchangeID_get, _thostmduserapi.CThostFtdcExchangeOptionSelfCloseField_ExchangeID_set)
    ParticipantID = property(_thostmduserapi.CThostFtdcExchangeOptionSelfCloseField_ParticipantID_get, _thostmduserapi.CThostFtdcExchangeOptionSelfCloseField_ParticipantID_set)
    ClientID = property(_thostmduserapi.CThostFtdcExchangeOptionSelfCloseField_ClientID_get, _thostmduserapi.CThostFtdcExchangeOptionSelfCloseField_ClientID_set)
    ExchangeInstID = property(_thostmduserapi.CThostFtdcExchangeOptionSelfCloseField_ExchangeInstID_get, _thostmduserapi.CThostFtdcExchangeOptionSelfCloseField_ExchangeInstID_set)
    TraderID = property(_thostmduserapi.CThostFtdcExchangeOptionSelfCloseField_TraderID_get, _thostmduserapi.CThostFtdcExchangeOptionSelfCloseField_TraderID_set)
    InstallID = property(_thostmduserapi.CThostFtdcExchangeOptionSelfCloseField_InstallID_get, _thostmduserapi.CThostFtdcExchangeOptionSelfCloseField_InstallID_set)
    OrderSubmitStatus = property(_thostmduserapi.CThostFtdcExchangeOptionSelfCloseField_OrderSubmitStatus_get, _thostmduserapi.CThostFtdcExchangeOptionSelfCloseField_OrderSubmitStatus_set)
    NotifySequence = property(_thostmduserapi.CThostFtdcExchangeOptionSelfCloseField_NotifySequence_get, _thostmduserapi.CThostFtdcExchangeOptionSelfCloseField_NotifySequence_set)
    TradingDay = property(_thostmduserapi.CThostFtdcExchangeOptionSelfCloseField_TradingDay_get, _thostmduserapi.CThostFtdcExchangeOptionSelfCloseField_TradingDay_set)
    SettlementID = property(_thostmduserapi.CThostFtdcExchangeOptionSelfCloseField_SettlementID_get, _thostmduserapi.CThostFtdcExchangeOptionSelfCloseField_SettlementID_set)
    OptionSelfCloseSysID = property(_thostmduserapi.CThostFtdcExchangeOptionSelfCloseField_OptionSelfCloseSysID_get, _thostmduserapi.CThostFtdcExchangeOptionSelfCloseField_OptionSelfCloseSysID_set)
    InsertDate = property(_thostmduserapi.CThostFtdcExchangeOptionSelfCloseField_InsertDate_get, _thostmduserapi.CThostFtdcExchangeOptionSelfCloseField_InsertDate_set)
    InsertTime = property(_thostmduserapi.CThostFtdcExchangeOptionSelfCloseField_InsertTime_get, _thostmduserapi.CThostFtdcExchangeOptionSelfCloseField_InsertTime_set)
    CancelTime = property(_thostmduserapi.CThostFtdcExchangeOptionSelfCloseField_CancelTime_get, _thostmduserapi.CThostFtdcExchangeOptionSelfCloseField_CancelTime_set)
    ExecResult = property(_thostmduserapi.CThostFtdcExchangeOptionSelfCloseField_ExecResult_get, _thostmduserapi.CThostFtdcExchangeOptionSelfCloseField_ExecResult_set)
    ClearingPartID = property(_thostmduserapi.CThostFtdcExchangeOptionSelfCloseField_ClearingPartID_get, _thostmduserapi.CThostFtdcExchangeOptionSelfCloseField_ClearingPartID_set)
    SequenceNo = property(_thostmduserapi.CThostFtdcExchangeOptionSelfCloseField_SequenceNo_get, _thostmduserapi.CThostFtdcExchangeOptionSelfCloseField_SequenceNo_set)
    BranchID = property(_thostmduserapi.CThostFtdcExchangeOptionSelfCloseField_BranchID_get, _thostmduserapi.CThostFtdcExchangeOptionSelfCloseField_BranchID_set)
    IPAddress = property(_thostmduserapi.CThostFtdcExchangeOptionSelfCloseField_IPAddress_get, _thostmduserapi.CThostFtdcExchangeOptionSelfCloseField_IPAddress_set)
    MacAddress = property(_thostmduserapi.CThostFtdcExchangeOptionSelfCloseField_MacAddress_get, _thostmduserapi.CThostFtdcExchangeOptionSelfCloseField_MacAddress_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcExchangeOptionSelfCloseField_swiginit(self, _thostmduserapi.new_CThostFtdcExchangeOptionSelfCloseField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcExchangeOptionSelfCloseField

# Register CThostFtdcExchangeOptionSelfCloseField in _thostmduserapi:
_thostmduserapi.CThostFtdcExchangeOptionSelfCloseField_swigregister(CThostFtdcExchangeOptionSelfCloseField)

class CThostFtdcQryOptionSelfCloseActionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQryOptionSelfCloseActionField_BrokerID_get, _thostmduserapi.CThostFtdcQryOptionSelfCloseActionField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcQryOptionSelfCloseActionField_InvestorID_get, _thostmduserapi.CThostFtdcQryOptionSelfCloseActionField_InvestorID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcQryOptionSelfCloseActionField_ExchangeID_get, _thostmduserapi.CThostFtdcQryOptionSelfCloseActionField_ExchangeID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryOptionSelfCloseActionField_swiginit(self, _thostmduserapi.new_CThostFtdcQryOptionSelfCloseActionField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryOptionSelfCloseActionField

# Register CThostFtdcQryOptionSelfCloseActionField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryOptionSelfCloseActionField_swigregister(CThostFtdcQryOptionSelfCloseActionField)

class CThostFtdcExchangeOptionSelfCloseActionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ExchangeID = property(_thostmduserapi.CThostFtdcExchangeOptionSelfCloseActionField_ExchangeID_get, _thostmduserapi.CThostFtdcExchangeOptionSelfCloseActionField_ExchangeID_set)
    OptionSelfCloseSysID = property(_thostmduserapi.CThostFtdcExchangeOptionSelfCloseActionField_OptionSelfCloseSysID_get, _thostmduserapi.CThostFtdcExchangeOptionSelfCloseActionField_OptionSelfCloseSysID_set)
    ActionFlag = property(_thostmduserapi.CThostFtdcExchangeOptionSelfCloseActionField_ActionFlag_get, _thostmduserapi.CThostFtdcExchangeOptionSelfCloseActionField_ActionFlag_set)
    ActionDate = property(_thostmduserapi.CThostFtdcExchangeOptionSelfCloseActionField_ActionDate_get, _thostmduserapi.CThostFtdcExchangeOptionSelfCloseActionField_ActionDate_set)
    ActionTime = property(_thostmduserapi.CThostFtdcExchangeOptionSelfCloseActionField_ActionTime_get, _thostmduserapi.CThostFtdcExchangeOptionSelfCloseActionField_ActionTime_set)
    TraderID = property(_thostmduserapi.CThostFtdcExchangeOptionSelfCloseActionField_TraderID_get, _thostmduserapi.CThostFtdcExchangeOptionSelfCloseActionField_TraderID_set)
    InstallID = property(_thostmduserapi.CThostFtdcExchangeOptionSelfCloseActionField_InstallID_get, _thostmduserapi.CThostFtdcExchangeOptionSelfCloseActionField_InstallID_set)
    OptionSelfCloseLocalID = property(_thostmduserapi.CThostFtdcExchangeOptionSelfCloseActionField_OptionSelfCloseLocalID_get, _thostmduserapi.CThostFtdcExchangeOptionSelfCloseActionField_OptionSelfCloseLocalID_set)
    ActionLocalID = property(_thostmduserapi.CThostFtdcExchangeOptionSelfCloseActionField_ActionLocalID_get, _thostmduserapi.CThostFtdcExchangeOptionSelfCloseActionField_ActionLocalID_set)
    ParticipantID = property(_thostmduserapi.CThostFtdcExchangeOptionSelfCloseActionField_ParticipantID_get, _thostmduserapi.CThostFtdcExchangeOptionSelfCloseActionField_ParticipantID_set)
    ClientID = property(_thostmduserapi.CThostFtdcExchangeOptionSelfCloseActionField_ClientID_get, _thostmduserapi.CThostFtdcExchangeOptionSelfCloseActionField_ClientID_set)
    BusinessUnit = property(_thostmduserapi.CThostFtdcExchangeOptionSelfCloseActionField_BusinessUnit_get, _thostmduserapi.CThostFtdcExchangeOptionSelfCloseActionField_BusinessUnit_set)
    OrderActionStatus = property(_thostmduserapi.CThostFtdcExchangeOptionSelfCloseActionField_OrderActionStatus_get, _thostmduserapi.CThostFtdcExchangeOptionSelfCloseActionField_OrderActionStatus_set)
    UserID = property(_thostmduserapi.CThostFtdcExchangeOptionSelfCloseActionField_UserID_get, _thostmduserapi.CThostFtdcExchangeOptionSelfCloseActionField_UserID_set)
    BranchID = property(_thostmduserapi.CThostFtdcExchangeOptionSelfCloseActionField_BranchID_get, _thostmduserapi.CThostFtdcExchangeOptionSelfCloseActionField_BranchID_set)
    IPAddress = property(_thostmduserapi.CThostFtdcExchangeOptionSelfCloseActionField_IPAddress_get, _thostmduserapi.CThostFtdcExchangeOptionSelfCloseActionField_IPAddress_set)
    MacAddress = property(_thostmduserapi.CThostFtdcExchangeOptionSelfCloseActionField_MacAddress_get, _thostmduserapi.CThostFtdcExchangeOptionSelfCloseActionField_MacAddress_set)
    ExchangeInstID = property(_thostmduserapi.CThostFtdcExchangeOptionSelfCloseActionField_ExchangeInstID_get, _thostmduserapi.CThostFtdcExchangeOptionSelfCloseActionField_ExchangeInstID_set)
    OptSelfCloseFlag = property(_thostmduserapi.CThostFtdcExchangeOptionSelfCloseActionField_OptSelfCloseFlag_get, _thostmduserapi.CThostFtdcExchangeOptionSelfCloseActionField_OptSelfCloseFlag_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcExchangeOptionSelfCloseActionField_swiginit(self, _thostmduserapi.new_CThostFtdcExchangeOptionSelfCloseActionField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcExchangeOptionSelfCloseActionField

# Register CThostFtdcExchangeOptionSelfCloseActionField in _thostmduserapi:
_thostmduserapi.CThostFtdcExchangeOptionSelfCloseActionField_swigregister(CThostFtdcExchangeOptionSelfCloseActionField)

class CThostFtdcSyncDelaySwapField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    DelaySwapSeqNo = property(_thostmduserapi.CThostFtdcSyncDelaySwapField_DelaySwapSeqNo_get, _thostmduserapi.CThostFtdcSyncDelaySwapField_DelaySwapSeqNo_set)
    BrokerID = property(_thostmduserapi.CThostFtdcSyncDelaySwapField_BrokerID_get, _thostmduserapi.CThostFtdcSyncDelaySwapField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcSyncDelaySwapField_InvestorID_get, _thostmduserapi.CThostFtdcSyncDelaySwapField_InvestorID_set)
    FromCurrencyID = property(_thostmduserapi.CThostFtdcSyncDelaySwapField_FromCurrencyID_get, _thostmduserapi.CThostFtdcSyncDelaySwapField_FromCurrencyID_set)
    FromAmount = property(_thostmduserapi.CThostFtdcSyncDelaySwapField_FromAmount_get, _thostmduserapi.CThostFtdcSyncDelaySwapField_FromAmount_set)
    FromFrozenSwap = property(_thostmduserapi.CThostFtdcSyncDelaySwapField_FromFrozenSwap_get, _thostmduserapi.CThostFtdcSyncDelaySwapField_FromFrozenSwap_set)
    FromRemainSwap = property(_thostmduserapi.CThostFtdcSyncDelaySwapField_FromRemainSwap_get, _thostmduserapi.CThostFtdcSyncDelaySwapField_FromRemainSwap_set)
    ToCurrencyID = property(_thostmduserapi.CThostFtdcSyncDelaySwapField_ToCurrencyID_get, _thostmduserapi.CThostFtdcSyncDelaySwapField_ToCurrencyID_set)
    ToAmount = property(_thostmduserapi.CThostFtdcSyncDelaySwapField_ToAmount_get, _thostmduserapi.CThostFtdcSyncDelaySwapField_ToAmount_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcSyncDelaySwapField_swiginit(self, _thostmduserapi.new_CThostFtdcSyncDelaySwapField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcSyncDelaySwapField

# Register CThostFtdcSyncDelaySwapField in _thostmduserapi:
_thostmduserapi.CThostFtdcSyncDelaySwapField_swigregister(CThostFtdcSyncDelaySwapField)

class CThostFtdcQrySyncDelaySwapField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQrySyncDelaySwapField_BrokerID_get, _thostmduserapi.CThostFtdcQrySyncDelaySwapField_BrokerID_set)
    DelaySwapSeqNo = property(_thostmduserapi.CThostFtdcQrySyncDelaySwapField_DelaySwapSeqNo_get, _thostmduserapi.CThostFtdcQrySyncDelaySwapField_DelaySwapSeqNo_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQrySyncDelaySwapField_swiginit(self, _thostmduserapi.new_CThostFtdcQrySyncDelaySwapField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQrySyncDelaySwapField

# Register CThostFtdcQrySyncDelaySwapField in _thostmduserapi:
_thostmduserapi.CThostFtdcQrySyncDelaySwapField_swigregister(CThostFtdcQrySyncDelaySwapField)

class CThostFtdcInvestUnitField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcInvestUnitField_BrokerID_get, _thostmduserapi.CThostFtdcInvestUnitField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcInvestUnitField_InvestorID_get, _thostmduserapi.CThostFtdcInvestUnitField_InvestorID_set)
    InvestUnitID = property(_thostmduserapi.CThostFtdcInvestUnitField_InvestUnitID_get, _thostmduserapi.CThostFtdcInvestUnitField_InvestUnitID_set)
    InvestorUnitName = property(_thostmduserapi.CThostFtdcInvestUnitField_InvestorUnitName_get, _thostmduserapi.CThostFtdcInvestUnitField_InvestorUnitName_set)
    InvestorGroupID = property(_thostmduserapi.CThostFtdcInvestUnitField_InvestorGroupID_get, _thostmduserapi.CThostFtdcInvestUnitField_InvestorGroupID_set)
    CommModelID = property(_thostmduserapi.CThostFtdcInvestUnitField_CommModelID_get, _thostmduserapi.CThostFtdcInvestUnitField_CommModelID_set)
    MarginModelID = property(_thostmduserapi.CThostFtdcInvestUnitField_MarginModelID_get, _thostmduserapi.CThostFtdcInvestUnitField_MarginModelID_set)
    AccountID = property(_thostmduserapi.CThostFtdcInvestUnitField_AccountID_get, _thostmduserapi.CThostFtdcInvestUnitField_AccountID_set)
    CurrencyID = property(_thostmduserapi.CThostFtdcInvestUnitField_CurrencyID_get, _thostmduserapi.CThostFtdcInvestUnitField_CurrencyID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcInvestUnitField_swiginit(self, _thostmduserapi.new_CThostFtdcInvestUnitField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcInvestUnitField

# Register CThostFtdcInvestUnitField in _thostmduserapi:
_thostmduserapi.CThostFtdcInvestUnitField_swigregister(CThostFtdcInvestUnitField)

class CThostFtdcQryInvestUnitField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQryInvestUnitField_BrokerID_get, _thostmduserapi.CThostFtdcQryInvestUnitField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcQryInvestUnitField_InvestorID_get, _thostmduserapi.CThostFtdcQryInvestUnitField_InvestorID_set)
    InvestUnitID = property(_thostmduserapi.CThostFtdcQryInvestUnitField_InvestUnitID_get, _thostmduserapi.CThostFtdcQryInvestUnitField_InvestUnitID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryInvestUnitField_swiginit(self, _thostmduserapi.new_CThostFtdcQryInvestUnitField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryInvestUnitField

# Register CThostFtdcQryInvestUnitField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryInvestUnitField_swigregister(CThostFtdcQryInvestUnitField)

class CThostFtdcSecAgentCheckModeField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    InvestorID = property(_thostmduserapi.CThostFtdcSecAgentCheckModeField_InvestorID_get, _thostmduserapi.CThostFtdcSecAgentCheckModeField_InvestorID_set)
    BrokerID = property(_thostmduserapi.CThostFtdcSecAgentCheckModeField_BrokerID_get, _thostmduserapi.CThostFtdcSecAgentCheckModeField_BrokerID_set)
    CurrencyID = property(_thostmduserapi.CThostFtdcSecAgentCheckModeField_CurrencyID_get, _thostmduserapi.CThostFtdcSecAgentCheckModeField_CurrencyID_set)
    BrokerSecAgentID = property(_thostmduserapi.CThostFtdcSecAgentCheckModeField_BrokerSecAgentID_get, _thostmduserapi.CThostFtdcSecAgentCheckModeField_BrokerSecAgentID_set)
    CheckSelfAccount = property(_thostmduserapi.CThostFtdcSecAgentCheckModeField_CheckSelfAccount_get, _thostmduserapi.CThostFtdcSecAgentCheckModeField_CheckSelfAccount_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcSecAgentCheckModeField_swiginit(self, _thostmduserapi.new_CThostFtdcSecAgentCheckModeField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcSecAgentCheckModeField

# Register CThostFtdcSecAgentCheckModeField in _thostmduserapi:
_thostmduserapi.CThostFtdcSecAgentCheckModeField_swigregister(CThostFtdcSecAgentCheckModeField)

class CThostFtdcSecAgentTradeInfoField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcSecAgentTradeInfoField_BrokerID_get, _thostmduserapi.CThostFtdcSecAgentTradeInfoField_BrokerID_set)
    BrokerSecAgentID = property(_thostmduserapi.CThostFtdcSecAgentTradeInfoField_BrokerSecAgentID_get, _thostmduserapi.CThostFtdcSecAgentTradeInfoField_BrokerSecAgentID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcSecAgentTradeInfoField_InvestorID_get, _thostmduserapi.CThostFtdcSecAgentTradeInfoField_InvestorID_set)
    LongCustomerName = property(_thostmduserapi.CThostFtdcSecAgentTradeInfoField_LongCustomerName_get, _thostmduserapi.CThostFtdcSecAgentTradeInfoField_LongCustomerName_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcSecAgentTradeInfoField_swiginit(self, _thostmduserapi.new_CThostFtdcSecAgentTradeInfoField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcSecAgentTradeInfoField

# Register CThostFtdcSecAgentTradeInfoField in _thostmduserapi:
_thostmduserapi.CThostFtdcSecAgentTradeInfoField_swigregister(CThostFtdcSecAgentTradeInfoField)

class CThostFtdcMarketDataField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradingDay = property(_thostmduserapi.CThostFtdcMarketDataField_TradingDay_get, _thostmduserapi.CThostFtdcMarketDataField_TradingDay_set)
    InstrumentID = property(_thostmduserapi.CThostFtdcMarketDataField_InstrumentID_get, _thostmduserapi.CThostFtdcMarketDataField_InstrumentID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcMarketDataField_ExchangeID_get, _thostmduserapi.CThostFtdcMarketDataField_ExchangeID_set)
    ExchangeInstID = property(_thostmduserapi.CThostFtdcMarketDataField_ExchangeInstID_get, _thostmduserapi.CThostFtdcMarketDataField_ExchangeInstID_set)
    LastPrice = property(_thostmduserapi.CThostFtdcMarketDataField_LastPrice_get, _thostmduserapi.CThostFtdcMarketDataField_LastPrice_set)
    PreSettlementPrice = property(_thostmduserapi.CThostFtdcMarketDataField_PreSettlementPrice_get, _thostmduserapi.CThostFtdcMarketDataField_PreSettlementPrice_set)
    PreClosePrice = property(_thostmduserapi.CThostFtdcMarketDataField_PreClosePrice_get, _thostmduserapi.CThostFtdcMarketDataField_PreClosePrice_set)
    PreOpenInterest = property(_thostmduserapi.CThostFtdcMarketDataField_PreOpenInterest_get, _thostmduserapi.CThostFtdcMarketDataField_PreOpenInterest_set)
    OpenPrice = property(_thostmduserapi.CThostFtdcMarketDataField_OpenPrice_get, _thostmduserapi.CThostFtdcMarketDataField_OpenPrice_set)
    HighestPrice = property(_thostmduserapi.CThostFtdcMarketDataField_HighestPrice_get, _thostmduserapi.CThostFtdcMarketDataField_HighestPrice_set)
    LowestPrice = property(_thostmduserapi.CThostFtdcMarketDataField_LowestPrice_get, _thostmduserapi.CThostFtdcMarketDataField_LowestPrice_set)
    Volume = property(_thostmduserapi.CThostFtdcMarketDataField_Volume_get, _thostmduserapi.CThostFtdcMarketDataField_Volume_set)
    Turnover = property(_thostmduserapi.CThostFtdcMarketDataField_Turnover_get, _thostmduserapi.CThostFtdcMarketDataField_Turnover_set)
    OpenInterest = property(_thostmduserapi.CThostFtdcMarketDataField_OpenInterest_get, _thostmduserapi.CThostFtdcMarketDataField_OpenInterest_set)
    ClosePrice = property(_thostmduserapi.CThostFtdcMarketDataField_ClosePrice_get, _thostmduserapi.CThostFtdcMarketDataField_ClosePrice_set)
    SettlementPrice = property(_thostmduserapi.CThostFtdcMarketDataField_SettlementPrice_get, _thostmduserapi.CThostFtdcMarketDataField_SettlementPrice_set)
    UpperLimitPrice = property(_thostmduserapi.CThostFtdcMarketDataField_UpperLimitPrice_get, _thostmduserapi.CThostFtdcMarketDataField_UpperLimitPrice_set)
    LowerLimitPrice = property(_thostmduserapi.CThostFtdcMarketDataField_LowerLimitPrice_get, _thostmduserapi.CThostFtdcMarketDataField_LowerLimitPrice_set)
    PreDelta = property(_thostmduserapi.CThostFtdcMarketDataField_PreDelta_get, _thostmduserapi.CThostFtdcMarketDataField_PreDelta_set)
    CurrDelta = property(_thostmduserapi.CThostFtdcMarketDataField_CurrDelta_get, _thostmduserapi.CThostFtdcMarketDataField_CurrDelta_set)
    UpdateTime = property(_thostmduserapi.CThostFtdcMarketDataField_UpdateTime_get, _thostmduserapi.CThostFtdcMarketDataField_UpdateTime_set)
    UpdateMillisec = property(_thostmduserapi.CThostFtdcMarketDataField_UpdateMillisec_get, _thostmduserapi.CThostFtdcMarketDataField_UpdateMillisec_set)
    ActionDay = property(_thostmduserapi.CThostFtdcMarketDataField_ActionDay_get, _thostmduserapi.CThostFtdcMarketDataField_ActionDay_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcMarketDataField_swiginit(self, _thostmduserapi.new_CThostFtdcMarketDataField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcMarketDataField

# Register CThostFtdcMarketDataField in _thostmduserapi:
_thostmduserapi.CThostFtdcMarketDataField_swigregister(CThostFtdcMarketDataField)

class CThostFtdcMarketDataBaseField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradingDay = property(_thostmduserapi.CThostFtdcMarketDataBaseField_TradingDay_get, _thostmduserapi.CThostFtdcMarketDataBaseField_TradingDay_set)
    PreSettlementPrice = property(_thostmduserapi.CThostFtdcMarketDataBaseField_PreSettlementPrice_get, _thostmduserapi.CThostFtdcMarketDataBaseField_PreSettlementPrice_set)
    PreClosePrice = property(_thostmduserapi.CThostFtdcMarketDataBaseField_PreClosePrice_get, _thostmduserapi.CThostFtdcMarketDataBaseField_PreClosePrice_set)
    PreOpenInterest = property(_thostmduserapi.CThostFtdcMarketDataBaseField_PreOpenInterest_get, _thostmduserapi.CThostFtdcMarketDataBaseField_PreOpenInterest_set)
    PreDelta = property(_thostmduserapi.CThostFtdcMarketDataBaseField_PreDelta_get, _thostmduserapi.CThostFtdcMarketDataBaseField_PreDelta_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcMarketDataBaseField_swiginit(self, _thostmduserapi.new_CThostFtdcMarketDataBaseField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcMarketDataBaseField

# Register CThostFtdcMarketDataBaseField in _thostmduserapi:
_thostmduserapi.CThostFtdcMarketDataBaseField_swigregister(CThostFtdcMarketDataBaseField)

class CThostFtdcMarketDataStaticField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    OpenPrice = property(_thostmduserapi.CThostFtdcMarketDataStaticField_OpenPrice_get, _thostmduserapi.CThostFtdcMarketDataStaticField_OpenPrice_set)
    HighestPrice = property(_thostmduserapi.CThostFtdcMarketDataStaticField_HighestPrice_get, _thostmduserapi.CThostFtdcMarketDataStaticField_HighestPrice_set)
    LowestPrice = property(_thostmduserapi.CThostFtdcMarketDataStaticField_LowestPrice_get, _thostmduserapi.CThostFtdcMarketDataStaticField_LowestPrice_set)
    ClosePrice = property(_thostmduserapi.CThostFtdcMarketDataStaticField_ClosePrice_get, _thostmduserapi.CThostFtdcMarketDataStaticField_ClosePrice_set)
    UpperLimitPrice = property(_thostmduserapi.CThostFtdcMarketDataStaticField_UpperLimitPrice_get, _thostmduserapi.CThostFtdcMarketDataStaticField_UpperLimitPrice_set)
    LowerLimitPrice = property(_thostmduserapi.CThostFtdcMarketDataStaticField_LowerLimitPrice_get, _thostmduserapi.CThostFtdcMarketDataStaticField_LowerLimitPrice_set)
    SettlementPrice = property(_thostmduserapi.CThostFtdcMarketDataStaticField_SettlementPrice_get, _thostmduserapi.CThostFtdcMarketDataStaticField_SettlementPrice_set)
    CurrDelta = property(_thostmduserapi.CThostFtdcMarketDataStaticField_CurrDelta_get, _thostmduserapi.CThostFtdcMarketDataStaticField_CurrDelta_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcMarketDataStaticField_swiginit(self, _thostmduserapi.new_CThostFtdcMarketDataStaticField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcMarketDataStaticField

# Register CThostFtdcMarketDataStaticField in _thostmduserapi:
_thostmduserapi.CThostFtdcMarketDataStaticField_swigregister(CThostFtdcMarketDataStaticField)

class CThostFtdcMarketDataLastMatchField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    LastPrice = property(_thostmduserapi.CThostFtdcMarketDataLastMatchField_LastPrice_get, _thostmduserapi.CThostFtdcMarketDataLastMatchField_LastPrice_set)
    Volume = property(_thostmduserapi.CThostFtdcMarketDataLastMatchField_Volume_get, _thostmduserapi.CThostFtdcMarketDataLastMatchField_Volume_set)
    Turnover = property(_thostmduserapi.CThostFtdcMarketDataLastMatchField_Turnover_get, _thostmduserapi.CThostFtdcMarketDataLastMatchField_Turnover_set)
    OpenInterest = property(_thostmduserapi.CThostFtdcMarketDataLastMatchField_OpenInterest_get, _thostmduserapi.CThostFtdcMarketDataLastMatchField_OpenInterest_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcMarketDataLastMatchField_swiginit(self, _thostmduserapi.new_CThostFtdcMarketDataLastMatchField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcMarketDataLastMatchField

# Register CThostFtdcMarketDataLastMatchField in _thostmduserapi:
_thostmduserapi.CThostFtdcMarketDataLastMatchField_swigregister(CThostFtdcMarketDataLastMatchField)

class CThostFtdcMarketDataBestPriceField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BidPrice1 = property(_thostmduserapi.CThostFtdcMarketDataBestPriceField_BidPrice1_get, _thostmduserapi.CThostFtdcMarketDataBestPriceField_BidPrice1_set)
    BidVolume1 = property(_thostmduserapi.CThostFtdcMarketDataBestPriceField_BidVolume1_get, _thostmduserapi.CThostFtdcMarketDataBestPriceField_BidVolume1_set)
    AskPrice1 = property(_thostmduserapi.CThostFtdcMarketDataBestPriceField_AskPrice1_get, _thostmduserapi.CThostFtdcMarketDataBestPriceField_AskPrice1_set)
    AskVolume1 = property(_thostmduserapi.CThostFtdcMarketDataBestPriceField_AskVolume1_get, _thostmduserapi.CThostFtdcMarketDataBestPriceField_AskVolume1_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcMarketDataBestPriceField_swiginit(self, _thostmduserapi.new_CThostFtdcMarketDataBestPriceField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcMarketDataBestPriceField

# Register CThostFtdcMarketDataBestPriceField in _thostmduserapi:
_thostmduserapi.CThostFtdcMarketDataBestPriceField_swigregister(CThostFtdcMarketDataBestPriceField)

class CThostFtdcMarketDataBid23Field(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BidPrice2 = property(_thostmduserapi.CThostFtdcMarketDataBid23Field_BidPrice2_get, _thostmduserapi.CThostFtdcMarketDataBid23Field_BidPrice2_set)
    BidVolume2 = property(_thostmduserapi.CThostFtdcMarketDataBid23Field_BidVolume2_get, _thostmduserapi.CThostFtdcMarketDataBid23Field_BidVolume2_set)
    BidPrice3 = property(_thostmduserapi.CThostFtdcMarketDataBid23Field_BidPrice3_get, _thostmduserapi.CThostFtdcMarketDataBid23Field_BidPrice3_set)
    BidVolume3 = property(_thostmduserapi.CThostFtdcMarketDataBid23Field_BidVolume3_get, _thostmduserapi.CThostFtdcMarketDataBid23Field_BidVolume3_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcMarketDataBid23Field_swiginit(self, _thostmduserapi.new_CThostFtdcMarketDataBid23Field())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcMarketDataBid23Field

# Register CThostFtdcMarketDataBid23Field in _thostmduserapi:
_thostmduserapi.CThostFtdcMarketDataBid23Field_swigregister(CThostFtdcMarketDataBid23Field)

class CThostFtdcMarketDataAsk23Field(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    AskPrice2 = property(_thostmduserapi.CThostFtdcMarketDataAsk23Field_AskPrice2_get, _thostmduserapi.CThostFtdcMarketDataAsk23Field_AskPrice2_set)
    AskVolume2 = property(_thostmduserapi.CThostFtdcMarketDataAsk23Field_AskVolume2_get, _thostmduserapi.CThostFtdcMarketDataAsk23Field_AskVolume2_set)
    AskPrice3 = property(_thostmduserapi.CThostFtdcMarketDataAsk23Field_AskPrice3_get, _thostmduserapi.CThostFtdcMarketDataAsk23Field_AskPrice3_set)
    AskVolume3 = property(_thostmduserapi.CThostFtdcMarketDataAsk23Field_AskVolume3_get, _thostmduserapi.CThostFtdcMarketDataAsk23Field_AskVolume3_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcMarketDataAsk23Field_swiginit(self, _thostmduserapi.new_CThostFtdcMarketDataAsk23Field())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcMarketDataAsk23Field

# Register CThostFtdcMarketDataAsk23Field in _thostmduserapi:
_thostmduserapi.CThostFtdcMarketDataAsk23Field_swigregister(CThostFtdcMarketDataAsk23Field)

class CThostFtdcMarketDataBid45Field(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BidPrice4 = property(_thostmduserapi.CThostFtdcMarketDataBid45Field_BidPrice4_get, _thostmduserapi.CThostFtdcMarketDataBid45Field_BidPrice4_set)
    BidVolume4 = property(_thostmduserapi.CThostFtdcMarketDataBid45Field_BidVolume4_get, _thostmduserapi.CThostFtdcMarketDataBid45Field_BidVolume4_set)
    BidPrice5 = property(_thostmduserapi.CThostFtdcMarketDataBid45Field_BidPrice5_get, _thostmduserapi.CThostFtdcMarketDataBid45Field_BidPrice5_set)
    BidVolume5 = property(_thostmduserapi.CThostFtdcMarketDataBid45Field_BidVolume5_get, _thostmduserapi.CThostFtdcMarketDataBid45Field_BidVolume5_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcMarketDataBid45Field_swiginit(self, _thostmduserapi.new_CThostFtdcMarketDataBid45Field())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcMarketDataBid45Field

# Register CThostFtdcMarketDataBid45Field in _thostmduserapi:
_thostmduserapi.CThostFtdcMarketDataBid45Field_swigregister(CThostFtdcMarketDataBid45Field)

class CThostFtdcMarketDataAsk45Field(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    AskPrice4 = property(_thostmduserapi.CThostFtdcMarketDataAsk45Field_AskPrice4_get, _thostmduserapi.CThostFtdcMarketDataAsk45Field_AskPrice4_set)
    AskVolume4 = property(_thostmduserapi.CThostFtdcMarketDataAsk45Field_AskVolume4_get, _thostmduserapi.CThostFtdcMarketDataAsk45Field_AskVolume4_set)
    AskPrice5 = property(_thostmduserapi.CThostFtdcMarketDataAsk45Field_AskPrice5_get, _thostmduserapi.CThostFtdcMarketDataAsk45Field_AskPrice5_set)
    AskVolume5 = property(_thostmduserapi.CThostFtdcMarketDataAsk45Field_AskVolume5_get, _thostmduserapi.CThostFtdcMarketDataAsk45Field_AskVolume5_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcMarketDataAsk45Field_swiginit(self, _thostmduserapi.new_CThostFtdcMarketDataAsk45Field())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcMarketDataAsk45Field

# Register CThostFtdcMarketDataAsk45Field in _thostmduserapi:
_thostmduserapi.CThostFtdcMarketDataAsk45Field_swigregister(CThostFtdcMarketDataAsk45Field)

class CThostFtdcMarketDataUpdateTimeField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    InstrumentID = property(_thostmduserapi.CThostFtdcMarketDataUpdateTimeField_InstrumentID_get, _thostmduserapi.CThostFtdcMarketDataUpdateTimeField_InstrumentID_set)
    UpdateTime = property(_thostmduserapi.CThostFtdcMarketDataUpdateTimeField_UpdateTime_get, _thostmduserapi.CThostFtdcMarketDataUpdateTimeField_UpdateTime_set)
    UpdateMillisec = property(_thostmduserapi.CThostFtdcMarketDataUpdateTimeField_UpdateMillisec_get, _thostmduserapi.CThostFtdcMarketDataUpdateTimeField_UpdateMillisec_set)
    ActionDay = property(_thostmduserapi.CThostFtdcMarketDataUpdateTimeField_ActionDay_get, _thostmduserapi.CThostFtdcMarketDataUpdateTimeField_ActionDay_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcMarketDataUpdateTimeField_swiginit(self, _thostmduserapi.new_CThostFtdcMarketDataUpdateTimeField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcMarketDataUpdateTimeField

# Register CThostFtdcMarketDataUpdateTimeField in _thostmduserapi:
_thostmduserapi.CThostFtdcMarketDataUpdateTimeField_swigregister(CThostFtdcMarketDataUpdateTimeField)

class CThostFtdcMarketDataExchangeField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ExchangeID = property(_thostmduserapi.CThostFtdcMarketDataExchangeField_ExchangeID_get, _thostmduserapi.CThostFtdcMarketDataExchangeField_ExchangeID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcMarketDataExchangeField_swiginit(self, _thostmduserapi.new_CThostFtdcMarketDataExchangeField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcMarketDataExchangeField

# Register CThostFtdcMarketDataExchangeField in _thostmduserapi:
_thostmduserapi.CThostFtdcMarketDataExchangeField_swigregister(CThostFtdcMarketDataExchangeField)

class CThostFtdcSpecificInstrumentField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    InstrumentID = property(_thostmduserapi.CThostFtdcSpecificInstrumentField_InstrumentID_get, _thostmduserapi.CThostFtdcSpecificInstrumentField_InstrumentID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcSpecificInstrumentField_swiginit(self, _thostmduserapi.new_CThostFtdcSpecificInstrumentField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcSpecificInstrumentField

# Register CThostFtdcSpecificInstrumentField in _thostmduserapi:
_thostmduserapi.CThostFtdcSpecificInstrumentField_swigregister(CThostFtdcSpecificInstrumentField)

class CThostFtdcInstrumentStatusField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ExchangeID = property(_thostmduserapi.CThostFtdcInstrumentStatusField_ExchangeID_get, _thostmduserapi.CThostFtdcInstrumentStatusField_ExchangeID_set)
    ExchangeInstID = property(_thostmduserapi.CThostFtdcInstrumentStatusField_ExchangeInstID_get, _thostmduserapi.CThostFtdcInstrumentStatusField_ExchangeInstID_set)
    SettlementGroupID = property(_thostmduserapi.CThostFtdcInstrumentStatusField_SettlementGroupID_get, _thostmduserapi.CThostFtdcInstrumentStatusField_SettlementGroupID_set)
    InstrumentID = property(_thostmduserapi.CThostFtdcInstrumentStatusField_InstrumentID_get, _thostmduserapi.CThostFtdcInstrumentStatusField_InstrumentID_set)
    InstrumentStatus = property(_thostmduserapi.CThostFtdcInstrumentStatusField_InstrumentStatus_get, _thostmduserapi.CThostFtdcInstrumentStatusField_InstrumentStatus_set)
    TradingSegmentSN = property(_thostmduserapi.CThostFtdcInstrumentStatusField_TradingSegmentSN_get, _thostmduserapi.CThostFtdcInstrumentStatusField_TradingSegmentSN_set)
    EnterTime = property(_thostmduserapi.CThostFtdcInstrumentStatusField_EnterTime_get, _thostmduserapi.CThostFtdcInstrumentStatusField_EnterTime_set)
    EnterReason = property(_thostmduserapi.CThostFtdcInstrumentStatusField_EnterReason_get, _thostmduserapi.CThostFtdcInstrumentStatusField_EnterReason_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcInstrumentStatusField_swiginit(self, _thostmduserapi.new_CThostFtdcInstrumentStatusField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcInstrumentStatusField

# Register CThostFtdcInstrumentStatusField in _thostmduserapi:
_thostmduserapi.CThostFtdcInstrumentStatusField_swigregister(CThostFtdcInstrumentStatusField)

class CThostFtdcQryInstrumentStatusField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ExchangeID = property(_thostmduserapi.CThostFtdcQryInstrumentStatusField_ExchangeID_get, _thostmduserapi.CThostFtdcQryInstrumentStatusField_ExchangeID_set)
    ExchangeInstID = property(_thostmduserapi.CThostFtdcQryInstrumentStatusField_ExchangeInstID_get, _thostmduserapi.CThostFtdcQryInstrumentStatusField_ExchangeInstID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryInstrumentStatusField_swiginit(self, _thostmduserapi.new_CThostFtdcQryInstrumentStatusField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryInstrumentStatusField

# Register CThostFtdcQryInstrumentStatusField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryInstrumentStatusField_swigregister(CThostFtdcQryInstrumentStatusField)

class CThostFtdcInvestorAccountField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcInvestorAccountField_BrokerID_get, _thostmduserapi.CThostFtdcInvestorAccountField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcInvestorAccountField_InvestorID_get, _thostmduserapi.CThostFtdcInvestorAccountField_InvestorID_set)
    AccountID = property(_thostmduserapi.CThostFtdcInvestorAccountField_AccountID_get, _thostmduserapi.CThostFtdcInvestorAccountField_AccountID_set)
    CurrencyID = property(_thostmduserapi.CThostFtdcInvestorAccountField_CurrencyID_get, _thostmduserapi.CThostFtdcInvestorAccountField_CurrencyID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcInvestorAccountField_swiginit(self, _thostmduserapi.new_CThostFtdcInvestorAccountField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcInvestorAccountField

# Register CThostFtdcInvestorAccountField in _thostmduserapi:
_thostmduserapi.CThostFtdcInvestorAccountField_swigregister(CThostFtdcInvestorAccountField)

class CThostFtdcPositionProfitAlgorithmField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcPositionProfitAlgorithmField_BrokerID_get, _thostmduserapi.CThostFtdcPositionProfitAlgorithmField_BrokerID_set)
    AccountID = property(_thostmduserapi.CThostFtdcPositionProfitAlgorithmField_AccountID_get, _thostmduserapi.CThostFtdcPositionProfitAlgorithmField_AccountID_set)
    Algorithm = property(_thostmduserapi.CThostFtdcPositionProfitAlgorithmField_Algorithm_get, _thostmduserapi.CThostFtdcPositionProfitAlgorithmField_Algorithm_set)
    Memo = property(_thostmduserapi.CThostFtdcPositionProfitAlgorithmField_Memo_get, _thostmduserapi.CThostFtdcPositionProfitAlgorithmField_Memo_set)
    CurrencyID = property(_thostmduserapi.CThostFtdcPositionProfitAlgorithmField_CurrencyID_get, _thostmduserapi.CThostFtdcPositionProfitAlgorithmField_CurrencyID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcPositionProfitAlgorithmField_swiginit(self, _thostmduserapi.new_CThostFtdcPositionProfitAlgorithmField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcPositionProfitAlgorithmField

# Register CThostFtdcPositionProfitAlgorithmField in _thostmduserapi:
_thostmduserapi.CThostFtdcPositionProfitAlgorithmField_swigregister(CThostFtdcPositionProfitAlgorithmField)

class CThostFtdcDiscountField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcDiscountField_BrokerID_get, _thostmduserapi.CThostFtdcDiscountField_BrokerID_set)
    InvestorRange = property(_thostmduserapi.CThostFtdcDiscountField_InvestorRange_get, _thostmduserapi.CThostFtdcDiscountField_InvestorRange_set)
    InvestorID = property(_thostmduserapi.CThostFtdcDiscountField_InvestorID_get, _thostmduserapi.CThostFtdcDiscountField_InvestorID_set)
    Discount = property(_thostmduserapi.CThostFtdcDiscountField_Discount_get, _thostmduserapi.CThostFtdcDiscountField_Discount_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcDiscountField_swiginit(self, _thostmduserapi.new_CThostFtdcDiscountField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcDiscountField

# Register CThostFtdcDiscountField in _thostmduserapi:
_thostmduserapi.CThostFtdcDiscountField_swigregister(CThostFtdcDiscountField)

class CThostFtdcQryTransferBankField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BankID = property(_thostmduserapi.CThostFtdcQryTransferBankField_BankID_get, _thostmduserapi.CThostFtdcQryTransferBankField_BankID_set)
    BankBrchID = property(_thostmduserapi.CThostFtdcQryTransferBankField_BankBrchID_get, _thostmduserapi.CThostFtdcQryTransferBankField_BankBrchID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryTransferBankField_swiginit(self, _thostmduserapi.new_CThostFtdcQryTransferBankField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryTransferBankField

# Register CThostFtdcQryTransferBankField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryTransferBankField_swigregister(CThostFtdcQryTransferBankField)

class CThostFtdcTransferBankField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BankID = property(_thostmduserapi.CThostFtdcTransferBankField_BankID_get, _thostmduserapi.CThostFtdcTransferBankField_BankID_set)
    BankBrchID = property(_thostmduserapi.CThostFtdcTransferBankField_BankBrchID_get, _thostmduserapi.CThostFtdcTransferBankField_BankBrchID_set)
    BankName = property(_thostmduserapi.CThostFtdcTransferBankField_BankName_get, _thostmduserapi.CThostFtdcTransferBankField_BankName_set)
    IsActive = property(_thostmduserapi.CThostFtdcTransferBankField_IsActive_get, _thostmduserapi.CThostFtdcTransferBankField_IsActive_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcTransferBankField_swiginit(self, _thostmduserapi.new_CThostFtdcTransferBankField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcTransferBankField

# Register CThostFtdcTransferBankField in _thostmduserapi:
_thostmduserapi.CThostFtdcTransferBankField_swigregister(CThostFtdcTransferBankField)

class CThostFtdcQryInvestorPositionDetailField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQryInvestorPositionDetailField_BrokerID_get, _thostmduserapi.CThostFtdcQryInvestorPositionDetailField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcQryInvestorPositionDetailField_InvestorID_get, _thostmduserapi.CThostFtdcQryInvestorPositionDetailField_InvestorID_set)
    InstrumentID = property(_thostmduserapi.CThostFtdcQryInvestorPositionDetailField_InstrumentID_get, _thostmduserapi.CThostFtdcQryInvestorPositionDetailField_InstrumentID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcQryInvestorPositionDetailField_ExchangeID_get, _thostmduserapi.CThostFtdcQryInvestorPositionDetailField_ExchangeID_set)
    InvestUnitID = property(_thostmduserapi.CThostFtdcQryInvestorPositionDetailField_InvestUnitID_get, _thostmduserapi.CThostFtdcQryInvestorPositionDetailField_InvestUnitID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryInvestorPositionDetailField_swiginit(self, _thostmduserapi.new_CThostFtdcQryInvestorPositionDetailField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryInvestorPositionDetailField

# Register CThostFtdcQryInvestorPositionDetailField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryInvestorPositionDetailField_swigregister(CThostFtdcQryInvestorPositionDetailField)

class CThostFtdcInvestorPositionDetailField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    InstrumentID = property(_thostmduserapi.CThostFtdcInvestorPositionDetailField_InstrumentID_get, _thostmduserapi.CThostFtdcInvestorPositionDetailField_InstrumentID_set)
    BrokerID = property(_thostmduserapi.CThostFtdcInvestorPositionDetailField_BrokerID_get, _thostmduserapi.CThostFtdcInvestorPositionDetailField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcInvestorPositionDetailField_InvestorID_get, _thostmduserapi.CThostFtdcInvestorPositionDetailField_InvestorID_set)
    HedgeFlag = property(_thostmduserapi.CThostFtdcInvestorPositionDetailField_HedgeFlag_get, _thostmduserapi.CThostFtdcInvestorPositionDetailField_HedgeFlag_set)
    Direction = property(_thostmduserapi.CThostFtdcInvestorPositionDetailField_Direction_get, _thostmduserapi.CThostFtdcInvestorPositionDetailField_Direction_set)
    OpenDate = property(_thostmduserapi.CThostFtdcInvestorPositionDetailField_OpenDate_get, _thostmduserapi.CThostFtdcInvestorPositionDetailField_OpenDate_set)
    TradeID = property(_thostmduserapi.CThostFtdcInvestorPositionDetailField_TradeID_get, _thostmduserapi.CThostFtdcInvestorPositionDetailField_TradeID_set)
    Volume = property(_thostmduserapi.CThostFtdcInvestorPositionDetailField_Volume_get, _thostmduserapi.CThostFtdcInvestorPositionDetailField_Volume_set)
    OpenPrice = property(_thostmduserapi.CThostFtdcInvestorPositionDetailField_OpenPrice_get, _thostmduserapi.CThostFtdcInvestorPositionDetailField_OpenPrice_set)
    TradingDay = property(_thostmduserapi.CThostFtdcInvestorPositionDetailField_TradingDay_get, _thostmduserapi.CThostFtdcInvestorPositionDetailField_TradingDay_set)
    SettlementID = property(_thostmduserapi.CThostFtdcInvestorPositionDetailField_SettlementID_get, _thostmduserapi.CThostFtdcInvestorPositionDetailField_SettlementID_set)
    TradeType = property(_thostmduserapi.CThostFtdcInvestorPositionDetailField_TradeType_get, _thostmduserapi.CThostFtdcInvestorPositionDetailField_TradeType_set)
    CombInstrumentID = property(_thostmduserapi.CThostFtdcInvestorPositionDetailField_CombInstrumentID_get, _thostmduserapi.CThostFtdcInvestorPositionDetailField_CombInstrumentID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcInvestorPositionDetailField_ExchangeID_get, _thostmduserapi.CThostFtdcInvestorPositionDetailField_ExchangeID_set)
    CloseProfitByDate = property(_thostmduserapi.CThostFtdcInvestorPositionDetailField_CloseProfitByDate_get, _thostmduserapi.CThostFtdcInvestorPositionDetailField_CloseProfitByDate_set)
    CloseProfitByTrade = property(_thostmduserapi.CThostFtdcInvestorPositionDetailField_CloseProfitByTrade_get, _thostmduserapi.CThostFtdcInvestorPositionDetailField_CloseProfitByTrade_set)
    PositionProfitByDate = property(_thostmduserapi.CThostFtdcInvestorPositionDetailField_PositionProfitByDate_get, _thostmduserapi.CThostFtdcInvestorPositionDetailField_PositionProfitByDate_set)
    PositionProfitByTrade = property(_thostmduserapi.CThostFtdcInvestorPositionDetailField_PositionProfitByTrade_get, _thostmduserapi.CThostFtdcInvestorPositionDetailField_PositionProfitByTrade_set)
    Margin = property(_thostmduserapi.CThostFtdcInvestorPositionDetailField_Margin_get, _thostmduserapi.CThostFtdcInvestorPositionDetailField_Margin_set)
    ExchMargin = property(_thostmduserapi.CThostFtdcInvestorPositionDetailField_ExchMargin_get, _thostmduserapi.CThostFtdcInvestorPositionDetailField_ExchMargin_set)
    MarginRateByMoney = property(_thostmduserapi.CThostFtdcInvestorPositionDetailField_MarginRateByMoney_get, _thostmduserapi.CThostFtdcInvestorPositionDetailField_MarginRateByMoney_set)
    MarginRateByVolume = property(_thostmduserapi.CThostFtdcInvestorPositionDetailField_MarginRateByVolume_get, _thostmduserapi.CThostFtdcInvestorPositionDetailField_MarginRateByVolume_set)
    LastSettlementPrice = property(_thostmduserapi.CThostFtdcInvestorPositionDetailField_LastSettlementPrice_get, _thostmduserapi.CThostFtdcInvestorPositionDetailField_LastSettlementPrice_set)
    SettlementPrice = property(_thostmduserapi.CThostFtdcInvestorPositionDetailField_SettlementPrice_get, _thostmduserapi.CThostFtdcInvestorPositionDetailField_SettlementPrice_set)
    CloseVolume = property(_thostmduserapi.CThostFtdcInvestorPositionDetailField_CloseVolume_get, _thostmduserapi.CThostFtdcInvestorPositionDetailField_CloseVolume_set)
    CloseAmount = property(_thostmduserapi.CThostFtdcInvestorPositionDetailField_CloseAmount_get, _thostmduserapi.CThostFtdcInvestorPositionDetailField_CloseAmount_set)
    TimeFirstVolume = property(_thostmduserapi.CThostFtdcInvestorPositionDetailField_TimeFirstVolume_get, _thostmduserapi.CThostFtdcInvestorPositionDetailField_TimeFirstVolume_set)
    InvestUnitID = property(_thostmduserapi.CThostFtdcInvestorPositionDetailField_InvestUnitID_get, _thostmduserapi.CThostFtdcInvestorPositionDetailField_InvestUnitID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcInvestorPositionDetailField_swiginit(self, _thostmduserapi.new_CThostFtdcInvestorPositionDetailField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcInvestorPositionDetailField

# Register CThostFtdcInvestorPositionDetailField in _thostmduserapi:
_thostmduserapi.CThostFtdcInvestorPositionDetailField_swigregister(CThostFtdcInvestorPositionDetailField)

class CThostFtdcTradingAccountPasswordField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcTradingAccountPasswordField_BrokerID_get, _thostmduserapi.CThostFtdcTradingAccountPasswordField_BrokerID_set)
    AccountID = property(_thostmduserapi.CThostFtdcTradingAccountPasswordField_AccountID_get, _thostmduserapi.CThostFtdcTradingAccountPasswordField_AccountID_set)
    Password = property(_thostmduserapi.CThostFtdcTradingAccountPasswordField_Password_get, _thostmduserapi.CThostFtdcTradingAccountPasswordField_Password_set)
    CurrencyID = property(_thostmduserapi.CThostFtdcTradingAccountPasswordField_CurrencyID_get, _thostmduserapi.CThostFtdcTradingAccountPasswordField_CurrencyID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcTradingAccountPasswordField_swiginit(self, _thostmduserapi.new_CThostFtdcTradingAccountPasswordField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcTradingAccountPasswordField

# Register CThostFtdcTradingAccountPasswordField in _thostmduserapi:
_thostmduserapi.CThostFtdcTradingAccountPasswordField_swigregister(CThostFtdcTradingAccountPasswordField)

class CThostFtdcMDTraderOfferField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ExchangeID = property(_thostmduserapi.CThostFtdcMDTraderOfferField_ExchangeID_get, _thostmduserapi.CThostFtdcMDTraderOfferField_ExchangeID_set)
    TraderID = property(_thostmduserapi.CThostFtdcMDTraderOfferField_TraderID_get, _thostmduserapi.CThostFtdcMDTraderOfferField_TraderID_set)
    ParticipantID = property(_thostmduserapi.CThostFtdcMDTraderOfferField_ParticipantID_get, _thostmduserapi.CThostFtdcMDTraderOfferField_ParticipantID_set)
    Password = property(_thostmduserapi.CThostFtdcMDTraderOfferField_Password_get, _thostmduserapi.CThostFtdcMDTraderOfferField_Password_set)
    InstallID = property(_thostmduserapi.CThostFtdcMDTraderOfferField_InstallID_get, _thostmduserapi.CThostFtdcMDTraderOfferField_InstallID_set)
    OrderLocalID = property(_thostmduserapi.CThostFtdcMDTraderOfferField_OrderLocalID_get, _thostmduserapi.CThostFtdcMDTraderOfferField_OrderLocalID_set)
    TraderConnectStatus = property(_thostmduserapi.CThostFtdcMDTraderOfferField_TraderConnectStatus_get, _thostmduserapi.CThostFtdcMDTraderOfferField_TraderConnectStatus_set)
    ConnectRequestDate = property(_thostmduserapi.CThostFtdcMDTraderOfferField_ConnectRequestDate_get, _thostmduserapi.CThostFtdcMDTraderOfferField_ConnectRequestDate_set)
    ConnectRequestTime = property(_thostmduserapi.CThostFtdcMDTraderOfferField_ConnectRequestTime_get, _thostmduserapi.CThostFtdcMDTraderOfferField_ConnectRequestTime_set)
    LastReportDate = property(_thostmduserapi.CThostFtdcMDTraderOfferField_LastReportDate_get, _thostmduserapi.CThostFtdcMDTraderOfferField_LastReportDate_set)
    LastReportTime = property(_thostmduserapi.CThostFtdcMDTraderOfferField_LastReportTime_get, _thostmduserapi.CThostFtdcMDTraderOfferField_LastReportTime_set)
    ConnectDate = property(_thostmduserapi.CThostFtdcMDTraderOfferField_ConnectDate_get, _thostmduserapi.CThostFtdcMDTraderOfferField_ConnectDate_set)
    ConnectTime = property(_thostmduserapi.CThostFtdcMDTraderOfferField_ConnectTime_get, _thostmduserapi.CThostFtdcMDTraderOfferField_ConnectTime_set)
    StartDate = property(_thostmduserapi.CThostFtdcMDTraderOfferField_StartDate_get, _thostmduserapi.CThostFtdcMDTraderOfferField_StartDate_set)
    StartTime = property(_thostmduserapi.CThostFtdcMDTraderOfferField_StartTime_get, _thostmduserapi.CThostFtdcMDTraderOfferField_StartTime_set)
    TradingDay = property(_thostmduserapi.CThostFtdcMDTraderOfferField_TradingDay_get, _thostmduserapi.CThostFtdcMDTraderOfferField_TradingDay_set)
    BrokerID = property(_thostmduserapi.CThostFtdcMDTraderOfferField_BrokerID_get, _thostmduserapi.CThostFtdcMDTraderOfferField_BrokerID_set)
    MaxTradeID = property(_thostmduserapi.CThostFtdcMDTraderOfferField_MaxTradeID_get, _thostmduserapi.CThostFtdcMDTraderOfferField_MaxTradeID_set)
    MaxOrderMessageReference = property(_thostmduserapi.CThostFtdcMDTraderOfferField_MaxOrderMessageReference_get, _thostmduserapi.CThostFtdcMDTraderOfferField_MaxOrderMessageReference_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcMDTraderOfferField_swiginit(self, _thostmduserapi.new_CThostFtdcMDTraderOfferField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcMDTraderOfferField

# Register CThostFtdcMDTraderOfferField in _thostmduserapi:
_thostmduserapi.CThostFtdcMDTraderOfferField_swigregister(CThostFtdcMDTraderOfferField)

class CThostFtdcQryMDTraderOfferField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ExchangeID = property(_thostmduserapi.CThostFtdcQryMDTraderOfferField_ExchangeID_get, _thostmduserapi.CThostFtdcQryMDTraderOfferField_ExchangeID_set)
    ParticipantID = property(_thostmduserapi.CThostFtdcQryMDTraderOfferField_ParticipantID_get, _thostmduserapi.CThostFtdcQryMDTraderOfferField_ParticipantID_set)
    TraderID = property(_thostmduserapi.CThostFtdcQryMDTraderOfferField_TraderID_get, _thostmduserapi.CThostFtdcQryMDTraderOfferField_TraderID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryMDTraderOfferField_swiginit(self, _thostmduserapi.new_CThostFtdcQryMDTraderOfferField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryMDTraderOfferField

# Register CThostFtdcQryMDTraderOfferField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryMDTraderOfferField_swigregister(CThostFtdcQryMDTraderOfferField)

class CThostFtdcQryNoticeField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQryNoticeField_BrokerID_get, _thostmduserapi.CThostFtdcQryNoticeField_BrokerID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryNoticeField_swiginit(self, _thostmduserapi.new_CThostFtdcQryNoticeField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryNoticeField

# Register CThostFtdcQryNoticeField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryNoticeField_swigregister(CThostFtdcQryNoticeField)

class CThostFtdcNoticeField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcNoticeField_BrokerID_get, _thostmduserapi.CThostFtdcNoticeField_BrokerID_set)
    Content = property(_thostmduserapi.CThostFtdcNoticeField_Content_get, _thostmduserapi.CThostFtdcNoticeField_Content_set)
    SequenceLabel = property(_thostmduserapi.CThostFtdcNoticeField_SequenceLabel_get, _thostmduserapi.CThostFtdcNoticeField_SequenceLabel_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcNoticeField_swiginit(self, _thostmduserapi.new_CThostFtdcNoticeField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcNoticeField

# Register CThostFtdcNoticeField in _thostmduserapi:
_thostmduserapi.CThostFtdcNoticeField_swigregister(CThostFtdcNoticeField)

class CThostFtdcUserRightField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcUserRightField_BrokerID_get, _thostmduserapi.CThostFtdcUserRightField_BrokerID_set)
    UserID = property(_thostmduserapi.CThostFtdcUserRightField_UserID_get, _thostmduserapi.CThostFtdcUserRightField_UserID_set)
    UserRightType = property(_thostmduserapi.CThostFtdcUserRightField_UserRightType_get, _thostmduserapi.CThostFtdcUserRightField_UserRightType_set)
    IsForbidden = property(_thostmduserapi.CThostFtdcUserRightField_IsForbidden_get, _thostmduserapi.CThostFtdcUserRightField_IsForbidden_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcUserRightField_swiginit(self, _thostmduserapi.new_CThostFtdcUserRightField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcUserRightField

# Register CThostFtdcUserRightField in _thostmduserapi:
_thostmduserapi.CThostFtdcUserRightField_swigregister(CThostFtdcUserRightField)

class CThostFtdcQrySettlementInfoConfirmField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQrySettlementInfoConfirmField_BrokerID_get, _thostmduserapi.CThostFtdcQrySettlementInfoConfirmField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcQrySettlementInfoConfirmField_InvestorID_get, _thostmduserapi.CThostFtdcQrySettlementInfoConfirmField_InvestorID_set)
    AccountID = property(_thostmduserapi.CThostFtdcQrySettlementInfoConfirmField_AccountID_get, _thostmduserapi.CThostFtdcQrySettlementInfoConfirmField_AccountID_set)
    CurrencyID = property(_thostmduserapi.CThostFtdcQrySettlementInfoConfirmField_CurrencyID_get, _thostmduserapi.CThostFtdcQrySettlementInfoConfirmField_CurrencyID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQrySettlementInfoConfirmField_swiginit(self, _thostmduserapi.new_CThostFtdcQrySettlementInfoConfirmField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQrySettlementInfoConfirmField

# Register CThostFtdcQrySettlementInfoConfirmField in _thostmduserapi:
_thostmduserapi.CThostFtdcQrySettlementInfoConfirmField_swigregister(CThostFtdcQrySettlementInfoConfirmField)

class CThostFtdcLoadSettlementInfoField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcLoadSettlementInfoField_BrokerID_get, _thostmduserapi.CThostFtdcLoadSettlementInfoField_BrokerID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcLoadSettlementInfoField_swiginit(self, _thostmduserapi.new_CThostFtdcLoadSettlementInfoField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcLoadSettlementInfoField

# Register CThostFtdcLoadSettlementInfoField in _thostmduserapi:
_thostmduserapi.CThostFtdcLoadSettlementInfoField_swigregister(CThostFtdcLoadSettlementInfoField)

class CThostFtdcBrokerWithdrawAlgorithmField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcBrokerWithdrawAlgorithmField_BrokerID_get, _thostmduserapi.CThostFtdcBrokerWithdrawAlgorithmField_BrokerID_set)
    WithdrawAlgorithm = property(_thostmduserapi.CThostFtdcBrokerWithdrawAlgorithmField_WithdrawAlgorithm_get, _thostmduserapi.CThostFtdcBrokerWithdrawAlgorithmField_WithdrawAlgorithm_set)
    UsingRatio = property(_thostmduserapi.CThostFtdcBrokerWithdrawAlgorithmField_UsingRatio_get, _thostmduserapi.CThostFtdcBrokerWithdrawAlgorithmField_UsingRatio_set)
    IncludeCloseProfit = property(_thostmduserapi.CThostFtdcBrokerWithdrawAlgorithmField_IncludeCloseProfit_get, _thostmduserapi.CThostFtdcBrokerWithdrawAlgorithmField_IncludeCloseProfit_set)
    AllWithoutTrade = property(_thostmduserapi.CThostFtdcBrokerWithdrawAlgorithmField_AllWithoutTrade_get, _thostmduserapi.CThostFtdcBrokerWithdrawAlgorithmField_AllWithoutTrade_set)
    AvailIncludeCloseProfit = property(_thostmduserapi.CThostFtdcBrokerWithdrawAlgorithmField_AvailIncludeCloseProfit_get, _thostmduserapi.CThostFtdcBrokerWithdrawAlgorithmField_AvailIncludeCloseProfit_set)
    IsBrokerUserEvent = property(_thostmduserapi.CThostFtdcBrokerWithdrawAlgorithmField_IsBrokerUserEvent_get, _thostmduserapi.CThostFtdcBrokerWithdrawAlgorithmField_IsBrokerUserEvent_set)
    CurrencyID = property(_thostmduserapi.CThostFtdcBrokerWithdrawAlgorithmField_CurrencyID_get, _thostmduserapi.CThostFtdcBrokerWithdrawAlgorithmField_CurrencyID_set)
    FundMortgageRatio = property(_thostmduserapi.CThostFtdcBrokerWithdrawAlgorithmField_FundMortgageRatio_get, _thostmduserapi.CThostFtdcBrokerWithdrawAlgorithmField_FundMortgageRatio_set)
    BalanceAlgorithm = property(_thostmduserapi.CThostFtdcBrokerWithdrawAlgorithmField_BalanceAlgorithm_get, _thostmduserapi.CThostFtdcBrokerWithdrawAlgorithmField_BalanceAlgorithm_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcBrokerWithdrawAlgorithmField_swiginit(self, _thostmduserapi.new_CThostFtdcBrokerWithdrawAlgorithmField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcBrokerWithdrawAlgorithmField

# Register CThostFtdcBrokerWithdrawAlgorithmField in _thostmduserapi:
_thostmduserapi.CThostFtdcBrokerWithdrawAlgorithmField_swigregister(CThostFtdcBrokerWithdrawAlgorithmField)

class CThostFtdcTradingAccountPasswordUpdateV1Field(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcTradingAccountPasswordUpdateV1Field_BrokerID_get, _thostmduserapi.CThostFtdcTradingAccountPasswordUpdateV1Field_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcTradingAccountPasswordUpdateV1Field_InvestorID_get, _thostmduserapi.CThostFtdcTradingAccountPasswordUpdateV1Field_InvestorID_set)
    OldPassword = property(_thostmduserapi.CThostFtdcTradingAccountPasswordUpdateV1Field_OldPassword_get, _thostmduserapi.CThostFtdcTradingAccountPasswordUpdateV1Field_OldPassword_set)
    NewPassword = property(_thostmduserapi.CThostFtdcTradingAccountPasswordUpdateV1Field_NewPassword_get, _thostmduserapi.CThostFtdcTradingAccountPasswordUpdateV1Field_NewPassword_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcTradingAccountPasswordUpdateV1Field_swiginit(self, _thostmduserapi.new_CThostFtdcTradingAccountPasswordUpdateV1Field())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcTradingAccountPasswordUpdateV1Field

# Register CThostFtdcTradingAccountPasswordUpdateV1Field in _thostmduserapi:
_thostmduserapi.CThostFtdcTradingAccountPasswordUpdateV1Field_swigregister(CThostFtdcTradingAccountPasswordUpdateV1Field)

class CThostFtdcTradingAccountPasswordUpdateField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcTradingAccountPasswordUpdateField_BrokerID_get, _thostmduserapi.CThostFtdcTradingAccountPasswordUpdateField_BrokerID_set)
    AccountID = property(_thostmduserapi.CThostFtdcTradingAccountPasswordUpdateField_AccountID_get, _thostmduserapi.CThostFtdcTradingAccountPasswordUpdateField_AccountID_set)
    OldPassword = property(_thostmduserapi.CThostFtdcTradingAccountPasswordUpdateField_OldPassword_get, _thostmduserapi.CThostFtdcTradingAccountPasswordUpdateField_OldPassword_set)
    NewPassword = property(_thostmduserapi.CThostFtdcTradingAccountPasswordUpdateField_NewPassword_get, _thostmduserapi.CThostFtdcTradingAccountPasswordUpdateField_NewPassword_set)
    CurrencyID = property(_thostmduserapi.CThostFtdcTradingAccountPasswordUpdateField_CurrencyID_get, _thostmduserapi.CThostFtdcTradingAccountPasswordUpdateField_CurrencyID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcTradingAccountPasswordUpdateField_swiginit(self, _thostmduserapi.new_CThostFtdcTradingAccountPasswordUpdateField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcTradingAccountPasswordUpdateField

# Register CThostFtdcTradingAccountPasswordUpdateField in _thostmduserapi:
_thostmduserapi.CThostFtdcTradingAccountPasswordUpdateField_swigregister(CThostFtdcTradingAccountPasswordUpdateField)

class CThostFtdcQryCombinationLegField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    CombInstrumentID = property(_thostmduserapi.CThostFtdcQryCombinationLegField_CombInstrumentID_get, _thostmduserapi.CThostFtdcQryCombinationLegField_CombInstrumentID_set)
    LegID = property(_thostmduserapi.CThostFtdcQryCombinationLegField_LegID_get, _thostmduserapi.CThostFtdcQryCombinationLegField_LegID_set)
    LegInstrumentID = property(_thostmduserapi.CThostFtdcQryCombinationLegField_LegInstrumentID_get, _thostmduserapi.CThostFtdcQryCombinationLegField_LegInstrumentID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryCombinationLegField_swiginit(self, _thostmduserapi.new_CThostFtdcQryCombinationLegField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryCombinationLegField

# Register CThostFtdcQryCombinationLegField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryCombinationLegField_swigregister(CThostFtdcQryCombinationLegField)

class CThostFtdcQrySyncStatusField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradingDay = property(_thostmduserapi.CThostFtdcQrySyncStatusField_TradingDay_get, _thostmduserapi.CThostFtdcQrySyncStatusField_TradingDay_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQrySyncStatusField_swiginit(self, _thostmduserapi.new_CThostFtdcQrySyncStatusField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQrySyncStatusField

# Register CThostFtdcQrySyncStatusField in _thostmduserapi:
_thostmduserapi.CThostFtdcQrySyncStatusField_swigregister(CThostFtdcQrySyncStatusField)

class CThostFtdcCombinationLegField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    CombInstrumentID = property(_thostmduserapi.CThostFtdcCombinationLegField_CombInstrumentID_get, _thostmduserapi.CThostFtdcCombinationLegField_CombInstrumentID_set)
    LegID = property(_thostmduserapi.CThostFtdcCombinationLegField_LegID_get, _thostmduserapi.CThostFtdcCombinationLegField_LegID_set)
    LegInstrumentID = property(_thostmduserapi.CThostFtdcCombinationLegField_LegInstrumentID_get, _thostmduserapi.CThostFtdcCombinationLegField_LegInstrumentID_set)
    Direction = property(_thostmduserapi.CThostFtdcCombinationLegField_Direction_get, _thostmduserapi.CThostFtdcCombinationLegField_Direction_set)
    LegMultiple = property(_thostmduserapi.CThostFtdcCombinationLegField_LegMultiple_get, _thostmduserapi.CThostFtdcCombinationLegField_LegMultiple_set)
    ImplyLevel = property(_thostmduserapi.CThostFtdcCombinationLegField_ImplyLevel_get, _thostmduserapi.CThostFtdcCombinationLegField_ImplyLevel_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcCombinationLegField_swiginit(self, _thostmduserapi.new_CThostFtdcCombinationLegField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcCombinationLegField

# Register CThostFtdcCombinationLegField in _thostmduserapi:
_thostmduserapi.CThostFtdcCombinationLegField_swigregister(CThostFtdcCombinationLegField)

class CThostFtdcSyncStatusField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradingDay = property(_thostmduserapi.CThostFtdcSyncStatusField_TradingDay_get, _thostmduserapi.CThostFtdcSyncStatusField_TradingDay_set)
    DataSyncStatus = property(_thostmduserapi.CThostFtdcSyncStatusField_DataSyncStatus_get, _thostmduserapi.CThostFtdcSyncStatusField_DataSyncStatus_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcSyncStatusField_swiginit(self, _thostmduserapi.new_CThostFtdcSyncStatusField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcSyncStatusField

# Register CThostFtdcSyncStatusField in _thostmduserapi:
_thostmduserapi.CThostFtdcSyncStatusField_swigregister(CThostFtdcSyncStatusField)

class CThostFtdcQryLinkManField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQryLinkManField_BrokerID_get, _thostmduserapi.CThostFtdcQryLinkManField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcQryLinkManField_InvestorID_get, _thostmduserapi.CThostFtdcQryLinkManField_InvestorID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryLinkManField_swiginit(self, _thostmduserapi.new_CThostFtdcQryLinkManField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryLinkManField

# Register CThostFtdcQryLinkManField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryLinkManField_swigregister(CThostFtdcQryLinkManField)

class CThostFtdcLinkManField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcLinkManField_BrokerID_get, _thostmduserapi.CThostFtdcLinkManField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcLinkManField_InvestorID_get, _thostmduserapi.CThostFtdcLinkManField_InvestorID_set)
    PersonType = property(_thostmduserapi.CThostFtdcLinkManField_PersonType_get, _thostmduserapi.CThostFtdcLinkManField_PersonType_set)
    IdentifiedCardType = property(_thostmduserapi.CThostFtdcLinkManField_IdentifiedCardType_get, _thostmduserapi.CThostFtdcLinkManField_IdentifiedCardType_set)
    IdentifiedCardNo = property(_thostmduserapi.CThostFtdcLinkManField_IdentifiedCardNo_get, _thostmduserapi.CThostFtdcLinkManField_IdentifiedCardNo_set)
    PersonName = property(_thostmduserapi.CThostFtdcLinkManField_PersonName_get, _thostmduserapi.CThostFtdcLinkManField_PersonName_set)
    Telephone = property(_thostmduserapi.CThostFtdcLinkManField_Telephone_get, _thostmduserapi.CThostFtdcLinkManField_Telephone_set)
    Address = property(_thostmduserapi.CThostFtdcLinkManField_Address_get, _thostmduserapi.CThostFtdcLinkManField_Address_set)
    ZipCode = property(_thostmduserapi.CThostFtdcLinkManField_ZipCode_get, _thostmduserapi.CThostFtdcLinkManField_ZipCode_set)
    Priority = property(_thostmduserapi.CThostFtdcLinkManField_Priority_get, _thostmduserapi.CThostFtdcLinkManField_Priority_set)
    UOAZipCode = property(_thostmduserapi.CThostFtdcLinkManField_UOAZipCode_get, _thostmduserapi.CThostFtdcLinkManField_UOAZipCode_set)
    PersonFullName = property(_thostmduserapi.CThostFtdcLinkManField_PersonFullName_get, _thostmduserapi.CThostFtdcLinkManField_PersonFullName_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcLinkManField_swiginit(self, _thostmduserapi.new_CThostFtdcLinkManField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcLinkManField

# Register CThostFtdcLinkManField in _thostmduserapi:
_thostmduserapi.CThostFtdcLinkManField_swigregister(CThostFtdcLinkManField)

class CThostFtdcQryBrokerUserEventField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQryBrokerUserEventField_BrokerID_get, _thostmduserapi.CThostFtdcQryBrokerUserEventField_BrokerID_set)
    UserID = property(_thostmduserapi.CThostFtdcQryBrokerUserEventField_UserID_get, _thostmduserapi.CThostFtdcQryBrokerUserEventField_UserID_set)
    UserEventType = property(_thostmduserapi.CThostFtdcQryBrokerUserEventField_UserEventType_get, _thostmduserapi.CThostFtdcQryBrokerUserEventField_UserEventType_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryBrokerUserEventField_swiginit(self, _thostmduserapi.new_CThostFtdcQryBrokerUserEventField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryBrokerUserEventField

# Register CThostFtdcQryBrokerUserEventField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryBrokerUserEventField_swigregister(CThostFtdcQryBrokerUserEventField)

class CThostFtdcBrokerUserEventField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcBrokerUserEventField_BrokerID_get, _thostmduserapi.CThostFtdcBrokerUserEventField_BrokerID_set)
    UserID = property(_thostmduserapi.CThostFtdcBrokerUserEventField_UserID_get, _thostmduserapi.CThostFtdcBrokerUserEventField_UserID_set)
    UserEventType = property(_thostmduserapi.CThostFtdcBrokerUserEventField_UserEventType_get, _thostmduserapi.CThostFtdcBrokerUserEventField_UserEventType_set)
    EventSequenceNo = property(_thostmduserapi.CThostFtdcBrokerUserEventField_EventSequenceNo_get, _thostmduserapi.CThostFtdcBrokerUserEventField_EventSequenceNo_set)
    EventDate = property(_thostmduserapi.CThostFtdcBrokerUserEventField_EventDate_get, _thostmduserapi.CThostFtdcBrokerUserEventField_EventDate_set)
    EventTime = property(_thostmduserapi.CThostFtdcBrokerUserEventField_EventTime_get, _thostmduserapi.CThostFtdcBrokerUserEventField_EventTime_set)
    UserEventInfo = property(_thostmduserapi.CThostFtdcBrokerUserEventField_UserEventInfo_get, _thostmduserapi.CThostFtdcBrokerUserEventField_UserEventInfo_set)
    InvestorID = property(_thostmduserapi.CThostFtdcBrokerUserEventField_InvestorID_get, _thostmduserapi.CThostFtdcBrokerUserEventField_InvestorID_set)
    InstrumentID = property(_thostmduserapi.CThostFtdcBrokerUserEventField_InstrumentID_get, _thostmduserapi.CThostFtdcBrokerUserEventField_InstrumentID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcBrokerUserEventField_swiginit(self, _thostmduserapi.new_CThostFtdcBrokerUserEventField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcBrokerUserEventField

# Register CThostFtdcBrokerUserEventField in _thostmduserapi:
_thostmduserapi.CThostFtdcBrokerUserEventField_swigregister(CThostFtdcBrokerUserEventField)

class CThostFtdcQryContractBankField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQryContractBankField_BrokerID_get, _thostmduserapi.CThostFtdcQryContractBankField_BrokerID_set)
    BankID = property(_thostmduserapi.CThostFtdcQryContractBankField_BankID_get, _thostmduserapi.CThostFtdcQryContractBankField_BankID_set)
    BankBrchID = property(_thostmduserapi.CThostFtdcQryContractBankField_BankBrchID_get, _thostmduserapi.CThostFtdcQryContractBankField_BankBrchID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryContractBankField_swiginit(self, _thostmduserapi.new_CThostFtdcQryContractBankField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryContractBankField

# Register CThostFtdcQryContractBankField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryContractBankField_swigregister(CThostFtdcQryContractBankField)

class CThostFtdcContractBankField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcContractBankField_BrokerID_get, _thostmduserapi.CThostFtdcContractBankField_BrokerID_set)
    BankID = property(_thostmduserapi.CThostFtdcContractBankField_BankID_get, _thostmduserapi.CThostFtdcContractBankField_BankID_set)
    BankBrchID = property(_thostmduserapi.CThostFtdcContractBankField_BankBrchID_get, _thostmduserapi.CThostFtdcContractBankField_BankBrchID_set)
    BankName = property(_thostmduserapi.CThostFtdcContractBankField_BankName_get, _thostmduserapi.CThostFtdcContractBankField_BankName_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcContractBankField_swiginit(self, _thostmduserapi.new_CThostFtdcContractBankField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcContractBankField

# Register CThostFtdcContractBankField in _thostmduserapi:
_thostmduserapi.CThostFtdcContractBankField_swigregister(CThostFtdcContractBankField)

class CThostFtdcInvestorPositionCombineDetailField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradingDay = property(_thostmduserapi.CThostFtdcInvestorPositionCombineDetailField_TradingDay_get, _thostmduserapi.CThostFtdcInvestorPositionCombineDetailField_TradingDay_set)
    OpenDate = property(_thostmduserapi.CThostFtdcInvestorPositionCombineDetailField_OpenDate_get, _thostmduserapi.CThostFtdcInvestorPositionCombineDetailField_OpenDate_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcInvestorPositionCombineDetailField_ExchangeID_get, _thostmduserapi.CThostFtdcInvestorPositionCombineDetailField_ExchangeID_set)
    SettlementID = property(_thostmduserapi.CThostFtdcInvestorPositionCombineDetailField_SettlementID_get, _thostmduserapi.CThostFtdcInvestorPositionCombineDetailField_SettlementID_set)
    BrokerID = property(_thostmduserapi.CThostFtdcInvestorPositionCombineDetailField_BrokerID_get, _thostmduserapi.CThostFtdcInvestorPositionCombineDetailField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcInvestorPositionCombineDetailField_InvestorID_get, _thostmduserapi.CThostFtdcInvestorPositionCombineDetailField_InvestorID_set)
    ComTradeID = property(_thostmduserapi.CThostFtdcInvestorPositionCombineDetailField_ComTradeID_get, _thostmduserapi.CThostFtdcInvestorPositionCombineDetailField_ComTradeID_set)
    TradeID = property(_thostmduserapi.CThostFtdcInvestorPositionCombineDetailField_TradeID_get, _thostmduserapi.CThostFtdcInvestorPositionCombineDetailField_TradeID_set)
    InstrumentID = property(_thostmduserapi.CThostFtdcInvestorPositionCombineDetailField_InstrumentID_get, _thostmduserapi.CThostFtdcInvestorPositionCombineDetailField_InstrumentID_set)
    HedgeFlag = property(_thostmduserapi.CThostFtdcInvestorPositionCombineDetailField_HedgeFlag_get, _thostmduserapi.CThostFtdcInvestorPositionCombineDetailField_HedgeFlag_set)
    Direction = property(_thostmduserapi.CThostFtdcInvestorPositionCombineDetailField_Direction_get, _thostmduserapi.CThostFtdcInvestorPositionCombineDetailField_Direction_set)
    TotalAmt = property(_thostmduserapi.CThostFtdcInvestorPositionCombineDetailField_TotalAmt_get, _thostmduserapi.CThostFtdcInvestorPositionCombineDetailField_TotalAmt_set)
    Margin = property(_thostmduserapi.CThostFtdcInvestorPositionCombineDetailField_Margin_get, _thostmduserapi.CThostFtdcInvestorPositionCombineDetailField_Margin_set)
    ExchMargin = property(_thostmduserapi.CThostFtdcInvestorPositionCombineDetailField_ExchMargin_get, _thostmduserapi.CThostFtdcInvestorPositionCombineDetailField_ExchMargin_set)
    MarginRateByMoney = property(_thostmduserapi.CThostFtdcInvestorPositionCombineDetailField_MarginRateByMoney_get, _thostmduserapi.CThostFtdcInvestorPositionCombineDetailField_MarginRateByMoney_set)
    MarginRateByVolume = property(_thostmduserapi.CThostFtdcInvestorPositionCombineDetailField_MarginRateByVolume_get, _thostmduserapi.CThostFtdcInvestorPositionCombineDetailField_MarginRateByVolume_set)
    LegID = property(_thostmduserapi.CThostFtdcInvestorPositionCombineDetailField_LegID_get, _thostmduserapi.CThostFtdcInvestorPositionCombineDetailField_LegID_set)
    LegMultiple = property(_thostmduserapi.CThostFtdcInvestorPositionCombineDetailField_LegMultiple_get, _thostmduserapi.CThostFtdcInvestorPositionCombineDetailField_LegMultiple_set)
    CombInstrumentID = property(_thostmduserapi.CThostFtdcInvestorPositionCombineDetailField_CombInstrumentID_get, _thostmduserapi.CThostFtdcInvestorPositionCombineDetailField_CombInstrumentID_set)
    TradeGroupID = property(_thostmduserapi.CThostFtdcInvestorPositionCombineDetailField_TradeGroupID_get, _thostmduserapi.CThostFtdcInvestorPositionCombineDetailField_TradeGroupID_set)
    InvestUnitID = property(_thostmduserapi.CThostFtdcInvestorPositionCombineDetailField_InvestUnitID_get, _thostmduserapi.CThostFtdcInvestorPositionCombineDetailField_InvestUnitID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcInvestorPositionCombineDetailField_swiginit(self, _thostmduserapi.new_CThostFtdcInvestorPositionCombineDetailField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcInvestorPositionCombineDetailField

# Register CThostFtdcInvestorPositionCombineDetailField in _thostmduserapi:
_thostmduserapi.CThostFtdcInvestorPositionCombineDetailField_swigregister(CThostFtdcInvestorPositionCombineDetailField)

class CThostFtdcParkedOrderField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcParkedOrderField_BrokerID_get, _thostmduserapi.CThostFtdcParkedOrderField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcParkedOrderField_InvestorID_get, _thostmduserapi.CThostFtdcParkedOrderField_InvestorID_set)
    InstrumentID = property(_thostmduserapi.CThostFtdcParkedOrderField_InstrumentID_get, _thostmduserapi.CThostFtdcParkedOrderField_InstrumentID_set)
    OrderRef = property(_thostmduserapi.CThostFtdcParkedOrderField_OrderRef_get, _thostmduserapi.CThostFtdcParkedOrderField_OrderRef_set)
    UserID = property(_thostmduserapi.CThostFtdcParkedOrderField_UserID_get, _thostmduserapi.CThostFtdcParkedOrderField_UserID_set)
    OrderPriceType = property(_thostmduserapi.CThostFtdcParkedOrderField_OrderPriceType_get, _thostmduserapi.CThostFtdcParkedOrderField_OrderPriceType_set)
    Direction = property(_thostmduserapi.CThostFtdcParkedOrderField_Direction_get, _thostmduserapi.CThostFtdcParkedOrderField_Direction_set)
    CombOffsetFlag = property(_thostmduserapi.CThostFtdcParkedOrderField_CombOffsetFlag_get, _thostmduserapi.CThostFtdcParkedOrderField_CombOffsetFlag_set)
    CombHedgeFlag = property(_thostmduserapi.CThostFtdcParkedOrderField_CombHedgeFlag_get, _thostmduserapi.CThostFtdcParkedOrderField_CombHedgeFlag_set)
    LimitPrice = property(_thostmduserapi.CThostFtdcParkedOrderField_LimitPrice_get, _thostmduserapi.CThostFtdcParkedOrderField_LimitPrice_set)
    VolumeTotalOriginal = property(_thostmduserapi.CThostFtdcParkedOrderField_VolumeTotalOriginal_get, _thostmduserapi.CThostFtdcParkedOrderField_VolumeTotalOriginal_set)
    TimeCondition = property(_thostmduserapi.CThostFtdcParkedOrderField_TimeCondition_get, _thostmduserapi.CThostFtdcParkedOrderField_TimeCondition_set)
    GTDDate = property(_thostmduserapi.CThostFtdcParkedOrderField_GTDDate_get, _thostmduserapi.CThostFtdcParkedOrderField_GTDDate_set)
    VolumeCondition = property(_thostmduserapi.CThostFtdcParkedOrderField_VolumeCondition_get, _thostmduserapi.CThostFtdcParkedOrderField_VolumeCondition_set)
    MinVolume = property(_thostmduserapi.CThostFtdcParkedOrderField_MinVolume_get, _thostmduserapi.CThostFtdcParkedOrderField_MinVolume_set)
    ContingentCondition = property(_thostmduserapi.CThostFtdcParkedOrderField_ContingentCondition_get, _thostmduserapi.CThostFtdcParkedOrderField_ContingentCondition_set)
    StopPrice = property(_thostmduserapi.CThostFtdcParkedOrderField_StopPrice_get, _thostmduserapi.CThostFtdcParkedOrderField_StopPrice_set)
    ForceCloseReason = property(_thostmduserapi.CThostFtdcParkedOrderField_ForceCloseReason_get, _thostmduserapi.CThostFtdcParkedOrderField_ForceCloseReason_set)
    IsAutoSuspend = property(_thostmduserapi.CThostFtdcParkedOrderField_IsAutoSuspend_get, _thostmduserapi.CThostFtdcParkedOrderField_IsAutoSuspend_set)
    BusinessUnit = property(_thostmduserapi.CThostFtdcParkedOrderField_BusinessUnit_get, _thostmduserapi.CThostFtdcParkedOrderField_BusinessUnit_set)
    RequestID = property(_thostmduserapi.CThostFtdcParkedOrderField_RequestID_get, _thostmduserapi.CThostFtdcParkedOrderField_RequestID_set)
    UserForceClose = property(_thostmduserapi.CThostFtdcParkedOrderField_UserForceClose_get, _thostmduserapi.CThostFtdcParkedOrderField_UserForceClose_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcParkedOrderField_ExchangeID_get, _thostmduserapi.CThostFtdcParkedOrderField_ExchangeID_set)
    ParkedOrderID = property(_thostmduserapi.CThostFtdcParkedOrderField_ParkedOrderID_get, _thostmduserapi.CThostFtdcParkedOrderField_ParkedOrderID_set)
    UserType = property(_thostmduserapi.CThostFtdcParkedOrderField_UserType_get, _thostmduserapi.CThostFtdcParkedOrderField_UserType_set)
    Status = property(_thostmduserapi.CThostFtdcParkedOrderField_Status_get, _thostmduserapi.CThostFtdcParkedOrderField_Status_set)
    ErrorID = property(_thostmduserapi.CThostFtdcParkedOrderField_ErrorID_get, _thostmduserapi.CThostFtdcParkedOrderField_ErrorID_set)
    ErrorMsg = property(_thostmduserapi.CThostFtdcParkedOrderField_ErrorMsg_get, _thostmduserapi.CThostFtdcParkedOrderField_ErrorMsg_set)
    IsSwapOrder = property(_thostmduserapi.CThostFtdcParkedOrderField_IsSwapOrder_get, _thostmduserapi.CThostFtdcParkedOrderField_IsSwapOrder_set)
    AccountID = property(_thostmduserapi.CThostFtdcParkedOrderField_AccountID_get, _thostmduserapi.CThostFtdcParkedOrderField_AccountID_set)
    CurrencyID = property(_thostmduserapi.CThostFtdcParkedOrderField_CurrencyID_get, _thostmduserapi.CThostFtdcParkedOrderField_CurrencyID_set)
    ClientID = property(_thostmduserapi.CThostFtdcParkedOrderField_ClientID_get, _thostmduserapi.CThostFtdcParkedOrderField_ClientID_set)
    InvestUnitID = property(_thostmduserapi.CThostFtdcParkedOrderField_InvestUnitID_get, _thostmduserapi.CThostFtdcParkedOrderField_InvestUnitID_set)
    IPAddress = property(_thostmduserapi.CThostFtdcParkedOrderField_IPAddress_get, _thostmduserapi.CThostFtdcParkedOrderField_IPAddress_set)
    MacAddress = property(_thostmduserapi.CThostFtdcParkedOrderField_MacAddress_get, _thostmduserapi.CThostFtdcParkedOrderField_MacAddress_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcParkedOrderField_swiginit(self, _thostmduserapi.new_CThostFtdcParkedOrderField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcParkedOrderField

# Register CThostFtdcParkedOrderField in _thostmduserapi:
_thostmduserapi.CThostFtdcParkedOrderField_swigregister(CThostFtdcParkedOrderField)

class CThostFtdcParkedOrderActionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcParkedOrderActionField_BrokerID_get, _thostmduserapi.CThostFtdcParkedOrderActionField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcParkedOrderActionField_InvestorID_get, _thostmduserapi.CThostFtdcParkedOrderActionField_InvestorID_set)
    OrderActionRef = property(_thostmduserapi.CThostFtdcParkedOrderActionField_OrderActionRef_get, _thostmduserapi.CThostFtdcParkedOrderActionField_OrderActionRef_set)
    OrderRef = property(_thostmduserapi.CThostFtdcParkedOrderActionField_OrderRef_get, _thostmduserapi.CThostFtdcParkedOrderActionField_OrderRef_set)
    RequestID = property(_thostmduserapi.CThostFtdcParkedOrderActionField_RequestID_get, _thostmduserapi.CThostFtdcParkedOrderActionField_RequestID_set)
    FrontID = property(_thostmduserapi.CThostFtdcParkedOrderActionField_FrontID_get, _thostmduserapi.CThostFtdcParkedOrderActionField_FrontID_set)
    SessionID = property(_thostmduserapi.CThostFtdcParkedOrderActionField_SessionID_get, _thostmduserapi.CThostFtdcParkedOrderActionField_SessionID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcParkedOrderActionField_ExchangeID_get, _thostmduserapi.CThostFtdcParkedOrderActionField_ExchangeID_set)
    OrderSysID = property(_thostmduserapi.CThostFtdcParkedOrderActionField_OrderSysID_get, _thostmduserapi.CThostFtdcParkedOrderActionField_OrderSysID_set)
    ActionFlag = property(_thostmduserapi.CThostFtdcParkedOrderActionField_ActionFlag_get, _thostmduserapi.CThostFtdcParkedOrderActionField_ActionFlag_set)
    LimitPrice = property(_thostmduserapi.CThostFtdcParkedOrderActionField_LimitPrice_get, _thostmduserapi.CThostFtdcParkedOrderActionField_LimitPrice_set)
    VolumeChange = property(_thostmduserapi.CThostFtdcParkedOrderActionField_VolumeChange_get, _thostmduserapi.CThostFtdcParkedOrderActionField_VolumeChange_set)
    UserID = property(_thostmduserapi.CThostFtdcParkedOrderActionField_UserID_get, _thostmduserapi.CThostFtdcParkedOrderActionField_UserID_set)
    InstrumentID = property(_thostmduserapi.CThostFtdcParkedOrderActionField_InstrumentID_get, _thostmduserapi.CThostFtdcParkedOrderActionField_InstrumentID_set)
    ParkedOrderActionID = property(_thostmduserapi.CThostFtdcParkedOrderActionField_ParkedOrderActionID_get, _thostmduserapi.CThostFtdcParkedOrderActionField_ParkedOrderActionID_set)
    UserType = property(_thostmduserapi.CThostFtdcParkedOrderActionField_UserType_get, _thostmduserapi.CThostFtdcParkedOrderActionField_UserType_set)
    Status = property(_thostmduserapi.CThostFtdcParkedOrderActionField_Status_get, _thostmduserapi.CThostFtdcParkedOrderActionField_Status_set)
    ErrorID = property(_thostmduserapi.CThostFtdcParkedOrderActionField_ErrorID_get, _thostmduserapi.CThostFtdcParkedOrderActionField_ErrorID_set)
    ErrorMsg = property(_thostmduserapi.CThostFtdcParkedOrderActionField_ErrorMsg_get, _thostmduserapi.CThostFtdcParkedOrderActionField_ErrorMsg_set)
    InvestUnitID = property(_thostmduserapi.CThostFtdcParkedOrderActionField_InvestUnitID_get, _thostmduserapi.CThostFtdcParkedOrderActionField_InvestUnitID_set)
    IPAddress = property(_thostmduserapi.CThostFtdcParkedOrderActionField_IPAddress_get, _thostmduserapi.CThostFtdcParkedOrderActionField_IPAddress_set)
    MacAddress = property(_thostmduserapi.CThostFtdcParkedOrderActionField_MacAddress_get, _thostmduserapi.CThostFtdcParkedOrderActionField_MacAddress_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcParkedOrderActionField_swiginit(self, _thostmduserapi.new_CThostFtdcParkedOrderActionField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcParkedOrderActionField

# Register CThostFtdcParkedOrderActionField in _thostmduserapi:
_thostmduserapi.CThostFtdcParkedOrderActionField_swigregister(CThostFtdcParkedOrderActionField)

class CThostFtdcQryParkedOrderField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQryParkedOrderField_BrokerID_get, _thostmduserapi.CThostFtdcQryParkedOrderField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcQryParkedOrderField_InvestorID_get, _thostmduserapi.CThostFtdcQryParkedOrderField_InvestorID_set)
    InstrumentID = property(_thostmduserapi.CThostFtdcQryParkedOrderField_InstrumentID_get, _thostmduserapi.CThostFtdcQryParkedOrderField_InstrumentID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcQryParkedOrderField_ExchangeID_get, _thostmduserapi.CThostFtdcQryParkedOrderField_ExchangeID_set)
    InvestUnitID = property(_thostmduserapi.CThostFtdcQryParkedOrderField_InvestUnitID_get, _thostmduserapi.CThostFtdcQryParkedOrderField_InvestUnitID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryParkedOrderField_swiginit(self, _thostmduserapi.new_CThostFtdcQryParkedOrderField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryParkedOrderField

# Register CThostFtdcQryParkedOrderField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryParkedOrderField_swigregister(CThostFtdcQryParkedOrderField)

class CThostFtdcQryParkedOrderActionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQryParkedOrderActionField_BrokerID_get, _thostmduserapi.CThostFtdcQryParkedOrderActionField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcQryParkedOrderActionField_InvestorID_get, _thostmduserapi.CThostFtdcQryParkedOrderActionField_InvestorID_set)
    InstrumentID = property(_thostmduserapi.CThostFtdcQryParkedOrderActionField_InstrumentID_get, _thostmduserapi.CThostFtdcQryParkedOrderActionField_InstrumentID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcQryParkedOrderActionField_ExchangeID_get, _thostmduserapi.CThostFtdcQryParkedOrderActionField_ExchangeID_set)
    InvestUnitID = property(_thostmduserapi.CThostFtdcQryParkedOrderActionField_InvestUnitID_get, _thostmduserapi.CThostFtdcQryParkedOrderActionField_InvestUnitID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryParkedOrderActionField_swiginit(self, _thostmduserapi.new_CThostFtdcQryParkedOrderActionField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryParkedOrderActionField

# Register CThostFtdcQryParkedOrderActionField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryParkedOrderActionField_swigregister(CThostFtdcQryParkedOrderActionField)

class CThostFtdcRemoveParkedOrderField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcRemoveParkedOrderField_BrokerID_get, _thostmduserapi.CThostFtdcRemoveParkedOrderField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcRemoveParkedOrderField_InvestorID_get, _thostmduserapi.CThostFtdcRemoveParkedOrderField_InvestorID_set)
    ParkedOrderID = property(_thostmduserapi.CThostFtdcRemoveParkedOrderField_ParkedOrderID_get, _thostmduserapi.CThostFtdcRemoveParkedOrderField_ParkedOrderID_set)
    InvestUnitID = property(_thostmduserapi.CThostFtdcRemoveParkedOrderField_InvestUnitID_get, _thostmduserapi.CThostFtdcRemoveParkedOrderField_InvestUnitID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcRemoveParkedOrderField_swiginit(self, _thostmduserapi.new_CThostFtdcRemoveParkedOrderField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcRemoveParkedOrderField

# Register CThostFtdcRemoveParkedOrderField in _thostmduserapi:
_thostmduserapi.CThostFtdcRemoveParkedOrderField_swigregister(CThostFtdcRemoveParkedOrderField)

class CThostFtdcRemoveParkedOrderActionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcRemoveParkedOrderActionField_BrokerID_get, _thostmduserapi.CThostFtdcRemoveParkedOrderActionField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcRemoveParkedOrderActionField_InvestorID_get, _thostmduserapi.CThostFtdcRemoveParkedOrderActionField_InvestorID_set)
    ParkedOrderActionID = property(_thostmduserapi.CThostFtdcRemoveParkedOrderActionField_ParkedOrderActionID_get, _thostmduserapi.CThostFtdcRemoveParkedOrderActionField_ParkedOrderActionID_set)
    InvestUnitID = property(_thostmduserapi.CThostFtdcRemoveParkedOrderActionField_InvestUnitID_get, _thostmduserapi.CThostFtdcRemoveParkedOrderActionField_InvestUnitID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcRemoveParkedOrderActionField_swiginit(self, _thostmduserapi.new_CThostFtdcRemoveParkedOrderActionField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcRemoveParkedOrderActionField

# Register CThostFtdcRemoveParkedOrderActionField in _thostmduserapi:
_thostmduserapi.CThostFtdcRemoveParkedOrderActionField_swigregister(CThostFtdcRemoveParkedOrderActionField)

class CThostFtdcInvestorWithdrawAlgorithmField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcInvestorWithdrawAlgorithmField_BrokerID_get, _thostmduserapi.CThostFtdcInvestorWithdrawAlgorithmField_BrokerID_set)
    InvestorRange = property(_thostmduserapi.CThostFtdcInvestorWithdrawAlgorithmField_InvestorRange_get, _thostmduserapi.CThostFtdcInvestorWithdrawAlgorithmField_InvestorRange_set)
    InvestorID = property(_thostmduserapi.CThostFtdcInvestorWithdrawAlgorithmField_InvestorID_get, _thostmduserapi.CThostFtdcInvestorWithdrawAlgorithmField_InvestorID_set)
    UsingRatio = property(_thostmduserapi.CThostFtdcInvestorWithdrawAlgorithmField_UsingRatio_get, _thostmduserapi.CThostFtdcInvestorWithdrawAlgorithmField_UsingRatio_set)
    CurrencyID = property(_thostmduserapi.CThostFtdcInvestorWithdrawAlgorithmField_CurrencyID_get, _thostmduserapi.CThostFtdcInvestorWithdrawAlgorithmField_CurrencyID_set)
    FundMortgageRatio = property(_thostmduserapi.CThostFtdcInvestorWithdrawAlgorithmField_FundMortgageRatio_get, _thostmduserapi.CThostFtdcInvestorWithdrawAlgorithmField_FundMortgageRatio_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcInvestorWithdrawAlgorithmField_swiginit(self, _thostmduserapi.new_CThostFtdcInvestorWithdrawAlgorithmField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcInvestorWithdrawAlgorithmField

# Register CThostFtdcInvestorWithdrawAlgorithmField in _thostmduserapi:
_thostmduserapi.CThostFtdcInvestorWithdrawAlgorithmField_swigregister(CThostFtdcInvestorWithdrawAlgorithmField)

class CThostFtdcQryInvestorPositionCombineDetailField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQryInvestorPositionCombineDetailField_BrokerID_get, _thostmduserapi.CThostFtdcQryInvestorPositionCombineDetailField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcQryInvestorPositionCombineDetailField_InvestorID_get, _thostmduserapi.CThostFtdcQryInvestorPositionCombineDetailField_InvestorID_set)
    CombInstrumentID = property(_thostmduserapi.CThostFtdcQryInvestorPositionCombineDetailField_CombInstrumentID_get, _thostmduserapi.CThostFtdcQryInvestorPositionCombineDetailField_CombInstrumentID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcQryInvestorPositionCombineDetailField_ExchangeID_get, _thostmduserapi.CThostFtdcQryInvestorPositionCombineDetailField_ExchangeID_set)
    InvestUnitID = property(_thostmduserapi.CThostFtdcQryInvestorPositionCombineDetailField_InvestUnitID_get, _thostmduserapi.CThostFtdcQryInvestorPositionCombineDetailField_InvestUnitID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryInvestorPositionCombineDetailField_swiginit(self, _thostmduserapi.new_CThostFtdcQryInvestorPositionCombineDetailField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryInvestorPositionCombineDetailField

# Register CThostFtdcQryInvestorPositionCombineDetailField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryInvestorPositionCombineDetailField_swigregister(CThostFtdcQryInvestorPositionCombineDetailField)

class CThostFtdcMarketDataAveragePriceField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    AveragePrice = property(_thostmduserapi.CThostFtdcMarketDataAveragePriceField_AveragePrice_get, _thostmduserapi.CThostFtdcMarketDataAveragePriceField_AveragePrice_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcMarketDataAveragePriceField_swiginit(self, _thostmduserapi.new_CThostFtdcMarketDataAveragePriceField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcMarketDataAveragePriceField

# Register CThostFtdcMarketDataAveragePriceField in _thostmduserapi:
_thostmduserapi.CThostFtdcMarketDataAveragePriceField_swigregister(CThostFtdcMarketDataAveragePriceField)

class CThostFtdcVerifyInvestorPasswordField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcVerifyInvestorPasswordField_BrokerID_get, _thostmduserapi.CThostFtdcVerifyInvestorPasswordField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcVerifyInvestorPasswordField_InvestorID_get, _thostmduserapi.CThostFtdcVerifyInvestorPasswordField_InvestorID_set)
    Password = property(_thostmduserapi.CThostFtdcVerifyInvestorPasswordField_Password_get, _thostmduserapi.CThostFtdcVerifyInvestorPasswordField_Password_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcVerifyInvestorPasswordField_swiginit(self, _thostmduserapi.new_CThostFtdcVerifyInvestorPasswordField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcVerifyInvestorPasswordField

# Register CThostFtdcVerifyInvestorPasswordField in _thostmduserapi:
_thostmduserapi.CThostFtdcVerifyInvestorPasswordField_swigregister(CThostFtdcVerifyInvestorPasswordField)

class CThostFtdcUserIPField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcUserIPField_BrokerID_get, _thostmduserapi.CThostFtdcUserIPField_BrokerID_set)
    UserID = property(_thostmduserapi.CThostFtdcUserIPField_UserID_get, _thostmduserapi.CThostFtdcUserIPField_UserID_set)
    IPAddress = property(_thostmduserapi.CThostFtdcUserIPField_IPAddress_get, _thostmduserapi.CThostFtdcUserIPField_IPAddress_set)
    IPMask = property(_thostmduserapi.CThostFtdcUserIPField_IPMask_get, _thostmduserapi.CThostFtdcUserIPField_IPMask_set)
    MacAddress = property(_thostmduserapi.CThostFtdcUserIPField_MacAddress_get, _thostmduserapi.CThostFtdcUserIPField_MacAddress_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcUserIPField_swiginit(self, _thostmduserapi.new_CThostFtdcUserIPField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcUserIPField

# Register CThostFtdcUserIPField in _thostmduserapi:
_thostmduserapi.CThostFtdcUserIPField_swigregister(CThostFtdcUserIPField)

class CThostFtdcTradingNoticeInfoField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcTradingNoticeInfoField_BrokerID_get, _thostmduserapi.CThostFtdcTradingNoticeInfoField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcTradingNoticeInfoField_InvestorID_get, _thostmduserapi.CThostFtdcTradingNoticeInfoField_InvestorID_set)
    SendTime = property(_thostmduserapi.CThostFtdcTradingNoticeInfoField_SendTime_get, _thostmduserapi.CThostFtdcTradingNoticeInfoField_SendTime_set)
    FieldContent = property(_thostmduserapi.CThostFtdcTradingNoticeInfoField_FieldContent_get, _thostmduserapi.CThostFtdcTradingNoticeInfoField_FieldContent_set)
    SequenceSeries = property(_thostmduserapi.CThostFtdcTradingNoticeInfoField_SequenceSeries_get, _thostmduserapi.CThostFtdcTradingNoticeInfoField_SequenceSeries_set)
    SequenceNo = property(_thostmduserapi.CThostFtdcTradingNoticeInfoField_SequenceNo_get, _thostmduserapi.CThostFtdcTradingNoticeInfoField_SequenceNo_set)
    InvestUnitID = property(_thostmduserapi.CThostFtdcTradingNoticeInfoField_InvestUnitID_get, _thostmduserapi.CThostFtdcTradingNoticeInfoField_InvestUnitID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcTradingNoticeInfoField_swiginit(self, _thostmduserapi.new_CThostFtdcTradingNoticeInfoField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcTradingNoticeInfoField

# Register CThostFtdcTradingNoticeInfoField in _thostmduserapi:
_thostmduserapi.CThostFtdcTradingNoticeInfoField_swigregister(CThostFtdcTradingNoticeInfoField)

class CThostFtdcTradingNoticeField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcTradingNoticeField_BrokerID_get, _thostmduserapi.CThostFtdcTradingNoticeField_BrokerID_set)
    InvestorRange = property(_thostmduserapi.CThostFtdcTradingNoticeField_InvestorRange_get, _thostmduserapi.CThostFtdcTradingNoticeField_InvestorRange_set)
    InvestorID = property(_thostmduserapi.CThostFtdcTradingNoticeField_InvestorID_get, _thostmduserapi.CThostFtdcTradingNoticeField_InvestorID_set)
    SequenceSeries = property(_thostmduserapi.CThostFtdcTradingNoticeField_SequenceSeries_get, _thostmduserapi.CThostFtdcTradingNoticeField_SequenceSeries_set)
    UserID = property(_thostmduserapi.CThostFtdcTradingNoticeField_UserID_get, _thostmduserapi.CThostFtdcTradingNoticeField_UserID_set)
    SendTime = property(_thostmduserapi.CThostFtdcTradingNoticeField_SendTime_get, _thostmduserapi.CThostFtdcTradingNoticeField_SendTime_set)
    SequenceNo = property(_thostmduserapi.CThostFtdcTradingNoticeField_SequenceNo_get, _thostmduserapi.CThostFtdcTradingNoticeField_SequenceNo_set)
    FieldContent = property(_thostmduserapi.CThostFtdcTradingNoticeField_FieldContent_get, _thostmduserapi.CThostFtdcTradingNoticeField_FieldContent_set)
    InvestUnitID = property(_thostmduserapi.CThostFtdcTradingNoticeField_InvestUnitID_get, _thostmduserapi.CThostFtdcTradingNoticeField_InvestUnitID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcTradingNoticeField_swiginit(self, _thostmduserapi.new_CThostFtdcTradingNoticeField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcTradingNoticeField

# Register CThostFtdcTradingNoticeField in _thostmduserapi:
_thostmduserapi.CThostFtdcTradingNoticeField_swigregister(CThostFtdcTradingNoticeField)

class CThostFtdcQryTradingNoticeField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQryTradingNoticeField_BrokerID_get, _thostmduserapi.CThostFtdcQryTradingNoticeField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcQryTradingNoticeField_InvestorID_get, _thostmduserapi.CThostFtdcQryTradingNoticeField_InvestorID_set)
    InvestUnitID = property(_thostmduserapi.CThostFtdcQryTradingNoticeField_InvestUnitID_get, _thostmduserapi.CThostFtdcQryTradingNoticeField_InvestUnitID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryTradingNoticeField_swiginit(self, _thostmduserapi.new_CThostFtdcQryTradingNoticeField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryTradingNoticeField

# Register CThostFtdcQryTradingNoticeField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryTradingNoticeField_swigregister(CThostFtdcQryTradingNoticeField)

class CThostFtdcQryErrOrderField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQryErrOrderField_BrokerID_get, _thostmduserapi.CThostFtdcQryErrOrderField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcQryErrOrderField_InvestorID_get, _thostmduserapi.CThostFtdcQryErrOrderField_InvestorID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryErrOrderField_swiginit(self, _thostmduserapi.new_CThostFtdcQryErrOrderField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryErrOrderField

# Register CThostFtdcQryErrOrderField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryErrOrderField_swigregister(CThostFtdcQryErrOrderField)

class CThostFtdcErrOrderField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcErrOrderField_BrokerID_get, _thostmduserapi.CThostFtdcErrOrderField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcErrOrderField_InvestorID_get, _thostmduserapi.CThostFtdcErrOrderField_InvestorID_set)
    InstrumentID = property(_thostmduserapi.CThostFtdcErrOrderField_InstrumentID_get, _thostmduserapi.CThostFtdcErrOrderField_InstrumentID_set)
    OrderRef = property(_thostmduserapi.CThostFtdcErrOrderField_OrderRef_get, _thostmduserapi.CThostFtdcErrOrderField_OrderRef_set)
    UserID = property(_thostmduserapi.CThostFtdcErrOrderField_UserID_get, _thostmduserapi.CThostFtdcErrOrderField_UserID_set)
    OrderPriceType = property(_thostmduserapi.CThostFtdcErrOrderField_OrderPriceType_get, _thostmduserapi.CThostFtdcErrOrderField_OrderPriceType_set)
    Direction = property(_thostmduserapi.CThostFtdcErrOrderField_Direction_get, _thostmduserapi.CThostFtdcErrOrderField_Direction_set)
    CombOffsetFlag = property(_thostmduserapi.CThostFtdcErrOrderField_CombOffsetFlag_get, _thostmduserapi.CThostFtdcErrOrderField_CombOffsetFlag_set)
    CombHedgeFlag = property(_thostmduserapi.CThostFtdcErrOrderField_CombHedgeFlag_get, _thostmduserapi.CThostFtdcErrOrderField_CombHedgeFlag_set)
    LimitPrice = property(_thostmduserapi.CThostFtdcErrOrderField_LimitPrice_get, _thostmduserapi.CThostFtdcErrOrderField_LimitPrice_set)
    VolumeTotalOriginal = property(_thostmduserapi.CThostFtdcErrOrderField_VolumeTotalOriginal_get, _thostmduserapi.CThostFtdcErrOrderField_VolumeTotalOriginal_set)
    TimeCondition = property(_thostmduserapi.CThostFtdcErrOrderField_TimeCondition_get, _thostmduserapi.CThostFtdcErrOrderField_TimeCondition_set)
    GTDDate = property(_thostmduserapi.CThostFtdcErrOrderField_GTDDate_get, _thostmduserapi.CThostFtdcErrOrderField_GTDDate_set)
    VolumeCondition = property(_thostmduserapi.CThostFtdcErrOrderField_VolumeCondition_get, _thostmduserapi.CThostFtdcErrOrderField_VolumeCondition_set)
    MinVolume = property(_thostmduserapi.CThostFtdcErrOrderField_MinVolume_get, _thostmduserapi.CThostFtdcErrOrderField_MinVolume_set)
    ContingentCondition = property(_thostmduserapi.CThostFtdcErrOrderField_ContingentCondition_get, _thostmduserapi.CThostFtdcErrOrderField_ContingentCondition_set)
    StopPrice = property(_thostmduserapi.CThostFtdcErrOrderField_StopPrice_get, _thostmduserapi.CThostFtdcErrOrderField_StopPrice_set)
    ForceCloseReason = property(_thostmduserapi.CThostFtdcErrOrderField_ForceCloseReason_get, _thostmduserapi.CThostFtdcErrOrderField_ForceCloseReason_set)
    IsAutoSuspend = property(_thostmduserapi.CThostFtdcErrOrderField_IsAutoSuspend_get, _thostmduserapi.CThostFtdcErrOrderField_IsAutoSuspend_set)
    BusinessUnit = property(_thostmduserapi.CThostFtdcErrOrderField_BusinessUnit_get, _thostmduserapi.CThostFtdcErrOrderField_BusinessUnit_set)
    RequestID = property(_thostmduserapi.CThostFtdcErrOrderField_RequestID_get, _thostmduserapi.CThostFtdcErrOrderField_RequestID_set)
    UserForceClose = property(_thostmduserapi.CThostFtdcErrOrderField_UserForceClose_get, _thostmduserapi.CThostFtdcErrOrderField_UserForceClose_set)
    ErrorID = property(_thostmduserapi.CThostFtdcErrOrderField_ErrorID_get, _thostmduserapi.CThostFtdcErrOrderField_ErrorID_set)
    ErrorMsg = property(_thostmduserapi.CThostFtdcErrOrderField_ErrorMsg_get, _thostmduserapi.CThostFtdcErrOrderField_ErrorMsg_set)
    IsSwapOrder = property(_thostmduserapi.CThostFtdcErrOrderField_IsSwapOrder_get, _thostmduserapi.CThostFtdcErrOrderField_IsSwapOrder_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcErrOrderField_ExchangeID_get, _thostmduserapi.CThostFtdcErrOrderField_ExchangeID_set)
    InvestUnitID = property(_thostmduserapi.CThostFtdcErrOrderField_InvestUnitID_get, _thostmduserapi.CThostFtdcErrOrderField_InvestUnitID_set)
    AccountID = property(_thostmduserapi.CThostFtdcErrOrderField_AccountID_get, _thostmduserapi.CThostFtdcErrOrderField_AccountID_set)
    CurrencyID = property(_thostmduserapi.CThostFtdcErrOrderField_CurrencyID_get, _thostmduserapi.CThostFtdcErrOrderField_CurrencyID_set)
    ClientID = property(_thostmduserapi.CThostFtdcErrOrderField_ClientID_get, _thostmduserapi.CThostFtdcErrOrderField_ClientID_set)
    IPAddress = property(_thostmduserapi.CThostFtdcErrOrderField_IPAddress_get, _thostmduserapi.CThostFtdcErrOrderField_IPAddress_set)
    MacAddress = property(_thostmduserapi.CThostFtdcErrOrderField_MacAddress_get, _thostmduserapi.CThostFtdcErrOrderField_MacAddress_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcErrOrderField_swiginit(self, _thostmduserapi.new_CThostFtdcErrOrderField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcErrOrderField

# Register CThostFtdcErrOrderField in _thostmduserapi:
_thostmduserapi.CThostFtdcErrOrderField_swigregister(CThostFtdcErrOrderField)

class CThostFtdcErrorConditionalOrderField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcErrorConditionalOrderField_BrokerID_get, _thostmduserapi.CThostFtdcErrorConditionalOrderField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcErrorConditionalOrderField_InvestorID_get, _thostmduserapi.CThostFtdcErrorConditionalOrderField_InvestorID_set)
    InstrumentID = property(_thostmduserapi.CThostFtdcErrorConditionalOrderField_InstrumentID_get, _thostmduserapi.CThostFtdcErrorConditionalOrderField_InstrumentID_set)
    OrderRef = property(_thostmduserapi.CThostFtdcErrorConditionalOrderField_OrderRef_get, _thostmduserapi.CThostFtdcErrorConditionalOrderField_OrderRef_set)
    UserID = property(_thostmduserapi.CThostFtdcErrorConditionalOrderField_UserID_get, _thostmduserapi.CThostFtdcErrorConditionalOrderField_UserID_set)
    OrderPriceType = property(_thostmduserapi.CThostFtdcErrorConditionalOrderField_OrderPriceType_get, _thostmduserapi.CThostFtdcErrorConditionalOrderField_OrderPriceType_set)
    Direction = property(_thostmduserapi.CThostFtdcErrorConditionalOrderField_Direction_get, _thostmduserapi.CThostFtdcErrorConditionalOrderField_Direction_set)
    CombOffsetFlag = property(_thostmduserapi.CThostFtdcErrorConditionalOrderField_CombOffsetFlag_get, _thostmduserapi.CThostFtdcErrorConditionalOrderField_CombOffsetFlag_set)
    CombHedgeFlag = property(_thostmduserapi.CThostFtdcErrorConditionalOrderField_CombHedgeFlag_get, _thostmduserapi.CThostFtdcErrorConditionalOrderField_CombHedgeFlag_set)
    LimitPrice = property(_thostmduserapi.CThostFtdcErrorConditionalOrderField_LimitPrice_get, _thostmduserapi.CThostFtdcErrorConditionalOrderField_LimitPrice_set)
    VolumeTotalOriginal = property(_thostmduserapi.CThostFtdcErrorConditionalOrderField_VolumeTotalOriginal_get, _thostmduserapi.CThostFtdcErrorConditionalOrderField_VolumeTotalOriginal_set)
    TimeCondition = property(_thostmduserapi.CThostFtdcErrorConditionalOrderField_TimeCondition_get, _thostmduserapi.CThostFtdcErrorConditionalOrderField_TimeCondition_set)
    GTDDate = property(_thostmduserapi.CThostFtdcErrorConditionalOrderField_GTDDate_get, _thostmduserapi.CThostFtdcErrorConditionalOrderField_GTDDate_set)
    VolumeCondition = property(_thostmduserapi.CThostFtdcErrorConditionalOrderField_VolumeCondition_get, _thostmduserapi.CThostFtdcErrorConditionalOrderField_VolumeCondition_set)
    MinVolume = property(_thostmduserapi.CThostFtdcErrorConditionalOrderField_MinVolume_get, _thostmduserapi.CThostFtdcErrorConditionalOrderField_MinVolume_set)
    ContingentCondition = property(_thostmduserapi.CThostFtdcErrorConditionalOrderField_ContingentCondition_get, _thostmduserapi.CThostFtdcErrorConditionalOrderField_ContingentCondition_set)
    StopPrice = property(_thostmduserapi.CThostFtdcErrorConditionalOrderField_StopPrice_get, _thostmduserapi.CThostFtdcErrorConditionalOrderField_StopPrice_set)
    ForceCloseReason = property(_thostmduserapi.CThostFtdcErrorConditionalOrderField_ForceCloseReason_get, _thostmduserapi.CThostFtdcErrorConditionalOrderField_ForceCloseReason_set)
    IsAutoSuspend = property(_thostmduserapi.CThostFtdcErrorConditionalOrderField_IsAutoSuspend_get, _thostmduserapi.CThostFtdcErrorConditionalOrderField_IsAutoSuspend_set)
    BusinessUnit = property(_thostmduserapi.CThostFtdcErrorConditionalOrderField_BusinessUnit_get, _thostmduserapi.CThostFtdcErrorConditionalOrderField_BusinessUnit_set)
    RequestID = property(_thostmduserapi.CThostFtdcErrorConditionalOrderField_RequestID_get, _thostmduserapi.CThostFtdcErrorConditionalOrderField_RequestID_set)
    OrderLocalID = property(_thostmduserapi.CThostFtdcErrorConditionalOrderField_OrderLocalID_get, _thostmduserapi.CThostFtdcErrorConditionalOrderField_OrderLocalID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcErrorConditionalOrderField_ExchangeID_get, _thostmduserapi.CThostFtdcErrorConditionalOrderField_ExchangeID_set)
    ParticipantID = property(_thostmduserapi.CThostFtdcErrorConditionalOrderField_ParticipantID_get, _thostmduserapi.CThostFtdcErrorConditionalOrderField_ParticipantID_set)
    ClientID = property(_thostmduserapi.CThostFtdcErrorConditionalOrderField_ClientID_get, _thostmduserapi.CThostFtdcErrorConditionalOrderField_ClientID_set)
    ExchangeInstID = property(_thostmduserapi.CThostFtdcErrorConditionalOrderField_ExchangeInstID_get, _thostmduserapi.CThostFtdcErrorConditionalOrderField_ExchangeInstID_set)
    TraderID = property(_thostmduserapi.CThostFtdcErrorConditionalOrderField_TraderID_get, _thostmduserapi.CThostFtdcErrorConditionalOrderField_TraderID_set)
    InstallID = property(_thostmduserapi.CThostFtdcErrorConditionalOrderField_InstallID_get, _thostmduserapi.CThostFtdcErrorConditionalOrderField_InstallID_set)
    OrderSubmitStatus = property(_thostmduserapi.CThostFtdcErrorConditionalOrderField_OrderSubmitStatus_get, _thostmduserapi.CThostFtdcErrorConditionalOrderField_OrderSubmitStatus_set)
    NotifySequence = property(_thostmduserapi.CThostFtdcErrorConditionalOrderField_NotifySequence_get, _thostmduserapi.CThostFtdcErrorConditionalOrderField_NotifySequence_set)
    TradingDay = property(_thostmduserapi.CThostFtdcErrorConditionalOrderField_TradingDay_get, _thostmduserapi.CThostFtdcErrorConditionalOrderField_TradingDay_set)
    SettlementID = property(_thostmduserapi.CThostFtdcErrorConditionalOrderField_SettlementID_get, _thostmduserapi.CThostFtdcErrorConditionalOrderField_SettlementID_set)
    OrderSysID = property(_thostmduserapi.CThostFtdcErrorConditionalOrderField_OrderSysID_get, _thostmduserapi.CThostFtdcErrorConditionalOrderField_OrderSysID_set)
    OrderSource = property(_thostmduserapi.CThostFtdcErrorConditionalOrderField_OrderSource_get, _thostmduserapi.CThostFtdcErrorConditionalOrderField_OrderSource_set)
    OrderStatus = property(_thostmduserapi.CThostFtdcErrorConditionalOrderField_OrderStatus_get, _thostmduserapi.CThostFtdcErrorConditionalOrderField_OrderStatus_set)
    OrderType = property(_thostmduserapi.CThostFtdcErrorConditionalOrderField_OrderType_get, _thostmduserapi.CThostFtdcErrorConditionalOrderField_OrderType_set)
    VolumeTraded = property(_thostmduserapi.CThostFtdcErrorConditionalOrderField_VolumeTraded_get, _thostmduserapi.CThostFtdcErrorConditionalOrderField_VolumeTraded_set)
    VolumeTotal = property(_thostmduserapi.CThostFtdcErrorConditionalOrderField_VolumeTotal_get, _thostmduserapi.CThostFtdcErrorConditionalOrderField_VolumeTotal_set)
    InsertDate = property(_thostmduserapi.CThostFtdcErrorConditionalOrderField_InsertDate_get, _thostmduserapi.CThostFtdcErrorConditionalOrderField_InsertDate_set)
    InsertTime = property(_thostmduserapi.CThostFtdcErrorConditionalOrderField_InsertTime_get, _thostmduserapi.CThostFtdcErrorConditionalOrderField_InsertTime_set)
    ActiveTime = property(_thostmduserapi.CThostFtdcErrorConditionalOrderField_ActiveTime_get, _thostmduserapi.CThostFtdcErrorConditionalOrderField_ActiveTime_set)
    SuspendTime = property(_thostmduserapi.CThostFtdcErrorConditionalOrderField_SuspendTime_get, _thostmduserapi.CThostFtdcErrorConditionalOrderField_SuspendTime_set)
    UpdateTime = property(_thostmduserapi.CThostFtdcErrorConditionalOrderField_UpdateTime_get, _thostmduserapi.CThostFtdcErrorConditionalOrderField_UpdateTime_set)
    CancelTime = property(_thostmduserapi.CThostFtdcErrorConditionalOrderField_CancelTime_get, _thostmduserapi.CThostFtdcErrorConditionalOrderField_CancelTime_set)
    ActiveTraderID = property(_thostmduserapi.CThostFtdcErrorConditionalOrderField_ActiveTraderID_get, _thostmduserapi.CThostFtdcErrorConditionalOrderField_ActiveTraderID_set)
    ClearingPartID = property(_thostmduserapi.CThostFtdcErrorConditionalOrderField_ClearingPartID_get, _thostmduserapi.CThostFtdcErrorConditionalOrderField_ClearingPartID_set)
    SequenceNo = property(_thostmduserapi.CThostFtdcErrorConditionalOrderField_SequenceNo_get, _thostmduserapi.CThostFtdcErrorConditionalOrderField_SequenceNo_set)
    FrontID = property(_thostmduserapi.CThostFtdcErrorConditionalOrderField_FrontID_get, _thostmduserapi.CThostFtdcErrorConditionalOrderField_FrontID_set)
    SessionID = property(_thostmduserapi.CThostFtdcErrorConditionalOrderField_SessionID_get, _thostmduserapi.CThostFtdcErrorConditionalOrderField_SessionID_set)
    UserProductInfo = property(_thostmduserapi.CThostFtdcErrorConditionalOrderField_UserProductInfo_get, _thostmduserapi.CThostFtdcErrorConditionalOrderField_UserProductInfo_set)
    StatusMsg = property(_thostmduserapi.CThostFtdcErrorConditionalOrderField_StatusMsg_get, _thostmduserapi.CThostFtdcErrorConditionalOrderField_StatusMsg_set)
    UserForceClose = property(_thostmduserapi.CThostFtdcErrorConditionalOrderField_UserForceClose_get, _thostmduserapi.CThostFtdcErrorConditionalOrderField_UserForceClose_set)
    ActiveUserID = property(_thostmduserapi.CThostFtdcErrorConditionalOrderField_ActiveUserID_get, _thostmduserapi.CThostFtdcErrorConditionalOrderField_ActiveUserID_set)
    BrokerOrderSeq = property(_thostmduserapi.CThostFtdcErrorConditionalOrderField_BrokerOrderSeq_get, _thostmduserapi.CThostFtdcErrorConditionalOrderField_BrokerOrderSeq_set)
    RelativeOrderSysID = property(_thostmduserapi.CThostFtdcErrorConditionalOrderField_RelativeOrderSysID_get, _thostmduserapi.CThostFtdcErrorConditionalOrderField_RelativeOrderSysID_set)
    ZCETotalTradedVolume = property(_thostmduserapi.CThostFtdcErrorConditionalOrderField_ZCETotalTradedVolume_get, _thostmduserapi.CThostFtdcErrorConditionalOrderField_ZCETotalTradedVolume_set)
    ErrorID = property(_thostmduserapi.CThostFtdcErrorConditionalOrderField_ErrorID_get, _thostmduserapi.CThostFtdcErrorConditionalOrderField_ErrorID_set)
    ErrorMsg = property(_thostmduserapi.CThostFtdcErrorConditionalOrderField_ErrorMsg_get, _thostmduserapi.CThostFtdcErrorConditionalOrderField_ErrorMsg_set)
    IsSwapOrder = property(_thostmduserapi.CThostFtdcErrorConditionalOrderField_IsSwapOrder_get, _thostmduserapi.CThostFtdcErrorConditionalOrderField_IsSwapOrder_set)
    BranchID = property(_thostmduserapi.CThostFtdcErrorConditionalOrderField_BranchID_get, _thostmduserapi.CThostFtdcErrorConditionalOrderField_BranchID_set)
    InvestUnitID = property(_thostmduserapi.CThostFtdcErrorConditionalOrderField_InvestUnitID_get, _thostmduserapi.CThostFtdcErrorConditionalOrderField_InvestUnitID_set)
    AccountID = property(_thostmduserapi.CThostFtdcErrorConditionalOrderField_AccountID_get, _thostmduserapi.CThostFtdcErrorConditionalOrderField_AccountID_set)
    CurrencyID = property(_thostmduserapi.CThostFtdcErrorConditionalOrderField_CurrencyID_get, _thostmduserapi.CThostFtdcErrorConditionalOrderField_CurrencyID_set)
    IPAddress = property(_thostmduserapi.CThostFtdcErrorConditionalOrderField_IPAddress_get, _thostmduserapi.CThostFtdcErrorConditionalOrderField_IPAddress_set)
    MacAddress = property(_thostmduserapi.CThostFtdcErrorConditionalOrderField_MacAddress_get, _thostmduserapi.CThostFtdcErrorConditionalOrderField_MacAddress_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcErrorConditionalOrderField_swiginit(self, _thostmduserapi.new_CThostFtdcErrorConditionalOrderField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcErrorConditionalOrderField

# Register CThostFtdcErrorConditionalOrderField in _thostmduserapi:
_thostmduserapi.CThostFtdcErrorConditionalOrderField_swigregister(CThostFtdcErrorConditionalOrderField)

class CThostFtdcQryErrOrderActionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQryErrOrderActionField_BrokerID_get, _thostmduserapi.CThostFtdcQryErrOrderActionField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcQryErrOrderActionField_InvestorID_get, _thostmduserapi.CThostFtdcQryErrOrderActionField_InvestorID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryErrOrderActionField_swiginit(self, _thostmduserapi.new_CThostFtdcQryErrOrderActionField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryErrOrderActionField

# Register CThostFtdcQryErrOrderActionField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryErrOrderActionField_swigregister(CThostFtdcQryErrOrderActionField)

class CThostFtdcErrOrderActionField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcErrOrderActionField_BrokerID_get, _thostmduserapi.CThostFtdcErrOrderActionField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcErrOrderActionField_InvestorID_get, _thostmduserapi.CThostFtdcErrOrderActionField_InvestorID_set)
    OrderActionRef = property(_thostmduserapi.CThostFtdcErrOrderActionField_OrderActionRef_get, _thostmduserapi.CThostFtdcErrOrderActionField_OrderActionRef_set)
    OrderRef = property(_thostmduserapi.CThostFtdcErrOrderActionField_OrderRef_get, _thostmduserapi.CThostFtdcErrOrderActionField_OrderRef_set)
    RequestID = property(_thostmduserapi.CThostFtdcErrOrderActionField_RequestID_get, _thostmduserapi.CThostFtdcErrOrderActionField_RequestID_set)
    FrontID = property(_thostmduserapi.CThostFtdcErrOrderActionField_FrontID_get, _thostmduserapi.CThostFtdcErrOrderActionField_FrontID_set)
    SessionID = property(_thostmduserapi.CThostFtdcErrOrderActionField_SessionID_get, _thostmduserapi.CThostFtdcErrOrderActionField_SessionID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcErrOrderActionField_ExchangeID_get, _thostmduserapi.CThostFtdcErrOrderActionField_ExchangeID_set)
    OrderSysID = property(_thostmduserapi.CThostFtdcErrOrderActionField_OrderSysID_get, _thostmduserapi.CThostFtdcErrOrderActionField_OrderSysID_set)
    ActionFlag = property(_thostmduserapi.CThostFtdcErrOrderActionField_ActionFlag_get, _thostmduserapi.CThostFtdcErrOrderActionField_ActionFlag_set)
    LimitPrice = property(_thostmduserapi.CThostFtdcErrOrderActionField_LimitPrice_get, _thostmduserapi.CThostFtdcErrOrderActionField_LimitPrice_set)
    VolumeChange = property(_thostmduserapi.CThostFtdcErrOrderActionField_VolumeChange_get, _thostmduserapi.CThostFtdcErrOrderActionField_VolumeChange_set)
    ActionDate = property(_thostmduserapi.CThostFtdcErrOrderActionField_ActionDate_get, _thostmduserapi.CThostFtdcErrOrderActionField_ActionDate_set)
    ActionTime = property(_thostmduserapi.CThostFtdcErrOrderActionField_ActionTime_get, _thostmduserapi.CThostFtdcErrOrderActionField_ActionTime_set)
    TraderID = property(_thostmduserapi.CThostFtdcErrOrderActionField_TraderID_get, _thostmduserapi.CThostFtdcErrOrderActionField_TraderID_set)
    InstallID = property(_thostmduserapi.CThostFtdcErrOrderActionField_InstallID_get, _thostmduserapi.CThostFtdcErrOrderActionField_InstallID_set)
    OrderLocalID = property(_thostmduserapi.CThostFtdcErrOrderActionField_OrderLocalID_get, _thostmduserapi.CThostFtdcErrOrderActionField_OrderLocalID_set)
    ActionLocalID = property(_thostmduserapi.CThostFtdcErrOrderActionField_ActionLocalID_get, _thostmduserapi.CThostFtdcErrOrderActionField_ActionLocalID_set)
    ParticipantID = property(_thostmduserapi.CThostFtdcErrOrderActionField_ParticipantID_get, _thostmduserapi.CThostFtdcErrOrderActionField_ParticipantID_set)
    ClientID = property(_thostmduserapi.CThostFtdcErrOrderActionField_ClientID_get, _thostmduserapi.CThostFtdcErrOrderActionField_ClientID_set)
    BusinessUnit = property(_thostmduserapi.CThostFtdcErrOrderActionField_BusinessUnit_get, _thostmduserapi.CThostFtdcErrOrderActionField_BusinessUnit_set)
    OrderActionStatus = property(_thostmduserapi.CThostFtdcErrOrderActionField_OrderActionStatus_get, _thostmduserapi.CThostFtdcErrOrderActionField_OrderActionStatus_set)
    UserID = property(_thostmduserapi.CThostFtdcErrOrderActionField_UserID_get, _thostmduserapi.CThostFtdcErrOrderActionField_UserID_set)
    StatusMsg = property(_thostmduserapi.CThostFtdcErrOrderActionField_StatusMsg_get, _thostmduserapi.CThostFtdcErrOrderActionField_StatusMsg_set)
    InstrumentID = property(_thostmduserapi.CThostFtdcErrOrderActionField_InstrumentID_get, _thostmduserapi.CThostFtdcErrOrderActionField_InstrumentID_set)
    BranchID = property(_thostmduserapi.CThostFtdcErrOrderActionField_BranchID_get, _thostmduserapi.CThostFtdcErrOrderActionField_BranchID_set)
    InvestUnitID = property(_thostmduserapi.CThostFtdcErrOrderActionField_InvestUnitID_get, _thostmduserapi.CThostFtdcErrOrderActionField_InvestUnitID_set)
    IPAddress = property(_thostmduserapi.CThostFtdcErrOrderActionField_IPAddress_get, _thostmduserapi.CThostFtdcErrOrderActionField_IPAddress_set)
    MacAddress = property(_thostmduserapi.CThostFtdcErrOrderActionField_MacAddress_get, _thostmduserapi.CThostFtdcErrOrderActionField_MacAddress_set)
    ErrorID = property(_thostmduserapi.CThostFtdcErrOrderActionField_ErrorID_get, _thostmduserapi.CThostFtdcErrOrderActionField_ErrorID_set)
    ErrorMsg = property(_thostmduserapi.CThostFtdcErrOrderActionField_ErrorMsg_get, _thostmduserapi.CThostFtdcErrOrderActionField_ErrorMsg_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcErrOrderActionField_swiginit(self, _thostmduserapi.new_CThostFtdcErrOrderActionField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcErrOrderActionField

# Register CThostFtdcErrOrderActionField in _thostmduserapi:
_thostmduserapi.CThostFtdcErrOrderActionField_swigregister(CThostFtdcErrOrderActionField)

class CThostFtdcQryExchangeSequenceField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ExchangeID = property(_thostmduserapi.CThostFtdcQryExchangeSequenceField_ExchangeID_get, _thostmduserapi.CThostFtdcQryExchangeSequenceField_ExchangeID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryExchangeSequenceField_swiginit(self, _thostmduserapi.new_CThostFtdcQryExchangeSequenceField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryExchangeSequenceField

# Register CThostFtdcQryExchangeSequenceField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryExchangeSequenceField_swigregister(CThostFtdcQryExchangeSequenceField)

class CThostFtdcExchangeSequenceField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ExchangeID = property(_thostmduserapi.CThostFtdcExchangeSequenceField_ExchangeID_get, _thostmduserapi.CThostFtdcExchangeSequenceField_ExchangeID_set)
    SequenceNo = property(_thostmduserapi.CThostFtdcExchangeSequenceField_SequenceNo_get, _thostmduserapi.CThostFtdcExchangeSequenceField_SequenceNo_set)
    MarketStatus = property(_thostmduserapi.CThostFtdcExchangeSequenceField_MarketStatus_get, _thostmduserapi.CThostFtdcExchangeSequenceField_MarketStatus_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcExchangeSequenceField_swiginit(self, _thostmduserapi.new_CThostFtdcExchangeSequenceField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcExchangeSequenceField

# Register CThostFtdcExchangeSequenceField in _thostmduserapi:
_thostmduserapi.CThostFtdcExchangeSequenceField_swigregister(CThostFtdcExchangeSequenceField)

class CThostFtdcQueryMaxOrderVolumeWithPriceField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQueryMaxOrderVolumeWithPriceField_BrokerID_get, _thostmduserapi.CThostFtdcQueryMaxOrderVolumeWithPriceField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcQueryMaxOrderVolumeWithPriceField_InvestorID_get, _thostmduserapi.CThostFtdcQueryMaxOrderVolumeWithPriceField_InvestorID_set)
    InstrumentID = property(_thostmduserapi.CThostFtdcQueryMaxOrderVolumeWithPriceField_InstrumentID_get, _thostmduserapi.CThostFtdcQueryMaxOrderVolumeWithPriceField_InstrumentID_set)
    Direction = property(_thostmduserapi.CThostFtdcQueryMaxOrderVolumeWithPriceField_Direction_get, _thostmduserapi.CThostFtdcQueryMaxOrderVolumeWithPriceField_Direction_set)
    OffsetFlag = property(_thostmduserapi.CThostFtdcQueryMaxOrderVolumeWithPriceField_OffsetFlag_get, _thostmduserapi.CThostFtdcQueryMaxOrderVolumeWithPriceField_OffsetFlag_set)
    HedgeFlag = property(_thostmduserapi.CThostFtdcQueryMaxOrderVolumeWithPriceField_HedgeFlag_get, _thostmduserapi.CThostFtdcQueryMaxOrderVolumeWithPriceField_HedgeFlag_set)
    MaxVolume = property(_thostmduserapi.CThostFtdcQueryMaxOrderVolumeWithPriceField_MaxVolume_get, _thostmduserapi.CThostFtdcQueryMaxOrderVolumeWithPriceField_MaxVolume_set)
    Price = property(_thostmduserapi.CThostFtdcQueryMaxOrderVolumeWithPriceField_Price_get, _thostmduserapi.CThostFtdcQueryMaxOrderVolumeWithPriceField_Price_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcQueryMaxOrderVolumeWithPriceField_ExchangeID_get, _thostmduserapi.CThostFtdcQueryMaxOrderVolumeWithPriceField_ExchangeID_set)
    InvestUnitID = property(_thostmduserapi.CThostFtdcQueryMaxOrderVolumeWithPriceField_InvestUnitID_get, _thostmduserapi.CThostFtdcQueryMaxOrderVolumeWithPriceField_InvestUnitID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQueryMaxOrderVolumeWithPriceField_swiginit(self, _thostmduserapi.new_CThostFtdcQueryMaxOrderVolumeWithPriceField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQueryMaxOrderVolumeWithPriceField

# Register CThostFtdcQueryMaxOrderVolumeWithPriceField in _thostmduserapi:
_thostmduserapi.CThostFtdcQueryMaxOrderVolumeWithPriceField_swigregister(CThostFtdcQueryMaxOrderVolumeWithPriceField)

class CThostFtdcQryBrokerTradingParamsField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQryBrokerTradingParamsField_BrokerID_get, _thostmduserapi.CThostFtdcQryBrokerTradingParamsField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcQryBrokerTradingParamsField_InvestorID_get, _thostmduserapi.CThostFtdcQryBrokerTradingParamsField_InvestorID_set)
    CurrencyID = property(_thostmduserapi.CThostFtdcQryBrokerTradingParamsField_CurrencyID_get, _thostmduserapi.CThostFtdcQryBrokerTradingParamsField_CurrencyID_set)
    AccountID = property(_thostmduserapi.CThostFtdcQryBrokerTradingParamsField_AccountID_get, _thostmduserapi.CThostFtdcQryBrokerTradingParamsField_AccountID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryBrokerTradingParamsField_swiginit(self, _thostmduserapi.new_CThostFtdcQryBrokerTradingParamsField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryBrokerTradingParamsField

# Register CThostFtdcQryBrokerTradingParamsField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryBrokerTradingParamsField_swigregister(CThostFtdcQryBrokerTradingParamsField)

class CThostFtdcBrokerTradingParamsField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcBrokerTradingParamsField_BrokerID_get, _thostmduserapi.CThostFtdcBrokerTradingParamsField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcBrokerTradingParamsField_InvestorID_get, _thostmduserapi.CThostFtdcBrokerTradingParamsField_InvestorID_set)
    MarginPriceType = property(_thostmduserapi.CThostFtdcBrokerTradingParamsField_MarginPriceType_get, _thostmduserapi.CThostFtdcBrokerTradingParamsField_MarginPriceType_set)
    Algorithm = property(_thostmduserapi.CThostFtdcBrokerTradingParamsField_Algorithm_get, _thostmduserapi.CThostFtdcBrokerTradingParamsField_Algorithm_set)
    AvailIncludeCloseProfit = property(_thostmduserapi.CThostFtdcBrokerTradingParamsField_AvailIncludeCloseProfit_get, _thostmduserapi.CThostFtdcBrokerTradingParamsField_AvailIncludeCloseProfit_set)
    CurrencyID = property(_thostmduserapi.CThostFtdcBrokerTradingParamsField_CurrencyID_get, _thostmduserapi.CThostFtdcBrokerTradingParamsField_CurrencyID_set)
    OptionRoyaltyPriceType = property(_thostmduserapi.CThostFtdcBrokerTradingParamsField_OptionRoyaltyPriceType_get, _thostmduserapi.CThostFtdcBrokerTradingParamsField_OptionRoyaltyPriceType_set)
    AccountID = property(_thostmduserapi.CThostFtdcBrokerTradingParamsField_AccountID_get, _thostmduserapi.CThostFtdcBrokerTradingParamsField_AccountID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcBrokerTradingParamsField_swiginit(self, _thostmduserapi.new_CThostFtdcBrokerTradingParamsField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcBrokerTradingParamsField

# Register CThostFtdcBrokerTradingParamsField in _thostmduserapi:
_thostmduserapi.CThostFtdcBrokerTradingParamsField_swigregister(CThostFtdcBrokerTradingParamsField)

class CThostFtdcQryBrokerTradingAlgosField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQryBrokerTradingAlgosField_BrokerID_get, _thostmduserapi.CThostFtdcQryBrokerTradingAlgosField_BrokerID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcQryBrokerTradingAlgosField_ExchangeID_get, _thostmduserapi.CThostFtdcQryBrokerTradingAlgosField_ExchangeID_set)
    InstrumentID = property(_thostmduserapi.CThostFtdcQryBrokerTradingAlgosField_InstrumentID_get, _thostmduserapi.CThostFtdcQryBrokerTradingAlgosField_InstrumentID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryBrokerTradingAlgosField_swiginit(self, _thostmduserapi.new_CThostFtdcQryBrokerTradingAlgosField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryBrokerTradingAlgosField

# Register CThostFtdcQryBrokerTradingAlgosField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryBrokerTradingAlgosField_swigregister(CThostFtdcQryBrokerTradingAlgosField)

class CThostFtdcBrokerTradingAlgosField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcBrokerTradingAlgosField_BrokerID_get, _thostmduserapi.CThostFtdcBrokerTradingAlgosField_BrokerID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcBrokerTradingAlgosField_ExchangeID_get, _thostmduserapi.CThostFtdcBrokerTradingAlgosField_ExchangeID_set)
    InstrumentID = property(_thostmduserapi.CThostFtdcBrokerTradingAlgosField_InstrumentID_get, _thostmduserapi.CThostFtdcBrokerTradingAlgosField_InstrumentID_set)
    HandlePositionAlgoID = property(_thostmduserapi.CThostFtdcBrokerTradingAlgosField_HandlePositionAlgoID_get, _thostmduserapi.CThostFtdcBrokerTradingAlgosField_HandlePositionAlgoID_set)
    FindMarginRateAlgoID = property(_thostmduserapi.CThostFtdcBrokerTradingAlgosField_FindMarginRateAlgoID_get, _thostmduserapi.CThostFtdcBrokerTradingAlgosField_FindMarginRateAlgoID_set)
    HandleTradingAccountAlgoID = property(_thostmduserapi.CThostFtdcBrokerTradingAlgosField_HandleTradingAccountAlgoID_get, _thostmduserapi.CThostFtdcBrokerTradingAlgosField_HandleTradingAccountAlgoID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcBrokerTradingAlgosField_swiginit(self, _thostmduserapi.new_CThostFtdcBrokerTradingAlgosField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcBrokerTradingAlgosField

# Register CThostFtdcBrokerTradingAlgosField in _thostmduserapi:
_thostmduserapi.CThostFtdcBrokerTradingAlgosField_swigregister(CThostFtdcBrokerTradingAlgosField)

class CThostFtdcQueryBrokerDepositField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQueryBrokerDepositField_BrokerID_get, _thostmduserapi.CThostFtdcQueryBrokerDepositField_BrokerID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcQueryBrokerDepositField_ExchangeID_get, _thostmduserapi.CThostFtdcQueryBrokerDepositField_ExchangeID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQueryBrokerDepositField_swiginit(self, _thostmduserapi.new_CThostFtdcQueryBrokerDepositField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQueryBrokerDepositField

# Register CThostFtdcQueryBrokerDepositField in _thostmduserapi:
_thostmduserapi.CThostFtdcQueryBrokerDepositField_swigregister(CThostFtdcQueryBrokerDepositField)

class CThostFtdcBrokerDepositField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradingDay = property(_thostmduserapi.CThostFtdcBrokerDepositField_TradingDay_get, _thostmduserapi.CThostFtdcBrokerDepositField_TradingDay_set)
    BrokerID = property(_thostmduserapi.CThostFtdcBrokerDepositField_BrokerID_get, _thostmduserapi.CThostFtdcBrokerDepositField_BrokerID_set)
    ParticipantID = property(_thostmduserapi.CThostFtdcBrokerDepositField_ParticipantID_get, _thostmduserapi.CThostFtdcBrokerDepositField_ParticipantID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcBrokerDepositField_ExchangeID_get, _thostmduserapi.CThostFtdcBrokerDepositField_ExchangeID_set)
    PreBalance = property(_thostmduserapi.CThostFtdcBrokerDepositField_PreBalance_get, _thostmduserapi.CThostFtdcBrokerDepositField_PreBalance_set)
    CurrMargin = property(_thostmduserapi.CThostFtdcBrokerDepositField_CurrMargin_get, _thostmduserapi.CThostFtdcBrokerDepositField_CurrMargin_set)
    CloseProfit = property(_thostmduserapi.CThostFtdcBrokerDepositField_CloseProfit_get, _thostmduserapi.CThostFtdcBrokerDepositField_CloseProfit_set)
    Balance = property(_thostmduserapi.CThostFtdcBrokerDepositField_Balance_get, _thostmduserapi.CThostFtdcBrokerDepositField_Balance_set)
    Deposit = property(_thostmduserapi.CThostFtdcBrokerDepositField_Deposit_get, _thostmduserapi.CThostFtdcBrokerDepositField_Deposit_set)
    Withdraw = property(_thostmduserapi.CThostFtdcBrokerDepositField_Withdraw_get, _thostmduserapi.CThostFtdcBrokerDepositField_Withdraw_set)
    Available = property(_thostmduserapi.CThostFtdcBrokerDepositField_Available_get, _thostmduserapi.CThostFtdcBrokerDepositField_Available_set)
    Reserve = property(_thostmduserapi.CThostFtdcBrokerDepositField_Reserve_get, _thostmduserapi.CThostFtdcBrokerDepositField_Reserve_set)
    FrozenMargin = property(_thostmduserapi.CThostFtdcBrokerDepositField_FrozenMargin_get, _thostmduserapi.CThostFtdcBrokerDepositField_FrozenMargin_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcBrokerDepositField_swiginit(self, _thostmduserapi.new_CThostFtdcBrokerDepositField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcBrokerDepositField

# Register CThostFtdcBrokerDepositField in _thostmduserapi:
_thostmduserapi.CThostFtdcBrokerDepositField_swigregister(CThostFtdcBrokerDepositField)

class CThostFtdcQryCFMMCBrokerKeyField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQryCFMMCBrokerKeyField_BrokerID_get, _thostmduserapi.CThostFtdcQryCFMMCBrokerKeyField_BrokerID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryCFMMCBrokerKeyField_swiginit(self, _thostmduserapi.new_CThostFtdcQryCFMMCBrokerKeyField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryCFMMCBrokerKeyField

# Register CThostFtdcQryCFMMCBrokerKeyField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryCFMMCBrokerKeyField_swigregister(CThostFtdcQryCFMMCBrokerKeyField)

class CThostFtdcCFMMCBrokerKeyField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcCFMMCBrokerKeyField_BrokerID_get, _thostmduserapi.CThostFtdcCFMMCBrokerKeyField_BrokerID_set)
    ParticipantID = property(_thostmduserapi.CThostFtdcCFMMCBrokerKeyField_ParticipantID_get, _thostmduserapi.CThostFtdcCFMMCBrokerKeyField_ParticipantID_set)
    CreateDate = property(_thostmduserapi.CThostFtdcCFMMCBrokerKeyField_CreateDate_get, _thostmduserapi.CThostFtdcCFMMCBrokerKeyField_CreateDate_set)
    CreateTime = property(_thostmduserapi.CThostFtdcCFMMCBrokerKeyField_CreateTime_get, _thostmduserapi.CThostFtdcCFMMCBrokerKeyField_CreateTime_set)
    KeyID = property(_thostmduserapi.CThostFtdcCFMMCBrokerKeyField_KeyID_get, _thostmduserapi.CThostFtdcCFMMCBrokerKeyField_KeyID_set)
    CurrentKey = property(_thostmduserapi.CThostFtdcCFMMCBrokerKeyField_CurrentKey_get, _thostmduserapi.CThostFtdcCFMMCBrokerKeyField_CurrentKey_set)
    KeyKind = property(_thostmduserapi.CThostFtdcCFMMCBrokerKeyField_KeyKind_get, _thostmduserapi.CThostFtdcCFMMCBrokerKeyField_KeyKind_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcCFMMCBrokerKeyField_swiginit(self, _thostmduserapi.new_CThostFtdcCFMMCBrokerKeyField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcCFMMCBrokerKeyField

# Register CThostFtdcCFMMCBrokerKeyField in _thostmduserapi:
_thostmduserapi.CThostFtdcCFMMCBrokerKeyField_swigregister(CThostFtdcCFMMCBrokerKeyField)

class CThostFtdcCFMMCTradingAccountKeyField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcCFMMCTradingAccountKeyField_BrokerID_get, _thostmduserapi.CThostFtdcCFMMCTradingAccountKeyField_BrokerID_set)
    ParticipantID = property(_thostmduserapi.CThostFtdcCFMMCTradingAccountKeyField_ParticipantID_get, _thostmduserapi.CThostFtdcCFMMCTradingAccountKeyField_ParticipantID_set)
    AccountID = property(_thostmduserapi.CThostFtdcCFMMCTradingAccountKeyField_AccountID_get, _thostmduserapi.CThostFtdcCFMMCTradingAccountKeyField_AccountID_set)
    KeyID = property(_thostmduserapi.CThostFtdcCFMMCTradingAccountKeyField_KeyID_get, _thostmduserapi.CThostFtdcCFMMCTradingAccountKeyField_KeyID_set)
    CurrentKey = property(_thostmduserapi.CThostFtdcCFMMCTradingAccountKeyField_CurrentKey_get, _thostmduserapi.CThostFtdcCFMMCTradingAccountKeyField_CurrentKey_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcCFMMCTradingAccountKeyField_swiginit(self, _thostmduserapi.new_CThostFtdcCFMMCTradingAccountKeyField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcCFMMCTradingAccountKeyField

# Register CThostFtdcCFMMCTradingAccountKeyField in _thostmduserapi:
_thostmduserapi.CThostFtdcCFMMCTradingAccountKeyField_swigregister(CThostFtdcCFMMCTradingAccountKeyField)

class CThostFtdcQryCFMMCTradingAccountKeyField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQryCFMMCTradingAccountKeyField_BrokerID_get, _thostmduserapi.CThostFtdcQryCFMMCTradingAccountKeyField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcQryCFMMCTradingAccountKeyField_InvestorID_get, _thostmduserapi.CThostFtdcQryCFMMCTradingAccountKeyField_InvestorID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryCFMMCTradingAccountKeyField_swiginit(self, _thostmduserapi.new_CThostFtdcQryCFMMCTradingAccountKeyField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryCFMMCTradingAccountKeyField

# Register CThostFtdcQryCFMMCTradingAccountKeyField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryCFMMCTradingAccountKeyField_swigregister(CThostFtdcQryCFMMCTradingAccountKeyField)

class CThostFtdcBrokerUserOTPParamField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcBrokerUserOTPParamField_BrokerID_get, _thostmduserapi.CThostFtdcBrokerUserOTPParamField_BrokerID_set)
    UserID = property(_thostmduserapi.CThostFtdcBrokerUserOTPParamField_UserID_get, _thostmduserapi.CThostFtdcBrokerUserOTPParamField_UserID_set)
    OTPVendorsID = property(_thostmduserapi.CThostFtdcBrokerUserOTPParamField_OTPVendorsID_get, _thostmduserapi.CThostFtdcBrokerUserOTPParamField_OTPVendorsID_set)
    SerialNumber = property(_thostmduserapi.CThostFtdcBrokerUserOTPParamField_SerialNumber_get, _thostmduserapi.CThostFtdcBrokerUserOTPParamField_SerialNumber_set)
    AuthKey = property(_thostmduserapi.CThostFtdcBrokerUserOTPParamField_AuthKey_get, _thostmduserapi.CThostFtdcBrokerUserOTPParamField_AuthKey_set)
    LastDrift = property(_thostmduserapi.CThostFtdcBrokerUserOTPParamField_LastDrift_get, _thostmduserapi.CThostFtdcBrokerUserOTPParamField_LastDrift_set)
    LastSuccess = property(_thostmduserapi.CThostFtdcBrokerUserOTPParamField_LastSuccess_get, _thostmduserapi.CThostFtdcBrokerUserOTPParamField_LastSuccess_set)
    OTPType = property(_thostmduserapi.CThostFtdcBrokerUserOTPParamField_OTPType_get, _thostmduserapi.CThostFtdcBrokerUserOTPParamField_OTPType_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcBrokerUserOTPParamField_swiginit(self, _thostmduserapi.new_CThostFtdcBrokerUserOTPParamField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcBrokerUserOTPParamField

# Register CThostFtdcBrokerUserOTPParamField in _thostmduserapi:
_thostmduserapi.CThostFtdcBrokerUserOTPParamField_swigregister(CThostFtdcBrokerUserOTPParamField)

class CThostFtdcManualSyncBrokerUserOTPField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcManualSyncBrokerUserOTPField_BrokerID_get, _thostmduserapi.CThostFtdcManualSyncBrokerUserOTPField_BrokerID_set)
    UserID = property(_thostmduserapi.CThostFtdcManualSyncBrokerUserOTPField_UserID_get, _thostmduserapi.CThostFtdcManualSyncBrokerUserOTPField_UserID_set)
    OTPType = property(_thostmduserapi.CThostFtdcManualSyncBrokerUserOTPField_OTPType_get, _thostmduserapi.CThostFtdcManualSyncBrokerUserOTPField_OTPType_set)
    FirstOTP = property(_thostmduserapi.CThostFtdcManualSyncBrokerUserOTPField_FirstOTP_get, _thostmduserapi.CThostFtdcManualSyncBrokerUserOTPField_FirstOTP_set)
    SecondOTP = property(_thostmduserapi.CThostFtdcManualSyncBrokerUserOTPField_SecondOTP_get, _thostmduserapi.CThostFtdcManualSyncBrokerUserOTPField_SecondOTP_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcManualSyncBrokerUserOTPField_swiginit(self, _thostmduserapi.new_CThostFtdcManualSyncBrokerUserOTPField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcManualSyncBrokerUserOTPField

# Register CThostFtdcManualSyncBrokerUserOTPField in _thostmduserapi:
_thostmduserapi.CThostFtdcManualSyncBrokerUserOTPField_swigregister(CThostFtdcManualSyncBrokerUserOTPField)

class CThostFtdcCommRateModelField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcCommRateModelField_BrokerID_get, _thostmduserapi.CThostFtdcCommRateModelField_BrokerID_set)
    CommModelID = property(_thostmduserapi.CThostFtdcCommRateModelField_CommModelID_get, _thostmduserapi.CThostFtdcCommRateModelField_CommModelID_set)
    CommModelName = property(_thostmduserapi.CThostFtdcCommRateModelField_CommModelName_get, _thostmduserapi.CThostFtdcCommRateModelField_CommModelName_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcCommRateModelField_swiginit(self, _thostmduserapi.new_CThostFtdcCommRateModelField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcCommRateModelField

# Register CThostFtdcCommRateModelField in _thostmduserapi:
_thostmduserapi.CThostFtdcCommRateModelField_swigregister(CThostFtdcCommRateModelField)

class CThostFtdcQryCommRateModelField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQryCommRateModelField_BrokerID_get, _thostmduserapi.CThostFtdcQryCommRateModelField_BrokerID_set)
    CommModelID = property(_thostmduserapi.CThostFtdcQryCommRateModelField_CommModelID_get, _thostmduserapi.CThostFtdcQryCommRateModelField_CommModelID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryCommRateModelField_swiginit(self, _thostmduserapi.new_CThostFtdcQryCommRateModelField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryCommRateModelField

# Register CThostFtdcQryCommRateModelField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryCommRateModelField_swigregister(CThostFtdcQryCommRateModelField)

class CThostFtdcMarginModelField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcMarginModelField_BrokerID_get, _thostmduserapi.CThostFtdcMarginModelField_BrokerID_set)
    MarginModelID = property(_thostmduserapi.CThostFtdcMarginModelField_MarginModelID_get, _thostmduserapi.CThostFtdcMarginModelField_MarginModelID_set)
    MarginModelName = property(_thostmduserapi.CThostFtdcMarginModelField_MarginModelName_get, _thostmduserapi.CThostFtdcMarginModelField_MarginModelName_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcMarginModelField_swiginit(self, _thostmduserapi.new_CThostFtdcMarginModelField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcMarginModelField

# Register CThostFtdcMarginModelField in _thostmduserapi:
_thostmduserapi.CThostFtdcMarginModelField_swigregister(CThostFtdcMarginModelField)

class CThostFtdcQryMarginModelField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQryMarginModelField_BrokerID_get, _thostmduserapi.CThostFtdcQryMarginModelField_BrokerID_set)
    MarginModelID = property(_thostmduserapi.CThostFtdcQryMarginModelField_MarginModelID_get, _thostmduserapi.CThostFtdcQryMarginModelField_MarginModelID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryMarginModelField_swiginit(self, _thostmduserapi.new_CThostFtdcQryMarginModelField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryMarginModelField

# Register CThostFtdcQryMarginModelField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryMarginModelField_swigregister(CThostFtdcQryMarginModelField)

class CThostFtdcEWarrantOffsetField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradingDay = property(_thostmduserapi.CThostFtdcEWarrantOffsetField_TradingDay_get, _thostmduserapi.CThostFtdcEWarrantOffsetField_TradingDay_set)
    BrokerID = property(_thostmduserapi.CThostFtdcEWarrantOffsetField_BrokerID_get, _thostmduserapi.CThostFtdcEWarrantOffsetField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcEWarrantOffsetField_InvestorID_get, _thostmduserapi.CThostFtdcEWarrantOffsetField_InvestorID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcEWarrantOffsetField_ExchangeID_get, _thostmduserapi.CThostFtdcEWarrantOffsetField_ExchangeID_set)
    InstrumentID = property(_thostmduserapi.CThostFtdcEWarrantOffsetField_InstrumentID_get, _thostmduserapi.CThostFtdcEWarrantOffsetField_InstrumentID_set)
    Direction = property(_thostmduserapi.CThostFtdcEWarrantOffsetField_Direction_get, _thostmduserapi.CThostFtdcEWarrantOffsetField_Direction_set)
    HedgeFlag = property(_thostmduserapi.CThostFtdcEWarrantOffsetField_HedgeFlag_get, _thostmduserapi.CThostFtdcEWarrantOffsetField_HedgeFlag_set)
    Volume = property(_thostmduserapi.CThostFtdcEWarrantOffsetField_Volume_get, _thostmduserapi.CThostFtdcEWarrantOffsetField_Volume_set)
    InvestUnitID = property(_thostmduserapi.CThostFtdcEWarrantOffsetField_InvestUnitID_get, _thostmduserapi.CThostFtdcEWarrantOffsetField_InvestUnitID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcEWarrantOffsetField_swiginit(self, _thostmduserapi.new_CThostFtdcEWarrantOffsetField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcEWarrantOffsetField

# Register CThostFtdcEWarrantOffsetField in _thostmduserapi:
_thostmduserapi.CThostFtdcEWarrantOffsetField_swigregister(CThostFtdcEWarrantOffsetField)

class CThostFtdcQryEWarrantOffsetField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQryEWarrantOffsetField_BrokerID_get, _thostmduserapi.CThostFtdcQryEWarrantOffsetField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcQryEWarrantOffsetField_InvestorID_get, _thostmduserapi.CThostFtdcQryEWarrantOffsetField_InvestorID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcQryEWarrantOffsetField_ExchangeID_get, _thostmduserapi.CThostFtdcQryEWarrantOffsetField_ExchangeID_set)
    InstrumentID = property(_thostmduserapi.CThostFtdcQryEWarrantOffsetField_InstrumentID_get, _thostmduserapi.CThostFtdcQryEWarrantOffsetField_InstrumentID_set)
    InvestUnitID = property(_thostmduserapi.CThostFtdcQryEWarrantOffsetField_InvestUnitID_get, _thostmduserapi.CThostFtdcQryEWarrantOffsetField_InvestUnitID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryEWarrantOffsetField_swiginit(self, _thostmduserapi.new_CThostFtdcQryEWarrantOffsetField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryEWarrantOffsetField

# Register CThostFtdcQryEWarrantOffsetField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryEWarrantOffsetField_swigregister(CThostFtdcQryEWarrantOffsetField)

class CThostFtdcQryInvestorProductGroupMarginField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQryInvestorProductGroupMarginField_BrokerID_get, _thostmduserapi.CThostFtdcQryInvestorProductGroupMarginField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcQryInvestorProductGroupMarginField_InvestorID_get, _thostmduserapi.CThostFtdcQryInvestorProductGroupMarginField_InvestorID_set)
    ProductGroupID = property(_thostmduserapi.CThostFtdcQryInvestorProductGroupMarginField_ProductGroupID_get, _thostmduserapi.CThostFtdcQryInvestorProductGroupMarginField_ProductGroupID_set)
    HedgeFlag = property(_thostmduserapi.CThostFtdcQryInvestorProductGroupMarginField_HedgeFlag_get, _thostmduserapi.CThostFtdcQryInvestorProductGroupMarginField_HedgeFlag_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcQryInvestorProductGroupMarginField_ExchangeID_get, _thostmduserapi.CThostFtdcQryInvestorProductGroupMarginField_ExchangeID_set)
    InvestUnitID = property(_thostmduserapi.CThostFtdcQryInvestorProductGroupMarginField_InvestUnitID_get, _thostmduserapi.CThostFtdcQryInvestorProductGroupMarginField_InvestUnitID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryInvestorProductGroupMarginField_swiginit(self, _thostmduserapi.new_CThostFtdcQryInvestorProductGroupMarginField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryInvestorProductGroupMarginField

# Register CThostFtdcQryInvestorProductGroupMarginField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryInvestorProductGroupMarginField_swigregister(CThostFtdcQryInvestorProductGroupMarginField)

class CThostFtdcInvestorProductGroupMarginField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ProductGroupID = property(_thostmduserapi.CThostFtdcInvestorProductGroupMarginField_ProductGroupID_get, _thostmduserapi.CThostFtdcInvestorProductGroupMarginField_ProductGroupID_set)
    BrokerID = property(_thostmduserapi.CThostFtdcInvestorProductGroupMarginField_BrokerID_get, _thostmduserapi.CThostFtdcInvestorProductGroupMarginField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcInvestorProductGroupMarginField_InvestorID_get, _thostmduserapi.CThostFtdcInvestorProductGroupMarginField_InvestorID_set)
    TradingDay = property(_thostmduserapi.CThostFtdcInvestorProductGroupMarginField_TradingDay_get, _thostmduserapi.CThostFtdcInvestorProductGroupMarginField_TradingDay_set)
    SettlementID = property(_thostmduserapi.CThostFtdcInvestorProductGroupMarginField_SettlementID_get, _thostmduserapi.CThostFtdcInvestorProductGroupMarginField_SettlementID_set)
    FrozenMargin = property(_thostmduserapi.CThostFtdcInvestorProductGroupMarginField_FrozenMargin_get, _thostmduserapi.CThostFtdcInvestorProductGroupMarginField_FrozenMargin_set)
    LongFrozenMargin = property(_thostmduserapi.CThostFtdcInvestorProductGroupMarginField_LongFrozenMargin_get, _thostmduserapi.CThostFtdcInvestorProductGroupMarginField_LongFrozenMargin_set)
    ShortFrozenMargin = property(_thostmduserapi.CThostFtdcInvestorProductGroupMarginField_ShortFrozenMargin_get, _thostmduserapi.CThostFtdcInvestorProductGroupMarginField_ShortFrozenMargin_set)
    UseMargin = property(_thostmduserapi.CThostFtdcInvestorProductGroupMarginField_UseMargin_get, _thostmduserapi.CThostFtdcInvestorProductGroupMarginField_UseMargin_set)
    LongUseMargin = property(_thostmduserapi.CThostFtdcInvestorProductGroupMarginField_LongUseMargin_get, _thostmduserapi.CThostFtdcInvestorProductGroupMarginField_LongUseMargin_set)
    ShortUseMargin = property(_thostmduserapi.CThostFtdcInvestorProductGroupMarginField_ShortUseMargin_get, _thostmduserapi.CThostFtdcInvestorProductGroupMarginField_ShortUseMargin_set)
    ExchMargin = property(_thostmduserapi.CThostFtdcInvestorProductGroupMarginField_ExchMargin_get, _thostmduserapi.CThostFtdcInvestorProductGroupMarginField_ExchMargin_set)
    LongExchMargin = property(_thostmduserapi.CThostFtdcInvestorProductGroupMarginField_LongExchMargin_get, _thostmduserapi.CThostFtdcInvestorProductGroupMarginField_LongExchMargin_set)
    ShortExchMargin = property(_thostmduserapi.CThostFtdcInvestorProductGroupMarginField_ShortExchMargin_get, _thostmduserapi.CThostFtdcInvestorProductGroupMarginField_ShortExchMargin_set)
    CloseProfit = property(_thostmduserapi.CThostFtdcInvestorProductGroupMarginField_CloseProfit_get, _thostmduserapi.CThostFtdcInvestorProductGroupMarginField_CloseProfit_set)
    FrozenCommission = property(_thostmduserapi.CThostFtdcInvestorProductGroupMarginField_FrozenCommission_get, _thostmduserapi.CThostFtdcInvestorProductGroupMarginField_FrozenCommission_set)
    Commission = property(_thostmduserapi.CThostFtdcInvestorProductGroupMarginField_Commission_get, _thostmduserapi.CThostFtdcInvestorProductGroupMarginField_Commission_set)
    FrozenCash = property(_thostmduserapi.CThostFtdcInvestorProductGroupMarginField_FrozenCash_get, _thostmduserapi.CThostFtdcInvestorProductGroupMarginField_FrozenCash_set)
    CashIn = property(_thostmduserapi.CThostFtdcInvestorProductGroupMarginField_CashIn_get, _thostmduserapi.CThostFtdcInvestorProductGroupMarginField_CashIn_set)
    PositionProfit = property(_thostmduserapi.CThostFtdcInvestorProductGroupMarginField_PositionProfit_get, _thostmduserapi.CThostFtdcInvestorProductGroupMarginField_PositionProfit_set)
    OffsetAmount = property(_thostmduserapi.CThostFtdcInvestorProductGroupMarginField_OffsetAmount_get, _thostmduserapi.CThostFtdcInvestorProductGroupMarginField_OffsetAmount_set)
    LongOffsetAmount = property(_thostmduserapi.CThostFtdcInvestorProductGroupMarginField_LongOffsetAmount_get, _thostmduserapi.CThostFtdcInvestorProductGroupMarginField_LongOffsetAmount_set)
    ShortOffsetAmount = property(_thostmduserapi.CThostFtdcInvestorProductGroupMarginField_ShortOffsetAmount_get, _thostmduserapi.CThostFtdcInvestorProductGroupMarginField_ShortOffsetAmount_set)
    ExchOffsetAmount = property(_thostmduserapi.CThostFtdcInvestorProductGroupMarginField_ExchOffsetAmount_get, _thostmduserapi.CThostFtdcInvestorProductGroupMarginField_ExchOffsetAmount_set)
    LongExchOffsetAmount = property(_thostmduserapi.CThostFtdcInvestorProductGroupMarginField_LongExchOffsetAmount_get, _thostmduserapi.CThostFtdcInvestorProductGroupMarginField_LongExchOffsetAmount_set)
    ShortExchOffsetAmount = property(_thostmduserapi.CThostFtdcInvestorProductGroupMarginField_ShortExchOffsetAmount_get, _thostmduserapi.CThostFtdcInvestorProductGroupMarginField_ShortExchOffsetAmount_set)
    HedgeFlag = property(_thostmduserapi.CThostFtdcInvestorProductGroupMarginField_HedgeFlag_get, _thostmduserapi.CThostFtdcInvestorProductGroupMarginField_HedgeFlag_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcInvestorProductGroupMarginField_ExchangeID_get, _thostmduserapi.CThostFtdcInvestorProductGroupMarginField_ExchangeID_set)
    InvestUnitID = property(_thostmduserapi.CThostFtdcInvestorProductGroupMarginField_InvestUnitID_get, _thostmduserapi.CThostFtdcInvestorProductGroupMarginField_InvestUnitID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcInvestorProductGroupMarginField_swiginit(self, _thostmduserapi.new_CThostFtdcInvestorProductGroupMarginField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcInvestorProductGroupMarginField

# Register CThostFtdcInvestorProductGroupMarginField in _thostmduserapi:
_thostmduserapi.CThostFtdcInvestorProductGroupMarginField_swigregister(CThostFtdcInvestorProductGroupMarginField)

class CThostFtdcQueryCFMMCTradingAccountTokenField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQueryCFMMCTradingAccountTokenField_BrokerID_get, _thostmduserapi.CThostFtdcQueryCFMMCTradingAccountTokenField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcQueryCFMMCTradingAccountTokenField_InvestorID_get, _thostmduserapi.CThostFtdcQueryCFMMCTradingAccountTokenField_InvestorID_set)
    InvestUnitID = property(_thostmduserapi.CThostFtdcQueryCFMMCTradingAccountTokenField_InvestUnitID_get, _thostmduserapi.CThostFtdcQueryCFMMCTradingAccountTokenField_InvestUnitID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQueryCFMMCTradingAccountTokenField_swiginit(self, _thostmduserapi.new_CThostFtdcQueryCFMMCTradingAccountTokenField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQueryCFMMCTradingAccountTokenField

# Register CThostFtdcQueryCFMMCTradingAccountTokenField in _thostmduserapi:
_thostmduserapi.CThostFtdcQueryCFMMCTradingAccountTokenField_swigregister(CThostFtdcQueryCFMMCTradingAccountTokenField)

class CThostFtdcCFMMCTradingAccountTokenField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcCFMMCTradingAccountTokenField_BrokerID_get, _thostmduserapi.CThostFtdcCFMMCTradingAccountTokenField_BrokerID_set)
    ParticipantID = property(_thostmduserapi.CThostFtdcCFMMCTradingAccountTokenField_ParticipantID_get, _thostmduserapi.CThostFtdcCFMMCTradingAccountTokenField_ParticipantID_set)
    AccountID = property(_thostmduserapi.CThostFtdcCFMMCTradingAccountTokenField_AccountID_get, _thostmduserapi.CThostFtdcCFMMCTradingAccountTokenField_AccountID_set)
    KeyID = property(_thostmduserapi.CThostFtdcCFMMCTradingAccountTokenField_KeyID_get, _thostmduserapi.CThostFtdcCFMMCTradingAccountTokenField_KeyID_set)
    Token = property(_thostmduserapi.CThostFtdcCFMMCTradingAccountTokenField_Token_get, _thostmduserapi.CThostFtdcCFMMCTradingAccountTokenField_Token_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcCFMMCTradingAccountTokenField_swiginit(self, _thostmduserapi.new_CThostFtdcCFMMCTradingAccountTokenField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcCFMMCTradingAccountTokenField

# Register CThostFtdcCFMMCTradingAccountTokenField in _thostmduserapi:
_thostmduserapi.CThostFtdcCFMMCTradingAccountTokenField_swigregister(CThostFtdcCFMMCTradingAccountTokenField)

class CThostFtdcQryProductGroupField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ProductID = property(_thostmduserapi.CThostFtdcQryProductGroupField_ProductID_get, _thostmduserapi.CThostFtdcQryProductGroupField_ProductID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcQryProductGroupField_ExchangeID_get, _thostmduserapi.CThostFtdcQryProductGroupField_ExchangeID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryProductGroupField_swiginit(self, _thostmduserapi.new_CThostFtdcQryProductGroupField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryProductGroupField

# Register CThostFtdcQryProductGroupField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryProductGroupField_swigregister(CThostFtdcQryProductGroupField)

class CThostFtdcProductGroupField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ProductID = property(_thostmduserapi.CThostFtdcProductGroupField_ProductID_get, _thostmduserapi.CThostFtdcProductGroupField_ProductID_set)
    ExchangeID = property(_thostmduserapi.CThostFtdcProductGroupField_ExchangeID_get, _thostmduserapi.CThostFtdcProductGroupField_ExchangeID_set)
    ProductGroupID = property(_thostmduserapi.CThostFtdcProductGroupField_ProductGroupID_get, _thostmduserapi.CThostFtdcProductGroupField_ProductGroupID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcProductGroupField_swiginit(self, _thostmduserapi.new_CThostFtdcProductGroupField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcProductGroupField

# Register CThostFtdcProductGroupField in _thostmduserapi:
_thostmduserapi.CThostFtdcProductGroupField_swigregister(CThostFtdcProductGroupField)

class CThostFtdcBulletinField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ExchangeID = property(_thostmduserapi.CThostFtdcBulletinField_ExchangeID_get, _thostmduserapi.CThostFtdcBulletinField_ExchangeID_set)
    TradingDay = property(_thostmduserapi.CThostFtdcBulletinField_TradingDay_get, _thostmduserapi.CThostFtdcBulletinField_TradingDay_set)
    BulletinID = property(_thostmduserapi.CThostFtdcBulletinField_BulletinID_get, _thostmduserapi.CThostFtdcBulletinField_BulletinID_set)
    SequenceNo = property(_thostmduserapi.CThostFtdcBulletinField_SequenceNo_get, _thostmduserapi.CThostFtdcBulletinField_SequenceNo_set)
    NewsType = property(_thostmduserapi.CThostFtdcBulletinField_NewsType_get, _thostmduserapi.CThostFtdcBulletinField_NewsType_set)
    NewsUrgency = property(_thostmduserapi.CThostFtdcBulletinField_NewsUrgency_get, _thostmduserapi.CThostFtdcBulletinField_NewsUrgency_set)
    SendTime = property(_thostmduserapi.CThostFtdcBulletinField_SendTime_get, _thostmduserapi.CThostFtdcBulletinField_SendTime_set)
    Abstract = property(_thostmduserapi.CThostFtdcBulletinField_Abstract_get, _thostmduserapi.CThostFtdcBulletinField_Abstract_set)
    ComeFrom = property(_thostmduserapi.CThostFtdcBulletinField_ComeFrom_get, _thostmduserapi.CThostFtdcBulletinField_ComeFrom_set)
    Content = property(_thostmduserapi.CThostFtdcBulletinField_Content_get, _thostmduserapi.CThostFtdcBulletinField_Content_set)
    URLLink = property(_thostmduserapi.CThostFtdcBulletinField_URLLink_get, _thostmduserapi.CThostFtdcBulletinField_URLLink_set)
    MarketID = property(_thostmduserapi.CThostFtdcBulletinField_MarketID_get, _thostmduserapi.CThostFtdcBulletinField_MarketID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcBulletinField_swiginit(self, _thostmduserapi.new_CThostFtdcBulletinField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcBulletinField

# Register CThostFtdcBulletinField in _thostmduserapi:
_thostmduserapi.CThostFtdcBulletinField_swigregister(CThostFtdcBulletinField)

class CThostFtdcQryBulletinField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ExchangeID = property(_thostmduserapi.CThostFtdcQryBulletinField_ExchangeID_get, _thostmduserapi.CThostFtdcQryBulletinField_ExchangeID_set)
    BulletinID = property(_thostmduserapi.CThostFtdcQryBulletinField_BulletinID_get, _thostmduserapi.CThostFtdcQryBulletinField_BulletinID_set)
    SequenceNo = property(_thostmduserapi.CThostFtdcQryBulletinField_SequenceNo_get, _thostmduserapi.CThostFtdcQryBulletinField_SequenceNo_set)
    NewsType = property(_thostmduserapi.CThostFtdcQryBulletinField_NewsType_get, _thostmduserapi.CThostFtdcQryBulletinField_NewsType_set)
    NewsUrgency = property(_thostmduserapi.CThostFtdcQryBulletinField_NewsUrgency_get, _thostmduserapi.CThostFtdcQryBulletinField_NewsUrgency_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryBulletinField_swiginit(self, _thostmduserapi.new_CThostFtdcQryBulletinField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryBulletinField

# Register CThostFtdcQryBulletinField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryBulletinField_swigregister(CThostFtdcQryBulletinField)

class CThostFtdcReqOpenAccountField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradeCode = property(_thostmduserapi.CThostFtdcReqOpenAccountField_TradeCode_get, _thostmduserapi.CThostFtdcReqOpenAccountField_TradeCode_set)
    BankID = property(_thostmduserapi.CThostFtdcReqOpenAccountField_BankID_get, _thostmduserapi.CThostFtdcReqOpenAccountField_BankID_set)
    BankBranchID = property(_thostmduserapi.CThostFtdcReqOpenAccountField_BankBranchID_get, _thostmduserapi.CThostFtdcReqOpenAccountField_BankBranchID_set)
    BrokerID = property(_thostmduserapi.CThostFtdcReqOpenAccountField_BrokerID_get, _thostmduserapi.CThostFtdcReqOpenAccountField_BrokerID_set)
    BrokerBranchID = property(_thostmduserapi.CThostFtdcReqOpenAccountField_BrokerBranchID_get, _thostmduserapi.CThostFtdcReqOpenAccountField_BrokerBranchID_set)
    TradeDate = property(_thostmduserapi.CThostFtdcReqOpenAccountField_TradeDate_get, _thostmduserapi.CThostFtdcReqOpenAccountField_TradeDate_set)
    TradeTime = property(_thostmduserapi.CThostFtdcReqOpenAccountField_TradeTime_get, _thostmduserapi.CThostFtdcReqOpenAccountField_TradeTime_set)
    BankSerial = property(_thostmduserapi.CThostFtdcReqOpenAccountField_BankSerial_get, _thostmduserapi.CThostFtdcReqOpenAccountField_BankSerial_set)
    TradingDay = property(_thostmduserapi.CThostFtdcReqOpenAccountField_TradingDay_get, _thostmduserapi.CThostFtdcReqOpenAccountField_TradingDay_set)
    PlateSerial = property(_thostmduserapi.CThostFtdcReqOpenAccountField_PlateSerial_get, _thostmduserapi.CThostFtdcReqOpenAccountField_PlateSerial_set)
    LastFragment = property(_thostmduserapi.CThostFtdcReqOpenAccountField_LastFragment_get, _thostmduserapi.CThostFtdcReqOpenAccountField_LastFragment_set)
    SessionID = property(_thostmduserapi.CThostFtdcReqOpenAccountField_SessionID_get, _thostmduserapi.CThostFtdcReqOpenAccountField_SessionID_set)
    CustomerName = property(_thostmduserapi.CThostFtdcReqOpenAccountField_CustomerName_get, _thostmduserapi.CThostFtdcReqOpenAccountField_CustomerName_set)
    IdCardType = property(_thostmduserapi.CThostFtdcReqOpenAccountField_IdCardType_get, _thostmduserapi.CThostFtdcReqOpenAccountField_IdCardType_set)
    IdentifiedCardNo = property(_thostmduserapi.CThostFtdcReqOpenAccountField_IdentifiedCardNo_get, _thostmduserapi.CThostFtdcReqOpenAccountField_IdentifiedCardNo_set)
    Gender = property(_thostmduserapi.CThostFtdcReqOpenAccountField_Gender_get, _thostmduserapi.CThostFtdcReqOpenAccountField_Gender_set)
    CountryCode = property(_thostmduserapi.CThostFtdcReqOpenAccountField_CountryCode_get, _thostmduserapi.CThostFtdcReqOpenAccountField_CountryCode_set)
    CustType = property(_thostmduserapi.CThostFtdcReqOpenAccountField_CustType_get, _thostmduserapi.CThostFtdcReqOpenAccountField_CustType_set)
    Address = property(_thostmduserapi.CThostFtdcReqOpenAccountField_Address_get, _thostmduserapi.CThostFtdcReqOpenAccountField_Address_set)
    ZipCode = property(_thostmduserapi.CThostFtdcReqOpenAccountField_ZipCode_get, _thostmduserapi.CThostFtdcReqOpenAccountField_ZipCode_set)
    Telephone = property(_thostmduserapi.CThostFtdcReqOpenAccountField_Telephone_get, _thostmduserapi.CThostFtdcReqOpenAccountField_Telephone_set)
    MobilePhone = property(_thostmduserapi.CThostFtdcReqOpenAccountField_MobilePhone_get, _thostmduserapi.CThostFtdcReqOpenAccountField_MobilePhone_set)
    Fax = property(_thostmduserapi.CThostFtdcReqOpenAccountField_Fax_get, _thostmduserapi.CThostFtdcReqOpenAccountField_Fax_set)
    EMail = property(_thostmduserapi.CThostFtdcReqOpenAccountField_EMail_get, _thostmduserapi.CThostFtdcReqOpenAccountField_EMail_set)
    MoneyAccountStatus = property(_thostmduserapi.CThostFtdcReqOpenAccountField_MoneyAccountStatus_get, _thostmduserapi.CThostFtdcReqOpenAccountField_MoneyAccountStatus_set)
    BankAccount = property(_thostmduserapi.CThostFtdcReqOpenAccountField_BankAccount_get, _thostmduserapi.CThostFtdcReqOpenAccountField_BankAccount_set)
    BankPassWord = property(_thostmduserapi.CThostFtdcReqOpenAccountField_BankPassWord_get, _thostmduserapi.CThostFtdcReqOpenAccountField_BankPassWord_set)
    AccountID = property(_thostmduserapi.CThostFtdcReqOpenAccountField_AccountID_get, _thostmduserapi.CThostFtdcReqOpenAccountField_AccountID_set)
    Password = property(_thostmduserapi.CThostFtdcReqOpenAccountField_Password_get, _thostmduserapi.CThostFtdcReqOpenAccountField_Password_set)
    InstallID = property(_thostmduserapi.CThostFtdcReqOpenAccountField_InstallID_get, _thostmduserapi.CThostFtdcReqOpenAccountField_InstallID_set)
    VerifyCertNoFlag = property(_thostmduserapi.CThostFtdcReqOpenAccountField_VerifyCertNoFlag_get, _thostmduserapi.CThostFtdcReqOpenAccountField_VerifyCertNoFlag_set)
    CurrencyID = property(_thostmduserapi.CThostFtdcReqOpenAccountField_CurrencyID_get, _thostmduserapi.CThostFtdcReqOpenAccountField_CurrencyID_set)
    CashExchangeCode = property(_thostmduserapi.CThostFtdcReqOpenAccountField_CashExchangeCode_get, _thostmduserapi.CThostFtdcReqOpenAccountField_CashExchangeCode_set)
    Digest = property(_thostmduserapi.CThostFtdcReqOpenAccountField_Digest_get, _thostmduserapi.CThostFtdcReqOpenAccountField_Digest_set)
    BankAccType = property(_thostmduserapi.CThostFtdcReqOpenAccountField_BankAccType_get, _thostmduserapi.CThostFtdcReqOpenAccountField_BankAccType_set)
    DeviceID = property(_thostmduserapi.CThostFtdcReqOpenAccountField_DeviceID_get, _thostmduserapi.CThostFtdcReqOpenAccountField_DeviceID_set)
    BankSecuAccType = property(_thostmduserapi.CThostFtdcReqOpenAccountField_BankSecuAccType_get, _thostmduserapi.CThostFtdcReqOpenAccountField_BankSecuAccType_set)
    BrokerIDByBank = property(_thostmduserapi.CThostFtdcReqOpenAccountField_BrokerIDByBank_get, _thostmduserapi.CThostFtdcReqOpenAccountField_BrokerIDByBank_set)
    BankSecuAcc = property(_thostmduserapi.CThostFtdcReqOpenAccountField_BankSecuAcc_get, _thostmduserapi.CThostFtdcReqOpenAccountField_BankSecuAcc_set)
    BankPwdFlag = property(_thostmduserapi.CThostFtdcReqOpenAccountField_BankPwdFlag_get, _thostmduserapi.CThostFtdcReqOpenAccountField_BankPwdFlag_set)
    SecuPwdFlag = property(_thostmduserapi.CThostFtdcReqOpenAccountField_SecuPwdFlag_get, _thostmduserapi.CThostFtdcReqOpenAccountField_SecuPwdFlag_set)
    OperNo = property(_thostmduserapi.CThostFtdcReqOpenAccountField_OperNo_get, _thostmduserapi.CThostFtdcReqOpenAccountField_OperNo_set)
    TID = property(_thostmduserapi.CThostFtdcReqOpenAccountField_TID_get, _thostmduserapi.CThostFtdcReqOpenAccountField_TID_set)
    UserID = property(_thostmduserapi.CThostFtdcReqOpenAccountField_UserID_get, _thostmduserapi.CThostFtdcReqOpenAccountField_UserID_set)
    LongCustomerName = property(_thostmduserapi.CThostFtdcReqOpenAccountField_LongCustomerName_get, _thostmduserapi.CThostFtdcReqOpenAccountField_LongCustomerName_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcReqOpenAccountField_swiginit(self, _thostmduserapi.new_CThostFtdcReqOpenAccountField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcReqOpenAccountField

# Register CThostFtdcReqOpenAccountField in _thostmduserapi:
_thostmduserapi.CThostFtdcReqOpenAccountField_swigregister(CThostFtdcReqOpenAccountField)

class CThostFtdcReqCancelAccountField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradeCode = property(_thostmduserapi.CThostFtdcReqCancelAccountField_TradeCode_get, _thostmduserapi.CThostFtdcReqCancelAccountField_TradeCode_set)
    BankID = property(_thostmduserapi.CThostFtdcReqCancelAccountField_BankID_get, _thostmduserapi.CThostFtdcReqCancelAccountField_BankID_set)
    BankBranchID = property(_thostmduserapi.CThostFtdcReqCancelAccountField_BankBranchID_get, _thostmduserapi.CThostFtdcReqCancelAccountField_BankBranchID_set)
    BrokerID = property(_thostmduserapi.CThostFtdcReqCancelAccountField_BrokerID_get, _thostmduserapi.CThostFtdcReqCancelAccountField_BrokerID_set)
    BrokerBranchID = property(_thostmduserapi.CThostFtdcReqCancelAccountField_BrokerBranchID_get, _thostmduserapi.CThostFtdcReqCancelAccountField_BrokerBranchID_set)
    TradeDate = property(_thostmduserapi.CThostFtdcReqCancelAccountField_TradeDate_get, _thostmduserapi.CThostFtdcReqCancelAccountField_TradeDate_set)
    TradeTime = property(_thostmduserapi.CThostFtdcReqCancelAccountField_TradeTime_get, _thostmduserapi.CThostFtdcReqCancelAccountField_TradeTime_set)
    BankSerial = property(_thostmduserapi.CThostFtdcReqCancelAccountField_BankSerial_get, _thostmduserapi.CThostFtdcReqCancelAccountField_BankSerial_set)
    TradingDay = property(_thostmduserapi.CThostFtdcReqCancelAccountField_TradingDay_get, _thostmduserapi.CThostFtdcReqCancelAccountField_TradingDay_set)
    PlateSerial = property(_thostmduserapi.CThostFtdcReqCancelAccountField_PlateSerial_get, _thostmduserapi.CThostFtdcReqCancelAccountField_PlateSerial_set)
    LastFragment = property(_thostmduserapi.CThostFtdcReqCancelAccountField_LastFragment_get, _thostmduserapi.CThostFtdcReqCancelAccountField_LastFragment_set)
    SessionID = property(_thostmduserapi.CThostFtdcReqCancelAccountField_SessionID_get, _thostmduserapi.CThostFtdcReqCancelAccountField_SessionID_set)
    CustomerName = property(_thostmduserapi.CThostFtdcReqCancelAccountField_CustomerName_get, _thostmduserapi.CThostFtdcReqCancelAccountField_CustomerName_set)
    IdCardType = property(_thostmduserapi.CThostFtdcReqCancelAccountField_IdCardType_get, _thostmduserapi.CThostFtdcReqCancelAccountField_IdCardType_set)
    IdentifiedCardNo = property(_thostmduserapi.CThostFtdcReqCancelAccountField_IdentifiedCardNo_get, _thostmduserapi.CThostFtdcReqCancelAccountField_IdentifiedCardNo_set)
    Gender = property(_thostmduserapi.CThostFtdcReqCancelAccountField_Gender_get, _thostmduserapi.CThostFtdcReqCancelAccountField_Gender_set)
    CountryCode = property(_thostmduserapi.CThostFtdcReqCancelAccountField_CountryCode_get, _thostmduserapi.CThostFtdcReqCancelAccountField_CountryCode_set)
    CustType = property(_thostmduserapi.CThostFtdcReqCancelAccountField_CustType_get, _thostmduserapi.CThostFtdcReqCancelAccountField_CustType_set)
    Address = property(_thostmduserapi.CThostFtdcReqCancelAccountField_Address_get, _thostmduserapi.CThostFtdcReqCancelAccountField_Address_set)
    ZipCode = property(_thostmduserapi.CThostFtdcReqCancelAccountField_ZipCode_get, _thostmduserapi.CThostFtdcReqCancelAccountField_ZipCode_set)
    Telephone = property(_thostmduserapi.CThostFtdcReqCancelAccountField_Telephone_get, _thostmduserapi.CThostFtdcReqCancelAccountField_Telephone_set)
    MobilePhone = property(_thostmduserapi.CThostFtdcReqCancelAccountField_MobilePhone_get, _thostmduserapi.CThostFtdcReqCancelAccountField_MobilePhone_set)
    Fax = property(_thostmduserapi.CThostFtdcReqCancelAccountField_Fax_get, _thostmduserapi.CThostFtdcReqCancelAccountField_Fax_set)
    EMail = property(_thostmduserapi.CThostFtdcReqCancelAccountField_EMail_get, _thostmduserapi.CThostFtdcReqCancelAccountField_EMail_set)
    MoneyAccountStatus = property(_thostmduserapi.CThostFtdcReqCancelAccountField_MoneyAccountStatus_get, _thostmduserapi.CThostFtdcReqCancelAccountField_MoneyAccountStatus_set)
    BankAccount = property(_thostmduserapi.CThostFtdcReqCancelAccountField_BankAccount_get, _thostmduserapi.CThostFtdcReqCancelAccountField_BankAccount_set)
    BankPassWord = property(_thostmduserapi.CThostFtdcReqCancelAccountField_BankPassWord_get, _thostmduserapi.CThostFtdcReqCancelAccountField_BankPassWord_set)
    AccountID = property(_thostmduserapi.CThostFtdcReqCancelAccountField_AccountID_get, _thostmduserapi.CThostFtdcReqCancelAccountField_AccountID_set)
    Password = property(_thostmduserapi.CThostFtdcReqCancelAccountField_Password_get, _thostmduserapi.CThostFtdcReqCancelAccountField_Password_set)
    InstallID = property(_thostmduserapi.CThostFtdcReqCancelAccountField_InstallID_get, _thostmduserapi.CThostFtdcReqCancelAccountField_InstallID_set)
    VerifyCertNoFlag = property(_thostmduserapi.CThostFtdcReqCancelAccountField_VerifyCertNoFlag_get, _thostmduserapi.CThostFtdcReqCancelAccountField_VerifyCertNoFlag_set)
    CurrencyID = property(_thostmduserapi.CThostFtdcReqCancelAccountField_CurrencyID_get, _thostmduserapi.CThostFtdcReqCancelAccountField_CurrencyID_set)
    CashExchangeCode = property(_thostmduserapi.CThostFtdcReqCancelAccountField_CashExchangeCode_get, _thostmduserapi.CThostFtdcReqCancelAccountField_CashExchangeCode_set)
    Digest = property(_thostmduserapi.CThostFtdcReqCancelAccountField_Digest_get, _thostmduserapi.CThostFtdcReqCancelAccountField_Digest_set)
    BankAccType = property(_thostmduserapi.CThostFtdcReqCancelAccountField_BankAccType_get, _thostmduserapi.CThostFtdcReqCancelAccountField_BankAccType_set)
    DeviceID = property(_thostmduserapi.CThostFtdcReqCancelAccountField_DeviceID_get, _thostmduserapi.CThostFtdcReqCancelAccountField_DeviceID_set)
    BankSecuAccType = property(_thostmduserapi.CThostFtdcReqCancelAccountField_BankSecuAccType_get, _thostmduserapi.CThostFtdcReqCancelAccountField_BankSecuAccType_set)
    BrokerIDByBank = property(_thostmduserapi.CThostFtdcReqCancelAccountField_BrokerIDByBank_get, _thostmduserapi.CThostFtdcReqCancelAccountField_BrokerIDByBank_set)
    BankSecuAcc = property(_thostmduserapi.CThostFtdcReqCancelAccountField_BankSecuAcc_get, _thostmduserapi.CThostFtdcReqCancelAccountField_BankSecuAcc_set)
    BankPwdFlag = property(_thostmduserapi.CThostFtdcReqCancelAccountField_BankPwdFlag_get, _thostmduserapi.CThostFtdcReqCancelAccountField_BankPwdFlag_set)
    SecuPwdFlag = property(_thostmduserapi.CThostFtdcReqCancelAccountField_SecuPwdFlag_get, _thostmduserapi.CThostFtdcReqCancelAccountField_SecuPwdFlag_set)
    OperNo = property(_thostmduserapi.CThostFtdcReqCancelAccountField_OperNo_get, _thostmduserapi.CThostFtdcReqCancelAccountField_OperNo_set)
    TID = property(_thostmduserapi.CThostFtdcReqCancelAccountField_TID_get, _thostmduserapi.CThostFtdcReqCancelAccountField_TID_set)
    UserID = property(_thostmduserapi.CThostFtdcReqCancelAccountField_UserID_get, _thostmduserapi.CThostFtdcReqCancelAccountField_UserID_set)
    LongCustomerName = property(_thostmduserapi.CThostFtdcReqCancelAccountField_LongCustomerName_get, _thostmduserapi.CThostFtdcReqCancelAccountField_LongCustomerName_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcReqCancelAccountField_swiginit(self, _thostmduserapi.new_CThostFtdcReqCancelAccountField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcReqCancelAccountField

# Register CThostFtdcReqCancelAccountField in _thostmduserapi:
_thostmduserapi.CThostFtdcReqCancelAccountField_swigregister(CThostFtdcReqCancelAccountField)

class CThostFtdcReqChangeAccountField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradeCode = property(_thostmduserapi.CThostFtdcReqChangeAccountField_TradeCode_get, _thostmduserapi.CThostFtdcReqChangeAccountField_TradeCode_set)
    BankID = property(_thostmduserapi.CThostFtdcReqChangeAccountField_BankID_get, _thostmduserapi.CThostFtdcReqChangeAccountField_BankID_set)
    BankBranchID = property(_thostmduserapi.CThostFtdcReqChangeAccountField_BankBranchID_get, _thostmduserapi.CThostFtdcReqChangeAccountField_BankBranchID_set)
    BrokerID = property(_thostmduserapi.CThostFtdcReqChangeAccountField_BrokerID_get, _thostmduserapi.CThostFtdcReqChangeAccountField_BrokerID_set)
    BrokerBranchID = property(_thostmduserapi.CThostFtdcReqChangeAccountField_BrokerBranchID_get, _thostmduserapi.CThostFtdcReqChangeAccountField_BrokerBranchID_set)
    TradeDate = property(_thostmduserapi.CThostFtdcReqChangeAccountField_TradeDate_get, _thostmduserapi.CThostFtdcReqChangeAccountField_TradeDate_set)
    TradeTime = property(_thostmduserapi.CThostFtdcReqChangeAccountField_TradeTime_get, _thostmduserapi.CThostFtdcReqChangeAccountField_TradeTime_set)
    BankSerial = property(_thostmduserapi.CThostFtdcReqChangeAccountField_BankSerial_get, _thostmduserapi.CThostFtdcReqChangeAccountField_BankSerial_set)
    TradingDay = property(_thostmduserapi.CThostFtdcReqChangeAccountField_TradingDay_get, _thostmduserapi.CThostFtdcReqChangeAccountField_TradingDay_set)
    PlateSerial = property(_thostmduserapi.CThostFtdcReqChangeAccountField_PlateSerial_get, _thostmduserapi.CThostFtdcReqChangeAccountField_PlateSerial_set)
    LastFragment = property(_thostmduserapi.CThostFtdcReqChangeAccountField_LastFragment_get, _thostmduserapi.CThostFtdcReqChangeAccountField_LastFragment_set)
    SessionID = property(_thostmduserapi.CThostFtdcReqChangeAccountField_SessionID_get, _thostmduserapi.CThostFtdcReqChangeAccountField_SessionID_set)
    CustomerName = property(_thostmduserapi.CThostFtdcReqChangeAccountField_CustomerName_get, _thostmduserapi.CThostFtdcReqChangeAccountField_CustomerName_set)
    IdCardType = property(_thostmduserapi.CThostFtdcReqChangeAccountField_IdCardType_get, _thostmduserapi.CThostFtdcReqChangeAccountField_IdCardType_set)
    IdentifiedCardNo = property(_thostmduserapi.CThostFtdcReqChangeAccountField_IdentifiedCardNo_get, _thostmduserapi.CThostFtdcReqChangeAccountField_IdentifiedCardNo_set)
    Gender = property(_thostmduserapi.CThostFtdcReqChangeAccountField_Gender_get, _thostmduserapi.CThostFtdcReqChangeAccountField_Gender_set)
    CountryCode = property(_thostmduserapi.CThostFtdcReqChangeAccountField_CountryCode_get, _thostmduserapi.CThostFtdcReqChangeAccountField_CountryCode_set)
    CustType = property(_thostmduserapi.CThostFtdcReqChangeAccountField_CustType_get, _thostmduserapi.CThostFtdcReqChangeAccountField_CustType_set)
    Address = property(_thostmduserapi.CThostFtdcReqChangeAccountField_Address_get, _thostmduserapi.CThostFtdcReqChangeAccountField_Address_set)
    ZipCode = property(_thostmduserapi.CThostFtdcReqChangeAccountField_ZipCode_get, _thostmduserapi.CThostFtdcReqChangeAccountField_ZipCode_set)
    Telephone = property(_thostmduserapi.CThostFtdcReqChangeAccountField_Telephone_get, _thostmduserapi.CThostFtdcReqChangeAccountField_Telephone_set)
    MobilePhone = property(_thostmduserapi.CThostFtdcReqChangeAccountField_MobilePhone_get, _thostmduserapi.CThostFtdcReqChangeAccountField_MobilePhone_set)
    Fax = property(_thostmduserapi.CThostFtdcReqChangeAccountField_Fax_get, _thostmduserapi.CThostFtdcReqChangeAccountField_Fax_set)
    EMail = property(_thostmduserapi.CThostFtdcReqChangeAccountField_EMail_get, _thostmduserapi.CThostFtdcReqChangeAccountField_EMail_set)
    MoneyAccountStatus = property(_thostmduserapi.CThostFtdcReqChangeAccountField_MoneyAccountStatus_get, _thostmduserapi.CThostFtdcReqChangeAccountField_MoneyAccountStatus_set)
    BankAccount = property(_thostmduserapi.CThostFtdcReqChangeAccountField_BankAccount_get, _thostmduserapi.CThostFtdcReqChangeAccountField_BankAccount_set)
    BankPassWord = property(_thostmduserapi.CThostFtdcReqChangeAccountField_BankPassWord_get, _thostmduserapi.CThostFtdcReqChangeAccountField_BankPassWord_set)
    NewBankAccount = property(_thostmduserapi.CThostFtdcReqChangeAccountField_NewBankAccount_get, _thostmduserapi.CThostFtdcReqChangeAccountField_NewBankAccount_set)
    NewBankPassWord = property(_thostmduserapi.CThostFtdcReqChangeAccountField_NewBankPassWord_get, _thostmduserapi.CThostFtdcReqChangeAccountField_NewBankPassWord_set)
    AccountID = property(_thostmduserapi.CThostFtdcReqChangeAccountField_AccountID_get, _thostmduserapi.CThostFtdcReqChangeAccountField_AccountID_set)
    Password = property(_thostmduserapi.CThostFtdcReqChangeAccountField_Password_get, _thostmduserapi.CThostFtdcReqChangeAccountField_Password_set)
    BankAccType = property(_thostmduserapi.CThostFtdcReqChangeAccountField_BankAccType_get, _thostmduserapi.CThostFtdcReqChangeAccountField_BankAccType_set)
    InstallID = property(_thostmduserapi.CThostFtdcReqChangeAccountField_InstallID_get, _thostmduserapi.CThostFtdcReqChangeAccountField_InstallID_set)
    VerifyCertNoFlag = property(_thostmduserapi.CThostFtdcReqChangeAccountField_VerifyCertNoFlag_get, _thostmduserapi.CThostFtdcReqChangeAccountField_VerifyCertNoFlag_set)
    CurrencyID = property(_thostmduserapi.CThostFtdcReqChangeAccountField_CurrencyID_get, _thostmduserapi.CThostFtdcReqChangeAccountField_CurrencyID_set)
    BrokerIDByBank = property(_thostmduserapi.CThostFtdcReqChangeAccountField_BrokerIDByBank_get, _thostmduserapi.CThostFtdcReqChangeAccountField_BrokerIDByBank_set)
    BankPwdFlag = property(_thostmduserapi.CThostFtdcReqChangeAccountField_BankPwdFlag_get, _thostmduserapi.CThostFtdcReqChangeAccountField_BankPwdFlag_set)
    SecuPwdFlag = property(_thostmduserapi.CThostFtdcReqChangeAccountField_SecuPwdFlag_get, _thostmduserapi.CThostFtdcReqChangeAccountField_SecuPwdFlag_set)
    TID = property(_thostmduserapi.CThostFtdcReqChangeAccountField_TID_get, _thostmduserapi.CThostFtdcReqChangeAccountField_TID_set)
    Digest = property(_thostmduserapi.CThostFtdcReqChangeAccountField_Digest_get, _thostmduserapi.CThostFtdcReqChangeAccountField_Digest_set)
    LongCustomerName = property(_thostmduserapi.CThostFtdcReqChangeAccountField_LongCustomerName_get, _thostmduserapi.CThostFtdcReqChangeAccountField_LongCustomerName_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcReqChangeAccountField_swiginit(self, _thostmduserapi.new_CThostFtdcReqChangeAccountField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcReqChangeAccountField

# Register CThostFtdcReqChangeAccountField in _thostmduserapi:
_thostmduserapi.CThostFtdcReqChangeAccountField_swigregister(CThostFtdcReqChangeAccountField)

class CThostFtdcReqTransferField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradeCode = property(_thostmduserapi.CThostFtdcReqTransferField_TradeCode_get, _thostmduserapi.CThostFtdcReqTransferField_TradeCode_set)
    BankID = property(_thostmduserapi.CThostFtdcReqTransferField_BankID_get, _thostmduserapi.CThostFtdcReqTransferField_BankID_set)
    BankBranchID = property(_thostmduserapi.CThostFtdcReqTransferField_BankBranchID_get, _thostmduserapi.CThostFtdcReqTransferField_BankBranchID_set)
    BrokerID = property(_thostmduserapi.CThostFtdcReqTransferField_BrokerID_get, _thostmduserapi.CThostFtdcReqTransferField_BrokerID_set)
    BrokerBranchID = property(_thostmduserapi.CThostFtdcReqTransferField_BrokerBranchID_get, _thostmduserapi.CThostFtdcReqTransferField_BrokerBranchID_set)
    TradeDate = property(_thostmduserapi.CThostFtdcReqTransferField_TradeDate_get, _thostmduserapi.CThostFtdcReqTransferField_TradeDate_set)
    TradeTime = property(_thostmduserapi.CThostFtdcReqTransferField_TradeTime_get, _thostmduserapi.CThostFtdcReqTransferField_TradeTime_set)
    BankSerial = property(_thostmduserapi.CThostFtdcReqTransferField_BankSerial_get, _thostmduserapi.CThostFtdcReqTransferField_BankSerial_set)
    TradingDay = property(_thostmduserapi.CThostFtdcReqTransferField_TradingDay_get, _thostmduserapi.CThostFtdcReqTransferField_TradingDay_set)
    PlateSerial = property(_thostmduserapi.CThostFtdcReqTransferField_PlateSerial_get, _thostmduserapi.CThostFtdcReqTransferField_PlateSerial_set)
    LastFragment = property(_thostmduserapi.CThostFtdcReqTransferField_LastFragment_get, _thostmduserapi.CThostFtdcReqTransferField_LastFragment_set)
    SessionID = property(_thostmduserapi.CThostFtdcReqTransferField_SessionID_get, _thostmduserapi.CThostFtdcReqTransferField_SessionID_set)
    CustomerName = property(_thostmduserapi.CThostFtdcReqTransferField_CustomerName_get, _thostmduserapi.CThostFtdcReqTransferField_CustomerName_set)
    IdCardType = property(_thostmduserapi.CThostFtdcReqTransferField_IdCardType_get, _thostmduserapi.CThostFtdcReqTransferField_IdCardType_set)
    IdentifiedCardNo = property(_thostmduserapi.CThostFtdcReqTransferField_IdentifiedCardNo_get, _thostmduserapi.CThostFtdcReqTransferField_IdentifiedCardNo_set)
    CustType = property(_thostmduserapi.CThostFtdcReqTransferField_CustType_get, _thostmduserapi.CThostFtdcReqTransferField_CustType_set)
    BankAccount = property(_thostmduserapi.CThostFtdcReqTransferField_BankAccount_get, _thostmduserapi.CThostFtdcReqTransferField_BankAccount_set)
    BankPassWord = property(_thostmduserapi.CThostFtdcReqTransferField_BankPassWord_get, _thostmduserapi.CThostFtdcReqTransferField_BankPassWord_set)
    AccountID = property(_thostmduserapi.CThostFtdcReqTransferField_AccountID_get, _thostmduserapi.CThostFtdcReqTransferField_AccountID_set)
    Password = property(_thostmduserapi.CThostFtdcReqTransferField_Password_get, _thostmduserapi.CThostFtdcReqTransferField_Password_set)
    InstallID = property(_thostmduserapi.CThostFtdcReqTransferField_InstallID_get, _thostmduserapi.CThostFtdcReqTransferField_InstallID_set)
    FutureSerial = property(_thostmduserapi.CThostFtdcReqTransferField_FutureSerial_get, _thostmduserapi.CThostFtdcReqTransferField_FutureSerial_set)
    UserID = property(_thostmduserapi.CThostFtdcReqTransferField_UserID_get, _thostmduserapi.CThostFtdcReqTransferField_UserID_set)
    VerifyCertNoFlag = property(_thostmduserapi.CThostFtdcReqTransferField_VerifyCertNoFlag_get, _thostmduserapi.CThostFtdcReqTransferField_VerifyCertNoFlag_set)
    CurrencyID = property(_thostmduserapi.CThostFtdcReqTransferField_CurrencyID_get, _thostmduserapi.CThostFtdcReqTransferField_CurrencyID_set)
    TradeAmount = property(_thostmduserapi.CThostFtdcReqTransferField_TradeAmount_get, _thostmduserapi.CThostFtdcReqTransferField_TradeAmount_set)
    FutureFetchAmount = property(_thostmduserapi.CThostFtdcReqTransferField_FutureFetchAmount_get, _thostmduserapi.CThostFtdcReqTransferField_FutureFetchAmount_set)
    FeePayFlag = property(_thostmduserapi.CThostFtdcReqTransferField_FeePayFlag_get, _thostmduserapi.CThostFtdcReqTransferField_FeePayFlag_set)
    CustFee = property(_thostmduserapi.CThostFtdcReqTransferField_CustFee_get, _thostmduserapi.CThostFtdcReqTransferField_CustFee_set)
    BrokerFee = property(_thostmduserapi.CThostFtdcReqTransferField_BrokerFee_get, _thostmduserapi.CThostFtdcReqTransferField_BrokerFee_set)
    Message = property(_thostmduserapi.CThostFtdcReqTransferField_Message_get, _thostmduserapi.CThostFtdcReqTransferField_Message_set)
    Digest = property(_thostmduserapi.CThostFtdcReqTransferField_Digest_get, _thostmduserapi.CThostFtdcReqTransferField_Digest_set)
    BankAccType = property(_thostmduserapi.CThostFtdcReqTransferField_BankAccType_get, _thostmduserapi.CThostFtdcReqTransferField_BankAccType_set)
    DeviceID = property(_thostmduserapi.CThostFtdcReqTransferField_DeviceID_get, _thostmduserapi.CThostFtdcReqTransferField_DeviceID_set)
    BankSecuAccType = property(_thostmduserapi.CThostFtdcReqTransferField_BankSecuAccType_get, _thostmduserapi.CThostFtdcReqTransferField_BankSecuAccType_set)
    BrokerIDByBank = property(_thostmduserapi.CThostFtdcReqTransferField_BrokerIDByBank_get, _thostmduserapi.CThostFtdcReqTransferField_BrokerIDByBank_set)
    BankSecuAcc = property(_thostmduserapi.CThostFtdcReqTransferField_BankSecuAcc_get, _thostmduserapi.CThostFtdcReqTransferField_BankSecuAcc_set)
    BankPwdFlag = property(_thostmduserapi.CThostFtdcReqTransferField_BankPwdFlag_get, _thostmduserapi.CThostFtdcReqTransferField_BankPwdFlag_set)
    SecuPwdFlag = property(_thostmduserapi.CThostFtdcReqTransferField_SecuPwdFlag_get, _thostmduserapi.CThostFtdcReqTransferField_SecuPwdFlag_set)
    OperNo = property(_thostmduserapi.CThostFtdcReqTransferField_OperNo_get, _thostmduserapi.CThostFtdcReqTransferField_OperNo_set)
    RequestID = property(_thostmduserapi.CThostFtdcReqTransferField_RequestID_get, _thostmduserapi.CThostFtdcReqTransferField_RequestID_set)
    TID = property(_thostmduserapi.CThostFtdcReqTransferField_TID_get, _thostmduserapi.CThostFtdcReqTransferField_TID_set)
    TransferStatus = property(_thostmduserapi.CThostFtdcReqTransferField_TransferStatus_get, _thostmduserapi.CThostFtdcReqTransferField_TransferStatus_set)
    LongCustomerName = property(_thostmduserapi.CThostFtdcReqTransferField_LongCustomerName_get, _thostmduserapi.CThostFtdcReqTransferField_LongCustomerName_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcReqTransferField_swiginit(self, _thostmduserapi.new_CThostFtdcReqTransferField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcReqTransferField

# Register CThostFtdcReqTransferField in _thostmduserapi:
_thostmduserapi.CThostFtdcReqTransferField_swigregister(CThostFtdcReqTransferField)

class CThostFtdcRspTransferField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradeCode = property(_thostmduserapi.CThostFtdcRspTransferField_TradeCode_get, _thostmduserapi.CThostFtdcRspTransferField_TradeCode_set)
    BankID = property(_thostmduserapi.CThostFtdcRspTransferField_BankID_get, _thostmduserapi.CThostFtdcRspTransferField_BankID_set)
    BankBranchID = property(_thostmduserapi.CThostFtdcRspTransferField_BankBranchID_get, _thostmduserapi.CThostFtdcRspTransferField_BankBranchID_set)
    BrokerID = property(_thostmduserapi.CThostFtdcRspTransferField_BrokerID_get, _thostmduserapi.CThostFtdcRspTransferField_BrokerID_set)
    BrokerBranchID = property(_thostmduserapi.CThostFtdcRspTransferField_BrokerBranchID_get, _thostmduserapi.CThostFtdcRspTransferField_BrokerBranchID_set)
    TradeDate = property(_thostmduserapi.CThostFtdcRspTransferField_TradeDate_get, _thostmduserapi.CThostFtdcRspTransferField_TradeDate_set)
    TradeTime = property(_thostmduserapi.CThostFtdcRspTransferField_TradeTime_get, _thostmduserapi.CThostFtdcRspTransferField_TradeTime_set)
    BankSerial = property(_thostmduserapi.CThostFtdcRspTransferField_BankSerial_get, _thostmduserapi.CThostFtdcRspTransferField_BankSerial_set)
    TradingDay = property(_thostmduserapi.CThostFtdcRspTransferField_TradingDay_get, _thostmduserapi.CThostFtdcRspTransferField_TradingDay_set)
    PlateSerial = property(_thostmduserapi.CThostFtdcRspTransferField_PlateSerial_get, _thostmduserapi.CThostFtdcRspTransferField_PlateSerial_set)
    LastFragment = property(_thostmduserapi.CThostFtdcRspTransferField_LastFragment_get, _thostmduserapi.CThostFtdcRspTransferField_LastFragment_set)
    SessionID = property(_thostmduserapi.CThostFtdcRspTransferField_SessionID_get, _thostmduserapi.CThostFtdcRspTransferField_SessionID_set)
    CustomerName = property(_thostmduserapi.CThostFtdcRspTransferField_CustomerName_get, _thostmduserapi.CThostFtdcRspTransferField_CustomerName_set)
    IdCardType = property(_thostmduserapi.CThostFtdcRspTransferField_IdCardType_get, _thostmduserapi.CThostFtdcRspTransferField_IdCardType_set)
    IdentifiedCardNo = property(_thostmduserapi.CThostFtdcRspTransferField_IdentifiedCardNo_get, _thostmduserapi.CThostFtdcRspTransferField_IdentifiedCardNo_set)
    CustType = property(_thostmduserapi.CThostFtdcRspTransferField_CustType_get, _thostmduserapi.CThostFtdcRspTransferField_CustType_set)
    BankAccount = property(_thostmduserapi.CThostFtdcRspTransferField_BankAccount_get, _thostmduserapi.CThostFtdcRspTransferField_BankAccount_set)
    BankPassWord = property(_thostmduserapi.CThostFtdcRspTransferField_BankPassWord_get, _thostmduserapi.CThostFtdcRspTransferField_BankPassWord_set)
    AccountID = property(_thostmduserapi.CThostFtdcRspTransferField_AccountID_get, _thostmduserapi.CThostFtdcRspTransferField_AccountID_set)
    Password = property(_thostmduserapi.CThostFtdcRspTransferField_Password_get, _thostmduserapi.CThostFtdcRspTransferField_Password_set)
    InstallID = property(_thostmduserapi.CThostFtdcRspTransferField_InstallID_get, _thostmduserapi.CThostFtdcRspTransferField_InstallID_set)
    FutureSerial = property(_thostmduserapi.CThostFtdcRspTransferField_FutureSerial_get, _thostmduserapi.CThostFtdcRspTransferField_FutureSerial_set)
    UserID = property(_thostmduserapi.CThostFtdcRspTransferField_UserID_get, _thostmduserapi.CThostFtdcRspTransferField_UserID_set)
    VerifyCertNoFlag = property(_thostmduserapi.CThostFtdcRspTransferField_VerifyCertNoFlag_get, _thostmduserapi.CThostFtdcRspTransferField_VerifyCertNoFlag_set)
    CurrencyID = property(_thostmduserapi.CThostFtdcRspTransferField_CurrencyID_get, _thostmduserapi.CThostFtdcRspTransferField_CurrencyID_set)
    TradeAmount = property(_thostmduserapi.CThostFtdcRspTransferField_TradeAmount_get, _thostmduserapi.CThostFtdcRspTransferField_TradeAmount_set)
    FutureFetchAmount = property(_thostmduserapi.CThostFtdcRspTransferField_FutureFetchAmount_get, _thostmduserapi.CThostFtdcRspTransferField_FutureFetchAmount_set)
    FeePayFlag = property(_thostmduserapi.CThostFtdcRspTransferField_FeePayFlag_get, _thostmduserapi.CThostFtdcRspTransferField_FeePayFlag_set)
    CustFee = property(_thostmduserapi.CThostFtdcRspTransferField_CustFee_get, _thostmduserapi.CThostFtdcRspTransferField_CustFee_set)
    BrokerFee = property(_thostmduserapi.CThostFtdcRspTransferField_BrokerFee_get, _thostmduserapi.CThostFtdcRspTransferField_BrokerFee_set)
    Message = property(_thostmduserapi.CThostFtdcRspTransferField_Message_get, _thostmduserapi.CThostFtdcRspTransferField_Message_set)
    Digest = property(_thostmduserapi.CThostFtdcRspTransferField_Digest_get, _thostmduserapi.CThostFtdcRspTransferField_Digest_set)
    BankAccType = property(_thostmduserapi.CThostFtdcRspTransferField_BankAccType_get, _thostmduserapi.CThostFtdcRspTransferField_BankAccType_set)
    DeviceID = property(_thostmduserapi.CThostFtdcRspTransferField_DeviceID_get, _thostmduserapi.CThostFtdcRspTransferField_DeviceID_set)
    BankSecuAccType = property(_thostmduserapi.CThostFtdcRspTransferField_BankSecuAccType_get, _thostmduserapi.CThostFtdcRspTransferField_BankSecuAccType_set)
    BrokerIDByBank = property(_thostmduserapi.CThostFtdcRspTransferField_BrokerIDByBank_get, _thostmduserapi.CThostFtdcRspTransferField_BrokerIDByBank_set)
    BankSecuAcc = property(_thostmduserapi.CThostFtdcRspTransferField_BankSecuAcc_get, _thostmduserapi.CThostFtdcRspTransferField_BankSecuAcc_set)
    BankPwdFlag = property(_thostmduserapi.CThostFtdcRspTransferField_BankPwdFlag_get, _thostmduserapi.CThostFtdcRspTransferField_BankPwdFlag_set)
    SecuPwdFlag = property(_thostmduserapi.CThostFtdcRspTransferField_SecuPwdFlag_get, _thostmduserapi.CThostFtdcRspTransferField_SecuPwdFlag_set)
    OperNo = property(_thostmduserapi.CThostFtdcRspTransferField_OperNo_get, _thostmduserapi.CThostFtdcRspTransferField_OperNo_set)
    RequestID = property(_thostmduserapi.CThostFtdcRspTransferField_RequestID_get, _thostmduserapi.CThostFtdcRspTransferField_RequestID_set)
    TID = property(_thostmduserapi.CThostFtdcRspTransferField_TID_get, _thostmduserapi.CThostFtdcRspTransferField_TID_set)
    TransferStatus = property(_thostmduserapi.CThostFtdcRspTransferField_TransferStatus_get, _thostmduserapi.CThostFtdcRspTransferField_TransferStatus_set)
    ErrorID = property(_thostmduserapi.CThostFtdcRspTransferField_ErrorID_get, _thostmduserapi.CThostFtdcRspTransferField_ErrorID_set)
    ErrorMsg = property(_thostmduserapi.CThostFtdcRspTransferField_ErrorMsg_get, _thostmduserapi.CThostFtdcRspTransferField_ErrorMsg_set)
    LongCustomerName = property(_thostmduserapi.CThostFtdcRspTransferField_LongCustomerName_get, _thostmduserapi.CThostFtdcRspTransferField_LongCustomerName_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcRspTransferField_swiginit(self, _thostmduserapi.new_CThostFtdcRspTransferField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcRspTransferField

# Register CThostFtdcRspTransferField in _thostmduserapi:
_thostmduserapi.CThostFtdcRspTransferField_swigregister(CThostFtdcRspTransferField)

class CThostFtdcReqRepealField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    RepealTimeInterval = property(_thostmduserapi.CThostFtdcReqRepealField_RepealTimeInterval_get, _thostmduserapi.CThostFtdcReqRepealField_RepealTimeInterval_set)
    RepealedTimes = property(_thostmduserapi.CThostFtdcReqRepealField_RepealedTimes_get, _thostmduserapi.CThostFtdcReqRepealField_RepealedTimes_set)
    BankRepealFlag = property(_thostmduserapi.CThostFtdcReqRepealField_BankRepealFlag_get, _thostmduserapi.CThostFtdcReqRepealField_BankRepealFlag_set)
    BrokerRepealFlag = property(_thostmduserapi.CThostFtdcReqRepealField_BrokerRepealFlag_get, _thostmduserapi.CThostFtdcReqRepealField_BrokerRepealFlag_set)
    PlateRepealSerial = property(_thostmduserapi.CThostFtdcReqRepealField_PlateRepealSerial_get, _thostmduserapi.CThostFtdcReqRepealField_PlateRepealSerial_set)
    BankRepealSerial = property(_thostmduserapi.CThostFtdcReqRepealField_BankRepealSerial_get, _thostmduserapi.CThostFtdcReqRepealField_BankRepealSerial_set)
    FutureRepealSerial = property(_thostmduserapi.CThostFtdcReqRepealField_FutureRepealSerial_get, _thostmduserapi.CThostFtdcReqRepealField_FutureRepealSerial_set)
    TradeCode = property(_thostmduserapi.CThostFtdcReqRepealField_TradeCode_get, _thostmduserapi.CThostFtdcReqRepealField_TradeCode_set)
    BankID = property(_thostmduserapi.CThostFtdcReqRepealField_BankID_get, _thostmduserapi.CThostFtdcReqRepealField_BankID_set)
    BankBranchID = property(_thostmduserapi.CThostFtdcReqRepealField_BankBranchID_get, _thostmduserapi.CThostFtdcReqRepealField_BankBranchID_set)
    BrokerID = property(_thostmduserapi.CThostFtdcReqRepealField_BrokerID_get, _thostmduserapi.CThostFtdcReqRepealField_BrokerID_set)
    BrokerBranchID = property(_thostmduserapi.CThostFtdcReqRepealField_BrokerBranchID_get, _thostmduserapi.CThostFtdcReqRepealField_BrokerBranchID_set)
    TradeDate = property(_thostmduserapi.CThostFtdcReqRepealField_TradeDate_get, _thostmduserapi.CThostFtdcReqRepealField_TradeDate_set)
    TradeTime = property(_thostmduserapi.CThostFtdcReqRepealField_TradeTime_get, _thostmduserapi.CThostFtdcReqRepealField_TradeTime_set)
    BankSerial = property(_thostmduserapi.CThostFtdcReqRepealField_BankSerial_get, _thostmduserapi.CThostFtdcReqRepealField_BankSerial_set)
    TradingDay = property(_thostmduserapi.CThostFtdcReqRepealField_TradingDay_get, _thostmduserapi.CThostFtdcReqRepealField_TradingDay_set)
    PlateSerial = property(_thostmduserapi.CThostFtdcReqRepealField_PlateSerial_get, _thostmduserapi.CThostFtdcReqRepealField_PlateSerial_set)
    LastFragment = property(_thostmduserapi.CThostFtdcReqRepealField_LastFragment_get, _thostmduserapi.CThostFtdcReqRepealField_LastFragment_set)
    SessionID = property(_thostmduserapi.CThostFtdcReqRepealField_SessionID_get, _thostmduserapi.CThostFtdcReqRepealField_SessionID_set)
    CustomerName = property(_thostmduserapi.CThostFtdcReqRepealField_CustomerName_get, _thostmduserapi.CThostFtdcReqRepealField_CustomerName_set)
    IdCardType = property(_thostmduserapi.CThostFtdcReqRepealField_IdCardType_get, _thostmduserapi.CThostFtdcReqRepealField_IdCardType_set)
    IdentifiedCardNo = property(_thostmduserapi.CThostFtdcReqRepealField_IdentifiedCardNo_get, _thostmduserapi.CThostFtdcReqRepealField_IdentifiedCardNo_set)
    CustType = property(_thostmduserapi.CThostFtdcReqRepealField_CustType_get, _thostmduserapi.CThostFtdcReqRepealField_CustType_set)
    BankAccount = property(_thostmduserapi.CThostFtdcReqRepealField_BankAccount_get, _thostmduserapi.CThostFtdcReqRepealField_BankAccount_set)
    BankPassWord = property(_thostmduserapi.CThostFtdcReqRepealField_BankPassWord_get, _thostmduserapi.CThostFtdcReqRepealField_BankPassWord_set)
    AccountID = property(_thostmduserapi.CThostFtdcReqRepealField_AccountID_get, _thostmduserapi.CThostFtdcReqRepealField_AccountID_set)
    Password = property(_thostmduserapi.CThostFtdcReqRepealField_Password_get, _thostmduserapi.CThostFtdcReqRepealField_Password_set)
    InstallID = property(_thostmduserapi.CThostFtdcReqRepealField_InstallID_get, _thostmduserapi.CThostFtdcReqRepealField_InstallID_set)
    FutureSerial = property(_thostmduserapi.CThostFtdcReqRepealField_FutureSerial_get, _thostmduserapi.CThostFtdcReqRepealField_FutureSerial_set)
    UserID = property(_thostmduserapi.CThostFtdcReqRepealField_UserID_get, _thostmduserapi.CThostFtdcReqRepealField_UserID_set)
    VerifyCertNoFlag = property(_thostmduserapi.CThostFtdcReqRepealField_VerifyCertNoFlag_get, _thostmduserapi.CThostFtdcReqRepealField_VerifyCertNoFlag_set)
    CurrencyID = property(_thostmduserapi.CThostFtdcReqRepealField_CurrencyID_get, _thostmduserapi.CThostFtdcReqRepealField_CurrencyID_set)
    TradeAmount = property(_thostmduserapi.CThostFtdcReqRepealField_TradeAmount_get, _thostmduserapi.CThostFtdcReqRepealField_TradeAmount_set)
    FutureFetchAmount = property(_thostmduserapi.CThostFtdcReqRepealField_FutureFetchAmount_get, _thostmduserapi.CThostFtdcReqRepealField_FutureFetchAmount_set)
    FeePayFlag = property(_thostmduserapi.CThostFtdcReqRepealField_FeePayFlag_get, _thostmduserapi.CThostFtdcReqRepealField_FeePayFlag_set)
    CustFee = property(_thostmduserapi.CThostFtdcReqRepealField_CustFee_get, _thostmduserapi.CThostFtdcReqRepealField_CustFee_set)
    BrokerFee = property(_thostmduserapi.CThostFtdcReqRepealField_BrokerFee_get, _thostmduserapi.CThostFtdcReqRepealField_BrokerFee_set)
    Message = property(_thostmduserapi.CThostFtdcReqRepealField_Message_get, _thostmduserapi.CThostFtdcReqRepealField_Message_set)
    Digest = property(_thostmduserapi.CThostFtdcReqRepealField_Digest_get, _thostmduserapi.CThostFtdcReqRepealField_Digest_set)
    BankAccType = property(_thostmduserapi.CThostFtdcReqRepealField_BankAccType_get, _thostmduserapi.CThostFtdcReqRepealField_BankAccType_set)
    DeviceID = property(_thostmduserapi.CThostFtdcReqRepealField_DeviceID_get, _thostmduserapi.CThostFtdcReqRepealField_DeviceID_set)
    BankSecuAccType = property(_thostmduserapi.CThostFtdcReqRepealField_BankSecuAccType_get, _thostmduserapi.CThostFtdcReqRepealField_BankSecuAccType_set)
    BrokerIDByBank = property(_thostmduserapi.CThostFtdcReqRepealField_BrokerIDByBank_get, _thostmduserapi.CThostFtdcReqRepealField_BrokerIDByBank_set)
    BankSecuAcc = property(_thostmduserapi.CThostFtdcReqRepealField_BankSecuAcc_get, _thostmduserapi.CThostFtdcReqRepealField_BankSecuAcc_set)
    BankPwdFlag = property(_thostmduserapi.CThostFtdcReqRepealField_BankPwdFlag_get, _thostmduserapi.CThostFtdcReqRepealField_BankPwdFlag_set)
    SecuPwdFlag = property(_thostmduserapi.CThostFtdcReqRepealField_SecuPwdFlag_get, _thostmduserapi.CThostFtdcReqRepealField_SecuPwdFlag_set)
    OperNo = property(_thostmduserapi.CThostFtdcReqRepealField_OperNo_get, _thostmduserapi.CThostFtdcReqRepealField_OperNo_set)
    RequestID = property(_thostmduserapi.CThostFtdcReqRepealField_RequestID_get, _thostmduserapi.CThostFtdcReqRepealField_RequestID_set)
    TID = property(_thostmduserapi.CThostFtdcReqRepealField_TID_get, _thostmduserapi.CThostFtdcReqRepealField_TID_set)
    TransferStatus = property(_thostmduserapi.CThostFtdcReqRepealField_TransferStatus_get, _thostmduserapi.CThostFtdcReqRepealField_TransferStatus_set)
    LongCustomerName = property(_thostmduserapi.CThostFtdcReqRepealField_LongCustomerName_get, _thostmduserapi.CThostFtdcReqRepealField_LongCustomerName_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcReqRepealField_swiginit(self, _thostmduserapi.new_CThostFtdcReqRepealField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcReqRepealField

# Register CThostFtdcReqRepealField in _thostmduserapi:
_thostmduserapi.CThostFtdcReqRepealField_swigregister(CThostFtdcReqRepealField)

class CThostFtdcRspRepealField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    RepealTimeInterval = property(_thostmduserapi.CThostFtdcRspRepealField_RepealTimeInterval_get, _thostmduserapi.CThostFtdcRspRepealField_RepealTimeInterval_set)
    RepealedTimes = property(_thostmduserapi.CThostFtdcRspRepealField_RepealedTimes_get, _thostmduserapi.CThostFtdcRspRepealField_RepealedTimes_set)
    BankRepealFlag = property(_thostmduserapi.CThostFtdcRspRepealField_BankRepealFlag_get, _thostmduserapi.CThostFtdcRspRepealField_BankRepealFlag_set)
    BrokerRepealFlag = property(_thostmduserapi.CThostFtdcRspRepealField_BrokerRepealFlag_get, _thostmduserapi.CThostFtdcRspRepealField_BrokerRepealFlag_set)
    PlateRepealSerial = property(_thostmduserapi.CThostFtdcRspRepealField_PlateRepealSerial_get, _thostmduserapi.CThostFtdcRspRepealField_PlateRepealSerial_set)
    BankRepealSerial = property(_thostmduserapi.CThostFtdcRspRepealField_BankRepealSerial_get, _thostmduserapi.CThostFtdcRspRepealField_BankRepealSerial_set)
    FutureRepealSerial = property(_thostmduserapi.CThostFtdcRspRepealField_FutureRepealSerial_get, _thostmduserapi.CThostFtdcRspRepealField_FutureRepealSerial_set)
    TradeCode = property(_thostmduserapi.CThostFtdcRspRepealField_TradeCode_get, _thostmduserapi.CThostFtdcRspRepealField_TradeCode_set)
    BankID = property(_thostmduserapi.CThostFtdcRspRepealField_BankID_get, _thostmduserapi.CThostFtdcRspRepealField_BankID_set)
    BankBranchID = property(_thostmduserapi.CThostFtdcRspRepealField_BankBranchID_get, _thostmduserapi.CThostFtdcRspRepealField_BankBranchID_set)
    BrokerID = property(_thostmduserapi.CThostFtdcRspRepealField_BrokerID_get, _thostmduserapi.CThostFtdcRspRepealField_BrokerID_set)
    BrokerBranchID = property(_thostmduserapi.CThostFtdcRspRepealField_BrokerBranchID_get, _thostmduserapi.CThostFtdcRspRepealField_BrokerBranchID_set)
    TradeDate = property(_thostmduserapi.CThostFtdcRspRepealField_TradeDate_get, _thostmduserapi.CThostFtdcRspRepealField_TradeDate_set)
    TradeTime = property(_thostmduserapi.CThostFtdcRspRepealField_TradeTime_get, _thostmduserapi.CThostFtdcRspRepealField_TradeTime_set)
    BankSerial = property(_thostmduserapi.CThostFtdcRspRepealField_BankSerial_get, _thostmduserapi.CThostFtdcRspRepealField_BankSerial_set)
    TradingDay = property(_thostmduserapi.CThostFtdcRspRepealField_TradingDay_get, _thostmduserapi.CThostFtdcRspRepealField_TradingDay_set)
    PlateSerial = property(_thostmduserapi.CThostFtdcRspRepealField_PlateSerial_get, _thostmduserapi.CThostFtdcRspRepealField_PlateSerial_set)
    LastFragment = property(_thostmduserapi.CThostFtdcRspRepealField_LastFragment_get, _thostmduserapi.CThostFtdcRspRepealField_LastFragment_set)
    SessionID = property(_thostmduserapi.CThostFtdcRspRepealField_SessionID_get, _thostmduserapi.CThostFtdcRspRepealField_SessionID_set)
    CustomerName = property(_thostmduserapi.CThostFtdcRspRepealField_CustomerName_get, _thostmduserapi.CThostFtdcRspRepealField_CustomerName_set)
    IdCardType = property(_thostmduserapi.CThostFtdcRspRepealField_IdCardType_get, _thostmduserapi.CThostFtdcRspRepealField_IdCardType_set)
    IdentifiedCardNo = property(_thostmduserapi.CThostFtdcRspRepealField_IdentifiedCardNo_get, _thostmduserapi.CThostFtdcRspRepealField_IdentifiedCardNo_set)
    CustType = property(_thostmduserapi.CThostFtdcRspRepealField_CustType_get, _thostmduserapi.CThostFtdcRspRepealField_CustType_set)
    BankAccount = property(_thostmduserapi.CThostFtdcRspRepealField_BankAccount_get, _thostmduserapi.CThostFtdcRspRepealField_BankAccount_set)
    BankPassWord = property(_thostmduserapi.CThostFtdcRspRepealField_BankPassWord_get, _thostmduserapi.CThostFtdcRspRepealField_BankPassWord_set)
    AccountID = property(_thostmduserapi.CThostFtdcRspRepealField_AccountID_get, _thostmduserapi.CThostFtdcRspRepealField_AccountID_set)
    Password = property(_thostmduserapi.CThostFtdcRspRepealField_Password_get, _thostmduserapi.CThostFtdcRspRepealField_Password_set)
    InstallID = property(_thostmduserapi.CThostFtdcRspRepealField_InstallID_get, _thostmduserapi.CThostFtdcRspRepealField_InstallID_set)
    FutureSerial = property(_thostmduserapi.CThostFtdcRspRepealField_FutureSerial_get, _thostmduserapi.CThostFtdcRspRepealField_FutureSerial_set)
    UserID = property(_thostmduserapi.CThostFtdcRspRepealField_UserID_get, _thostmduserapi.CThostFtdcRspRepealField_UserID_set)
    VerifyCertNoFlag = property(_thostmduserapi.CThostFtdcRspRepealField_VerifyCertNoFlag_get, _thostmduserapi.CThostFtdcRspRepealField_VerifyCertNoFlag_set)
    CurrencyID = property(_thostmduserapi.CThostFtdcRspRepealField_CurrencyID_get, _thostmduserapi.CThostFtdcRspRepealField_CurrencyID_set)
    TradeAmount = property(_thostmduserapi.CThostFtdcRspRepealField_TradeAmount_get, _thostmduserapi.CThostFtdcRspRepealField_TradeAmount_set)
    FutureFetchAmount = property(_thostmduserapi.CThostFtdcRspRepealField_FutureFetchAmount_get, _thostmduserapi.CThostFtdcRspRepealField_FutureFetchAmount_set)
    FeePayFlag = property(_thostmduserapi.CThostFtdcRspRepealField_FeePayFlag_get, _thostmduserapi.CThostFtdcRspRepealField_FeePayFlag_set)
    CustFee = property(_thostmduserapi.CThostFtdcRspRepealField_CustFee_get, _thostmduserapi.CThostFtdcRspRepealField_CustFee_set)
    BrokerFee = property(_thostmduserapi.CThostFtdcRspRepealField_BrokerFee_get, _thostmduserapi.CThostFtdcRspRepealField_BrokerFee_set)
    Message = property(_thostmduserapi.CThostFtdcRspRepealField_Message_get, _thostmduserapi.CThostFtdcRspRepealField_Message_set)
    Digest = property(_thostmduserapi.CThostFtdcRspRepealField_Digest_get, _thostmduserapi.CThostFtdcRspRepealField_Digest_set)
    BankAccType = property(_thostmduserapi.CThostFtdcRspRepealField_BankAccType_get, _thostmduserapi.CThostFtdcRspRepealField_BankAccType_set)
    DeviceID = property(_thostmduserapi.CThostFtdcRspRepealField_DeviceID_get, _thostmduserapi.CThostFtdcRspRepealField_DeviceID_set)
    BankSecuAccType = property(_thostmduserapi.CThostFtdcRspRepealField_BankSecuAccType_get, _thostmduserapi.CThostFtdcRspRepealField_BankSecuAccType_set)
    BrokerIDByBank = property(_thostmduserapi.CThostFtdcRspRepealField_BrokerIDByBank_get, _thostmduserapi.CThostFtdcRspRepealField_BrokerIDByBank_set)
    BankSecuAcc = property(_thostmduserapi.CThostFtdcRspRepealField_BankSecuAcc_get, _thostmduserapi.CThostFtdcRspRepealField_BankSecuAcc_set)
    BankPwdFlag = property(_thostmduserapi.CThostFtdcRspRepealField_BankPwdFlag_get, _thostmduserapi.CThostFtdcRspRepealField_BankPwdFlag_set)
    SecuPwdFlag = property(_thostmduserapi.CThostFtdcRspRepealField_SecuPwdFlag_get, _thostmduserapi.CThostFtdcRspRepealField_SecuPwdFlag_set)
    OperNo = property(_thostmduserapi.CThostFtdcRspRepealField_OperNo_get, _thostmduserapi.CThostFtdcRspRepealField_OperNo_set)
    RequestID = property(_thostmduserapi.CThostFtdcRspRepealField_RequestID_get, _thostmduserapi.CThostFtdcRspRepealField_RequestID_set)
    TID = property(_thostmduserapi.CThostFtdcRspRepealField_TID_get, _thostmduserapi.CThostFtdcRspRepealField_TID_set)
    TransferStatus = property(_thostmduserapi.CThostFtdcRspRepealField_TransferStatus_get, _thostmduserapi.CThostFtdcRspRepealField_TransferStatus_set)
    ErrorID = property(_thostmduserapi.CThostFtdcRspRepealField_ErrorID_get, _thostmduserapi.CThostFtdcRspRepealField_ErrorID_set)
    ErrorMsg = property(_thostmduserapi.CThostFtdcRspRepealField_ErrorMsg_get, _thostmduserapi.CThostFtdcRspRepealField_ErrorMsg_set)
    LongCustomerName = property(_thostmduserapi.CThostFtdcRspRepealField_LongCustomerName_get, _thostmduserapi.CThostFtdcRspRepealField_LongCustomerName_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcRspRepealField_swiginit(self, _thostmduserapi.new_CThostFtdcRspRepealField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcRspRepealField

# Register CThostFtdcRspRepealField in _thostmduserapi:
_thostmduserapi.CThostFtdcRspRepealField_swigregister(CThostFtdcRspRepealField)

class CThostFtdcReqQueryAccountField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradeCode = property(_thostmduserapi.CThostFtdcReqQueryAccountField_TradeCode_get, _thostmduserapi.CThostFtdcReqQueryAccountField_TradeCode_set)
    BankID = property(_thostmduserapi.CThostFtdcReqQueryAccountField_BankID_get, _thostmduserapi.CThostFtdcReqQueryAccountField_BankID_set)
    BankBranchID = property(_thostmduserapi.CThostFtdcReqQueryAccountField_BankBranchID_get, _thostmduserapi.CThostFtdcReqQueryAccountField_BankBranchID_set)
    BrokerID = property(_thostmduserapi.CThostFtdcReqQueryAccountField_BrokerID_get, _thostmduserapi.CThostFtdcReqQueryAccountField_BrokerID_set)
    BrokerBranchID = property(_thostmduserapi.CThostFtdcReqQueryAccountField_BrokerBranchID_get, _thostmduserapi.CThostFtdcReqQueryAccountField_BrokerBranchID_set)
    TradeDate = property(_thostmduserapi.CThostFtdcReqQueryAccountField_TradeDate_get, _thostmduserapi.CThostFtdcReqQueryAccountField_TradeDate_set)
    TradeTime = property(_thostmduserapi.CThostFtdcReqQueryAccountField_TradeTime_get, _thostmduserapi.CThostFtdcReqQueryAccountField_TradeTime_set)
    BankSerial = property(_thostmduserapi.CThostFtdcReqQueryAccountField_BankSerial_get, _thostmduserapi.CThostFtdcReqQueryAccountField_BankSerial_set)
    TradingDay = property(_thostmduserapi.CThostFtdcReqQueryAccountField_TradingDay_get, _thostmduserapi.CThostFtdcReqQueryAccountField_TradingDay_set)
    PlateSerial = property(_thostmduserapi.CThostFtdcReqQueryAccountField_PlateSerial_get, _thostmduserapi.CThostFtdcReqQueryAccountField_PlateSerial_set)
    LastFragment = property(_thostmduserapi.CThostFtdcReqQueryAccountField_LastFragment_get, _thostmduserapi.CThostFtdcReqQueryAccountField_LastFragment_set)
    SessionID = property(_thostmduserapi.CThostFtdcReqQueryAccountField_SessionID_get, _thostmduserapi.CThostFtdcReqQueryAccountField_SessionID_set)
    CustomerName = property(_thostmduserapi.CThostFtdcReqQueryAccountField_CustomerName_get, _thostmduserapi.CThostFtdcReqQueryAccountField_CustomerName_set)
    IdCardType = property(_thostmduserapi.CThostFtdcReqQueryAccountField_IdCardType_get, _thostmduserapi.CThostFtdcReqQueryAccountField_IdCardType_set)
    IdentifiedCardNo = property(_thostmduserapi.CThostFtdcReqQueryAccountField_IdentifiedCardNo_get, _thostmduserapi.CThostFtdcReqQueryAccountField_IdentifiedCardNo_set)
    CustType = property(_thostmduserapi.CThostFtdcReqQueryAccountField_CustType_get, _thostmduserapi.CThostFtdcReqQueryAccountField_CustType_set)
    BankAccount = property(_thostmduserapi.CThostFtdcReqQueryAccountField_BankAccount_get, _thostmduserapi.CThostFtdcReqQueryAccountField_BankAccount_set)
    BankPassWord = property(_thostmduserapi.CThostFtdcReqQueryAccountField_BankPassWord_get, _thostmduserapi.CThostFtdcReqQueryAccountField_BankPassWord_set)
    AccountID = property(_thostmduserapi.CThostFtdcReqQueryAccountField_AccountID_get, _thostmduserapi.CThostFtdcReqQueryAccountField_AccountID_set)
    Password = property(_thostmduserapi.CThostFtdcReqQueryAccountField_Password_get, _thostmduserapi.CThostFtdcReqQueryAccountField_Password_set)
    FutureSerial = property(_thostmduserapi.CThostFtdcReqQueryAccountField_FutureSerial_get, _thostmduserapi.CThostFtdcReqQueryAccountField_FutureSerial_set)
    InstallID = property(_thostmduserapi.CThostFtdcReqQueryAccountField_InstallID_get, _thostmduserapi.CThostFtdcReqQueryAccountField_InstallID_set)
    UserID = property(_thostmduserapi.CThostFtdcReqQueryAccountField_UserID_get, _thostmduserapi.CThostFtdcReqQueryAccountField_UserID_set)
    VerifyCertNoFlag = property(_thostmduserapi.CThostFtdcReqQueryAccountField_VerifyCertNoFlag_get, _thostmduserapi.CThostFtdcReqQueryAccountField_VerifyCertNoFlag_set)
    CurrencyID = property(_thostmduserapi.CThostFtdcReqQueryAccountField_CurrencyID_get, _thostmduserapi.CThostFtdcReqQueryAccountField_CurrencyID_set)
    Digest = property(_thostmduserapi.CThostFtdcReqQueryAccountField_Digest_get, _thostmduserapi.CThostFtdcReqQueryAccountField_Digest_set)
    BankAccType = property(_thostmduserapi.CThostFtdcReqQueryAccountField_BankAccType_get, _thostmduserapi.CThostFtdcReqQueryAccountField_BankAccType_set)
    DeviceID = property(_thostmduserapi.CThostFtdcReqQueryAccountField_DeviceID_get, _thostmduserapi.CThostFtdcReqQueryAccountField_DeviceID_set)
    BankSecuAccType = property(_thostmduserapi.CThostFtdcReqQueryAccountField_BankSecuAccType_get, _thostmduserapi.CThostFtdcReqQueryAccountField_BankSecuAccType_set)
    BrokerIDByBank = property(_thostmduserapi.CThostFtdcReqQueryAccountField_BrokerIDByBank_get, _thostmduserapi.CThostFtdcReqQueryAccountField_BrokerIDByBank_set)
    BankSecuAcc = property(_thostmduserapi.CThostFtdcReqQueryAccountField_BankSecuAcc_get, _thostmduserapi.CThostFtdcReqQueryAccountField_BankSecuAcc_set)
    BankPwdFlag = property(_thostmduserapi.CThostFtdcReqQueryAccountField_BankPwdFlag_get, _thostmduserapi.CThostFtdcReqQueryAccountField_BankPwdFlag_set)
    SecuPwdFlag = property(_thostmduserapi.CThostFtdcReqQueryAccountField_SecuPwdFlag_get, _thostmduserapi.CThostFtdcReqQueryAccountField_SecuPwdFlag_set)
    OperNo = property(_thostmduserapi.CThostFtdcReqQueryAccountField_OperNo_get, _thostmduserapi.CThostFtdcReqQueryAccountField_OperNo_set)
    RequestID = property(_thostmduserapi.CThostFtdcReqQueryAccountField_RequestID_get, _thostmduserapi.CThostFtdcReqQueryAccountField_RequestID_set)
    TID = property(_thostmduserapi.CThostFtdcReqQueryAccountField_TID_get, _thostmduserapi.CThostFtdcReqQueryAccountField_TID_set)
    LongCustomerName = property(_thostmduserapi.CThostFtdcReqQueryAccountField_LongCustomerName_get, _thostmduserapi.CThostFtdcReqQueryAccountField_LongCustomerName_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcReqQueryAccountField_swiginit(self, _thostmduserapi.new_CThostFtdcReqQueryAccountField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcReqQueryAccountField

# Register CThostFtdcReqQueryAccountField in _thostmduserapi:
_thostmduserapi.CThostFtdcReqQueryAccountField_swigregister(CThostFtdcReqQueryAccountField)

class CThostFtdcRspQueryAccountField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradeCode = property(_thostmduserapi.CThostFtdcRspQueryAccountField_TradeCode_get, _thostmduserapi.CThostFtdcRspQueryAccountField_TradeCode_set)
    BankID = property(_thostmduserapi.CThostFtdcRspQueryAccountField_BankID_get, _thostmduserapi.CThostFtdcRspQueryAccountField_BankID_set)
    BankBranchID = property(_thostmduserapi.CThostFtdcRspQueryAccountField_BankBranchID_get, _thostmduserapi.CThostFtdcRspQueryAccountField_BankBranchID_set)
    BrokerID = property(_thostmduserapi.CThostFtdcRspQueryAccountField_BrokerID_get, _thostmduserapi.CThostFtdcRspQueryAccountField_BrokerID_set)
    BrokerBranchID = property(_thostmduserapi.CThostFtdcRspQueryAccountField_BrokerBranchID_get, _thostmduserapi.CThostFtdcRspQueryAccountField_BrokerBranchID_set)
    TradeDate = property(_thostmduserapi.CThostFtdcRspQueryAccountField_TradeDate_get, _thostmduserapi.CThostFtdcRspQueryAccountField_TradeDate_set)
    TradeTime = property(_thostmduserapi.CThostFtdcRspQueryAccountField_TradeTime_get, _thostmduserapi.CThostFtdcRspQueryAccountField_TradeTime_set)
    BankSerial = property(_thostmduserapi.CThostFtdcRspQueryAccountField_BankSerial_get, _thostmduserapi.CThostFtdcRspQueryAccountField_BankSerial_set)
    TradingDay = property(_thostmduserapi.CThostFtdcRspQueryAccountField_TradingDay_get, _thostmduserapi.CThostFtdcRspQueryAccountField_TradingDay_set)
    PlateSerial = property(_thostmduserapi.CThostFtdcRspQueryAccountField_PlateSerial_get, _thostmduserapi.CThostFtdcRspQueryAccountField_PlateSerial_set)
    LastFragment = property(_thostmduserapi.CThostFtdcRspQueryAccountField_LastFragment_get, _thostmduserapi.CThostFtdcRspQueryAccountField_LastFragment_set)
    SessionID = property(_thostmduserapi.CThostFtdcRspQueryAccountField_SessionID_get, _thostmduserapi.CThostFtdcRspQueryAccountField_SessionID_set)
    CustomerName = property(_thostmduserapi.CThostFtdcRspQueryAccountField_CustomerName_get, _thostmduserapi.CThostFtdcRspQueryAccountField_CustomerName_set)
    IdCardType = property(_thostmduserapi.CThostFtdcRspQueryAccountField_IdCardType_get, _thostmduserapi.CThostFtdcRspQueryAccountField_IdCardType_set)
    IdentifiedCardNo = property(_thostmduserapi.CThostFtdcRspQueryAccountField_IdentifiedCardNo_get, _thostmduserapi.CThostFtdcRspQueryAccountField_IdentifiedCardNo_set)
    CustType = property(_thostmduserapi.CThostFtdcRspQueryAccountField_CustType_get, _thostmduserapi.CThostFtdcRspQueryAccountField_CustType_set)
    BankAccount = property(_thostmduserapi.CThostFtdcRspQueryAccountField_BankAccount_get, _thostmduserapi.CThostFtdcRspQueryAccountField_BankAccount_set)
    BankPassWord = property(_thostmduserapi.CThostFtdcRspQueryAccountField_BankPassWord_get, _thostmduserapi.CThostFtdcRspQueryAccountField_BankPassWord_set)
    AccountID = property(_thostmduserapi.CThostFtdcRspQueryAccountField_AccountID_get, _thostmduserapi.CThostFtdcRspQueryAccountField_AccountID_set)
    Password = property(_thostmduserapi.CThostFtdcRspQueryAccountField_Password_get, _thostmduserapi.CThostFtdcRspQueryAccountField_Password_set)
    FutureSerial = property(_thostmduserapi.CThostFtdcRspQueryAccountField_FutureSerial_get, _thostmduserapi.CThostFtdcRspQueryAccountField_FutureSerial_set)
    InstallID = property(_thostmduserapi.CThostFtdcRspQueryAccountField_InstallID_get, _thostmduserapi.CThostFtdcRspQueryAccountField_InstallID_set)
    UserID = property(_thostmduserapi.CThostFtdcRspQueryAccountField_UserID_get, _thostmduserapi.CThostFtdcRspQueryAccountField_UserID_set)
    VerifyCertNoFlag = property(_thostmduserapi.CThostFtdcRspQueryAccountField_VerifyCertNoFlag_get, _thostmduserapi.CThostFtdcRspQueryAccountField_VerifyCertNoFlag_set)
    CurrencyID = property(_thostmduserapi.CThostFtdcRspQueryAccountField_CurrencyID_get, _thostmduserapi.CThostFtdcRspQueryAccountField_CurrencyID_set)
    Digest = property(_thostmduserapi.CThostFtdcRspQueryAccountField_Digest_get, _thostmduserapi.CThostFtdcRspQueryAccountField_Digest_set)
    BankAccType = property(_thostmduserapi.CThostFtdcRspQueryAccountField_BankAccType_get, _thostmduserapi.CThostFtdcRspQueryAccountField_BankAccType_set)
    DeviceID = property(_thostmduserapi.CThostFtdcRspQueryAccountField_DeviceID_get, _thostmduserapi.CThostFtdcRspQueryAccountField_DeviceID_set)
    BankSecuAccType = property(_thostmduserapi.CThostFtdcRspQueryAccountField_BankSecuAccType_get, _thostmduserapi.CThostFtdcRspQueryAccountField_BankSecuAccType_set)
    BrokerIDByBank = property(_thostmduserapi.CThostFtdcRspQueryAccountField_BrokerIDByBank_get, _thostmduserapi.CThostFtdcRspQueryAccountField_BrokerIDByBank_set)
    BankSecuAcc = property(_thostmduserapi.CThostFtdcRspQueryAccountField_BankSecuAcc_get, _thostmduserapi.CThostFtdcRspQueryAccountField_BankSecuAcc_set)
    BankPwdFlag = property(_thostmduserapi.CThostFtdcRspQueryAccountField_BankPwdFlag_get, _thostmduserapi.CThostFtdcRspQueryAccountField_BankPwdFlag_set)
    SecuPwdFlag = property(_thostmduserapi.CThostFtdcRspQueryAccountField_SecuPwdFlag_get, _thostmduserapi.CThostFtdcRspQueryAccountField_SecuPwdFlag_set)
    OperNo = property(_thostmduserapi.CThostFtdcRspQueryAccountField_OperNo_get, _thostmduserapi.CThostFtdcRspQueryAccountField_OperNo_set)
    RequestID = property(_thostmduserapi.CThostFtdcRspQueryAccountField_RequestID_get, _thostmduserapi.CThostFtdcRspQueryAccountField_RequestID_set)
    TID = property(_thostmduserapi.CThostFtdcRspQueryAccountField_TID_get, _thostmduserapi.CThostFtdcRspQueryAccountField_TID_set)
    BankUseAmount = property(_thostmduserapi.CThostFtdcRspQueryAccountField_BankUseAmount_get, _thostmduserapi.CThostFtdcRspQueryAccountField_BankUseAmount_set)
    BankFetchAmount = property(_thostmduserapi.CThostFtdcRspQueryAccountField_BankFetchAmount_get, _thostmduserapi.CThostFtdcRspQueryAccountField_BankFetchAmount_set)
    LongCustomerName = property(_thostmduserapi.CThostFtdcRspQueryAccountField_LongCustomerName_get, _thostmduserapi.CThostFtdcRspQueryAccountField_LongCustomerName_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcRspQueryAccountField_swiginit(self, _thostmduserapi.new_CThostFtdcRspQueryAccountField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcRspQueryAccountField

# Register CThostFtdcRspQueryAccountField in _thostmduserapi:
_thostmduserapi.CThostFtdcRspQueryAccountField_swigregister(CThostFtdcRspQueryAccountField)

class CThostFtdcFutureSignIOField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradeCode = property(_thostmduserapi.CThostFtdcFutureSignIOField_TradeCode_get, _thostmduserapi.CThostFtdcFutureSignIOField_TradeCode_set)
    BankID = property(_thostmduserapi.CThostFtdcFutureSignIOField_BankID_get, _thostmduserapi.CThostFtdcFutureSignIOField_BankID_set)
    BankBranchID = property(_thostmduserapi.CThostFtdcFutureSignIOField_BankBranchID_get, _thostmduserapi.CThostFtdcFutureSignIOField_BankBranchID_set)
    BrokerID = property(_thostmduserapi.CThostFtdcFutureSignIOField_BrokerID_get, _thostmduserapi.CThostFtdcFutureSignIOField_BrokerID_set)
    BrokerBranchID = property(_thostmduserapi.CThostFtdcFutureSignIOField_BrokerBranchID_get, _thostmduserapi.CThostFtdcFutureSignIOField_BrokerBranchID_set)
    TradeDate = property(_thostmduserapi.CThostFtdcFutureSignIOField_TradeDate_get, _thostmduserapi.CThostFtdcFutureSignIOField_TradeDate_set)
    TradeTime = property(_thostmduserapi.CThostFtdcFutureSignIOField_TradeTime_get, _thostmduserapi.CThostFtdcFutureSignIOField_TradeTime_set)
    BankSerial = property(_thostmduserapi.CThostFtdcFutureSignIOField_BankSerial_get, _thostmduserapi.CThostFtdcFutureSignIOField_BankSerial_set)
    TradingDay = property(_thostmduserapi.CThostFtdcFutureSignIOField_TradingDay_get, _thostmduserapi.CThostFtdcFutureSignIOField_TradingDay_set)
    PlateSerial = property(_thostmduserapi.CThostFtdcFutureSignIOField_PlateSerial_get, _thostmduserapi.CThostFtdcFutureSignIOField_PlateSerial_set)
    LastFragment = property(_thostmduserapi.CThostFtdcFutureSignIOField_LastFragment_get, _thostmduserapi.CThostFtdcFutureSignIOField_LastFragment_set)
    SessionID = property(_thostmduserapi.CThostFtdcFutureSignIOField_SessionID_get, _thostmduserapi.CThostFtdcFutureSignIOField_SessionID_set)
    InstallID = property(_thostmduserapi.CThostFtdcFutureSignIOField_InstallID_get, _thostmduserapi.CThostFtdcFutureSignIOField_InstallID_set)
    UserID = property(_thostmduserapi.CThostFtdcFutureSignIOField_UserID_get, _thostmduserapi.CThostFtdcFutureSignIOField_UserID_set)
    Digest = property(_thostmduserapi.CThostFtdcFutureSignIOField_Digest_get, _thostmduserapi.CThostFtdcFutureSignIOField_Digest_set)
    CurrencyID = property(_thostmduserapi.CThostFtdcFutureSignIOField_CurrencyID_get, _thostmduserapi.CThostFtdcFutureSignIOField_CurrencyID_set)
    DeviceID = property(_thostmduserapi.CThostFtdcFutureSignIOField_DeviceID_get, _thostmduserapi.CThostFtdcFutureSignIOField_DeviceID_set)
    BrokerIDByBank = property(_thostmduserapi.CThostFtdcFutureSignIOField_BrokerIDByBank_get, _thostmduserapi.CThostFtdcFutureSignIOField_BrokerIDByBank_set)
    OperNo = property(_thostmduserapi.CThostFtdcFutureSignIOField_OperNo_get, _thostmduserapi.CThostFtdcFutureSignIOField_OperNo_set)
    RequestID = property(_thostmduserapi.CThostFtdcFutureSignIOField_RequestID_get, _thostmduserapi.CThostFtdcFutureSignIOField_RequestID_set)
    TID = property(_thostmduserapi.CThostFtdcFutureSignIOField_TID_get, _thostmduserapi.CThostFtdcFutureSignIOField_TID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcFutureSignIOField_swiginit(self, _thostmduserapi.new_CThostFtdcFutureSignIOField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcFutureSignIOField

# Register CThostFtdcFutureSignIOField in _thostmduserapi:
_thostmduserapi.CThostFtdcFutureSignIOField_swigregister(CThostFtdcFutureSignIOField)

class CThostFtdcRspFutureSignInField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradeCode = property(_thostmduserapi.CThostFtdcRspFutureSignInField_TradeCode_get, _thostmduserapi.CThostFtdcRspFutureSignInField_TradeCode_set)
    BankID = property(_thostmduserapi.CThostFtdcRspFutureSignInField_BankID_get, _thostmduserapi.CThostFtdcRspFutureSignInField_BankID_set)
    BankBranchID = property(_thostmduserapi.CThostFtdcRspFutureSignInField_BankBranchID_get, _thostmduserapi.CThostFtdcRspFutureSignInField_BankBranchID_set)
    BrokerID = property(_thostmduserapi.CThostFtdcRspFutureSignInField_BrokerID_get, _thostmduserapi.CThostFtdcRspFutureSignInField_BrokerID_set)
    BrokerBranchID = property(_thostmduserapi.CThostFtdcRspFutureSignInField_BrokerBranchID_get, _thostmduserapi.CThostFtdcRspFutureSignInField_BrokerBranchID_set)
    TradeDate = property(_thostmduserapi.CThostFtdcRspFutureSignInField_TradeDate_get, _thostmduserapi.CThostFtdcRspFutureSignInField_TradeDate_set)
    TradeTime = property(_thostmduserapi.CThostFtdcRspFutureSignInField_TradeTime_get, _thostmduserapi.CThostFtdcRspFutureSignInField_TradeTime_set)
    BankSerial = property(_thostmduserapi.CThostFtdcRspFutureSignInField_BankSerial_get, _thostmduserapi.CThostFtdcRspFutureSignInField_BankSerial_set)
    TradingDay = property(_thostmduserapi.CThostFtdcRspFutureSignInField_TradingDay_get, _thostmduserapi.CThostFtdcRspFutureSignInField_TradingDay_set)
    PlateSerial = property(_thostmduserapi.CThostFtdcRspFutureSignInField_PlateSerial_get, _thostmduserapi.CThostFtdcRspFutureSignInField_PlateSerial_set)
    LastFragment = property(_thostmduserapi.CThostFtdcRspFutureSignInField_LastFragment_get, _thostmduserapi.CThostFtdcRspFutureSignInField_LastFragment_set)
    SessionID = property(_thostmduserapi.CThostFtdcRspFutureSignInField_SessionID_get, _thostmduserapi.CThostFtdcRspFutureSignInField_SessionID_set)
    InstallID = property(_thostmduserapi.CThostFtdcRspFutureSignInField_InstallID_get, _thostmduserapi.CThostFtdcRspFutureSignInField_InstallID_set)
    UserID = property(_thostmduserapi.CThostFtdcRspFutureSignInField_UserID_get, _thostmduserapi.CThostFtdcRspFutureSignInField_UserID_set)
    Digest = property(_thostmduserapi.CThostFtdcRspFutureSignInField_Digest_get, _thostmduserapi.CThostFtdcRspFutureSignInField_Digest_set)
    CurrencyID = property(_thostmduserapi.CThostFtdcRspFutureSignInField_CurrencyID_get, _thostmduserapi.CThostFtdcRspFutureSignInField_CurrencyID_set)
    DeviceID = property(_thostmduserapi.CThostFtdcRspFutureSignInField_DeviceID_get, _thostmduserapi.CThostFtdcRspFutureSignInField_DeviceID_set)
    BrokerIDByBank = property(_thostmduserapi.CThostFtdcRspFutureSignInField_BrokerIDByBank_get, _thostmduserapi.CThostFtdcRspFutureSignInField_BrokerIDByBank_set)
    OperNo = property(_thostmduserapi.CThostFtdcRspFutureSignInField_OperNo_get, _thostmduserapi.CThostFtdcRspFutureSignInField_OperNo_set)
    RequestID = property(_thostmduserapi.CThostFtdcRspFutureSignInField_RequestID_get, _thostmduserapi.CThostFtdcRspFutureSignInField_RequestID_set)
    TID = property(_thostmduserapi.CThostFtdcRspFutureSignInField_TID_get, _thostmduserapi.CThostFtdcRspFutureSignInField_TID_set)
    ErrorID = property(_thostmduserapi.CThostFtdcRspFutureSignInField_ErrorID_get, _thostmduserapi.CThostFtdcRspFutureSignInField_ErrorID_set)
    ErrorMsg = property(_thostmduserapi.CThostFtdcRspFutureSignInField_ErrorMsg_get, _thostmduserapi.CThostFtdcRspFutureSignInField_ErrorMsg_set)
    PinKey = property(_thostmduserapi.CThostFtdcRspFutureSignInField_PinKey_get, _thostmduserapi.CThostFtdcRspFutureSignInField_PinKey_set)
    MacKey = property(_thostmduserapi.CThostFtdcRspFutureSignInField_MacKey_get, _thostmduserapi.CThostFtdcRspFutureSignInField_MacKey_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcRspFutureSignInField_swiginit(self, _thostmduserapi.new_CThostFtdcRspFutureSignInField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcRspFutureSignInField

# Register CThostFtdcRspFutureSignInField in _thostmduserapi:
_thostmduserapi.CThostFtdcRspFutureSignInField_swigregister(CThostFtdcRspFutureSignInField)

class CThostFtdcReqFutureSignOutField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradeCode = property(_thostmduserapi.CThostFtdcReqFutureSignOutField_TradeCode_get, _thostmduserapi.CThostFtdcReqFutureSignOutField_TradeCode_set)
    BankID = property(_thostmduserapi.CThostFtdcReqFutureSignOutField_BankID_get, _thostmduserapi.CThostFtdcReqFutureSignOutField_BankID_set)
    BankBranchID = property(_thostmduserapi.CThostFtdcReqFutureSignOutField_BankBranchID_get, _thostmduserapi.CThostFtdcReqFutureSignOutField_BankBranchID_set)
    BrokerID = property(_thostmduserapi.CThostFtdcReqFutureSignOutField_BrokerID_get, _thostmduserapi.CThostFtdcReqFutureSignOutField_BrokerID_set)
    BrokerBranchID = property(_thostmduserapi.CThostFtdcReqFutureSignOutField_BrokerBranchID_get, _thostmduserapi.CThostFtdcReqFutureSignOutField_BrokerBranchID_set)
    TradeDate = property(_thostmduserapi.CThostFtdcReqFutureSignOutField_TradeDate_get, _thostmduserapi.CThostFtdcReqFutureSignOutField_TradeDate_set)
    TradeTime = property(_thostmduserapi.CThostFtdcReqFutureSignOutField_TradeTime_get, _thostmduserapi.CThostFtdcReqFutureSignOutField_TradeTime_set)
    BankSerial = property(_thostmduserapi.CThostFtdcReqFutureSignOutField_BankSerial_get, _thostmduserapi.CThostFtdcReqFutureSignOutField_BankSerial_set)
    TradingDay = property(_thostmduserapi.CThostFtdcReqFutureSignOutField_TradingDay_get, _thostmduserapi.CThostFtdcReqFutureSignOutField_TradingDay_set)
    PlateSerial = property(_thostmduserapi.CThostFtdcReqFutureSignOutField_PlateSerial_get, _thostmduserapi.CThostFtdcReqFutureSignOutField_PlateSerial_set)
    LastFragment = property(_thostmduserapi.CThostFtdcReqFutureSignOutField_LastFragment_get, _thostmduserapi.CThostFtdcReqFutureSignOutField_LastFragment_set)
    SessionID = property(_thostmduserapi.CThostFtdcReqFutureSignOutField_SessionID_get, _thostmduserapi.CThostFtdcReqFutureSignOutField_SessionID_set)
    InstallID = property(_thostmduserapi.CThostFtdcReqFutureSignOutField_InstallID_get, _thostmduserapi.CThostFtdcReqFutureSignOutField_InstallID_set)
    UserID = property(_thostmduserapi.CThostFtdcReqFutureSignOutField_UserID_get, _thostmduserapi.CThostFtdcReqFutureSignOutField_UserID_set)
    Digest = property(_thostmduserapi.CThostFtdcReqFutureSignOutField_Digest_get, _thostmduserapi.CThostFtdcReqFutureSignOutField_Digest_set)
    CurrencyID = property(_thostmduserapi.CThostFtdcReqFutureSignOutField_CurrencyID_get, _thostmduserapi.CThostFtdcReqFutureSignOutField_CurrencyID_set)
    DeviceID = property(_thostmduserapi.CThostFtdcReqFutureSignOutField_DeviceID_get, _thostmduserapi.CThostFtdcReqFutureSignOutField_DeviceID_set)
    BrokerIDByBank = property(_thostmduserapi.CThostFtdcReqFutureSignOutField_BrokerIDByBank_get, _thostmduserapi.CThostFtdcReqFutureSignOutField_BrokerIDByBank_set)
    OperNo = property(_thostmduserapi.CThostFtdcReqFutureSignOutField_OperNo_get, _thostmduserapi.CThostFtdcReqFutureSignOutField_OperNo_set)
    RequestID = property(_thostmduserapi.CThostFtdcReqFutureSignOutField_RequestID_get, _thostmduserapi.CThostFtdcReqFutureSignOutField_RequestID_set)
    TID = property(_thostmduserapi.CThostFtdcReqFutureSignOutField_TID_get, _thostmduserapi.CThostFtdcReqFutureSignOutField_TID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcReqFutureSignOutField_swiginit(self, _thostmduserapi.new_CThostFtdcReqFutureSignOutField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcReqFutureSignOutField

# Register CThostFtdcReqFutureSignOutField in _thostmduserapi:
_thostmduserapi.CThostFtdcReqFutureSignOutField_swigregister(CThostFtdcReqFutureSignOutField)

class CThostFtdcRspFutureSignOutField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradeCode = property(_thostmduserapi.CThostFtdcRspFutureSignOutField_TradeCode_get, _thostmduserapi.CThostFtdcRspFutureSignOutField_TradeCode_set)
    BankID = property(_thostmduserapi.CThostFtdcRspFutureSignOutField_BankID_get, _thostmduserapi.CThostFtdcRspFutureSignOutField_BankID_set)
    BankBranchID = property(_thostmduserapi.CThostFtdcRspFutureSignOutField_BankBranchID_get, _thostmduserapi.CThostFtdcRspFutureSignOutField_BankBranchID_set)
    BrokerID = property(_thostmduserapi.CThostFtdcRspFutureSignOutField_BrokerID_get, _thostmduserapi.CThostFtdcRspFutureSignOutField_BrokerID_set)
    BrokerBranchID = property(_thostmduserapi.CThostFtdcRspFutureSignOutField_BrokerBranchID_get, _thostmduserapi.CThostFtdcRspFutureSignOutField_BrokerBranchID_set)
    TradeDate = property(_thostmduserapi.CThostFtdcRspFutureSignOutField_TradeDate_get, _thostmduserapi.CThostFtdcRspFutureSignOutField_TradeDate_set)
    TradeTime = property(_thostmduserapi.CThostFtdcRspFutureSignOutField_TradeTime_get, _thostmduserapi.CThostFtdcRspFutureSignOutField_TradeTime_set)
    BankSerial = property(_thostmduserapi.CThostFtdcRspFutureSignOutField_BankSerial_get, _thostmduserapi.CThostFtdcRspFutureSignOutField_BankSerial_set)
    TradingDay = property(_thostmduserapi.CThostFtdcRspFutureSignOutField_TradingDay_get, _thostmduserapi.CThostFtdcRspFutureSignOutField_TradingDay_set)
    PlateSerial = property(_thostmduserapi.CThostFtdcRspFutureSignOutField_PlateSerial_get, _thostmduserapi.CThostFtdcRspFutureSignOutField_PlateSerial_set)
    LastFragment = property(_thostmduserapi.CThostFtdcRspFutureSignOutField_LastFragment_get, _thostmduserapi.CThostFtdcRspFutureSignOutField_LastFragment_set)
    SessionID = property(_thostmduserapi.CThostFtdcRspFutureSignOutField_SessionID_get, _thostmduserapi.CThostFtdcRspFutureSignOutField_SessionID_set)
    InstallID = property(_thostmduserapi.CThostFtdcRspFutureSignOutField_InstallID_get, _thostmduserapi.CThostFtdcRspFutureSignOutField_InstallID_set)
    UserID = property(_thostmduserapi.CThostFtdcRspFutureSignOutField_UserID_get, _thostmduserapi.CThostFtdcRspFutureSignOutField_UserID_set)
    Digest = property(_thostmduserapi.CThostFtdcRspFutureSignOutField_Digest_get, _thostmduserapi.CThostFtdcRspFutureSignOutField_Digest_set)
    CurrencyID = property(_thostmduserapi.CThostFtdcRspFutureSignOutField_CurrencyID_get, _thostmduserapi.CThostFtdcRspFutureSignOutField_CurrencyID_set)
    DeviceID = property(_thostmduserapi.CThostFtdcRspFutureSignOutField_DeviceID_get, _thostmduserapi.CThostFtdcRspFutureSignOutField_DeviceID_set)
    BrokerIDByBank = property(_thostmduserapi.CThostFtdcRspFutureSignOutField_BrokerIDByBank_get, _thostmduserapi.CThostFtdcRspFutureSignOutField_BrokerIDByBank_set)
    OperNo = property(_thostmduserapi.CThostFtdcRspFutureSignOutField_OperNo_get, _thostmduserapi.CThostFtdcRspFutureSignOutField_OperNo_set)
    RequestID = property(_thostmduserapi.CThostFtdcRspFutureSignOutField_RequestID_get, _thostmduserapi.CThostFtdcRspFutureSignOutField_RequestID_set)
    TID = property(_thostmduserapi.CThostFtdcRspFutureSignOutField_TID_get, _thostmduserapi.CThostFtdcRspFutureSignOutField_TID_set)
    ErrorID = property(_thostmduserapi.CThostFtdcRspFutureSignOutField_ErrorID_get, _thostmduserapi.CThostFtdcRspFutureSignOutField_ErrorID_set)
    ErrorMsg = property(_thostmduserapi.CThostFtdcRspFutureSignOutField_ErrorMsg_get, _thostmduserapi.CThostFtdcRspFutureSignOutField_ErrorMsg_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcRspFutureSignOutField_swiginit(self, _thostmduserapi.new_CThostFtdcRspFutureSignOutField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcRspFutureSignOutField

# Register CThostFtdcRspFutureSignOutField in _thostmduserapi:
_thostmduserapi.CThostFtdcRspFutureSignOutField_swigregister(CThostFtdcRspFutureSignOutField)

class CThostFtdcReqQueryTradeResultBySerialField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradeCode = property(_thostmduserapi.CThostFtdcReqQueryTradeResultBySerialField_TradeCode_get, _thostmduserapi.CThostFtdcReqQueryTradeResultBySerialField_TradeCode_set)
    BankID = property(_thostmduserapi.CThostFtdcReqQueryTradeResultBySerialField_BankID_get, _thostmduserapi.CThostFtdcReqQueryTradeResultBySerialField_BankID_set)
    BankBranchID = property(_thostmduserapi.CThostFtdcReqQueryTradeResultBySerialField_BankBranchID_get, _thostmduserapi.CThostFtdcReqQueryTradeResultBySerialField_BankBranchID_set)
    BrokerID = property(_thostmduserapi.CThostFtdcReqQueryTradeResultBySerialField_BrokerID_get, _thostmduserapi.CThostFtdcReqQueryTradeResultBySerialField_BrokerID_set)
    BrokerBranchID = property(_thostmduserapi.CThostFtdcReqQueryTradeResultBySerialField_BrokerBranchID_get, _thostmduserapi.CThostFtdcReqQueryTradeResultBySerialField_BrokerBranchID_set)
    TradeDate = property(_thostmduserapi.CThostFtdcReqQueryTradeResultBySerialField_TradeDate_get, _thostmduserapi.CThostFtdcReqQueryTradeResultBySerialField_TradeDate_set)
    TradeTime = property(_thostmduserapi.CThostFtdcReqQueryTradeResultBySerialField_TradeTime_get, _thostmduserapi.CThostFtdcReqQueryTradeResultBySerialField_TradeTime_set)
    BankSerial = property(_thostmduserapi.CThostFtdcReqQueryTradeResultBySerialField_BankSerial_get, _thostmduserapi.CThostFtdcReqQueryTradeResultBySerialField_BankSerial_set)
    TradingDay = property(_thostmduserapi.CThostFtdcReqQueryTradeResultBySerialField_TradingDay_get, _thostmduserapi.CThostFtdcReqQueryTradeResultBySerialField_TradingDay_set)
    PlateSerial = property(_thostmduserapi.CThostFtdcReqQueryTradeResultBySerialField_PlateSerial_get, _thostmduserapi.CThostFtdcReqQueryTradeResultBySerialField_PlateSerial_set)
    LastFragment = property(_thostmduserapi.CThostFtdcReqQueryTradeResultBySerialField_LastFragment_get, _thostmduserapi.CThostFtdcReqQueryTradeResultBySerialField_LastFragment_set)
    SessionID = property(_thostmduserapi.CThostFtdcReqQueryTradeResultBySerialField_SessionID_get, _thostmduserapi.CThostFtdcReqQueryTradeResultBySerialField_SessionID_set)
    Reference = property(_thostmduserapi.CThostFtdcReqQueryTradeResultBySerialField_Reference_get, _thostmduserapi.CThostFtdcReqQueryTradeResultBySerialField_Reference_set)
    RefrenceIssureType = property(_thostmduserapi.CThostFtdcReqQueryTradeResultBySerialField_RefrenceIssureType_get, _thostmduserapi.CThostFtdcReqQueryTradeResultBySerialField_RefrenceIssureType_set)
    RefrenceIssure = property(_thostmduserapi.CThostFtdcReqQueryTradeResultBySerialField_RefrenceIssure_get, _thostmduserapi.CThostFtdcReqQueryTradeResultBySerialField_RefrenceIssure_set)
    CustomerName = property(_thostmduserapi.CThostFtdcReqQueryTradeResultBySerialField_CustomerName_get, _thostmduserapi.CThostFtdcReqQueryTradeResultBySerialField_CustomerName_set)
    IdCardType = property(_thostmduserapi.CThostFtdcReqQueryTradeResultBySerialField_IdCardType_get, _thostmduserapi.CThostFtdcReqQueryTradeResultBySerialField_IdCardType_set)
    IdentifiedCardNo = property(_thostmduserapi.CThostFtdcReqQueryTradeResultBySerialField_IdentifiedCardNo_get, _thostmduserapi.CThostFtdcReqQueryTradeResultBySerialField_IdentifiedCardNo_set)
    CustType = property(_thostmduserapi.CThostFtdcReqQueryTradeResultBySerialField_CustType_get, _thostmduserapi.CThostFtdcReqQueryTradeResultBySerialField_CustType_set)
    BankAccount = property(_thostmduserapi.CThostFtdcReqQueryTradeResultBySerialField_BankAccount_get, _thostmduserapi.CThostFtdcReqQueryTradeResultBySerialField_BankAccount_set)
    BankPassWord = property(_thostmduserapi.CThostFtdcReqQueryTradeResultBySerialField_BankPassWord_get, _thostmduserapi.CThostFtdcReqQueryTradeResultBySerialField_BankPassWord_set)
    AccountID = property(_thostmduserapi.CThostFtdcReqQueryTradeResultBySerialField_AccountID_get, _thostmduserapi.CThostFtdcReqQueryTradeResultBySerialField_AccountID_set)
    Password = property(_thostmduserapi.CThostFtdcReqQueryTradeResultBySerialField_Password_get, _thostmduserapi.CThostFtdcReqQueryTradeResultBySerialField_Password_set)
    CurrencyID = property(_thostmduserapi.CThostFtdcReqQueryTradeResultBySerialField_CurrencyID_get, _thostmduserapi.CThostFtdcReqQueryTradeResultBySerialField_CurrencyID_set)
    TradeAmount = property(_thostmduserapi.CThostFtdcReqQueryTradeResultBySerialField_TradeAmount_get, _thostmduserapi.CThostFtdcReqQueryTradeResultBySerialField_TradeAmount_set)
    Digest = property(_thostmduserapi.CThostFtdcReqQueryTradeResultBySerialField_Digest_get, _thostmduserapi.CThostFtdcReqQueryTradeResultBySerialField_Digest_set)
    LongCustomerName = property(_thostmduserapi.CThostFtdcReqQueryTradeResultBySerialField_LongCustomerName_get, _thostmduserapi.CThostFtdcReqQueryTradeResultBySerialField_LongCustomerName_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcReqQueryTradeResultBySerialField_swiginit(self, _thostmduserapi.new_CThostFtdcReqQueryTradeResultBySerialField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcReqQueryTradeResultBySerialField

# Register CThostFtdcReqQueryTradeResultBySerialField in _thostmduserapi:
_thostmduserapi.CThostFtdcReqQueryTradeResultBySerialField_swigregister(CThostFtdcReqQueryTradeResultBySerialField)

class CThostFtdcRspQueryTradeResultBySerialField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradeCode = property(_thostmduserapi.CThostFtdcRspQueryTradeResultBySerialField_TradeCode_get, _thostmduserapi.CThostFtdcRspQueryTradeResultBySerialField_TradeCode_set)
    BankID = property(_thostmduserapi.CThostFtdcRspQueryTradeResultBySerialField_BankID_get, _thostmduserapi.CThostFtdcRspQueryTradeResultBySerialField_BankID_set)
    BankBranchID = property(_thostmduserapi.CThostFtdcRspQueryTradeResultBySerialField_BankBranchID_get, _thostmduserapi.CThostFtdcRspQueryTradeResultBySerialField_BankBranchID_set)
    BrokerID = property(_thostmduserapi.CThostFtdcRspQueryTradeResultBySerialField_BrokerID_get, _thostmduserapi.CThostFtdcRspQueryTradeResultBySerialField_BrokerID_set)
    BrokerBranchID = property(_thostmduserapi.CThostFtdcRspQueryTradeResultBySerialField_BrokerBranchID_get, _thostmduserapi.CThostFtdcRspQueryTradeResultBySerialField_BrokerBranchID_set)
    TradeDate = property(_thostmduserapi.CThostFtdcRspQueryTradeResultBySerialField_TradeDate_get, _thostmduserapi.CThostFtdcRspQueryTradeResultBySerialField_TradeDate_set)
    TradeTime = property(_thostmduserapi.CThostFtdcRspQueryTradeResultBySerialField_TradeTime_get, _thostmduserapi.CThostFtdcRspQueryTradeResultBySerialField_TradeTime_set)
    BankSerial = property(_thostmduserapi.CThostFtdcRspQueryTradeResultBySerialField_BankSerial_get, _thostmduserapi.CThostFtdcRspQueryTradeResultBySerialField_BankSerial_set)
    TradingDay = property(_thostmduserapi.CThostFtdcRspQueryTradeResultBySerialField_TradingDay_get, _thostmduserapi.CThostFtdcRspQueryTradeResultBySerialField_TradingDay_set)
    PlateSerial = property(_thostmduserapi.CThostFtdcRspQueryTradeResultBySerialField_PlateSerial_get, _thostmduserapi.CThostFtdcRspQueryTradeResultBySerialField_PlateSerial_set)
    LastFragment = property(_thostmduserapi.CThostFtdcRspQueryTradeResultBySerialField_LastFragment_get, _thostmduserapi.CThostFtdcRspQueryTradeResultBySerialField_LastFragment_set)
    SessionID = property(_thostmduserapi.CThostFtdcRspQueryTradeResultBySerialField_SessionID_get, _thostmduserapi.CThostFtdcRspQueryTradeResultBySerialField_SessionID_set)
    ErrorID = property(_thostmduserapi.CThostFtdcRspQueryTradeResultBySerialField_ErrorID_get, _thostmduserapi.CThostFtdcRspQueryTradeResultBySerialField_ErrorID_set)
    ErrorMsg = property(_thostmduserapi.CThostFtdcRspQueryTradeResultBySerialField_ErrorMsg_get, _thostmduserapi.CThostFtdcRspQueryTradeResultBySerialField_ErrorMsg_set)
    Reference = property(_thostmduserapi.CThostFtdcRspQueryTradeResultBySerialField_Reference_get, _thostmduserapi.CThostFtdcRspQueryTradeResultBySerialField_Reference_set)
    RefrenceIssureType = property(_thostmduserapi.CThostFtdcRspQueryTradeResultBySerialField_RefrenceIssureType_get, _thostmduserapi.CThostFtdcRspQueryTradeResultBySerialField_RefrenceIssureType_set)
    RefrenceIssure = property(_thostmduserapi.CThostFtdcRspQueryTradeResultBySerialField_RefrenceIssure_get, _thostmduserapi.CThostFtdcRspQueryTradeResultBySerialField_RefrenceIssure_set)
    OriginReturnCode = property(_thostmduserapi.CThostFtdcRspQueryTradeResultBySerialField_OriginReturnCode_get, _thostmduserapi.CThostFtdcRspQueryTradeResultBySerialField_OriginReturnCode_set)
    OriginDescrInfoForReturnCode = property(_thostmduserapi.CThostFtdcRspQueryTradeResultBySerialField_OriginDescrInfoForReturnCode_get, _thostmduserapi.CThostFtdcRspQueryTradeResultBySerialField_OriginDescrInfoForReturnCode_set)
    BankAccount = property(_thostmduserapi.CThostFtdcRspQueryTradeResultBySerialField_BankAccount_get, _thostmduserapi.CThostFtdcRspQueryTradeResultBySerialField_BankAccount_set)
    BankPassWord = property(_thostmduserapi.CThostFtdcRspQueryTradeResultBySerialField_BankPassWord_get, _thostmduserapi.CThostFtdcRspQueryTradeResultBySerialField_BankPassWord_set)
    AccountID = property(_thostmduserapi.CThostFtdcRspQueryTradeResultBySerialField_AccountID_get, _thostmduserapi.CThostFtdcRspQueryTradeResultBySerialField_AccountID_set)
    Password = property(_thostmduserapi.CThostFtdcRspQueryTradeResultBySerialField_Password_get, _thostmduserapi.CThostFtdcRspQueryTradeResultBySerialField_Password_set)
    CurrencyID = property(_thostmduserapi.CThostFtdcRspQueryTradeResultBySerialField_CurrencyID_get, _thostmduserapi.CThostFtdcRspQueryTradeResultBySerialField_CurrencyID_set)
    TradeAmount = property(_thostmduserapi.CThostFtdcRspQueryTradeResultBySerialField_TradeAmount_get, _thostmduserapi.CThostFtdcRspQueryTradeResultBySerialField_TradeAmount_set)
    Digest = property(_thostmduserapi.CThostFtdcRspQueryTradeResultBySerialField_Digest_get, _thostmduserapi.CThostFtdcRspQueryTradeResultBySerialField_Digest_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcRspQueryTradeResultBySerialField_swiginit(self, _thostmduserapi.new_CThostFtdcRspQueryTradeResultBySerialField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcRspQueryTradeResultBySerialField

# Register CThostFtdcRspQueryTradeResultBySerialField in _thostmduserapi:
_thostmduserapi.CThostFtdcRspQueryTradeResultBySerialField_swigregister(CThostFtdcRspQueryTradeResultBySerialField)

class CThostFtdcReqDayEndFileReadyField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradeCode = property(_thostmduserapi.CThostFtdcReqDayEndFileReadyField_TradeCode_get, _thostmduserapi.CThostFtdcReqDayEndFileReadyField_TradeCode_set)
    BankID = property(_thostmduserapi.CThostFtdcReqDayEndFileReadyField_BankID_get, _thostmduserapi.CThostFtdcReqDayEndFileReadyField_BankID_set)
    BankBranchID = property(_thostmduserapi.CThostFtdcReqDayEndFileReadyField_BankBranchID_get, _thostmduserapi.CThostFtdcReqDayEndFileReadyField_BankBranchID_set)
    BrokerID = property(_thostmduserapi.CThostFtdcReqDayEndFileReadyField_BrokerID_get, _thostmduserapi.CThostFtdcReqDayEndFileReadyField_BrokerID_set)
    BrokerBranchID = property(_thostmduserapi.CThostFtdcReqDayEndFileReadyField_BrokerBranchID_get, _thostmduserapi.CThostFtdcReqDayEndFileReadyField_BrokerBranchID_set)
    TradeDate = property(_thostmduserapi.CThostFtdcReqDayEndFileReadyField_TradeDate_get, _thostmduserapi.CThostFtdcReqDayEndFileReadyField_TradeDate_set)
    TradeTime = property(_thostmduserapi.CThostFtdcReqDayEndFileReadyField_TradeTime_get, _thostmduserapi.CThostFtdcReqDayEndFileReadyField_TradeTime_set)
    BankSerial = property(_thostmduserapi.CThostFtdcReqDayEndFileReadyField_BankSerial_get, _thostmduserapi.CThostFtdcReqDayEndFileReadyField_BankSerial_set)
    TradingDay = property(_thostmduserapi.CThostFtdcReqDayEndFileReadyField_TradingDay_get, _thostmduserapi.CThostFtdcReqDayEndFileReadyField_TradingDay_set)
    PlateSerial = property(_thostmduserapi.CThostFtdcReqDayEndFileReadyField_PlateSerial_get, _thostmduserapi.CThostFtdcReqDayEndFileReadyField_PlateSerial_set)
    LastFragment = property(_thostmduserapi.CThostFtdcReqDayEndFileReadyField_LastFragment_get, _thostmduserapi.CThostFtdcReqDayEndFileReadyField_LastFragment_set)
    SessionID = property(_thostmduserapi.CThostFtdcReqDayEndFileReadyField_SessionID_get, _thostmduserapi.CThostFtdcReqDayEndFileReadyField_SessionID_set)
    FileBusinessCode = property(_thostmduserapi.CThostFtdcReqDayEndFileReadyField_FileBusinessCode_get, _thostmduserapi.CThostFtdcReqDayEndFileReadyField_FileBusinessCode_set)
    Digest = property(_thostmduserapi.CThostFtdcReqDayEndFileReadyField_Digest_get, _thostmduserapi.CThostFtdcReqDayEndFileReadyField_Digest_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcReqDayEndFileReadyField_swiginit(self, _thostmduserapi.new_CThostFtdcReqDayEndFileReadyField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcReqDayEndFileReadyField

# Register CThostFtdcReqDayEndFileReadyField in _thostmduserapi:
_thostmduserapi.CThostFtdcReqDayEndFileReadyField_swigregister(CThostFtdcReqDayEndFileReadyField)

class CThostFtdcReturnResultField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ReturnCode = property(_thostmduserapi.CThostFtdcReturnResultField_ReturnCode_get, _thostmduserapi.CThostFtdcReturnResultField_ReturnCode_set)
    DescrInfoForReturnCode = property(_thostmduserapi.CThostFtdcReturnResultField_DescrInfoForReturnCode_get, _thostmduserapi.CThostFtdcReturnResultField_DescrInfoForReturnCode_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcReturnResultField_swiginit(self, _thostmduserapi.new_CThostFtdcReturnResultField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcReturnResultField

# Register CThostFtdcReturnResultField in _thostmduserapi:
_thostmduserapi.CThostFtdcReturnResultField_swigregister(CThostFtdcReturnResultField)

class CThostFtdcVerifyFuturePasswordField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradeCode = property(_thostmduserapi.CThostFtdcVerifyFuturePasswordField_TradeCode_get, _thostmduserapi.CThostFtdcVerifyFuturePasswordField_TradeCode_set)
    BankID = property(_thostmduserapi.CThostFtdcVerifyFuturePasswordField_BankID_get, _thostmduserapi.CThostFtdcVerifyFuturePasswordField_BankID_set)
    BankBranchID = property(_thostmduserapi.CThostFtdcVerifyFuturePasswordField_BankBranchID_get, _thostmduserapi.CThostFtdcVerifyFuturePasswordField_BankBranchID_set)
    BrokerID = property(_thostmduserapi.CThostFtdcVerifyFuturePasswordField_BrokerID_get, _thostmduserapi.CThostFtdcVerifyFuturePasswordField_BrokerID_set)
    BrokerBranchID = property(_thostmduserapi.CThostFtdcVerifyFuturePasswordField_BrokerBranchID_get, _thostmduserapi.CThostFtdcVerifyFuturePasswordField_BrokerBranchID_set)
    TradeDate = property(_thostmduserapi.CThostFtdcVerifyFuturePasswordField_TradeDate_get, _thostmduserapi.CThostFtdcVerifyFuturePasswordField_TradeDate_set)
    TradeTime = property(_thostmduserapi.CThostFtdcVerifyFuturePasswordField_TradeTime_get, _thostmduserapi.CThostFtdcVerifyFuturePasswordField_TradeTime_set)
    BankSerial = property(_thostmduserapi.CThostFtdcVerifyFuturePasswordField_BankSerial_get, _thostmduserapi.CThostFtdcVerifyFuturePasswordField_BankSerial_set)
    TradingDay = property(_thostmduserapi.CThostFtdcVerifyFuturePasswordField_TradingDay_get, _thostmduserapi.CThostFtdcVerifyFuturePasswordField_TradingDay_set)
    PlateSerial = property(_thostmduserapi.CThostFtdcVerifyFuturePasswordField_PlateSerial_get, _thostmduserapi.CThostFtdcVerifyFuturePasswordField_PlateSerial_set)
    LastFragment = property(_thostmduserapi.CThostFtdcVerifyFuturePasswordField_LastFragment_get, _thostmduserapi.CThostFtdcVerifyFuturePasswordField_LastFragment_set)
    SessionID = property(_thostmduserapi.CThostFtdcVerifyFuturePasswordField_SessionID_get, _thostmduserapi.CThostFtdcVerifyFuturePasswordField_SessionID_set)
    AccountID = property(_thostmduserapi.CThostFtdcVerifyFuturePasswordField_AccountID_get, _thostmduserapi.CThostFtdcVerifyFuturePasswordField_AccountID_set)
    Password = property(_thostmduserapi.CThostFtdcVerifyFuturePasswordField_Password_get, _thostmduserapi.CThostFtdcVerifyFuturePasswordField_Password_set)
    BankAccount = property(_thostmduserapi.CThostFtdcVerifyFuturePasswordField_BankAccount_get, _thostmduserapi.CThostFtdcVerifyFuturePasswordField_BankAccount_set)
    BankPassWord = property(_thostmduserapi.CThostFtdcVerifyFuturePasswordField_BankPassWord_get, _thostmduserapi.CThostFtdcVerifyFuturePasswordField_BankPassWord_set)
    InstallID = property(_thostmduserapi.CThostFtdcVerifyFuturePasswordField_InstallID_get, _thostmduserapi.CThostFtdcVerifyFuturePasswordField_InstallID_set)
    TID = property(_thostmduserapi.CThostFtdcVerifyFuturePasswordField_TID_get, _thostmduserapi.CThostFtdcVerifyFuturePasswordField_TID_set)
    CurrencyID = property(_thostmduserapi.CThostFtdcVerifyFuturePasswordField_CurrencyID_get, _thostmduserapi.CThostFtdcVerifyFuturePasswordField_CurrencyID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcVerifyFuturePasswordField_swiginit(self, _thostmduserapi.new_CThostFtdcVerifyFuturePasswordField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcVerifyFuturePasswordField

# Register CThostFtdcVerifyFuturePasswordField in _thostmduserapi:
_thostmduserapi.CThostFtdcVerifyFuturePasswordField_swigregister(CThostFtdcVerifyFuturePasswordField)

class CThostFtdcVerifyCustInfoField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    CustomerName = property(_thostmduserapi.CThostFtdcVerifyCustInfoField_CustomerName_get, _thostmduserapi.CThostFtdcVerifyCustInfoField_CustomerName_set)
    IdCardType = property(_thostmduserapi.CThostFtdcVerifyCustInfoField_IdCardType_get, _thostmduserapi.CThostFtdcVerifyCustInfoField_IdCardType_set)
    IdentifiedCardNo = property(_thostmduserapi.CThostFtdcVerifyCustInfoField_IdentifiedCardNo_get, _thostmduserapi.CThostFtdcVerifyCustInfoField_IdentifiedCardNo_set)
    CustType = property(_thostmduserapi.CThostFtdcVerifyCustInfoField_CustType_get, _thostmduserapi.CThostFtdcVerifyCustInfoField_CustType_set)
    LongCustomerName = property(_thostmduserapi.CThostFtdcVerifyCustInfoField_LongCustomerName_get, _thostmduserapi.CThostFtdcVerifyCustInfoField_LongCustomerName_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcVerifyCustInfoField_swiginit(self, _thostmduserapi.new_CThostFtdcVerifyCustInfoField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcVerifyCustInfoField

# Register CThostFtdcVerifyCustInfoField in _thostmduserapi:
_thostmduserapi.CThostFtdcVerifyCustInfoField_swigregister(CThostFtdcVerifyCustInfoField)

class CThostFtdcVerifyFuturePasswordAndCustInfoField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    CustomerName = property(_thostmduserapi.CThostFtdcVerifyFuturePasswordAndCustInfoField_CustomerName_get, _thostmduserapi.CThostFtdcVerifyFuturePasswordAndCustInfoField_CustomerName_set)
    IdCardType = property(_thostmduserapi.CThostFtdcVerifyFuturePasswordAndCustInfoField_IdCardType_get, _thostmduserapi.CThostFtdcVerifyFuturePasswordAndCustInfoField_IdCardType_set)
    IdentifiedCardNo = property(_thostmduserapi.CThostFtdcVerifyFuturePasswordAndCustInfoField_IdentifiedCardNo_get, _thostmduserapi.CThostFtdcVerifyFuturePasswordAndCustInfoField_IdentifiedCardNo_set)
    CustType = property(_thostmduserapi.CThostFtdcVerifyFuturePasswordAndCustInfoField_CustType_get, _thostmduserapi.CThostFtdcVerifyFuturePasswordAndCustInfoField_CustType_set)
    AccountID = property(_thostmduserapi.CThostFtdcVerifyFuturePasswordAndCustInfoField_AccountID_get, _thostmduserapi.CThostFtdcVerifyFuturePasswordAndCustInfoField_AccountID_set)
    Password = property(_thostmduserapi.CThostFtdcVerifyFuturePasswordAndCustInfoField_Password_get, _thostmduserapi.CThostFtdcVerifyFuturePasswordAndCustInfoField_Password_set)
    CurrencyID = property(_thostmduserapi.CThostFtdcVerifyFuturePasswordAndCustInfoField_CurrencyID_get, _thostmduserapi.CThostFtdcVerifyFuturePasswordAndCustInfoField_CurrencyID_set)
    LongCustomerName = property(_thostmduserapi.CThostFtdcVerifyFuturePasswordAndCustInfoField_LongCustomerName_get, _thostmduserapi.CThostFtdcVerifyFuturePasswordAndCustInfoField_LongCustomerName_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcVerifyFuturePasswordAndCustInfoField_swiginit(self, _thostmduserapi.new_CThostFtdcVerifyFuturePasswordAndCustInfoField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcVerifyFuturePasswordAndCustInfoField

# Register CThostFtdcVerifyFuturePasswordAndCustInfoField in _thostmduserapi:
_thostmduserapi.CThostFtdcVerifyFuturePasswordAndCustInfoField_swigregister(CThostFtdcVerifyFuturePasswordAndCustInfoField)

class CThostFtdcDepositResultInformField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    DepositSeqNo = property(_thostmduserapi.CThostFtdcDepositResultInformField_DepositSeqNo_get, _thostmduserapi.CThostFtdcDepositResultInformField_DepositSeqNo_set)
    BrokerID = property(_thostmduserapi.CThostFtdcDepositResultInformField_BrokerID_get, _thostmduserapi.CThostFtdcDepositResultInformField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcDepositResultInformField_InvestorID_get, _thostmduserapi.CThostFtdcDepositResultInformField_InvestorID_set)
    Deposit = property(_thostmduserapi.CThostFtdcDepositResultInformField_Deposit_get, _thostmduserapi.CThostFtdcDepositResultInformField_Deposit_set)
    RequestID = property(_thostmduserapi.CThostFtdcDepositResultInformField_RequestID_get, _thostmduserapi.CThostFtdcDepositResultInformField_RequestID_set)
    ReturnCode = property(_thostmduserapi.CThostFtdcDepositResultInformField_ReturnCode_get, _thostmduserapi.CThostFtdcDepositResultInformField_ReturnCode_set)
    DescrInfoForReturnCode = property(_thostmduserapi.CThostFtdcDepositResultInformField_DescrInfoForReturnCode_get, _thostmduserapi.CThostFtdcDepositResultInformField_DescrInfoForReturnCode_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcDepositResultInformField_swiginit(self, _thostmduserapi.new_CThostFtdcDepositResultInformField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcDepositResultInformField

# Register CThostFtdcDepositResultInformField in _thostmduserapi:
_thostmduserapi.CThostFtdcDepositResultInformField_swigregister(CThostFtdcDepositResultInformField)

class CThostFtdcReqSyncKeyField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradeCode = property(_thostmduserapi.CThostFtdcReqSyncKeyField_TradeCode_get, _thostmduserapi.CThostFtdcReqSyncKeyField_TradeCode_set)
    BankID = property(_thostmduserapi.CThostFtdcReqSyncKeyField_BankID_get, _thostmduserapi.CThostFtdcReqSyncKeyField_BankID_set)
    BankBranchID = property(_thostmduserapi.CThostFtdcReqSyncKeyField_BankBranchID_get, _thostmduserapi.CThostFtdcReqSyncKeyField_BankBranchID_set)
    BrokerID = property(_thostmduserapi.CThostFtdcReqSyncKeyField_BrokerID_get, _thostmduserapi.CThostFtdcReqSyncKeyField_BrokerID_set)
    BrokerBranchID = property(_thostmduserapi.CThostFtdcReqSyncKeyField_BrokerBranchID_get, _thostmduserapi.CThostFtdcReqSyncKeyField_BrokerBranchID_set)
    TradeDate = property(_thostmduserapi.CThostFtdcReqSyncKeyField_TradeDate_get, _thostmduserapi.CThostFtdcReqSyncKeyField_TradeDate_set)
    TradeTime = property(_thostmduserapi.CThostFtdcReqSyncKeyField_TradeTime_get, _thostmduserapi.CThostFtdcReqSyncKeyField_TradeTime_set)
    BankSerial = property(_thostmduserapi.CThostFtdcReqSyncKeyField_BankSerial_get, _thostmduserapi.CThostFtdcReqSyncKeyField_BankSerial_set)
    TradingDay = property(_thostmduserapi.CThostFtdcReqSyncKeyField_TradingDay_get, _thostmduserapi.CThostFtdcReqSyncKeyField_TradingDay_set)
    PlateSerial = property(_thostmduserapi.CThostFtdcReqSyncKeyField_PlateSerial_get, _thostmduserapi.CThostFtdcReqSyncKeyField_PlateSerial_set)
    LastFragment = property(_thostmduserapi.CThostFtdcReqSyncKeyField_LastFragment_get, _thostmduserapi.CThostFtdcReqSyncKeyField_LastFragment_set)
    SessionID = property(_thostmduserapi.CThostFtdcReqSyncKeyField_SessionID_get, _thostmduserapi.CThostFtdcReqSyncKeyField_SessionID_set)
    InstallID = property(_thostmduserapi.CThostFtdcReqSyncKeyField_InstallID_get, _thostmduserapi.CThostFtdcReqSyncKeyField_InstallID_set)
    UserID = property(_thostmduserapi.CThostFtdcReqSyncKeyField_UserID_get, _thostmduserapi.CThostFtdcReqSyncKeyField_UserID_set)
    Message = property(_thostmduserapi.CThostFtdcReqSyncKeyField_Message_get, _thostmduserapi.CThostFtdcReqSyncKeyField_Message_set)
    DeviceID = property(_thostmduserapi.CThostFtdcReqSyncKeyField_DeviceID_get, _thostmduserapi.CThostFtdcReqSyncKeyField_DeviceID_set)
    BrokerIDByBank = property(_thostmduserapi.CThostFtdcReqSyncKeyField_BrokerIDByBank_get, _thostmduserapi.CThostFtdcReqSyncKeyField_BrokerIDByBank_set)
    OperNo = property(_thostmduserapi.CThostFtdcReqSyncKeyField_OperNo_get, _thostmduserapi.CThostFtdcReqSyncKeyField_OperNo_set)
    RequestID = property(_thostmduserapi.CThostFtdcReqSyncKeyField_RequestID_get, _thostmduserapi.CThostFtdcReqSyncKeyField_RequestID_set)
    TID = property(_thostmduserapi.CThostFtdcReqSyncKeyField_TID_get, _thostmduserapi.CThostFtdcReqSyncKeyField_TID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcReqSyncKeyField_swiginit(self, _thostmduserapi.new_CThostFtdcReqSyncKeyField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcReqSyncKeyField

# Register CThostFtdcReqSyncKeyField in _thostmduserapi:
_thostmduserapi.CThostFtdcReqSyncKeyField_swigregister(CThostFtdcReqSyncKeyField)

class CThostFtdcRspSyncKeyField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradeCode = property(_thostmduserapi.CThostFtdcRspSyncKeyField_TradeCode_get, _thostmduserapi.CThostFtdcRspSyncKeyField_TradeCode_set)
    BankID = property(_thostmduserapi.CThostFtdcRspSyncKeyField_BankID_get, _thostmduserapi.CThostFtdcRspSyncKeyField_BankID_set)
    BankBranchID = property(_thostmduserapi.CThostFtdcRspSyncKeyField_BankBranchID_get, _thostmduserapi.CThostFtdcRspSyncKeyField_BankBranchID_set)
    BrokerID = property(_thostmduserapi.CThostFtdcRspSyncKeyField_BrokerID_get, _thostmduserapi.CThostFtdcRspSyncKeyField_BrokerID_set)
    BrokerBranchID = property(_thostmduserapi.CThostFtdcRspSyncKeyField_BrokerBranchID_get, _thostmduserapi.CThostFtdcRspSyncKeyField_BrokerBranchID_set)
    TradeDate = property(_thostmduserapi.CThostFtdcRspSyncKeyField_TradeDate_get, _thostmduserapi.CThostFtdcRspSyncKeyField_TradeDate_set)
    TradeTime = property(_thostmduserapi.CThostFtdcRspSyncKeyField_TradeTime_get, _thostmduserapi.CThostFtdcRspSyncKeyField_TradeTime_set)
    BankSerial = property(_thostmduserapi.CThostFtdcRspSyncKeyField_BankSerial_get, _thostmduserapi.CThostFtdcRspSyncKeyField_BankSerial_set)
    TradingDay = property(_thostmduserapi.CThostFtdcRspSyncKeyField_TradingDay_get, _thostmduserapi.CThostFtdcRspSyncKeyField_TradingDay_set)
    PlateSerial = property(_thostmduserapi.CThostFtdcRspSyncKeyField_PlateSerial_get, _thostmduserapi.CThostFtdcRspSyncKeyField_PlateSerial_set)
    LastFragment = property(_thostmduserapi.CThostFtdcRspSyncKeyField_LastFragment_get, _thostmduserapi.CThostFtdcRspSyncKeyField_LastFragment_set)
    SessionID = property(_thostmduserapi.CThostFtdcRspSyncKeyField_SessionID_get, _thostmduserapi.CThostFtdcRspSyncKeyField_SessionID_set)
    InstallID = property(_thostmduserapi.CThostFtdcRspSyncKeyField_InstallID_get, _thostmduserapi.CThostFtdcRspSyncKeyField_InstallID_set)
    UserID = property(_thostmduserapi.CThostFtdcRspSyncKeyField_UserID_get, _thostmduserapi.CThostFtdcRspSyncKeyField_UserID_set)
    Message = property(_thostmduserapi.CThostFtdcRspSyncKeyField_Message_get, _thostmduserapi.CThostFtdcRspSyncKeyField_Message_set)
    DeviceID = property(_thostmduserapi.CThostFtdcRspSyncKeyField_DeviceID_get, _thostmduserapi.CThostFtdcRspSyncKeyField_DeviceID_set)
    BrokerIDByBank = property(_thostmduserapi.CThostFtdcRspSyncKeyField_BrokerIDByBank_get, _thostmduserapi.CThostFtdcRspSyncKeyField_BrokerIDByBank_set)
    OperNo = property(_thostmduserapi.CThostFtdcRspSyncKeyField_OperNo_get, _thostmduserapi.CThostFtdcRspSyncKeyField_OperNo_set)
    RequestID = property(_thostmduserapi.CThostFtdcRspSyncKeyField_RequestID_get, _thostmduserapi.CThostFtdcRspSyncKeyField_RequestID_set)
    TID = property(_thostmduserapi.CThostFtdcRspSyncKeyField_TID_get, _thostmduserapi.CThostFtdcRspSyncKeyField_TID_set)
    ErrorID = property(_thostmduserapi.CThostFtdcRspSyncKeyField_ErrorID_get, _thostmduserapi.CThostFtdcRspSyncKeyField_ErrorID_set)
    ErrorMsg = property(_thostmduserapi.CThostFtdcRspSyncKeyField_ErrorMsg_get, _thostmduserapi.CThostFtdcRspSyncKeyField_ErrorMsg_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcRspSyncKeyField_swiginit(self, _thostmduserapi.new_CThostFtdcRspSyncKeyField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcRspSyncKeyField

# Register CThostFtdcRspSyncKeyField in _thostmduserapi:
_thostmduserapi.CThostFtdcRspSyncKeyField_swigregister(CThostFtdcRspSyncKeyField)

class CThostFtdcNotifyQueryAccountField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradeCode = property(_thostmduserapi.CThostFtdcNotifyQueryAccountField_TradeCode_get, _thostmduserapi.CThostFtdcNotifyQueryAccountField_TradeCode_set)
    BankID = property(_thostmduserapi.CThostFtdcNotifyQueryAccountField_BankID_get, _thostmduserapi.CThostFtdcNotifyQueryAccountField_BankID_set)
    BankBranchID = property(_thostmduserapi.CThostFtdcNotifyQueryAccountField_BankBranchID_get, _thostmduserapi.CThostFtdcNotifyQueryAccountField_BankBranchID_set)
    BrokerID = property(_thostmduserapi.CThostFtdcNotifyQueryAccountField_BrokerID_get, _thostmduserapi.CThostFtdcNotifyQueryAccountField_BrokerID_set)
    BrokerBranchID = property(_thostmduserapi.CThostFtdcNotifyQueryAccountField_BrokerBranchID_get, _thostmduserapi.CThostFtdcNotifyQueryAccountField_BrokerBranchID_set)
    TradeDate = property(_thostmduserapi.CThostFtdcNotifyQueryAccountField_TradeDate_get, _thostmduserapi.CThostFtdcNotifyQueryAccountField_TradeDate_set)
    TradeTime = property(_thostmduserapi.CThostFtdcNotifyQueryAccountField_TradeTime_get, _thostmduserapi.CThostFtdcNotifyQueryAccountField_TradeTime_set)
    BankSerial = property(_thostmduserapi.CThostFtdcNotifyQueryAccountField_BankSerial_get, _thostmduserapi.CThostFtdcNotifyQueryAccountField_BankSerial_set)
    TradingDay = property(_thostmduserapi.CThostFtdcNotifyQueryAccountField_TradingDay_get, _thostmduserapi.CThostFtdcNotifyQueryAccountField_TradingDay_set)
    PlateSerial = property(_thostmduserapi.CThostFtdcNotifyQueryAccountField_PlateSerial_get, _thostmduserapi.CThostFtdcNotifyQueryAccountField_PlateSerial_set)
    LastFragment = property(_thostmduserapi.CThostFtdcNotifyQueryAccountField_LastFragment_get, _thostmduserapi.CThostFtdcNotifyQueryAccountField_LastFragment_set)
    SessionID = property(_thostmduserapi.CThostFtdcNotifyQueryAccountField_SessionID_get, _thostmduserapi.CThostFtdcNotifyQueryAccountField_SessionID_set)
    CustomerName = property(_thostmduserapi.CThostFtdcNotifyQueryAccountField_CustomerName_get, _thostmduserapi.CThostFtdcNotifyQueryAccountField_CustomerName_set)
    IdCardType = property(_thostmduserapi.CThostFtdcNotifyQueryAccountField_IdCardType_get, _thostmduserapi.CThostFtdcNotifyQueryAccountField_IdCardType_set)
    IdentifiedCardNo = property(_thostmduserapi.CThostFtdcNotifyQueryAccountField_IdentifiedCardNo_get, _thostmduserapi.CThostFtdcNotifyQueryAccountField_IdentifiedCardNo_set)
    CustType = property(_thostmduserapi.CThostFtdcNotifyQueryAccountField_CustType_get, _thostmduserapi.CThostFtdcNotifyQueryAccountField_CustType_set)
    BankAccount = property(_thostmduserapi.CThostFtdcNotifyQueryAccountField_BankAccount_get, _thostmduserapi.CThostFtdcNotifyQueryAccountField_BankAccount_set)
    BankPassWord = property(_thostmduserapi.CThostFtdcNotifyQueryAccountField_BankPassWord_get, _thostmduserapi.CThostFtdcNotifyQueryAccountField_BankPassWord_set)
    AccountID = property(_thostmduserapi.CThostFtdcNotifyQueryAccountField_AccountID_get, _thostmduserapi.CThostFtdcNotifyQueryAccountField_AccountID_set)
    Password = property(_thostmduserapi.CThostFtdcNotifyQueryAccountField_Password_get, _thostmduserapi.CThostFtdcNotifyQueryAccountField_Password_set)
    FutureSerial = property(_thostmduserapi.CThostFtdcNotifyQueryAccountField_FutureSerial_get, _thostmduserapi.CThostFtdcNotifyQueryAccountField_FutureSerial_set)
    InstallID = property(_thostmduserapi.CThostFtdcNotifyQueryAccountField_InstallID_get, _thostmduserapi.CThostFtdcNotifyQueryAccountField_InstallID_set)
    UserID = property(_thostmduserapi.CThostFtdcNotifyQueryAccountField_UserID_get, _thostmduserapi.CThostFtdcNotifyQueryAccountField_UserID_set)
    VerifyCertNoFlag = property(_thostmduserapi.CThostFtdcNotifyQueryAccountField_VerifyCertNoFlag_get, _thostmduserapi.CThostFtdcNotifyQueryAccountField_VerifyCertNoFlag_set)
    CurrencyID = property(_thostmduserapi.CThostFtdcNotifyQueryAccountField_CurrencyID_get, _thostmduserapi.CThostFtdcNotifyQueryAccountField_CurrencyID_set)
    Digest = property(_thostmduserapi.CThostFtdcNotifyQueryAccountField_Digest_get, _thostmduserapi.CThostFtdcNotifyQueryAccountField_Digest_set)
    BankAccType = property(_thostmduserapi.CThostFtdcNotifyQueryAccountField_BankAccType_get, _thostmduserapi.CThostFtdcNotifyQueryAccountField_BankAccType_set)
    DeviceID = property(_thostmduserapi.CThostFtdcNotifyQueryAccountField_DeviceID_get, _thostmduserapi.CThostFtdcNotifyQueryAccountField_DeviceID_set)
    BankSecuAccType = property(_thostmduserapi.CThostFtdcNotifyQueryAccountField_BankSecuAccType_get, _thostmduserapi.CThostFtdcNotifyQueryAccountField_BankSecuAccType_set)
    BrokerIDByBank = property(_thostmduserapi.CThostFtdcNotifyQueryAccountField_BrokerIDByBank_get, _thostmduserapi.CThostFtdcNotifyQueryAccountField_BrokerIDByBank_set)
    BankSecuAcc = property(_thostmduserapi.CThostFtdcNotifyQueryAccountField_BankSecuAcc_get, _thostmduserapi.CThostFtdcNotifyQueryAccountField_BankSecuAcc_set)
    BankPwdFlag = property(_thostmduserapi.CThostFtdcNotifyQueryAccountField_BankPwdFlag_get, _thostmduserapi.CThostFtdcNotifyQueryAccountField_BankPwdFlag_set)
    SecuPwdFlag = property(_thostmduserapi.CThostFtdcNotifyQueryAccountField_SecuPwdFlag_get, _thostmduserapi.CThostFtdcNotifyQueryAccountField_SecuPwdFlag_set)
    OperNo = property(_thostmduserapi.CThostFtdcNotifyQueryAccountField_OperNo_get, _thostmduserapi.CThostFtdcNotifyQueryAccountField_OperNo_set)
    RequestID = property(_thostmduserapi.CThostFtdcNotifyQueryAccountField_RequestID_get, _thostmduserapi.CThostFtdcNotifyQueryAccountField_RequestID_set)
    TID = property(_thostmduserapi.CThostFtdcNotifyQueryAccountField_TID_get, _thostmduserapi.CThostFtdcNotifyQueryAccountField_TID_set)
    BankUseAmount = property(_thostmduserapi.CThostFtdcNotifyQueryAccountField_BankUseAmount_get, _thostmduserapi.CThostFtdcNotifyQueryAccountField_BankUseAmount_set)
    BankFetchAmount = property(_thostmduserapi.CThostFtdcNotifyQueryAccountField_BankFetchAmount_get, _thostmduserapi.CThostFtdcNotifyQueryAccountField_BankFetchAmount_set)
    ErrorID = property(_thostmduserapi.CThostFtdcNotifyQueryAccountField_ErrorID_get, _thostmduserapi.CThostFtdcNotifyQueryAccountField_ErrorID_set)
    ErrorMsg = property(_thostmduserapi.CThostFtdcNotifyQueryAccountField_ErrorMsg_get, _thostmduserapi.CThostFtdcNotifyQueryAccountField_ErrorMsg_set)
    LongCustomerName = property(_thostmduserapi.CThostFtdcNotifyQueryAccountField_LongCustomerName_get, _thostmduserapi.CThostFtdcNotifyQueryAccountField_LongCustomerName_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcNotifyQueryAccountField_swiginit(self, _thostmduserapi.new_CThostFtdcNotifyQueryAccountField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcNotifyQueryAccountField

# Register CThostFtdcNotifyQueryAccountField in _thostmduserapi:
_thostmduserapi.CThostFtdcNotifyQueryAccountField_swigregister(CThostFtdcNotifyQueryAccountField)

class CThostFtdcTransferSerialField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    PlateSerial = property(_thostmduserapi.CThostFtdcTransferSerialField_PlateSerial_get, _thostmduserapi.CThostFtdcTransferSerialField_PlateSerial_set)
    TradeDate = property(_thostmduserapi.CThostFtdcTransferSerialField_TradeDate_get, _thostmduserapi.CThostFtdcTransferSerialField_TradeDate_set)
    TradingDay = property(_thostmduserapi.CThostFtdcTransferSerialField_TradingDay_get, _thostmduserapi.CThostFtdcTransferSerialField_TradingDay_set)
    TradeTime = property(_thostmduserapi.CThostFtdcTransferSerialField_TradeTime_get, _thostmduserapi.CThostFtdcTransferSerialField_TradeTime_set)
    TradeCode = property(_thostmduserapi.CThostFtdcTransferSerialField_TradeCode_get, _thostmduserapi.CThostFtdcTransferSerialField_TradeCode_set)
    SessionID = property(_thostmduserapi.CThostFtdcTransferSerialField_SessionID_get, _thostmduserapi.CThostFtdcTransferSerialField_SessionID_set)
    BankID = property(_thostmduserapi.CThostFtdcTransferSerialField_BankID_get, _thostmduserapi.CThostFtdcTransferSerialField_BankID_set)
    BankBranchID = property(_thostmduserapi.CThostFtdcTransferSerialField_BankBranchID_get, _thostmduserapi.CThostFtdcTransferSerialField_BankBranchID_set)
    BankAccType = property(_thostmduserapi.CThostFtdcTransferSerialField_BankAccType_get, _thostmduserapi.CThostFtdcTransferSerialField_BankAccType_set)
    BankAccount = property(_thostmduserapi.CThostFtdcTransferSerialField_BankAccount_get, _thostmduserapi.CThostFtdcTransferSerialField_BankAccount_set)
    BankSerial = property(_thostmduserapi.CThostFtdcTransferSerialField_BankSerial_get, _thostmduserapi.CThostFtdcTransferSerialField_BankSerial_set)
    BrokerID = property(_thostmduserapi.CThostFtdcTransferSerialField_BrokerID_get, _thostmduserapi.CThostFtdcTransferSerialField_BrokerID_set)
    BrokerBranchID = property(_thostmduserapi.CThostFtdcTransferSerialField_BrokerBranchID_get, _thostmduserapi.CThostFtdcTransferSerialField_BrokerBranchID_set)
    FutureAccType = property(_thostmduserapi.CThostFtdcTransferSerialField_FutureAccType_get, _thostmduserapi.CThostFtdcTransferSerialField_FutureAccType_set)
    AccountID = property(_thostmduserapi.CThostFtdcTransferSerialField_AccountID_get, _thostmduserapi.CThostFtdcTransferSerialField_AccountID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcTransferSerialField_InvestorID_get, _thostmduserapi.CThostFtdcTransferSerialField_InvestorID_set)
    FutureSerial = property(_thostmduserapi.CThostFtdcTransferSerialField_FutureSerial_get, _thostmduserapi.CThostFtdcTransferSerialField_FutureSerial_set)
    IdCardType = property(_thostmduserapi.CThostFtdcTransferSerialField_IdCardType_get, _thostmduserapi.CThostFtdcTransferSerialField_IdCardType_set)
    IdentifiedCardNo = property(_thostmduserapi.CThostFtdcTransferSerialField_IdentifiedCardNo_get, _thostmduserapi.CThostFtdcTransferSerialField_IdentifiedCardNo_set)
    CurrencyID = property(_thostmduserapi.CThostFtdcTransferSerialField_CurrencyID_get, _thostmduserapi.CThostFtdcTransferSerialField_CurrencyID_set)
    TradeAmount = property(_thostmduserapi.CThostFtdcTransferSerialField_TradeAmount_get, _thostmduserapi.CThostFtdcTransferSerialField_TradeAmount_set)
    CustFee = property(_thostmduserapi.CThostFtdcTransferSerialField_CustFee_get, _thostmduserapi.CThostFtdcTransferSerialField_CustFee_set)
    BrokerFee = property(_thostmduserapi.CThostFtdcTransferSerialField_BrokerFee_get, _thostmduserapi.CThostFtdcTransferSerialField_BrokerFee_set)
    AvailabilityFlag = property(_thostmduserapi.CThostFtdcTransferSerialField_AvailabilityFlag_get, _thostmduserapi.CThostFtdcTransferSerialField_AvailabilityFlag_set)
    OperatorCode = property(_thostmduserapi.CThostFtdcTransferSerialField_OperatorCode_get, _thostmduserapi.CThostFtdcTransferSerialField_OperatorCode_set)
    BankNewAccount = property(_thostmduserapi.CThostFtdcTransferSerialField_BankNewAccount_get, _thostmduserapi.CThostFtdcTransferSerialField_BankNewAccount_set)
    ErrorID = property(_thostmduserapi.CThostFtdcTransferSerialField_ErrorID_get, _thostmduserapi.CThostFtdcTransferSerialField_ErrorID_set)
    ErrorMsg = property(_thostmduserapi.CThostFtdcTransferSerialField_ErrorMsg_get, _thostmduserapi.CThostFtdcTransferSerialField_ErrorMsg_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcTransferSerialField_swiginit(self, _thostmduserapi.new_CThostFtdcTransferSerialField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcTransferSerialField

# Register CThostFtdcTransferSerialField in _thostmduserapi:
_thostmduserapi.CThostFtdcTransferSerialField_swigregister(CThostFtdcTransferSerialField)

class CThostFtdcQryTransferSerialField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQryTransferSerialField_BrokerID_get, _thostmduserapi.CThostFtdcQryTransferSerialField_BrokerID_set)
    AccountID = property(_thostmduserapi.CThostFtdcQryTransferSerialField_AccountID_get, _thostmduserapi.CThostFtdcQryTransferSerialField_AccountID_set)
    BankID = property(_thostmduserapi.CThostFtdcQryTransferSerialField_BankID_get, _thostmduserapi.CThostFtdcQryTransferSerialField_BankID_set)
    CurrencyID = property(_thostmduserapi.CThostFtdcQryTransferSerialField_CurrencyID_get, _thostmduserapi.CThostFtdcQryTransferSerialField_CurrencyID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryTransferSerialField_swiginit(self, _thostmduserapi.new_CThostFtdcQryTransferSerialField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryTransferSerialField

# Register CThostFtdcQryTransferSerialField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryTransferSerialField_swigregister(CThostFtdcQryTransferSerialField)

class CThostFtdcNotifyFutureSignInField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradeCode = property(_thostmduserapi.CThostFtdcNotifyFutureSignInField_TradeCode_get, _thostmduserapi.CThostFtdcNotifyFutureSignInField_TradeCode_set)
    BankID = property(_thostmduserapi.CThostFtdcNotifyFutureSignInField_BankID_get, _thostmduserapi.CThostFtdcNotifyFutureSignInField_BankID_set)
    BankBranchID = property(_thostmduserapi.CThostFtdcNotifyFutureSignInField_BankBranchID_get, _thostmduserapi.CThostFtdcNotifyFutureSignInField_BankBranchID_set)
    BrokerID = property(_thostmduserapi.CThostFtdcNotifyFutureSignInField_BrokerID_get, _thostmduserapi.CThostFtdcNotifyFutureSignInField_BrokerID_set)
    BrokerBranchID = property(_thostmduserapi.CThostFtdcNotifyFutureSignInField_BrokerBranchID_get, _thostmduserapi.CThostFtdcNotifyFutureSignInField_BrokerBranchID_set)
    TradeDate = property(_thostmduserapi.CThostFtdcNotifyFutureSignInField_TradeDate_get, _thostmduserapi.CThostFtdcNotifyFutureSignInField_TradeDate_set)
    TradeTime = property(_thostmduserapi.CThostFtdcNotifyFutureSignInField_TradeTime_get, _thostmduserapi.CThostFtdcNotifyFutureSignInField_TradeTime_set)
    BankSerial = property(_thostmduserapi.CThostFtdcNotifyFutureSignInField_BankSerial_get, _thostmduserapi.CThostFtdcNotifyFutureSignInField_BankSerial_set)
    TradingDay = property(_thostmduserapi.CThostFtdcNotifyFutureSignInField_TradingDay_get, _thostmduserapi.CThostFtdcNotifyFutureSignInField_TradingDay_set)
    PlateSerial = property(_thostmduserapi.CThostFtdcNotifyFutureSignInField_PlateSerial_get, _thostmduserapi.CThostFtdcNotifyFutureSignInField_PlateSerial_set)
    LastFragment = property(_thostmduserapi.CThostFtdcNotifyFutureSignInField_LastFragment_get, _thostmduserapi.CThostFtdcNotifyFutureSignInField_LastFragment_set)
    SessionID = property(_thostmduserapi.CThostFtdcNotifyFutureSignInField_SessionID_get, _thostmduserapi.CThostFtdcNotifyFutureSignInField_SessionID_set)
    InstallID = property(_thostmduserapi.CThostFtdcNotifyFutureSignInField_InstallID_get, _thostmduserapi.CThostFtdcNotifyFutureSignInField_InstallID_set)
    UserID = property(_thostmduserapi.CThostFtdcNotifyFutureSignInField_UserID_get, _thostmduserapi.CThostFtdcNotifyFutureSignInField_UserID_set)
    Digest = property(_thostmduserapi.CThostFtdcNotifyFutureSignInField_Digest_get, _thostmduserapi.CThostFtdcNotifyFutureSignInField_Digest_set)
    CurrencyID = property(_thostmduserapi.CThostFtdcNotifyFutureSignInField_CurrencyID_get, _thostmduserapi.CThostFtdcNotifyFutureSignInField_CurrencyID_set)
    DeviceID = property(_thostmduserapi.CThostFtdcNotifyFutureSignInField_DeviceID_get, _thostmduserapi.CThostFtdcNotifyFutureSignInField_DeviceID_set)
    BrokerIDByBank = property(_thostmduserapi.CThostFtdcNotifyFutureSignInField_BrokerIDByBank_get, _thostmduserapi.CThostFtdcNotifyFutureSignInField_BrokerIDByBank_set)
    OperNo = property(_thostmduserapi.CThostFtdcNotifyFutureSignInField_OperNo_get, _thostmduserapi.CThostFtdcNotifyFutureSignInField_OperNo_set)
    RequestID = property(_thostmduserapi.CThostFtdcNotifyFutureSignInField_RequestID_get, _thostmduserapi.CThostFtdcNotifyFutureSignInField_RequestID_set)
    TID = property(_thostmduserapi.CThostFtdcNotifyFutureSignInField_TID_get, _thostmduserapi.CThostFtdcNotifyFutureSignInField_TID_set)
    ErrorID = property(_thostmduserapi.CThostFtdcNotifyFutureSignInField_ErrorID_get, _thostmduserapi.CThostFtdcNotifyFutureSignInField_ErrorID_set)
    ErrorMsg = property(_thostmduserapi.CThostFtdcNotifyFutureSignInField_ErrorMsg_get, _thostmduserapi.CThostFtdcNotifyFutureSignInField_ErrorMsg_set)
    PinKey = property(_thostmduserapi.CThostFtdcNotifyFutureSignInField_PinKey_get, _thostmduserapi.CThostFtdcNotifyFutureSignInField_PinKey_set)
    MacKey = property(_thostmduserapi.CThostFtdcNotifyFutureSignInField_MacKey_get, _thostmduserapi.CThostFtdcNotifyFutureSignInField_MacKey_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcNotifyFutureSignInField_swiginit(self, _thostmduserapi.new_CThostFtdcNotifyFutureSignInField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcNotifyFutureSignInField

# Register CThostFtdcNotifyFutureSignInField in _thostmduserapi:
_thostmduserapi.CThostFtdcNotifyFutureSignInField_swigregister(CThostFtdcNotifyFutureSignInField)

class CThostFtdcNotifyFutureSignOutField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradeCode = property(_thostmduserapi.CThostFtdcNotifyFutureSignOutField_TradeCode_get, _thostmduserapi.CThostFtdcNotifyFutureSignOutField_TradeCode_set)
    BankID = property(_thostmduserapi.CThostFtdcNotifyFutureSignOutField_BankID_get, _thostmduserapi.CThostFtdcNotifyFutureSignOutField_BankID_set)
    BankBranchID = property(_thostmduserapi.CThostFtdcNotifyFutureSignOutField_BankBranchID_get, _thostmduserapi.CThostFtdcNotifyFutureSignOutField_BankBranchID_set)
    BrokerID = property(_thostmduserapi.CThostFtdcNotifyFutureSignOutField_BrokerID_get, _thostmduserapi.CThostFtdcNotifyFutureSignOutField_BrokerID_set)
    BrokerBranchID = property(_thostmduserapi.CThostFtdcNotifyFutureSignOutField_BrokerBranchID_get, _thostmduserapi.CThostFtdcNotifyFutureSignOutField_BrokerBranchID_set)
    TradeDate = property(_thostmduserapi.CThostFtdcNotifyFutureSignOutField_TradeDate_get, _thostmduserapi.CThostFtdcNotifyFutureSignOutField_TradeDate_set)
    TradeTime = property(_thostmduserapi.CThostFtdcNotifyFutureSignOutField_TradeTime_get, _thostmduserapi.CThostFtdcNotifyFutureSignOutField_TradeTime_set)
    BankSerial = property(_thostmduserapi.CThostFtdcNotifyFutureSignOutField_BankSerial_get, _thostmduserapi.CThostFtdcNotifyFutureSignOutField_BankSerial_set)
    TradingDay = property(_thostmduserapi.CThostFtdcNotifyFutureSignOutField_TradingDay_get, _thostmduserapi.CThostFtdcNotifyFutureSignOutField_TradingDay_set)
    PlateSerial = property(_thostmduserapi.CThostFtdcNotifyFutureSignOutField_PlateSerial_get, _thostmduserapi.CThostFtdcNotifyFutureSignOutField_PlateSerial_set)
    LastFragment = property(_thostmduserapi.CThostFtdcNotifyFutureSignOutField_LastFragment_get, _thostmduserapi.CThostFtdcNotifyFutureSignOutField_LastFragment_set)
    SessionID = property(_thostmduserapi.CThostFtdcNotifyFutureSignOutField_SessionID_get, _thostmduserapi.CThostFtdcNotifyFutureSignOutField_SessionID_set)
    InstallID = property(_thostmduserapi.CThostFtdcNotifyFutureSignOutField_InstallID_get, _thostmduserapi.CThostFtdcNotifyFutureSignOutField_InstallID_set)
    UserID = property(_thostmduserapi.CThostFtdcNotifyFutureSignOutField_UserID_get, _thostmduserapi.CThostFtdcNotifyFutureSignOutField_UserID_set)
    Digest = property(_thostmduserapi.CThostFtdcNotifyFutureSignOutField_Digest_get, _thostmduserapi.CThostFtdcNotifyFutureSignOutField_Digest_set)
    CurrencyID = property(_thostmduserapi.CThostFtdcNotifyFutureSignOutField_CurrencyID_get, _thostmduserapi.CThostFtdcNotifyFutureSignOutField_CurrencyID_set)
    DeviceID = property(_thostmduserapi.CThostFtdcNotifyFutureSignOutField_DeviceID_get, _thostmduserapi.CThostFtdcNotifyFutureSignOutField_DeviceID_set)
    BrokerIDByBank = property(_thostmduserapi.CThostFtdcNotifyFutureSignOutField_BrokerIDByBank_get, _thostmduserapi.CThostFtdcNotifyFutureSignOutField_BrokerIDByBank_set)
    OperNo = property(_thostmduserapi.CThostFtdcNotifyFutureSignOutField_OperNo_get, _thostmduserapi.CThostFtdcNotifyFutureSignOutField_OperNo_set)
    RequestID = property(_thostmduserapi.CThostFtdcNotifyFutureSignOutField_RequestID_get, _thostmduserapi.CThostFtdcNotifyFutureSignOutField_RequestID_set)
    TID = property(_thostmduserapi.CThostFtdcNotifyFutureSignOutField_TID_get, _thostmduserapi.CThostFtdcNotifyFutureSignOutField_TID_set)
    ErrorID = property(_thostmduserapi.CThostFtdcNotifyFutureSignOutField_ErrorID_get, _thostmduserapi.CThostFtdcNotifyFutureSignOutField_ErrorID_set)
    ErrorMsg = property(_thostmduserapi.CThostFtdcNotifyFutureSignOutField_ErrorMsg_get, _thostmduserapi.CThostFtdcNotifyFutureSignOutField_ErrorMsg_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcNotifyFutureSignOutField_swiginit(self, _thostmduserapi.new_CThostFtdcNotifyFutureSignOutField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcNotifyFutureSignOutField

# Register CThostFtdcNotifyFutureSignOutField in _thostmduserapi:
_thostmduserapi.CThostFtdcNotifyFutureSignOutField_swigregister(CThostFtdcNotifyFutureSignOutField)

class CThostFtdcNotifySyncKeyField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradeCode = property(_thostmduserapi.CThostFtdcNotifySyncKeyField_TradeCode_get, _thostmduserapi.CThostFtdcNotifySyncKeyField_TradeCode_set)
    BankID = property(_thostmduserapi.CThostFtdcNotifySyncKeyField_BankID_get, _thostmduserapi.CThostFtdcNotifySyncKeyField_BankID_set)
    BankBranchID = property(_thostmduserapi.CThostFtdcNotifySyncKeyField_BankBranchID_get, _thostmduserapi.CThostFtdcNotifySyncKeyField_BankBranchID_set)
    BrokerID = property(_thostmduserapi.CThostFtdcNotifySyncKeyField_BrokerID_get, _thostmduserapi.CThostFtdcNotifySyncKeyField_BrokerID_set)
    BrokerBranchID = property(_thostmduserapi.CThostFtdcNotifySyncKeyField_BrokerBranchID_get, _thostmduserapi.CThostFtdcNotifySyncKeyField_BrokerBranchID_set)
    TradeDate = property(_thostmduserapi.CThostFtdcNotifySyncKeyField_TradeDate_get, _thostmduserapi.CThostFtdcNotifySyncKeyField_TradeDate_set)
    TradeTime = property(_thostmduserapi.CThostFtdcNotifySyncKeyField_TradeTime_get, _thostmduserapi.CThostFtdcNotifySyncKeyField_TradeTime_set)
    BankSerial = property(_thostmduserapi.CThostFtdcNotifySyncKeyField_BankSerial_get, _thostmduserapi.CThostFtdcNotifySyncKeyField_BankSerial_set)
    TradingDay = property(_thostmduserapi.CThostFtdcNotifySyncKeyField_TradingDay_get, _thostmduserapi.CThostFtdcNotifySyncKeyField_TradingDay_set)
    PlateSerial = property(_thostmduserapi.CThostFtdcNotifySyncKeyField_PlateSerial_get, _thostmduserapi.CThostFtdcNotifySyncKeyField_PlateSerial_set)
    LastFragment = property(_thostmduserapi.CThostFtdcNotifySyncKeyField_LastFragment_get, _thostmduserapi.CThostFtdcNotifySyncKeyField_LastFragment_set)
    SessionID = property(_thostmduserapi.CThostFtdcNotifySyncKeyField_SessionID_get, _thostmduserapi.CThostFtdcNotifySyncKeyField_SessionID_set)
    InstallID = property(_thostmduserapi.CThostFtdcNotifySyncKeyField_InstallID_get, _thostmduserapi.CThostFtdcNotifySyncKeyField_InstallID_set)
    UserID = property(_thostmduserapi.CThostFtdcNotifySyncKeyField_UserID_get, _thostmduserapi.CThostFtdcNotifySyncKeyField_UserID_set)
    Message = property(_thostmduserapi.CThostFtdcNotifySyncKeyField_Message_get, _thostmduserapi.CThostFtdcNotifySyncKeyField_Message_set)
    DeviceID = property(_thostmduserapi.CThostFtdcNotifySyncKeyField_DeviceID_get, _thostmduserapi.CThostFtdcNotifySyncKeyField_DeviceID_set)
    BrokerIDByBank = property(_thostmduserapi.CThostFtdcNotifySyncKeyField_BrokerIDByBank_get, _thostmduserapi.CThostFtdcNotifySyncKeyField_BrokerIDByBank_set)
    OperNo = property(_thostmduserapi.CThostFtdcNotifySyncKeyField_OperNo_get, _thostmduserapi.CThostFtdcNotifySyncKeyField_OperNo_set)
    RequestID = property(_thostmduserapi.CThostFtdcNotifySyncKeyField_RequestID_get, _thostmduserapi.CThostFtdcNotifySyncKeyField_RequestID_set)
    TID = property(_thostmduserapi.CThostFtdcNotifySyncKeyField_TID_get, _thostmduserapi.CThostFtdcNotifySyncKeyField_TID_set)
    ErrorID = property(_thostmduserapi.CThostFtdcNotifySyncKeyField_ErrorID_get, _thostmduserapi.CThostFtdcNotifySyncKeyField_ErrorID_set)
    ErrorMsg = property(_thostmduserapi.CThostFtdcNotifySyncKeyField_ErrorMsg_get, _thostmduserapi.CThostFtdcNotifySyncKeyField_ErrorMsg_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcNotifySyncKeyField_swiginit(self, _thostmduserapi.new_CThostFtdcNotifySyncKeyField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcNotifySyncKeyField

# Register CThostFtdcNotifySyncKeyField in _thostmduserapi:
_thostmduserapi.CThostFtdcNotifySyncKeyField_swigregister(CThostFtdcNotifySyncKeyField)

class CThostFtdcQryAccountregisterField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQryAccountregisterField_BrokerID_get, _thostmduserapi.CThostFtdcQryAccountregisterField_BrokerID_set)
    AccountID = property(_thostmduserapi.CThostFtdcQryAccountregisterField_AccountID_get, _thostmduserapi.CThostFtdcQryAccountregisterField_AccountID_set)
    BankID = property(_thostmduserapi.CThostFtdcQryAccountregisterField_BankID_get, _thostmduserapi.CThostFtdcQryAccountregisterField_BankID_set)
    BankBranchID = property(_thostmduserapi.CThostFtdcQryAccountregisterField_BankBranchID_get, _thostmduserapi.CThostFtdcQryAccountregisterField_BankBranchID_set)
    CurrencyID = property(_thostmduserapi.CThostFtdcQryAccountregisterField_CurrencyID_get, _thostmduserapi.CThostFtdcQryAccountregisterField_CurrencyID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryAccountregisterField_swiginit(self, _thostmduserapi.new_CThostFtdcQryAccountregisterField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryAccountregisterField

# Register CThostFtdcQryAccountregisterField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryAccountregisterField_swigregister(CThostFtdcQryAccountregisterField)

class CThostFtdcAccountregisterField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradeDay = property(_thostmduserapi.CThostFtdcAccountregisterField_TradeDay_get, _thostmduserapi.CThostFtdcAccountregisterField_TradeDay_set)
    BankID = property(_thostmduserapi.CThostFtdcAccountregisterField_BankID_get, _thostmduserapi.CThostFtdcAccountregisterField_BankID_set)
    BankBranchID = property(_thostmduserapi.CThostFtdcAccountregisterField_BankBranchID_get, _thostmduserapi.CThostFtdcAccountregisterField_BankBranchID_set)
    BankAccount = property(_thostmduserapi.CThostFtdcAccountregisterField_BankAccount_get, _thostmduserapi.CThostFtdcAccountregisterField_BankAccount_set)
    BrokerID = property(_thostmduserapi.CThostFtdcAccountregisterField_BrokerID_get, _thostmduserapi.CThostFtdcAccountregisterField_BrokerID_set)
    BrokerBranchID = property(_thostmduserapi.CThostFtdcAccountregisterField_BrokerBranchID_get, _thostmduserapi.CThostFtdcAccountregisterField_BrokerBranchID_set)
    AccountID = property(_thostmduserapi.CThostFtdcAccountregisterField_AccountID_get, _thostmduserapi.CThostFtdcAccountregisterField_AccountID_set)
    IdCardType = property(_thostmduserapi.CThostFtdcAccountregisterField_IdCardType_get, _thostmduserapi.CThostFtdcAccountregisterField_IdCardType_set)
    IdentifiedCardNo = property(_thostmduserapi.CThostFtdcAccountregisterField_IdentifiedCardNo_get, _thostmduserapi.CThostFtdcAccountregisterField_IdentifiedCardNo_set)
    CustomerName = property(_thostmduserapi.CThostFtdcAccountregisterField_CustomerName_get, _thostmduserapi.CThostFtdcAccountregisterField_CustomerName_set)
    CurrencyID = property(_thostmduserapi.CThostFtdcAccountregisterField_CurrencyID_get, _thostmduserapi.CThostFtdcAccountregisterField_CurrencyID_set)
    OpenOrDestroy = property(_thostmduserapi.CThostFtdcAccountregisterField_OpenOrDestroy_get, _thostmduserapi.CThostFtdcAccountregisterField_OpenOrDestroy_set)
    RegDate = property(_thostmduserapi.CThostFtdcAccountregisterField_RegDate_get, _thostmduserapi.CThostFtdcAccountregisterField_RegDate_set)
    OutDate = property(_thostmduserapi.CThostFtdcAccountregisterField_OutDate_get, _thostmduserapi.CThostFtdcAccountregisterField_OutDate_set)
    TID = property(_thostmduserapi.CThostFtdcAccountregisterField_TID_get, _thostmduserapi.CThostFtdcAccountregisterField_TID_set)
    CustType = property(_thostmduserapi.CThostFtdcAccountregisterField_CustType_get, _thostmduserapi.CThostFtdcAccountregisterField_CustType_set)
    BankAccType = property(_thostmduserapi.CThostFtdcAccountregisterField_BankAccType_get, _thostmduserapi.CThostFtdcAccountregisterField_BankAccType_set)
    LongCustomerName = property(_thostmduserapi.CThostFtdcAccountregisterField_LongCustomerName_get, _thostmduserapi.CThostFtdcAccountregisterField_LongCustomerName_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcAccountregisterField_swiginit(self, _thostmduserapi.new_CThostFtdcAccountregisterField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcAccountregisterField

# Register CThostFtdcAccountregisterField in _thostmduserapi:
_thostmduserapi.CThostFtdcAccountregisterField_swigregister(CThostFtdcAccountregisterField)

class CThostFtdcOpenAccountField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradeCode = property(_thostmduserapi.CThostFtdcOpenAccountField_TradeCode_get, _thostmduserapi.CThostFtdcOpenAccountField_TradeCode_set)
    BankID = property(_thostmduserapi.CThostFtdcOpenAccountField_BankID_get, _thostmduserapi.CThostFtdcOpenAccountField_BankID_set)
    BankBranchID = property(_thostmduserapi.CThostFtdcOpenAccountField_BankBranchID_get, _thostmduserapi.CThostFtdcOpenAccountField_BankBranchID_set)
    BrokerID = property(_thostmduserapi.CThostFtdcOpenAccountField_BrokerID_get, _thostmduserapi.CThostFtdcOpenAccountField_BrokerID_set)
    BrokerBranchID = property(_thostmduserapi.CThostFtdcOpenAccountField_BrokerBranchID_get, _thostmduserapi.CThostFtdcOpenAccountField_BrokerBranchID_set)
    TradeDate = property(_thostmduserapi.CThostFtdcOpenAccountField_TradeDate_get, _thostmduserapi.CThostFtdcOpenAccountField_TradeDate_set)
    TradeTime = property(_thostmduserapi.CThostFtdcOpenAccountField_TradeTime_get, _thostmduserapi.CThostFtdcOpenAccountField_TradeTime_set)
    BankSerial = property(_thostmduserapi.CThostFtdcOpenAccountField_BankSerial_get, _thostmduserapi.CThostFtdcOpenAccountField_BankSerial_set)
    TradingDay = property(_thostmduserapi.CThostFtdcOpenAccountField_TradingDay_get, _thostmduserapi.CThostFtdcOpenAccountField_TradingDay_set)
    PlateSerial = property(_thostmduserapi.CThostFtdcOpenAccountField_PlateSerial_get, _thostmduserapi.CThostFtdcOpenAccountField_PlateSerial_set)
    LastFragment = property(_thostmduserapi.CThostFtdcOpenAccountField_LastFragment_get, _thostmduserapi.CThostFtdcOpenAccountField_LastFragment_set)
    SessionID = property(_thostmduserapi.CThostFtdcOpenAccountField_SessionID_get, _thostmduserapi.CThostFtdcOpenAccountField_SessionID_set)
    CustomerName = property(_thostmduserapi.CThostFtdcOpenAccountField_CustomerName_get, _thostmduserapi.CThostFtdcOpenAccountField_CustomerName_set)
    IdCardType = property(_thostmduserapi.CThostFtdcOpenAccountField_IdCardType_get, _thostmduserapi.CThostFtdcOpenAccountField_IdCardType_set)
    IdentifiedCardNo = property(_thostmduserapi.CThostFtdcOpenAccountField_IdentifiedCardNo_get, _thostmduserapi.CThostFtdcOpenAccountField_IdentifiedCardNo_set)
    Gender = property(_thostmduserapi.CThostFtdcOpenAccountField_Gender_get, _thostmduserapi.CThostFtdcOpenAccountField_Gender_set)
    CountryCode = property(_thostmduserapi.CThostFtdcOpenAccountField_CountryCode_get, _thostmduserapi.CThostFtdcOpenAccountField_CountryCode_set)
    CustType = property(_thostmduserapi.CThostFtdcOpenAccountField_CustType_get, _thostmduserapi.CThostFtdcOpenAccountField_CustType_set)
    Address = property(_thostmduserapi.CThostFtdcOpenAccountField_Address_get, _thostmduserapi.CThostFtdcOpenAccountField_Address_set)
    ZipCode = property(_thostmduserapi.CThostFtdcOpenAccountField_ZipCode_get, _thostmduserapi.CThostFtdcOpenAccountField_ZipCode_set)
    Telephone = property(_thostmduserapi.CThostFtdcOpenAccountField_Telephone_get, _thostmduserapi.CThostFtdcOpenAccountField_Telephone_set)
    MobilePhone = property(_thostmduserapi.CThostFtdcOpenAccountField_MobilePhone_get, _thostmduserapi.CThostFtdcOpenAccountField_MobilePhone_set)
    Fax = property(_thostmduserapi.CThostFtdcOpenAccountField_Fax_get, _thostmduserapi.CThostFtdcOpenAccountField_Fax_set)
    EMail = property(_thostmduserapi.CThostFtdcOpenAccountField_EMail_get, _thostmduserapi.CThostFtdcOpenAccountField_EMail_set)
    MoneyAccountStatus = property(_thostmduserapi.CThostFtdcOpenAccountField_MoneyAccountStatus_get, _thostmduserapi.CThostFtdcOpenAccountField_MoneyAccountStatus_set)
    BankAccount = property(_thostmduserapi.CThostFtdcOpenAccountField_BankAccount_get, _thostmduserapi.CThostFtdcOpenAccountField_BankAccount_set)
    BankPassWord = property(_thostmduserapi.CThostFtdcOpenAccountField_BankPassWord_get, _thostmduserapi.CThostFtdcOpenAccountField_BankPassWord_set)
    AccountID = property(_thostmduserapi.CThostFtdcOpenAccountField_AccountID_get, _thostmduserapi.CThostFtdcOpenAccountField_AccountID_set)
    Password = property(_thostmduserapi.CThostFtdcOpenAccountField_Password_get, _thostmduserapi.CThostFtdcOpenAccountField_Password_set)
    InstallID = property(_thostmduserapi.CThostFtdcOpenAccountField_InstallID_get, _thostmduserapi.CThostFtdcOpenAccountField_InstallID_set)
    VerifyCertNoFlag = property(_thostmduserapi.CThostFtdcOpenAccountField_VerifyCertNoFlag_get, _thostmduserapi.CThostFtdcOpenAccountField_VerifyCertNoFlag_set)
    CurrencyID = property(_thostmduserapi.CThostFtdcOpenAccountField_CurrencyID_get, _thostmduserapi.CThostFtdcOpenAccountField_CurrencyID_set)
    CashExchangeCode = property(_thostmduserapi.CThostFtdcOpenAccountField_CashExchangeCode_get, _thostmduserapi.CThostFtdcOpenAccountField_CashExchangeCode_set)
    Digest = property(_thostmduserapi.CThostFtdcOpenAccountField_Digest_get, _thostmduserapi.CThostFtdcOpenAccountField_Digest_set)
    BankAccType = property(_thostmduserapi.CThostFtdcOpenAccountField_BankAccType_get, _thostmduserapi.CThostFtdcOpenAccountField_BankAccType_set)
    DeviceID = property(_thostmduserapi.CThostFtdcOpenAccountField_DeviceID_get, _thostmduserapi.CThostFtdcOpenAccountField_DeviceID_set)
    BankSecuAccType = property(_thostmduserapi.CThostFtdcOpenAccountField_BankSecuAccType_get, _thostmduserapi.CThostFtdcOpenAccountField_BankSecuAccType_set)
    BrokerIDByBank = property(_thostmduserapi.CThostFtdcOpenAccountField_BrokerIDByBank_get, _thostmduserapi.CThostFtdcOpenAccountField_BrokerIDByBank_set)
    BankSecuAcc = property(_thostmduserapi.CThostFtdcOpenAccountField_BankSecuAcc_get, _thostmduserapi.CThostFtdcOpenAccountField_BankSecuAcc_set)
    BankPwdFlag = property(_thostmduserapi.CThostFtdcOpenAccountField_BankPwdFlag_get, _thostmduserapi.CThostFtdcOpenAccountField_BankPwdFlag_set)
    SecuPwdFlag = property(_thostmduserapi.CThostFtdcOpenAccountField_SecuPwdFlag_get, _thostmduserapi.CThostFtdcOpenAccountField_SecuPwdFlag_set)
    OperNo = property(_thostmduserapi.CThostFtdcOpenAccountField_OperNo_get, _thostmduserapi.CThostFtdcOpenAccountField_OperNo_set)
    TID = property(_thostmduserapi.CThostFtdcOpenAccountField_TID_get, _thostmduserapi.CThostFtdcOpenAccountField_TID_set)
    UserID = property(_thostmduserapi.CThostFtdcOpenAccountField_UserID_get, _thostmduserapi.CThostFtdcOpenAccountField_UserID_set)
    ErrorID = property(_thostmduserapi.CThostFtdcOpenAccountField_ErrorID_get, _thostmduserapi.CThostFtdcOpenAccountField_ErrorID_set)
    ErrorMsg = property(_thostmduserapi.CThostFtdcOpenAccountField_ErrorMsg_get, _thostmduserapi.CThostFtdcOpenAccountField_ErrorMsg_set)
    LongCustomerName = property(_thostmduserapi.CThostFtdcOpenAccountField_LongCustomerName_get, _thostmduserapi.CThostFtdcOpenAccountField_LongCustomerName_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcOpenAccountField_swiginit(self, _thostmduserapi.new_CThostFtdcOpenAccountField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcOpenAccountField

# Register CThostFtdcOpenAccountField in _thostmduserapi:
_thostmduserapi.CThostFtdcOpenAccountField_swigregister(CThostFtdcOpenAccountField)

class CThostFtdcCancelAccountField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradeCode = property(_thostmduserapi.CThostFtdcCancelAccountField_TradeCode_get, _thostmduserapi.CThostFtdcCancelAccountField_TradeCode_set)
    BankID = property(_thostmduserapi.CThostFtdcCancelAccountField_BankID_get, _thostmduserapi.CThostFtdcCancelAccountField_BankID_set)
    BankBranchID = property(_thostmduserapi.CThostFtdcCancelAccountField_BankBranchID_get, _thostmduserapi.CThostFtdcCancelAccountField_BankBranchID_set)
    BrokerID = property(_thostmduserapi.CThostFtdcCancelAccountField_BrokerID_get, _thostmduserapi.CThostFtdcCancelAccountField_BrokerID_set)
    BrokerBranchID = property(_thostmduserapi.CThostFtdcCancelAccountField_BrokerBranchID_get, _thostmduserapi.CThostFtdcCancelAccountField_BrokerBranchID_set)
    TradeDate = property(_thostmduserapi.CThostFtdcCancelAccountField_TradeDate_get, _thostmduserapi.CThostFtdcCancelAccountField_TradeDate_set)
    TradeTime = property(_thostmduserapi.CThostFtdcCancelAccountField_TradeTime_get, _thostmduserapi.CThostFtdcCancelAccountField_TradeTime_set)
    BankSerial = property(_thostmduserapi.CThostFtdcCancelAccountField_BankSerial_get, _thostmduserapi.CThostFtdcCancelAccountField_BankSerial_set)
    TradingDay = property(_thostmduserapi.CThostFtdcCancelAccountField_TradingDay_get, _thostmduserapi.CThostFtdcCancelAccountField_TradingDay_set)
    PlateSerial = property(_thostmduserapi.CThostFtdcCancelAccountField_PlateSerial_get, _thostmduserapi.CThostFtdcCancelAccountField_PlateSerial_set)
    LastFragment = property(_thostmduserapi.CThostFtdcCancelAccountField_LastFragment_get, _thostmduserapi.CThostFtdcCancelAccountField_LastFragment_set)
    SessionID = property(_thostmduserapi.CThostFtdcCancelAccountField_SessionID_get, _thostmduserapi.CThostFtdcCancelAccountField_SessionID_set)
    CustomerName = property(_thostmduserapi.CThostFtdcCancelAccountField_CustomerName_get, _thostmduserapi.CThostFtdcCancelAccountField_CustomerName_set)
    IdCardType = property(_thostmduserapi.CThostFtdcCancelAccountField_IdCardType_get, _thostmduserapi.CThostFtdcCancelAccountField_IdCardType_set)
    IdentifiedCardNo = property(_thostmduserapi.CThostFtdcCancelAccountField_IdentifiedCardNo_get, _thostmduserapi.CThostFtdcCancelAccountField_IdentifiedCardNo_set)
    Gender = property(_thostmduserapi.CThostFtdcCancelAccountField_Gender_get, _thostmduserapi.CThostFtdcCancelAccountField_Gender_set)
    CountryCode = property(_thostmduserapi.CThostFtdcCancelAccountField_CountryCode_get, _thostmduserapi.CThostFtdcCancelAccountField_CountryCode_set)
    CustType = property(_thostmduserapi.CThostFtdcCancelAccountField_CustType_get, _thostmduserapi.CThostFtdcCancelAccountField_CustType_set)
    Address = property(_thostmduserapi.CThostFtdcCancelAccountField_Address_get, _thostmduserapi.CThostFtdcCancelAccountField_Address_set)
    ZipCode = property(_thostmduserapi.CThostFtdcCancelAccountField_ZipCode_get, _thostmduserapi.CThostFtdcCancelAccountField_ZipCode_set)
    Telephone = property(_thostmduserapi.CThostFtdcCancelAccountField_Telephone_get, _thostmduserapi.CThostFtdcCancelAccountField_Telephone_set)
    MobilePhone = property(_thostmduserapi.CThostFtdcCancelAccountField_MobilePhone_get, _thostmduserapi.CThostFtdcCancelAccountField_MobilePhone_set)
    Fax = property(_thostmduserapi.CThostFtdcCancelAccountField_Fax_get, _thostmduserapi.CThostFtdcCancelAccountField_Fax_set)
    EMail = property(_thostmduserapi.CThostFtdcCancelAccountField_EMail_get, _thostmduserapi.CThostFtdcCancelAccountField_EMail_set)
    MoneyAccountStatus = property(_thostmduserapi.CThostFtdcCancelAccountField_MoneyAccountStatus_get, _thostmduserapi.CThostFtdcCancelAccountField_MoneyAccountStatus_set)
    BankAccount = property(_thostmduserapi.CThostFtdcCancelAccountField_BankAccount_get, _thostmduserapi.CThostFtdcCancelAccountField_BankAccount_set)
    BankPassWord = property(_thostmduserapi.CThostFtdcCancelAccountField_BankPassWord_get, _thostmduserapi.CThostFtdcCancelAccountField_BankPassWord_set)
    AccountID = property(_thostmduserapi.CThostFtdcCancelAccountField_AccountID_get, _thostmduserapi.CThostFtdcCancelAccountField_AccountID_set)
    Password = property(_thostmduserapi.CThostFtdcCancelAccountField_Password_get, _thostmduserapi.CThostFtdcCancelAccountField_Password_set)
    InstallID = property(_thostmduserapi.CThostFtdcCancelAccountField_InstallID_get, _thostmduserapi.CThostFtdcCancelAccountField_InstallID_set)
    VerifyCertNoFlag = property(_thostmduserapi.CThostFtdcCancelAccountField_VerifyCertNoFlag_get, _thostmduserapi.CThostFtdcCancelAccountField_VerifyCertNoFlag_set)
    CurrencyID = property(_thostmduserapi.CThostFtdcCancelAccountField_CurrencyID_get, _thostmduserapi.CThostFtdcCancelAccountField_CurrencyID_set)
    CashExchangeCode = property(_thostmduserapi.CThostFtdcCancelAccountField_CashExchangeCode_get, _thostmduserapi.CThostFtdcCancelAccountField_CashExchangeCode_set)
    Digest = property(_thostmduserapi.CThostFtdcCancelAccountField_Digest_get, _thostmduserapi.CThostFtdcCancelAccountField_Digest_set)
    BankAccType = property(_thostmduserapi.CThostFtdcCancelAccountField_BankAccType_get, _thostmduserapi.CThostFtdcCancelAccountField_BankAccType_set)
    DeviceID = property(_thostmduserapi.CThostFtdcCancelAccountField_DeviceID_get, _thostmduserapi.CThostFtdcCancelAccountField_DeviceID_set)
    BankSecuAccType = property(_thostmduserapi.CThostFtdcCancelAccountField_BankSecuAccType_get, _thostmduserapi.CThostFtdcCancelAccountField_BankSecuAccType_set)
    BrokerIDByBank = property(_thostmduserapi.CThostFtdcCancelAccountField_BrokerIDByBank_get, _thostmduserapi.CThostFtdcCancelAccountField_BrokerIDByBank_set)
    BankSecuAcc = property(_thostmduserapi.CThostFtdcCancelAccountField_BankSecuAcc_get, _thostmduserapi.CThostFtdcCancelAccountField_BankSecuAcc_set)
    BankPwdFlag = property(_thostmduserapi.CThostFtdcCancelAccountField_BankPwdFlag_get, _thostmduserapi.CThostFtdcCancelAccountField_BankPwdFlag_set)
    SecuPwdFlag = property(_thostmduserapi.CThostFtdcCancelAccountField_SecuPwdFlag_get, _thostmduserapi.CThostFtdcCancelAccountField_SecuPwdFlag_set)
    OperNo = property(_thostmduserapi.CThostFtdcCancelAccountField_OperNo_get, _thostmduserapi.CThostFtdcCancelAccountField_OperNo_set)
    TID = property(_thostmduserapi.CThostFtdcCancelAccountField_TID_get, _thostmduserapi.CThostFtdcCancelAccountField_TID_set)
    UserID = property(_thostmduserapi.CThostFtdcCancelAccountField_UserID_get, _thostmduserapi.CThostFtdcCancelAccountField_UserID_set)
    ErrorID = property(_thostmduserapi.CThostFtdcCancelAccountField_ErrorID_get, _thostmduserapi.CThostFtdcCancelAccountField_ErrorID_set)
    ErrorMsg = property(_thostmduserapi.CThostFtdcCancelAccountField_ErrorMsg_get, _thostmduserapi.CThostFtdcCancelAccountField_ErrorMsg_set)
    LongCustomerName = property(_thostmduserapi.CThostFtdcCancelAccountField_LongCustomerName_get, _thostmduserapi.CThostFtdcCancelAccountField_LongCustomerName_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcCancelAccountField_swiginit(self, _thostmduserapi.new_CThostFtdcCancelAccountField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcCancelAccountField

# Register CThostFtdcCancelAccountField in _thostmduserapi:
_thostmduserapi.CThostFtdcCancelAccountField_swigregister(CThostFtdcCancelAccountField)

class CThostFtdcChangeAccountField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradeCode = property(_thostmduserapi.CThostFtdcChangeAccountField_TradeCode_get, _thostmduserapi.CThostFtdcChangeAccountField_TradeCode_set)
    BankID = property(_thostmduserapi.CThostFtdcChangeAccountField_BankID_get, _thostmduserapi.CThostFtdcChangeAccountField_BankID_set)
    BankBranchID = property(_thostmduserapi.CThostFtdcChangeAccountField_BankBranchID_get, _thostmduserapi.CThostFtdcChangeAccountField_BankBranchID_set)
    BrokerID = property(_thostmduserapi.CThostFtdcChangeAccountField_BrokerID_get, _thostmduserapi.CThostFtdcChangeAccountField_BrokerID_set)
    BrokerBranchID = property(_thostmduserapi.CThostFtdcChangeAccountField_BrokerBranchID_get, _thostmduserapi.CThostFtdcChangeAccountField_BrokerBranchID_set)
    TradeDate = property(_thostmduserapi.CThostFtdcChangeAccountField_TradeDate_get, _thostmduserapi.CThostFtdcChangeAccountField_TradeDate_set)
    TradeTime = property(_thostmduserapi.CThostFtdcChangeAccountField_TradeTime_get, _thostmduserapi.CThostFtdcChangeAccountField_TradeTime_set)
    BankSerial = property(_thostmduserapi.CThostFtdcChangeAccountField_BankSerial_get, _thostmduserapi.CThostFtdcChangeAccountField_BankSerial_set)
    TradingDay = property(_thostmduserapi.CThostFtdcChangeAccountField_TradingDay_get, _thostmduserapi.CThostFtdcChangeAccountField_TradingDay_set)
    PlateSerial = property(_thostmduserapi.CThostFtdcChangeAccountField_PlateSerial_get, _thostmduserapi.CThostFtdcChangeAccountField_PlateSerial_set)
    LastFragment = property(_thostmduserapi.CThostFtdcChangeAccountField_LastFragment_get, _thostmduserapi.CThostFtdcChangeAccountField_LastFragment_set)
    SessionID = property(_thostmduserapi.CThostFtdcChangeAccountField_SessionID_get, _thostmduserapi.CThostFtdcChangeAccountField_SessionID_set)
    CustomerName = property(_thostmduserapi.CThostFtdcChangeAccountField_CustomerName_get, _thostmduserapi.CThostFtdcChangeAccountField_CustomerName_set)
    IdCardType = property(_thostmduserapi.CThostFtdcChangeAccountField_IdCardType_get, _thostmduserapi.CThostFtdcChangeAccountField_IdCardType_set)
    IdentifiedCardNo = property(_thostmduserapi.CThostFtdcChangeAccountField_IdentifiedCardNo_get, _thostmduserapi.CThostFtdcChangeAccountField_IdentifiedCardNo_set)
    Gender = property(_thostmduserapi.CThostFtdcChangeAccountField_Gender_get, _thostmduserapi.CThostFtdcChangeAccountField_Gender_set)
    CountryCode = property(_thostmduserapi.CThostFtdcChangeAccountField_CountryCode_get, _thostmduserapi.CThostFtdcChangeAccountField_CountryCode_set)
    CustType = property(_thostmduserapi.CThostFtdcChangeAccountField_CustType_get, _thostmduserapi.CThostFtdcChangeAccountField_CustType_set)
    Address = property(_thostmduserapi.CThostFtdcChangeAccountField_Address_get, _thostmduserapi.CThostFtdcChangeAccountField_Address_set)
    ZipCode = property(_thostmduserapi.CThostFtdcChangeAccountField_ZipCode_get, _thostmduserapi.CThostFtdcChangeAccountField_ZipCode_set)
    Telephone = property(_thostmduserapi.CThostFtdcChangeAccountField_Telephone_get, _thostmduserapi.CThostFtdcChangeAccountField_Telephone_set)
    MobilePhone = property(_thostmduserapi.CThostFtdcChangeAccountField_MobilePhone_get, _thostmduserapi.CThostFtdcChangeAccountField_MobilePhone_set)
    Fax = property(_thostmduserapi.CThostFtdcChangeAccountField_Fax_get, _thostmduserapi.CThostFtdcChangeAccountField_Fax_set)
    EMail = property(_thostmduserapi.CThostFtdcChangeAccountField_EMail_get, _thostmduserapi.CThostFtdcChangeAccountField_EMail_set)
    MoneyAccountStatus = property(_thostmduserapi.CThostFtdcChangeAccountField_MoneyAccountStatus_get, _thostmduserapi.CThostFtdcChangeAccountField_MoneyAccountStatus_set)
    BankAccount = property(_thostmduserapi.CThostFtdcChangeAccountField_BankAccount_get, _thostmduserapi.CThostFtdcChangeAccountField_BankAccount_set)
    BankPassWord = property(_thostmduserapi.CThostFtdcChangeAccountField_BankPassWord_get, _thostmduserapi.CThostFtdcChangeAccountField_BankPassWord_set)
    NewBankAccount = property(_thostmduserapi.CThostFtdcChangeAccountField_NewBankAccount_get, _thostmduserapi.CThostFtdcChangeAccountField_NewBankAccount_set)
    NewBankPassWord = property(_thostmduserapi.CThostFtdcChangeAccountField_NewBankPassWord_get, _thostmduserapi.CThostFtdcChangeAccountField_NewBankPassWord_set)
    AccountID = property(_thostmduserapi.CThostFtdcChangeAccountField_AccountID_get, _thostmduserapi.CThostFtdcChangeAccountField_AccountID_set)
    Password = property(_thostmduserapi.CThostFtdcChangeAccountField_Password_get, _thostmduserapi.CThostFtdcChangeAccountField_Password_set)
    BankAccType = property(_thostmduserapi.CThostFtdcChangeAccountField_BankAccType_get, _thostmduserapi.CThostFtdcChangeAccountField_BankAccType_set)
    InstallID = property(_thostmduserapi.CThostFtdcChangeAccountField_InstallID_get, _thostmduserapi.CThostFtdcChangeAccountField_InstallID_set)
    VerifyCertNoFlag = property(_thostmduserapi.CThostFtdcChangeAccountField_VerifyCertNoFlag_get, _thostmduserapi.CThostFtdcChangeAccountField_VerifyCertNoFlag_set)
    CurrencyID = property(_thostmduserapi.CThostFtdcChangeAccountField_CurrencyID_get, _thostmduserapi.CThostFtdcChangeAccountField_CurrencyID_set)
    BrokerIDByBank = property(_thostmduserapi.CThostFtdcChangeAccountField_BrokerIDByBank_get, _thostmduserapi.CThostFtdcChangeAccountField_BrokerIDByBank_set)
    BankPwdFlag = property(_thostmduserapi.CThostFtdcChangeAccountField_BankPwdFlag_get, _thostmduserapi.CThostFtdcChangeAccountField_BankPwdFlag_set)
    SecuPwdFlag = property(_thostmduserapi.CThostFtdcChangeAccountField_SecuPwdFlag_get, _thostmduserapi.CThostFtdcChangeAccountField_SecuPwdFlag_set)
    TID = property(_thostmduserapi.CThostFtdcChangeAccountField_TID_get, _thostmduserapi.CThostFtdcChangeAccountField_TID_set)
    Digest = property(_thostmduserapi.CThostFtdcChangeAccountField_Digest_get, _thostmduserapi.CThostFtdcChangeAccountField_Digest_set)
    ErrorID = property(_thostmduserapi.CThostFtdcChangeAccountField_ErrorID_get, _thostmduserapi.CThostFtdcChangeAccountField_ErrorID_set)
    ErrorMsg = property(_thostmduserapi.CThostFtdcChangeAccountField_ErrorMsg_get, _thostmduserapi.CThostFtdcChangeAccountField_ErrorMsg_set)
    LongCustomerName = property(_thostmduserapi.CThostFtdcChangeAccountField_LongCustomerName_get, _thostmduserapi.CThostFtdcChangeAccountField_LongCustomerName_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcChangeAccountField_swiginit(self, _thostmduserapi.new_CThostFtdcChangeAccountField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcChangeAccountField

# Register CThostFtdcChangeAccountField in _thostmduserapi:
_thostmduserapi.CThostFtdcChangeAccountField_swigregister(CThostFtdcChangeAccountField)

class CThostFtdcSecAgentACIDMapField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcSecAgentACIDMapField_BrokerID_get, _thostmduserapi.CThostFtdcSecAgentACIDMapField_BrokerID_set)
    UserID = property(_thostmduserapi.CThostFtdcSecAgentACIDMapField_UserID_get, _thostmduserapi.CThostFtdcSecAgentACIDMapField_UserID_set)
    AccountID = property(_thostmduserapi.CThostFtdcSecAgentACIDMapField_AccountID_get, _thostmduserapi.CThostFtdcSecAgentACIDMapField_AccountID_set)
    CurrencyID = property(_thostmduserapi.CThostFtdcSecAgentACIDMapField_CurrencyID_get, _thostmduserapi.CThostFtdcSecAgentACIDMapField_CurrencyID_set)
    BrokerSecAgentID = property(_thostmduserapi.CThostFtdcSecAgentACIDMapField_BrokerSecAgentID_get, _thostmduserapi.CThostFtdcSecAgentACIDMapField_BrokerSecAgentID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcSecAgentACIDMapField_swiginit(self, _thostmduserapi.new_CThostFtdcSecAgentACIDMapField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcSecAgentACIDMapField

# Register CThostFtdcSecAgentACIDMapField in _thostmduserapi:
_thostmduserapi.CThostFtdcSecAgentACIDMapField_swigregister(CThostFtdcSecAgentACIDMapField)

class CThostFtdcQrySecAgentACIDMapField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQrySecAgentACIDMapField_BrokerID_get, _thostmduserapi.CThostFtdcQrySecAgentACIDMapField_BrokerID_set)
    UserID = property(_thostmduserapi.CThostFtdcQrySecAgentACIDMapField_UserID_get, _thostmduserapi.CThostFtdcQrySecAgentACIDMapField_UserID_set)
    AccountID = property(_thostmduserapi.CThostFtdcQrySecAgentACIDMapField_AccountID_get, _thostmduserapi.CThostFtdcQrySecAgentACIDMapField_AccountID_set)
    CurrencyID = property(_thostmduserapi.CThostFtdcQrySecAgentACIDMapField_CurrencyID_get, _thostmduserapi.CThostFtdcQrySecAgentACIDMapField_CurrencyID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQrySecAgentACIDMapField_swiginit(self, _thostmduserapi.new_CThostFtdcQrySecAgentACIDMapField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQrySecAgentACIDMapField

# Register CThostFtdcQrySecAgentACIDMapField in _thostmduserapi:
_thostmduserapi.CThostFtdcQrySecAgentACIDMapField_swigregister(CThostFtdcQrySecAgentACIDMapField)

class CThostFtdcUserRightsAssignField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcUserRightsAssignField_BrokerID_get, _thostmduserapi.CThostFtdcUserRightsAssignField_BrokerID_set)
    UserID = property(_thostmduserapi.CThostFtdcUserRightsAssignField_UserID_get, _thostmduserapi.CThostFtdcUserRightsAssignField_UserID_set)
    DRIdentityID = property(_thostmduserapi.CThostFtdcUserRightsAssignField_DRIdentityID_get, _thostmduserapi.CThostFtdcUserRightsAssignField_DRIdentityID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcUserRightsAssignField_swiginit(self, _thostmduserapi.new_CThostFtdcUserRightsAssignField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcUserRightsAssignField

# Register CThostFtdcUserRightsAssignField in _thostmduserapi:
_thostmduserapi.CThostFtdcUserRightsAssignField_swigregister(CThostFtdcUserRightsAssignField)

class CThostFtdcBrokerUserRightAssignField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcBrokerUserRightAssignField_BrokerID_get, _thostmduserapi.CThostFtdcBrokerUserRightAssignField_BrokerID_set)
    DRIdentityID = property(_thostmduserapi.CThostFtdcBrokerUserRightAssignField_DRIdentityID_get, _thostmduserapi.CThostFtdcBrokerUserRightAssignField_DRIdentityID_set)
    Tradeable = property(_thostmduserapi.CThostFtdcBrokerUserRightAssignField_Tradeable_get, _thostmduserapi.CThostFtdcBrokerUserRightAssignField_Tradeable_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcBrokerUserRightAssignField_swiginit(self, _thostmduserapi.new_CThostFtdcBrokerUserRightAssignField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcBrokerUserRightAssignField

# Register CThostFtdcBrokerUserRightAssignField in _thostmduserapi:
_thostmduserapi.CThostFtdcBrokerUserRightAssignField_swigregister(CThostFtdcBrokerUserRightAssignField)

class CThostFtdcDRTransferField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    OrigDRIdentityID = property(_thostmduserapi.CThostFtdcDRTransferField_OrigDRIdentityID_get, _thostmduserapi.CThostFtdcDRTransferField_OrigDRIdentityID_set)
    DestDRIdentityID = property(_thostmduserapi.CThostFtdcDRTransferField_DestDRIdentityID_get, _thostmduserapi.CThostFtdcDRTransferField_DestDRIdentityID_set)
    OrigBrokerID = property(_thostmduserapi.CThostFtdcDRTransferField_OrigBrokerID_get, _thostmduserapi.CThostFtdcDRTransferField_OrigBrokerID_set)
    DestBrokerID = property(_thostmduserapi.CThostFtdcDRTransferField_DestBrokerID_get, _thostmduserapi.CThostFtdcDRTransferField_DestBrokerID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcDRTransferField_swiginit(self, _thostmduserapi.new_CThostFtdcDRTransferField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcDRTransferField

# Register CThostFtdcDRTransferField in _thostmduserapi:
_thostmduserapi.CThostFtdcDRTransferField_swigregister(CThostFtdcDRTransferField)

class CThostFtdcFensUserInfoField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcFensUserInfoField_BrokerID_get, _thostmduserapi.CThostFtdcFensUserInfoField_BrokerID_set)
    UserID = property(_thostmduserapi.CThostFtdcFensUserInfoField_UserID_get, _thostmduserapi.CThostFtdcFensUserInfoField_UserID_set)
    LoginMode = property(_thostmduserapi.CThostFtdcFensUserInfoField_LoginMode_get, _thostmduserapi.CThostFtdcFensUserInfoField_LoginMode_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcFensUserInfoField_swiginit(self, _thostmduserapi.new_CThostFtdcFensUserInfoField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcFensUserInfoField

# Register CThostFtdcFensUserInfoField in _thostmduserapi:
_thostmduserapi.CThostFtdcFensUserInfoField_swigregister(CThostFtdcFensUserInfoField)

class CThostFtdcCurrTransferIdentityField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    IdentityID = property(_thostmduserapi.CThostFtdcCurrTransferIdentityField_IdentityID_get, _thostmduserapi.CThostFtdcCurrTransferIdentityField_IdentityID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcCurrTransferIdentityField_swiginit(self, _thostmduserapi.new_CThostFtdcCurrTransferIdentityField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcCurrTransferIdentityField

# Register CThostFtdcCurrTransferIdentityField in _thostmduserapi:
_thostmduserapi.CThostFtdcCurrTransferIdentityField_swigregister(CThostFtdcCurrTransferIdentityField)

class CThostFtdcLoginForbiddenUserField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcLoginForbiddenUserField_BrokerID_get, _thostmduserapi.CThostFtdcLoginForbiddenUserField_BrokerID_set)
    UserID = property(_thostmduserapi.CThostFtdcLoginForbiddenUserField_UserID_get, _thostmduserapi.CThostFtdcLoginForbiddenUserField_UserID_set)
    IPAddress = property(_thostmduserapi.CThostFtdcLoginForbiddenUserField_IPAddress_get, _thostmduserapi.CThostFtdcLoginForbiddenUserField_IPAddress_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcLoginForbiddenUserField_swiginit(self, _thostmduserapi.new_CThostFtdcLoginForbiddenUserField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcLoginForbiddenUserField

# Register CThostFtdcLoginForbiddenUserField in _thostmduserapi:
_thostmduserapi.CThostFtdcLoginForbiddenUserField_swigregister(CThostFtdcLoginForbiddenUserField)

class CThostFtdcQryLoginForbiddenUserField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQryLoginForbiddenUserField_BrokerID_get, _thostmduserapi.CThostFtdcQryLoginForbiddenUserField_BrokerID_set)
    UserID = property(_thostmduserapi.CThostFtdcQryLoginForbiddenUserField_UserID_get, _thostmduserapi.CThostFtdcQryLoginForbiddenUserField_UserID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryLoginForbiddenUserField_swiginit(self, _thostmduserapi.new_CThostFtdcQryLoginForbiddenUserField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryLoginForbiddenUserField

# Register CThostFtdcQryLoginForbiddenUserField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryLoginForbiddenUserField_swigregister(CThostFtdcQryLoginForbiddenUserField)

class CThostFtdcMulticastGroupInfoField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    GroupIP = property(_thostmduserapi.CThostFtdcMulticastGroupInfoField_GroupIP_get, _thostmduserapi.CThostFtdcMulticastGroupInfoField_GroupIP_set)
    GroupPort = property(_thostmduserapi.CThostFtdcMulticastGroupInfoField_GroupPort_get, _thostmduserapi.CThostFtdcMulticastGroupInfoField_GroupPort_set)
    SourceIP = property(_thostmduserapi.CThostFtdcMulticastGroupInfoField_SourceIP_get, _thostmduserapi.CThostFtdcMulticastGroupInfoField_SourceIP_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcMulticastGroupInfoField_swiginit(self, _thostmduserapi.new_CThostFtdcMulticastGroupInfoField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcMulticastGroupInfoField

# Register CThostFtdcMulticastGroupInfoField in _thostmduserapi:
_thostmduserapi.CThostFtdcMulticastGroupInfoField_swigregister(CThostFtdcMulticastGroupInfoField)

class CThostFtdcTradingAccountReserveField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcTradingAccountReserveField_BrokerID_get, _thostmduserapi.CThostFtdcTradingAccountReserveField_BrokerID_set)
    AccountID = property(_thostmduserapi.CThostFtdcTradingAccountReserveField_AccountID_get, _thostmduserapi.CThostFtdcTradingAccountReserveField_AccountID_set)
    Reserve = property(_thostmduserapi.CThostFtdcTradingAccountReserveField_Reserve_get, _thostmduserapi.CThostFtdcTradingAccountReserveField_Reserve_set)
    CurrencyID = property(_thostmduserapi.CThostFtdcTradingAccountReserveField_CurrencyID_get, _thostmduserapi.CThostFtdcTradingAccountReserveField_CurrencyID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcTradingAccountReserveField_swiginit(self, _thostmduserapi.new_CThostFtdcTradingAccountReserveField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcTradingAccountReserveField

# Register CThostFtdcTradingAccountReserveField in _thostmduserapi:
_thostmduserapi.CThostFtdcTradingAccountReserveField_swigregister(CThostFtdcTradingAccountReserveField)

class CThostFtdcQryLoginForbiddenIPField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    IPAddress = property(_thostmduserapi.CThostFtdcQryLoginForbiddenIPField_IPAddress_get, _thostmduserapi.CThostFtdcQryLoginForbiddenIPField_IPAddress_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryLoginForbiddenIPField_swiginit(self, _thostmduserapi.new_CThostFtdcQryLoginForbiddenIPField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryLoginForbiddenIPField

# Register CThostFtdcQryLoginForbiddenIPField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryLoginForbiddenIPField_swigregister(CThostFtdcQryLoginForbiddenIPField)

class CThostFtdcQryIPListField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    IPAddress = property(_thostmduserapi.CThostFtdcQryIPListField_IPAddress_get, _thostmduserapi.CThostFtdcQryIPListField_IPAddress_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryIPListField_swiginit(self, _thostmduserapi.new_CThostFtdcQryIPListField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryIPListField

# Register CThostFtdcQryIPListField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryIPListField_swigregister(CThostFtdcQryIPListField)

class CThostFtdcQryUserRightsAssignField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQryUserRightsAssignField_BrokerID_get, _thostmduserapi.CThostFtdcQryUserRightsAssignField_BrokerID_set)
    UserID = property(_thostmduserapi.CThostFtdcQryUserRightsAssignField_UserID_get, _thostmduserapi.CThostFtdcQryUserRightsAssignField_UserID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryUserRightsAssignField_swiginit(self, _thostmduserapi.new_CThostFtdcQryUserRightsAssignField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryUserRightsAssignField

# Register CThostFtdcQryUserRightsAssignField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryUserRightsAssignField_swigregister(CThostFtdcQryUserRightsAssignField)

class CThostFtdcReserveOpenAccountConfirmField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradeCode = property(_thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_TradeCode_get, _thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_TradeCode_set)
    BankID = property(_thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_BankID_get, _thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_BankID_set)
    BankBranchID = property(_thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_BankBranchID_get, _thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_BankBranchID_set)
    BrokerID = property(_thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_BrokerID_get, _thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_BrokerID_set)
    BrokerBranchID = property(_thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_BrokerBranchID_get, _thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_BrokerBranchID_set)
    TradeDate = property(_thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_TradeDate_get, _thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_TradeDate_set)
    TradeTime = property(_thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_TradeTime_get, _thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_TradeTime_set)
    BankSerial = property(_thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_BankSerial_get, _thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_BankSerial_set)
    TradingDay = property(_thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_TradingDay_get, _thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_TradingDay_set)
    PlateSerial = property(_thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_PlateSerial_get, _thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_PlateSerial_set)
    LastFragment = property(_thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_LastFragment_get, _thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_LastFragment_set)
    SessionID = property(_thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_SessionID_get, _thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_SessionID_set)
    CustomerName = property(_thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_CustomerName_get, _thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_CustomerName_set)
    IdCardType = property(_thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_IdCardType_get, _thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_IdCardType_set)
    IdentifiedCardNo = property(_thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_IdentifiedCardNo_get, _thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_IdentifiedCardNo_set)
    Gender = property(_thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_Gender_get, _thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_Gender_set)
    CountryCode = property(_thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_CountryCode_get, _thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_CountryCode_set)
    CustType = property(_thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_CustType_get, _thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_CustType_set)
    Address = property(_thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_Address_get, _thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_Address_set)
    ZipCode = property(_thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_ZipCode_get, _thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_ZipCode_set)
    Telephone = property(_thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_Telephone_get, _thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_Telephone_set)
    MobilePhone = property(_thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_MobilePhone_get, _thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_MobilePhone_set)
    Fax = property(_thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_Fax_get, _thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_Fax_set)
    EMail = property(_thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_EMail_get, _thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_EMail_set)
    MoneyAccountStatus = property(_thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_MoneyAccountStatus_get, _thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_MoneyAccountStatus_set)
    BankAccount = property(_thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_BankAccount_get, _thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_BankAccount_set)
    BankPassWord = property(_thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_BankPassWord_get, _thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_BankPassWord_set)
    InstallID = property(_thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_InstallID_get, _thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_InstallID_set)
    VerifyCertNoFlag = property(_thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_VerifyCertNoFlag_get, _thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_VerifyCertNoFlag_set)
    CurrencyID = property(_thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_CurrencyID_get, _thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_CurrencyID_set)
    Digest = property(_thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_Digest_get, _thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_Digest_set)
    BankAccType = property(_thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_BankAccType_get, _thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_BankAccType_set)
    BrokerIDByBank = property(_thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_BrokerIDByBank_get, _thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_BrokerIDByBank_set)
    TID = property(_thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_TID_get, _thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_TID_set)
    AccountID = property(_thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_AccountID_get, _thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_AccountID_set)
    Password = property(_thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_Password_get, _thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_Password_set)
    BankReserveOpenSeq = property(_thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_BankReserveOpenSeq_get, _thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_BankReserveOpenSeq_set)
    BookDate = property(_thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_BookDate_get, _thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_BookDate_set)
    BookPsw = property(_thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_BookPsw_get, _thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_BookPsw_set)
    ErrorID = property(_thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_ErrorID_get, _thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_ErrorID_set)
    ErrorMsg = property(_thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_ErrorMsg_get, _thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_ErrorMsg_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_swiginit(self, _thostmduserapi.new_CThostFtdcReserveOpenAccountConfirmField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcReserveOpenAccountConfirmField

# Register CThostFtdcReserveOpenAccountConfirmField in _thostmduserapi:
_thostmduserapi.CThostFtdcReserveOpenAccountConfirmField_swigregister(CThostFtdcReserveOpenAccountConfirmField)

class CThostFtdcReserveOpenAccountField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradeCode = property(_thostmduserapi.CThostFtdcReserveOpenAccountField_TradeCode_get, _thostmduserapi.CThostFtdcReserveOpenAccountField_TradeCode_set)
    BankID = property(_thostmduserapi.CThostFtdcReserveOpenAccountField_BankID_get, _thostmduserapi.CThostFtdcReserveOpenAccountField_BankID_set)
    BankBranchID = property(_thostmduserapi.CThostFtdcReserveOpenAccountField_BankBranchID_get, _thostmduserapi.CThostFtdcReserveOpenAccountField_BankBranchID_set)
    BrokerID = property(_thostmduserapi.CThostFtdcReserveOpenAccountField_BrokerID_get, _thostmduserapi.CThostFtdcReserveOpenAccountField_BrokerID_set)
    BrokerBranchID = property(_thostmduserapi.CThostFtdcReserveOpenAccountField_BrokerBranchID_get, _thostmduserapi.CThostFtdcReserveOpenAccountField_BrokerBranchID_set)
    TradeDate = property(_thostmduserapi.CThostFtdcReserveOpenAccountField_TradeDate_get, _thostmduserapi.CThostFtdcReserveOpenAccountField_TradeDate_set)
    TradeTime = property(_thostmduserapi.CThostFtdcReserveOpenAccountField_TradeTime_get, _thostmduserapi.CThostFtdcReserveOpenAccountField_TradeTime_set)
    BankSerial = property(_thostmduserapi.CThostFtdcReserveOpenAccountField_BankSerial_get, _thostmduserapi.CThostFtdcReserveOpenAccountField_BankSerial_set)
    TradingDay = property(_thostmduserapi.CThostFtdcReserveOpenAccountField_TradingDay_get, _thostmduserapi.CThostFtdcReserveOpenAccountField_TradingDay_set)
    PlateSerial = property(_thostmduserapi.CThostFtdcReserveOpenAccountField_PlateSerial_get, _thostmduserapi.CThostFtdcReserveOpenAccountField_PlateSerial_set)
    LastFragment = property(_thostmduserapi.CThostFtdcReserveOpenAccountField_LastFragment_get, _thostmduserapi.CThostFtdcReserveOpenAccountField_LastFragment_set)
    SessionID = property(_thostmduserapi.CThostFtdcReserveOpenAccountField_SessionID_get, _thostmduserapi.CThostFtdcReserveOpenAccountField_SessionID_set)
    CustomerName = property(_thostmduserapi.CThostFtdcReserveOpenAccountField_CustomerName_get, _thostmduserapi.CThostFtdcReserveOpenAccountField_CustomerName_set)
    IdCardType = property(_thostmduserapi.CThostFtdcReserveOpenAccountField_IdCardType_get, _thostmduserapi.CThostFtdcReserveOpenAccountField_IdCardType_set)
    IdentifiedCardNo = property(_thostmduserapi.CThostFtdcReserveOpenAccountField_IdentifiedCardNo_get, _thostmduserapi.CThostFtdcReserveOpenAccountField_IdentifiedCardNo_set)
    Gender = property(_thostmduserapi.CThostFtdcReserveOpenAccountField_Gender_get, _thostmduserapi.CThostFtdcReserveOpenAccountField_Gender_set)
    CountryCode = property(_thostmduserapi.CThostFtdcReserveOpenAccountField_CountryCode_get, _thostmduserapi.CThostFtdcReserveOpenAccountField_CountryCode_set)
    CustType = property(_thostmduserapi.CThostFtdcReserveOpenAccountField_CustType_get, _thostmduserapi.CThostFtdcReserveOpenAccountField_CustType_set)
    Address = property(_thostmduserapi.CThostFtdcReserveOpenAccountField_Address_get, _thostmduserapi.CThostFtdcReserveOpenAccountField_Address_set)
    ZipCode = property(_thostmduserapi.CThostFtdcReserveOpenAccountField_ZipCode_get, _thostmduserapi.CThostFtdcReserveOpenAccountField_ZipCode_set)
    Telephone = property(_thostmduserapi.CThostFtdcReserveOpenAccountField_Telephone_get, _thostmduserapi.CThostFtdcReserveOpenAccountField_Telephone_set)
    MobilePhone = property(_thostmduserapi.CThostFtdcReserveOpenAccountField_MobilePhone_get, _thostmduserapi.CThostFtdcReserveOpenAccountField_MobilePhone_set)
    Fax = property(_thostmduserapi.CThostFtdcReserveOpenAccountField_Fax_get, _thostmduserapi.CThostFtdcReserveOpenAccountField_Fax_set)
    EMail = property(_thostmduserapi.CThostFtdcReserveOpenAccountField_EMail_get, _thostmduserapi.CThostFtdcReserveOpenAccountField_EMail_set)
    MoneyAccountStatus = property(_thostmduserapi.CThostFtdcReserveOpenAccountField_MoneyAccountStatus_get, _thostmduserapi.CThostFtdcReserveOpenAccountField_MoneyAccountStatus_set)
    BankAccount = property(_thostmduserapi.CThostFtdcReserveOpenAccountField_BankAccount_get, _thostmduserapi.CThostFtdcReserveOpenAccountField_BankAccount_set)
    BankPassWord = property(_thostmduserapi.CThostFtdcReserveOpenAccountField_BankPassWord_get, _thostmduserapi.CThostFtdcReserveOpenAccountField_BankPassWord_set)
    InstallID = property(_thostmduserapi.CThostFtdcReserveOpenAccountField_InstallID_get, _thostmduserapi.CThostFtdcReserveOpenAccountField_InstallID_set)
    VerifyCertNoFlag = property(_thostmduserapi.CThostFtdcReserveOpenAccountField_VerifyCertNoFlag_get, _thostmduserapi.CThostFtdcReserveOpenAccountField_VerifyCertNoFlag_set)
    CurrencyID = property(_thostmduserapi.CThostFtdcReserveOpenAccountField_CurrencyID_get, _thostmduserapi.CThostFtdcReserveOpenAccountField_CurrencyID_set)
    Digest = property(_thostmduserapi.CThostFtdcReserveOpenAccountField_Digest_get, _thostmduserapi.CThostFtdcReserveOpenAccountField_Digest_set)
    BankAccType = property(_thostmduserapi.CThostFtdcReserveOpenAccountField_BankAccType_get, _thostmduserapi.CThostFtdcReserveOpenAccountField_BankAccType_set)
    BrokerIDByBank = property(_thostmduserapi.CThostFtdcReserveOpenAccountField_BrokerIDByBank_get, _thostmduserapi.CThostFtdcReserveOpenAccountField_BrokerIDByBank_set)
    TID = property(_thostmduserapi.CThostFtdcReserveOpenAccountField_TID_get, _thostmduserapi.CThostFtdcReserveOpenAccountField_TID_set)
    ReserveOpenAccStas = property(_thostmduserapi.CThostFtdcReserveOpenAccountField_ReserveOpenAccStas_get, _thostmduserapi.CThostFtdcReserveOpenAccountField_ReserveOpenAccStas_set)
    ErrorID = property(_thostmduserapi.CThostFtdcReserveOpenAccountField_ErrorID_get, _thostmduserapi.CThostFtdcReserveOpenAccountField_ErrorID_set)
    ErrorMsg = property(_thostmduserapi.CThostFtdcReserveOpenAccountField_ErrorMsg_get, _thostmduserapi.CThostFtdcReserveOpenAccountField_ErrorMsg_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcReserveOpenAccountField_swiginit(self, _thostmduserapi.new_CThostFtdcReserveOpenAccountField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcReserveOpenAccountField

# Register CThostFtdcReserveOpenAccountField in _thostmduserapi:
_thostmduserapi.CThostFtdcReserveOpenAccountField_swigregister(CThostFtdcReserveOpenAccountField)

class CThostFtdcAccountPropertyField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcAccountPropertyField_BrokerID_get, _thostmduserapi.CThostFtdcAccountPropertyField_BrokerID_set)
    AccountID = property(_thostmduserapi.CThostFtdcAccountPropertyField_AccountID_get, _thostmduserapi.CThostFtdcAccountPropertyField_AccountID_set)
    BankID = property(_thostmduserapi.CThostFtdcAccountPropertyField_BankID_get, _thostmduserapi.CThostFtdcAccountPropertyField_BankID_set)
    BankAccount = property(_thostmduserapi.CThostFtdcAccountPropertyField_BankAccount_get, _thostmduserapi.CThostFtdcAccountPropertyField_BankAccount_set)
    OpenName = property(_thostmduserapi.CThostFtdcAccountPropertyField_OpenName_get, _thostmduserapi.CThostFtdcAccountPropertyField_OpenName_set)
    OpenBank = property(_thostmduserapi.CThostFtdcAccountPropertyField_OpenBank_get, _thostmduserapi.CThostFtdcAccountPropertyField_OpenBank_set)
    IsActive = property(_thostmduserapi.CThostFtdcAccountPropertyField_IsActive_get, _thostmduserapi.CThostFtdcAccountPropertyField_IsActive_set)
    AccountSourceType = property(_thostmduserapi.CThostFtdcAccountPropertyField_AccountSourceType_get, _thostmduserapi.CThostFtdcAccountPropertyField_AccountSourceType_set)
    OpenDate = property(_thostmduserapi.CThostFtdcAccountPropertyField_OpenDate_get, _thostmduserapi.CThostFtdcAccountPropertyField_OpenDate_set)
    CancelDate = property(_thostmduserapi.CThostFtdcAccountPropertyField_CancelDate_get, _thostmduserapi.CThostFtdcAccountPropertyField_CancelDate_set)
    OperatorID = property(_thostmduserapi.CThostFtdcAccountPropertyField_OperatorID_get, _thostmduserapi.CThostFtdcAccountPropertyField_OperatorID_set)
    OperateDate = property(_thostmduserapi.CThostFtdcAccountPropertyField_OperateDate_get, _thostmduserapi.CThostFtdcAccountPropertyField_OperateDate_set)
    OperateTime = property(_thostmduserapi.CThostFtdcAccountPropertyField_OperateTime_get, _thostmduserapi.CThostFtdcAccountPropertyField_OperateTime_set)
    CurrencyID = property(_thostmduserapi.CThostFtdcAccountPropertyField_CurrencyID_get, _thostmduserapi.CThostFtdcAccountPropertyField_CurrencyID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcAccountPropertyField_swiginit(self, _thostmduserapi.new_CThostFtdcAccountPropertyField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcAccountPropertyField

# Register CThostFtdcAccountPropertyField in _thostmduserapi:
_thostmduserapi.CThostFtdcAccountPropertyField_swigregister(CThostFtdcAccountPropertyField)

class CThostFtdcQryCurrDRIdentityField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    DRIdentityID = property(_thostmduserapi.CThostFtdcQryCurrDRIdentityField_DRIdentityID_get, _thostmduserapi.CThostFtdcQryCurrDRIdentityField_DRIdentityID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQryCurrDRIdentityField_swiginit(self, _thostmduserapi.new_CThostFtdcQryCurrDRIdentityField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQryCurrDRIdentityField

# Register CThostFtdcQryCurrDRIdentityField in _thostmduserapi:
_thostmduserapi.CThostFtdcQryCurrDRIdentityField_swigregister(CThostFtdcQryCurrDRIdentityField)

class CThostFtdcCurrDRIdentityField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    DRIdentityID = property(_thostmduserapi.CThostFtdcCurrDRIdentityField_DRIdentityID_get, _thostmduserapi.CThostFtdcCurrDRIdentityField_DRIdentityID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcCurrDRIdentityField_swiginit(self, _thostmduserapi.new_CThostFtdcCurrDRIdentityField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcCurrDRIdentityField

# Register CThostFtdcCurrDRIdentityField in _thostmduserapi:
_thostmduserapi.CThostFtdcCurrDRIdentityField_swigregister(CThostFtdcCurrDRIdentityField)

class CThostFtdcQrySecAgentCheckModeField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQrySecAgentCheckModeField_BrokerID_get, _thostmduserapi.CThostFtdcQrySecAgentCheckModeField_BrokerID_set)
    InvestorID = property(_thostmduserapi.CThostFtdcQrySecAgentCheckModeField_InvestorID_get, _thostmduserapi.CThostFtdcQrySecAgentCheckModeField_InvestorID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQrySecAgentCheckModeField_swiginit(self, _thostmduserapi.new_CThostFtdcQrySecAgentCheckModeField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQrySecAgentCheckModeField

# Register CThostFtdcQrySecAgentCheckModeField in _thostmduserapi:
_thostmduserapi.CThostFtdcQrySecAgentCheckModeField_swigregister(CThostFtdcQrySecAgentCheckModeField)

class CThostFtdcQrySecAgentTradeInfoField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcQrySecAgentTradeInfoField_BrokerID_get, _thostmduserapi.CThostFtdcQrySecAgentTradeInfoField_BrokerID_set)
    BrokerSecAgentID = property(_thostmduserapi.CThostFtdcQrySecAgentTradeInfoField_BrokerSecAgentID_get, _thostmduserapi.CThostFtdcQrySecAgentTradeInfoField_BrokerSecAgentID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQrySecAgentTradeInfoField_swiginit(self, _thostmduserapi.new_CThostFtdcQrySecAgentTradeInfoField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQrySecAgentTradeInfoField

# Register CThostFtdcQrySecAgentTradeInfoField in _thostmduserapi:
_thostmduserapi.CThostFtdcQrySecAgentTradeInfoField_swigregister(CThostFtdcQrySecAgentTradeInfoField)

class CThostFtdcUserSystemInfoField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcUserSystemInfoField_BrokerID_get, _thostmduserapi.CThostFtdcUserSystemInfoField_BrokerID_set)
    UserID = property(_thostmduserapi.CThostFtdcUserSystemInfoField_UserID_get, _thostmduserapi.CThostFtdcUserSystemInfoField_UserID_set)
    ClientSystemInfoLen = property(_thostmduserapi.CThostFtdcUserSystemInfoField_ClientSystemInfoLen_get, _thostmduserapi.CThostFtdcUserSystemInfoField_ClientSystemInfoLen_set)
    ClientSystemInfo = property(_thostmduserapi.CThostFtdcUserSystemInfoField_ClientSystemInfo_get, _thostmduserapi.CThostFtdcUserSystemInfoField_ClientSystemInfo_set)
    ClientPublicIP = property(_thostmduserapi.CThostFtdcUserSystemInfoField_ClientPublicIP_get, _thostmduserapi.CThostFtdcUserSystemInfoField_ClientPublicIP_set)
    ClientIPPort = property(_thostmduserapi.CThostFtdcUserSystemInfoField_ClientIPPort_get, _thostmduserapi.CThostFtdcUserSystemInfoField_ClientIPPort_set)
    ClientLoginTime = property(_thostmduserapi.CThostFtdcUserSystemInfoField_ClientLoginTime_get, _thostmduserapi.CThostFtdcUserSystemInfoField_ClientLoginTime_set)
    ClientAppID = property(_thostmduserapi.CThostFtdcUserSystemInfoField_ClientAppID_get, _thostmduserapi.CThostFtdcUserSystemInfoField_ClientAppID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcUserSystemInfoField_swiginit(self, _thostmduserapi.new_CThostFtdcUserSystemInfoField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcUserSystemInfoField

# Register CThostFtdcUserSystemInfoField in _thostmduserapi:
_thostmduserapi.CThostFtdcUserSystemInfoField_swigregister(CThostFtdcUserSystemInfoField)

class CThostFtdcReqUserAuthMethodField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradingDay = property(_thostmduserapi.CThostFtdcReqUserAuthMethodField_TradingDay_get, _thostmduserapi.CThostFtdcReqUserAuthMethodField_TradingDay_set)
    BrokerID = property(_thostmduserapi.CThostFtdcReqUserAuthMethodField_BrokerID_get, _thostmduserapi.CThostFtdcReqUserAuthMethodField_BrokerID_set)
    UserID = property(_thostmduserapi.CThostFtdcReqUserAuthMethodField_UserID_get, _thostmduserapi.CThostFtdcReqUserAuthMethodField_UserID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcReqUserAuthMethodField_swiginit(self, _thostmduserapi.new_CThostFtdcReqUserAuthMethodField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcReqUserAuthMethodField

# Register CThostFtdcReqUserAuthMethodField in _thostmduserapi:
_thostmduserapi.CThostFtdcReqUserAuthMethodField_swigregister(CThostFtdcReqUserAuthMethodField)

class CThostFtdcRspUserAuthMethodField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    UsableAuthMethod = property(_thostmduserapi.CThostFtdcRspUserAuthMethodField_UsableAuthMethod_get, _thostmduserapi.CThostFtdcRspUserAuthMethodField_UsableAuthMethod_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcRspUserAuthMethodField_swiginit(self, _thostmduserapi.new_CThostFtdcRspUserAuthMethodField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcRspUserAuthMethodField

# Register CThostFtdcRspUserAuthMethodField in _thostmduserapi:
_thostmduserapi.CThostFtdcRspUserAuthMethodField_swigregister(CThostFtdcRspUserAuthMethodField)

class CThostFtdcReqGenUserCaptchaField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradingDay = property(_thostmduserapi.CThostFtdcReqGenUserCaptchaField_TradingDay_get, _thostmduserapi.CThostFtdcReqGenUserCaptchaField_TradingDay_set)
    BrokerID = property(_thostmduserapi.CThostFtdcReqGenUserCaptchaField_BrokerID_get, _thostmduserapi.CThostFtdcReqGenUserCaptchaField_BrokerID_set)
    UserID = property(_thostmduserapi.CThostFtdcReqGenUserCaptchaField_UserID_get, _thostmduserapi.CThostFtdcReqGenUserCaptchaField_UserID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcReqGenUserCaptchaField_swiginit(self, _thostmduserapi.new_CThostFtdcReqGenUserCaptchaField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcReqGenUserCaptchaField

# Register CThostFtdcReqGenUserCaptchaField in _thostmduserapi:
_thostmduserapi.CThostFtdcReqGenUserCaptchaField_swigregister(CThostFtdcReqGenUserCaptchaField)

class CThostFtdcRspGenUserCaptchaField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcRspGenUserCaptchaField_BrokerID_get, _thostmduserapi.CThostFtdcRspGenUserCaptchaField_BrokerID_set)
    UserID = property(_thostmduserapi.CThostFtdcRspGenUserCaptchaField_UserID_get, _thostmduserapi.CThostFtdcRspGenUserCaptchaField_UserID_set)
    CaptchaInfoLen = property(_thostmduserapi.CThostFtdcRspGenUserCaptchaField_CaptchaInfoLen_get, _thostmduserapi.CThostFtdcRspGenUserCaptchaField_CaptchaInfoLen_set)
    CaptchaInfo = property(_thostmduserapi.CThostFtdcRspGenUserCaptchaField_CaptchaInfo_get, _thostmduserapi.CThostFtdcRspGenUserCaptchaField_CaptchaInfo_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcRspGenUserCaptchaField_swiginit(self, _thostmduserapi.new_CThostFtdcRspGenUserCaptchaField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcRspGenUserCaptchaField

# Register CThostFtdcRspGenUserCaptchaField in _thostmduserapi:
_thostmduserapi.CThostFtdcRspGenUserCaptchaField_swigregister(CThostFtdcRspGenUserCaptchaField)

class CThostFtdcReqGenUserTextField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradingDay = property(_thostmduserapi.CThostFtdcReqGenUserTextField_TradingDay_get, _thostmduserapi.CThostFtdcReqGenUserTextField_TradingDay_set)
    BrokerID = property(_thostmduserapi.CThostFtdcReqGenUserTextField_BrokerID_get, _thostmduserapi.CThostFtdcReqGenUserTextField_BrokerID_set)
    UserID = property(_thostmduserapi.CThostFtdcReqGenUserTextField_UserID_get, _thostmduserapi.CThostFtdcReqGenUserTextField_UserID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcReqGenUserTextField_swiginit(self, _thostmduserapi.new_CThostFtdcReqGenUserTextField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcReqGenUserTextField

# Register CThostFtdcReqGenUserTextField in _thostmduserapi:
_thostmduserapi.CThostFtdcReqGenUserTextField_swigregister(CThostFtdcReqGenUserTextField)

class CThostFtdcRspGenUserTextField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    UserTextSeq = property(_thostmduserapi.CThostFtdcRspGenUserTextField_UserTextSeq_get, _thostmduserapi.CThostFtdcRspGenUserTextField_UserTextSeq_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcRspGenUserTextField_swiginit(self, _thostmduserapi.new_CThostFtdcRspGenUserTextField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcRspGenUserTextField

# Register CThostFtdcRspGenUserTextField in _thostmduserapi:
_thostmduserapi.CThostFtdcRspGenUserTextField_swigregister(CThostFtdcRspGenUserTextField)

class CThostFtdcReqUserLoginWithCaptchaField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradingDay = property(_thostmduserapi.CThostFtdcReqUserLoginWithCaptchaField_TradingDay_get, _thostmduserapi.CThostFtdcReqUserLoginWithCaptchaField_TradingDay_set)
    BrokerID = property(_thostmduserapi.CThostFtdcReqUserLoginWithCaptchaField_BrokerID_get, _thostmduserapi.CThostFtdcReqUserLoginWithCaptchaField_BrokerID_set)
    UserID = property(_thostmduserapi.CThostFtdcReqUserLoginWithCaptchaField_UserID_get, _thostmduserapi.CThostFtdcReqUserLoginWithCaptchaField_UserID_set)
    Password = property(_thostmduserapi.CThostFtdcReqUserLoginWithCaptchaField_Password_get, _thostmduserapi.CThostFtdcReqUserLoginWithCaptchaField_Password_set)
    UserProductInfo = property(_thostmduserapi.CThostFtdcReqUserLoginWithCaptchaField_UserProductInfo_get, _thostmduserapi.CThostFtdcReqUserLoginWithCaptchaField_UserProductInfo_set)
    InterfaceProductInfo = property(_thostmduserapi.CThostFtdcReqUserLoginWithCaptchaField_InterfaceProductInfo_get, _thostmduserapi.CThostFtdcReqUserLoginWithCaptchaField_InterfaceProductInfo_set)
    ProtocolInfo = property(_thostmduserapi.CThostFtdcReqUserLoginWithCaptchaField_ProtocolInfo_get, _thostmduserapi.CThostFtdcReqUserLoginWithCaptchaField_ProtocolInfo_set)
    MacAddress = property(_thostmduserapi.CThostFtdcReqUserLoginWithCaptchaField_MacAddress_get, _thostmduserapi.CThostFtdcReqUserLoginWithCaptchaField_MacAddress_set)
    ClientIPAddress = property(_thostmduserapi.CThostFtdcReqUserLoginWithCaptchaField_ClientIPAddress_get, _thostmduserapi.CThostFtdcReqUserLoginWithCaptchaField_ClientIPAddress_set)
    LoginRemark = property(_thostmduserapi.CThostFtdcReqUserLoginWithCaptchaField_LoginRemark_get, _thostmduserapi.CThostFtdcReqUserLoginWithCaptchaField_LoginRemark_set)
    Captcha = property(_thostmduserapi.CThostFtdcReqUserLoginWithCaptchaField_Captcha_get, _thostmduserapi.CThostFtdcReqUserLoginWithCaptchaField_Captcha_set)
    ClientIPPort = property(_thostmduserapi.CThostFtdcReqUserLoginWithCaptchaField_ClientIPPort_get, _thostmduserapi.CThostFtdcReqUserLoginWithCaptchaField_ClientIPPort_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcReqUserLoginWithCaptchaField_swiginit(self, _thostmduserapi.new_CThostFtdcReqUserLoginWithCaptchaField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcReqUserLoginWithCaptchaField

# Register CThostFtdcReqUserLoginWithCaptchaField in _thostmduserapi:
_thostmduserapi.CThostFtdcReqUserLoginWithCaptchaField_swigregister(CThostFtdcReqUserLoginWithCaptchaField)

class CThostFtdcReqUserLoginWithTextField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradingDay = property(_thostmduserapi.CThostFtdcReqUserLoginWithTextField_TradingDay_get, _thostmduserapi.CThostFtdcReqUserLoginWithTextField_TradingDay_set)
    BrokerID = property(_thostmduserapi.CThostFtdcReqUserLoginWithTextField_BrokerID_get, _thostmduserapi.CThostFtdcReqUserLoginWithTextField_BrokerID_set)
    UserID = property(_thostmduserapi.CThostFtdcReqUserLoginWithTextField_UserID_get, _thostmduserapi.CThostFtdcReqUserLoginWithTextField_UserID_set)
    Password = property(_thostmduserapi.CThostFtdcReqUserLoginWithTextField_Password_get, _thostmduserapi.CThostFtdcReqUserLoginWithTextField_Password_set)
    UserProductInfo = property(_thostmduserapi.CThostFtdcReqUserLoginWithTextField_UserProductInfo_get, _thostmduserapi.CThostFtdcReqUserLoginWithTextField_UserProductInfo_set)
    InterfaceProductInfo = property(_thostmduserapi.CThostFtdcReqUserLoginWithTextField_InterfaceProductInfo_get, _thostmduserapi.CThostFtdcReqUserLoginWithTextField_InterfaceProductInfo_set)
    ProtocolInfo = property(_thostmduserapi.CThostFtdcReqUserLoginWithTextField_ProtocolInfo_get, _thostmduserapi.CThostFtdcReqUserLoginWithTextField_ProtocolInfo_set)
    MacAddress = property(_thostmduserapi.CThostFtdcReqUserLoginWithTextField_MacAddress_get, _thostmduserapi.CThostFtdcReqUserLoginWithTextField_MacAddress_set)
    ClientIPAddress = property(_thostmduserapi.CThostFtdcReqUserLoginWithTextField_ClientIPAddress_get, _thostmduserapi.CThostFtdcReqUserLoginWithTextField_ClientIPAddress_set)
    LoginRemark = property(_thostmduserapi.CThostFtdcReqUserLoginWithTextField_LoginRemark_get, _thostmduserapi.CThostFtdcReqUserLoginWithTextField_LoginRemark_set)
    Text = property(_thostmduserapi.CThostFtdcReqUserLoginWithTextField_Text_get, _thostmduserapi.CThostFtdcReqUserLoginWithTextField_Text_set)
    ClientIPPort = property(_thostmduserapi.CThostFtdcReqUserLoginWithTextField_ClientIPPort_get, _thostmduserapi.CThostFtdcReqUserLoginWithTextField_ClientIPPort_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcReqUserLoginWithTextField_swiginit(self, _thostmduserapi.new_CThostFtdcReqUserLoginWithTextField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcReqUserLoginWithTextField

# Register CThostFtdcReqUserLoginWithTextField in _thostmduserapi:
_thostmduserapi.CThostFtdcReqUserLoginWithTextField_swigregister(CThostFtdcReqUserLoginWithTextField)

class CThostFtdcReqUserLoginWithOTPField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    TradingDay = property(_thostmduserapi.CThostFtdcReqUserLoginWithOTPField_TradingDay_get, _thostmduserapi.CThostFtdcReqUserLoginWithOTPField_TradingDay_set)
    BrokerID = property(_thostmduserapi.CThostFtdcReqUserLoginWithOTPField_BrokerID_get, _thostmduserapi.CThostFtdcReqUserLoginWithOTPField_BrokerID_set)
    UserID = property(_thostmduserapi.CThostFtdcReqUserLoginWithOTPField_UserID_get, _thostmduserapi.CThostFtdcReqUserLoginWithOTPField_UserID_set)
    Password = property(_thostmduserapi.CThostFtdcReqUserLoginWithOTPField_Password_get, _thostmduserapi.CThostFtdcReqUserLoginWithOTPField_Password_set)
    UserProductInfo = property(_thostmduserapi.CThostFtdcReqUserLoginWithOTPField_UserProductInfo_get, _thostmduserapi.CThostFtdcReqUserLoginWithOTPField_UserProductInfo_set)
    InterfaceProductInfo = property(_thostmduserapi.CThostFtdcReqUserLoginWithOTPField_InterfaceProductInfo_get, _thostmduserapi.CThostFtdcReqUserLoginWithOTPField_InterfaceProductInfo_set)
    ProtocolInfo = property(_thostmduserapi.CThostFtdcReqUserLoginWithOTPField_ProtocolInfo_get, _thostmduserapi.CThostFtdcReqUserLoginWithOTPField_ProtocolInfo_set)
    MacAddress = property(_thostmduserapi.CThostFtdcReqUserLoginWithOTPField_MacAddress_get, _thostmduserapi.CThostFtdcReqUserLoginWithOTPField_MacAddress_set)
    ClientIPAddress = property(_thostmduserapi.CThostFtdcReqUserLoginWithOTPField_ClientIPAddress_get, _thostmduserapi.CThostFtdcReqUserLoginWithOTPField_ClientIPAddress_set)
    LoginRemark = property(_thostmduserapi.CThostFtdcReqUserLoginWithOTPField_LoginRemark_get, _thostmduserapi.CThostFtdcReqUserLoginWithOTPField_LoginRemark_set)
    OTPPassword = property(_thostmduserapi.CThostFtdcReqUserLoginWithOTPField_OTPPassword_get, _thostmduserapi.CThostFtdcReqUserLoginWithOTPField_OTPPassword_set)
    ClientIPPort = property(_thostmduserapi.CThostFtdcReqUserLoginWithOTPField_ClientIPPort_get, _thostmduserapi.CThostFtdcReqUserLoginWithOTPField_ClientIPPort_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcReqUserLoginWithOTPField_swiginit(self, _thostmduserapi.new_CThostFtdcReqUserLoginWithOTPField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcReqUserLoginWithOTPField

# Register CThostFtdcReqUserLoginWithOTPField in _thostmduserapi:
_thostmduserapi.CThostFtdcReqUserLoginWithOTPField_swigregister(CThostFtdcReqUserLoginWithOTPField)

class CThostFtdcReqApiHandshakeField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    CryptoKeyVersion = property(_thostmduserapi.CThostFtdcReqApiHandshakeField_CryptoKeyVersion_get, _thostmduserapi.CThostFtdcReqApiHandshakeField_CryptoKeyVersion_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcReqApiHandshakeField_swiginit(self, _thostmduserapi.new_CThostFtdcReqApiHandshakeField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcReqApiHandshakeField

# Register CThostFtdcReqApiHandshakeField in _thostmduserapi:
_thostmduserapi.CThostFtdcReqApiHandshakeField_swigregister(CThostFtdcReqApiHandshakeField)

class CThostFtdcRspApiHandshakeField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    FrontHandshakeDataLen = property(_thostmduserapi.CThostFtdcRspApiHandshakeField_FrontHandshakeDataLen_get, _thostmduserapi.CThostFtdcRspApiHandshakeField_FrontHandshakeDataLen_set)
    FrontHandshakeData = property(_thostmduserapi.CThostFtdcRspApiHandshakeField_FrontHandshakeData_get, _thostmduserapi.CThostFtdcRspApiHandshakeField_FrontHandshakeData_set)
    IsApiAuthEnabled = property(_thostmduserapi.CThostFtdcRspApiHandshakeField_IsApiAuthEnabled_get, _thostmduserapi.CThostFtdcRspApiHandshakeField_IsApiAuthEnabled_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcRspApiHandshakeField_swiginit(self, _thostmduserapi.new_CThostFtdcRspApiHandshakeField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcRspApiHandshakeField

# Register CThostFtdcRspApiHandshakeField in _thostmduserapi:
_thostmduserapi.CThostFtdcRspApiHandshakeField_swigregister(CThostFtdcRspApiHandshakeField)

class CThostFtdcReqVerifyApiKeyField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    ApiHandshakeDataLen = property(_thostmduserapi.CThostFtdcReqVerifyApiKeyField_ApiHandshakeDataLen_get, _thostmduserapi.CThostFtdcReqVerifyApiKeyField_ApiHandshakeDataLen_set)
    ApiHandshakeData = property(_thostmduserapi.CThostFtdcReqVerifyApiKeyField_ApiHandshakeData_get, _thostmduserapi.CThostFtdcReqVerifyApiKeyField_ApiHandshakeData_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcReqVerifyApiKeyField_swiginit(self, _thostmduserapi.new_CThostFtdcReqVerifyApiKeyField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcReqVerifyApiKeyField

# Register CThostFtdcReqVerifyApiKeyField in _thostmduserapi:
_thostmduserapi.CThostFtdcReqVerifyApiKeyField_swigregister(CThostFtdcReqVerifyApiKeyField)

class CThostFtdcDepartmentUserField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    BrokerID = property(_thostmduserapi.CThostFtdcDepartmentUserField_BrokerID_get, _thostmduserapi.CThostFtdcDepartmentUserField_BrokerID_set)
    UserID = property(_thostmduserapi.CThostFtdcDepartmentUserField_UserID_get, _thostmduserapi.CThostFtdcDepartmentUserField_UserID_set)
    InvestorRange = property(_thostmduserapi.CThostFtdcDepartmentUserField_InvestorRange_get, _thostmduserapi.CThostFtdcDepartmentUserField_InvestorRange_set)
    InvestorID = property(_thostmduserapi.CThostFtdcDepartmentUserField_InvestorID_get, _thostmduserapi.CThostFtdcDepartmentUserField_InvestorID_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcDepartmentUserField_swiginit(self, _thostmduserapi.new_CThostFtdcDepartmentUserField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcDepartmentUserField

# Register CThostFtdcDepartmentUserField in _thostmduserapi:
_thostmduserapi.CThostFtdcDepartmentUserField_swigregister(CThostFtdcDepartmentUserField)

class CThostFtdcQueryFreqField(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    QueryFreq = property(_thostmduserapi.CThostFtdcQueryFreqField_QueryFreq_get, _thostmduserapi.CThostFtdcQueryFreqField_QueryFreq_set)

    def __init__(self):
        _thostmduserapi.CThostFtdcQueryFreqField_swiginit(self, _thostmduserapi.new_CThostFtdcQueryFreqField())
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcQueryFreqField

# Register CThostFtdcQueryFreqField in _thostmduserapi:
_thostmduserapi.CThostFtdcQueryFreqField_swigregister(CThostFtdcQueryFreqField)

class CThostFtdcMdSpi(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def OnFrontConnected(self):
        return _thostmduserapi.CThostFtdcMdSpi_OnFrontConnected(self)

    def OnFrontDisconnected(self, nReason):
        return _thostmduserapi.CThostFtdcMdSpi_OnFrontDisconnected(self, nReason)

    def OnHeartBeatWarning(self, nTimeLapse):
        return _thostmduserapi.CThostFtdcMdSpi_OnHeartBeatWarning(self, nTimeLapse)

    def OnRspUserLogin(self, pRspUserLogin, pRspInfo, nRequestID, bIsLast):
        return _thostmduserapi.CThostFtdcMdSpi_OnRspUserLogin(self, pRspUserLogin, pRspInfo, nRequestID, bIsLast)

    def OnRspUserLogout(self, pUserLogout, pRspInfo, nRequestID, bIsLast):
        return _thostmduserapi.CThostFtdcMdSpi_OnRspUserLogout(self, pUserLogout, pRspInfo, nRequestID, bIsLast)

    def OnRspError(self, pRspInfo, nRequestID, bIsLast):
        return _thostmduserapi.CThostFtdcMdSpi_OnRspError(self, pRspInfo, nRequestID, bIsLast)

    def OnRspSubMarketData(self, pSpecificInstrument, pRspInfo, nRequestID, bIsLast):
        return _thostmduserapi.CThostFtdcMdSpi_OnRspSubMarketData(self, pSpecificInstrument, pRspInfo, nRequestID, bIsLast)

    def OnRspUnSubMarketData(self, pSpecificInstrument, pRspInfo, nRequestID, bIsLast):
        return _thostmduserapi.CThostFtdcMdSpi_OnRspUnSubMarketData(self, pSpecificInstrument, pRspInfo, nRequestID, bIsLast)

    def OnRspSubForQuoteRsp(self, pSpecificInstrument, pRspInfo, nRequestID, bIsLast):
        return _thostmduserapi.CThostFtdcMdSpi_OnRspSubForQuoteRsp(self, pSpecificInstrument, pRspInfo, nRequestID, bIsLast)

    def OnRspUnSubForQuoteRsp(self, pSpecificInstrument, pRspInfo, nRequestID, bIsLast):
        return _thostmduserapi.CThostFtdcMdSpi_OnRspUnSubForQuoteRsp(self, pSpecificInstrument, pRspInfo, nRequestID, bIsLast)

    def OnRtnDepthMarketData(self, pDepthMarketData):
        return _thostmduserapi.CThostFtdcMdSpi_OnRtnDepthMarketData(self, pDepthMarketData)

    def OnRtnForQuoteRsp(self, pForQuoteRsp):
        return _thostmduserapi.CThostFtdcMdSpi_OnRtnForQuoteRsp(self, pForQuoteRsp)

    def __init__(self):
        if self.__class__ == CThostFtdcMdSpi:
            _self = None
        else:
            _self = self
        _thostmduserapi.CThostFtdcMdSpi_swiginit(self, _thostmduserapi.new_CThostFtdcMdSpi(_self, ))
    __swig_destroy__ = _thostmduserapi.delete_CThostFtdcMdSpi
    def __disown__(self):
        self.this.disown()
        _thostmduserapi.disown_CThostFtdcMdSpi(self)
        return weakref.proxy(self)

# Register CThostFtdcMdSpi in _thostmduserapi:
_thostmduserapi.CThostFtdcMdSpi_swigregister(CThostFtdcMdSpi)

class CThostFtdcMdApi(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr

    @staticmethod
    def CreateFtdcMdApi(*args):
        return _thostmduserapi.CThostFtdcMdApi_CreateFtdcMdApi(*args)

    @staticmethod
    def GetApiVersion():
        return _thostmduserapi.CThostFtdcMdApi_GetApiVersion()

    def Release(self):
        return _thostmduserapi.CThostFtdcMdApi_Release(self)

    def Init(self):
        return _thostmduserapi.CThostFtdcMdApi_Init(self)

    def Join(self):
        return _thostmduserapi.CThostFtdcMdApi_Join(self)

    def GetTradingDay(self):
        return _thostmduserapi.CThostFtdcMdApi_GetTradingDay(self)

    def RegisterFront(self, pszFrontAddress):
        return _thostmduserapi.CThostFtdcMdApi_RegisterFront(self, pszFrontAddress)

    def RegisterNameServer(self, pszNsAddress):
        return _thostmduserapi.CThostFtdcMdApi_RegisterNameServer(self, pszNsAddress)

    def RegisterFensUserInfo(self, pFensUserInfo):
        return _thostmduserapi.CThostFtdcMdApi_RegisterFensUserInfo(self, pFensUserInfo)

    def RegisterSpi(self, pSpi):
        return _thostmduserapi.CThostFtdcMdApi_RegisterSpi(self, pSpi)

    def SubscribeMarketData(self, ppInstrumentID, nCount):
        return _thostmduserapi.CThostFtdcMdApi_SubscribeMarketData(self, ppInstrumentID, nCount)

    def UnSubscribeMarketData(self, ppInstrumentID, nCount):
        return _thostmduserapi.CThostFtdcMdApi_UnSubscribeMarketData(self, ppInstrumentID, nCount)

    def SubscribeForQuoteRsp(self, ppInstrumentID, nCount):
        return _thostmduserapi.CThostFtdcMdApi_SubscribeForQuoteRsp(self, ppInstrumentID, nCount)

    def UnSubscribeForQuoteRsp(self, ppInstrumentID, nCount):
        return _thostmduserapi.CThostFtdcMdApi_UnSubscribeForQuoteRsp(self, ppInstrumentID, nCount)

    def ReqUserLogin(self, pReqUserLoginField, nRequestID):
        return _thostmduserapi.CThostFtdcMdApi_ReqUserLogin(self, pReqUserLoginField, nRequestID)

    def ReqUserLogout(self, pUserLogout, nRequestID):
        return _thostmduserapi.CThostFtdcMdApi_ReqUserLogout(self, pUserLogout, nRequestID)

# Register CThostFtdcMdApi in _thostmduserapi:
_thostmduserapi.CThostFtdcMdApi_swigregister(CThostFtdcMdApi)

def CThostFtdcMdApi_CreateFtdcMdApi(*args):
    return _thostmduserapi.CThostFtdcMdApi_CreateFtdcMdApi(*args)

def CThostFtdcMdApi_GetApiVersion():
    return _thostmduserapi.CThostFtdcMdApi_GetApiVersion()



